/************************************************************************/
/* dlTypes.h								*/
/*                                                                      */
/* This file contains type defintions of externally visible delay 	*/
/* line structures							*/
/* Contains both the JPL dlTypes and isitExt.h stuff.			*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar & Brad Himes of JPL                     */
/* Date   : SEPTEMBER 1998                                              */
/************************************************************************/
 
#ifndef INC_dlTypesh
#define INC_dlTypesh

#include <sys/types.h>

#define uint32_t unsigned int

#define INCR1
#define MADE_UP
#define INCR3

typedef int ERROR_CODE;
typedef size_t SIZE;
typedef uint32_t OPLE_TIME;
typedef unsigned char BYTE;

typedef int32_t SHMEM_WORD;
typedef SHMEM_WORD IPC_NAME;  //Temporary
typedef int IPC_ID;           //Temporary

// Hardware typedefs
typedef unsigned int    INT_RATE;
typedef double          LASER_VALUE;
typedef double          PZT_DOUBLE;
typedef int             PZT_POSITION;
typedef int             PZT_VALUE;
typedef int             VC_VALUE;
typedef int             DIAG_VALUE;
typedef int16_t         MOTOR_VALUE;
typedef int             GIZMO_ID;
typedef int             EDDY_VALUE;     /* phil, what should this be? */


// These should have come from the config manager but we wont change it now
#define DL_SERVO_RATE    (5000)   /* rate of fastest servo task */
#define DL_DATA_RATE     (2)
#define DL_DATA_BUF_SIZE (DL_SERVO_RATE/DL_DATA_RATE)  /* must be integer */
#define DL_DATA_BUF_LENGTH (1)   /* how many tDlDatas are sent by a tDlDataStore */

/* The following macros allow concatenation of entities including macros */
#define CONCAT(a,b) a ## b
#define EXPAND_AND_CONCAT(a,b) CONCAT(a,b)

/* One "tick" of time in the interferometer will be defined
 * as 25 microseconds.  This gives a 40 kHz max interrupt rate.
 */

#define TIME_TICKS_PER_SECOND (40000UL)
#define TIME_TICKS_PER_DAY (3456000000UL)
#define TIME_SECONDS_PER_TICK (1.0/TIME_TICKS_PER_SECOND)


#define WHITE_LIGHT_LAMBDA_IN_M (550.0e-9)
// (663.85 nm / 512)
#define METERS_PER_LV (1.296582e-9) 
#define WHITE_LIGHT_LAMBDA_IN_LASER_UNITS ( (WHITE_LIGHT_LAMBDA_IN_M)/ (METERS_PER_LV) )


typedef enum CMD_TYPE {
    DL_CMD_SELECT_TARGET, DL_CMD_DITHER_ON, DL_CMD_DATA_RECORD_ON_OFF,
    INST_CMD_PING, INST_CMD_SET_OPEN_LOOP_TARGET, INST_CMD_SET_MANUAL_TARGET,
    INST_CMD_SET_TIME,  /* Added for GSU -- PCI -- 6/9/98 */
    DL_CMD_OFF, DL_CMD_FRONT, DL_CMD_BACK, DL_CMD_TRACK, DL_CMD_USER1,
    DL_CMD_USER2, DL_CMD_HOME_CHECK, DL_CMD_HOME_SET, DL_CMD_DITHER_CALIBRATE,
    DL_CMD_DITHER_CHECK, DL_CMD_PING, DL_CMD_CLR_LASER_CTR, DL_CMD_MODULATE,
    DL_CMD_SET_DITHER, /*DL_CMD_CHNG_SERVO_PARAM,*/
    GSUDL_CMD_CHNG_SERVO_PARAM,
    FT_CMD_OFF, FT_CMD_TRACK, FT_CMD_CHNG_PARAM, FT_CMD_PING,
    FT_CMD_GET_IMAGE, FT_CMD_SET_ROI, FT_CMD_CALIBRATE, FT_CMD_SET_MT,
    EM_CMD_OFF, EM_CMD_TRACK, EM_CMD_CHNG_PARAM, EM_CMD_PING,
    ST_CMD_OFF, ST_CMD_TRACK, ST_CMD_PING, 
    ST_CMD_HOME, ST_CMD_CALIBRATE, ST_CMD_SETSS,

    WAP_CMD_OFF, WAP_CMD_SLEW, WAP_CMD_TRACK, WAP_CMD_PING, 
    WAP_CMD_HOME, WAP_CMD_CALIBRATE, WAP_CMD_SETSS, WAP_CMD_MOVE,
    EXTINT_CMD_SET_DATA_RECORDING, /* Added for GSU -- KY -- 7/13/98 */
    DISCONNECT, RESET
} CMD_TYPE;

typedef enum GIZMO_TYPE {
   GT_UNKNOWN=0, GT_INSTRUMENT, GT_DELAY_LINE,
   GT_FRINGE_TRACKER, GT_EXTERNAL_METROLOGY, GT_DL_BB1,
   GT_DL_BB2, GT_DL_GSU, GT_STAR_TRACKER, GT_WAP, 
   GT_NUM_GIZMO_TYPES
} GIZMO_TYPE;

typedef struct {
   OPLE_TIME            tTimeStamp;
   ERROR_CODE           ecErrCode;
   int                  iWhichRte;
   int                  iWhichCpu;

} ERROR_INFO;

typedef struct {
   OPLE_TIME            tTimeStamp;
   int			iGizmoId;
   int                  iWhichRte;
   int                  iWhichCpu;
   CMD_TYPE             Command;

} tIsitCmd;

// needed because unions dont allow classes to have non default constructors
// and we need a struct for a tDlTarget as a part of target 
// Commands in ISIT_CMD

typedef struct {
      double      dPosition;
      double      dRate;
      OPLE_TIME   tTime;
} tDlTargetBase;

typedef enum {
   BAD, GOOD, VALID, INVALID
} VAL;

/*This structure is intended to represent a point, and is for use IPC objects 
which are parameterized by data type.  The class tPoint cannot be used for this
purpose, because it contains a pointer to the heap.
*/
struct tIpcPoint {
   double dX,dY;
};

#define JITTER_SCALE 1
#define JITTER_HP_BLKSZ (DL_DATA_BUF_SIZE/2)  /* 2 high pass blks per buffer */

/* diff types of DL targets; used by SetActiveTarget commands and dlTargGen */
/******************************************************************
* The final value here is used to size arrays, etc.  It is not an
* actual servo tracking mode.  It should always be kept at the last
* of these enums.
*/
/* diff types of DL targets; used by SetActiveTarget commands and dlTargGen */

typedef int BOOL;

typedef enum MOD_TYPE {
  MT_NONE, MT_SQUARE, MT_TRIANGLE, MT_SINE, MT_RANDOM, MT_WOBBLE
} MOD_TYPE ;

typedef enum TARGET_TYPE { TT_MANUAL=0, TT_OPEN_LOOP=1, TT_FT=2, 
  TT_EM=3, TT_NUM_DL_TARGETS} TARGET_TYPE;


typedef enum {
  DLCS_SYSTEM_READY, DLCS_OFF_START, DLCS_OFF_DONE, 
  DLCS_FRONT_START, DLCS_FRONT_MOVING, DLCS_FRONT_DONE, DLCS_FRONT_ABORTED,
  DLCS_BACK_START, DLCS_BACK_MOVING, DLCS_BACK_DONE, DLCS_BACK_ABORTED,
  DLCS_TRACK_START, DLCS_TRACK_ACTIVE, DLCS_TRACK_ABORTED,
  DLCS_CLEAR_LASER_COUNTER_START, DLCS_CLEAR_LASER_COUNTER_DONE,
  DLCS_SET_MODULATION_COMPLETE,
  DLCS_SET_SERVO_PARAMETERS_COMPLETE,

  DLCS_HOMESET_BACK_START, DLCS_HOMESET_BACK_MOVING,
    DLCS_HOMESET_FRONT_START, DLCS_HOMESET_FRONT_MOVING,
    DLCS_HOMESET_CREEPBACK_START, DLCS_HOMESET_CREEPBACK_MOVING,
    DLCS_HOMESET_CREEPFRONT_START, DLCS_HOMESET_CREEPFRONT_MOVING,
    DLCS_HOMESET_START, DLCS_HOMESET_DONE, DLCS_HOMESET_ABORTED, 

  DLCS_HOMECHECK_BACK_START, DLCS_HOMECHECK_BACK_MOVING,
    DLCS_HOMECHECK_FRONT_START, DLCS_HOMECHECK_FRONT_MOVING,
    DLCS_HOMECHECK_CREEPBACK_START, DLCS_HOMECHECK_CREEPBACK_MOVING,
    DLCS_HOMECHECK_CREEPFRONT_START, DLCS_HOMECHECK_CREEPFRONT_MOVING,
    DLCS_HOMECHECK_START, DLCS_HOMECHECK_DONE, DLCS_HOMECHECK_ABORTED,

  DLCS_USER1_START, DLCS_USER1_ACTIVE, DLCS_USER1_ABORTED, 
  DLCS_USER2_START, DLCS_USER2_ACTIVE, DLCS_USER2_ABORTED, 
  DLCS_CALIB_DITHER_START, DLCS_CALIB_DITHER_ACTIVE, 
    DLCS_CALIB_DITHER_DONE, DLCS_CALIB_DITHER_ABORTED, 
  DLCS_CHECK_DITHER_START, DLCS_CHECK_DITHER_ACTIVE, 
    DLCS_CHECK_DITHER_DONE, DLCS_CHECK_DITHER_ABORTED, 
  DLCS_SELECT_TARGET_DONE,
  DLCS_TURN_DITHER_ON_DONE, DLCS_TURN_DITHER_OFF_DONE,
  DLCS_SET_DITHER_COMPLETE
  
} DL_COMMAND_STATUS;

typedef enum DL_MODE {
  DM_OFF, DM_FRONT, DM_BACK, DM_TRACK, DM_DITHERCALIB, 
  DM_DITHERCHECK, DM_HOMESET, DM_HOMECHECK, DM_USER1, DM_USER2
} DL_MODE;

/******************************************************************
* The final value here is used to size arrays, etc.  It is not an
* actual servo tracking mode.  It should always be kept at the last
* of these enums.
*/

typedef enum DL_SERVO_MODE {
  DSM_OFF, DSM_FRONT, DSM_BACK, DSM_TRACK, 
  DSM_CREEPFRONT, DSM_CREEPBACK, DSM_USER1, DSM_USER2, DSM_NUM_SERVO_MODES
} DL_SERVO_MODE;


/******************************************************************
* The final value here is used to size arrays, etc.  It is not an
* actual servo tracking mode.  It should always be kept at the last
* of these enums.
*/

enum DL_OFF_CONTROLLER_MODE {DOCM_IDLE, DOCM_DECELERATE, DOCM_NUM_OFF_MODES};
enum DL_TRACK_CONTROLLER_MODE {DTCM_TRACK, DTCM_SLEW, DTM_NUM_TRACKING_MODES};

/******************************************************************
* The final value here is used to size arrays, etc.  It is not an
* actual servo task speed.  It should always be kept at the last
* of these enums.
*
* THE PARITION INFO ENTRIES IN THE GIZMO INFO STRUCTURE MUST BE
* ORDERED THE SAME AS THESE ENTRIES!
*/

enum DL_SERVO_PARTITION { DSP_FAST, DSP_MEDIUM, DSP_SLOW, DSP_NUM_DL_SERVO_TASKS};

typedef enum DL_DIRECTION {DD_NEGATIVE, DD_POSITIVE} DL_DIRECTION;

/*
 * The servo parameters
 */

typedef struct {
  
    // Servo Rates in Hz
    
    double dPztServoRate;
    double dVcServoRate;
    double dMotorServoRate;
 
    // Servo Enables
    
    BOOL bPztServoEnabled;
    BOOL bPztLagEnabled;
    BOOL bVc1TrackServoEnabled;
    BOOL bVc1StabilizeServoEnabled;
    BOOL bVc2TrackServoEnabled;
    BOOL bVc2StabilizeServoEnabled;
    BOOL bMotorServoEnabled;

    // Pzt Control Parameters 
    
#ifndef SPARC
    int	   spacer1;	/* Sparc does odd boundry things */
#endif
    double dPztGain;
    double dPztLagPole;
    double dPztLagZero;
 
    // VC 1 Control Parameters
    
    double dVc1StabilizePidGain;
    double dVc1StabilizePidZero1;
    double dVc1StabilizePidZero2;
    double dVc1StabilizePidPole;

    double dVc1TrackPidGain;
    double dVc1TrackPidZero1;
    double dVc1TrackPidZero2;
    double dVc1TrackPidPole;
    double dVc1TrackLaserFeedforwardGain;
    double dVc1TrackFilterMemory;

    // VC 2 Control Parameters

 
    double dVc2StabilizePidGain;
    double dVc2StabilizePidZero1;
    double dVc2StabilizePidZero2;
    double dVc2StabilizePidPole;
    double dVc2StabilizeMotorAccelerationFeedforwardGain;
 
    double dVc2TrackPidGain;
    double dVc2TrackPidZero1;
    double dVc2TrackPidZero2;
    double dVc2TrackPidPole;
    double dVc2TrackVc1ForceFeedforwardGain;
   
    // Motor Control Parameters
    
    double dMotorSlewVelocity;
    double dMotorCreepVelocity;
    double dDecelerationAggressiveness;            
    double dMotorPidGain;
    double dMotorPidZero1;
    double dMotorPidZero2;
    double dMotorPidPole;
    double dMotorFeedForwardGain;
    

    // Controller Mode Thresholds

    double dDecelToIdleVelocityThreshold;
    double dSlewToTrackThresholdInMm;
    double dTrackToSlewThresholdInMm;


} tGuiGsuDlServoParameters;

typedef struct {
#ifndef SPARC
    int	   spacer1;	/* Sparc does odd boundry things */
#endif
    MOD_TYPE mtModulationType;
#ifndef SPARC
    int	   spacer2;	/* Sparc does odd boundry things */
#endif
    double   dFrequency;
    double   dAmplitude;
    double   dMeanVal;
    OPLE_TIME tSyncTime;
#ifndef SPARC
    int	   spacer3;	/* Sparc does odd boundry things */
#endif
    double   dDutyCycle;

}tModulationParams;  

typedef struct {
  int iVersion;

  int iWhichDl;
  OPLE_TIME tTime;
  
  /* Sensors */

#ifndef SPARC
    int	   spacer1;	/* Sparc does odd boundry things */
#endif
  LASER_VALUE laLaser;
  PZT_DOUBLE  pdPztPos;        /* PZT position */
  EDDY_VALUE    evEddy1;        /* GSU delay line only */
  EDDY_VALUE    evEddy2;        /* GSU delay line only */

  BOOL bLimitFrontSoft;
  BOOL bLimitFrontHard;
  BOOL bLimitBackSoft;
  BOOL bLimitBackHard;
  BOOL bOnHomeSensor;

  /* Sensor modulation info */

  //#warning THIS HAS BEN REPLACED TO FORCE THINGS TO BE THE RIGHT SIZE FOR 64 BIT
  //tModulationParams SensorModulation;
  char dumby[52];
  
  /* Dither calibration info */

  BOOL       bDoingDitherCheckOrCalib;
#ifndef SPARC
    int	   spacer2;	/* Sparc does odd boundry things */
#endif
  double     dDitherAmplitude;
  double     dDitherPhase;
  double     dDitherDistortion;

  /* Actuators */

  PZT_VALUE    pvPztVelocity;   /* PZT Velocity */
  VC_VALUE     vvVc1;            /* VC1 Output */
  VC_VALUE     vvVc2;            /* VC2 Output (GSU only; ignored otherwise) */
  MOTOR_VALUE  mvMotorVelocity; /* Motor speed */
#ifndef SPARC
    short int	   spacerA;	/* 32/64 bit differnce */
#endif

  /* Actuator modulation */

  /* Servo performance info */

/*TRACK_SERVO_PARAMS ServoParams; moved to separate status types... */
  DL_SERVO_MODE  dsmServoMode;
#ifndef SPARC
    int	   spacer3;	/* Sparc does odd boundry things */
#endif
  LASER_VALUE    laTarget;
  LASER_VALUE    laError;
  double         dJitter;
  double         dHighPassJitter;
  int iPztClip;      /* Pzt Clip counter */

  /* Status of current command */

  DL_COMMAND_STATUS  DlCommandStatus;

  /* Servo Parameters */

  tGuiGsuDlServoParameters ServoParameters;

}tDlStatus;

/* Here's the stuff required in order to read and inspect data */

typedef struct DL_DATUM {
  LASER_VALUE laLaser;
  LASER_VALUE laError;
} DL_DATUM;

typedef struct {

  int iVersion;
  int iWhichDl;
  //unsigned long tTimeStamp; CHANGED TO INT FOR 64 BIT MACHINES
  unsigned int tTimeStamp;
  int	   spacer3;	/* Sparc does odd boundry things */
  DL_DATUM ServoDataArray[DL_DATA_BUF_SIZE];
} tDlData_Version_1; 

#endif  /* INC_dlTypesh */
