/************************************************************************/
/* astrogtk_control_window.c                                            */
/*                                                                      */
/* Routine for controlling the basic CHARA astrogtk functions.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : May   2006                                                  */
/************************************************************************/

#include "../cliserv/include/cliserv.h"
#include "../charalib/chara.h"
#include "../include/chara_messages.h"
#include "astrogtk.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include "../slalib/slalib.h"

/* Globals */

static GtkWidget *astrogtk_control_window;
static char astrogtk_control_window_up = FALSE;
static GtkWidget *chara_entry = NULL;
static GtkWidget *hd_entry = NULL;
struct s_obswave obs_ok;
struct s_obswave obs_cancel;

/************************************************************************/
/* create_astrogtk_control_window()					*/
/*									*/
/* ROutine for creating the astrogtk control window. This should be self*/
/* contained and give you all you need to control the basic astrogtk 	*/
/* functionality. Argumentis the window name.				*/
/************************************************************************/

GtkWidget* create_astrogtk_control_window (GtkButton *thebutton, gpointer *data)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *button;
	struct astro_control_window_data *the_data;

	/* Do nothing if we exist already */

	if (astrogtk_control_window_up) return astrogtk_control_window;

	/* Get the data */

	the_data = (struct astro_control_window_data *)data;

	/* Create the window itself */

	astrogtk_control_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (astrogtk_control_window),
		the_data->name);
	gtk_container_set_border_width (GTK_CONTAINER 
		(astrogtk_control_window), 1);

	/* Box to put things in */

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (astrogtk_control_window), vbox);

	/* Fill things out */

	fill_astrogtk_control_page (vbox, the_data->server);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* CANCEL button */

	button = gtk_button_new_with_label ("CANCEL");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (astrogtk_control_ok_callback), (gpointer)P_FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, ASTRO_HEIGHT);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* OK button */

	button = gtk_button_new_with_label ("OK");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (astrogtk_control_ok_callback), (gpointer)P_TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, ASTRO_HEIGHT);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* Now, we can show this window if we like */

	gtk_widget_show(astrogtk_control_window);

	/* Make sure we only get one version of this window */

	astrogtk_control_window_up = TRUE;

	/* That should be all */

	return astrogtk_control_window;

} /* create_astrogtk_control_window() */

/************************************************************************/
/* fill_astrogtk_control_page()						*/
/*									*/
/* Given a server number and a container this routine fills the required*/
/* things out to help select a star.					*/
/************************************************************************/

void fill_astrogtk_control_page(GtkWidget *vbox, int server)
{
	GtkWidget *hbox;
	GtkWidget *button;
	GtkWidget *label;

	/* Setup message data */

	nostar.server = server;
	nostar.chara_number = NOSTAR;
	nostar.hd = 0;
	zenith.server = server;
	zenith.chara_number = ZENITH;
	zenith.hd = 0;
	internal.server = server;
	internal.chara_number = INTERNAL;
	internal.hd = 0;
	autocoll.server = server;
	autocoll.chara_number = AUTOCOLL;
	autocoll.hd = 0;
	charanum.server = server;
	charanum.chara_number = chara_number[server];
	charanum.hd = 0;

	/* Find out which star we are looking at */

	astrogtk_get_current_star(server);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* NOSTAR button */

	button = gtk_button_new_with_label ("NOSTAR");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (astrogtk_setstar_callback), (gpointer)&nostar);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, ASTRO_HEIGHT);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* ZENITH button */

	button = gtk_button_new_with_label ("ZENITH");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (astrogtk_setstar_callback), (gpointer)&zenith);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, ASTRO_HEIGHT);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* INTERNAL button */

	button = gtk_button_new_with_label ("INTERNAL");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (astrogtk_setstar_callback),
		(gpointer)&internal);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, ASTRO_HEIGHT);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* AUTOCOLL button */

	button = gtk_button_new_with_label ("AUTOCOLL");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (astrogtk_setstar_callback),
		(gpointer)&autocoll);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, ASTRO_HEIGHT);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

        /* Now put the label and the entry area for CHARA number */

        label = gtk_label_new("CHARA Number");
        gtk_widget_set_usize (label, ASTRO_CONTROL_WIDTH/2, ASTRO_HEIGHT);
        gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
        gtk_widget_show(label);

        chara_entry = gtk_entry_new ();
        gtk_widget_set_usize (chara_entry, ASTRO_CONTROL_WIDTH/2, ASTRO_HEIGHT);
        gtk_box_pack_start(GTK_BOX(hbox), chara_entry,TRUE,TRUE, 0);
        gtk_signal_connect (GTK_OBJECT (chara_entry), "activate",
                        GTK_SIGNAL_FUNC (astrogtk_chara_callback), NULL);
        gtk_widget_show(chara_entry);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

        /* Now put the label and the entry area for HD number */

        label = gtk_label_new("HD Number");
        gtk_widget_set_usize (label, ASTRO_CONTROL_WIDTH/2, ASTRO_HEIGHT);
        gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
        gtk_widget_show(label);

        hd_entry = gtk_entry_new ();
        gtk_widget_set_usize (hd_entry, ASTRO_CONTROL_WIDTH/2, ASTRO_HEIGHT);
        gtk_box_pack_start(GTK_BOX(hbox), hd_entry,TRUE,TRUE, 0);
        gtk_signal_connect (GTK_OBJECT (hd_entry), "activate",
                        GTK_SIGNAL_FUNC (astrogtk_hd_callback), NULL);
        gtk_widget_show(hd_entry);

	/* Ensure these have the right things in them */

	astrogtk_control_set_chara_hd(server);

	/* OK to see it now */

	gtk_widget_show(vbox);

} /* fill_astrogtk_control_page() */

/************************************************************************/
/* astrogtk_control_set_chara_chara()					*/
/************************************************************************/

void astrogtk_control_set_chara_hd(int server)
{
	char	s[81];
	int	hd;
	struct s_star star;

	/* Get the HD number for this star */

	if (chara_number[server] > 0)
	{
		if (!get_star_by_number(chara_number[server],&star))
		{
		    hd = 0;
		}
		else
		{
		    sscanf(star.hd,"HD_%d",&hd);
		}
	}
	else
		hd = 0;

	/* Fill things out */

	if (chara_entry != NULL)
	{
		sprintf(s,"%d", chara_number[server]);
		gtk_entry_set_text (GTK_ENTRY (chara_entry),s);
	}

	if (hd_entry != NULL)
	{
		sprintf(s,"%d", hd);
		gtk_entry_set_text (GTK_ENTRY (hd_entry),s);
	}

} /* astrogtk_control_set_chara_chara() */

/************************************************************************/
/* astrogtk_control_ok_callback()					*/
/*									*/
/* Callback function for astrogtk control OK button. Stop background	*/
/* processing and destroy astrogtk control window.			*/
/************************************************************************/

void astrogtk_control_ok_callback(GtkButton *button, gpointer doit)
{
	if (!astrogtk_control_window_up) return;

	if (*((int *)doit))
	{
		astrogtk_chara_callback(button, NULL);
	}

	gtk_widget_destroy(astrogtk_control_window);
	astrogtk_control_window_up = FALSE;

} /* astrogtk_control_ok_callback() */

/************************************************************************/
/* astrogtk_setstar_callback()						*/
/*									*/
/************************************************************************/

void astrogtk_setstar_callback(GtkButton *button, gpointer user_data)
{
	struct	s_star_message *star;
	struct	smessage mess;
	char	s[81];
	int	i;

	star = (struct s_star_message *)user_data;

	mess.type = ASTROMOD_SET_STAR;
	i = (int)star->chara_number;
	mess.data = (unsigned char *)&i;
	mess.length = sizeof(int);

	if (!send_message(star->server, &mess))
	{
	    if (astrogtk_control_window_up)
		create_message_window(
			"Failed to send ASTROMOD_SET_STAR message.");
	    else
		print_status(ERROR,
			"Failed to send ASTROMOD_SET_STAR message.\n");
	    return;
	}

	chara_number[star->server] = star->chara_number;

        sprintf(s,"%d", star->chara_number);
        gtk_entry_set_text (GTK_ENTRY (chara_entry),s);

        sprintf(s,"%d", star->hd);
        gtk_entry_set_text (GTK_ENTRY (hd_entry),s);
	
} /* astrogtk_setstar_callback() */

/************************************************************************/
/* astrogtk_chara_callback()						*/
/*									*/
/************************************************************************/

void astrogtk_chara_callback(GtkButton *button, gpointer user_data)
{
	char	*entry;
	struct s_star star;
	int	chara_number;
	int	hd;
	
	entry = (char *)gtk_entry_get_text(GTK_ENTRY(chara_entry));
	sscanf(entry,"%d", &chara_number);
	if (chara_number <= 0) return;
	if (!get_star_by_number(chara_number,&star))
	{
	    if (astrogtk_control_window_up)
	        create_message_window("Failed to find CHARA_%d.",chara_number);
	    else
	        print_status(ERROR,"Failed to find CHARA_%d.\n",chara_number);
	    return;
	}
	else
	{
	    sscanf(star.hd,"HD_%d",&hd);
	}
	charanum.chara_number = chara_number;
	charanum.hd = hd;

	astrogtk_setstar_callback(button, (gpointer)&charanum);

} /* astrogtk_chara_callback() */

/************************************************************************/
/* astrogtk_hd_callback()						*/
/*									*/
/************************************************************************/

void astrogtk_hd_callback(GtkButton *button, gpointer user_data)
{
	char	*entry;
	struct s_star star;
	int	hd;
	
	entry = (char *)gtk_entry_get_text(GTK_ENTRY(hd_entry));
	sscanf(entry,"%d", &hd);
	if (!get_star_by_hd(hd,&star))
	{
	    if (astrogtk_control_window_up)
	        create_message_window("Failed to find HD_%d.",hd);
	    else
	        print_status(ERROR,"Failed to find HD_%d.\n",hd);
	    return;
	}
	charanum.chara_number = star.chara_number;
	charanum.hd = hd;

	astrogtk_setstar_callback(button, (gpointer)&charanum);

} /* astrogtk_hd_callback() */

/************************************************************************/
/* astrogtk_get_current_star()						*/
/*									*/
/* Try and find out what the current star is right now.			*/
/* Returns TRUE if things go well.					*/
/************************************************************************/

bool astrogtk_get_current_star(int server)
{
        struct smessage mess;
        time_t  current_time;
	int	old_chara_number;

        /* The current targets are not valid */

	old_chara_number = chara_number[server];
        chara_number[server] = -100;

        /* Send message requesting targets */

        mess.type = ASTROMOD_WHICH_STAR;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
        {
		chara_number[server] = old_chara_number;
		return FALSE;
        }

        /* Now wait for a while and see if they are updated */

        current_time = time(0);
        while(chara_number[server] == -100)
        {
                process_server_socket(server);
                if (time(0) > current_time + 2)
		{
			chara_number[server] = old_chara_number;
			return FALSE;
		}
        }

        /* That is all */

        return TRUE;

} /* astrogtk_get_current_star() */

/************************************************************************/
/* astrogtk_set_server()						*/
/*									*/
/* In rare cases, the server used to call the function			*/
/* fill_astrogtk_control_page() changes, most notably in cosmic_debris. */
/* This routine ensures any globals setup by that function are set	*/
/* to the new server value.						*/
/************************************************************************/

void astrogtk_set_server(int server)
{
	nostar.server = server;
	zenith.server = server;
	internal.server = server;
	autocoll.server = server;
	charanum.server = server;
	init_ok.server = server;
        init_cancel.server = server;
	obs_ok.server = server;
        obs_cancel.server = server;

} /* astrogtk_set_server() */
