/************************************************************************/
/* astro_messages.c							*/
/*                                                                      */
/* Message handling routines.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : May 2006							*/
/************************************************************************/

#include "../cliserv/include/cliserv.h"
#include "../charalib/chara.h"
#include "../include/chara_messages.h"
#include "astrogtk.h"

/* Globals */

int	chara_number[MAX_SERVERS]; /* Chara number of current star */
int	reference_cart[MAX_SERVERS];   /* Which reference cart */
double	obs_wavelength[MAX_SERVERS];   /* Which observing wavelength? */
char	scope_active[MAX_SERVERS][NUM_SCOPES];
char got_active_scopes = FALSE;
astromod_init_struct beam_config[MAX_SERVERS];
static void (*active_scopes_callback)(void) = NULL;

/************************************************************************/
/* setup_astro_messages()						*/
/*									*/
/* Guess what this does.						*/
/************************************************************************/

int setup_astro_messages(int server, char scopes_active)
{
	int	i;

	/* Here we set up the local messages */

	if (!add_message_job(server, ASTROMOD_INIT,
		message_astromod_init)) return FALSE;

	if (!add_message_job(server, ASTROMOD_SET_STAR,
		message_astromod_set_star)) return FALSE;

	if (!add_message_job(server, ASTROMOD_SET_REFERENCE_CART,
		message_astromod_set_reference_cart)) return FALSE;

	if (!add_message_job(server, ASTROMOD_SET_OBS_WAVELENGTH,
		message_astromod_set_obs_wavelength)) return FALSE;

	if (!add_message_job(server, ASTROMOD_SET_ACTIVE_SCOPES,
		message_astromod_set_active_scopes)) return FALSE;
	chara_number[server] = NOSTAR;
	chara_number[server] = NOSTAR;
	reference_cart[server] = S1;
	for(i=0; i<NUM_SCOPES; i++)
	{
		beam_config[server].beams[i] = BEAM_1;
		beam_config[server].pops[i] = POP_1;
		scope_active[server][i] = scopes_active;
	}

	return TRUE;

} /* setup_astro_messages() */

/************************************************************************/
/* message_astromod_init()						*/
/*									*/
/* Responds to ASTROMOD_INIT message from server.			*/
/************************************************************************/

int message_astromod_init(int server, struct smessage *mess)
{
	struct s_astromod_init	*data;
	int	i;

	/* Make sure data is right length */

	if (mess->length != sizeof(*data))
	{
	    create_message_window(
	    "Recieved ASTROMOD_INIT message with wrong amount of data.");
	}

	data = (struct s_astromod_init *)mess->data;

	for(i=0; i<NUM_SCOPES; i++)
	{
		beam_config[server].beams[i] = data->beams[i];
		beam_config[server].pops[i] = data->pops[i];
	}

	astrogtk_set_beams_pops(server);

	return NOERROR;

} /* message_astromod_init() */

/************************************************************************/
/* message_astromod_set_star()						*/
/*									*/
/* Responds to ASTROMOD_SET_STAR message from server.			*/
/************************************************************************/

int message_astromod_set_star(int server, struct smessage *mess)
{
	int	*data;

	/* Make sure data is right length */

	if (mess->length != sizeof(*data))
	{
	    create_message_window(
	    "Recieved ASTROMOD_SET_STAR message with wrong amount of data.");
	}

	data = (int *)mess->data;

	chara_number[server] = *data;

	astrogtk_control_set_chara_hd(server);

	return TRUE;

} /* message_astromod_set_star() */

/************************************************************************/
/* message_astromod_set_reference_cart()				*/
/*									*/
/* Responds to ASTROMOD_SET_REFERENCE_CART message from server.		*/
/************************************************************************/

int message_astromod_set_reference_cart(int server, struct smessage *mess)
{
	int	*data;

	/* Make sure data is right length */

	if (mess->length != sizeof(*data))
	{
	    create_message_window(
"Recieved ASTROMOD_WHICH_REFERENCE_CART message with wrong amount of data.");
	}

	data = (int *)mess->data;

	reference_cart[server] = *data;

	astrogtk_set_ref(server);

	return TRUE;

} /* message_astromod_set_reference_cart() */

/************************************************************************/
/* message_astromod_set_obs_wavelength()				*/
/*									*/
/* Responds to ASTROMOD_SET_OBS_WAVELENGTH message from server.		*/
/************************************************************************/

int message_astromod_set_obs_wavelength(int server, struct smessage *message)
{
        double  *data;

        if (message->length != sizeof(*data))
        {
	    create_message_window("ASTROMOD_SET_OBS_WAVELENGTH with bad data.");
            return FALSE;
        }

        data = (double *)message->data;

        /* OK, set the relevant global */

        obs_wavelength[server] = *data;

        /* That is all */

        return TRUE;

} /* message_astromod_set_obs_wavelength() */

/************************************************************************/
/* message_astromod_set_active_scopes()					*/
/*									*/
/* Responds to ASTROMOD_SET_ACTIVE_SCOPES message from server.		*/
/************************************************************************/

int message_astromod_set_active_scopes(int server, struct smessage *message)
{
        char  *data;
	int	i;

        if (message->length != NUM_SCOPES*sizeof(*data))
        {
	    create_message_window("ASTROMOD_SET_ACTIVE_SCOPES with bad data.");
            return FALSE;
        }

        data = (char *)message->data;

        /* OK, set the relevant global */

	for(i=0; i<NUM_SCOPES; i++) scope_active[server][i] = data[i];

        /* That is all */

	got_active_scopes = TRUE;

	if (active_scopes_callback != NULL) (*active_scopes_callback)();

	astrogtk_set_active_scopes(server);

        return TRUE;

} /* message_astromod_set_active_scopes() */

/************************************************************************/
/* set_active_scope_callback()                                          */
/*                                                                      */
/* Setup a callback function to be called whenever a new star is set.   */
/* The callback function gets no data, but should return an error level.*/
/************************************************************************/

void set_active_scopes_callback(void (*callback)(void))
{
        active_scopes_callback = callback;

} /* set_active_scopes_callback() */

/************************************************************************/
/* unset_active_scopes_callback()                                       */
/*                                                                      */
/* Remove the  callback called whenever a new star is set.              */
/************************************************************************/

void unset_active_scopes_callback(void)
{
        active_scopes_callback = NULL;

} /* unset_active_scopes_callback() */
