/************************************************************************/
/* astromod_refract.c							*/
/*									*/
/* Function that finds the index of refraction in air.		 	*/
/* Copy of same file in the old astromod program with some cleaning up.	*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1369                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : berger@chara.gsu.edu                                      */
/* WWW      : http://www.chara.gsu.edu/~berger                          */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*									*/
/*	Author: Dave Berger						*/
/*		Telephone: 1-404-651-1369				*/
/*		EMail:	berger@chara.gsu.edu				*/
/*	Date:   28 October 1998						*/
/*									*/
/************************************************************************/

#include "../charalib/chara.h"
#include "astromod.h"

/************************************************************************/
/* astromod_refract()							*/
/*									*/
/* Calculate the refractive index of air.				*/
/************************************************************************/

double astromod_refract(double temp, double pressure, double rel_humid, double CO2,
	double wavelength)
{

	/*
	 * constants used in saturation vapor pressure
	 */
	const double	A =  1.2378847E-5;
	const double	B = -1.9121316E-2;
	const double	C = 33.93711047;
	const double	D = -6.3431645E3;
	
	/*
	 * constants used in the enhancement factor of water vapor
	 */
	const double	alpha = 1.00062;
	const double	beta = 3.14E-8;
	const double	gamma = 5.6E-7;

	/*
	 * constants used in the refractivities of dry air
	 */
	const double	k0 = 238.0185;
	const double	k1 = 5792105;
	const double 	k2 = 57.362;
	const double	k3 = 167917;

	/*
	 * constants used in the density of standard air
	 */
	const double	R = 8.314510;
	const double	Mw = 0.018015;

	/*
	 * constants used for the refractivity of Barrel and Sear
	 */
	const double	cf = 1.022;
	const double	w0 =  295.235;
	const double	w1 =  2.6422;
	const double	w2 = -0.032380;
	const double	w3 =  0.004028;

	/*
	 * Internal variables
	 */
	double T, p, h, lamda, xc;
	double svp, t, f, xw, sigma, n_as, n_axs, Ma, Za, Zw;
	double rho_axs, rho_ws, Z, rho_a, rho_w, n_ws, n_prop;

	/* Do the variable switch */

	T = temp;
	p = 100*pressure;
	h = rel_humid;
	lamda = wavelength;
	xc = CO2;

	/*
	 * define the saturation vapor pressure
	 */
	svp = exp(A*T*T + B*T + C + D/T );

	/*
	 * temperature conversion
	 */
	t = T - 273.15;

	/*
	 * enhancement factor of water vapor in air
	 */
	f = alpha + beta*p + gamma*t*t;

	/*
	 * molar fraction of water vapor in moist air
	 */
	xw = (f*h*svp) / p;

	/*
	 * wave number
	 */
	sigma = 1 / lamda;

	/*
	 * refractivities of standard air at 15 Celsius, 101,235 Pa, and
	 * 0% humidity, and with 450 ppm (n_as) or xc ppm (n_axs)
	 * of CO2
	 */
	n_as = 1E-8 * ((k1/(k0-sigma*sigma))+(k3/(k2-sigma*sigma))) + 1;

	n_axs = (n_as-1)*(1+0.534E-6*(xc-450)) + 1;

	/*
	 * molar mass of dry air (kg/mol)
	 */
	Ma = 1E-3*(28.9635+12.011E-6*(xc-400));

	/*
	 * compressibility of dry air
	 * (T=288.15 K, p=101,325 Pa, xw=0)
	 */
	Za = astromod_compressibility(288.15, 101325, 0.0);

	/*
	 * compressibility of pure water vapor
	 * (T=293.15 K, p=1333 Pa, xw=1)
	 */
	Zw = astromod_compressibility(293.15, 1333, 1.0);

	/*
	 * density of standard air
	 */
	rho_axs = ((p*Ma)/(Za*R*T)) * ( 1 - xw*(1 - (Mw/Ma)) );

	/*
	 * density of standard water vapor
	 */ 
	rho_ws =  ((p*Ma)/(Zw*R*T)) * ( 1 - xw*(1 - (Mw/Ma)) );

	/*
	 * compressibility under present conditions
	 */
	Z = astromod_compressibility(T,p,xw);

	/*
	 * density of the dry component of the moist air
	 */
	rho_a = (p*Ma*(1-xw))/(Z*R*T);

	/*
	 * density of the water vapor comnponent of the moist air
	 */
	rho_w = (p*Mw*xw)/(Z*R*T);

	/*
	 * refractivity of water vapor at the standard conditions
	 * as defined by Barrels and Sears (see reference)
	 * (20 Celsius and 1333 Pa)
	 */
	n_ws = 1E-8*cf*(w0 + w1*sigma*sigma + w2*sigma*sigma*sigma*sigma
			+ w3*sigma*sigma*sigma*sigma*sigma*sigma) + 1;

	/*
	 * drum roll please....
	 * the refractive index of moist air!!!!
	 */
	n_prop = ((rho_a/rho_axs)*(n_axs-1) + (rho_w/rho_ws)*(n_ws-1)) + 1;

	return n_prop;

} /* astromod_refract() */

double astromod_compressibility(double temp, double pressure, double CO2)
{

	/*
	 * Constants
	 */
        const double    a0 =  1.58123E-6;
        const double    a1 = -2.9331E-8;
        const double    a2 =  1.1043E-10;
        const double    b0 =  5.707E-6;
        const double    b1 = -2.501E-8;
        const double    c0 =  1.9898E-4;
        const double    c1 = -2.376E-6;
        const double    d  =  1.83E-11;
        const double    e  = -0.765E-8;

	/*
	 * Internals
	 */
	double T, t, p, xw, compress;

	/*
	 * shorten up some variable names to make me happy
	 * and more similar to the equations in the reference
	 */
	T = temp;
	p = pressure;
	xw = CO2;

	/*
	 * temperature conversion
	 */
	t = T - 273.15;

	/*
	 * the actual compressibility equation
	 * (eq. 12 in the reference)
	 */
	compress = 1 - (p/T) * (a0 + a1*t + a2*t*t + (b0 + b1*t)*xw
	                 + (c0 + c1*t)*xw*xw)
 	    + (p/T)*(p/t)*(d + e*xw*xw);

	return compress;

} /* astromod_compressibility() */
