/************************************************************************/
/* astromod_ui.c							*/
/*                                                                      */
/* Routines for dealing with the astrometric model stuff in a UI.	*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY SERVER LIB				*/
/*		   Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : May 2006							*/
/************************************************************************/

/*
 * Include files
 */

#include "../charaui/src/charaui.h"
#include "../include/chara_messages.h"
#include "../charalib/chara.h"
#include <netdb.h>
#include <resolv.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <signal.h>
#include "astromod.h"
#include "astromod_ui.h"

/* Global for the star we are currently looking at */

struct s_star star;
int reference_cart = S1;
bool use_astromod_lib = TRUE;
char scope_active[NUM_SCOPES] = {FALSE, FALSE, FALSE, FALSE, FALSE, FALSE};

/* Locals */

static int (*star_callback)(void) = NULL;
static int (*current_config_callback)(void) = NULL;
static int (*init_callback)(void) = NULL;
static int (*active_scopes_callback)(void) = NULL;

/************************************************************************/
/* setup_astromod_messages()						*/
/*									*/
/* Setup the messages for astromod in teh UI environment. Returns	*/
/* error level.								*/
/************************************************************************/

void setup_astromod_messages(void)
{
        ui_add_message_job(ASTROMOD_CURRENT_CONFIG, 
		message_current_config);
        ui_add_message_job(ASTROMOD_INIT, message_astromod_init);
        ui_add_message_job(ASTROMOD_SET_STAR, message_set_star);
        ui_add_message_job(ASTROMOD_WHICH_STAR, message_which_star);
        ui_add_message_job(ASTROMOD_SET_REFERENCE_CART, 
		message_set_reference_cart);
        ui_add_message_job(ASTROMOD_WHICH_REFERENCE_CART, 
		message_which_reference_cart);
        ui_add_message_job(ASTROMOD_SET_USE_ASTROMOD_LIB, 
		message_set_use_astromod_lib);
        ui_add_message_job(ASTROMOD_STAR_DATA, 
		message_star_data);
        ui_add_message_job(ASTROMOD_UV_DATA, message_uv_data);
        ui_add_message_job(ASTROMOD_SET_OBS_WAVELENGTH, 
		message_set_obs_wavelength);
        ui_add_message_job(ASTROMOD_GET_OBS_WAVELENGTH, 
		message_get_obs_wavelength);
        ui_add_message_job(ASTROMOD_GET_ACTIVE_SCOPES, 
		message_get_active_scopes);
        ui_add_message_job(ASTROMOD_SET_ACTIVE_SCOPES, 
		message_set_active_scopes);

} /* setup_astromod_messages() */

/************************************************************************/
/* set_current_config_callback()					*/
/*									*/
/* Setup a callback function to be called whenever a configration 	*/
/* is set. The callback function gets no data, but should return 	*/
/* an error level.							*/
/************************************************************************/

void set_current_config_callback(int (*callback)(void))
{
	current_config_callback = callback;

} /* set_current_config_callback() */

/************************************************************************/
/* unset_current_config_callback()					*/
/*									*/
/* Remove the  callback called whenever a configration is set.		*/
/************************************************************************/

void unset_current_config_callback(void)
{
	current_config_callback = NULL;

} /* unset_current_config_callback() */

/************************************************************************/
/* message_current_config()						*/
/*									*/
/* For when we get a ASTROMOD_CURRENT_CONFIG message.	 		*/
/************************************************************************/

int message_current_config(struct smessage *message)
{
	struct smessage mess;
	struct s_astromod_init config;
	int	i;

	if (message->length != 0)
	{
		ui_print_message(client_socket,
			"ASTROMOD_CURRENT_CONFIG with bad data.");
		return ERROR;
	}

	/* Callback function if we need one */

	if (current_config_callback != NULL && 
			(*current_config_callback)() != NOERROR) return ERROR;

	/* Send the message */

	for(i=0; i<NUM_SCOPES; i++)
	{
		config.beams[i] = scopes[i].beam;
		config.pops[i] = scopes[i].pop;
	}

	mess.type = ASTROMOD_INIT;
	mess.data = (unsigned char *)&config;
	mess.length = sizeof(config);

	ui_send_message(client_socket, &mess);

	return NOERROR;

} /* message_current_config() */

/************************************************************************/
/* message_astromod_init()						*/
/*									*/
/* For when we get a ASTROMOD_INIT message.	 			*/
/************************************************************************/

int message_astromod_init(struct smessage *message)
{
	int	*beams;
	int	*pops;
	struct s_astromod_init *data;
	int	i;
	bool	use_defaults;

	if (message->length != sizeof(*data))
	{
		ui_print_message(client_socket,"ASTROMOD_INIT with bad data.");
		return ERROR;
	}

	data = (struct s_astromod_init *) message->data;

	/* Are we using defaults for beams? */

	for(use_defaults = FALSE,i=0; i<NUM_SCOPES; i++)
	{
		if (data->beams[i] < 0)
		{
			use_defaults = TRUE;
			break;
		}
	}

	if (use_defaults)
		beams = NULL;
	else
		beams = (int *)&(data->beams);

	/* Are we using defaults for pops? */

	for(use_defaults = FALSE,i=0; i<NUM_SCOPES; i++)
	{
		if (data->pops[i] < 0)
		{
			use_defaults = TRUE;
			break;
		}
	}

	if (use_defaults)
		pops = NULL;
	else
		pops = (int *)&(data->pops);

	switch(astromod_init(beams, pops))
	{
		case 0: break;

		case -1: ui_print_message(client_socket,
				"ASTROMOD_INIT failed to get system data.");
			return ERROR;

		case -2: ui_print_message(client_socket,
				"ASTROMOD_INIT bad beam information.");
			return ERROR;

		case -3: ui_print_message(client_socket,
				"ASTROMOD_INIT bad pops information.");
			return ERROR;

		default: ui_print_message(client_socket,
				"ASTROMOD_INIT unknown error.");
			return ERROR;
	}

	/* Make sure all the clients know about this */

	ui_send_message_all(message);

	/* Call the callback function */

	if (init_callback != NULL) 
		return (*init_callback)();
	else
		return NOERROR;

} /* message_astromod_init() */

/************************************************************************/
/* show_astromod_system()						*/
/*									*/
/* Display all information on the CHARA system as we understand it.	*/
/************************************************************************/

int show_astromod_system(int argc, char **argv)
{
	char	s[256];
	int	i;

        /* Initialisation */

        number_text_lines = -1;
        clear_display();
        inc_number_text_lines();

	/* Fill out display structure with system data. */

	sprintf(display->string[number_text_lines],
		"Longitude   : %s", time_to_string(chara.longitude,s));
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Latitude    : %s", angle_to_string(chara.latitude,s));
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Fixed Cart  : %s", chara.fixed_cart);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Height      : %.2f m", chara.height);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Obs Lambda  : %.2f um", chara.obs_wavelength);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Met Lambda  : %.4f um", chara.met_wavelength);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Polar Motion: %.4e,%.2e", chara.pole.x, chara.pole.y);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"UT1-UT2     : %.5f", chara.dut);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Num Scopes  : %d", chara.num_telescopes);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Num Baseline: %d", chara.num_baselines);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Scopes used : %s", chara.scopes_used);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	display->number_items = number_text_lines;

	/* Display it */

        if (active_socket == -1)
        {
                /* Called by keyboard user, scroll the text on screen */

                active_window = main_window;
                heading(heading_window,"Current System Information","");
                scroll_text(sub_main_window);
        }
        else
        {
                /* Called by a socket, spit it out the socket */

                socket_print(active_socket,"Current System Information:\n");
                for(i=0; i<display->number_items; i++)
                {
                        socket_print(active_socket,"%s\n",display->string[i]);
                }
        }

	return NOERROR;

} /* show_astromod_system() */

/************************************************************************/
/* show_astromod_scopes()						*/
/*									*/
/* Display all information on the CHARA scopes as we understand it.	*/
/************************************************************************/

int show_astromod_scopes(int argc, char **argv)
{
	char	s[256];
	int	i,j;

        /* Initialisation */

        number_text_lines = -1;
        clear_display();
        inc_number_text_lines();

	/* Fill out display structure with system data. */

	for(i=0; i<MAX_NUMBER_TELESCOPES;i++)
	{
	    sprintf(display->string[number_text_lines],
		"Scope Name: %c%d", scopes[i].arm, scopes[i].station);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    if (scope_active[i])
		sprintf(display->string[number_text_lines], "    Active: YES");
	    else
		sprintf(display->string[number_text_lines], "    Active: NO");
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"      Beam: %s (%s)", beam_types[scopes[i].beam], 
					  beam_types[scopes[i].default_beam]);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"       Pop: %s (%s)", pop_types[scopes[i].pop], 
					  pop_types[scopes[i].default_pop]);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		" Longitude: %s", time_to_string(scopes[i].longitude,s));
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"  Latitude: %s", angle_to_string(scopes[i].latitude,s));
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"  Baseline: (%.3f,%.3f,%.3f)m", 
		scopes[i].baseline.x,
		scopes[i].baseline.y,
		scopes[i].baseline.z);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"    EC/SAS: %.2f/%.2fmm", scopes[i].ec*1e3,scopes[i].sas*1e3);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"  SAC/HSA2: %.2f/%.2fmm",scopes[i].sac*1e3,scopes[i].hsa2*1e3);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"       WLF: %.3fm", scopes[i].wlf);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"   Airpath: %.3fm", scopes[i].airpath);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"  Internal: %.3fm", scopes[i].internal);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		" Lightpipe: %.3fm", scopes[i].lightpipe);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],"     Beams: ");
	    for(j=0; j<NUM_BEAMS; j++)
	    {
		sprintf(s," %.3f", scopes[i].beampath[j]);
		strcat(display->string[number_text_lines],s);
	    }
	    strcat(display->string[number_text_lines],"m");
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],"      Pops: ");
	    for(j=0; j<NUM_POPS; j++)
	    {
		sprintf(s," %.3f", scopes[i].poppath[j]);
		strcat(display->string[number_text_lines],s);
	    }
	    strcat(display->string[number_text_lines],"m");
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"     Az/El: %.3f/%.3f", 
		scopes[i].star.az/D2R, scopes[i].star.el/D2R);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"    Vector: (%.4f,%.4f,%.4f)", 
		scopes[i].to_star.x, scopes[i].to_star.y, scopes[i].to_star.z);
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    sprintf(display->string[number_text_lines],
		"Hour angle: %s", time_to_string(scopes[i].hour_angle,s));
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	    *(display->string[number_text_lines]) = 0;
	    if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	}

	display->number_items = number_text_lines;

	/* Display it */

        if (active_socket == -1)
        {
                /* Called by keyboard user, scroll the text on screen */

                active_window = main_window;
                heading(heading_window,"Current Telescope Information","");
                scroll_text(sub_main_window);
        }
        else
        {
                /* Called by a socket, spit it out the socket */

                socket_print(active_socket,"Current Telescope Information:\n");
                for(i=0; i<display->number_items; i++)
                {
                        socket_print(active_socket,"%s\n",display->string[i]);
                }
        }

	return NOERROR;

} /* show_astromod_system() */

/************************************************************************/
/* set_no_star()							*/
/*									*/
/* Set the system so it points at no star at all.			*/
/************************************************************************/

int set_no_star(int argc, char **argv)
{
	strcpy(star.name, "NOSTAR");
	star.chara_number = NOSTAR;
	*star.irc = 0;
	*star.hr = 0;
	*star.hd = 0;
	*star.sao = 0;
	star.ra = 0.0;
	*star.rastr = 0;
	star.dec = 0.0;
	*star.decstr = 0;
	star.pm_ra = 0.0;
	star.pm_dec = 0.0;
	star.parallax = 0.0;
	star.radvel = 0.0;
	star.vmag = 0.0;
	star.kmag = 0.0;
	*star.spectype = 0;

	message(system_window,"Star data cleared.");

	if (star_callback != NULL) 
		 return (*star_callback)();
	else
		return NOERROR;

} /* set_no_star() */

/************************************************************************/
/* set_zenith()								*/
/*									*/
/* Set the system so it points at zenith.				*/
/************************************************************************/

int set_zenith(int argc, char **argv)
{
	set_no_star(0, NULL);
	strcpy(star.name, "ZENITH");
	star.chara_number = ZENITH;
	return NOERROR;

} /* set_zenith() */

/************************************************************************/
/* set_internal()							*/
/*									*/
/* Set the system so it points at Internal fringes.			*/
/************************************************************************/

int set_internal(int argc, char **argv)
{
	set_no_star(0, NULL);
	strcpy(star.name, "INTERNAL");
	star.chara_number = INTERNAL;
	message(system_window,"Internal.");
	return NOERROR;

} /* set_internal() */

/************************************************************************/
/* set_autocoll()							*/
/*									*/
/* Set the system so it points at autocoll fringes.			*/
/************************************************************************/

int set_autocoll(int argc, char **argv)
{
	set_no_star(0, NULL);
	strcpy(star.name, "AUTOCOLL");
	star.chara_number = AUTOCOLL;
	message(system_window,"Autocoll.");

	return NOERROR;

} /* set_autocoll() */

/************************************************************************/
/* show_astromod_star()							*/
/*									*/
/* Display all information on the star as we understand it.		*/
/************************************************************************/

int show_astromod_star(int argc, char **argv)
{
	int	i;

        /* Initialisation */

        number_text_lines = -1;
        clear_display();
        inc_number_text_lines();

	/* Fill out display structure with system data. */

	sprintf(display->string[number_text_lines],
		"Star Name   : %s", star.name);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"CHARA Number: %d", (int)star.chara_number);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"IRC         : %s", star.irc);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"HR          : %s", star.hr);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"HD          : %s", star.hd);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"SAO         : %s", star.sao);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"RA          : %s (%f)", star.rastr, star.ra);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"DEC         : %s (%f)", star.decstr, star.dec);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"PM RA       : %e", star.pm_ra);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"PM DEC      : %e", star.pm_dec);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Parallax    : %e", star.parallax);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Radial vel  : %e", star.radvel);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"K mag       : %.2f", star.kmag);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"V mag       : %.2f", star.vmag);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");
	sprintf(display->string[number_text_lines],
		"Spectral Typ: %s", star.spectype);
	if (!inc_number_text_lines()) return error(ERROR,"Display error.");

	display->number_items = number_text_lines;

	/* Display it */

	if (active_socket == -1)
	{
            /* Called by keyboard user, scroll the text on screen */

            active_window = main_window;
            heading(heading_window,"Current Star Information","");
            scroll_text(sub_main_window);
	}
	else
	{
            /* Called by a socket, spit it out the socket */

            socket_print(active_socket,"Current Star Information:\n");
            for(i=0; i<display->number_items; i++)
            {
                    socket_print(active_socket,"%s\n",display->string[i]);
            }
	}

	return NOERROR;

} /* show_astromod_star() */

/************************************************************************/
/* set_chara_star()							*/
/*									*/
/* Set current star by CHARA number.					*/
/************************************************************************/

int set_chara_star(int argc, char **argv)
{
	char	s[256];
	int	chara_number;
	struct smessage mess;

	/* Did the user tell us anything interesting? */

	clean_command_line();
	if (argc > 1)
        {
                sscanf(argv[1],"%d",&chara_number);
        }
        else
        {
		sprintf(s,"            0");
                if (quick_edit("CHARA number",
                        s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

                sscanf(s,"%d",&chara_number);
        }

	working();

	if (chara_number <= 0)
	{
		switch(chara_number)
		{
			case NOSTAR: set_no_star(0, NULL);
				     break;

			case ZENITH: set_no_star(0, NULL);
				     strcpy(star.name, "ZENITH");
				     star.chara_number = chara_number;
				     message(system_window,"Zenith");
				     break;

			case INTERNAL: set_no_star(0, NULL);
				     strcpy(star.name, "INTERNAL");
				     star.chara_number = chara_number;
				     message(system_window,"Internal");
				     break;

			case AUTOCOLL: set_no_star(0, NULL);
				     strcpy(star.name, "AUTOCOLL");
				     star.chara_number = chara_number;
				     message(system_window,"Autocoll");
				     break;

			default: not_working();
				 return error(ERROR,"Unknown star type %d.",
						chara_number);
		}
	}
	else
	{	
		if (!get_star_by_number(chara_number, &star))
		{
			not_working();
			return error(ERROR,
				"Star CHARA_%d does not exist in the database.",
				chara_number);
		}
		message(system_window,"Star Name   : %s", star.name);
	}

	not_working();
	
	/* Make sure all the clients know about this. */

	mess.type = ASTROMOD_SET_STAR;
	mess.data = (unsigned char *)&chara_number;
	mess.length = sizeof(chara_number);

	ui_send_message_all(&mess);

	/* Do the callback if we have one */

	if (star_callback != NULL) 
		 return (*star_callback)();
	else
		return NOERROR;

} /* set_chara_star() */

/************************************************************************/
/* set_star_callback()							*/
/*									*/
/* Setup a callback function to be called whenever a new star is set.	*/
/* The callback function gets no data, but should return an error level.*/
/************************************************************************/

void set_star_callback(int (*callback)(void))
{
	star_callback = callback;

} /* set_star_callback() */

/************************************************************************/
/* unset_star_callback()						*/
/*									*/
/* Remove the  callback called whenever a new star is set.		*/
/************************************************************************/

void unset_star_callback(void)
{
	star_callback = NULL;

} /* unset_star_callback() */

/************************************************************************/
/* message_set_star()							*/
/*									*/
/* For when we get a ASTROMOD_SET_STAR message.	 			*/
/************************************************************************/

int message_set_star(struct smessage *message)
{
	int	*chara_number;
	char	*argv[2];
	char	argv1[80];

	if (message->length != sizeof(int))
	{
		ui_print_message(client_socket,
			"ASTROMOD_SET_STAR with bad data.");
		return ERROR;
	}

	chara_number = (int *)message->data;

	sprintf(argv1,"%d", *chara_number);
	argv[0] = "chara";
	argv[1] = argv1;

	return set_chara_star(2, argv);

} /* message_set_star() */

/************************************************************************/
/* message_which_star()							*/
/*									*/
/* For when we get a ASTROMOD_WHICH_STAR message.	 		*/
/************************************************************************/

int message_which_star(struct smessage *message)
{
	struct smessage mess;
	int	chara_number;

	if (message->length != 0)
	{
		ui_print_message(client_socket,
			"ASTROMOD_WHICHT_STAR with bad data.");
		return ERROR;
	}

	mess.type = ASTROMOD_SET_STAR;
	chara_number = star.chara_number;
	mess.data = (unsigned char *)&chara_number;
	mess.length = sizeof(int);

	ui_send_message_all(&mess);

	return NOERROR;

} /* message_set_star() */

/************************************************************************/
/* set_hd_star()							*/
/*									*/
/* Set current star by CHARA number.					*/
/************************************************************************/

int set_hd_star(int argc, char **argv)
{
	char	s[256];
	int	hd_number;
	struct smessage mess;
	int	chara_number;

	/* Did the user tell us anything interesting? */

	clean_command_line();
	if (argc > 1)
        {
                sscanf(argv[1],"%d",&hd_number);
        }
        else
        {
		sprintf(s,"            0");
                if (quick_edit("HD number",
                        s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

                sscanf(s,"%d",&hd_number);
        }

	working();
	if (!get_star_by_hd(hd_number, &star))
	{
		not_working();
		error(ERROR,"Star HD_%d does not exist in the database.",
			hd_number);
	}

	not_working();

	chara_number = (int)star.chara_number;
	mess.type = ASTROMOD_SET_STAR;
	mess.data = (unsigned char *)&chara_number;
	mess.length = sizeof(chara_number);

	ui_send_message_all(&mess);

	if (star_callback != NULL) 
		 return (*star_callback)();
	else
		return NOERROR;

} /* set_hd_star() */

/************************************************************************/
/* select_reference_cart()						*/
/*									*/
/* Choose which cart is reference.					*/
/************************************************************************/

int select_reference_cart(int argc, char **argv)
{
        int     dl_number;
	struct smessage mess;

        /* Check out the command line */

        clean_command_line();
        if (argc > 1)
        {
                for (dl_number=0; dl_number<NUM_DELAY_LINES; dl_number++)
                {
                        if (strcmpi(argv[1],ople_types[dl_number]) == 0)
                        {
                                break;
                        }
                }

                if (dl_number >= NUM_DELAY_LINES)
                {
                        error(ERROR,"Unkown ople cart %s.",argv[1]);
                        return -2;
                }
        }
        else
        {
                dl_number = S1;
                if (quick_edit("Reference cart",ople_types[dl_number],
                        &dl_number,ople_types,ENUMERATED) == KEY_ESC)
                                return -1;
        }

        reference_cart = dl_number;

	/* Make sure all clients know about this. */

	mess.type = ASTROMOD_SET_REFERENCE_CART;
	mess.data = (unsigned char *)&reference_cart;
	mess.length = sizeof(int);

	ui_send_message_all(&mess);

	return NOERROR;

} /* select_reference_cart() */

/************************************************************************/
/* message_set_reference_cart()						*/
/*									*/
/* For when we get a ASTROMOD_SET_REFERENCE_CART message.		*/
/************************************************************************/

int message_set_reference_cart(struct smessage *message)
{
	int	*cart;

	if (message->length != sizeof(int))
	{
		ui_print_message(client_socket,
			"ASTROMOD_SET_REFERENCE_CART with bad data.");
		return ERROR;
	}

	cart = (int *)message->data;

	if (*cart < 0 || *cart >= NUM_DELAY_LINES)
	{
		ui_print_message(client_socket,
			"No such cart as %d.",*cart);
		return ERROR;
	}
	reference_cart = *cart;

	/* Tell other clients */

	ui_send_message_all(message);

	/* Call the callback function */

	if (init_callback != NULL) 
		 return (*init_callback)();
	else
		return NOERROR;

} /* message_set_reference_cart() */

/************************************************************************/
/* message_which_reference_cart()					*/
/*									*/
/* For when we get a ASTROMOD_WHICH_REFERENCE_CART message. 		*/
/************************************************************************/

int message_which_reference_cart(struct smessage *message)
{
	struct smessage mess;

	if (message->length != 0)
	{
		ui_print_message(client_socket,
			"ASTROMOD_WHICH_REFERENCE_CART with bad data.");
		return ERROR;
	}

	mess.type = ASTROMOD_SET_REFERENCE_CART;
	mess.data = (unsigned char *)&reference_cart;
	mess.length = sizeof(int);

	ui_send_message_all(&mess);

	return NOERROR;

} /* message_set_reference_cart() */

/************************************************************************/
/* message_set_use_astromod_lib()					*/
/*									*/
/* For when we get a ASTROMOD_SET_USE_ASTROMOD_LIB message. 		*/
/************************************************************************/

int message_set_use_astromod_lib(struct smessage *mess)
{
	if (mess->length != sizeof(int))
	{
		ui_print_message(client_socket,
			"ASTROMOD_SET_USE_ASTROMOD_LIB with bad data.");
		return ERROR;
	}

	if (*((int*)mess->data))
	{
		use_astromod_lib = TRUE;
		ui_print_message(client_socket,
			"Using astromod library for all calculations.");
	}
	else
	{
		use_astromod_lib = FALSE;
		ui_print_message(client_socket,
			"Using astromod program for all calculations.");
	}
	return NOERROR;

} /* message_set_use_astromod_lib() */

/************************************************************************/
/* set_use_astromod_lib()						*/
/*									*/
/* Set global variable.							*/
/************************************************************************/

int set_use_astromod_lib(int argc, char **argv)
{
	if (argc > 1)
	{
            use_astromod_lib = *argv[1] == 'y' || *argv[1] == 'Y';
	}
        else
	{
            use_astromod_lib =
		ask_yes_no("Do you wish to use the astromod library?","");
	}

	if (use_astromod_lib)
	    message(system_window,
			"Using astromod library for all calculations.");
	else
	    message(system_window,
			"Using astromod program for all calculations.");
	return NOERROR;

} /* set_use_astromod_lib() */

/************************************************************************/
/* create_astromod_data_file_name()					*/
/*									*/
/* Given a name this function returns a data filename starting with the */
/* current object name, the date, containing the name given and ending  */
/* with a unique three digit number.					*/
/* It assumes space is provided in the string s.			*/
/* Returns file name or NULL if it fails to find one.			*/
/************************************************************************/

char *create_astromod_data_file_name(char *dir, char *name, char *filename)
{
	FILE	*fp;
	char	filename_base[256];
	int     year,month,day;
	struct  tm      *gmt_now;
        time_t current_time;
	char	star_name[256];
	int	file_number;
	char	*p;

	/* What day is it right now? */

	time(&current_time);
	gmt_now = gmtime(&current_time);

	/* Note... not really Y2K cool.... */

	year = gmt_now->tm_year + 1900;
	if (year < 1950) year+=100;
	month = gmt_now->tm_mon+1;
	day = gmt_now->tm_mday;

	/* No we need a star name */

	if (*star.hd != 0)
		strcpy(star_name, star.hd);
	else if (strcmp(star.name,"NOSTAR"))
		strcpy(star_name, star.name);
	else
		strcpy(star_name, "noname");

	/* Find the end of the directory name */

	for(p=dir; *p != 0; p++);
	if (p != dir) p--;

	/* Start by creating the base filename */

	if (*p == '/')
		sprintf(filename_base,"%s%4d_%02d_%02d_%s_%s_",
			dir, year, month, day, star_name, name);
	else
		sprintf(filename_base,"%s/%4d_%02d_%02d_%s_",
			dir, year, month, day, star_name);

	/* OK, find a unique number */

	for (file_number=1;file_number<=999;file_number++)
        {
		/* Create the filename */

		sprintf(filename,"%s%03d.dat",filename_base,file_number);

		/* Does it already exist ? */

		if ((fp = fopen(filename,"r")) == NULL) break;

		fclose(fp);
	}

	if (file_number >= 999)
	{
		return NULL;
	}

	return filename;

} /* create_astromod_data_file_name() */

/************************************************************************/
/* message_star_data()							*/
/*									*/
/* For when we get a ASTROMOD_STAR_DATA message.	 		*/
/************************************************************************/

int message_star_data(struct smessage *message)
{
	struct smessage mess;

	if (message->length != 0)
	{
		ui_print_message(client_socket,
			"ASTROMOD_STAR_DATA with bad data.");
		return ERROR;
	}

	mess.type = ASTROMOD_STAR_DATA;
	mess.data = (unsigned char *)&star;
	mess.length = sizeof(star);

	ui_send_message(client_socket, &mess);

	return NOERROR;

} /* message_star_data() */

/************************************************************************/
/* message_uv_data()							*/
/*									*/
/* For when we get a ASTROMOD_UV_DATA message.		 		*/
/************************************************************************/

int message_uv_data(struct smessage *message)
{
	struct smessage mess;
	double  mjd;
        double ref_el;
	struct s_time_now *data;
	struct s_uv result;
	time_t  current_time;
	struct  tm      *gmt_now;
	int	i,j,k;
	double	u,v;

	if (message->length != sizeof(*data))
	{
		ui_print_message(client_socket,
			"ASTROMOD_UV_DATA with bad data.");
		return ERROR;
	}

	data = (struct s_time_now *)message->data;

	/* Do we use our own time or what? */

	if (data->year <= 0)
	{
		time(&current_time);
		gmt_now = gmtime(&current_time);

		/* Note... not really Y2K cool.... */

		data->year = gmt_now->tm_year + 1900;
		if (data->year < 1950) data->year+=100;
		data->month = gmt_now->tm_mon+1;
		data->day = gmt_now->tm_mday;
		data->ut = ((double)gmt_now->tm_sec/(24.0*60.0*60.0) +
		      (double)gmt_now->tm_min/(24.0*60.0) +
		      (double)gmt_now->tm_hour/(24.0)) * M_PI * 2.0;
	}

	mjd = astromod_mjd(data->year, data->month, data->day, data->ut);
	astromod_azel_mjd(star, mjd, &(result.az), &(result.el),
		&ref_el, &(result.ha));
	result.ut = data->ut;

	/* And now the UV coordinates */

	for(k=0,i=0; i<NUM_SCOPES-1; i++)
	for(j=i+1; j<NUM_SCOPES; j++)
	{
		sprintf(result.uv_names[k],"%s-%s", scope_types[i],
			scope_types[j]);

#warning The sign here could be wrong, but changing it now may be a problem.
		if (data->lambda != 0.0 && scope_active[i] && scope_active[j])
			calc_uv(star.dec, result.ha,
                                scopes[i].baseline.x - scopes[j].baseline.x,
                                scopes[i].baseline.y - scopes[j].baseline.y,
                                scopes[i].baseline.z - scopes[j].baseline.z,
                                data->lambda * 1.0e-6, &u, &v);
		else
			u = v = 0.0;

		result.uv[k][U] = u;
		result.uv[k][V] = v;

		k++;
	}

	/* Send the message back */

	mess.type = ASTROMOD_UV_DATA;
	mess.data = (unsigned char *)&result;
	mess.length = sizeof(result);

	ui_send_message(client_socket, &mess);

	return NOERROR;

} /* message_uv_data() */

/************************************************************************/
/* message_set_obs_wavelength()						*/
/*									*/
/* For when we get a ASTROMOD_SET_OBS_WAVELENGTH message.		*/
/************************************************************************/

int message_set_obs_wavelength(struct smessage *message)
{
	double	*data;

	if (message->length != sizeof(*data))
	{
		ui_print_message(client_socket,
			"ASTROMOD_SET_OBS_WAVELENGTH with bad data.");
		return ERROR;
	}

	data = (double *)message->data;

	/* OK, set the relevant global */

	chara.obs_wavelength = *data;

	/* That is all */

	return NOERROR;

} /* message_set_obs_wavelength() */

/************************************************************************/
/* message_get_obs_wavelength()						*/
/*									*/
/* For when we get a ASTROMOD_GET_OBS_WAVELENGTH message.		*/
/************************************************************************/

int message_get_obs_wavelength(struct smessage *message)
{
	double	*data;
	struct smessage mess;

	if (message->length != 0)
	{
		ui_print_message(client_socket,
			"ASTROMOD_GET_OBS_WAVELENGTH with bad data.");
		return ERROR;
	}

	data = (double *)message->data;

	/* OK, setup the relevant message */

	mess.type = ASTROMOD_SET_OBS_WAVELENGTH;
	mess.data = (unsigned char *)&chara.obs_wavelength;
	mess.length = sizeof(chara.obs_wavelength);

	/* Send it now */

	ui_send_message(client_socket, &mess);

	/* That is all */

	return NOERROR;

} /* message_get_obs_wavelength() */

/************************************************************************/
/* set_init_callback()							*/
/*									*/
/* Setup a callback function to be called whenever a new star is set.	*/
/* The callback function gets no data, but should return an error level.*/
/************************************************************************/

void set_init_callback(int (*callback)(void))
{
	init_callback = callback;

} /* set_init_callback() */

/************************************************************************/
/* unset_init_callback()						*/
/*									*/
/* Remove the  callback called whenever a new star is set.		*/
/************************************************************************/

void unset_init_callback(void)
{
	init_callback = NULL;

} /* unset_init_callback() */

/************************************************************************/
/* message_set_active_scopes()						*/
/*									*/
/* Message based interface to ople_set_active_scopes()			*/
/* All targets are in meters.						*/
/************************************************************************/

int message_set_active_scopes(struct smessage *message)
{
	int	i;
	char	*data;

	/* Is the message the right length? */

	if (message->length != NUM_SCOPES*sizeof(*data))
	{
	   return error(ERROR,
		"ASTROMOD_SET_ACTIVE_SCOPES message with wrong data.");
	}
	
	/* It's all OK, let's do it */

	data = (char *) (message->data);

	/* We may have to force the reference cart to be active. */

	if (!data[reference_cart])
	{
		error(ERROR,"Reference cart %s is not active.",
			scope_types[reference_cart]);
		//data[reference_cart] = TRUE;
	}

	/* Loop through all targets */

	for(i=0; i<NUM_DELAY_LINES; i++)
	{
		scope_active[i] = data[i];
	}

	/* Tell all the clients about this */

	ui_send_message_all(message);

	/* Call the callback function */

	if (active_scopes_callback != NULL) 
		 return (*active_scopes_callback)();
	else
		return NOERROR;

	return NOERROR;

} /* message_set_active_scopes() */

/************************************************************************/
/* message_get_active_scopes()						*/
/*									*/
/* Message based interface to ople_get_active_scopes()			*/
/* All targets are in meters.						*/
/************************************************************************/

int message_get_active_scopes(struct smessage *message)
{
	int	i;
	char	data[NUM_DELAY_LINES];
	struct smessage mess;

	/* Is the message the right length? */

	if (message->length != 0)
	{
	   return error(ERROR,
		"ASTROMOD_GET_ACTIVE_SCOPES message with wrong data.");
	}
	
	/* Loop through all targets */

	for(i=0; i<NUM_DELAY_LINES; i++)
	{
		data[i] = scope_active[i];
	}

	mess.type = ASTROMOD_SET_ACTIVE_SCOPES;
	mess.length = NUM_DELAY_LINES*(int)sizeof(char);
	mess.data = (unsigned char *)data;

	ui_send_message(client_socket,&mess);

	return NOERROR;

} /* message_get_active_scopes() */

/************************************************************************/
/* set_active_scope_callback()						*/
/*									*/
/* Setup a callback function to be called whenever a new star is set.	*/
/* The callback function gets no data, but should return an error level.*/
/************************************************************************/

void set_active_scopes_callback(int (*callback)(void))
{
	active_scopes_callback = callback;

} /* set_active_scopes_callback() */

/************************************************************************/
/* unset_active_scopes_callback()					*/
/*									*/
/* Remove the  callback called whenever a new star is set.		*/
/************************************************************************/

void unset_active_scopes_callback(void)
{
	active_scopes_callback = NULL;

} /* unset_active_scopes_callback() */

/************************************************************************/
/* set_active_scopes()							*/
/*									*/
/* Allows user to control which carts are active.			*/
/************************************************************************/

int set_active_scopes(int argc, char **argv)
{
	int	i;
	char	s[256];
	struct smessage mess;

	/* Cycle through the carts and see */

	for(i=0; i<NUM_SCOPES; i++)
	{
		if (argc > 1+i)
		{
			scope_active[i] = *argv[1+i] == 'y' || *argv[1+i] =='Y';
		}
		else
		{
			if (scope_active[i])
			{
				sprintf(s,"Scope %s is active.", ople_types[i]);
	    		        scope_active[i] = ask_yes_no(s,
				    "Do you want to keep it active?");
			}
			else
			{
				sprintf(s,"Scope %s is not active.",
					ople_types[i]);
	    		        scope_active[i] = ask_yes_no(s,
				    "Do you want to make it active?");
			}
		}
	}

	/* Tell all the clients */

	mess.type = ASTROMOD_SET_ACTIVE_SCOPES;
	mess.length = NUM_SCOPES*(int)sizeof(char);
	mess.data = (unsigned char *)scope_active;
	ui_send_message_all(&mess);

	/* That is all */

	if (active_scopes_callback != NULL) 
		 return (*active_scopes_callback)();
	else
		return NOERROR;

} /* set_active_scopes() */

/************************************************************************/
/* set_all_active_scopes()						*/
/*									*/
/* Set all scopes to be either active or not active.			*/
/************************************************************************/

void set_all_active_scopes(bool active_scopes)
{
	int	i;

	for(i=0; i<NUM_SCOPES; i++) scope_active[i] = active_scopes;

} /* set_all_active_scopes() */
