/************************************************************************/
/* baytech_m4.c                                                         */
/*                                                                      */
/* Routines to control the baytech M4-DAC modules.			*/
/* Assumes the following units are in the following slots:		*/
/* V71-DAC   Module 1							*/
/* V78 CRM-1 Module 2							*/
/* V81 DAC DAM-1 Module 3						*/
/* Also assumes serial port set to 115200 N81.				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar	                                        */
/* Date   : Feb 1999                                                    */
/************************************************************************/

#include <time.h>
#include "../nrc/nrc.h"
#include "../charaui/src/charaui.h"
#include "baytech.h"

#define MODULE_STRING "$BT"

#define RELAY_STATUS_FORMAT "%d:%d,%d %d" /* Unit:Module,Relay Status */
#define ADC_STATUS_FORMAT "%d:%d,%d %f" /* Unit:Module,ADC Status */

#define RELAY_MODULE	2
#define DAC_MODULE	3
#define ADC_MODULE	4

/************************************************************************/
/* open_baytech_m4_port()						*/
/*									*/
/* Opens the serial port for talking to a Baytech M4 unit.		*/
/* Returns the serial port structure. If it fails to open the port	*/
/* the port.fd is set to -1.						*/
/* Use the normal close_serial_port to close the port.			*/
/************************************************************************/

struct sserial open_baytech_m4_port(char *port_name)
{
	struct sserial port;
	char	c;

	/* Try and open the port */

	port = open_serial_port(port_name, FALSE);

	if (port.fd == -1) return port;

	/* Set it to 9600 N81 */

        if (set_serial_baud_rate(&port, B9600) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

        if (set_serial_parity(&port, OFF) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

        if (set_serial_bitlength(&port, 8) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

        if (set_serial_stopbits(&port, 1) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

	/* Use XON/XOFF handshaking */

        if (set_serial_xonxoff(&port, ON) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

        if (set_serial_hard_handshake(&port, OFF) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

	/* remove any characters there may be in the buffer */

        while(read(port.fd, &c, 1) == 1);

	/* Try and ensure we are not connected to any module */

	serial_print(port, "%s\r", MODULE_STRING);

	/* That is all */

	return port;

} /* open_baytech_m4_port() */

/************************************************************************/
/* baytech_m4_relay_status()						*/
/*									*/
/* Returns an unsigned byte containing the status info.			*/
/* Bit one on means channel one is enabled and so on. If it does not	*/
/* know the status of some channel it assumes it is zero. 		*/
/* Unfortunately the calling program has no way of knowing about an	*/
/* error. On error always returns 0x00.					*/
/************************************************************************/

unsigned char baytech_m4_relay_status(struct sserial port)
{
	unsigned char status = 0x0;
	time_t start_time;
	int	unit,module,relay,enabled;
	int 	found_data;
	char	s[81],*p;

	if (port.fd == -1) return 0x40;

	/* Disconnect from module */

	/*error(MESSAGE,"%s\r", MODULE_STRING);*/
	serial_print(port, "%s\r", MODULE_STRING);

	/* Connect to the module */

	/*error(MESSAGE,"%s%d\r",MODULE_STRING,RELAY_MODULE);*/
	serial_print(port,"%s%d\r",MODULE_STRING,RELAY_MODULE);

	/* Send the command to report */

	/*error(MESSAGE,"SA0\r");*/
	serial_print(port,"SA0\r");

	/* OK get lines until we read the lot */

	found_data = 0;
	start_time = time(NULL);
	do
	{
		/* Get stuff up to a newline or a prompt */

		for(p = s, *p = 0; p-s<80; p++)
		{
			/* Get the next character */

			while(read(port.fd, p, 1) != 1)
			{
				if (time(NULL) >= start_time+BAYTECH_TIME_OUT)
				{
					error(ERROR,
					"Timed out waiting for status data.");
					return 0x40;
				}
			}

			/* Add a terminator */

			*(p+1) = 0;
			/* error(MESSAGE,"%s",p); */

			if (*p =='\r') break; 
		}

		/* Is it a status line? */
			
		if (sscanf(s,RELAY_STATUS_FORMAT,&unit,&module,&relay,&enabled)
			== 4)
		{
			found_data++;
			if (enabled)
				status |= (1<<(relay-1));
			else
				status &= ~(1<<(relay-1));
		}

	} while(found_data < 8);

	/* Disconnect from module */

	serial_print(port, "%s\r", MODULE_STRING);

	return status;

} /* baytech_m4_relay_status() */

/************************************************************************/
/* baytech_m4_relay_energize(struct sserial port, int channel)		*/
/* 									*/
/* Attempts to send the command to energize a relay in the M4 box.	*/
/* As nothing is returned from this box it has no idea if the command	*/
/* works or not. Returns an error level anyway.				*/
/* If channel si zero all eight channels are energized.			*/
/************************************************************************/

int baytech_m4_relay_energize(struct sserial port, int channel)
{

	/* Is the port open */

	if (port.fd == -1)
	{
		return error(ERROR,"M4 Port not open.");
	}

	/* Is the channel number reasonable */

	if (channel < 0 || channel > 8)
	{
		return error(ERROR,"M4 Relay channel must be 0 to 8.");
	}

	/* Disconnect from module */

	serial_print(port, "%s\r", MODULE_STRING);

	/* Connect to the module */

	serial_print(port,"%s%d\r",MODULE_STRING,RELAY_MODULE);

	/* Send the command */

	serial_print(port,"ER%d\r",channel);

	/* Disconnect from module */

	serial_print(port, "%s\r", MODULE_STRING);

	return NOERROR;

} /* baytech_m4_relay_energize() */

/************************************************************************/
/* baytech_m4_relay_de_energize(struct sserial port, int channel)	*/
/* 									*/
/* Attempts to send the command to de-energize a relay in the M4 box.	*/
/* As nothing is returned from this box it has no idea if the command	*/
/* works or not. Returns an error level anyway.				*/
/* If channel si zero all eight channels are de-energized.		*/
/************************************************************************/

int baytech_m4_relay_de_energize(struct sserial port, int channel)
{

	/* Is the port open */

	if (port.fd == -1)
	{
		return error(ERROR,"M4 Port not open.");
	}

	/* Is the channel number reasonable */

	if (channel < 0 || channel > 8)
	{
		return error(ERROR,"M4 Relay channel must be 0 to 8.");
	}

	/* Disconnect from module */

	serial_print(port, "%s\r", MODULE_STRING);

	/* Connect to the module */

	serial_print(port,"%s%d\r",MODULE_STRING,RELAY_MODULE);

	/* Send the command */

	serial_print(port,"DR%d\r",channel);

	/* Disconnect from module */

	serial_print(port, "%s\r", MODULE_STRING);

	return NOERROR;

} /* baytech_m4_relay_de_energize() */

/************************************************************************/
/* baytech_m4_set_dac(struct sserial port, int channel, int value)	*/
/* 									*/
/* Attempts to set the voltage on a DAC.				*/
/* As nothing is returned from this box it has no idea if the command	*/
/* works or not. Returns an error level anyway.				*/
/* If channel si zero all eight channels are set.			*/
/************************************************************************/

int baytech_m4_set_dac(struct sserial port, int channel, int value)
{
	/* Is the port open */

	if (port.fd == -1)
	{
		return error(ERROR,"M4 Port not open.");
	}

	/* Is the channel number reasonable */

	if (channel < 0 || channel > 8)
	{
		return error(ERROR,"M4 DAC channel must be 0 to 8.");
	}

	/* Is the value reasonable */

	if (value < 0 || value > 4095)
	{
		return error(ERROR,"M4 DAC value must be 0 to 4095.");
	}
		
	/* Disconnect from module */

	serial_print(port, "%s\r", MODULE_STRING);

	/* Connect to the module */

	serial_print(port,"%s%d\r",MODULE_STRING,DAC_MODULE);

	/* Send the command */

	serial_print(port,"WD%d;%d\r",channel,value);

	/* Disconnect from module */

	serial_print(port, "%s\r", MODULE_STRING);

	return NOERROR;

} /* baytech_m4_set_dac() */

/************************************************************************/
/* baytech_m4_dac_array()						*/
/* 									*/
/* Attempts to send and array of voltages to a DAC.			*/
/* As nothing is returned from this box it has no idea if the command	*/
/* works or not. Returns an error level anyway.				*/
/* If channel si zero all eight channels are set.			*/
/* The values are forced to be reasonable.				*/
/* Arrays go from 0 to n-1.						*/
/************************************************************************/

int baytech_m4_dac_array(struct sserial port, int channel, 
			int values[], int n)
{
	int i;

	/* Is the port open */

	if (port.fd == -1)
	{
		return error(ERROR,"M4 Port not open.");
	}

	/* Is the channel number reasonable */

	if (channel < 0 || channel > 8)
	{
		return error(ERROR,"M4 DAC channel must be 0 to 8.");
	}

	/* Disconnect from module */

	serial_print(port, "%s\r", MODULE_STRING);

	/* Connect to the module */

	serial_print(port,"%s%d\r",MODULE_STRING,DAC_MODULE);

	/* Send the command */

	for(i=0; i<n; i++) 
	{
		if (values[i] < 0) 
			values[i] = 0;
		else if (values[i] > 4095) 
			values[i] = 4095;

		serial_print(port,"WD%d;%d\r",channel,values[i]);
	}

	/* Disconnect from module */

	serial_print(port, "%s\r", MODULE_STRING);

	return NOERROR;

} /* baytech_m4_dac_array() */

/************************************************************************/
/* baytech_m4_adc_status()						*/
/*									*/
/* Returns an float vector containing the values in the ADCs.		*/
/* If an error has occurred, returns all -1s.				*/
/* The array is allocated in the routine and must be deallocated by the */
/* calling routine.							*/
/************************************************************************/

float *baytech_m4_adc_status(struct sserial port)
{
	float	*result;
	time_t  start_time;
	int	unit,module,adc;
	float	volts;
	char	s[256],*p;
	int	found_data;

	/* Allocate memory */

	result = vector(1,16);
	for(unit =1; unit <=16; unit++) result[unit] = -1.0;

	/* Only do this if the port is open. */

	if (port.fd == -1) return result;

	/* Disconnect from module */

	//error(MESSAGE,"%s\r", MODULE_STRING);
	serial_print(port, "%s\r", MODULE_STRING);

	/* Connect to the module */

	//error(MESSAGE,"%s%d\r",MODULE_STRING,ADC_MODULE);
	serial_print(port,"%s%d\r",MODULE_STRING,ADC_MODULE);

	/* Send the command to report */

	//error(MESSAGE,"SA0\r");
	serial_print(port,"SA0\r");

	/* Now we sit and wait for some characters to come back */

	start_time = time(NULL);

	/* OK get lines until we read the lot */

	found_data = 0;
	start_time = time(NULL);
	do
	{
		/* Get stuff up to a newline or a prompt */

		for(p = s, *p = 0; p-s<80; p++)
		{
			/* Get the next character */

			//usleep(2);
			while(read(port.fd, p, 1) != 1)
			{
				if (time(NULL) >= start_time+BAYTECH_TIME_OUT)
				{
					/* error(ERROR,
					"Timed out waiting for status data.");*/
					return result;
				}
			}

			/* Add a terminator */

			*(p+1) = 0;
			//error(MESSAGE,"%s",s);

			if (*p =='\r') break; 
		}

		/* Is it a status line? */
			
		if (sscanf(s,ADC_STATUS_FORMAT,&unit,&module,&adc,&volts)
			== 4)
		{
			found_data++;
			result[adc] = volts;
			//error(MESSAGE,"adc = %d volts = %.4f",adc,volts);
		}

	} while(found_data < 16);

	/* Disconnect from module */

	serial_print(port, "%s\r", MODULE_STRING);

	return result;

} /* baytech_m4_adc_status() */

/************************************************************************/
/* baytech_m4_clear()							*/
/*									*/
/* Disconnect from the Baytech M4 module. With luck this will clear 	*/
/* a fault.								*/
/************************************************************************/

void baytech_m4_clear(struct sserial port)
{
	serial_print(port, "%s\r", MODULE_STRING);

} /* baytech_m4_clear() */
