/************************************************************************/
/* baytech_rpc.c                                                        */
/*                                                                      */
/* Routines to control the baytech RPC-2 modules.			*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                  	*/
/* Date   : Feb 1999                                                    */
/************************************************************************/

#include "../charaui/src/charaui.h"
#include "baytech.h"
#include <time.h>

#define STATUS_FORMAT "%1d)...Outlet %1d  : %s" 
					/* Format of a status line from RPC-1 */
#define CB_FORMAT "Circuit Breaker: %s" /* Format for the circuit breaker info*/

#define RPC_PROMPT "RPC>"

#define RPC_ERROR_BIT  0x40;

/************************************************************************/
/* open_baytech_rpc_port()						*/
/*									*/
/* Opens the serial port for talking to a Baytech RPC unit.		*/
/* Returns the serial port structure. If it fails to open the port	*/
/* the port.fd is set to -1.						*/
/* Use the normal close_serial_port to close the port.			*/
/************************************************************************/

struct sserial open_baytech_rpc_port(char *port_name)
{
	struct sserial port;
	char c;

	/* Try and open the port in nonblocking mode */

	port = open_serial_port(port_name, FALSE);

	if (port.fd == -1) return port;

	/* Set it to N81 */

        if (set_serial_baud_rate(&port, B9600) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

        if (set_serial_parity(&port, OFF) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

        if (set_serial_bitlength(&port, 8) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

        if (set_serial_stopbits(&port, 1) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

	/* No handshaking */

        if (set_serial_xonxoff(&port, OFF) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

        if (set_serial_hard_handshake(&port, OFF) != NOERROR)
	{
		close_serial_port(port);
		port.fd = -1;
		return port;
	}

	/* remove any characters there may be in the buffer */

	while(read(port.fd, &c, 1) == 1);

	/* That is all */

	return port;

} /* open_baytech_rpc_port() */

/************************************************************************/
/* reopen_baytech_rpc_port()						*/
/*									*/
/* Sometimes the port get's screwed up and you need to reopen it.       */
/* Returns the status information.  					*/
/* Also set's the port sent to it to the new port values.		*/
/************************************************************************/

unsigned char  reopen_baytech_rpc_port(struct sserial *port)
{
	struct sserial new_port;
	unsigned char *p1,*p2;
	int	i;

	/* OK, is it open at all? */

	if (port->fd == -1) return 0x40;

	/* OK, close the port */

	close_serial_port(*port);

	/* Now open it up again */

	new_port = open_baytech_rpc_port(port->name);
	
	/* Copy over the data */

	p1 = (unsigned char *) port;
	p2 = (unsigned char*) &new_port;
	for(i=0; i<sizeof(new_port); i++) *p1++ = *p2++;

	/* Force the thing to react to us */

#ifdef JUNK
	serial_putchar(*port,'\r');
	serial_putchar(*port,'\r');
	
	/* remove any characters there may be in the buffer */

	while(read(port->fd, &c, 1) == 1);
#endif

	/* Try and get the status info */

	serial_putchar(*port,'\r');
	return baytech_rpc_get_status(new_port);

} /* reopen_baytech_rpc_port() */

/************************************************************************/
/* baytech_rpc_get_status()						*/
/*									*/
/* Sends a carraige return to the baytech serial port and tries to	*/
/* read in the status information. This means reading untill it finds	*/
/* a prompt string.							*/
/* Returns an unsigned byte containing the status info.			*/
/* Bit one on means channel one is one and so on. If it does not	*/
/* know the status of some channel it assumes it is zero. Bit eight	*/
/* represents the circuit breaker condition.				*/
/* If an error has occured it sets bit 7 on.				*/
/* Error messages sent using message(), not error() so it doesn't stop  */
/* other processing.							*/
/************************************************************************/

unsigned char baytech_rpc_get_status(struct sserial port)
{
	unsigned char status = 0x0;
	time_t start_time;
	int	channel,line;
	char	on_or_off[81],s[81],*p;
	int 	found_data;

	if (port.fd == -1) return RPC_ERROR_BIT;

	/* First we send a carraige return */

	serial_putchar(port,'\r');

	/* OK get lines until we read the lot */

	found_data = FALSE;
	start_time = time(NULL);
	do
	{
		/* Get stuff up to a newline or a prompt */

		for(p = s, *p = 0; p-s<80; p++)
		{
			/* Get the next character */

			while(read(port.fd, p, 1) != 1)
			{
				if (time(NULL) >= start_time+BAYTECH_TIME_OUT)
				{
					message(system_window,
					"Timed out waiting for status data.");
					return RPC_ERROR_BIT;
				}
			}

			/* Add a terminator */

			*(p+1) = 0;

			if (*p == '\n' || *p =='\r' || strcmpi(RPC_PROMPT,s)==0)				break; 
		}

		/* Is it the circuit breaker line ? */
	
		if (sscanf(s,CB_FORMAT,on_or_off) == 1)
		{
			found_data = TRUE;
			if (strcmpi(on_or_off,"On") == 0) status |= 0x80;
		}

		/* Is it some other status line? */
			
		if (sscanf(s,STATUS_FORMAT,&line,&channel,on_or_off) == 3)
		{
			found_data = TRUE;
			if (strcmpi(on_or_off,"On") == 0)
			{
				status |= (1<<(channel-1));
			}
		}

	} while(strcmpi(s,RPC_PROMPT));

	if (!found_data)
	{
		message(system_window,
		"Failed to get any status data from Baytech RPC-2 port %s.",
		port.name);
		return RPC_ERROR_BIT;
	}

	return status;

} /* baytech_rpc_get_status() */

/************************************************************************/
/* baytech_rpc_on()							*/
/* 									*/
/* Turns on a given channel on the BAYTECH RPC unit.			*/
/* Returns the baytech status. Bits one to six represent channels one 	*/
/* through six. Bit seven is an error flag, bit eight is the circuit	*/
/* breaker.								*/
/* Errors are sent as messages, not using the normal error system	*/
/* so that it doesn't stop other processing.				*/
/************************************************************************/

unsigned char baytech_rpc_on(struct sserial port, int channel)
{
	char	command[81],c;
	time_t start_time;

	/* Is the port open? */

	if (port.fd < 0)
	{
		message(system_window,"Baytech RPC Port not open.");
		return RPC_ERROR_BIT;
	}

	/* Does the channel make sense ? */

	if (channel < 1 || channel > 6)
	{
		message(system_window,
			"Channel number %d invalid, should be 1-6.",
			channel);
		return RPC_ERROR_BIT;
	}

	/* Construct the command */

	sprintf(command,"on %d\r",channel);

	/* Send the command */

	serial_print(port, command);

	start_time = time(NULL);

	/* Wait for a question mark */

	do
	{
		while(read(port.fd, &c, 1) != 1)
		{
			if (time(NULL) >= start_time+BAYTECH_TIME_OUT)
			{
				message(system_window,
				"Timed out sending on %d.",channel);
				return RPC_ERROR_BIT;
			}
		}
	} while(c != '?');

	/* Now wait for a space */

	do
	{
		while(read(port.fd, &c, 1) != 1)
		{
			if (time(NULL) >= start_time+BAYTECH_TIME_OUT)
			{
				message(system_window,
				"Timed out sending on %d.",channel);
				return RPC_ERROR_BIT;
			}
		}
	} while(c != ' ');

	/* Now send the answer yes to the question */

	serial_putchar(port,'y');

	/* And return the status */

	return baytech_rpc_get_status(port);

} /* baytech_rpc_on() */

/************************************************************************/
/* baytech_rpc_off()							*/
/* 									*/
/* Turns off a given channel on the BAYTECH RPC unit.			*/
/* Returns the baytech status. Bits one to six represent channels one 	*/
/* through six. Bit seven is an error flag, bit eight is the circuit	*/
/* breaker.								*/
/* Errors are sent as messages, not using the normal error system	*/
/* so that it doesn't stop other processing.				*/
/************************************************************************/

unsigned char baytech_rpc_off(struct sserial port, int channel)
{
	char	command[81],c;
	time_t start_time;

	/* Is the port open? */

	if (port.fd < 0)
	{
		message(system_window,"Baytech RPC Port not open.");
		return RPC_ERROR_BIT;
	}

	/* Does the channel make sense ? */

	if (channel < 1 || channel > 6)
	{
		message(system_window,
				"Channel number %d invalid, should be 1-6.",
				channel);
		return RPC_ERROR_BIT;
	}

	/* Construct the command */

	sprintf(command,"off %d\r",channel);

	/* Send the command */

	serial_print(port, command);

	start_time = time(NULL);

	/* Wait for a question mark */

	do
	{
		while(read(port.fd, &c, 1) != 1)
		{
			if (time(NULL) >= start_time+BAYTECH_TIME_OUT)
			{
				message(system_window,
				"Timed out sending off %d.",channel);
				return RPC_ERROR_BIT;
			}
		}
	} while(c != '?');

	/* Now wait for a space */

	do
	{
		while(read(port.fd, &c, 1) != 1)
		{
			if (time(NULL) >= start_time+BAYTECH_TIME_OUT)
			{
				message(system_window,
				"Timed out sending off %d.",channel);
				return RPC_ERROR_BIT;
			}
		}
	} while(c != ' ');

	/* Now send the answer yes to the question */

	serial_putchar(port,'y');

	/* And return the status */

	return baytech_rpc_get_status(port);

} /* baytech_rpc_off() */

/************************************************************************/
/* baytech_rpc_all_on()							*/
/*									*/
/* Turns all six baytech ourlets on. Returns final status.		*/
/************************************************************************/

unsigned char baytech_rpc_all_on(struct sserial port)
{
	int i;
	unsigned char status;

	status = 0;
        if (port.fd > -1)
        {
                for(i=1; i<= 6; i++)
                {
                        message(system_window,"Turning off RPC channel %d.",i);
                        if ((status = baytech_rpc_on(port,i)) & 0x40)
                        {
                                if ((status = baytech_rpc_on(port,i)) & 0x40)
                                {
                                        status = baytech_rpc_on(port,i);
                                }
                        }
                }
        }
	return status;

} /* baytech_rpc_all_on() */

/************************************************************************/
/* baytech_rpc_all_off()						*/
/*									*/
/* Turns all six baytech ourlets off. Returns final status.		*/
/************************************************************************/

unsigned char baytech_rpc_all_off(struct sserial port)
{
	int i;
	unsigned char status;

	status = 0;
        if (port.fd > -1)
        {
                for(i=1; i<= 6; i++)
                {
                        message(system_window,"Turning off RPC channel %d.",i);
                        if ((status = baytech_rpc_off(port,i)) & 0x40)
                        {
                                if ((status = baytech_rpc_off(port,i)) & 0x40)
                                {
                                        status = baytech_rpc_off(port,i);
                                }
                        }
                }
        }
	return status;

} /* baytech_rpc_all_off() */
