/************************************************************************/
/* bunker_clock.c 							*/
/*                                                                      */
/* Library routines for dealing with the bunker GPS unit.		*/
/* This version for non-rt 2.6 kernels.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@@chara.gsu.edu                                       */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Nov 2009							*/
/************************************************************************/

/* Include the header file */

#include "../../modules/bcdriver/bcdriver.h"
#include "../charaui/src/charaui.h"
#include "bunker_clock.h"
#include "../clock/clock.h"
#include "../include/chara_messages.h"
#include "../sockman/sockman.h"

static FILE *bcfp = NULL; /* File pointer for talking to the driver */
static struct BUNKER_TIME_STRUCT bunker_clock;

/************************************************************************/
/* bunker_clock_open()							*/
/*									*/
/* Opens device driver.							*/
/* Returns TRUE if all goes well.					*/
/************************************************************************/

bool bunker_clock_open(void)
{
	if (bcfp != NULL) return TRUE;

	if ((bcfp = fopen(BC_NODE_NAME, "r+")) == NULL) return FALSE;

	if (fread(&bunker_clock, sizeof(bunker_clock), 1, bcfp) != 1)
		return FALSE;

	return TRUE;

} /* bunker_clock_open() */

/************************************************************************/
/* bunker_clock_close()							*/
/*									*/
/* Closes things down for using the bunker clock module.		*/
/************************************************************************/

void bunker_clock_close(void)
{
        if (bcfp == NULL) return;

	fclose(bcfp);
	bcfp = NULL;

} /* bunker_clock_close() */

/************************************************************************/
/* bunker_clock_reset()							*/
/*									*/
/* Forces bunker clock to reset. Also stops the clock.			*/
/* Returns 0 if all goes well.						*/
/* -1 if memory not there.						*/
/* -2 if we fail to send the commands.					*/
/************************************************************************/

int bunker_clock_reset(void)
{
	unsigned char arg;

	if (bcfp == NULL) return -1;

	/* Tell the user */

	message(system_window,"Reseting Bunker Clock.");

	/* 
	 * Select PIC for TX and RX.
	 * Stop it from Running.
	 * Initialize reset
	 * Should be 0x14
	 */

	arg = BUNKER_CLOCK_RX_PIC | BUNKER_CLOCK_TX_PIC | BUNKER_CLOCK_RUN;

	/* Send it */

	if (bunker_clock_set(arg))
	{
		werase(system_window);
		wrefresh(system_window);
		return -2;
	}

	/* Wait for this to happen */

	usleep(25000);

	/* 
	 * Select PIC for TX and RX.
	 * Ensure it doesn't run but go to normal operations.
	 * This should be 0x15
	 */

	arg = BUNKER_CLOCK_RX_PIC | BUNKER_CLOCK_TX_PIC | BUNKER_CLOCK_RUN |
			BUNKER_CLOCK_RESET;

	/* Send it */

	if (bunker_clock_set(arg))
	{
		werase(system_window);
		wrefresh(system_window);
		return -2;
	}

	/* Wait for the reset to complete */

	sleep(5);

	/* 
	 * Select PIC for TX and RX.
	 * Let it running.
	 * This should be 0x13
	 */

	arg = BUNKER_CLOCK_RX_PIC | BUNKER_CLOCK_TX_PIC | BUNKER_CLOCK_RESET;

	/* Send it */

	if (bunker_clock_set(arg))
	{
		werase(system_window);
		wrefresh(system_window);
		return -2;
	}

	/* That is all */

	werase(system_window);
	wrefresh(system_window);
	return 0;

} /* bunker_clock_reset() */

/************************************************************************/
/* bunker_clock_stop()							*/
/*									*/
/* Forces bunker clock to stop. Also stops the clock.			*/
/* Returns 0 if all goes well.						*/
/* -1 if memory not there.						*/
/* -2 if we fail to send the command.					*/
/************************************************************************/

int bunker_clock_stop(void)
{
	unsigned char arg;

	if (bcfp == NULL) return -1;

	/* Unset the relevant bit */

	arg = bunker_clock.control_reg | BUNKER_CLOCK_RUN;

	/* Send it */

	if (bunker_clock_set(arg)) return -2;

	/* That is all */

	return 0;

} /* bunker_clock_stop() */

/************************************************************************/
/* bunker_clock_start()							*/
/*									*/
/* Forces bunker clock to start. Also starts the clock.			*/
/* Returns 0 if all goes well.						*/
/* -1 if memory not there.						*/
/* -2 if we fail to send the command.					*/
/************************************************************************/

int bunker_clock_start(void)
{
	unsigned char arg;

	if (bcfp == NULL) return -1;

	/* Unset the relevant bit */

	arg = bunker_clock.control_reg & ~BUNKER_CLOCK_RUN;

	/* Send it */

	if (bunker_clock_set(arg)) return -2;

	/* That is all */

	return 0;

} /* bunker_clock_start() */

/************************************************************************/
/* bunker_clock_set()							*/
/*									*/
/* Set control register on bunker clock card.				*/
/* Returns 0 if all goes well.						*/
/* -2 if we fail to send the command.					*/
/************************************************************************/

int bunker_clock_set(unsigned char arg)
{
	int	i;

	/* Send it */

	if ((i=fwrite(&arg, sizeof(unsigned char), 1, bcfp)) != 1)
	{
		error(MESSAGE,"%d",i);
		return -2;
	}
	fflush(bcfp);

	/* That is all */

	return 0;

} /* bunker_clock_set() */

/************************************************************************/
/* call_bunker_clock_reset()						*/
/*									*/
/* User callable version of bunker_clock_reset()			*/
/************************************************************************/

int call_bunker_clock_reset(int argc, char **argv) 
{
	if (bunker_clock_reset() == 0)
		return NOERROR;
	else
		return error(ERROR, "Failed to reset clock.");
}

/************************************************************************/
/* call_bunker_clock_stop()						*/
/*									*/
/* User callable version of bunker_clock_stop()				*/
/************************************************************************/

int call_bunker_clock_stop(int argc, char **argv) 
{
	if (bunker_clock_stop() == 0)
		return NOERROR;
	else
		return error(ERROR, "Failed to stop clock.");
}

/************************************************************************/
/* call_bunker_clock_start()						*/
/*									*/
/* User callable version of bunker_clock_start()			*/
/************************************************************************/

int call_bunker_clock_start(int argc, char **argv) 
{
	int i;

	if ((i=bunker_clock_start()) == 0)
		return NOERROR;
	else
		return error(ERROR, "Failed to start clock (%d).",i);
} 

/************************************************************************/
/* call_bunker_clock_set()						*/
/*									*/
/* User callable version of bunker_clock_set()				*/
/************************************************************************/

int call_bunker_clock_set(int argc, char **argv) 
{
	unsigned char arg;
	int	i;

	if (argc != 2) bunker_clock_set(0xd);
	else
	{	sscanf(argv[1], "%x", &i);
		if (i < 0 || i > 255)
			return error(ERROR,"Out of range.");
		arg = (unsigned char)i;
		bunker_clock_set(arg);
	}

	return NOERROR;

} /* call_bunker_clock_set() */

/************************************************************************/
/* show_bunker_clock_struct() 						*/
/*									*/
/* DIsplays information contatined in the bunker clock structure.	*/
/************************************************************************/

int	show_bunker_clockstruct(int argc, char **argv)
{
	char	loop;
	int	keystroke;
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
#endif

        if (bcfp == NULL)
		return error(ERROR,"No bunker clock to show.");

	no_socket();

	active_window = sub_main_window;
	loop = TRUE;
	while(loop)
	{
		if (fread(&bunker_clock, sizeof(bunker_clock), 1, bcfp) != 1)
			return error(ERROR,
				"Failed to read bunker clock structurer.");

		heading(heading_window,"BUNKER CLOCK DATA","");

		/* run the background */

		background();

		/* And service the clients */

		process_message_socket();

		/* Display it */

		werase(sub_main_window);
		message(system_window,CONT_TEXT);

		mvwprintw(sub_main_window,0,1,"UT DATE: %4d-%02d-%02d",
			bunker_clock.year+2000, bunker_clock.month,
			bunker_clock.day);
		mvwprintw(sub_main_window,1,1,"UT TIME:   %2d:%02d:%02d",
			bunker_clock.hour, bunker_clock.min,
			bunker_clock.sec);
		mvwprintw(sub_main_window,2,1,"Msec   :        %03d",
			bunker_clock.msec);
		mvwprintw(sub_main_window,3,1,"CHARA  :   %8d",
			bunker_clock.chara_now);
		mvwprintw(sub_main_window,4,1,"Num Sat:        %3d",
			bunker_clock.num_sat);
		mvwprintw(sub_main_window,5,1,"Num Trk:        %3d",
			bunker_clock.num_tracked);

		mvwprintw(sub_main_window,0,29,"Pos prop mode: %3s",
		(bunker_clock.pico_t_status&BUNKER_POSITION_PROPAGATE_MODE)
			?"YES":" NO");
		mvwprintw(sub_main_window,1,29,"Poor geometry: %3s",
		(bunker_clock.pico_t_status&BUNKER_POOR_GEOMETRY)
			?"YES":" NO");
		mvwprintw(sub_main_window,2,29,"3D Fix       : %3s",
		(bunker_clock.pico_t_status&BUNKER_3D_FIX)
			?"YES":" NO");
		mvwprintw(sub_main_window,3,29,"2D Fix       : %3s",
		(bunker_clock.pico_t_status&BUNKER_2D_FIX)
			?"YES":" NO");
		mvwprintw(sub_main_window,4,29,"ACQ Sat      : %3s",
		(bunker_clock.pico_t_status&BUNKER_ACQUIRING_SATELLITES)
			?"YES":" NO");
		mvwprintw(sub_main_window,5,29,"Diff Fix     : %3s",
		(bunker_clock.pico_t_status&BUNKER_DIFFERENTIAL_FIX)
			?"YES":" NO");
		mvwprintw(sub_main_window,6,29,"Insuff Sat   : %3s",
		(bunker_clock.pico_t_status&BUNKER_INSUFFICIENT_SATELLITES)
			?"YES":" NO");
		mvwprintw(sub_main_window,7,29,"Bad Almanac  : %3s",
		(bunker_clock.pico_t_status&BUNKER_BAD_ALMANAC)
			?"YES":" NO");

		mvwprintw(sub_main_window,0,57,"PIC time     : %3s",
		(bunker_clock.card_status&BUNKER_PIC_TIME)
			?"YES":" NO");
		mvwprintw(sub_main_window,1,57,"GPS time     : %3s",
		(bunker_clock.card_status&BUNKER_GPS_TIME)
			?"YES":" NO");

		mvwprintw(sub_main_window,2,57,"Reset        : %3s",
		(bunker_clock.control_reg&BUNKER_CLOCK_RESET)
			?" NO":"YES");
		mvwprintw(sub_main_window,3,57,"Run          : %3s",
		~(bunker_clock.control_reg&BUNKER_CLOCK_RUN)
			?"YES":" NO");
		mvwprintw(sub_main_window,4,57,"PIC Serial TX: %3s",
		(bunker_clock.control_reg&BUNKER_CLOCK_TX_PIC)
			?"YES":" NO");
		mvwprintw(sub_main_window,5,57,"PIC Serial XX: %3s",
		(bunker_clock.control_reg&BUNKER_CLOCK_RX_PIC)
			?"YES":" NO");

		mvwprintw(sub_main_window,7,57,"Second Tick  : %3s",
		bunker_clock.second_tick?"YES":" NO");
		wrefresh(sub_main_window);

		/* Was a key hit ? */

		if (kbhit())
		{
			keystroke = get_command();

			/* Was it a mouse click? */
#ifdef __NCURSES_H
			if (keystroke == KEY_MOUSE)
			{
				getmouse(&mouse);

				if (mouse.bstate == BUTTON1_CLICKED)
				{
					if (mouse.y == SYSTEM_BEGIN_Y &&
					mouse.x <  strlen(CONT_TEXT)+2)
					{
						keystroke = KEY_ESC;
					}
				}
			}

#endif
			/* Was it KEY_ESC? */

			if (keystroke == KEY_ESC) 
			{
				loop = FALSE;
			}
			else
			{
				/* Is it a command ? */

				command_processor(keystroke);
			}
		}
	}
	werase(system_window);
	wrefresh(system_window);

	return NOERROR;

} /* show_bunker_clockstruct() */

/************************************************************************/
/* sync_machine()							*/
/*									*/
/* Routine to get another machine synced up to the time on this one.	*/
/* This is the same as the equivalent function on GPS only it uses	*/
/* the bunker clock to get the time.					*/
/************************************************************************/

int sync_machine(int argc, char **argv)
{
	char    name[LENGTH_CL+1];
	char    s[81];
	ap_time_struct ap_time;
	CHARA_TIME time_next_second;
	struct s_process_sockets process;

	/* check out the command line */

        if (argc > 1)
        {
                strcpy(name,argv[1]);
        }
        else
        {
                clean_command_line();
                werase(system_window);
                wrefresh(system_window);
                strcpy(name,"       ople");
                if (quick_edit("Process to sync",name,name,NULL,STRING)
                        == KEY_ESC) return NOERROR;
        }

	/* Go find out about the process */

	if (socket_manager_get_process(name, &process) != 0)
	{
		return error(ERROR, "Process %s doesn't seem to exist.", name);
	}
	
	message(system_window,"Syncing process %s.",name);

	/* OK, Get time of next second */

        if ((time_next_second=get_time_next_second(TRUE)) == -1)
        {
		if ((client_socket != -1 || active_socket != -1) 
                        && active_menuwindow != NULL) 
                {
                        touchwin(active_menuwindow);
                        wrefresh(active_menuwindow);
                }
                return error(ERROR, "Failed to get chara time of next second.");
        }

	/* Set the time on the given machine */

	ap_time = rt_to_ap_time(time_next_second);
	sprintf(s,"settime %02d:%02d:%02d",
		ap_time.hrs,ap_time.min,ap_time.sec);

        if (send_socket_command(process.machine, process.commands,s) != NOERROR)
	{
		if ((client_socket != -1 || active_socket != -1) 
                        && active_menuwindow != NULL) 
                {
                        touchwin(active_menuwindow);
                        wrefresh(active_menuwindow);
                }
                return error(ERROR , "Failed to sync clock on %s", name);
	}
	message(system_window,"Sync complete");

        /* Now wait for a second while this happens */

        sleep(1);

        /* And call the clrcnt function  on that machine */

        if (send_socket_command(process.machine, process.commands, "clrcnt") 
			!= NOERROR)
	{
		if ((client_socket != -1 || active_socket != -1) 
                        && active_menuwindow != NULL) 
                {
                        touchwin(active_menuwindow);
                        wrefresh(active_menuwindow);
                }
                return error(ERROR,"Failed to clear counters on %s",name);
	}
	message(system_window,"Cleared Counters");

	/* Tell client if that's who asked for this */

	if (client_socket != -1) 
		ui_print_message(client_socket, 
			"Machine %s synced to UT clock.", name);
	
	/* That is all */

	if ((client_socket != -1 || active_socket != -1) 
		&& active_menuwindow != NULL) 
	{
		touchwin(active_menuwindow);
		wrefresh(active_menuwindow);
	}

	/* Tell central scrutinizer what we've done */


	return NOERROR;

} /* sync_machine() */

/************************************************************************/
/* bunker_clock_start_message()                                         */
/*                                                                      */
/* Message based interface to bunker_clock_start()                      */
/************************************************************************/

int bunker_clock_start_message(struct smessage *mess)
{
	return call_bunker_clock_start(0, NULL);

} /* bunker_clock_start_message() */
	
/************************************************************************/
/* bunker_clock_stop_message()                                          */
/*                                                                      */
/* Message based interface to bunker_clock_stop()                       */
/************************************************************************/

int bunker_clock_stop_message(struct smessage *mess)
{
	return call_bunker_clock_stop(0, NULL);

} /* bunker_clock_stop_message() */
	
/************************************************************************/
/* bunker_clock_reset_message()                                         */
/*                                                                      */
/* Message based interface to bunker_clock_reset()                      */
/************************************************************************/

int bunker_clock_reset_gps_message(struct smessage *mess)
{
	return call_bunker_clock_reset(0, NULL);

} /* bunker_clock_reset_message() */
	
/************************************************************************/
/* bunker_clock_msync_message()                                         */
/*                                                                      */
/* Message based interface to sync_machine()                            */
/************************************************************************/

int bunker_clock_msync_message(struct smessage *mess)
{
        char    *data;
        char    *argv[2];

        if (mess->length <= 1)
        {
                return error(ERROR,"Wrong data for BUNKR_CLOCK_MSYNC message.");
        }

        /* It's all OK, let's do it */

        data = (char *) (mess->data);
        argv[0] = "msync";
        argv[1] = data;

        return sync_machine(2, argv);

} /* bunker_clock_msync_message() */

/************************************************************************/
/* setup_standard_bunker_clock_messages()                               */
/*                                                                      */
/* Sets up the standard BUNKER CLOCK messages.                          */
/************************************************************************/

void setup_standard_bunker_clock_messages(void)
{
        ui_add_message_job(BUNKER_CLOCK_STOP, bunker_clock_stop_message);
        ui_add_message_job(BUNKER_CLOCK_START, bunker_clock_start_message);
        ui_add_message_job(BUNKER_CLOCK_RESET_GPS, 
		bunker_clock_reset_gps_message);
        ui_add_message_job(BUNKER_CLOCK_MSYNC, bunker_clock_msync_message);

} /* setup_standard_bunker_clock_messages() */

/************************************************************************/
/* bunker_clock_date()							*/
/*									*/
/* Returns the data according to the bunker clock.			*/
/************************************************************************/

void bunker_clock_date(int *year, int *month, int *day)
{
	if (fread(&bunker_clock, sizeof(bunker_clock), 1, bcfp) != 1)
		return error(ERROR,
			"Failed to read bunker clock structurer.");

	*year = bunker_clock.year+2000;
	*month = bunker_clock.month;
	*day = bunker_clock.day;

} /* bunker_clock_date() */
