/***************************************************************************
 *   Copyright (C) 2007 by Arnaud Sevin
 *   Arnaud.Sevin@obspm.fr
 * 
 *   With the help of Eric Gendron, Zoltan Hubert, Denis Peret 
 *
 * Copyright (c) 1998, Regents of the University of California
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
#ifndef COM_PERSEE_H
#define COM_PERSEE_H

#include <sys/time.h>
#include <cstdlib>

#include <cstdio>

#include "dico_persee.h"
#include "tFTDI.h"

/***************************************************
 *                                                 *
 *    Error codes, returned by most functions      *
 *                                                 *
 ***************************************************/

typedef enum
{
    PerseeSuccess       = 0,        // No error
    PerseeCameraFault   = 1,        // Unexpected camera fault

} PerseeErr;

/***************************************************
 *                                                 *
 *          Classe Frame_PERSEE                    *
 *                                                 *
 ***************************************************/
/**
 * @brief Classe permettant de gerer les trames PERSEE
 * @author Arnaud Sevin <Arnaud.Sevin@obspm.fr>
 * @version  1.0
 * @date     02/06/2008
 */
class Frame_PERSEE{
public:
  /// Trame d'acquisition ou pas
  bool acqui;
  /// Valeurs du Housekeeping
  unsigned int HK[HK_SIZE];
  /// Nombre de pixels de l'image
  unsigned int nb_pix;
  /// Nombre de lecture consecutive du meme pixel
  unsigned int nb_read;
  /// Nombre de lecture de la cible entre deux reset
  unsigned int nb_loop;
  /// Tableau des pixels de la trame
  unsigned short int *donnees;

  /// verification du checksum
  bool checkData;

  /**
   * Permet de creer une trame de nb_pix pixels
   * @param nb_pix nombre de pixels de la trame
   */
  Frame_PERSEE(unsigned int nb_pix, unsigned int nb_read, unsigned int nb_loop,  bool checkData){
    acqui=false;
    bzero(HK, HK_SIZE*sizeof(int));
    this->nb_pix = nb_pix*(nb_loop+1)*nb_read;
    this->nb_read = nb_read;
    this->nb_loop = nb_loop;
    this->donnees = new unsigned short int[this->nb_pix];
    bzero(donnees, this->nb_pix*sizeof(short int));
    this->checkData=checkData;
  }

  /**
   * Permet de creer une trame a partir d'une autre
   * @param frame Trame a copier
   */
  Frame_PERSEE(const Frame_PERSEE& frame){
    acqui=frame.acqui;
    memcpy(HK, frame.HK, HK_SIZE*sizeof(int));
    nb_pix = frame.nb_pix;
    nb_read = frame.nb_read;
    nb_loop = frame.nb_loop;
    this->donnees = new unsigned short int[nb_pix];
    memcpy(donnees, frame.donnees, nb_pix*sizeof(short int));
    this->checkData=checkData;
   }

  ~Frame_PERSEE(){
    delete donnees;
  }

  /**
   * Permet de copier une trame a partir d'une autre
   * @param frame Trame a copier
   */
  Frame_PERSEE& operator= (const Frame_PERSEE& frame)
    {
      delete this;
      Frame_PERSEE* tmp = new Frame_PERSEE(frame);
      return *tmp;
    }

};


/***************************************************
 *                                                 *
 *          Classe Com_PERSEE                      *
 *                                                 *
 ***************************************************/
/**
 * @brief Classe permettant de gerer la communication avec l'electronique de PERSEE
 * @author   Arnaud Sevin <Arnaud.Sevin@obspm.fr>
 * @version  1.0
 * @date     02/06/2008
 */
class Com_PERSEE : public tFTDI {

public:
  /**
   * Permet de configurer la communication avec PERSEE
   * @param pDevice Port serie de communication
   * @param Vitesse Vitesse du port
   * @param NONBLOCK Est-il bloquant ?
   */
  Com_PERSEE(string pDevice, int Vitesse=9600, bool NONBLOCK=true);
  
  ~Com_PERSEE();
  
  /**
   * Permet de recevoir une trame de donnees
   * @param frame Trame recevant les donnees
   * @param Attente Timeout
   * @return TIMEOUT ou le nombre de pixels recus
   */
  int RecevoirTrame(Frame_PERSEE *frame, unsigned int Attente);
  int jouflu_RecevoirTrame(Frame_PERSEE *frame, unsigned int Attente);
  int jouflu_clearftdi(void);
  int RecevoirTrame_new(Frame_PERSEE *frame, unsigned int Attente);
  int RecevoirTrame2(Frame_PERSEE *frame, unsigned int Attente);

  /**
   * Permet de recevoir une trame de HKs
   * @param frame Trame recevant les HKs
   * @param Attente Timeout
   * @return TIMEOUT ou le nombre de HKs recus
   */
  int RecevoirHK(Frame_PERSEE *frame, unsigned int Attente);

  /**
   * Permet de envoyer une demande de RESET
   * @param Attente Timeout
   * @return TIMEOUT ou OK
   */
  int Reset(unsigned int Attente);

  /**
   * Permet de envoyer une demande d'acquisition
   * @param Attente Timeout
   * @return TIMEOUT ou OK
   */
  int Acqui(unsigned int Attente);

  /**
   * Permet de modifier de la division d'horloge
   * @param division division d'horloge souhaitee
   * @param Attente Timeout
   * @return TIMEOUT, ERREUR ou OK
   */
  int Set_division(unsigned short division, unsigned int Attente);

  /**
   * Permet de modifier de la sommation d'images
   * @param sommation sommation d'images souhaitee
   * @param Attente Timeout
   * @return TIMEOUT, ERREUR ou OK
   */
  int Set_sommation(unsigned short sommation, unsigned int Attente);

  /**
   * Permet de positionner de la roue a filtres
   * @param position positionnement de la roue a filtres souhaitee
   * @param Attente Timeout
   * @return TIMEOUT, ERREUR ou OK
   */
  int Set_filter(unsigned short position, unsigned int Attente);

  /**
   * Permet de realigner de la roue a filtres
   * @param Attente Timeout
   * @return TIMEOUT, ERREUR ou OK
   */
  int Set_filterhome(unsigned int Attente);
  int Save_filter(unsigned short position, unsigned int Attente);
  int Move_filter(unsigned short position, int Attente);
  int Get_position(int Attente);
  int Read_ADC(unsigned short position, int Attente);
  int Get_version(int Attente);
  int Set_cumul(unsigned short cumul, int Attente);
  int Nb_HK(unsigned short nb_HK, int Attente);
  int Set_motors(unsigned short enable, int Attente);

  /**
   * Permet de changer le temps d'exposition
   * @param exposure Temps d'exposition souhaite
   * @param Attente Timeout
   * @return TIMEOUT, ERREUR ou OK
   */
  int Set_exposure(unsigned int exposure, unsigned int Attente);
  int Set_latency(unsigned int latency, unsigned int Attente);

  /**
   * Permet de creer d'une fenetre
   * @param x Abcisse (haut)
   * @param y Ordonnee (gauche)
   * @param size Taille du cote de la fenetre
   * @param Attente Timeout
   * @return TIMEOUT, ERREUR ou OK
   */
  int Create_window(unsigned short x, unsigned short y, unsigned short size, unsigned int Attente);
  int Set_bias(unsigned short bias, unsigned short value, unsigned int Attente);

  /**
   * Permet de selectionner un banque de fenetres
   * @param bank Banque a selectionner
   * @param Attente Timeout
   * @return TIMEOUT, ERREUR ou OK
   */
  int Select_bank(unsigned short bank, unsigned int Attente);

  /**
   * Permet de definir le nombre de lectures d'un meme pixel avant de passer au suivant
   * @param nb_read Nombre de lectures d'un pixel
   * @param Attente Timeout
   * @return TIMEOUT, ERREUR ou OK
   */
  int Set_nbread(unsigned short nb_read, unsigned int Attente);

  /**
   * Permet de definir le nombre de lectures de cible entre deux RESET
   * @param nb_loop Nombre de lectures de la cible
   * @param Attente Timeout
   * @return TIMEOUT, ERREUR ou OK
   */
  int Set_nbloop(unsigned short nb_loop, unsigned int Attente);

  /**
   * Permet de trigger la camera
   */
  int trigg(unsigned int Attente);

  /**
   * Permet de stopper la camera
   */
  int triggOff(unsigned int Attente);

  /**
   * Permet de ... la camera
   */
  int Set_nbpix(unsigned int nb_pix, unsigned int Attente);
  int Set_pause(unsigned char pause, unsigned int Attente);
  int Set_trigg(unsigned char trigg, unsigned int Attente);

  int Set_topleft(unsigned char top, unsigned char left, unsigned int Attente);
protected:

  /**
   * Permet d'envoyer un message et de retourner le resultat
   * @param message Le message a envoyer
   * @param Attente Timeout
   * @return La reponse sinon OK, ERREUR ou TIMEOUT
   */
  int Envoyer(char *message, unsigned int Attente);
  
  /**
   * Permet de convertir un entier en une chaine de 2 caracteres
   * @param tmpInt Entier a convertir
   * @return Chaine de 2 caracteres representant l'entier
   */
  char *int2char(unsigned int tmpInt);

  /**
   * Permet de convertir un entier en une chaine de 3 caracteres
   * @param tmpInt Entier a convertir
   * @return Chaine de 3 caracteres representant l'entier
   */
  char *int3char(unsigned int tmpInt);

  /**
   * Permet de convertir une chaine de caracteres en un tableau d'entier cod©s sur 2 caract¨res
   * @param tmpInt[] Tableau d'entier a remplir
   * @param nb_elems Taille du tableau
   * @param tmpStr   Chaine de caracteres a convertir
   */
  void char2int(unsigned int tmpInt[], unsigned int nb_elems, char *tmpStr);

  /**
   * Permet de convertir une chaine de caracteres en un tableau d'entier cod©s sur 3 caract¨res
   * @param tmpInt[] Tableau d'entier a remplir
   * @param nb_elems Taille du tableau
   * @param tmpStr   Chaine de caracteres a convertir
   */
  void char3int(unsigned int tmpInt[], unsigned int nb_elems, char *tmpStr);

 private:
  char buffer[16];

};

#endif
