/*********************************************************************
 *
 * ANSI C Example program:
 *    ReadDigPort-ExtClk.c
 *
 * Example Category:
 *    DI
 *
 * Description:
 *    This example demonstrates how to input a finite digital pattern
 *    using an external clock.
 *
 * Instructions for Running:
 *    1. Select the Physical Channel to correspond to where your
 *       signal is input on the DAQ device.
 *    2. Select the Clock Source for the acquisition.
 *    3. Select how many Samples to Acquire.
 *    4. Set the approximate Sample Clock Rate of the external clock.
 *       This allows the internal characteristics of the acquisition
 *       to be as efficient as possible.
 *
 * Steps:
 *    1. Create a task.
 *    2. Create a Digital Input channel.
 *    3. Define the parameters for an External Clock Source.
 *       Additionally, set the sample mode to be finite.
 *    4. Call the Start function to begin the acquisition.
 *    5. Read the digital pattern. Set a timeout so an error is
 *       returned if the samples are not returned in the specified
 *       time limit.
 *    6. Call the Clear Task function to clear the Task.
 *    7. Display an error if any.
 *
 * I/O Connections Overview:
 *    Make sure your signal input terminal matches the Physical
 *    Channel I/O Control. Also, make sure your external clock
 *    terminal matches the Clock Source Control. For further
 *    connection information, refer to your hardware reference manual.
 *
 *********************************************************************/

#include <stdio.h>
#include <com_persee.h>
#include <NIDAQmx.h>

TaskHandle NIinitCard(uInt64 sampleToAcquire){
  int32		error=0;
  TaskHandle	taskHandle=0;
  char		errBuff[2048]={'\0'};

  /*********************************************/
  // DAQmx Configure Code
  /*********************************************/
  if (DAQmxFailed(error=DAQmxCreateTask("",&taskHandle))){
    DAQmxGetExtendedErrorInfo(errBuff,2048);
    fprintf(stderr,"DAQmx Error: %s\n",errBuff);
    return 0L;
  }
  if (DAQmxFailed(error=DAQmxCreateDIChan(taskHandle,"Dev2/port2_16","",DAQmx_Val_ChanForAllLines))){
    DAQmxGetExtendedErrorInfo(errBuff,2048);
    fprintf(stderr,"DAQmx Error: %s\n",errBuff);
    return 0L;
  }
  float64 rate = 1000000.0;
  uInt64  sampsPerChanToAcquire = sampleToAcquire;

  if (DAQmxFailed(error=DAQmxCfgSampClkTiming(taskHandle,"PFI3",rate,DAQmx_Val_Rising,DAQmx_Val_FiniteSamps,sampsPerChanToAcquire))){
    DAQmxGetExtendedErrorInfo(errBuff,2048);
    fprintf(stderr,"DAQmx Error: %s\n",errBuff);
    return 0L;
  }

  /*********************************************/
  // DAQmx Start Code
  /*********************************************/
  //DAQmxErrChk (DAQmxStartTask(taskHandle));
  if (DAQmxFailed(error=DAQmxCfgDigEdgeStartTrig ( taskHandle,"PFI7" , DAQmx_Val_Rising))){
    DAQmxGetExtendedErrorInfo(errBuff,2048);
    fprintf(stderr,"DAQmx Error: %s\n",errBuff);
    return 0L;
  }
  if (DAQmxFailed(error=DAQmxTaskControl ( taskHandle, DAQmx_Val_Task_Commit ))){
    DAQmxGetExtendedErrorInfo(errBuff,2048);
    fprintf(stderr,"DAQmx Error: %s\n",errBuff);
    return 0L;
  }
  return taskHandle;
}

int NIRecevoirTrame(TaskHandle	taskHandle, Frame_PERSEE* frame)
{
  uInt64 sampleToAcquire = frame->nb_pix;
  int32		error=0;
  int32		sampsRead;
  char		errBuff[2048]={'\0'};

  frame->acqui = 1;
  /*********************************************/
  // DAQmx Read Code
  /*********************************************/
  float64 timeout = 1.0;
  int32 numBytesPerSamp;
  if (DAQmxFailed(error=DAQmxReadRaw(taskHandle,-1,timeout,frame->donnees,sampleToAcquire*sizeof(short int),&sampsRead, &numBytesPerSamp,NULL))){
    DAQmxGetExtendedErrorInfo(errBuff,2048);
    fprintf(stderr,"DAQmx Error: %s\n",errBuff);
    return ERREUR;
  }
  return sampsRead;
}

int NIstopCard(TaskHandle	taskHandle)
{
  if( taskHandle!=0 ) {
    /*********************************************/
    // DAQmx Stop Code
    /*********************************************/
    DAQmxStopTask(taskHandle);
    DAQmxTaskControl ( taskHandle, DAQmx_Val_Task_Unreserve );
    DAQmxClearTask(taskHandle);
  }

  //cerr << 5 << endl;
  //usleep(50000);
  return 1;
}
