/************************************************************************/
/* fringe_track.c                                                       */
/*                                                                      */
/* ROutines to analyse fringe packets, look for fringes etc.		*/
/* THis version used the realft routine and had trouble.		*/
/* It also has the math for the apodization which the new version	*/
/* doesn't use.								*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar	                                        */
/* Date   : April 2005							*/
/************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "../nrc/nrc.h"
#include <unistd.h>
#include <math.h>
#include "chara.h"
#if defined(macintosh) || (defined(__MACH__) && defined(__APPLE__))
#include <malloc/malloc.h>
#else
#include <malloc.h>
#endif

/************************************************************************/
/* fft_fringe()								*/
/*									*/
/* Routine to calculate the FFT of a fringe packet. It does zero	*/
/* padding and places it in an array of the size specified by the caller*/
/* Returns the FFT[1..fft_n] of the fringe.				*/
/* Returns the FFT, as defined by the realft routine in Numerical 	*/
/* Recipies, or a NULL of it fails. Memory is allocated here.		*/
/* USes a copy of the data so data is not changed.			*/
/************************************************************************/

float	*fft_fringe(float *fringe,	/* [1..n] The data of the fringe */
		    int n,		/* How many data points in fringe? */
		    int fft_n,		/* How many points in the FFT? */
		    char subtract_mean, /* Do we subtract the mean first? */
		    char apodize)	/* Do we wish to apodize or use mean? */
{
	float	*fft;	/* Here's where the data goes */
	float	*copy;
	float	mean;
	int	start;
	float	y, c, m, mult;
	int	apodize_length;
	int	i;

	/* Is this big enough? */

	if (fft_n < n) return NULL;
	
	/* Where do we put it in the FFT array? */

	start = (fft_n - n)/2;
	if (start < 4) return NULL;

	/* Allocate the memory */

	fft = vector(1, fft_n);
	copy = vector(1, n);

	/* Workout mean */

	mean = 0.0;
	if (subtract_mean || !apodize)
	{
		for(i=1; i<=n; i++) mean += fringe[i];
		mean /= (float)n;
	}

	/* Make a copy */

	if (subtract_mean)
	{
		for(i=1; i<=n; i++) copy[i] = fringe[i] - mean;
		mean = 0.0;
	}
	else
	{
		for(i=1; i<=n; i++) copy[i] = fringe[i];
	}

	if (apodize)
	{
		/* Now fill out the padded array */

		for(i=1; i<=start; i++) fft[i] = copy[1];
		for(; i < start+n; i++) fft[i] = copy[i - start + 1];
		for(; i <= fft_n; i++)  fft[i] = copy[n];

		/* And apply apodization. See log book 4 page 159. */

		apodize_length = start/2;

		m = 3.0/(apodize_length - 1);
		c = -1.0 * m * apodize_length;
		if (copy[1] != 0.0)
		{
			mult = (copy[1] + copy[n])/(2.0*copy[1]);
			fft[fft_n] = fft[1] = mult * copy[1];
			for(i = 2; i <= apodize_length; i++)
			{
				y = m * i + c;
				fft[i] *= (mult+(1.0-mult)*exp(-1.0*y*y));
			}
		}
		
		c = m*(apodize_length - fft_n - 1.0);
		if (copy[n] != 0.0)
		{
			mult = (copy[1] + copy[n])/(2.0*copy[n]);
			for(i = fft_n-apodize_length+1; i < fft_n; i++)
			{
				y = m * i + c;
				fft[i] *= (mult+(1.0-mult)*exp(-1.0*y*y));
			}
		}
	}
	else
	{
		/* Now fill out the padded array */

		for(i=1; i<start; i++) fft[i] = mean;
		for(; i < start+n; i++) fft[i] = copy[i - start + 1];
		for(; i <= fft_n; i++)  fft[i] = mean;
	}

	/* Do the FFT */

	realft(fft, fft_n, 1);

	/* Release the memory */

	free_vector(copy,1,n);

	/* That should be all */

	return fft;

} /* fft_fringe() */

/************************************************************************/
/* invfft_fringe()							*/
/*									*/
/* Routine to calculate the inverse FFT of a fringe packet. 		*/
/* Returns the fringe, as extracted from the inverse FFT.		*/
/* Memory allocated in this function. Assumes input like that created 	*/
/* by fft_fringe. 							*/
/* Returns NULL it fails. Memory is allocated here.			*/
/* Uses local copy so FFT is not changed.				*/
/************************************************************************/

float	*invfft_fringe(float *fft,	/* [1..fft_n] The data of the FFT */
		    int n,		/* How many data points in fringe? */
		    int fft_n)		/* How many points in the FFT? */
{
	float	*fringe;
	int	start;
	int	i;
	float	norm;
	float	*copy;

	/* Is this big enough? */

	if (fft_n < n) return NULL;

	/* Where did we put it in the FFT array? */

	start = (fft_n - n)/2;

	/* Allocate the memory */

	fringe = vector(1, n);
	copy = vector(1, fft_n);

	/* copy */

	for(i=1; i<= fft_n; i++) copy[i] = fft[i];

	/* What is the normalization factor? */

	norm = 2.0/(float)fft_n;

	/* Do the inverse transform */

	realft(copy, fft_n, -1);

	/* Now fill in the right part back into the fringe */

	for(i = 1; i <=n ; i++)
	{
		fringe[i] = norm*copy[i + start - 1];
	}

	/* That should be all */

	free_vector(copy, 1, fft_n);

	return fringe;

} /* invfft_fringe() */

/************************************************************************/
/* fft_freqs()								*/
/*									*/
/* Calculates and returns a vector of the frequencies in the FFT.	*/
/* Returns NULL on error.						*/
/* Returns freqs[0..fft_n/2].						*/
/* These frequencies should correspond one to one with the output of	*/
/* ps_fringe(), but will not quite match those of fft_fringe.		*/
/* Memory allocated here, but you ened to free it up later.		*/
/************************************************************************/

float *fft_freqs(int fft_n, float sample_time)
{
	float	*freq;
	float	first_freq;
	int	i;

	/* Allocate memory */

	freq = vector(0, fft_n/2);

	/* What is the lowest frequency? */

	first_freq = 1.0/((float)fft_n * sample_time);

	/* OK, fill out the frequencies */

	for(i=0; i<=fft_n/2; i++) freq[i] = (float)i * first_freq;

	/* That is all */

	return freq;

} /* fft_freqs() */

/************************************************************************/
/* make_bandpass_filter()						*/
/*									*/
/* Based on a frequency array, a mid frequency and a FWHM, calculates	*/
/* a bandpass filter based on the combination of a lowpass and a	*/
/* lowpass. Memory allocated in this function. Returns NULL on error.	*/
/* Array returned is bp[0..fft_n/2].					*/
/************************************************************************/

float	*make_bandpass_filter(float *freq, int fft_n, float center, float width)
{
	float	*bp;
	float	*lp, *hp;
	int	i;

	/* Create low pass and high pass filters */

	lp =  make_lowpass_filter(freq, fft_n, center+width/2.0, width/4.0);
	hp = make_highpass_filter(freq, fft_n, center-width/2.0, width/4.0);

	/* Combine them into a bandpass */

	bp = vector(0, fft_n/2);
	for(i=0; i<= fft_n/2; i++) bp[i] = lp[i] * hp[i];

	/* Free memory */

	free_vector(lp, 0, fft_n/2);
	free_vector(hp, 0, fft_n/2);

	/* That is all */

	return bp;

} /* make_bandpass_filter() */

/************************************************************************/
/* make_gaussian_filter()						*/
/*									*/
/* Based on a frequency array, a mid frequency and a FWHM, calculates	*/
/* a filter function, which is returned, based on a Gausian function.	*/
/* Memory allocated in this function. Returns NULL on error.		*/
/* Array returned is bp[0..fft_n/2].					*/
/************************************************************************/

float	*make_gaussian_filter(float *freq, int fft_n, float center, float fwhm)
{
	float	*gp;
	int	i;
	float	sd;
	float	t1, t2;

	/* Convert FWHM to standard deviation */

	sd = fwhm / 2.354820045;

	/* Allocate memory */

	gp = vector(0, fft_n/2);

	/* Do the calculation */

	t2 = -2.0 * sd * sd;	
	for(i=0; i<= fft_n/2; i++)
	{
		t1 = freq[i]-center;
		gp[i] = exp((t1*t1)/t2);
	}

	/* That is all */

	return gp;

} /* make_gaussian_filter() */

/************************************************************************/
/* make_lowpass_filter()                                               */
/*                                                                      */
/* Based on a frequency array, a mid frequency and a FWHM, calculates   */
/* a filter function, which is returned, based on a Gausian function.   */
/* Memory allocated in this function. Returns NULL on error.            */
/* Array returned is lp[0..fft_n/2].                                    */
/************************************************************************/
                                                                                
float   *make_lowpass_filter(float *freq, int fft_n, float edge, float fwhm)
{
        int     i;
        float   sd;
        float   *lp;
        float   t1, t2;
                                                                                
        /* Convert FWHM to standard deviation */
                                                                                
        sd = fwhm / 2.354820045;
                                                                                
        /* Allocate memory */
                                                                                
        lp = vector(0, fft_n/2);
                                                                                
        /* Do the calculation */
                                                                                
        t2 = -2.0 * sd * sd;
        for(i=0; i<= fft_n/2; i++)
        {
                if (freq[i] < edge)
                {
                        lp[i] = 1.0;
                }
                else
                {
                        t1 = freq[i]-edge;
                        lp[i] = exp((t1*t1)/t2);
                }
        }
                                                                                
        /* That is all */
                                                                                
        return lp;
                                                                                
} /* make_lowpass_filter() */

/************************************************************************/
/* make_highpass_filter()                                                */
/*                                                                      */
/* Based on a frequency array, a mid frequency and a FWHM, calculates   */
/* a filter function, which is returned, based on a Gausian function.   */
/* Memory allocated in this function. Returns NULL on error.            */
/* Array returned is hp[0..fft_n/2].                                    */
/************************************************************************/
                                                                                
float   *make_highpass_filter(float *freq, int fft_n, float edge, float fwhm)
{
        int     i;
        float   sd;
        float   *hp;
        float   t1, t2;
                                                                                
        /* Convert FWHM to standard deviation */
                                                                                
        sd = fwhm / 2.354820045;
                                                                                
        /* Allocate memory */
                                                                                
        hp = vector(0, fft_n/2);
                                                                                
        /* Do the calculation */
                                                                                
        t2 = -2.0 * sd * sd;
        for(i=0; i<= fft_n/2; i++)
        {
		if (freq[i] > edge)
		{
			hp[i] = 1.0;
		}
		else
		{
			t1 = freq[i]-edge;
			hp[i] = exp((t1*t1)/t2);
		}
        }
                                                                                
        /* That is all */
                                                                                
        return hp;
                                                                                
} /* make_highpass_filter() */

/************************************************************************/
/* use_filter()								*/
/*									*/
/* Given a filter and an FFT this will impose the filter on the FFT.	*/
/* FFT should have been created bu fft_fringe().			*/
/************************************************************************/

void	use_filter(float *fft, float *filter, int fft_n)
{
	int	n,i,j;

	n = fft_n>>1;

	/* DC part first */

	fft[1] *= filter[0];

	/* Nyquist part */

	fft[2] *= filter[n];

	/* All the rest */

	for (i=1; i<n; i++)
	{
		j = i<<1;
		fft[j+1] *= filter[i];
		fft[j+2] *= filter[i];
	}

} /* use_filter() */

/************************************************************************/
/* ps_fringe()								*/
/*									*/
/* Given the output of fft_fringe(), this routine returns the power	*/
/* spectrum, or NULL if there's an error. Memory is allocated here.	*/
/* The returned vector is the ps[0..fft_n/2].				*/
/************************************************************************/

float	*ps_fringe(float *fft,		/* [1..fft_n] The fft the fringe */
		    int fft_n)		/* How many data points in the fft? */
{
	float	*ps;
	int	n,i,j;

	/* Allocate the memory */

	n = fft_n>>1;
	ps = vector(0, n);

	/* Do the DC and highest frequency calculation */

	ps[0] = (fft[1]*fft[1]);
	ps[n] = (fft[2]*fft[2]);

	/* Now the remaining frequencies. */

	for(i=1, j = 3; i<n; i++, j+=2)
	{
		ps[i] = (fft[j]*fft[j] +
		       fft[j+1]*fft[j+1]);
	}

	/* That should be all */

	return ps;

} /* ps_fringe() */

/************************************************************************/
/* phase_fringe()							*/
/*									*/
/* Given the output of fft_fringe(), this routine returns the phase	*/
/* or NULL if there's an error. Memory is allocated here.		*/
/* The returned vector is the phase[0..fft_n/2].			*/
/************************************************************************/

float	*phase_fringe(float *fft,	/* [1..fft_n] The fft the fringe */
		    int fft_n)		/* How many data points in the fft? */
{
	float	*phase;
	int	n,i,j;

	/* Allocate the memory */

	n = fft_n>>1;
	phase = vector(0, n);

	/* Do the DC and highest frequency calculation */

	phase[0] = 0.0;
	phase[n] = 0.0;

	/* Now the remaining frequencies. */

	for(i=1, j = 3; i<n; i++, j+=2)
	{
		phase[i] = atan2(fft[j+1],fft[j]);
	}

	/* That should be all */

	return phase;

} /* phase_fringe() */

/************************************************************************/
/* envelope_fringe()							*/
/*									*/
/* Given the output of fft_fringe(), this routine returns the enevelope	*/
/* of the fringe, or NULL if there's an error. Memory is allocated here.*/
/* The returned vector is the env[1..n].				*/
/* Note that this routine does not work well unless you have removed	*/
/* the DC part of the data. Using the subtract_mean part of the		*/
/* fft_fringe() routine will help here.					*/
/************************************************************************/

float	*envelope_fringe(float *fft,	/* [1..fft_n] The fft the fringe */
		    int n,		/* Number of points in data */
		    int fft_n,		/* How many data points in the fft? */
		    int *frg_pos)	/* Where the fringes might be. */
{
	float	*envelope_fft;
	float	*envelope;
	float	norm;
	int	i,j,k;
	int	start;
	float	max = 0.0;

	/* Allocate memory */

	k = fft_n << 1;
	envelope_fft = vector(1, k);
	envelope = vector(1, n);

	/* This will not work unless we have removed the DC part. */

	envelope_fft[1] = envelope_fft[2] = 0.0;

	/* 
	 * Now fill out only the positive parts of the FFT, 
	 * All negative frequencies are zero.
	 */

	for(i=3; i <= fft_n; i++)
	{
		envelope_fft[i] = fft[i];
	}
	for(; i <= k; i++) envelope_fft[i] = 0.0;

	/* Do the inverse transorm */

	four1(envelope_fft, fft_n, -1);

	/* Now find the amplitude of the result with normalization */

	norm = (float)fft_n/2.0;
 
        /* Where did we put it in the FFT array? */
 
        start = (fft_n - n)/2;
        *frg_pos = 1;
 
        for(i=1; i<=n; i++)
        {
	    j = 2*(i+start)-1;
            envelope[i]= sqrt(envelope_fft[j]*envelope_fft[j] +
                envelope_fft[j+1]*envelope_fft[j+1])/norm;
            if (envelope[i] > max)
            {
                *frg_pos = i;
                max = envelope[i];
            }
        }
 
        /* Free memory */
 
        free_vector(envelope_fft, 1, k);
 
        /* That should be all */
 
        return envelope;
 
} /* envelope_fringe() */

/************************************************************************/
/* cog_envelope_fringe()						*/
/*									*/
/* Tries to calculate the center of gravity of a fringe envelope around */
/* a given point and a given distance around that point.		*/
/* Returns that position.						*/
/************************************************************************/

int	cog_envelope_fringe(float *envelope,	/* [1..n] The fringe envelope */
		    int n,			/* Number of points in data */
		    int frg_pos,		/* Where the fringes might be */
		    int frg_width)		/* Area to search */
{
	float	sum_env, sum_pos, pos;
	int	start_i, stop_i, i;

	/* What range do we use? */

	frg_width /=2;
	if ((start_i = frg_pos - frg_width) < 1) start_i = 1;
	if ((stop_i = frg_pos + frg_width ) > n) stop_i  = n;

	/* OK, do the calculation */

	sum_env = sum_pos = 0.0;
	for(i=start_i; i <= stop_i; i++)
	{
		sum_env += envelope[i];
		sum_pos += (envelope[i] * (float)i);
	}
	pos = sum_pos / sum_env;

	/* Find the nearest integer */

	frg_pos = (int)(pos + 0.5);

	/* Test this is OK */

	if (frg_pos < 1) 
		frg_pos = 1;
	else if (frg_pos > n)
		frg_pos = n;
		
	/* That is all */

	return frg_pos;

} /* cog_envelope_fringe() */

/************************************************************************/
/* gdt_fringe_track()                                                   */
/*									*/
/* This code from JM for finding the fringe packet.			*/
/* Fringe packet Tracker based on pedretti et al. 2004. 		*/
/* input: fft data, npoint, fringe peak/half-width (in nyquist units) 	*/
/* returns: fringepos, weight 						*/
/* This has been slightly modified by Theo.				*/
/************************************************************************/

int  gdt_fringe_track(float *fft,         /* output from fft_fringe() */
	     int n,		 /* Samples in original fringe */
	     int fft_n,          /*   number of samples in scan (input) */
	     float sample_time,	 /* Sample period each sample (S) */
	     float frg_freq,	 /* Expected fringe frequency (Hz) */
	     float frg_width,	 /* How wide an area do we search? (Hz) */
	     float *weight,      /*   fringes weighting       */
	     int *frg_pos   	 /*   fringe pos in fringe array */
	   )
{
	short int j,k;
	int maxn; /* maxn to deal with artificially high Nyquist frequency */
	float noise_mean, power_mean, rawPhase, abs_rawPhase, amplitude ;
	float dPhase,re,im,a,b,c,d;
	float *crossSpec;
	fcomplex *xft;
	int pos = 0;                  /*   position of fringe peak */
	float	fringepos;
	int	start,stop;
	int filtWidth;      /*   width of fringe peak    */

	/* Allocate memory */

	crossSpec = vector(0,fft_n/2);
	xft = fcvector(0, fft_n/2);
 
	/* Work out some starting points */

	pos = (int)(frg_freq*((float)fft_n * sample_time) + 0.5);
	filtWidth = (int)(0.5 * pos * frg_width/frg_freq + 0.5);
	maxn = 2*pos;
	if (maxn > fft_n/2 ) maxn=fft_n/2; /* fft_n is power of 2 */

	/*   complex product between fft and fft shifted one sample */
	/*   also calculates pwr. spec., and X pwr. spec. */
	    
	for (k = 1; k < (maxn-1); ++k) 
	{
	  j = k<<1;
	  a = *(fft + j );
	  b = *(fft + j + 1);
	  c = *(fft + j + 2);
	  d = *(fft + j + 3);
	  re = a*c + b*d;
	  im = -a*d + b*c;
	  (xft+k)->r = re;
	  (xft+k)->i = im;
	  *(crossSpec + k) = sqr(re) + sqr(im);
	}

	/* 
	 * Calculate noise in all pspec but the filter.
	 * Customized for CHARA version since because of 
	 * funny data-repeating in scan resulting in false Nyquist frequency.
	 * Will assume true Nyquist is twice the fringe frequency, as 
	 * expected for 4 pix/fringe.
	 */

	if ((start = pos - filtWidth) < 1) start = 1;
	if ((stop = pos + filtWidth) > maxn) start = maxn;

	noise_mean = 0.0;
	power_mean = 0.0;

	for(k = start; k <= stop; k++) power_mean += *(crossSpec + k);
	for(; k<= maxn; k++) noise_mean += *(crossSpec + k);

	noise_mean = noise_mean / (maxn - pos - filtWidth);
	power_mean = power_mean / (filtWidth + filtWidth);

	if (fft_n % 2 == 0) 
	{ 
	  /* fft_n is even */ 

	  *(crossSpec + maxn ) = 
		sqr((xft + maxn)->r);  /* estimated true Nyquist freq. */    
	}

	/*   are fringes there?   */
	
	*weight = (power_mean - noise_mean) /  noise_mean;

	if (*weight <= 0.0 || isnan(*weight)) *weight = 1e-9;

	/*  No thresholding here -- assume fringes are there */

	re = 0.0;
	im = 0.0;

	/*   integrate re and im part of fringe peak in x spectrum */

	for (k = start; k <= stop; k++) 
	{
	    /* we weight the real and imaginary part by the power spectrum */

	    amplitude = sqrt(*(crossSpec + k));  /* weighting ... */

	    re += amplitude * (xft + k)->r;
	    im += amplitude * (xft + k)->i;
	}

	/*  calculate diff. phase */

	rawPhase=atan2(im, re);	              
	abs_rawPhase=fabs(rawPhase);
	dPhase=(-abs_rawPhase/rawPhase)*(abs_rawPhase - PI);

	/* dPhase is +-PI representing full fft_n area */

	fringepos = (dPhase * fft_n/(2*PI))+ fft_n/2; 

	/* This needs to be put into the space of the fringe */

	*frg_pos = n/2 - fft_n/2 + fringepos + 0.5;

	/* Force this to be within the right range */

	if (*frg_pos < 1)
		*frg_pos = 1;
	else if(*frg_pos > n)
		*frg_pos = n;

	/* Free memory */

	free_vector(crossSpec, 0,fft_n/2);
	free_fcvector(xft, 0, fft_n/2);

	/* That is all */

	return (1L); /* successfully finishd calculation */

} /* gdt_fringe_track() */

/************************************************************************/
/* fringe_weight()							*/
/*									*/
/* Works out fringe weight, a ratio of power on and off fringes.	*/
/************************************************************************/

float fringe_weight(float *ps, float *freq, int fft_n, 
		float frg_freq, float frg_width)
{
	float	off_fringe, on_fringe;
	float	weight;
	int	on_start, on_stop, weight_dist;
	int	off_start, off_stop;
	int 	i;

	/* Is this possible ? */

	if (freq[1] == 0.0) return -1.0;

	/* Where are we integtrating power? */

	on_start = (int)((frg_freq - frg_width/2.0)/freq[1]+0.5);
	on_stop  = (int)((frg_freq + frg_width/2.0)/freq[1]+0.5);
	weight_dist   = on_stop-on_start+1;

	off_start = on_start+weight_dist;
	off_stop = on_stop+weight_dist;

	if (on_start < 1) on_start = 1;
	if (on_stop > fft_n/2) on_stop = fft_n/2;
	if (off_start < 1) off_start = 1;
	if (off_stop > fft_n/2) off_start = fft_n/2;

	/* First, work out the power near the fringe. */

	on_fringe = 0.0;
	for(i=on_start; i<= on_stop; i++) on_fringe += ps[i];

	/* 
	 * Now the power off the fringe. 
	 * This assumes that the fringes are more than Nyquist sampled.
	 */

	off_fringe = 0.0;
	for(i=off_start; i<= off_stop; i++) off_fringe += ps[i];

	if (off_fringe == 0.0) return 0.0;

	/* Fractional fringe power is what we want */

	weight  = (on_fringe - off_fringe)/off_fringe;

	if (weight < 0.0 || isnan(weight)) return 1.0e-9;

	return weight;

} /* fringe_weight() */

/************************************************************************/
/* fringe_weight_linear()                                               */
/*                                                                      */
/* Works out fringe weight, a ratio of power on and off fringes.        */
/* In this version the noise floor is approximated by a straight line.  */
/* THIS DOES NOT WORK WELL AT ALL!					*/
/************************************************************************/

float fringe_weight_linear(float *ps, float *freq, int fft_n, 
                float frg_freq, float frg_width)
{
        float   off_fringe, on_fringe;
        float   weight;
        int     start, stop;
        int     i;
        float   m,c;

        /* Is this possible ? */

        if (freq[1] == 0.0) return -1.0;

        /* Where are we integtrating power? */

        start = (int)((frg_freq - frg_width/2.0)/freq[1]+0.5);
        stop  = (int)((frg_freq + frg_width/2.0)/freq[1]+0.5);

        if (start < 1) start = 1;
        if (stop > fft_n/2) stop = fft_n/2;

        /* Fit a line to the noise power */

        m = (ps[start] - ps[stop]) / (freq[start] - freq[stop]);
        c = ps[start] - m * freq[start];

        /* First, work out the power near the fringe. */

        on_fringe = 0.0;
        off_fringe = 0.0;
        for(i=start; i<= stop; i++) 
        {
                on_fringe += ps[i];
                off_fringe += (m * freq[i] + c);
        }

        if (off_fringe == 0.0) return 1.0e-9;

        /* Fractional fringe power is what we want */

        weight  = (on_fringe - off_fringe)/off_fringe;

        if (weight < 0.0 || isnan(weight)) return 1.0e-9;

        return weight;

} /* fringe_weight_linear() */

/************************************************************************/
/* find_fringes_avg()                                                   */
/*   Computes and returns the average of a 1D vector                    */
/*   DATA of size (1..N)                                                */
/*                                                                      */
/*  author: Antoine Merand 2006/12/05                                   */
/************************************************************************/

float find_fringes_avg(float *data, int n)
{
	int i; 
	float avg;
	
	if (n <= 0) return 0.0;

	avg = 0.0;
	
	for (i=1;i<n;i++)
		avg += data[i];
	avg = avg/n;
	
	return avg;

} /* find_fringes_avg() */

/************************************************************************/
/* find_fringes_rms()                                                   */
/* Compute and returns the root mean square of a 1D vector              */
/* DATA of size (1..N)                                                  */
/*                                                                      */
/* author: Antoine Merand 2006/12/05                                    */
/************************************************************************/

float find_fringes_rms(float *data, int n, float avg)
{
	int i;
	float rms;
	
	if (n <= 0) return 0.0;

	rms = 0.0;
	
	for (i=1;i<n;i++)
		rms += (data[i]-avg)*(data[i]-avg);
	rms = rms/n;
	
	return rms;

} /* find_fringes_rms() */


/************************************************************************/
/* find_fringes_rms_parts                                               */
/*   Compute the root mean square of P equal parts of a 1D vector       */
/*   DATA (size 1..N) return the min of these RMS's                     */
/*                                                                      */
/*   author: Antoine Merand 2006/12/05                                  */
/************************************************************************/

float find_fringes_rms_parts(float *data, int n, int p)
{
	int i,j;
	float min_rms;
	float avg;
	float rms;
	float *tmp;
	
	if ((tmp = malloc((n/p)*sizeof(float))) == NULL )
		printf("error\n");
	
	min_rms = 1.0e32;
	
	for (j=0;j<p;j++)
	{
		for (i=0;i<n/p;i++)
			tmp[i] = data[j*n/p+i];
		avg = find_fringes_avg(tmp, n/p);
		rms = find_fringes_rms(tmp, n/p, avg);
		if (rms<min_rms)
			min_rms = rms;
	}
	free(tmp);

	return min_rms;

} /* find_fringes_rms_parts() */

/************************************************************************/
/* find_fringes_outliers()						*/
/*   Takes a 1D scan DATA of size 1..N and detects fringe packet.       */
/*   !Assume that there is only one fringe packet in DATA scan!        	*/
/*   - if fringes are not found, returns -1				*/
/*   - if fringes are found, returns the position of the fringe		*/
/*     packet inside the scan (within 0..N-1)				*/
/*									*/   
/*   requires:								*/
/*   - RMS_NOISE: root mean square of a scan without fringes (noise)	*/
/*   - THRESHOLD: number of sigmas above the noise for wich		*/
/*     one considers a fringe is detected                               */
/*   - MIN_NUMBER_FRINGES               				*/
/*									*/   
/*   the function will detect all fringes and return the median		*/
/*     index of these detections.					*/
/*									*/
/*   author: Antoine Merand 2006/12/05					*/
/************************************************************************/

int find_fringes_outliers(float *data, int n, float rms_noise, 
			  float threshold, int min_number_fringes)
{
	float  avg;
	int    i, n_outliers, med_ind;
	int    *outliers;
	
	med_ind=-1; /* default: fringes not found */
	
	if ((outliers = malloc(n*sizeof(int)))== NULL )
		printf("error\n");
	
	avg = find_fringes_avg(data,n);
	
	n_outliers = 1; 
	
	/* flag outliers by their index */

	for (i=1;i<n;i++)
	{
		if (((data[i]-avg)*(data[i]-avg))>threshold*threshold*rms_noise)
		{
			outliers[i] = n_outliers;
			n_outliers++;
		}
		else
			outliers[i] = 0;
	}
	
	
	if (n_outliers>=min_number_fringes) /* if any outliers */
	{
		/* find median index */

		for (i=1;i<n;i++)
		{
			if (outliers[i]>n_outliers/2)
			{
				med_ind = i; 
				i = n; 
			}
		}
	}
	
	free(outliers);
	
	return med_ind;

} /* find_fringes_outliers() */

/************************************************************************/
/* corr_fringe()							*/
/*									*/
/* Given the output of fft_fringe(), this routine returns the 		*/
/* correlation, or NULL if there's an error. Memory is allocated here.	*/
/* Returns the FFT of the autocorrelation NOT the autocorrelation.	*/
/************************************************************************/

float	*corr_fringe(float *fft1,	/* [1..fft_n] The fft the fringe */
		     float *fft2,
		     int fft_n)		/* How many data points in the fft? */
{
	float	*corr_fft;
	int	i;

	/* Allocate the memory */

	corr_fft = vector(1, fft_n);

	/* Do the DC and highest frequency calculation */

	corr_fft[1] = fft1[1]*fft2[1];
	corr_fft[2] = fft1[2]*fft2[2];

	/* Now the remaining frequencies. */

	for(i=3; i<fft_n; i+=2)
	{
		corr_fft[i] = fft1[i]*fft2[i] -
		       fft1[i+1]*fft2[i+1];
		corr_fft[i+1] = fft1[i]*fft2[i+1] +
		       fft2[i]*fft1[i+1];
	}

	/* Return the transform */

	return corr_fft;

} /* corr_fringe() */

/************************************************************************/
/* linearize_vector()							*/
/*									*/
/* A routine to linearize a vector!.					*/
/* Data vectro must go from data[start..stop].				*/
/* Returns a linearized version of the vector.				*/
/* Memory allocated here.						*/
/************************************************************************/

float *linearize_vector(float *data, int start, int stop)
{
	float x, y, sx, sy, sxx, sxy;
	int	i,n;
	float	m,c;
	float	*result;

	x = sx =  sy =  sxx =  sxy = 0.0;

	for(n=0, i=start; i<= stop; i++, n++)
	{
		x = (float)i;
		y = data[i];
		sx += x;
		sy += y;
		sxx += (x*x);
		sxy += (x*y);
	}

	c = (sxx*sy - sx*sxy)/x;
        m = ((float)n*sxy - sx*sy)/x;

	result = vector(start, stop);

	for(i=start; i<=stop; i++) result[i] = (float)i * m + c;

	return result;

} /* linearize_vector() */

/************************************************************************/
/* dft_fringe()								*/
/*									*/
/* Does the full blown discrete Fourier Transform of a real signal.	*/
/* Does insist that the number of samples is even.			*/
/* Returns the same thing as fft_fringe().				*/
/************************************************************************/

float *dft_fringe(float *fringe, int N)
{
	int	j,k,n;
	double	t;
	float	*dft;

	if (N%2)
	{
		N++;
		dft = vector(1, N);
		for(j=1; j<N; j++) dft[j] = fringe[j];
		dft[N] = fringe [1];
		fringe = dft;
	}
	dft = vector(1, N);

	/* The first entry is the DC part */

	dft[1] = 0.0;
	for(n=1; n<=N; n++) dft[1] += fringe[n];

	/* Work out the non Nyquist freqencies */

	for(k=2, j=3; k<=N>>1; k++, j+=2)
	{
		dft[j] = 0.0;
		dft[j+1] = 0.0;

		for(n=1; n<=N; n++)
		{
			t = -2.0*M_PI*(double)(k-1)*(double)(n-1)/(double)N;
			dft[j] += (fringe[n] * cos(t));
			dft[j+1] += (fringe[n] * sin(t));
		}
	}

	/* Work out the Nyquist frequency value */

	dft[2] = 0.0;
	for(n=1; n<=N; n++)
	{
		t = -2.0*M_PI*(double)(k-1)*(double)(n-1)/(double)N;
		dft[2] += (fringe[n] * cos(t));
	}

	return dft;

} /* dft_fringe() */

/************************************************************************/
/* invdft_fringe()							*/
/*									*/
/* Does the full blown discrete Inverse Frouier Transform of a real	*/
/* signal.								*/
/* Assumes even number N, which is forced by dft_fringe anyway.		*/
/* Returns the same thing as invfft_fringe().				*/
/************************************************************************/

float *invdft_fringe(float *dft, int N)
{
	int	i,j,k,n;
	double	t;
	//float	temp;
	float	*fringe;
	float	*fft;

	fringe = vector(1, N);
	fft = vector(1, N<<1);

	/* First part is the DC part */

	fft[1] = dft[1];
	fft[2] = 0.0;

	/* Now we copy across the rest */

	for(i = 3; i <= N; i++) fft[i] = dft[i];

	/* The Nyquist part comes next. */

	fft[N+1] = dft[2];
	fft[N+2] = 0.0;

	/* The rest are a copy with negative imaginary parts */

	for(i = N+3, j=N-1; i < (N-1)<<1; i+=2, j-=2)
	{
		fft[i] = fft[j];
		fft[i+1] = -1.0 * fft[j+1];
	}

	/* Finally the last one is always zero */

	fft[(N<<1)-1] = 0.0;
	fft[N<<1] = 0.0;

	/* OK, do the transform for teh real part only */

	//printf("DFT reconstructed:\n");
	for(n = 1; n<= N; n++)
	{
		fringe[n] = 0.0;
		//temp = 0.0;

		for(j=1, k=1; k<=N; k++, j+=2)
		{
			t = 2.0*M_PI*(double)(k-1)*(double)(n-1)/(double)N;

			fringe[n] += (fft[j] * cos(t) - fft[j+1] * sin(t));

			//temp += (fft[j+1] * cos(t) + fft[j] * sin(t));
			//if (n == 1) printf("%f %f\n", fft[j], fft[j+1]);
		}
		//if (n == 1) printf("RESULT:\n");
		fringe[n] /= (float)N;
		//printf("%f %f\n", fringe[n], temp);
	}
	
	/* Free memory an go */

	free_vector(fft, 1, N<<1);

	return fringe;

} /* invdft_fringe() */

/************************************************************************/
/* envelope_dft_fringe()						*/
/*									*/
/* Given the output of dft_fringe(), this routine returns the enevelope	*/
/* of the fringe, or NULL if there's an error. Memory is allocated here.*/
/* The returned vector is the env[1..n].				*/
/************************************************************************/

float	*envelope_dft_fringe(float *dft, /* [1..N] The fft the fringe */
		    int N,		/* Number of points in data */
		    int *frg_pos)	/* Where the fringes might be. */
{
	float	*env_dft;
	float	*env_invdft;
	float	*env;
	float	norm;
	int	i,j,k,l,n, twoN;
	float	max = 0.0;
	double	t, ct, st;

	/* Allocate memory */

	twoN = N << 1;
	env_dft = vector(1, twoN);
	env_invdft = vector(1, twoN);
	env = vector(1, N);

	/* This will not work unless we have removed the DC part. */

	env_dft[1] = env_dft[2] = 0.0;

	/* 
	 * Now fill out only the positive parts of the FFT, 
	 * All negative frequencies are zero.
	 */

	for(i=3; i <= N; i++)
	{
		env_dft[i] = dft[i];
	}
	for(; i <= twoN; i++) env_dft[i] = 0.0;

	/* Do the inverse transform */

	for(l = 1, n = 1; n<= N; n++, l += 2)
	{
		env_invdft[l] = 0.0;
		env_invdft[l+1] = 0.0;

		for(j=1, k=1; k<=N; k++, j+=2)
		{
			t = 2.0*M_PI*(double)(k-1)*(double)(n-1)/(double)N;
			ct = cos(t);
			st = sin(t);

			env_invdft[l] += (env_dft[j]*ct - env_dft[j+1]*st);

			env_invdft[l+1] += (env_dft[j+1]*ct + env_dft[j]*st);
		}
	}
	
	/* Now find the amplitude of the result with normalization */

	norm = (float)N/2.0;

	/* Extract the final thing */

	max = -1e32;
	for(j=1, i=1; i<= N; i++, j+=2)
	{
            env[i]= sqrt(env_invdft[j]*env_invdft[j] +
                env_invdft[j+1]*env_invdft[j+1])/norm;
            if (env[i] > max)
            {
                *frg_pos = i;
                max = env[i];
            }
	}
 
        /* Free memory */
 
        free_vector(env_dft, 1, twoN);
	free_vector(env_invdft, 1, twoN);
 
        /* That should be all */
 
        return env;
 
} /* envelope_dft_fringe() */

/************************************************************************/
/* smooth_vector()							*/
/*									*/
/* Perform a weighted smoothing of a vector. If the weight vector is	*/
/* NULL it will use even weighting.					*/
/* Data in the vector itself are not changed.				*/
/* Returns new, smoothed, vector (sd..nd).				*/
/* weight[1..nw].							*/
/* The weight vector should have an odd number for the length.		*/
/* Returns NULL on error or smoothed vector if things go well.		*/
/************************************************************************/

float *smooth_vector(float *data, int sd, int nd, float *weight, int nw)
{
	int	i, j, k, n;
	float	*result, *weight_use, weight_sum = 0.0;

	/* Is the weighting an odd number? */

	if (nw%2 == 0) return NULL;

	/* Is it too big? */

	if (nw > nd/2) return NULL;

	/* Allocate memory */

	result = vector(sd, nd);

	/* Where is the middle of the weighting vector? */

	n = (nw+1)/2;

	/* Do we have a weighting vector? */

	if (weight != NULL)
	{
		weight_use = weight;
	}
	else
	{
		weight_use = vector(1, nw);
		for(i=1; i<= nw; i++) weight_use[i] = 1.0;
	}

	/* Go for it */

	for(i=sd;  i<=nd; i++)
	{
		weight_sum = 0.0;
		result[i] = 0.0;
		for(j=1; j<=nw; j++)
		{
			k = i - n + j;

			if (k<sd)
				k = sd;
			else if (k>nd)
				k = nd;

			weight_sum += weight_use[j];
			result[i] += (data[k]*weight_use[j]);
		}
		result[i] /= weight_sum;
	}

	/* Free meomry if we need to */

	if (weight == NULL) free_vector(weight_use, 1, nw);

	/* That is all */

	return result;

} /* smooth_vector() */

/************************************************************************/
/* opd_close()								*/
/*									*/
/* Routine from John Monnier. Tries to use weights to ensure OPD offsets*/
/* are closed. MOstly for use in CLIMB.					*/
/* Assumes that the first OPD is real and correct. Then calculates new	*/
/* versions of the other two using weights and closure.			*/
/************************************************************************/

void opd_close(double opd[], double weights[], double corrected_opd[])
{
	double tmp;

	/* First one is "correct" */

	corrected_opd[0] = opd[0];

	/* Others are calculated */

	tmp = 1.0 / (1.0 / weights[0] + 1.0 / weights[1]);
	corrected_opd[2]  = (weights[2] * opd[2] + (opd[0] + opd[1]) * tmp) / 
		(weights[2] + tmp);

	tmp = 1.0 / (1.0 / weights[0] + 1.0 / weights[2]);
	corrected_opd[1]  = (weights[1] * opd[1] + (opd[2] - opd[0]) * tmp) /
		(weights[1] + tmp);

} /* opd_close() */


/************************************************************************/
/* dft_single_frequency()						*/
/*									*/
/* Given an array this will calculate the real andimaginary parts of 	*/
/* the descrete Fourier Transform for a given number of cycles k	*/
/* per the length of the array N.					*/
/*									*/
/* See page 33 of log book #6.						*/
/* Returns 0 if all goes well. 						*/
/* -1 if N<=0.								*/
/************************************************************************/

int dft_single_frequency(float *x, int N, int k, float *real, float *imaginary)
{
	int	n;
	float	theta;

	if (N <=0) return -1;

	*real = 0.0;
	*imaginary = 0.0;

	for(n=0; n<N; n++)
	{
		theta = -2.0 * M_PI * (float)k * (float)n / (float)N;

		*real += (x[n] * cos(theta));
		*imaginary += (x[n] * sin(theta));
	}

	return 0;

} /* dft_single_frequency() */

/************************************************************************/
/* climb_closure_triple_product_by_segment()				*/
/*									*/
/* Given an array sig[0...N-1] and a segment length returns an array	*/
/* tp[0...2N-1] of the resulting triple product for each segment.	*/
/* parts on ends forced to be zero. Data in result is real-imaginary.	*/
/* Returns NULL if there is a problem. Result memory allocated in	*/
/* this function.							*/
/************************************************************************/

float *climb_closure_triple_product_by_segment(float *sig, int N, int segN)
{
	int	i,j,k;
	float	*tp;
	float	real, imaginary, temp;

	/* Allocate the required memory */

	if ((tp = calloc(N<<1, sizeof(float))) == NULL) return NULL;

	/* Now, let's do this */

	for(i = 0, k=segN/2; i < N-segN-1; i++, k++)
	{
		j = k<<1;

		/* Beams 1&2 have the cirrct sign */

		dft_single_frequency(sig+i, segN, 3, &real, &imaginary);
		tp[j] = real;
		tp[j+1] = imaginary;

		/* Beams 2&3 have reverse sign due to dither mirror motion */

		dft_single_frequency(sig+i, segN, 2, &real, &imaginary);
		temp = real*tp[j] + imaginary*tp[j+1];
		tp[j+1] = real*tp[j+1] - imaginary*tp[j];
		tp[j] = temp;

		/* Beams 3&1 have reversed sign due to combiner geometry. */

		dft_single_frequency(sig+i, segN, 1, &real, &imaginary);
		temp = real*tp[j] + imaginary*tp[j+1];
		tp[j+1] = real*tp[j+1] - imaginary*tp[j];
		tp[j] = temp;
	}

	return tp;

} /* climb_closure_triple_product_by_segment() */

/************************************************************************/
/* climb_closure_triple_product_amplitude()				*/
/*									*/
/* Given an array tp[0..2N-1] as produced by 				*/
/* closure_triple_product_by_segment() returns an array amp[0..N-1]	*/
/* of amplitudes. Memory allocated within function. Returns NULL on 	*/
/* error.								*/
/************************************************************************/

float *climb_closure_triple_product_amplitude(float *tp, int N)
{
	int i,j;
	float *amp;

	if ((amp = calloc(N, sizeof(float))) == NULL) return NULL;

	for(i=0; i<N; i++)
	{ 
		j = i<<1;

		amp[i] = sqrt(tp[j]*tp[j] + tp[j+1]*tp[j+1]);
	}

	return amp;

} /* climb_closure_triple_product_amplitude() */

/************************************************************************/
/* climb_closure_triple_product_phase()					*/
/*									*/
/* Given an array tp[0..2N-1] as produced by 				*/
/* closure_triple_product_by_segment() returns an array phase[0..N-1]	*/
/* of phases. Memory allocated within function. Returns NULL on 	*/
/* error.								*/
/************************************************************************/

float *climb_closure_triple_product_phase(float *tp, int N)
{
	int i,j;
	float *phase;

	if ((phase = calloc(N, sizeof(float))) == NULL) return NULL;

	for(i=0; i<N; i++)
	{ 
		j = i<<1;

		phase[i] = atan2(tp[j+1], tp[j]);
	}

	return phase;

} /* climb_closure_triple_product_phase() */
	
/************************************************************************/
/* climb_triple_product_sum()						*/
/*									*/
/* Given an array tp[0..2N-1] as produced by 				*/
/* closure_triple_product_by_segment() returns the mean phase weighted	*/
/* by amplitude.							*/
/* Also calculates the sum of the real and imaginary parts.		*/
/************************************************************************/

float climb_triple_product_sum(float *tp, int N, float *real, float *imaginary)
{
	int i,j;

	*real = 0.0;
	*imaginary = 0.0;

	for(i=0; i<N; i++)
	{ 
		j = i<<1;
		*real += tp[j];
		*imaginary += tp[j+1];
	}

	return atan2(*imaginary, *real);

} /* climb_triple_product_sum() */

/************************************************************************/
/* calculate_v2_from_power_spectra()                                    */
/*                                                                      */
/* Tries to do the integral calculation of V^2 given the, assumed       */
/* normalized, power spectra of teh fringe signal in question.          */
/* Also calculates the group velocity from this signal. Wavelength in   */
/* micons, group velocity in microns per second.                        */
/************************************************************************/

float calculate_v2_from_power_spectra(float *ps_frg, float *freq, int freq_n,
                float int_start, float int_stop, float lambda, float dlambda,
                float *vg)
{
        float   s;
        float   ps_mean_pos;
        float   v2;
        int     i;
        float   wn0, dwn, wn1, wn2;

        /*
         * Integrate the fringe power and look for the
         * mean frequency to calculate the group velocity
         */

        s = 0.0;
        ps_mean_pos = 0.0;
        for(i=0; i<=freq_n; i++)
        {
                if (freq[i] < int_start || freq[i] > int_stop) continue;
                s += ps_frg[i];
                ps_mean_pos += (i * ps_frg[i]);
        }
        ps_mean_pos /= s;
        s *= freq[1];

        /* Work out the wavenumber */

        wn1 = 1.0e6/(lambda - 0.5*dlambda);
        wn2 = 1.0e6/(lambda + 0.5*dlambda);

        dwn = wn1 - wn2;
        wn0 = (wn1 + wn2)/2.0;

        /* We should now be able to calculate the group velocity */

        *vg = freq[1] * ps_mean_pos / wn0;

        /* And now the V2 */

        v2 = 4.0 * s * dwn * *vg;

	/* Put Vg into microns/S */

	*vg *= 1.0e6;

        //printf("wn0 = %.4e dwn = %.4e s = %.4e vg = %.4e v2 = %.4e\n",
                //wn0, dwn, s, *vg, v2);

        /* That is all */

        return v2;

} /* calculate_v2_from_power_spectra() */

/************************************************************************/
/* calc_unbiased_v_norm_estimate()                                      */
/*                                                                      */
/* Calculates unbias V estimator and variance based on V^2 and V^4      */
/* measurements. This version uses normal statistics.                   */
/************************************************************************/

void calc_unbiased_v_norm_estimate(float v2, float v4,
                                float *v_norm, float *v_norm_var)
{
        *v_norm = (3.0*v2*v2 - v4)/2.0;

        if (*v_norm <= 0.0)
                *v_norm = 0.0;
        else
                *v_norm = pow(*v_norm, 0.25);

        *v_norm_var = v4 - 0.5*(v2*v2 - v4);

        if (*v_norm_var <= 0.0)
                *v_norm_var = 0.0;
        else
                *v_norm_var = sqrt(*v_norm_var) - v2;

} /* calc_unbiased_v_norm_estimate() */

/************************************************************************/
/* calc_unbiased_v_norm_estimate()                                      */
/*                                                                      */
/* Calculates unbias V estimator and variance based on V^2 and V^4      */
/* measurements. This version uses log-normal statistics.               */
/************************************************************************/

void calc_unbiased_v_lognorm_estimate(float v2, float v4,
                                float *v_lognorm, float *v_lognorm_var)
{
        float   mu, var;

        mu = 0.25 * log(pow(v2, 4.0)/v4);

        var = 0.25 * log(v4/pow(v2,2.0));

        *v_lognorm = exp(mu + 0.5 * var);

        *v_lognorm_var = exp(2.0*mu +  2.0*var) - exp(2.0*mu + var);

} /* calc_unbiased_v_lognorm_estimate() */
