/************************************************************************/
/* read_astromod.c                                                      */
/*                                                                      */
/* Reads astrometric model data						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Dave Berger		                                        */
/* Date   : January 1999                                                */
/************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "../nrc/nrc.h"
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include "chara.h"

#define BUFFER_SIZE (1024)

/*
 * Externals
 */

static	FILE *data_file; 
static  char	star_file[BUFFER_SIZE];
static  char	baseline_file[BUFFER_SIZE];
static	char	config_file[BUFFER_SIZE];
static	STAR_DATA star;
static	BASELINE *base = 0;
static	SYSTEM chara;
static	ASTROMOD_DATA current;
static	int data_file_open = FALSE; 
static	time_t start_time;

/************************************************************************/
/* open_astromod()                                                      */
/*                                                                      */
/* Opens star, baseline, config, and data files for reading.	 	*/
/* It then reads in the first line of the data file and parses the	*/
/* info into the ASTROMOD_DATA structure.  Then it does the same with	*/
/* the star, baseline, and config data.  If no filenumber is sent, the	*/
/* files defined in chara.h are used.  Otherwise the given file is	*/
/* used and default values for the other three files are used.		*/
/*									*/
/* Returns 0 if it's ok and >1 if it's not.				*/
/*									*/
/************************************************************************/

int open_astromod (char *directory, char *filenumber)
{

	/*
	 * Internals
	 */
	char	buffer[BUFFER_SIZE];
	char	machine[256];
	char    the_star_file[2049];
	char	baseline_file[2049];
	char    the_conf_file[2049];
	char    astro_pid_file[2049];
	char    the_data_file[2049];
	char    s[2049];

	sprintf(the_star_file,"%s%s", get_data_directory(s), THE_STAR_FILE);
        sprintf(the_conf_file,"%s%s", get_data_directory(s), THE_CONF_FILE);
        sprintf(baseline_file,"%s%s", get_data_directory(s), BASELINE_FILE);
	sprintf(astro_pid_file,"%s%s", get_etc_directory(s), ASTRO_PID_FILE);
	sprintf(the_data_file,"%s%s", get_data_directory(s), DATA_FILE);

	/*
 	 * ignore this if it's already open
	 */

	if (data_file_open) return 0;

	/*
	 * If no filename is supplied upon calling this function,
	 * use the standard filenames defined in chara.h
	 * Otherwise, use the datafile provided and a set of 
	 * default values for the other stuff.
	 */

	if (directory == NULL || filenumber == NULL)
	{
	   if ((data_file = fopen(the_data_file, "r")) == NULL)
	      return (EOO_BADFILE);
           else
              data_file_open = TRUE;

	   /*
	    * Check if astromod is running.
	    * If it isn't, return an error code.
	    */
	   if (!check_unique(astro_pid_file, machine)) return (EOO_BADFILE);

	   strcpy(star_file, the_star_file);
	   strcpy(baseline_file, baseline_file);
	   strcpy(config_file, the_conf_file);
 
	}
	else
	{
	   sprintf(buffer,"%s/%s.%s",directory,the_data_file,filenumber);
	   if ((data_file = fopen(buffer, "r")) == NULL)
	      return (EOO_BADFILE);
           else
              data_file_open = TRUE;

	   sprintf(star_file,"%s/%s.%s",
		directory, the_star_file,filenumber);
	   sprintf(baseline_file,"%s/%s.%s",
		directory, baseline_file,filenumber);
	   sprintf(config_file,"%s/%s.%s",
		directory, the_conf_file,filenumber);
	}

	/* Go get the data out of the respective files */

	get_star_data ();
	get_config_data ();
	get_baseline_data ();

#ifdef INCLUDE_ROTAION
	/*
	 * Create the 2x2 rotation matrices
	 */
	for (i = 0; i < chara.num_telescopes; ++i)
	{
	   current.quad_rotation_matrix[i] = matrix(1,2,1,2);
	   current.acq_rotation_matrix[i] = matrix(1,2,1,2);
	}
#endif

	/*
	 * Get the first line of the data_file which is
	 * the length of each data line.
	 * Put that value into the ASTROMOD_DATA structure.
	 */
	
	rewind (data_file);
	fread (&buffer, 4, 1, data_file);
	current.record_length = atoi(buffer);

	/*
	 * Get the first date and time in the data file
	 * and put them into the ASTROMOD_DATA structure.
	 * Then put the time into start_time.
	 */
	fread (&buffer, 5, 1, data_file);
	current.mjd = atol(buffer);
	fread (&buffer, 6, 1, data_file);
	current.ut = atol(buffer);
	start_time = current.ut;

	/*
	 * Get the first batch of astrometric model data
	 */
	get_astromod_data (start_time);

	/* 
	 * printf's used for testing
	 *
	printf ("%3d\n", current.record_length);
	printf ("%5d\n", current.mjd);
	printf ("%6d\n", current.ut);
	printf ("%18.10e\n", current.n_air_obs);
        printf ("%18.10e\n", current.n_air_met);
	printf ("%13.8f\n", current.position_ref.az);
        printf ("%13.8f\n", current.position_ref.el);
        printf ("%13.8f\n", current.position_noref.az);
        printf ("%13.8f\n", current.position_noref.el);
	{
		int i;
		for (i = 0; i < chara.num_telescopes; ++i)
		   printf ("%15.10f\n", current.cart_pos[i]); 
	
#ifdef INCLUDE_ROTATION
		for (i = 0; i < chara.num_telescopes; ++i)
		   printf ("%9.4f %9.4f %9.4f %9.4f\n", 
			current.quad_rotation_matrix[i][1][1],
			current.quad_rotation_matrix[i][1][2], 
                        current.quad_rotation_matrix[i][2][1], 
                        current.quad_rotation_matrix[i][2][2]);
#endif
	}
	 */

	return (0);

} /* open_astromod */

/************************************************************************/
/* close_astromod()                                                     */
/*                                                                      */
/* Closes the star, baseline, config, and data files and then sets	*/
/* the open_file flags to FALSE.					*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not				*/
/*                                                                      */
/************************************************************************/

int close_astromod (void)
{

	/*
	 * Internals
	 */

	if (!data_file_open) return 0;

	/* release memory */

	if (base != 0)
	{
		free(base);
		base = 0;
	}

#ifdef INCLUDE_ROTATION
	/*
	 * Deallocate the memory for the rotation matrix
	 */
        for (i = 0; i < chara.num_telescopes; ++i)
	{
	   free_matrix(current.quad_rotation_matrix[i], 1, 2, 1, 2);
	   free_matrix(current.acq_rotation_matrix[i], 1, 2, 1, 2);
	}
#endif

	/*
	 * Close the files if they are open
	 * and set the open_file flags to FALSE
	 */

        fclose(data_file);
	data_file_open = FALSE;

	return(0);

} /* close_astromod */

/************************************************************************/
/* get_star_data()                                                      */
/*									*/
/* Gets the star data and parse it into the STAR structure		*/
/*									*/
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_star_data (void)
{

        /*
         * Internals
         */

	char	buffer[BUFFER_SIZE];

	/* First we set some defaults */

	*(star.name) = 0;
	star.chara_number = 0;
	*(star.hd) = 0;
	star.ra = 0;
	*(star.rastr) = 0;
	star.dec = 0;
	*(star.decstr) = 0;
	star.pm_ra = 0;
	star.pm_dec = 0;
	star.parallax = 0;
	star.radvel = 0;
	star.vmag = 0;
	star.kmag = 0;
	*(star.spectype) = 0;

	/* Is the file open? */

        if (!data_file_open) return (EOO_BADFILE);

	/* Go in search of the right tokens */

	if (find_token_line(star_file, "", "ID", star.name) == NULL)
	{
		fprintf(stderr,"Failed to get ID from file %s.\n",star_file);
		strcpy(star.name,"NONAME");
	}

	if ((find_token_line(star_file, "", "CHARA", buffer) == NULL) ||
		(sscanf(buffer,"%d",&star.chara_number) != 1))
	{
		fprintf(stderr,"Failed to get CHARA from file %s.\n",star_file);
		star.chara_number = 0;
	}

	if (find_token_line(star_file, "", "CAT3", star.hd) == NULL)
	{
		fprintf(stderr,"Failed to get CAT3 from file %s.\n",star_file);
		strcpy(star.hd,"0000");
	}

	if (find_token_line(star_file, "", "RA", star.rastr) == NULL)
	{
		fprintf(stderr,"Failed to get RA from file %s.\n",star_file);
		strcpy(star.rastr,"00 00 00");
	}

	if ((find_token_line(star_file, "", "RARAD", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&star.ra) != 1))
	{
		fprintf(stderr,"Failed to get RARAD from file %s.\n",star_file);
		star.ra = 0;
	}

	if (find_token_line(star_file, "", "DEC", star.decstr) == NULL)
	{
		fprintf(stderr,
			"Failed to get DEC from file %s.\n",star_file);
		strcpy(star.decstr,"00 00 00");
	}

	if ((find_token_line(star_file, "", "DECRAD", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&star.dec) != 1))
	{
		fprintf(stderr,
			"Failed to get DECRAD from file %s.\n",star_file);
		star.dec = 0;
	}

	if ((find_token_line(star_file, "", "PMRA", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&star.pm_ra) != 1))
	{
		fprintf(stderr,
			"Failed to get PMRA from file %s.\n",star_file);
		star.pm_ra = 0;
	}

	if ((find_token_line(star_file, "", "PMDEC", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&star.pm_dec) != 1))
	{
		fprintf(stderr,
			"Failed to get PMDEC from file %s.\n",star_file);
		star.pm_dec = 0;
	}

	if ((find_token_line(star_file, "", "PAR", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&star.parallax) != 1))
	{
		fprintf(stderr,
			"Failed to get PAR from file %s.\n",star_file);
		star.parallax = 0;
	}

	if ((find_token_line(star_file, "", "RVEL", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&star.radvel) != 1))
	{
		fprintf(stderr,
			"Failed to get RVEL from file %s.\n",star_file);
		star.radvel = 0;
	}

	if ((find_token_line(star_file, "", "VMAG", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&star.vmag) != 1))
	{
		fprintf(stderr,
			"Failed to get VMAG from file %s.\n",star_file);
		star.vmag = 0;
	}

	if ((find_token_line(star_file, "", "KMAG", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&star.kmag) != 1))
	{
		fprintf(stderr,
			"Failed to get KMAG from file %s.\n",star_file);
		star.kmag = 0;
	}

	if (find_token_line(star_file, "", "SP", star.spectype) == NULL)
	{
		fprintf(stderr,
			"Failed to get SP from file %s.\n",star_file);
		strcpy(star.spectype,"UNKNOWN");
	}

        return (0);

} /* get_star_data */

/************************************************************************/
/* get_baseline_data()                                                  */
/*                                                                      */
/* Gets the baseline information and parses it out to the BASLINE	*/
/* structure								*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/* This routine depends on knowing how many baselines there are		*/
/*& and so must always be run AFTER get_config_data().			*/
/************************************************************************/

int get_baseline_data (void)
{

        /*
         * Internals
         */

	char	buffer[BUFFER_SIZE];
	int	n;
	FILE	*input = NULL;

        if (!data_file_open) 
	{
		return (EOO_BADFILE);
	}

	/* Allocate memory for the baseline data */

	if ((base = malloc(sizeof(*base)*chara.num_baselines)) == NULL)
	{
	 	fprintf(stderr,"Ran out of memory for base array.\n");
		fclose(input);
		return 1;
	}

	/* Fill in data with dumby values */

	for (n=0; n < chara.num_baselines; n++)
	{
		strcpy(base[n].tel1,"NONAME");
		strcpy(base[n].tel2,"NONAME");
		base[n].baseline.x = 0.0;
		base[n].baseline.y = 0.0;
		base[n].baseline.z = 0.0;
		base[n].length = 0.0;
	}

	/* Open the file */

	if ((input = fopen(baseline_file, "r")) == NULL) 
	{
		fprintf(stderr,"Failed to open baseline file %s.\n",
			baseline_file);
		return (EOO_BADFILE);
	}

	for (n=0; n < chara.num_baselines && 
		GetDataLine(buffer, BUFFER_SIZE, input) != -1; n++)
	{
		if (sscanf(buffer,"%s %s %lf %lf %lf %lf",
			base[n].tel1,base[n].tel2,
			&(base[n].baseline.x),
			&(base[n].baseline.y),
			&(base[n].baseline.z),
			&(base[n].length)) != 6)
		{
		  strcpy(base[n].tel1,"NONAME");
		  strcpy(base[n].tel2,"NONAME");
		  base[n].baseline.x = 0.0;
		  base[n].baseline.y = 0.0;
		  base[n].baseline.z = 0.0;
		  base[n].length = 0.0;
		  fprintf(stderr,"Failed to read baseline data line %d.\n",n);
		}

   	} /* end while loop */

	fclose(input);

	if (n < chara.num_baselines)
	{
	  fprintf(stderr,"Too few baselines in baseline.dat file.\n");
	  for(; n < chara.num_baselines; n++)
	  {
		strcpy(base[n].tel1,"UNKNOWN");
		strcpy(base[n].tel2,"UNKNOWN");
		base[n].baseline.x = 0.0;
		base[n].baseline.y = 0.0;
		base[n].baseline.z = 0.0;
		base[n].length = 0.0;
	  }
	  return 2;
	}
	return (0);

} /* get_baseline_data */

/************************************************************************/
/* get_config_data()                                                    */
/*                                                                      */
/* Gets the config data and parses it into the SYSTEM structure		*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_config_data (void)
{

        /*
	 * Internals
	 */

	char	buffer[BUFFER_SIZE];

        if (!data_file_open) return (EOO_BADFILE);

	if ((find_token_line(config_file, "", "LONG", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&chara.longitude) != 1))
	{
		fprintf(stderr,
			"Failed to get LONG from file %s.\n",config_file);
		chara.longitude = CHARA_LONGITUDE;
	}

	if ((find_token_line(config_file, "", "LAT", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&chara.latitude) != 1))
	{
		fprintf(stderr,
			"Failed to get LAT from file %s.\n",config_file);
		chara.latitude = CHARA_LATITUDE;
	}

	if ((find_token_line(config_file, "", "OBSWAVE", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&chara.obs_wavelength) != 1))
	{
		fprintf(stderr,
			"Failed to get OBSWAVE from file %s.\n",config_file);
		chara.obs_wavelength = 0.0;
	}

	if ((find_token_line(config_file, "", "METWAVE", buffer) == NULL) ||
		(sscanf(buffer,"%lf",&chara.met_wavelength) != 1))
	{
		fprintf(stderr,
			"Failed to get METWAVE from file %s.\n",config_file);
		chara.met_wavelength = 0.0;
	}

	if ((find_token_line(config_file, "", "NUMTEL", buffer) == NULL) ||
		(sscanf(buffer,"%d",&chara.num_telescopes) != 1))
	{
		fprintf(stderr,
			"Failed to get NUMTEL from file %s.\n",config_file);
		chara.num_telescopes = 6;
	}

	if (find_token_line(config_file, "", "TELUSED", chara.scopes_used) == 
			NULL)
	{
		fprintf(stderr,
			"Failed to get TELUSED from file %s.\n",config_file);
		strcpy(chara.scopes_used,"S1 S2 E1 E2 W1 W2");
	}

	if (find_token_line(config_file, "", "FIXED", chara.fixed_cart) == 
			NULL)
	{
		fprintf(stderr,
			"Failed to get FIXED from file %s.\n",config_file);
		strcpy(chara.fixed_cart,"S1");
	}

	if ((find_token_line(config_file, "", "NUMBASE", buffer) == NULL) ||
		(sscanf(buffer,"%d",&chara.num_baselines) != 1))
	{
		fprintf(stderr,
			"Failed to get NUMBASE from file %s.\n",config_file);
		chara.num_baselines = 15;
	}

	return 0;

} /* get_config_data */

/************************************************************************/
/* get_astromod_data()                                                  */
/*                                                                      */
/* Gets the astrometric model data for a given time.  The time is in	*/
/* units of seconds since midnight.  If the time is zero, then just	*/
/* the first line of the data file is parsed into the ASTROMOD_DATA	*/
/* structure.								*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not (ie invalid time)		*/
/*                                                                      */
/************************************************************************/

int get_astromod_data (int time)
{
	/*
	 * Internals
	 */
	int i;
	int fixed_cart_num;
	char buffer[BUFFER_SIZE];
	int n, offset;
	
	/*
	 * See if the time is a valid one
	 */
	if (time % ASTROMOD_TICK_TIME != 0 || time < start_time)
           return (EOO_BADTIME);

	/*
	 * If the time is the same one that was last called
	 * and it's NOT the first call, return without doing anything.
	 */
	if (time == current.ut && time != start_time)
	   return (0);

	/*
	 * Put the fixed_cart in integer format
	 */
	if ((fixed_cart_num = GetTelescopeNumber (chara.fixed_cart, 
					     chara.scopes_used))<0)
		return EOO_BADNAME;

	/*
	 * Check to see if the file is open.
	 * If so, away we go...
	 */
        if (!data_file_open)
        {
          return (EOO_BADFILE);
        }
        else
	{
	   /*
	    * Offset equation so fseek knows where to put the
	    * file pointer given any time that it is given.
	    * It should be of type LONG.
	    */
	   n = (time-start_time) / 10;
	   offset = (int) (5 + n*current.record_length);
	   if (fseek (data_file, offset, SEEK_SET))
	      return (EOO_BADFILE);

	   /*
	    * Now that the file pointer is in the right place,
	    * put that entire line into the buffer.
	    */
	   if (fgets (buffer, BUFFER_SIZE, data_file) == NULL)
              return (EOO_BADFILE);

	   /*
	    * If the string length is zero there is no data
	    */

	   if (strlen(buffer) == 0) return (EOO_BADFILE);

	   /*
	    * Let the parsing begin!
	    */
	   current.mjd = atol((strtok (buffer, " ") ));
	   current.ut  = atol((strtok ((char *)NULL, " ") ));
	   current.n_air_obs = atof((strtok ((char *)NULL, " ") ));
	   current.n_air_met = atof((strtok ((char *)NULL, " ") ));
	   current.position_ref.az = atof((strtok ((char *)NULL, " ") ));
	   current.position_ref.el = atof((strtok ((char *)NULL, " ") ));
	   current.position_noref.az = atof((strtok ((char *)NULL, " ") ));
	   current.position_noref.el = atof((strtok ((char *)NULL, " ") ));

	   /*
	    * Depending on how many telescopes were used, get the
	    * positions of the OPLE carts.
	    * Set the position of the fixed cart to zero
	    */
	   for (i = 0; i < chara.num_telescopes; i++)
	   {
	      if (i == fixed_cart_num)
	      {
	         current.cart_pos[i] = 0.0;
	      }
	      else
	      {
	         current.cart_pos[i] = atof((strtok ((char *)NULL, " ") ));
	      }
	   } /* end for loop */

#ifdef INCLUDE_ROTATION
	   /*
	    * Do the same for the rotation matrices (except for
	    * that last bit about the fixed cart...).
	    */
	   for (i = 0; i < chara.num_telescopes; i++)
	   {
              current.quad_rotation_matrix[i][1][1] = 
			atof((strtok ((char *)NULL, " ") ));
              current.quad_rotation_matrix[i][1][2] =
			atof((strtok ((char *)NULL, " ") ));
              current.quad_rotation_matrix[i][2][1] =
			atof((strtok ((char *)NULL, " ") ));
              current.quad_rotation_matrix[i][2][2] =
			atof((strtok ((char *)NULL, " ") ));
              current.acq_rotation_matrix[i][1][1] =
                        atof((strtok ((char *)NULL, " ") ));
              current.acq_rotation_matrix[i][1][2] =
                        atof((strtok ((char *)NULL, " ") ));
              current.acq_rotation_matrix[i][2][1] =
                        atof((strtok ((char *)NULL, " ") ));
              current.acq_rotation_matrix[i][2][2] =
                        atof((strtok ((char *)NULL, " ") ));
	   } /* end for loop */
#endif

	} /* end else */

	/*
	 * If we made it here, be happy and return 0
	 */
	return (0);

} /* get_astromod_data */

/************************************************************************/
/* get_cart_pos()							*/
/*                                                                      */
/* Gets the cart position.						*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_cart_pos(int time, char *name, double *position)
{

	/*
	 * Internals
	 */
	int	cart_num;

	if ((cart_num = get_astromod_data (time)) != 0) return cart_num;

	if ((cart_num = GetTelescopeNumber (name, chara.scopes_used))< 0) 
		return EOO_BADNAME;

	*position = current.cart_pos[cart_num];

	return 0;
}

#ifdef INCLUDE_ROTATION
/************************************************************************/
/* get_tiptilt_matrix()							*/
/*                                                                      */
/* Gets the tiptilt rotation matrix for a telescope.			*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_tiptilt_matrix(int time, char *name, float **rot_matrix)
{

	/*
	 * Internals
	 */
	int	detect_num;

	if ((detect_num = get_astromod_data (time)) != 0) return detect_num;

	if ((detect_num = GetTelescopeNumber (name, chara.scopes_used))< 0) 
		return EOO_BADNAME;

        rot_matrix[1][1] = current.quad_rotation_matrix[detect_num][1][1];
        rot_matrix[1][2] = current.quad_rotation_matrix[detect_num][1][2];
        rot_matrix[2][1] = current.quad_rotation_matrix[detect_num][2][1];
        rot_matrix[2][2] = current.quad_rotation_matrix[detect_num][2][2];

	return 0;

} /* get_tiptilt_matrix() */

/************************************************************************/
/* get_acquisition_matrix()						*/
/*                                                                      */
/* Gets the acquisition rotation matrix for a telescope.		*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_acquisition_matrix(int time, char *name, float **rot_matrix)
{

	/*
	 * Internals
	 */
	int	detect_num;

	if ((detect_num = get_astromod_data (time)) != 0) return detect_num;

	if ((detect_num = GetTelescopeNumber (name, chara.scopes_used))< 0) 
		return EOO_BADNAME;

        rot_matrix[1][1] = current.acq_rotation_matrix[detect_num][1][1];
        rot_matrix[1][2] = current.acq_rotation_matrix[detect_num][1][2];
        rot_matrix[2][1] = current.acq_rotation_matrix[detect_num][2][1];
        rot_matrix[2][2] = current.acq_rotation_matrix[detect_num][2][2];

	return 0;

} /* get_acquisition_matrix() */
#endif

/************************************************************************/
/* get_star_name()							*/
/*                                                                      */
/* Gets the star name.							*/
/* Assumes that the memory has been allocated.  			*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_star_name(char *name)
{

	/*
	 * Check to see if the file is open.
	 */

        if (!data_file_open || *(star.name) == 0)
        {
          return (EOO_BADFILE);
        }

	strcpy(name, star.name);

	return 0;

} /* get_star_name() */ 

/************************************************************************/
/* get_star_chara_number()						*/
/*                                                                      */
/* Gets the star chara number.						*/
/*                                                                      */
/* Returns chara number if it's ok and -1 if it's not                   */
/*                                                                      */
/************************************************************************/

int get_star_chara_number(void)
{

	/*
	 * Check to see if the file is open.
	 */

        if (!data_file_open || *(star.name) == 0) return -1;

	return star.chara_number;

} /* get_star_chara_number() */ 

/************************************************************************/
/* get_star_hd()							*/
/*                                                                      */
/* Gets the star HD identifier.						*/
/* Assumes that the memory has been allocated.  			*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_star_hd(char *hd)
{

	/*
	 * Check to see if the file is open.
	 */

        if (!data_file_open || *(star.hd) == 0)
        {
          return (EOO_BADFILE);
        }

	if (*(star.hd) == 0 || *(star.hd) != 'H') return 1;

	strcpy(hd, star.hd);

	return 0;

} /* get_star_hd() */ 

/************************************************************************/
/* get_fixed_cart()							*/
/*                                                                      */
/* Gets the fixed cart name.						*/
/* Assumes that the memory has been allocated.  			*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_fixed_cart(char *fixed_cart)
{

	/*
	 * Check to see if the file is open.
	 */

        if (!data_file_open)
        {
          return (EOO_BADFILE);
        }

	strcpy(fixed_cart, chara.fixed_cart);

	return 0;

} /* get_fixed_cart() */ 

/************************************************************************/
/* get_met_wavelength()							*/
/*                                                                      */
/* Gets the metrology wavelength in meters.				*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_met_wavelength(double *met_wavelength)
{

	/*
	 * Check to see if the file is open.
	 */

        if (!data_file_open)
        {
          return (EOO_BADFILE);
        }

	*met_wavelength = chara.met_wavelength*1e-6;

	return 0;

} /* get_met_wavelength() */ 

/************************************************************************/
/* get_n_air_obs()							*/
/*                                                                      */
/* Gets the refractive index at observational wavelength.		*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_n_air_obs(int time, double *n_air_obs)
{

	int	i;

	if ((i = get_astromod_data (time)) != 0) return i;

	*n_air_obs = current.n_air_obs + 1.0;

	return 0;

} /* get_n_air_obs() */

/************************************************************************/
/* get_n_air_met()							*/
/*                                                                      */
/* Gets the refractive index at metrology wavelength.			*/
/*                                                                      */
/* Returns 0 if it's ok and >0 if it's not                              */
/*                                                                      */
/************************************************************************/

int get_n_air_met(int time, double *n_air_met)
{
	int	i;

	if ((i = get_astromod_data (time)) != 0) return i;

	*n_air_met = current.n_air_met + 1.0;

	return 0;

} /* get_n_air_met() */

/************************************************************************/
/* GetTelescopeNumber()							*/ 
/*                                                                      */
/* Given the name of the telescope (ie S1, E1, W2), this function	*/
/* will return the number of the element.  Good for use in matrices.	*/
/*									*/
/* *tel_name is the name of the telescope you want to look up		*/
/* *tels_used is a string with a list of the telescope names		*/
/*		which looks something like:  S1 S2 E1 E2 W1 W2		*/
/*                                                                      */
/* Returns -1 if there was an error					*/ 
/*                                                                      */
/************************************************************************/

int GetTelescopeNumber (char *tel_name, char *tels_used)
{
	/*
	 * Internals
	 */
	int	n;
	char *check;
	char s[81];

	n = 0;
	strcpy(s, tels_used);
	check = strtok (s, " ");
	if ( !strcmp (tel_name, check) )
	   return (n);
	else
	{
	   while (strcmp (tel_name, check)) 
	   {
	      if ((check = (char *)strtok((char *)NULL," ")) == NULL)
		return -1;
	      n++;
	   }
	}

	return (n);

} /* GetTelescopeNumber */

/************************************************************************/
/* check_unique()                                 		        */
/*									*/
/* This function checks for a running astromod process.           	*/
/*      Return values:  0       no astromod running                     */
/*                     -1       a previous astromod terminated          */
/*                              abnormally or a system error occurred   */
/*                      >0      the PID of the running astromod         */
/* Variable Machine is set to machine name. Assumes enough memeory!	*/
/*                                                                      */
/************************************************************************/

int check_unique(char *pidfile, char *machine)
{
        /* Internals */

        FILE    *fp;     /* Pointer to data file */
        int     PID;     /* PID of astromodel that is running */
        char    s[256];   /* Temporary string storage */

        /* Try to open the data file */

        if ((fp = fopen(pidfile,"r")) == NULL)
            return (0);         /* no astromod running */

        /* Try to get the PID of the current model */

        PID = 0;
        fgets(s,256,fp);
        if (sscanf(s,"MACHINE %s",machine) != 1)
	{
                fprintf(stderr,
                "check_unique: cannot get MACHINE for current process\n");
                return (-1);
	}
	
        fgets(s,256,fp);
        if (sscanf(s,"PID %d",&PID) != 1 || PID < 0)
	{
                fprintf(stderr,
                "check_unique: cannot get PID for current process\n");
                return (-1);
	}
        fclose(fp);

        /* Find out if the alleged astromod process is in fact
         * present.  If the system crashed or astromod was terminated
         * abnormally the PID file may still be present but astromod
         * may actually be defunct.
         */

	gethostname(s,256);
        if (strcmp(s, machine) == 0 && kill(PID, 0) )
        {
                if (errno == ESRCH)
                    fprintf(stderr,
                    "check_unique: previous process terminated abnormally.\n");
                else
                        fprintf(stderr,
                        "check_unique: old process reports unknown error.\n");
                return -1;
        }
        else
                return PID;

} /*check_unique(void) */

/************************************************************************/
/* get_current_astromod_data()						*/
/*									*/
/* Simply gives access to internal representation of astromod data.	*/
/* Probably not completely safe but it can be useful.			*/
/* Returns 0 if all went well and non-zero itherwise.			*/
/************************************************************************/

int get_current_astromod_data(int time, ASTROMOD_DATA **data)
{
        int     i;

        if ((i = get_astromod_data (time)) != 0) return i;

	*data = &current;

        return 0;

} /* get_current_astromod_data() */

/************************************************************************/
/* get_current_star_data()						*/
/*									*/
/* Simply gives access to internal representation of star data.		*/
/* Probably not completely safe but it can be useful.			*/
/* Returns 0 if all went well and non-zero itherwise.			*/
/************************************************************************/

int get_current_star_data(int time, STAR_DATA **data)
{
        int     i;

        if ((i = get_astromod_data (time)) != 0) return i;

	*data = &star;

        return 0;

} /* get_current_star_data() */

/************************************************************************/
/* get_current_baseline_data()						*/
/*									*/
/* Simply gives access to internal representation of baseline data.	*/
/* Probably not completely safe but it can be useful.			*/
/* Returns 0 if all went well and non-zero itherwise.			*/
/************************************************************************/

int get_current_baseline_data(int time, BASELINE **data)
{
        int     i;

        if ((i = get_astromod_data (time)) != 0) return i;

	*data = base;

        return 0;

} /* get_current_baseline_data() */

/************************************************************************/
/* get_current_chara_data()						*/
/*									*/
/* Simply gives access to internal representation of chara data.	*/
/* Probably not completely safe but it can be useful.			*/
/* Returns 0 if all went well and non-zero itherwise.			*/
/************************************************************************/

int get_current_chara_data(int time, SYSTEM **data)
{
        int     i;

        if ((i = get_astromod_data (time)) != 0) return i;

	*data = &chara;

        return 0;

} /* get_current_chara_data() */
