/************************************************************************/
/*	stardata.c							*/ 
/*									*/
/* Routines for finding and extracting data on stars from the big	*/
/* starlist file.							*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*									*/
/* Author: Theo								*/
/* Date: March 2003							*/
/*									*/
/************************************************************************/

#include "chara.h"
#include <string.h>
#include <stdlib.h>
#include <math.h>

/*
 *	input buffer used for processing input lines. 
 */

#define BUF_LENGTH	128
static	char	buffer[BUF_LENGTH];
static FILE	*fp = NULL;
static FILE	*fpa = NULL;

/************************************************************************/
/*	read_star_data_main()						*/
/*									*/
/*	Guess what this does!!  					*/
/*	The input data is reformatted as required and stored in the	*/
/*	star structure.							*/
/*									*/
/*	If the data file has already been opened it will continue to	*/
/*	read from it. If it is closed it will open the file.		*/
/*									*/
/* UNITS: Conversions must be done here fore the units. The star	*/
/* data base is done using the SimBad coordinates which are:		*/
/* 	Right ascension HH MM SS					*/
/* 	Declination     DD MM SS					*/
/*	pm RA  		mas/yr						*/
/*	pm Dec 		mas/yr						*/
/*	Parallax  	mas						*/
/*	rad vel  	km/s						*/
/*									*/
/* These must be converted into the right coordinates for Slalib	*/
/*									*/
/* 	Right ascension Radians						*/
/* 	Declination     Radians						*/
/*	pm RA  (mas/yr)	Radians per year				*/
/*	pm Dec (mas/yr)	Radians per year				*/
/*	Parallax  (mas) Arcseconds					*/
/*	rad vel  (km/s) km/S						*/
/*									*/
/* Returns 0 if all went well.						*/
/* -1 if it failed to open the data file.				*/
/************************************************************************/

int read_star_data_main(STAR_DATA *star)
{
	/*	Internals	*/

	char *label;
	char *field;
	char s[2048];

	/*
	 * 	First initialize the star data to default values.
	 *	RA and declination will not be initialized, because
	 *	they MUST be included in the input data.  
	 */

	strcpy(star->name, NAME_DFT);
	star->chara_number = 0;
	strcpy(star->irc, "");
	strcpy(star->hr, "");
	strcpy(star->hd, "");
	strcpy(star->sao, "");
	strcpy(star->fk5, "");
	strcpy(star->hip, "");
	strcpy(star->gj, "");
	strcpy(star->twomass, "");
	star->ra = star->pm_ra = PM_DFT;
	time_to_string(star->ra, star->rastr);
	star->dec = star->pm_dec = PM_DFT;
	angle_to_string(star->dec, star->decstr);
	star->pm_ra = 0;
	star->pm_dec = 0;
	star->parallax = PX_DFT;
	star->radvel = RV_DFT;
	star->vmag = VMAG_DFT;
	strcpy(star->spectype, SP_DFT);
	star->kmag = KMAG_DFT;

	/* 
	 * First open the data file if we have to.	
	 */

	if (fp == NULL && (fp = fopen(get_star_data_file(s),"r")) == NULL)
		return -1;
	
	/* Go through until we get an "END" record */

	while(1)
	{
	 	if (GetDataLine(buffer, BUF_LENGTH, fp) == -1) return -2;

		/*
		 *  	Get next data item.  Note 
		 * 	strcmp returns FALSE when arguments are equal!
		 */

		label = ToUpper( strtok(buffer," ;\t\n") );
		field = strtok((char *)NULL,";\n");

		/*
		 * 	Compare label with the valid data types and
		 *	convert the data as required.
		 */
		
		/* ID */

		if (!strcmp(label, "ID"))
		{
			strcpy(star->name, field);
		}

		/* CHARA number */

		else if (!strcmp(label, "CHARA"))
		{
			star->chara_number = atoi(field);
		}

		/* IRC identifier */

                else if ( !strcmp(label, "CAT1") )
		{
                   strcpy(star->irc, field);
		}

		/* HR identifier */

                else if ( !strcmp(label, "CAT2") )
		{
                   strcpy(star->hr, field);
		}

		/* HD identifier */

                else if ( !strcmp(label, "CAT3") )
		{
                   strcpy(star->hd, field);
		}

		/* SAO identifier */

                else if ( !strcmp(label, "CAT4") )
		{
                   strcpy(star->sao, field);
		}

		/* FK5 identifier */

                else if ( !strcmp(label, "CAT5") )
		{
                   strcpy(star->fk5, field);
		}

		/* Hip identifier */

                else if ( !strcmp(label, "CAT6") )
		{
                   strcpy(star->hip, field);
		}

		/* GJ identifier */

                else if ( !strcmp(label, "CAT7") )
		{
                   strcpy(star->gj, field);
		}

		/* 2mass identifier */

                else if ( !strcmp(label, "CAT8") )
		{
                   strcpy(star->twomass, field);
		}

		/*  right ascension (hours) */

		else if ( !strcmp(label, "RA") )
		{
			star->ra = string_to_time(field);
			strcpy(star->rastr, field);
		}

		/*  declination (degrees) */

		else if	( !strcmp(label, "DEC") )
		{
			star->dec = string_to_angle(field);
			strcpy(star->decstr, field);
		}
		
		/* proper motion in right ascension (mas/yr) */

		else if ( !strcmp(label, "PMRA") )
		{
#warning CHECK ALL THE OTHER PLACES THIS HAPPENS
/*

There is a factor of 15 missing here.... probably!
Remember 1 hour = 15 degrees!
NO NO NO..... the factor of 15 is STILL wrong.
This from Patrick Wallace:

a) Whenever the RA proper motion is given in mas per year (or arcsec per
year) you can rely on it representing the velocity on the sky.  Such
proper motions show no size change as you approach the poles.

b) Whenever the RA proper motion is given in seconds per year you can rely
on it representing the rate of change of RA.  Such proper motions grow in
size change as you approach the poles.

SLALIB requires RA proper motions to be in terms of rate of change of RA.

To convert an RA proper motion of type (a), in mas per year, into the form
used by SLALIB, requires a factor of pi/(180*3600e3*cos(dec)) to be
applied.  So if you're near a pole, and cos(dec) is small, the number you
hand to SLALIB will be large.

To convert an RA proper motion of type (b), in s per year, into the form
used by SLALIB, requires a factor of pi/(12*3600) to be applied.  A simple
change of units.

*/
			//star->pm_ra = AS2R*atof(field)*cos(star->dec)/1000.0;
			//star->pm_ra=AS2R*atof(field)/(cos(star->dec)*15000.0);
			/* Simbad gives proper motion as mas/yr */
			star->pm_ra = AS2R*atof(field)/(cos(star->dec)*1000.0);
		}

		/* proper motion in declination (mas/yr) */

		else if ( !strcmp(label, "PMDEC") )
				/* convert from mas/yr to rad/yr */
				star->pm_dec = AS2R*atof(field)/1000.0;

		/* parallax  (arcsec) */

		else if ( !strcmp(label, "PAR") )
			/*
			 *  N. B. Parallax is not converted to radians
			 *  but is converted to arcesec!
			 */
				star->parallax = atof(field)/1000.0;

		/* radial velocity (km/sec) */

		else if ( !strcmp(label, "RVEL") )
				star->radvel = atof(field);

		/* magnitude (magnitude!) */

		else if ( !strcmp(label, "VMAG") )
				star->vmag = atof(field);

		/* B-V color (mag) */

		else if (  !strcmp(label, "KMAG") )
				star->kmag = atof(field);

		/* spectral type  */

		else if ( !strcmp(label, "SP") )
				strcpy(star->spectype, field);

		else if ( !strcmp(label, "END") )
		{
			return 0;
		}

	} /* end while loop */
	
	/* We should never get here */

	return -3;

} /* read_star_data_main() */

/************************************************************************/
/*	read_star_data_abrig()						*/
/*									*/
/*	Guess what this does!!  					*/
/*	The input data is reformatted as required and stored in the	*/
/*	star structure.							*/
/*									*/
/*	If the data file has already been opened it will continue to	*/
/*	read from it. If it is closed it will open the file.		*/
/*									*/
/* UNITS: Conversions must be done here fore the units. The star	*/
/* data base is done using the SimBad coordinates which are:		*/
/* 	Right ascension HH MM SS					*/
/* 	Declination     DD MM SS					*/
/*	pm RA  		mas/yr						*/
/*	pm Dec 		mas/yr						*/
/*	Parallax  	mas						*/
/*	rad vel  	km/s						*/
/*									*/
/* These must be converted into the right coordinates for Slalib	*/
/*									*/
/* 	Right ascension Radians						*/
/* 	Declination     Radians						*/
/*	pm RA  (mas/yr)	Radians per year				*/
/*	pm Dec (mas/yr)	Radians per year				*/
/*	Parallax  (mas) Arcseconds					*/
/*	rad vel  (km/s) km/S						*/
/*									*/
/* Returns 0 if all went well.						*/
/* -1 if it failed to open the data file.				*/
/************************************************************************/

int read_star_data_abrig(STAR_DATA *star)
{
	/*	Internals	*/

	char *label;
	char *field;
	char s[2048];

	/*
	 * 	First initialize the star data to default values.
	 *	RA and declination will not be initialized, because
	 *	they MUST be included in the input data.  
	 */

	strcpy(star->name, NAME_DFT);
	star->chara_number = 0;
	strcpy(star->irc, "");
	strcpy(star->hr, "");
	strcpy(star->hd, "");
	strcpy(star->sao, "");
	strcpy(star->fk5, "");
	strcpy(star->hip, "");
	strcpy(star->gj, "");
	strcpy(star->twomass, "");
	star->ra = star->pm_ra = PM_DFT;
	time_to_string(star->ra, star->rastr);
	star->dec = star->pm_dec = PM_DFT;
	angle_to_string(star->dec, star->decstr);
	star->pm_ra = 0;
	star->pm_dec = 0;
	star->parallax = PX_DFT;
	star->radvel = RV_DFT;
	star->vmag = VMAG_DFT;
	strcpy(star->spectype, SP_DFT);
	star->kmag = KMAG_DFT;

	/* 
	 * First open the data file if we have to.	
	 */

	if (fpa == NULL)
	{
		if ((fpa = fopen(get_abrig_star_data_file(s),"a+")) == NULL)
		{
			return -1;
		}
		else
		{
			rewind(fpa);
		}
	}
	
	/* Go through until we get an "END" record */

	while(1)
	{
	 	if (GetDataLine(buffer, BUF_LENGTH, fpa) == -1) return -2;

		/*
		 *  	Get next data item.  Note 
		 * 	strcmp returns FALSE when arguments are equal!
		 */

		label = ToUpper( strtok(buffer," ;\t\n") );
		field = strtok((char *)NULL,";\n");

		/*
		 * 	Compare label with the valid data types and
		 *	convert the data as required.
		 */
		
		/* ID */

		if (!strcmp(label, "ID"))
		{
			strcpy(star->name, field);
		}

		/* CHARA number */

		else if (!strcmp(label, "CHARA"))
		{
			star->chara_number = atoi(field);
		}

		/* IRC identifier */

                else if ( !strcmp(label, "CAT1") )
		{
                   strcpy(star->irc, field);
		}

		/* HR identifier */

                else if ( !strcmp(label, "CAT2") )
		{
                   strcpy(star->hr, field);
		}

		/* HD identifier */

                else if ( !strcmp(label, "CAT3") )
		{
                   strcpy(star->hd, field);
		}

		/* SAO identifier */

                else if ( !strcmp(label, "CAT4") )
		{
                   strcpy(star->sao, field);
		}

		/* FK5 identifier */

                else if ( !strcmp(label, "CAT5") )
		{
                   strcpy(star->fk5, field);
		}

		/* Hip identifier */

                else if ( !strcmp(label, "CAT6") )
		{
                   strcpy(star->hip, field);
		}

		/* GJ identifier */

                else if ( !strcmp(label, "CAT7") )
		{
                   strcpy(star->gj, field);
		}

		/* 2mass identifier */

                else if ( !strcmp(label, "CAT8") )
		{
                   strcpy(star->twomass, field);
		}

		/*  right ascension (hours) */

		else if ( !strcmp(label, "RA") )
		{
			star->ra = string_to_time(field);
			strcpy(star->rastr, field);
		}

		/*  declination (degrees) */

		else if	( !strcmp(label, "DEC") )
		{
			star->dec = string_to_angle(field);
			strcpy(star->decstr, field);
		}
		
		/* proper motion in right ascension (mas/yr) */

		else if ( !strcmp(label, "PMRA") )
		{
			/* convert from mas/yr to rad/yr */
			star->pm_ra = AS2R*atof(field)/(cos(star->dec)*1000.0);
		}

		/* proper motion in declination (mas/yr) */

		else if ( !strcmp(label, "PMDEC") )
			/* convert from mas/yr to rad/yr */
			star->pm_dec = AS2R*atof(field)/1000.0;

		/* parallax  (arcsec) */

		else if ( !strcmp(label, "PAR") )
			/*
			 *  N. B. Parallax is not converted to radians
			 *  but is converted to arcesec!
			 */
				star->parallax = atof(field)/1000.0;

		/* radial velocity (km/sec) */

		else if ( !strcmp(label, "RVEL") )
				star->radvel = atof(field);

		/* magnitude (magnitude!) */

		else if ( !strcmp(label, "VMAG") )
				star->vmag = atof(field);

		/* B-V color (mag) */

		else if (  !strcmp(label, "KMAG") )
				star->kmag = atof(field);

		/* spectral type  */

		else if ( !strcmp(label, "SP") )
				strcpy(star->spectype, field);

		else if ( !strcmp(label, "END") )
		{
			return 0;
		}

	} /* end while loop */
	
	/* We should never get here */

	return -3;

} /* read_star_data_abrig() */

/************************************************************************/
/* close_starfile()							*/
/* 									*/
/* Closes the star data file.						*/
/************************************************************************/

void close_starfile(STAR_DATA *star)
{
	if (fp != NULL)
	{
		fclose(fp);
		fp = NULL;
	}
	if (fpa != NULL)
	{
		fclose(fpa);
		fpa = NULL;
	}

} /* close_starfile() */

/************************************************************************/
/*	update_abrig()							*/
/*									*/
/*	Adds the new information to the ABRIG_STAR_DATA_LIST		*/
/************************************************************************/

int update_abrig(STAR_DATA *star)
{
	if (fpa == NULL) return 1;

	fflush(fpa);
	if (strcmp(star->name,NAME_DFT))
	{
		fprintf(fpa,"ID\t%s\n",star->name);
	}
	fprintf(fpa,"CHARA\t%d\n2000.0\n",(int)star->chara_number);
	if (strcmp(star->irc,""))
	{
		fprintf(fpa,"CAT1\t%s\n",star->irc);
	}
	if (strcmp(star->hr,""))
	{
		fprintf(fpa,"CAT2\t%s\n",star->hr);
	}
	if (strcmp(star->hd,""))
	{
		fprintf(fpa,"CAT3\t%s\n",star->hd);
	}
	if (strcmp(star->sao,""))
	{
		fprintf(fpa,"CAT4\t%s\n",star->sao);
	}
	if (strcmp(star->fk5,""))
	{
		fprintf(fpa,"CAT5\t%s\n",star->fk5);
	}
	if (strcmp(star->hip,""))
	{
		fprintf(fpa,"CAT6\t%s\n",star->hip);
	}
	if (strcmp(star->gj,""))
	{
		fprintf(fpa,"CAT7\t%s\n",star->gj);
	}
	if (strcmp(star->twomass,""))
	{
		fprintf(fpa,"CAT8\t%s\n",star->twomass);
	}
	fprintf(fpa,"RA\t%s\n",star->rastr);
	fprintf(fpa,"DEC\t%s\n",star->decstr);
	fprintf(fpa,"PMRA\t%.2f\n",(star->pm_ra*1000.0*cos(star->dec))/AS2R);
	fprintf(fpa,"PMDEC\t%.2f\n",star->pm_dec*1000.0/AS2R);
	if (star->parallax == 0.0)
	{
		fprintf(fpa,"PAR\tnone\n");
	}
	else
	{
		fprintf(fpa,"PAR\t%.2f\n",star->parallax*1000.0);
	}
	if (star->radvel == 0.0)
	{
		fprintf(fpa,"RVEL\tnone\n");
	}
	else
	{
		fprintf(fpa,"RVEL\t%.2f\n",star->radvel);
	}
	if (star->vmag == 0.0)
	{
		fprintf(fpa,"VMAG\tnone\n");
	}
	else
	{
		fprintf(fpa,"VMAG\t%.2f\n",star->vmag);
	}
	if (star->kmag != KMAG_DFT)
	{
		fprintf(fpa,"KMAG\t%.2f\n",star->kmag);
	}
	if (strcmp(star->spectype,SP_DFT))
	{
		fprintf(fpa,"SP\t%s\n",star->spectype);
	}
	fprintf(fpa,"END\n\n");

	return 1;

} /* update_abrig() */

/************************************************************************/
/* get_star_by_number()							*/
/*									*/
/* Search through star data file and get one of correct number.		*/
/* Returns TRUE if found and FALSE if not found.			*/
/************************************************************************/

int get_star_by_number(int number, STAR_DATA *star)
{
	int	found = FALSE;

	/* First check abridged version */

	while(!found)
	{
		if (read_star_data_abrig(star) != 0) break;
		if (star->chara_number == number) found = TRUE;
	}

	/* Now check long version */

	if (!found)
	{
		while(!found)
		{
			if (read_star_data_main(star) != 0) break;
			if (star->chara_number == number) found = TRUE;
		}

		if (found) update_abrig(star);
	}
	close_starfile(star);

	return found;

} /* get_star_by_number() */

/************************************************************************/
/* get_star_by_hr()							*/
/*									*/
/* Search through star data file and get one of correct number.		*/
/* Returns TRUE if found and FALSE if not found.			*/
/************************************************************************/

int get_star_by_hr(int hrnum, STAR_DATA *star)
{
	int	found = FALSE;
	char	s[80];

	sprintf(s,"HR_%d", hrnum);

	/* First check abridged version */

	while(!found)
	{
		if (read_star_data_abrig(star) != 0) break;
		if (!strcmp(s, star->hr)) found = TRUE;
	}

	/* Now check long version */

	if (!found)
	{
		while(!found)
		{
			if (read_star_data_main(star) != 0) break;
			if (!strcmp(s, star->hr)) found = TRUE;
		}

		if (found) update_abrig(star);
	}
	close_starfile(star);

	return found;

} /* get_star_by_hr() */

/************************************************************************/
/* get_star_by_hd()							*/
/*									*/
/* Search through star data file and get one of correct number.		*/
/* Returns TRUE if found and FALSE if not found.			*/
/************************************************************************/

int get_star_by_hd(int hdnum, STAR_DATA *star)
{
	int	found = FALSE;
	char	s[80];

	sprintf(s,"HD_%d", hdnum);

	/* First check abridged version */

	while(!found)
	{
		if (read_star_data_abrig(star) != 0) break;
		if (!strcmp(s, star->hd)) found = TRUE;
	}

	/* Now check long version */

	if (!found)
	{
		while(!found)
		{
			if (read_star_data_main(star) != 0) break;
			if (!strcmp(s, star->hd)) found = TRUE;
		}

		if (found) update_abrig(star);
	}
	close_starfile(star);

	return found;

} /* get_star_by_hd() */

/************************************************************************/
/* get_star_by_sao()							*/
/*									*/
/* Search through star data file and get one of correct number.		*/
/* Returns TRUE if found and FALSE if not found.			*/
/************************************************************************/

int get_star_by_sao(int saonum, STAR_DATA *star)
{
	int	found = FALSE;
	char	s[80];

	sprintf(s,"SAO_%d", saonum);

	/* First check abridged version */

	while(!found)
	{
		if (read_star_data_abrig(star) != 0) break;
		if (!strcmp(s, star->sao)) found = TRUE;
	}

	/* Now check long version */

	if (!found)
	{
		while(!found)
		{
			if (read_star_data_main(star) != 0) break;
			if (!strcmp(s, star->sao)) found = TRUE;
		}

		if (found) update_abrig(star);
	}
	close_starfile(star);

	return found;

} /* get_star_by_sao() */

/************************************************************************/
/* get_star_by_irc()							*/
/*									*/
/* Search through star data file and get one of correct number.		*/
/* Returns TRUE if found and FALSE if not found.			*/
/************************************************************************/

int get_star_by_irc(int ircnum, STAR_DATA *star)
{
	int	found = FALSE;
	char	s[80];

	sprintf(s,"IRC%+06d", ircnum);

	/* First check abridged version */

	while(!found)
	{
		if (read_star_data_abrig(star) != 0) break;
		if (!strcmp(s, star->irc)) found = TRUE;
	}

	/* Now check long version */

	if (!found)
	{
		while(!found)
		{
			if (read_star_data_main(star) != 0) break;
			if (!strcmp(s, star->irc)) found = TRUE;
		}

		if (found) update_abrig(star);
	}
	close_starfile(star);

	return found;

} /* get_star_by_irc() */

/************************************************************************/
/* get_star_by_name()							*/
/*									*/
/* Search through star data file and get one of correct name.		*/
/* Returns TRUE if found and FALSE if not found.			*/
/************************************************************************/

int get_star_by_name(char *name, STAR_DATA *star)
{
	int	found = FALSE;

	/* First check abridged version */

	while(!found)
	{
		if (read_star_data_abrig(star) != 0) break;
		if (!strcmp(name, star->name)) found = TRUE;
	}

	/* Now check long version */

	if (!found)
	{
		while(!found)
		{
			if (read_star_data_main(star) != 0) break;
			if (!strcmp(name, star->name)) found = TRUE;
		}

		if (found) update_abrig(star);
	}
	close_starfile(star);

	return found;

} /* get_star_by_name() */
