/************************************************************************/
/* charaoif.c								*/
/*									*/
/* Routines for setting up what we need to create an OIFITS file.	*/
/* See doc/howto for PDF file explaining all this.			*/
/* This code largely based on the demo.c code in the oifitslib		*/
/* directory.								*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*									*/
/*	Author: Theo ten Brummelaar					*/
/*	Date:   September 2009						*/
/*									*/
/************************************************************************/

/* Uncomment this to have test messages printed out. */

//#define TEST

/*
 * Include files
 */	

#include "../charalib/chara.h"
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include "../oifitslib/exchange.h"
#include "../slalib/slalib.h"

/************************************************************************/
/* setup_oi_array()							*/
/*									*/
/* Sets up the OIFIFTS structure ARRAY, which contains information	*/
/* about CHARA.								*/
/* Returns 0 if things went well, -1 if we failed to get system 	*/
/* information, -2 if qwe run out of memory.				*/
/************************************************************************/

int setup_oi_array(oi_array *array)
{
	double	r, z, geo_lat, geo_long, d;
	SYSTEM  chara;
	TELESCOPE scopes[NUM_SCOPES];
	int	i;
#ifdef TEST
	char	s1[80],s2[80];
	int	j;
	double x, y;
#endif

	/* Get information about the scopes and the array. */

	if (extract_system_data(&chara, scopes) != 0)
		return -1;

	array->revision = 1;
	strcpy(array->arrname, "CHARA_ARRAY");
	strcpy(array->frame, "GEOCENTRIC");

	/*
	 * Go get the geocentric position as defined by SLALIB 
 	 * which is distance from earth axis (AU) and distance from plan
 	 * of earth equator (AU).
 	 */

#ifdef TEST
	/* Use COAST to check calculations */

	slaGeoc(string_to_angle("52 09 50.18"), 19.0, &r, &z);
#else
	slaGeoc(chara.latitude, chara.height, &r, &z);
#endif
			
	/* convert to geocentric latitude */

	geo_lat = atan2(z,r);

	/* The geocentric longitude is the same as normal */

#ifdef TEST
	/* Use COAST to check calculations */

	geo_long = string_to_angle("00 02 26.16");
#else
	geo_long = chara.longitude;
#endif

	/* Get the distance from earth's center */

	d = sqrt (r*r + z*z);

	/* Convert from AU to metres */

	d *= 1.49597870e11;

#ifdef TEST
	printf("GEO Long %s Lat %s R: %f\n",time_to_string(geo_long,s1),
		angle_to_string(geo_lat,s2), r);
#endif

	/* Now convert to cartesian coordinates */

	array->arrayx = d * cos(geo_lat);
	array->arrayy = d * sin(geo_long) * sin(geo_lat);
	array->arrayz = d * cos(geo_long) * sin(geo_lat);

	/* Now the telescopes */

	array->nelement = chara.num_telescopes;

	array->elem = malloc(array->nelement*sizeof(element));
	if (array->elem == NULL) return -2;

	for(i=0; i<array->nelement; i++)
	{
		strcpy(array->elem[i].tel_name, scope_types[i]);
		strcpy(array->elem[i].sta_name, scope_types[i]);
		array->elem[i].sta_index=i;
		array->elem[i].diameter = 1.0;

		/* 
		 * This is from en.wikipedia.org/wiki/Geodetic_system
		 */

		array->elem[i].staxyz[0] = 
			-1.0 *sin(geo_long) * scopes[i].baseline.x -
			sin(geo_lat) * cos(geo_long) * scopes[i].baseline.y +
			cos(geo_lat) * cos(geo_long) * scopes[i].baseline.z;

		array->elem[i].staxyz[1] = 
			cos(geo_long) * scopes[i].baseline.x -
			sin(geo_lat) * sin(geo_long) * scopes[i].baseline.y +
			cos(geo_lat) * sin(geo_long) * scopes[i].baseline.z;

		array->elem[i].staxyz[2] = 
			cos(geo_lat) * scopes[i].baseline.y +
			sin(geo_lat) * scopes[i].baseline.z;
	}

#ifdef TEST
	for(i=0; i<NUM_SCOPES-1; i++)
	{
		for(j=i+1; j<NUM_SCOPES; j++)
		{
			x = scopes[i].baseline.x - scopes[j].baseline.x;
			y = scopes[i].baseline.y - scopes[j].baseline.y;
			z = scopes[i].baseline.z - scopes[j].baseline.z;
			printf("%s->%s %.2f\n",
				scope_types[i], scope_types[j],
				sqrt(x*x + y*y + z*z));
		}
	}
#endif
		
	return 0;

} /* setup_oi_array() */

/************************************************************************/
/* clear_oi_array()							*/
/*									*/
/* Deallocates memory in an oi_array structure.				*/
/************************************************************************/

void clear_oi_array(oi_array array)
{
	if (array.elem != NULL) free(array.elem);

} /* clear_oi_array() */

/************************************************************************/
/* setup oi_wavelength()						*/
/* 									*/
/* Setup values in an oi_wavelength structure.				*/
/************************************************************************/

void setup_oi_wavelength(oi_wavelength *wave, char *name)
{
	strcpy(wave->insname, name);
	wave->nwave = 0;
	wave->revision = 1;
	wave->eff_wave = NULL;
	wave->eff_band = NULL;
	wave->revision = 1;

} /* setup_oi_wavelength() */

/************************************************************************/
/* add_oi_wavelength()							*/
/* 									*/
/* Addes a waveband to the wavelength structure.			*/
/* Expects units of meters.						*/
/* Returns 0 if all OK, -1 if out of memory.				*/
/************************************************************************/

int add_oi_wavelength(oi_wavelength *wave, 
			 float lambda, float dlambda)
{
	float *wavelength, *bandwidth;
	int	i;

	wavelength = wave->eff_wave;
	bandwidth  = wave->eff_band;

	wave->nwave++;
	wave->eff_wave = malloc(wave->nwave*sizeof(float));
	wave->eff_band = malloc(wave->nwave*sizeof(float));

	if (wave->eff_wave == NULL || wave->eff_band == NULL) return -1;

	for(i=0; i<wave->nwave-1; i++)
	{
		wave->eff_wave[i] = wavelength[i];
		wave->eff_band[i] = bandwidth[i];
	}

	wave->eff_wave[wave->nwave-1] = lambda;
	wave->eff_band[wave->nwave-1] = dlambda;

	if (wavelength != NULL) free(wavelength);
	if (bandwidth  != NULL) free(bandwidth);

	return 0;

} /* setup_oi_wavelength() */

/************************************************************************/
/* clear_oi_wavelength()						*/
/*									*/
/* Deallocates memory in an oi_wavelength structure.			*/
/************************************************************************/

void clear_oi_wavelength(oi_wavelength wave)
{
	if (wave.eff_wave != NULL) free(wave.eff_wave);
	if (wave.eff_band != NULL) free(wave.eff_band);

} /* clear_oi_wavelength() */

/************************************************************************/
/* setup_oi_target()							*/
/*									*/
/* Initialize oi_target structure.					*/
/************************************************************************/

void setup_oi_target(oi_target *targets)
{
	targets->ntarget = 0;
	targets->targ = NULL;
	targets->revision = 1;

} /* setup_oi_target() */

/************************************************************************/
/* add_oi_target()							*/
/*									*/
/* Add a target to the list of targets. Allocates memory.		*/
/* Returns 0 if all OK, -1 if out of memory.				*/
/************************************************************************/

int add_oi_target(oi_target *targets, STAR_DATA star)
{
	target *old_targ, *new_targ;
	int	i;

	old_targ = targets->targ;

	targets->ntarget++;
	targets->targ = malloc(targets->ntarget * sizeof(target));

	if (targets->targ == NULL) return -1;

	for(i=0; i<targets->ntarget-1; i++)
	{
		targets->targ[i] = old_targ[i];
	}

	if (old_targ != NULL) free(old_targ);

	new_targ = (targets->targ)+(targets->ntarget-1);

	//new_targ->target_id = star.chara_number;
	new_targ->target_id = targets->ntarget;
	strcpy(new_targ->target, star.name);
	new_targ->raep0 = star.ra / M_PI * 180.0;
	new_targ->decep0 = star.dec / M_PI * 180.0;
	new_targ->equinox = 2000.0;
	new_targ->ra_err = 0.0001; /* Guess! */
	new_targ->dec_err = 0.0001; /* Guess! */
	new_targ->sysvel = star.radvel * 1000.0;
	strcpy(new_targ->veltyp, "HELIOCEN"); /* Don't really know this. */
	strcpy(new_targ->veldef, "OPTICAL"); /* Don't really know this. */
	new_targ->pmra = star.pm_ra * cos(star.dec) / M_PI * 180.0;
	new_targ->pmdec = star.pm_dec / M_PI * 180.0;
	new_targ->pmra_err = 0.0; /* Don't know what this is. */
	new_targ->pmdec_err = 0.0; /* Don't know what this is. */
	new_targ->parallax = star.parallax / 3600.0;
	new_targ->para_err = 0.0; /* Don't know what this is. */
	strcpy(new_targ->spectyp, star.spectype);

	return 0;

} /* add_oi_target() */

/************************************************************************/
/* clear_oi_target()							*/
/*									*/
/* Decallocate memory for an oi_target structure.			*/
/************************************************************************/

void clear_oi_target(oi_target targets)
{
	if (targets.targ != NULL) free(targets.targ);

} /* clear_oi_target() */

/************************************************************************/
/* setup_oi_vis()							*/
/*									*/
/* INitilize an oi_vis structure.					*/
/************************************************************************/

void setup_oi_vis(oi_vis *vis, char *date, char *arrname, 
			char *insname, int nwave)
{
	vis->revision = 1;
	strcpy(vis->date_obs, date);
	strcpy(vis->arrname, arrname);
	strcpy(vis->insname, insname);
	vis->numrec = 0;
	vis->nwave = nwave;
	vis->record = NULL;

} /* setup_oi_vis() */

/************************************************************************/
/* add_oi_vis()								*/
/*									*/
/* Add a record to an oi_vis structure.					*/
/* Returns 0 if all OK, -1 if out of memory.				*/
/************************************************************************/
int add_oi_vis(oi_vis *vis, int id, double time, 
		double mjd, double int_time,
		double *visamp, double *visamperr,
		double *visphi, double *visphierr,
		double u, double v, int scope1, int scope2)
{
	oi_vis_record *old_record, *new_record;
	int	i;

	old_record = vis->record;

	vis->numrec++;
	vis->record = malloc(vis->numrec * sizeof(oi_vis_record));

	if (vis->record == NULL) return -1;

	for(i=0; i<vis->numrec-1; i++)
	{
		vis->record[i] = old_record[i];
	}

	if (old_record != NULL) free(old_record);

	new_record = (vis->record)+(vis->numrec-1);

	new_record->visamp = malloc(vis->nwave * sizeof(DATA));
	new_record->visamperr = malloc(vis->nwave * sizeof(DATA));
	new_record->visphi = malloc(vis->nwave * sizeof(DATA));
	new_record->visphierr = malloc(vis->nwave * sizeof(DATA));
	new_record->flag = malloc(vis->nwave * sizeof(char));

	if (new_record->visamp == NULL || new_record->visamperr == NULL ||
	    new_record->visphi == NULL || new_record->visphierr == NULL ||
	    new_record->flag == NULL) 
	{
		return -1;
	}

	new_record->target_id = id;
	new_record->time = time;
	new_record->mjd = mjd;
	new_record->int_time = int_time;
	new_record->ucoord = u;
	new_record->vcoord = v;
	new_record->sta_index[0] = scope1;
	new_record->sta_index[1] = scope2;
	
	for(i=0 ; i<vis->nwave; i++)
	{
		new_record->visamp[i] = visamp[i];
		new_record->visamperr[i] = visamperr[i];
		new_record->visphi[i] = visphi[i];
		new_record->visphierr[i] = visphierr[i];
		new_record->flag[i] = FALSE;
	}
	
	return 0;

} /* add_oi_vis() */

/************************************************************************/
/* clear_oi_vis()							*/
/* 									*/
/* Deallocate memory used by an oi_vis structure.			*/
/************************************************************************/

void clear_oi_vis(oi_vis vis)
{
	int	i;

	for(i=0; i<vis.numrec; i++)
	{
		free(vis.record[i].visamp);
		free(vis.record[i].visamperr);
		free(vis.record[i].visphi);
		free(vis.record[i].visphierr);
		free(vis.record[i].flag);
	}

	if (vis.record != NULL) free(vis.record);

} /* clear_oi_vis() */

/************************************************************************/
/* setup_oi_vis2()							*/
/*									*/
/* INitilize an oi_vis2 structure.					*/
/************************************************************************/

void setup_oi_vis2(oi_vis2 *vis2, char *date, char *arrname, 
			char *insname, int nwave)
{
	vis2->revision = 1;
	strcpy(vis2->date_obs, date);
	strcpy(vis2->arrname, arrname);
	strcpy(vis2->insname, insname);
	vis2->numrec = 0;
	vis2->nwave = nwave;
	vis2->record = NULL;

} /* setup_oi_vis2() */

/************************************************************************/
/* add_oi_vis2()							*/
/*									*/
/* Add a record to an oi_vis2 structure.				*/
/* Returns 0 if all OK, -1 if out of memory.				*/
/************************************************************************/

int add_oi_vis2(oi_vis2 *vis2, int id, double time, 
		double mjd, double int_time, 
		double *vis2data, double *vis2err,
		double u, double v, int scope1, int scope2)
{
	oi_vis2_record *old_record, *new_record;
	int	i;

	old_record = vis2->record;

	vis2->numrec++;
	vis2->record = malloc(vis2->numrec * sizeof(oi_vis2_record));

	if (vis2->record == NULL) return -1;

	for(i=0; i<vis2->numrec-1; i++)
	{
		vis2->record[i] = old_record[i];
	}

	if (old_record != NULL) free(old_record);

	new_record = (vis2->record)+(vis2->numrec-1);

	new_record->vis2data = malloc(vis2->nwave * sizeof(DATA));
	new_record->vis2err = malloc(vis2->nwave * sizeof(DATA));
	new_record->flag = malloc(vis2->nwave * sizeof(char));

	if (new_record->vis2data == NULL || new_record->vis2err == NULL ||
	    new_record->flag == NULL) 
	{
		return -1;
	}

	new_record->target_id = id;
	new_record->time = time;
	new_record->mjd = mjd;
	new_record->int_time = int_time;
	new_record->ucoord = u;
	new_record->vcoord = v;
	new_record->sta_index[0] = scope1;
	new_record->sta_index[1] = scope2;
	
	for(i=0 ; i<vis2->nwave; i++)
	{
		new_record->vis2data[i] = vis2data[i];
		new_record->vis2err[i] = vis2err[i];
		new_record->flag[i] = FALSE;
	}
	
	return 0;

} /* add_oi_vis2() */

/************************************************************************/
/* clear_oi_vis2()							*/
/* 									*/
/* Deallocate memory used by an oi_vis2 structure.			*/
/************************************************************************/

void clear_oi_vis2(oi_vis2 vis2)
{
	int	i;

	for(i=0; i<vis2.numrec; i++)
	{
		free(vis2.record[i].vis2data);
		free(vis2.record[i].vis2err);
		free(vis2.record[i].flag);
	}

	if (vis2.record != NULL) free(vis2.record);

} /* clear_oi_vis2() */

/************************************************************************/
/* setup_oi_t3()							*/
/*									*/
/* Initilize an oi_t3 structure.					*/
/************************************************************************/

void setup_oi_t3(oi_t3 *t3, char *date, char *arrname, 
			char *insname, int nwave)
{
	t3->revision = 1;
	strcpy(t3->date_obs, date);
	strcpy(t3->arrname, arrname);
	strcpy(t3->insname, insname);
	t3->numrec = 0;
	t3->nwave = nwave;
	t3->record = NULL;

} /* setup_oi_t3() */

/************************************************************************/
/* add_oi_t3()								*/
/*									*/
/* Add a record to an oi_t3 structure.					*/
/* Returns 0 if all OK, -1 if out of memory.				*/
/************************************************************************/

int add_oi_t3(oi_t3 *t3, int id, double time, 
		double mjd, double int_time, 
		double *t3amp, double *t3amperr,
		double *t3phi, double *t3phierr,
		double u1, double v1, double u2, double v2,
		int scope1, int scope2, int scope3)
{
	oi_t3_record *old_record, *new_record;
	int	i;

	old_record = t3->record;

	t3->numrec++;
	t3->record = malloc(t3->numrec * sizeof(oi_t3_record));

	if (t3->record == NULL) return -1;

	for(i=0; i<t3->numrec-1; i++)
	{
		t3->record[i] = old_record[i];
	}

	if (old_record != NULL) free(old_record);

	new_record = (t3->record)+(t3->numrec-1);

	new_record->t3amp = malloc(t3->nwave * sizeof(DATA));
	new_record->t3amperr = malloc(t3->nwave * sizeof(DATA));
	new_record->t3phi = malloc(t3->nwave * sizeof(DATA));
	new_record->t3phierr = malloc(t3->nwave * sizeof(DATA));
	new_record->flag = malloc(t3->nwave * sizeof(char));

	if (new_record->t3amp == NULL || new_record->t3amperr == NULL ||
	    new_record->t3phi == NULL || new_record->t3phierr == NULL ||
	    new_record->flag == NULL) 
	{
		return -1;
	}

	new_record->target_id = id;
	new_record->time = time;
	new_record->mjd = mjd;
	new_record->int_time = int_time;
	new_record->u1coord = u1;
	new_record->v1coord = v1;
	new_record->u2coord = u2;
	new_record->v2coord = v2;
	new_record->sta_index[0] = scope1;
	new_record->sta_index[1] = scope2;
	new_record->sta_index[2] = scope3;
	
	for(i=0 ; i<t3->nwave; i++)
	{
		new_record->t3amp[i] = t3amp[i];
		new_record->t3amperr[i] = t3amperr[i];
		new_record->t3phi[i] = t3phi[i];
		new_record->t3phierr[i] = t3phierr[i];
		new_record->flag[i] = FALSE;
	}
	
	return 0;

} /* add_oi_t3() */

/************************************************************************/
/* clear_oi_t3()							*/
/* 									*/
/* Deallocate memory used by an oi_t3 structure.			*/
/************************************************************************/

void clear_oi_t3(oi_t3 t3)
{
	int	i;

	for(i=0; i<t3.numrec; i++)
	{
		free(t3.record[i].t3amp);
		free(t3.record[i].t3amperr);
		free(t3.record[i].t3phi);
		free(t3.record[i].t3phierr);
		free(t3.record[i].flag);
	}

	if (t3.record != NULL) free(t3.record);

} /* clear_oi_t3() */

/************************************************************************/
/* print_oi_array()							*/
/*									*/
/* Print information in an oi_array structure.				*/
/************************************************************************/

void print_oi_array(FILE *fp, oi_array array)
{
	int	i,j;
	double	x,y,z;

	fprintf(fp,"ARRAY STRUCTURE:\n\n");
	fprintf(fp,"Array name : %s\n", array.arrname);
	fprintf(fp,"Revision   : %d\n", array.revision);
	fprintf(fp,"Frame      : %s\n", array.frame);
	fprintf(fp,"X          : %.4f\n", array.arrayx);
	fprintf(fp,"Y          : %.4f\n", array.arrayy);
	fprintf(fp,"Z          : %.4f\n", array.arrayz);
	for(i=0; i<array.nelement; i++)
	{
		fprintf(fp,"Telescope %d %s Station %s:\n",
			array.elem[i].sta_index,
			array.elem[i].tel_name, array.elem[i].sta_name);
		fprintf(fp,"  Diameter : %.1f\n", array.elem[i].diameter);
		fprintf(fp,"  Position : %.4f %.4f %.4f\n",
			array.elem[i].staxyz[0],
			array.elem[i].staxyz[1],
			array.elem[i].staxyz[2]);
	}
	fprintf(fp,"Baselines:\n");
	for(i=0; i<array.nelement-1; i++)
	{
		for(j=i+1; j<array.nelement; j++)
		{
			x = array.elem[i].staxyz[0] - array.elem[j].staxyz[0];
			y = array.elem[i].staxyz[1] - array.elem[j].staxyz[1];
			z = array.elem[i].staxyz[2] - array.elem[j].staxyz[2];
			fprintf(fp,
			"  %s-%s %.2f\n",
				array.elem[i].tel_name,
				array.elem[j].tel_name,
				sqrt(x*x + y*y + z*z));
		}
	}
	fprintf(fp,"\n");

} /* print_oi_array() */

/************************************************************************/
/* print_oi_wavelength()						*/
/*									*/
/* Print information in an oi_wavelength structure.			*/
/************************************************************************/

void print_oi_wavelength(FILE *fp, oi_wavelength wave)
{
	int	i;

	fprintf(fp,"WAVELENGTH STRUCTURE\n\n");
	fprintf(fp,"Revision   : %d\n", wave.revision);
	fprintf(fp,"Camera name: %s\n", wave.insname);
	for(i=0; i<wave.nwave; i++)
	{
		fprintf(fp,"Band %2d um : %.4f %.4f\n", 
			i+1,wave.eff_wave[i]*1e6, wave.eff_band[i]*1e6);
	}
	fprintf(fp,"\n");

} /* print_oi_wavelength() */

/************************************************************************/
/* print_oi_target()							*/
/*									*/
/* Print information in an oi_target structure.				*/
/************************************************************************/

void print_oi_target(FILE *fp, oi_target targets)
{
	int	i;
	char	s[256];

	printf("TARGETS STRUCTURE.\n\n");
	
	for(i=0; i<targets.ntarget; i++)
	{
		fprintf(fp,"Target Name: %s\n", targets.targ[i].target);
		fprintf(fp,"ID         : %d\n", targets.targ[i].target_id);
		fprintf(fp, "RA         : %s\n", time_to_string(
			targets.targ[i].raep0/180.0*M_PI, s));
		fprintf(fp,"DEC        : %s\n", angle_to_string(
			targets.targ[i].decep0/180.0*M_PI, s));
		fprintf(fp,"RA Err     : %s\n", time_to_string(
			targets.targ[i].ra_err/180.0*M_PI, s));
		fprintf(fp,"DEC Err    : %s\n", angle_to_string(
			targets.targ[i].dec_err/180.0*M_PI, s));
		fprintf(fp,"Sys Vel    : %.4f\n", 
				targets.targ[i].sysvel/1000.0);
		fprintf(fp,"Sys Type   : %s\n", targets.targ[i].veltyp);
		fprintf(fp,"Sys Def    : %s\n", targets.targ[i].veldef);
		fprintf(fp,"Pm RA      : %.4f\n", targets.targ[i].pmra*3.6e6);
		fprintf(fp,"Pm Dec     : %.4f\n", targets.targ[i].pmdec*3.60e6);
		fprintf(fp,"Pm RA Err  : %.4f\n", 
			targets.targ[i].pmra_err/180.0*M_PI*3600.0);
		fprintf(fp,"Pm Dec Err : %.4f\n", 
			targets.targ[i].pmdec_err/180.0*M_PI*3600.0);
		fprintf(fp,"Parallax   : %.4f\n", 
				targets.targ[i].parallax*3.6e6);
		fprintf(fp,"Para Err   : %.4f\n", 
				targets.targ[i].para_err*3.6e6);
		fprintf(fp,"Spec Type  : %s\n\n", targets.targ[i].spectyp);
	}

} /* print_oi_target() */

/************************************************************************/
/* print_oi_vis()							*/
/*									*/
/* Print information in an oi_vis structure.				*/
/************************************************************************/

void print_oi_vis(FILE *fp, oi_vis vis)
{
	int	i,j;

	fprintf(fp,"COMPLEX VISIBILITY STRUCTURE:\n\n"); 

	fprintf(fp,"Date Obs   : %s\n", vis.date_obs);
	fprintf(fp,"Array Name : %s\n", vis.arrname);
	fprintf(fp,"Instrument : %s\n", vis.insname);
	fprintf(fp,"Revision   : %d\n", vis.revision);

	for(i=0; i<vis.numrec; i++)
	{
		fprintf(fp,"\nTarget ID  : %d\n", vis.record[i].target_id);
		fprintf(fp,"Time       : %.3f\n", vis.record[i].time);
		fprintf(fp,"MJD        : %.5f\n", vis.record[i].mjd);
		fprintf(fp,"Integration: %.3f\n", vis.record[i].int_time);
		fprintf(fp,"U/V        : %.4f %.4f\n", vis.record[i].ucoord,
						vis.record[i].vcoord);
		fprintf(fp,"Scopes     : %d %d\n", 
			vis.record[i].sta_index[0],
			vis.record[i].sta_index[1]);
		fprintf(fp,"Visibilies\n");
		for(j=0; j<vis.nwave; j++)
		{
			fprintf(fp,"  Vis/Phase: %.4f %.4f %.4f %.4f\n",
				vis.record[i].visamp[j],
				vis.record[i].visamperr[j],
				vis.record[i].visphi[j],
				vis.record[i].visphierr[j]);
		}
	}
	fprintf(fp,"\n");

} /* print_oi_vis() */

/************************************************************************/
/* print_oi_vis2()							*/
/*									*/
/* Print information in an oi_vis2 structure.				*/
/************************************************************************/

void print_oi_vis2(FILE *fp, oi_vis2 vis2)
{
	int	i,j;

	fprintf(fp,"VISIBILITY SQUARED STRUCTURE:\n"); 

	fprintf(fp,"Date Obs   : %s\n", vis2.date_obs);
	fprintf(fp,"Array Name : %s\n", vis2.arrname);
	fprintf(fp,"Instrument : %s\n", vis2.insname);
	fprintf(fp,"Revision   : %d\n", vis2.revision);

	for(i=0; i<vis2.numrec; i++)
	{
		fprintf(fp,"\nTarget ID  : %d\n", vis2.record[i].target_id);
		fprintf(fp,"Time       : %.1f\n", vis2.record[i].time);
		fprintf(fp,"MJD        : %.5f\n", vis2.record[i].mjd);
		fprintf(fp,"Integration: %.1f\n", vis2.record[i].int_time);
		fprintf(fp,"U/V        : %.1f %.1f\n", vis2.record[i].ucoord,
						vis2.record[i].vcoord);
		fprintf(fp,"Scopes     : %d %d\n", 
			vis2.record[i].sta_index[0],
			vis2.record[i].sta_index[1]);
		fprintf(fp,"Vis^2\n");
		for(j=0; j<vis2.nwave; j++)
		{
			fprintf(fp,"  Vis2     : %.4f %.4f\n",
				vis2.record[i].vis2data[j],
				vis2.record[i].vis2err[j]);
		}
	}
	fprintf(fp, "\n");

} /* print_oi_vis2() */

/************************************************************************/
/* print_oi_t3()							*/
/*									*/
/* Print information in an oi_t3 structure.				*/
/************************************************************************/

void print_oi_t3(FILE *fp, oi_t3 t3)
{
	int	i,j;

	fprintf(fp, "TRIPPLE PRODUCT STRUCTURE:\n"); 

	fprintf(fp, "Date Obs   : %s\n", t3.date_obs);
	fprintf(fp, "Array Name : %s\n", t3.arrname);
	fprintf(fp, "Instrument : %s\n", t3.insname);
	fprintf(fp, "Revision   : %d\n", t3.revision);

	for(i=0; i<t3.numrec; i++)
	{
		fprintf(fp, "\nTarget ID  : %d\n", t3.record[i].target_id);
		fprintf(fp, "Time       : %.1f\n", t3.record[i].time);
		fprintf(fp, "MJD        : %.5f\n", t3.record[i].mjd);
		fprintf(fp, "Integration: %.1f\n", t3.record[i].int_time);
		fprintf(fp, "U/V        : %.1f/%.1f %.1f/%.1f\n",
			t3.record[i].u1coord, t3.record[i].v1coord,
			t3.record[i].u2coord, t3.record[i].v2coord);
		fprintf(fp, "Scopes     : %d %d %d\n", 
			t3.record[i].sta_index[0],
			t3.record[i].sta_index[1],
			t3.record[i].sta_index[2]);
		fprintf(fp, "T3 Visibilies\n");
		for(j=0; j<t3.nwave; j++)
		{
			fprintf(fp, "T3Amp/Phase: %.4f %.4f %.4f %.4f\n",
				t3.record[i].t3amp[j],
				t3.record[i].t3amperr[j],
				t3.record[i].t3phi[j],
				t3.record[i].t3phierr[j]);
		}
	}
	fprintf(fp,"\n");

} /* print_oi_t3() */
