/************************************************************************/
/* background_mirror.c							*/
/*                                                                      */
/* This file controls background_mirror function. Looks at the status   */
/* and spits it too a socket.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : March 1999     				                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/*
 * Static global - TRUE if not looking at keyboard
 */

#define MAX_MIRRORS	10
static int mirror_socket_fd[MAX_MIRRORS] = {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};
static int mirror_socket_ports[MAX_MIRRORS] = {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};
static char *mirror_names[MAX_MIRRORS];
static int number_mirrors = 0;

/************************************************************************/
/* open_mirror_socket()							*/
/*									*/
/* User callable function. Tries to open a socket to a given machine	*/
/* on a given socket. If this works it adds the mirror_bakcground()     */
/* function to the background processes to start send the status window */
/* information to the socket.						*/
/************************************************************************/

int open_mirror_socket(int argc, char **argv)
{
	int	socket_number;
	char	machine_name[81],s[81];
	int	i;

	/* If there are none send a warning and leave */

	if (number_mirrors >= MAX_MIRRORS)
	{
		return error(WARNING,"No more mirrors available.");
	}

	/* Find the first available mirror slot */

	for(i = 0; i < MAX_MIRRORS && mirror_socket_fd[i] != -1; i++);

	/* If there are none send a warning and leave (Should be redundent) */

	if (i >= MAX_MIRRORS)
	{
		return error(WARNING,"No more mirrors available.");
	}

	/* Try the command line args */

	socket_test_args(2,"machine socket");
        if (argc > 1)
        {
                sscanf(argv[1],"%s",machine_name);
        }
        else
        {
                sprintf(machine_name,"ctrscrut.mtwilson.edu");
                if (quick_edit("Machine",machine_name,machine_name,
			NULL,STRING) == KEY_ESC) return NOERROR;
        }

        if (argc > 2)
        {
                sscanf(argv[2],"%d",&socket_number);
        }
        else
        {
                sprintf(s,"3000");
                if (quick_edit("Socket",s,s, NULL,FLOAT) == KEY_ESC) 
			return NOERROR;
		sscanf(s,"%d",&socket_number);
        }

	/* Try and open the socket */

	if ((mirror_socket_fd[i]
		= open_connect_socket(machine_name,socket_number,TRUE)) == -1)
	{
		return error(ERROR,"Failed to open socket %d on %s",
			socket_number,machine_name);
	}
	
	/* Add the name to the list of names */

	if ((mirror_names[i] = malloc(strlen(machine_name+1))) == NULL)
	{
		mirror_socket_fd[i] = -1;
		return error(ERROR,"Ran out of memeory.");
	}
	strcpy(mirror_names[i], machine_name);

	/* and the port number */

	mirror_socket_ports[i] = socket_number;

#ifdef MIRROR_AS_BACKGROUND
	/* Try and add the background task */

	if (number_mirrors == 0 && background_add(mirror_background) != NOERROR)
	{
		close_mirror_socket(argc,argv);
		return error(ERROR,"Failed to add mirror background job.");
	}
#endif

	/* OK, incriment the number of mirrors */

	number_mirrors++;

	return NOERROR;

} /* open_mirror_socket() */

/************************************************************************/
/* close_mirror_socket()						*/
/*									*/
/* User callable funtion, closes mirror socket.				*/
/************************************************************************/

int close_mirror_socket(int argc, char **argv)
{
	int	socket_number;
	char	machine_name[81],s[81];
	int 	i;

	if (number_mirrors == 0) return NOERROR;

	/* Try the command line args */

	socket_test_args(2,"machine socket");
        if (argc == 3)
        {
                sscanf(argv[1],"%s",machine_name);
                sscanf(argv[2],"%d",&socket_number);
		for(i=0; i<MAX_MIRRORS; i++)
		{
			if (mirror_socket_fd[i] == -1) continue;
			if (strcmp(machine_name,mirror_names[i])==0 &&
			    mirror_socket_ports[i] == socket_number) break;
		}
		if (i >= MAX_MIRRORS)
			return error(ERROR,"No mirror on machine %s port %d.",
				machine_name, socket_number);
#ifdef MIRROR_AS_BACKGROUND
		if (number_mirrors == 1) 
			background_del(mirror_background);
#endif
		close_connect_socket(mirror_socket_fd[i]);
		mirror_socket_fd[i] = -1;
		free(mirror_names[i]);
		number_mirrors--;
        }
        else
        {
		for(i=0; i<MAX_MIRRORS; i++)
		{
			if (mirror_socket_fd[i] == -1) continue;

			sprintf(s,"A mirror is open on machine %s port %d.",
				mirror_names[i], mirror_socket_ports[i]);
			if (ask_yes_no(s,"Do you want to close it ?"))
			{
				if (mirror_socket_fd[i] == -1) continue;
#ifdef MIRROR_AS_BACKGROUND
				if (number_mirrors == 1) 
					background_del(mirror_background);
#endif
				close_connect_socket(mirror_socket_fd[i]);
				mirror_socket_fd[i] = -1;
				free(mirror_names[i]);
				number_mirrors--;
			}
		}
        }

	return NOERROR;

} /* close_mirror_socket() */

/************************************************************************/
/* mirror_background.c							*/
/*									*/
/* Should run in the background and spits the status window info	*/
/* down the mirror_socket line by line.					*/
/************************************************************************/

static int mirror_line = -1;

int mirror_background(void)
{
	char   s[100],buf[100];
	int    i;

	/* Incriment the line we are up to */

	if (++mirror_line >= STATUS_LINES) mirror_line = 0;

	/* suck the line into memory */

	for (i=0; i<STATUS_COLS ; i++)
	{
		s[i] = mvwinch(status_window,mirror_line,i);
	}
	s[STATUS_COLS] = 0;

	/* Cycle through all mirrors */

	for(i=0; i<MAX_MIRRORS; i++)
	{
		/* If nothing is connected do nothing */

		if (mirror_socket_fd[i] < 0) continue;

		/* Is the socket still there? */

		if (char_waiting(mirror_socket_fd[i]) &&
			socket_gets(mirror_socket_fd[i],buf,SOCKET_BUFFER) == 
			(char *)EOF)
		{
			/* It has closed! */

			message(system_window,"Status mirror on %s port %d closed.",
				mirror_names[i],mirror_socket_fd[i]);
#ifdef MIRROR_AS_BACKGROUND
			if (number_mirrors == 1) 
				background_del(mirror_background);
#endif
			close_connect_socket(mirror_socket_fd[i]);
			mirror_socket_fd[i] = -1;
			free(mirror_names[i]);
			number_mirrors--;

			return NOERROR;
		}

		/* Otherwise spit it out the message */

		sprintf(buf,"%02d%s",mirror_line,s);
		write(mirror_socket_fd[i],buf,sizeof(buf));

	}

	return NOERROR;
	
} /* mirror_background() */

/************************************************************************/
/* print_status()							*/
/*									*/
/* Intended for sockets only. Spits out whatever is in the 		*/
/* status_window.							*/
/************************************************************************/

int print_status(int argc, char **argv)
{
	int    i,j;

	/* This must be s socket */

	if (active_socket == -1)
	{
		message(system_window,"This command only works from a socket.");
		return NOERROR;
	}

	/* suck the line into memory */

	for (j=0; j<STATUS_LINES ; j++)
	{
		for (i=0; i<STATUS_COLS ; i++)
		{
			socket_print(active_socket,"%c",
				mvwinch(status_window,j,i));
		}
		socket_print(active_socket,"\n");
	}

	return NOERROR;

} /* print_status() */
