/************************************************************************/
/* startui.c								*/
/*                                                                      */
/* Starts User Interafce.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"
#include "../../charalib/chara.h"
#include "../../include/chara_messages.h"
#include "../../sockman/sockman.h"
#include "../../astromod/astromod.h"

static struct smenu *mainmenu;	/* Pointer to main window */
static void (*user_open_function)(void) = NULL;
char	sockman_name[256];
char 	use_socket_manager = TRUE;	/* Should we use it? */
char use_non_socket_manager = TRUE;     /* If we do which one? */

/************************************************************************/
/* start_ui()								*/
/*									*/
/* Sets up and starts user interface.					*/
/*									*/
/************************************************************************/

void start_ui(void)
{
	signal( SIGINT, sig_end);	/* Catch external kill signal */

        /* Make sure we catch and SIGPIPE signals */

        if (signal(SIGPIPE, sig_pipe) == SIG_ERR)
        {
                error(ERROR,"Failed to grab SIGPIPE signal");
        }

        /* Is there a user defined starting function call ? */

        if ( user_open_function != NULL) (*user_open_function)();

	menu(mainmenu, (struct smenu *)NULL, 0);	/* Start menu system */

	/* Note: Exit of the programme is via ui_end() */

} /* start_ui() */

/************************************************************************/
/* initialise_ui()							*/
/*									*/
/* This function will initialise everything that needs to be 		*/
/* initialised prior to putting up the main menu.			*/
/* Consult design and source for more information.			*/
/* The parameter menufile is the file name (full path) of the menu	*/
/* description file, helpfile is the same for help.			*/
/* The port is the port used for the command socket.  			*/
/* NOTE: This calls init_jobs() so it should never be called again,	*/
/* and this function should be caleld before opening any other server	*/
/* connections as a client.						*/
/*									*/
/************************************************************************/

void 	initialise_ui(char *menufile, char *helpfile, char *server_name)
{
#ifdef USE_OLD_PORT_FILE
	int	com_port, data_port, server_port;
	char	machine[256];
	char	filename[256];
	char	s[512];
	FILE	*output;
#endif
	char	hostname[256];
	int	ret;

	*sockman_name = 0; /* We have no socket name yet. */

	init_curses();  /* Initialise the curses system */

	working();	/* Message to user */

	/* Set ourselves up as a client */

	init_jobs();

	/* Are we even able to use the new version ? */

	if (!use_socket_manager) use_non_socket_manager = FALSE;

	if (use_non_socket_manager)
	{
	    if ((ret =
		socket_manager_non_socket_get_process(server_name, &my_process))
		!= 0 && ret != 5)
	    {
		use_non_socket_manager = FALSE;
	    }
	}

	socket_manager_set_version(use_non_socket_manager);

	/* Now we check with the socket manager */

	if (use_socket_manager)
	{
          switch (socket_manager_get_process(server_name, &my_process))
          {
            case 0:

                /* Yes, the process already exists, ping it */

                switch(socket_manager_ping_process(my_process))
                {
                    case 0: error(FATAL,
                           "Live process %s already running on %s:%d.\n",
                            my_process.name, my_process.machine,
                            my_process.messages);
                            break;

                    default: error(FATAL,
"Socket manager reports dead process %s on %s.\nTry removing this entry using sockman client.\n",
                            my_process.name, my_process.machine);
                            break;
                }
                break;

                /* Other error condiftions */

            case 1: error(FATAL,"Socket Manager dead!\n");
                break;

            case 2: error(FATAL, 
                        "Failed to speak with socket manager.\n");
                break;

            case 3: break;

            case 4: error(FATAL, 
                        "Names do not match with socket manager.\n");
                break;

	    default: error(FATAL,"We should never get here.");
          }
	}

#ifdef USE_OLD_PORT_FILE
	/* 
	 * Does this program already seem to be running? 
	 * This is here for backards compat with older clients.
	 */

	sprintf(filename, "%s%s.port", get_etc_directory(s), server_name);
	if (read_server_ports(server_name, machine, &com_port,
				&data_port, &server_port))
	{
		/* There's a file there..... is there someone listening? */

		if (send_socket_command(machine, com_port, "ping") == NOERROR)
		{
			error(FATAL,
			    "%s already running on %s.\n",server_name,machine);
		}

		sprintf(s,
			"%s already running on %s.\n",server_name,machine);
		strcat(s,"It is not responding to PING.\n");
		sprintf(machine,
			"Please remove file %s if all is OK.\n", filename);
		strcat(s, machine);
		error(FATAL,s);
	}
#endif

	init_auto();	/* Initialise the auto function caller */

	init_display();	/* Initialise the display structure */

        set_server_name(server_name);

	if (open_command_socket() != NOERROR)
	{
		error(WARNING,"No socket command functionality.");
	}

	if (open_data_socket() != NOERROR)
	{
		error(WARNING,"No data socket functionality.");
	}

        /* Set up the client port */

        if (open_message_socket() != NOERROR)
        {
                error(ERROR,
                    "Failed to open client port - no client functionality.");
        }
        standard_jobs();

	init_help(helpfile);	/* after init_curses !!!! */

	init_scripts();

	mainmenu = init_menu_structure(menufile); /* Parse data in menus file */

	show_ports(0, NULL);

#ifdef USE_OLD_PORT_FILE
	/* 
	 * Write the ports file 
	 * This is for backwards compatibility for those older
	 * clients who do not know about the socket manager.
	 */

	if ((output = fopen(filename,"w")) == NULL)
	{
		error(FATAL,"Failed to open ports file.\n");
	}
	fprintf(output, "%s %d %d %d\n", get_machine_name(hostname), 
		get_command_port(), get_data_port(), get_server_port());
	fclose(output);
#endif

	/* Now register with the real socket manager */

	if (use_socket_manager)
	{
	  strcpy(my_process.name, server_name);
	  strcpy(my_process.machine, get_machine_name(hostname));
	  my_process.pid = getpid();
	  my_process.commands = get_command_port();
	  my_process.data = get_data_port();
	  my_process.messages = get_server_port();
	  *my_process.restart_command = 0;

	  switch(socket_manager_set_process(my_process))
	  {
		
                case 1: error(FATAL,"Failed to talk to socket manager.");
                        break;

                case 2: error(FATAL,
                                "Failed to send message to socket manager.");
                        break;

                case 3: error(FATAL, "Process %s already exists!.", 
				server_name);
                        break;

                case 4: error(FATAL,
                                "Failed to match names with socket manager.");
                        break;

                case 5: error(FATAL,
                                "Machine name %s not allowed.",
				my_process.machine);
                        break;

                case 6: error(FATAL, "Restart command more than one line.");
                        break;

                case 7: error(FATAL,"Name, machine or PID is missin.");
                        break;

                default: break;

	  }
	  strcpy(sockman_name, server_name);
	}

	not_working();	/* Message to user */

} /* initialise_ui() */

/************************************************************************/
/* set_user_open_function()                                             */
/*                                                                      */
/* Setup a function for the system to call when started. Good for       */
/* proper setup of things.                                              */
/************************************************************************/

void set_user_open_function(void (*function)(void))
{
        user_open_function = function;

} /* set_user_open_function() */

