/************************************************************************/
/* window.c								*/
/*                                                                      */
/* 	This file contains all the window manipulation functions	*/
/*	for the user interface. That is, any function in the UI		*/
/*	that calls a curses routine should be in this file.		*/
/*	At the moment these functions are :				*/
/*									*/
/*	init_curses() : Sets up curses for whole programme		*/
/*	close_curses() : Closes curses at end of programme		*/
/*	set_up_menu_window() : Creates and fills menu window		*/
/*	close_menu_window() : Closes and deallocates menu window	*/
/*	refresh_menu_window() : Updates screen image of menu		*/
/*	ask_yes_no() : Requests confirmation from user			*/
/*	heading() : Puts something into the heading window for you	*/
/*	message() : Puts something into the system_windows for you	*/
/*	printd() : Places a line of text into the display structure	*/
/*	clear_display() : Clears the display structure			*/
/*	refresh_screen() : Reconstructs the UI screen			*/
/*									*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/* Global definitions */

WINDOW   *status_window;
WINDOW   *bmain_window;
WINDOW   *main_window;
WINDOW   *heading_window;
WINDOW   *sub_main_window;
WINDOW   *system_window;
WINDOW   *system_ol_window;
WINDOW   *command_window;
WINDOW   *mode_window;
WINDOW   *active_window;
WINDOW   *active_menuwindow;
struct sdisplay *display;


/*
 * A local global to see if curses is open or not
 */

static	bool	curses_open = FALSE;

/************************************************************************/
/* init_curses()							*/
/* Process 3.1								*/
/*									*/
/* Initialises curses for whole programme. It also sets up a basic	*/
/* title page on stdscn. It also creates all the main global		*/
/* windows (see window.h) for the system. The normal screen is		*/
/* also set up.								*/
/*									*/
/************************************************************************/

void 	init_curses(void)
{
	/* If curses is already open exit */

	if (curses_open) return;

	initscr();	/* Initialise curses */
	cbreak();	/* Raw mode */
	noecho();	/* No character echo */
	nonl();		/* No newline appended to returns */
	intrflush(stdscr,FALSE); /* No flushing on interrupt */
#ifdef JUNK
	cursoff();	/* Turn off hardware cursor */
#endif
	/*
	 * Something strange from AIX....
	 */

	ESCDELAY = 0;

	/* Do the mouse initilization */

#ifdef __NCURSES_H
	{
	        mmask_t old_mask;
		mousemask(ALL_MOUSE_EVENTS,&old_mask);
	}
#endif

	/* Set up all normal working windows */
	
	if ((status_window = newwin(STATUS_LINES, STATUS_COLS, STATUS_BEGIN_Y,
				  STATUS_BEGIN_X)) == (WINDOW *)ERR ||
		status_window == 0)
	{
		error(FATAL,FATAL_1,"status window");
	}

	if ((bmain_window = newwin(BMAIN_LINES, BMAIN_COLS, BMAIN_BEGIN_Y,
				  BMAIN_BEGIN_X)) == (WINDOW *)ERR ||
		bmain_window == 0)
	{
		error(FATAL,FATAL_1,"bmain window");
	}

	if ((main_window =subwin(bmain_window,MAIN_LINES,MAIN_COLS,MAIN_BEGIN_Y,
				  MAIN_BEGIN_X)) == (WINDOW *)ERR ||
		main_window == 0)
	{
		error(FATAL,FATAL_1,"main window");
	}

	if ((system_window = newwin(SYSTEM_LINES, SYSTEM_COLS, SYSTEM_BEGIN_Y,
				  SYSTEM_BEGIN_X)) == (WINDOW *)ERR ||
		system_window == 0)
	{
		error(FATAL,FATAL_1,"system window");
	}

	if ((system_ol_window = newwin(SYSTEM_LINES, SYSTEM_COLS, 
			SYSTEM_BEGIN_Y, SYSTEM_BEGIN_X)) == (WINDOW *)ERR ||
		system_ol_window == 0)
	{
		error(FATAL,FATAL_1,"system overlay window");
	}

	if ((command_window = newwin(COMMAND_LINES, COMMAND_COLS, 
			COMMAND_BEGIN_Y, COMMAND_BEGIN_X)) == (WINDOW *)ERR ||
		command_window == 0)
	{
		error(FATAL,FATAL_1,"command window");
	}

	if ((mode_window = newwin(MODE_LINES, MODE_COLS, 
			MODE_BEGIN_Y, MODE_BEGIN_X)) == (WINDOW *)ERR ||
		mode_window == 0)
	{
		error(FATAL,FATAL_1,"mode window");
	}

	if ((sub_main_window = subwin(bmain_window,SUB_MAIN_LINES,SUB_MAIN_COLS,
			SUB_MAIN_BEGIN_Y, SUB_MAIN_BEGIN_X)) == (WINDOW *)ERR ||
		sub_main_window == 0)
	{
		error(FATAL,FATAL_1,"sub-main window");
	}

	if ((heading_window = subwin(bmain_window,HEADING_LINES, HEADING_COLS, 
			HEADING_BEGIN_Y, HEADING_BEGIN_X)) == (WINDOW *)ERR ||
		heading_window == 0)
	{
		error(FATAL,FATAL_1,"heading window");
	}

	/*
	 * Put box around main window (this is bmain_window) and it
	 * should not be used again. Use main_window.
	 */

	box(bmain_window,0,0);
	touchwin(bmain_window);
	wrefresh(bmain_window);

	/* Put prompt on command line */

	mvwaddstr(command_window,0,0,PROMPT);
	touchwin(command_window);
	wrefresh(command_window);

	/* 
	 * Set system window and system overlay window and the mode window
 	 * to be always highlighted 
	 */

	wstandout(system_window);
	wstandout(system_ol_window);
	wstandout(mode_window);

	/* System window is used for input and should allow function keys */

	keypad(system_window,TRUE);

	wattrset(heading_window,A_STANDOUT | A_REVERSE);

	/* Set active window to sub_main_window */

	active_window = sub_main_window;

	/* Let the other routines know curses is open */

	curses_open = TRUE;

	/* Make the stdin nonblocking for the kbhit() function. */

	if (fcntl(fileno(stdin), F_SETFL, O_NONBLOCK) == -1)
		error(FATAL,"Failed to make stdin non-blocking.");

} /* init_curses() [3.1] */

/************************************************************************/
/* close_curses()							*/
/*									*/
/* Closes curses at end of programme. 					*/
/*									*/
/************************************************************************/

void	close_curses(void)
{
	/* See if it is already closed */

	if (!curses_open) return;

	/* Shut things down */

#ifdef JUNK
	curson();
#endif
	mvcur(0,COLS-1,LINES-2,0);
	echo();
	nocbreak();
	nl();
	endwin();	

	/* Curses now closed */

	curses_open = FALSE;

} /* close_curses() */

/************************************************************************/
/* set_up_menu_window()							*/
/*									*/
/* Creates a window for a menu. It then fills the menu information	*/
/* into the window. Returns a pointer to the menu window.		*/
/*									*/
/************************************************************************/

WINDOW 	*set_up_menu_window(
	struct 	smenu *thismenu,		/* Pointer to current menu */
	struct 	smenu *lastmenu,		/* Pointer to last menu */
	int	     level)		/* Level in menu hierarchy */
{
	/* Internal variables */

	WINDOW	*menu_window;	/* Pointer to window for menu */
	int	functionpt;	/* Index to functions[] array */
	struct smenu *menupt;	/* Pointer to menu */
	int	entrynum;	/* Index to menu entries */

	/* Create the window */

	if ((menu_window = newwin(MENU_LINES, MENU_COLS, MENU_BEGIN_Y,
				  MENU_BEGIN_X)) == (WINDOW *)ERR)
	{
		error(FATAL,FATAL_1,"menu window");
	}

	/* Print the menu title */

	mvwaddstr(menu_window,3,0,MENU_INFOB);
	if (lastmenu != (struct smenu *) NULL)
	{
		mvwprintw(menu_window,3,0,"Previous menu : %s",lastmenu->name);
	}
	else
	{
		mvwaddstr(menu_window,3,0,"Previous menu : None");
	}

	mvwaddstr(menu_window,2,0,MENU_INFOB);
	mvwprintw(menu_window,2,0," Current menu : %s",thismenu->name);
	mvwaddstr(menu_window,4,0,MENU_INFOB);
	mvwprintw(menu_window,4,0,"   Menu Depth : %d",level);

	/* Print up standard menu info */

	mvwaddstr(menu_window,6,0,MENU_INFO1);
	mvwaddstr(menu_window,7,0,MENU_INFO2);
	mvwaddstr(menu_window,8,0,MENU_INFO3);

	/* Print up menu selections */

	for (entrynum = 0; entrynum < MAXENTRY; entrynum++)
	{
		functionpt = thismenu->entry[entrynum].pt_funct;
		menupt = thismenu->entry[entrynum].pt_menu; 

		if (functionpt == 0 && (struct smenu *)menupt == NULL) 
			break;

		mvwprintw(menu_window,entrynum,39,"F%d",entrynum+1);
		mvwprintw(menu_window,entrynum,43,"%s",
			thismenu->entry[entrynum].prompt);
	}

	return menu_window;

} /* set_up_menu_window() */

/************************************************************************/
/* close_menu_window()							*/
/*									*/
/* Closes the menu window and releases memory. 				*/
/*									*/
/************************************************************************/

void 	close_menu_window(WINDOW *menuwindow)
{
	delwin(menuwindow);

} /* close_menu_window() */

/************************************************************************/
/* refresh_menu_window()						*/
/*									*/
/* Refreshes the menu window ie, actually makes it visible to user	*/
/* Some information regarding menu movement is also added to the	*/
/* screen.								*/
/*									*/
/************************************************************************/

void 	refresh_menu_window(
	WINDOW 	*menuwindow,		/* Pointer to window to refresh */
	int	level)			/* Current menu level */
{

	/* Put up the title */

	wstandout(menuwindow);
	mvwaddstr(menuwindow,0,0,TITLE);
	wstandend(menuwindow);

	/* 
	 * First place in the <TAB> or path swapping info
	 * Check the first entry in the store array which is
	 * which menu you will reach by hitting TAB.
	 * If this is a NULL print nothing if the level is zero
	 * or print "Operational after <ESC>".
	 */

	mvwaddstr(menuwindow,9,0,MENU_INFOB);
	if (store[0] != (struct smenu *) NULL)
	{
		mvwprintw(menuwindow,9,0,MENU_INFO4,store[0]->name);
	}
	else if (level != 0)
	{
		mvwaddstr(menuwindow,9,0,MENU_INFO5);
	}
	
	/* Now touch the window and refresh it */

	touchwin(menuwindow);
	wrefresh(menuwindow);

} /* refresh_menu_window() */

/************************************************************************/
/* ask_yes_no()								*/
/*									*/
/* Recieves two strings as arguments. Puts one on first line of system	*/
/* window, second string on second line of system window. Waits for	*/
/* user to type 'y' or 'n' and returns TRUE or FALSE respectively.	*/
/*									*/
/************************************************************************/

int	ask_yes_no(char *string1,char *string2)
{
	/* Internal variables */

	int	answer;		/* from get_command() */
	char    s[81];		/* Buffer for socket data */
#ifdef __NCURSES_H
        MEVENT  mouse;          /* A mouse event. */
#endif

	/* let's do it */

	message(system_ol_window,"%s\n%s <YES/NO>",string1,string2);

	/* Are we talking to a socket ? */

	if (active_socket != -1)
	{
		socket_gets(active_socket, s, 81);
		answer = *s;
	}
	else
	{
#ifdef __NCURSES_H
		while ((answer = get_command()) == KEY_MOUSE)
		{
			getmouse(&mouse);
			if (mouse.bstate == BUTTON1_CLICKED &&
			    mouse.y == SYSTEM_BEGIN_Y+1)
			{
				if (mouse.x > strlen(string2)+1 &&
				    mouse.x < strlen(string2)+5)
				{
					answer = 'Y';
					break;
				}
				else if (mouse.x > strlen(string2)+5 &&
				    mouse.x < strlen(string2)+8)
				{
					answer = 'N';
					break;
				}
			
			}
		}
#else
		answer = get_command();
#endif
	}

	/* Reset the windows */

	touchwin(system_window);
	wrefresh(system_window);

	return (answer == 'Y' || answer == 'y') ? YES : NO;

} /* ask_yes_no() */

/************************************************************************/
/* heading()								*/
/*									*/
/* Puts up user supplied strings in the given window.			*/
/* String 1 is centred and bold on line 0				*/
/* String 2 is as is on line 1						*/
/*									*/
/************************************************************************/

void	heading(WINDOW *win,char *string1,char *string2)
{
	werase(win);
	wmove(win,0,0);
	mvwaddstr(win,0,(HEADING_COLS-strlen(string1))/2,string1);
	mvwaddstr(win,1,0,string2);
	touchwin(win);
	wrefresh(win);

} /* heading() */

/************************************************************************/
/* message()								*/
/*									*/
/* Puts messages in either the system or the system_ol_window. Will	*/
/* accept up to 5 arguments. Limit of 2 lines.				*/
/* This function is only responsible for putting up the message and it	*/
/* is up to the calling function to clear the window it uses if 	*/
/* necessary.								*/
/*									*/
/************************************************************************/

int 	message(WINDOW *win, char *fmt, ...)
{
	/* Internal Variables */

	char	message_string[161];	/* 2 lines */
	va_list	args;
	int	return_value;

	/* Create the whole error string */

	va_start(args, fmt);
	return_value = vsprintf(message_string, fmt, args);

	/* To the screen or to a socket ? */

	if (active_socket != -1)
	{
		socket_print(active_socket,"%s\n",message_string);
	}
	else
	{
		nl(); 			/* To ensure new lines get new lines */
		werase(win);
		mvwaddstr(win,0,0,message_string);
		nonl();
		touchwin(win);
		wrefresh(win);
	}

	return return_value;

} /* message() */

/************************************************************************/
/* printd()								*/
/*									*/
/* Behaves exactly (almost) like printf except it places the stuff	*/
/* onto the next availbale line in the display structure.		*/
/* Lines that are too long will be truncated to their maximum length.	*/
/* Only one line at a time can be placed into the display structure	*/
/* via this function. Return value is the number of characters		*/
/* printed or -1 if something went wrong.				*/
/* Only up to five arguments can be used.				*/
/* If the display structure is full the whole thing is ignored.		*/
/************************************************************************/

int	printd(char *fmt, ...)
{
	char	string[100];	/* Temp storage for string */
	int	return_value;	/* return value you dumby ! */
	va_list	args;

	/* First see if we have room for this line */

	if (display->number_items >= NUM_TEXT_LINES) return -1;
	if ((display->string[display->number_items] = 
				(char *)malloc(LINE_LENGTH)) == NULL)
	{
		display->string[display->number_items] = "";
	 	return -1;
	}

	/* Print it into string buffer */

	va_start(args, fmt);
	return_value = vsprintf(string, fmt, args);

	/* Copy it into display structure */

	strncpy(display->string[display->number_items],string,LINE_LENGTH);

	/* increment the number of items */

	display->number_items++;

	/* That is all apart from length check */

	if (strlen(string) > LINE_LENGTH-1) return -1; else return return_value;

} /* printd() */

/************************************************************************/
/* clear_display()							*/
/*									*/
/* Clears the display structure. Sets the number of entries to zero	*/
/* and sets all values to NULL inside text itself.			*/
/************************************************************************/

void	clear_display(void)
{
	int	i;	/* Looping variables */

	/* set values of display structure = NULL */

	for(i=0; i < display->number_items ; i++) 
	{
		free(display->string[i]);
		display->string[i] = "";
	}

	display->number_items = 0;

} /* clear_display() */

/************************************************************************/
/* refresh_screen()							*/
/*									*/
/* Re-builds the user interface screen.					*/
/************************************************************************/

void	refresh_screen(void)
{
#ifdef JUNK
	cursoff();
#endif
	touchwin(bmain_window);
	wrefresh(bmain_window);

	touchwin(system_window);
	wrefresh(system_window);

	touchwin(command_window);
	wrefresh(command_window);

	touchwin(active_window);
	wrefresh(active_window);

	doupdate();

} /* refresh_screen() */

/*
 * Global for plain screen functions
 */

static bool plain_screen = FALSE;

/************************************************************************/
/* plain_screen_on()							*/
/*									*/
/* Clear standard screen and puts cursor at top left. Allows programme	*/
/* to use screen in normal manner. Use plain_screen_off() to stop it.	*/
/************************************************************************/

void plain_screen_on(void)
{
	/* If we arealy in plain screen mode exit */

	if (plain_screen) return;

	/* If curses is not open, open it up */

	if (curses_open) 
	{
		erase();
		move(0,0);
		refresh();
		nl();
		nocbreak();
		echo();
#ifdef JUNK
		curson();
#endif
	}
	else
	{
		ui_clear_screen();
	}

	/* Now in plain screen mode */

	plain_screen = TRUE;

} /* plain_screen_on() */	

/************************************************************************/
/* plain_screen_off()							*/
/*									*/
/* Terminates plain_screen mode.					*/
/************************************************************************/

void plain_screen_off(void)
{
	/* If it is not in plain_screen mode exit */

	if (!plain_screen) return;

	/* If we had curses initialised refresh the screen */

	if (curses_open) 
	{
		cbreak();
		noecho();
		nonl();
#ifdef JUNK
		cursoff();
#endif
		ui_clear_screen();
		refresh_screen();
                wrefresh(curscr);
	}
	else
	{
		ui_clear_screen();
	}

	/* Out of plain_screen mode */

	plain_screen = FALSE;

} /* plain_screen_off */
