/************************************************************************/
/* clock_control_window.c                                               */
/*                                                                      */
/* Routine for controlling the basic CHARA clock functions.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 2002                                                  */
/************************************************************************/

#include "../../../charalib/chara.h"
#include "../../../cliserv/include/cliserv.h"
#include "../../../include/chara_messages.h"
#include "../include/cliserv_clock.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include "../../../slalib/slalib.h"

/* Globals */

static GtkWidget *clock_control_window;
static GtkWidget *chara_time_label;
static GtkWidget *sidereal_time_label;
static GtkWidget *linux_time_label;
static GtkWidget *lost_seconds_label;
static GtkWidget *lost_ticks_label;
static char clock_control_window_up = FALSE;
static guint clock_control_background_id;
static time_t last_current_time = -1;

#define SYNC_OFFSET	50 

/************************************************************************/
/* create_clock_control_window()					*/
/*									*/
/* ROutine for creating the clock control window. This should be self	*/
/* contained and give you all you need to control the basic clock 	*/
/* functionality. Argumentis the window name.				*/
/************************************************************************/

GtkWidget* create_clock_control_window (GtkButton *button, gpointer *data)
{
  GtkWidget *clock_control_box;
  GtkWidget *ok_button;
  GtkWidget *reinit_button;
  GtkWidget *clear_lost_counts_button;
  GtkWidget *set_time_label;
  GtkWidget *time_entry;
  GtkWidget *sync_button;
  GtkWidget *flush_button;
  struct control_window_data *the_data;

  /* Do nothing if we exist already */

  if (clock_control_window_up) return clock_control_window;

  /* Get the data */

  the_data = (struct control_window_data *)data;

  /* Create the window itself */

  clock_control_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (clock_control_window), 
		"clock_control_window", clock_control_window);
  gtk_window_set_title (GTK_WINDOW (clock_control_window), the_data->name);

  /* Box to put things in */

  clock_control_box = gtk_fixed_new ();
  gtk_widget_ref (clock_control_box);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"clock_control_box", clock_control_box,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (clock_control_box);
  gtk_container_add (GTK_CONTAINER (clock_control_window), clock_control_box);

  /* Linux time label */

  linux_time_label = gtk_label_new ("     LINUX TIME: 00:00:00");
  gtk_widget_ref (linux_time_label);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"linux_time_label", linux_time_label,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (linux_time_label);
  gtk_fixed_put (GTK_FIXED (clock_control_box), linux_time_label, 0, 0);
  gtk_widget_set_uposition (linux_time_label, 0, 0);
  gtk_widget_set_usize (linux_time_label, 160, 16);

  /* CHARA Time label */

  chara_time_label = gtk_label_new ("   CHARA TIME: 00:00:00");
  gtk_widget_ref (chara_time_label);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"chara_time_label", chara_time_label,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (chara_time_label);
  gtk_fixed_put (GTK_FIXED (clock_control_box), chara_time_label, 0, 16);
  gtk_widget_set_uposition (chara_time_label, 0, 16);
  gtk_widget_set_usize (chara_time_label, 160, 16);

  /* Siderial time label */

  sidereal_time_label = gtk_label_new ("SIDERIAL TIME: 00:00:00");
  gtk_widget_ref (sidereal_time_label);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"sidereal_time_label", sidereal_time_label,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (sidereal_time_label);
  gtk_fixed_put (GTK_FIXED (clock_control_box), sidereal_time_label, 0, 32);
  gtk_widget_set_uposition (sidereal_time_label, 0, 32);
  gtk_widget_set_usize (sidereal_time_label, 160, 16);

  /* Lost ticks label */

  lost_ticks_label = gtk_label_new ("    LOST TICKS:  000000");
  gtk_widget_ref (lost_ticks_label);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"lost_ticks_label", lost_ticks_label,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lost_ticks_label);
  gtk_fixed_put (GTK_FIXED (clock_control_box), lost_ticks_label, 0, 48);
  gtk_widget_set_uposition (lost_ticks_label, 0, 48);
  gtk_widget_set_usize (lost_ticks_label, 160, 16);

  /* Lost seconds label */

  lost_seconds_label = gtk_label_new ("    LOST SECS:  000000");
  gtk_widget_ref (lost_seconds_label);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"lost_seconds_label", lost_seconds_label,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lost_seconds_label);
  gtk_fixed_put (GTK_FIXED (clock_control_box), lost_seconds_label, 0, 64);
  gtk_widget_set_uposition (lost_seconds_label, 0, 64);
  gtk_widget_set_usize (lost_seconds_label, 160, 16);

  /* Clear lost ticks label */

  clear_lost_counts_button = gtk_button_new_with_label ("CLEAR");
  gtk_widget_ref (clear_lost_counts_button);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"clear_lost_counts_button", clear_lost_counts_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (clear_lost_counts_button), "clicked",
                      GTK_SIGNAL_FUNC (clock_control_clear_callback),
                      (gpointer) &(the_data->server));
  gtk_widget_show (clear_lost_counts_button);
  gtk_fixed_put (GTK_FIXED (clock_control_box),clear_lost_counts_button, 0, 80);
  gtk_widget_set_uposition (clear_lost_counts_button, 0, 80);
  gtk_widget_set_usize (clear_lost_counts_button, 80, 24);

  /* Sync button */

  sync_button = gtk_button_new_with_label ("SYNC");
  gtk_widget_ref (sync_button);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"sync_button", sync_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (sync_button), "clicked",
                      GTK_SIGNAL_FUNC (clock_control_sync_callback),
                      (gpointer) &(the_data->server));
  gtk_widget_show (sync_button);
  gtk_fixed_put (GTK_FIXED (clock_control_box), sync_button, 80, 80);
  gtk_widget_set_uposition (sync_button, 80, 80);
  gtk_widget_set_usize (sync_button, 80, 24);

  /* Set time stuff */

  set_time_label = gtk_label_new ("Set Time:");
  gtk_widget_ref (set_time_label);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"lost_seconds_label", set_time_label,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (set_time_label);
  gtk_fixed_put (GTK_FIXED (clock_control_box), set_time_label, 0, 104);
  gtk_widget_set_uposition (set_time_label, 0, 104);
  gtk_widget_set_usize (set_time_label, 80, 24);

  time_entry = gtk_entry_new ();
  gtk_widget_ref (time_entry);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"time_entry", time_entry,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (time_entry), "activate",
                            GTK_SIGNAL_FUNC (clock_control_time_callback),
                            (gpointer) &(the_data->server));
  gtk_widget_show (time_entry);
  gtk_fixed_put (GTK_FIXED (clock_control_box), time_entry, 80, 104);
  gtk_widget_set_uposition (time_entry, 80, 104);
  gtk_widget_set_usize (time_entry, 80, 24);
  gtk_entry_set_text (GTK_ENTRY (time_entry), "00:00:00");

  /* Flush button */

  flush_button = gtk_button_new_with_label ("FLUSH");
  gtk_widget_ref (flush_button);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"flush_button", flush_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (flush_button), "clicked",
                      GTK_SIGNAL_FUNC (clock_control_flush_callback),
                      (gpointer) &(the_data->server));
  gtk_widget_show (flush_button);
  gtk_fixed_put (GTK_FIXED (clock_control_box), flush_button, 0, 128);
  gtk_widget_set_uposition (flush_button, 0, 128);
  gtk_widget_set_usize (flush_button, 80, 24);

  /* OK button */

  reinit_button = gtk_button_new_with_label ("REINIT");
  gtk_widget_ref (reinit_button);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"reinit_button", reinit_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (reinit_button), "clicked",
                      GTK_SIGNAL_FUNC (clock_control_reinit_callback),
                      (gpointer) &(the_data->server));
  gtk_widget_show (reinit_button);
  gtk_fixed_put (GTK_FIXED (clock_control_box), reinit_button, 80, 128);
  gtk_widget_set_uposition (reinit_button, 80, 128);
  gtk_widget_set_usize (reinit_button, 80, 24);

  /* OK button */

  ok_button = gtk_button_new_with_label ("OK");
  gtk_widget_ref (ok_button);
  gtk_object_set_data_full (GTK_OBJECT (clock_control_window), 
	"ok_button", ok_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (ok_button), "clicked",
                      GTK_SIGNAL_FUNC (clock_control_ok_callback),
                      NULL);
  gtk_widget_show (ok_button);
  gtk_fixed_put (GTK_FIXED (clock_control_box), ok_button, 0, 152);
  gtk_widget_set_uposition (ok_button, 0, 152);
  gtk_widget_set_usize (ok_button, 160, 24);

  /* Now, we can show this window if we like */

  gtk_widget_show(clock_control_window);

  /* Make sure we only get one version of this window */

  clock_control_window_up = TRUE;

  /* Start routine for background processing */

  clock_control_background_id = 
	gtk_timeout_add(100,clock_control_background, 
		(gpointer *)&(the_data->server));

  /* That should be all */

  return clock_control_window;

} /* create_clock_control_window() */

/************************************************************************/
/* clock_control_ok_callback()						*/
/*									*/
/* Callback function for clock control OK button. Stop background	*/
/* processing and destroy clock control window.				*/
/************************************************************************/

void clock_control_ok_callback(GtkButton *button, gpointer user_data)
{
	if (!clock_control_window_up) return;

	gtk_widget_destroy(clock_control_window);
	gtk_timeout_remove(clock_control_background_id);
  	clock_control_window_up = FALSE;

} /* clock_control_ok_callback() */

/************************************************************************/
/* clock_control_background()						*/
/*									*/
/* Here's the background job for looking at clock outputs and so on.	*/
/************************************************************************/

gint clock_control_background(gpointer data)
{
        struct tm *now;
        time_t current_time;
	char	s[256];
        double  st;
        char    slst[81];
        char    c,*p;
        TIME    temp;
	ap_time_struct t;
	int	*server;

	server = (int *) data;

	/* 
	 * Do the time based on local clock 
	 * but only if things have changed at all
	 */

        time(&current_time);
	if (current_time != last_current_time)
	{
		/* Local time */

		last_current_time = current_time;
		now = localtime(&current_time);
	        sprintf(s,"     LINUX TIME: %02d:%02d:%02d",
			now->tm_hour,now->tm_min,now->tm_sec);
		gtk_label_set_text((GtkLabel *)linux_time_label,s);

		/* Siderial time */

		st = slaGmst(GetTime(&temp)) + CHARA_LONGITUDE;
		while(st < 0) st += 2.0*M_PI;
		time_to_string(st,slst);
		for(p=slst; *p!= 0; p++)
		{
			if (p != slst && *p == ' ') *p = ':';
			if (*p == '.') *p = 0;
		}
  		sprintf(s,"SIDERIAL TIME:%s",slst);
		gtk_label_set_text((GtkLabel *)sidereal_time_label,s);
	}

	/* 
	 * Now the real time clock. 
	 * Again, only do this if the time is valid
	 */

	if (rt_time[*server].time_now >= 0)
	{
		t = rt_to_ap_time(rt_time[*server].time_now);
		if (t.sign == -1)
			c = '-';
		else
			c = ' ';
		sprintf(s,"   CHARA TIME:%c%02d:%02d:%02d",
			c,t.hrs,t.min,t.sec);
		gtk_label_set_text((GtkLabel *)chara_time_label,s);

		sprintf(s,"    LOST TICKS:  %06d",rt_time[*server].lost_ticks);
		gtk_label_set_text((GtkLabel *)lost_ticks_label,s);

		sprintf(s,
			"     LOST SECS:  %06d",rt_time[*server].lost_seconds);
		gtk_label_set_text((GtkLabel *)lost_seconds_label,s);
	}
	
	/* Go and get time again */

	send_rt_time(*server);

	/* That is all */

	return TRUE;

} /* clock_control_background() */

/************************************************************************/
/* clock_control_clear_callback()					*/
/*									*/
/* Sends MESS_CLEAR_LOST_COUNTS message to server.			*/
/************************************************************************/

void clock_control_clear_callback(GtkButton *button, gpointer user_data)
{
	struct smessage mess;
	int	*server;

	server = (int *)user_data;
	mess.type = MESS_CLEAR_LOST_COUNTS;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
				"Failed to send MESS_CLEAR_LOST_COUNTS.");
		else
			print_status(ERROR,
				"Failed to send MESS_CLEAR_LOST_COUNTS.\n");
	}

} /* clock_control_clear_callback() */

/************************************************************************/
/* clock_control_reinit_callback()					*/
/*									*/
/* Sends REINIT_CLOCK message to server.				*/
/************************************************************************/

void clock_control_reinit_callback(GtkButton *button, gpointer user_data)
{
	struct smessage mess;
	int	*server;

	server = (int *)user_data;
	mess.type = REINIT_CLOCK;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Failed to send REINIT_CLOCK.");
		else
			print_status(ERROR,"Failed to send REINIT_CLOCK.\n");
	}

} /* clock_control_reinit_callback() */

/************************************************************************/
/* clock_control_flush_callback()					*/
/*									*/
/* Sends MESS_CLEAR_LOST_COUNTS message to server.			*/
/************************************************************************/

void clock_control_flush_callback(GtkButton *button, gpointer user_data)
{
	struct smessage mess;
	int	*server;

	server = (int *)user_data;
	mess.type = FLUSH_FIFOS;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Failed to send FLUSH_FIFOS.");
		else
			print_status(ERROR,"Failed to send FLUSH_FIFOS.\n");
	}

} /* clock_control_flush_callback() */

/************************************************************************/
/* clock_control_sync_callback()					*/
/*									*/
/* Sends MESS_CLEAR_LOST_COUNTS message to server.			*/
/************************************************************************/

void clock_control_sync_callback(GtkButton *button, gpointer user_data)
{
	struct smessage mess;
	ap_time_struct t;
	char	command[256];
	int	*server;
	time_t	start;

	server = (int *)user_data;

	/* Send message to sync server clock */

	mess.type = SYNC_TIMES;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Failed to send SYNC_TIMES.");
		else
			print_status(ERROR,"Failed to send SYNC_TIMES.\n");
	}

	/* 
	 * Now we need to do our clock here as well 
	 * and we start by ensuring we know what the time of the next second
	 * is on our clock.
	 */

	send_time_next_second(*server);
	start = time(0);
	while(time_next_second[*server] == -1) 
	{
		process_server_socket(*server);

		if (time(0) > start+3)
		{
#ifdef GTK2
			if (status_buffer == NULL)
#else
			if (status_text == NULL)
#endif
			    create_message_window(
				"Timed out waiting for time next second.");
			else
			    print_status(ERROR,
				"Timed out waiting for time next second.\n");
		    return;
		}
	}

	/* Now construct the command to set the linux clock */

	t = rt_to_ap_time(time_next_second[*server]);
	sprintf(command,"setgmt %d %d %d", t.sign * t.hrs, t.min, t.sec); 

	/* Now wait until we get there */

	start = time(0);
	do
	{
		send_rt_time(*server);
		while(rt_time[*server].time_now == -1) 
			process_server_socket(*server);
		if (time(0) > start+3)
		{
#ifdef GTK2
			if (status_buffer == NULL)
#else
			if (status_text == NULL)
#endif
				    create_message_window(
					"Timed out waiting for sync time.");
			else
				    print_status(ERROR,
					"Timed out waiting for sync time.\n");
		    return;
		}

	} while(rt_time[*server].time_now < 
			(time_next_second[*server] - SYNC_OFFSET));

	/* And send the command */

	system(command);

} /* clock_control_sync_callback() */

/************************************************************************/
/* clock_control_time_callback()						*/
/************************************************************************/

void clock_control_time_callback(GtkWidget *widget, gpointer user_data)
{
	char	*text;
        CHARA_TIME      time_next_second;
        ap_time_struct  new_time;
	int	h,m,s;
	struct smessage mess;
	int	*server;

	server = (int *)user_data;

	/* Get the text */

	text = gtk_entry_get_text(GTK_ENTRY(widget));

	/* Is this the right format ? */

	if (sscanf(text,"%d:%d:%d",&h,&m,&s) != 3)
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Time format must be HH:MM:SS");
		else
			print_status(ERROR,"Time format must be HH:MM:SS\n");
		return;
	}


	/* Convert it to a CHARA_TIME */

	new_time = str_to_ap_time(text);
        time_next_second = ap_to_rt_time(new_time);

	/* Send the message */

	mess.type = SET_RT_TIME;
	mess.length = sizeof(time_next_second);
	mess.data = (unsigned char *)&time_next_second;
	if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Failed to send SET_RT_TIME.");
		else
			print_status(ERROR,"Failed to send SET_RT_TIME.\n");
	}

} /* clock_control_time_callback() */
