/************************************************************************/
/* clock_time_convert.c							*/
/*                                                                      */
/* CLient  routines routines for the CHARA real time clock.		*/
/* Will be part of a general library for use by other servers.		*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 2002                                                  */
/************************************************************************/

#include "../../../cliserv/include/cliserv.h"
#include "../include/cliserv_clock.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>

/************************************************************************/
/* rt_to_ap_time()							*/
/*									*/
/* Converts RT process time into an application time structure.		*/
/************************************************************************/

struct AP_TIME_STRUCT rt_to_ap_time(CHARA_TIME rt_time)
{
	ap_time_struct res;

	if (rt_time < 0)
	{
		rt_time *= (res.sign = -1);
	}
	else
	{
		res.sign = 1;
	}
		
	res.tick = rt_time % TICKS_PER_SECOND;
	rt_time -= res.tick;
	rt_time /= TICKS_PER_SECOND;

	res.sec = rt_time % 60;
	rt_time -= res.sec;
	rt_time /= 60;

	res.min = rt_time % 60;
	rt_time -= res.min;

	res.hrs = rt_time / 60;

	return res;

} /* rt_to_ap_time() */

/************************************************************************/
/* ap_to_rt_time()							*/
/*									*/
/* Converts an application time structure into a RT process time .	*/
/************************************************************************/

CHARA_TIME ap_to_rt_time(struct AP_TIME_STRUCT t)
{
	CHARA_TIME	res;

	res = t.tick + TICKS_PER_SECOND * (t.sec + 60 * (t.min + 60 * t.hrs));
	res *= t.sign;

	return res;

} /* ap_to_rt_time() */

/************************************************************************/
/* str_to_ap_time()							*/
/*									*/
/* Converts a string of the form [+-]HR:MN:SC into an application time	*/
/* structure.								*/
/************************************************************************/

struct AP_TIME_STRUCT str_to_ap_time(char *s)
{
	struct AP_TIME_STRUCT t;

	/* What is the sign? */

	while(*s == ' ' || *s == '\t') s++;
	if (*s == '-')
	{
		/* It's negative */

		t.sign = -1;
		*s = ' ';
	}
	else
	{
		/* It's positive */

		t.sign = 1;
	}

	switch(sscanf(s,"%d:%d:%d",&t.hrs,&t.min,&t.sec))
	{

		case 0: t.hrs = 0;

		case 1: t.min = 0;

		case 2: t.sec = 0;

		default: t.tick = 0;
			 break;
	}

	return t;
		
} /* str_to_ap_time() */

/************************************************************************/
/* ap_to_str_time()							*/
/*									*/
/* Converts a string of the form [+-]HR:MN:SC into an application time	*/
/* structure.								*/
/************************************************************************/

char *ap_to_str_time(char *s, struct AP_TIME_STRUCT t)
{
	/* What is the sign? */

	if (t.sign == -1)
	{
		/* It's negative */

		strcpy(s,"-");
	}
	else
	{
		/* It's positive */

		strcpy(s," ");
	}

	/* Round it */

	if (t.tick > TICKS_PER_SECOND/2 || t.tick < -TICKS_PER_SECOND/2)
	{
		t.sec++;
	}

	/* Print it */

	sprintf(s+1,"%02d:%02d:%02d",t.hrs,t.min,t.sec);

	return s;
		
} /* ap_to_str_time() */
