/************************************************************************/
/* clock_comms.c							*/
/*                                                                      */
/* Server  routines routines for the CHARA real time clock.		*/
/* Will be part of a general library for use by other servers.		*/
/* Routines that talk to real time side of things.			*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 2002                                                  */
/************************************************************************/

/* This is by definition part of a server */

#define SERVER

#include "../../../cliserv/include/cliserv.h"
#include "../include/cliserv_clock.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#ifndef USE_REAL_TIME_VERSION
#include "../../../../modules/cbdriver/cbdriver.h"
#endif

/*
 * Offset for setting the time in mS
 */

#define SYNC_OFFSET 10

/* Globals */

#ifdef USE_REAL_TIME_VERSION
int      fd_comm;                /* Command channel file handle */
int      fd_data_out;            /* Data out channel file handle */
int      fd_data_in;             /* Data in channel file handle */
int      fd_err;                 /* Error channel file handle */
#else
FILE	 *cbfp;			/* File pointer for clock board driver */
#endif

#ifdef USE_REAL_TIME_VERSION
/************************************************************************/
/* open_rt_fifos()							*/
/* 									*/
/* Opens all FIFOs for comms with RT module. Returns TRUE if all OK,	*/
/* FALSE otherwise.							*/
/* Sends no text to stderr.						*/
/************************************************************************/

char open_rt_fifos(void)
{
	if ((fd_comm = open(COMM_FIFO_DEV, O_WRONLY)) < 0) 
	{
		return FALSE;
	}

	if ((fd_data_out = open(DATA_OUT_FIFO_DEV, O_RDONLY)) < 0) 
	{
		return FALSE;
	}

	if ((fd_data_in = open(DATA_IN_FIFO_DEV, O_WRONLY)) < 0) 
	{
		return FALSE;
	}

	if ((fd_err = open(ERR_FIFO_DEV, O_RDONLY)) < 0) 
	{
		return FALSE;
	}

	return TRUE;

} /* open_rt_fifos() */

/************************************************************************/
/* close_rt_fifos()							*/
/* 									*/
/* CLoses all FIFOs for comms with RT module.				*/
/************************************************************************/

void close_rt_fifos(void)
{
	close(fd_comm);
	close(fd_data_out);
	close(fd_data_in);
	close(fd_err);

} /* close_rt_fifos() */

/************************************************************************/
/* send_rt_command()							*/
/*									*/
/* Attempts to send a command into the command FIFO. Returns TRUE if	*/
/* it went well and FALSE otherwise.					*/
/* Sends no text to stderr.						*/
/************************************************************************/

char send_rt_command(unsigned char command, unsigned char argument)
{
	rt_command_struct c;

	c.comm = command;
	c.data = argument;

	if (write(fd_comm,&c,sizeof(c)) != sizeof(c))
	{
	    error(WARNING,
	    "Failed to write command %d:%d : %s\nData fifo could be corrupt.",
				command,argument,strerror(errno));
	    return FALSE;
	}

	return TRUE;

} /* send_rt_command() */

/************************************************************************/
/* get_rt_error_message()						*/
/*									*/
/* Polls the ERR_FIFO and see's if there is anything in there to get.	*/
/* If there is nothing there, it returns 0. If there is something	*/
/* there is grabs it up till the first NULL it sees and places		*/
/* it into the string s (assumed large enough...) and returns the	*/
/* length of the string.						*/
/************************************************************************/

int get_rt_error_message(char *s)
{
	char	*ps;
	fd_set	err_fds;	/* fdset for select() function */
        struct timeval tv;      /* Time to wait */

	/* Set up for the select function call */

	FD_ZERO(&err_fds);
	FD_SET(fd_err, &err_fds);
	tv.tv_sec = 0;           /* Wait no more than 1 usec */
        tv.tv_usec = 1;

	select(FD_SETSIZE, &err_fds, NULL, NULL, &tv);			

	/* OK, let's get the string */

	if (FD_ISSET(fd_err, &err_fds)) 
	{
		/* Get the message */

		for (ps = s; read(fd_err , ps, 1) == 1 && *ps != 0; ps++);
		return strlen(s);
	}
	else
	{
		return 0;
	}

} /* get_rt_error_message() */

/************************************************************************/
/* read_rt_data()							*/
/*									*/
/* Polls the DATA_OUT_FIFO and see's if there is anything in there to 	*/
/* get.									*/
/* If there is nothing there, it returns 0. If there is something	*/
/* there it tries to read size bytes into the buffer.			*/
/* It's a lot like read() but it doesn't hang around too long.		*/
/************************************************************************/

int read_rt_data(void *buf, size_t count)
{
	fd_set	data_fds;	/* fdset for select() function */
        struct timeval tv;      /* Time to wait */

	/* Set up for the select function call */

	FD_ZERO(&data_fds);
	FD_SET(fd_data_out, &data_fds);
	tv.tv_sec = 0;           /* Wait no more than 10 usec */
        tv.tv_usec = 10;

	select(FD_SETSIZE, &data_fds, NULL, NULL, &tv);			

	/* OK, let's get the string */

	if (FD_ISSET(fd_data_out, &data_fds)) 
	{
		return read(fd_data_out , buf, count);
	}
	else
	{
		return 0;
	}

} /* read_rt_data() */

/************************************************************************/
/* write_rt_data()							*/
/*									*/
/* Sends data to the DATA IN FIFO, ie to the RT process.		*/
/* Some user command must be used to make the RT process do something   */
/* with it at the other end. Returns number of bytes actually sent.	*/
/* You need to be carefull not to send too much at a time as the FIFO	*/
/* buffers are not large.						*/
/* It's a lot like write() but it doesn't hang around too long.		*/
/************************************************************************/

int write_rt_data(void *buf, size_t count)
{
	
	return write(fd_data_in,buf,count);

} /* write_rt_data() */

/************************************************************************/
/* flush_rt_data_in()							*/
/*									*/
/* Calls the system RT function FLUSH_DATA, which tries to empty	*/
/* the DATA_IN_FIFO of the RT process. 					*/
/* Returns the number of btes actually flushed.				*/
/* Note that since this RT system call puts the number of bytes it	*/
/* flushed, you should almost always call flush_rt_data_out() before	*/
/* using this function.							*/
/************************************************************************/

int	flush_rt_data_in(void)
{
	CHARA_TIME    count;

	/* OK try and flush the DATA_IN buffer */

        if (!send_rt_command(FLUSH_DATA,0))
        {
                return 0;
        }

	read_rt_data(&count,sizeof(count));

	return (int)count;

} /* flush_rt_data_in() */

/************************************************************************/
/* flush_rt_data_out()							*/
/*									*/
/* Tries to read any existing 						*/
/* in the DATA_OUT_FIFO until it is empty. This function sometimes	*/
/* helpful to clear up locked data transfers.				*/
/* Returns the number of bytes actually flushed.			*/
/* Usually used with flush_rt_data_in().				*/
/************************************************************************/

int	flush_rt_data_out(void)
{
	unsigned char c;
	int    count;

	/* Empty the incoming buffer stream */

	count = 0;
	while(read_rt_data(&c, sizeof(c)) != 0) count++;

	return count;

} /* flush_rt_data_out() */

/************************************************************************/
/* flush_data()								*/
/*									*/
/* Ensures all FIFOs are empty.                                         */
/* Note that this removes everything from the ERROR stream also so if   */
/* there are any errors there they will be flushed too. Only looks for  */
/* up to 1000 errors in case something terrible is wrong.               */
/* Returns error level.							*/
/************************************************************************/

int flush_data(void)
{
        int count_in,count_out,count_error;
        char s[256];

        count_out = flush_rt_data_out();
        count_in = flush_rt_data_in();
        for(count_error=0; count_error<1000; count_error++)
        {
                if (get_rt_error_message(s) == 0) break;
        }
        if (count_error >= 1000) 
                error(WARNING,"More than 1000 errors flushed!");

        error(MESSAGE,
        "Bytes Flushed: DATA_IN_FIFO = %d DATA_OUT_FIFO = %d ERROR_FIFO = %d",
                        count_in,count_out,count_error);

	server_print_message_all(
	"FIFOS FLUSHED\nDATA_IN_FIFO = %d\nDATA_OUT_FIFO = %d\nERROR_FIFO = %d",
		count_in,count_out,count_error);

        return NOERROR;

} /* flush_data() */

/************************************************************************/
/* in_set()								*/
/*									*/
/* Uses RT system command TTL_CONTROL in order to poll the value	*/
/* of the input TTL pins. Arguments can be 1, 2 or 3, depending on	*/
/* which TTL input you want to check. Returns TRUE if TTL input is	*/
/* set and FALSE if it is not. Returns -1 on error.			*/
/************************************************************************/

int	in_set(int in_or_out, int port)
{
	char	result;

	/* Send the right command */

	if (in_or_out == TTL_IN)
	{
		switch(port)
		{
		case 1: result = send_rt_command(TTL_CONTROL,IN_1);
			break;

		case 2: result = send_rt_command(TTL_CONTROL,IN_2);
			break;

		case 3: result = send_rt_command(TTL_CONTROL,IN_3);
			break;
	
		default: return -1;
		}
	}
	else
	{
		switch(port)
		{
		case 1: result = send_rt_command(TTL_CONTROL,OUT_1);
			break;

		case 2: result = send_rt_command(TTL_CONTROL,OUT_2);
			break;

		default: return -1;
		}
	}

	if (!result) 
	{
		return -2;
	}

	/* Get the result */

	if (read_rt_data(&result,sizeof(result)) != sizeof(result)) 
	{
		return -3;
	}

	/* That is all */

	if (result)
		return TRUE;
	else
		return FALSE;

} /* in_set() */

/************************************************************************/
/* error_status() 							*/
/*									*/
/* Looks for an error in the ERR_FIFO, if there is one report it to	*/
/* the console in an ERROR window.					*/
/************************************************************************/

int error_status(void)
{
	char	message[256];

	if (get_rt_error_message(message) == 0) return NOERROR;

	error(WARNING,"Message from RT process:\n%s",message);

	return NOERROR;
}

/************************************************************************/
/* get_lpt_register()							*/
/*									*/
/* Uses one of the standard commands to get a register from the		*/
/* printer port.							*/
/************************************************************************/

unsigned char get_lpt_register(int reg)
{
	unsigned char c;

	/* Check the argument makes sense */

	if (reg != CONTROL_REG && reg != STATUS_REG)
	{
		error(WARNING,"get_lpt_register(): Bad register.");
		return 0;
	}

	/* Try sending the command */

        if (!send_rt_command(REPORT_LPT_REGISTER,reg))
        {
                return 0;
        }

        /* Now get the register value */

        if (read_rt_data(&c, sizeof(c)) != sizeof(c))
        {
                return error(WARNING,"Error reading register. No data\n");
        }

	return c;

} /* get_lpt_register() */

/************************************************************************/
/* call_sync_handler()                                                  */
/*                                                                      */
/* Sends command to real time side to forcibly call the sync handler.   */
/* Mostly used when the prio_sched get's confused.                      */
/************************************************************************/

int call_sync_handler(void)
{
        if (!send_rt_command(SYNC_HANDLER,0))
        {
                return error(WARNING,"Couldn't send the command.");
        }

        return NOERROR;

} /* call_sync_handler() */

/************************************************************************/
/* set_lpt()								*/
/*									*/
/* Alows user to set the printer port output to a specific value.	*/
/* Returns actual value.						*/
/************************************************************************/

int set_lpt(unsigned char value)
{

	/* Send the command */
	
        if (!send_rt_command(SET_LPT,value))
        {
                return error(WARNING,"Couldn't send the command.");
        }

        return get_lpt();

} /* set_lpt() */

/************************************************************************/
/* get_lpt()                                                            */
/*                                                                      */
/* Returns the current setting of printer port, or -1 on command        */
/* error, -2 on data error.                                             */
/************************************************************************/

int get_lpt(void)
{
        unsigned char value;

        if (!send_rt_command(GET_LPT,value)) return -1;
        if (read_rt_data(&value, sizeof(value)) != sizeof(value)) return -2;
        return (int)value;

} /* get_lpt() */

/************************************************************************/
/* set_bit_lpt()							*/
/*									*/
/* Alows user to set a bit in the printer port output.			*/
/* Returns actaul value of printer port.				*/
/************************************************************************/

int set_bit_lpt(unsigned char bit)
{
	/* Send the command */

        if (!send_rt_command(SET_BIT_LPT,bit))
        {
                return error(WARNING,"Couldn't send the command.");
        }

        return get_lpt();

} /* set_bit_lpt() */

/************************************************************************/
/* unset_bit_lpt()							*/
/*									*/
/* Alows user to unset a bit in the printer port output.		*/
/* Returns actaul value of printer port.				*/
/************************************************************************/

int unset_bit_lpt(unsigned char bit)
{
        if (!send_rt_command(UNSET_BIT_LPT,bit))
        {
                return error(WARNING,"Couldn't send the command.");
        }

        return get_lpt();

} /* unset_bit_lpt() */

/************************************************************************/
/* write_lpt_control()							*/
/*									*/
/* Sends a byte to the LPT register.					*/
/* Returns actual value in register 					*/
/************************************************************************/

unsigned char write_lpt_control(unsigned char reg)
{
	if (!send_rt_command(WRITE_LPT_CONTROL,(unsigned char)reg))
        {
                error(WARNING,"Couldn't send command WRITE_LPT_CONTROL.");
		return 0;
        }

        return get_lpt_register(CONTROL_REG);

} /* write_lpt_control() */

/************************************************************************/
/* initialize_clock()                                                   */
/*                                                                      */
/* ALlows user to force a clock reinitialization.                       */
/************************************************************************/

int initialize_clock(void)
{
        if (!send_rt_command(INITIALIZE_CLOCK,0))
        {
                return error(WARNING,"Couldn't send the command.");
        }

        error(MESSAGE,"Clock has been initialized.");

        return NOERROR;

} /* initialize_clock() */
#else

/************************************************************************/
/* open_clock_device()                                                  */
/*                                                                      */
/* OPens clock device. Returns TRUE if all went well, FALSE otherwise.  */
/************************************************************************/

int open_clock_device(void)
{
        if ((cbfp = fopen(DEVICE_NAME, "rb")) == NULL) 
        {
                return FALSE;
        }

        return TRUE;

} /* open_clock_device() */

/************************************************************************/
/* close_clock_device()                                                 */
/*                                                                      */
/* CLoses clock device.                                                 */
/************************************************************************/

void close_clock_device(void)
{
        if (cbfp == NULL) return;

        fclose(cbfp);

        cbfp = NULL;

} /* close_clock_device() */

#endif

/************************************************************************/
/* clear_lost_counts()						        */
/*									*/
/* Uses RT system command CLEAR_LOST_COUNTS to set lost_ticks=0 and	*/
/* lost_seconds = 0. Normally only used after you have set the clock.	*/
/************************************************************************/

int clear_lost_counts(void)
{

#ifdef USE_REAL_TIME_VERSION
        if (!send_rt_command(CLEAR_LOST_COUNTS,0))
        {
                return error(ERROR,"Failed to clear counters.");
        }
#else
	if (cbfp == NULL) return error(ERROR,"Clock device is not open.");

        if (ioctl(fileno(cbfp), CBDRIVER_CLEAR) != 0)
        {
                return error(ERROR,"Failed to clear lost counts: %s",
                        strerror(errno));
        }
#endif

	return NOERROR;

} /* clear_lost_counts() */

/************************************************************************/
/* get_rt_time()							*/
/*									*/
/* Uses one of the standard commands to get a structure from the	*/
/* RT process containing the time, lost ticks and lost seconds.		*/
/* Returns 0 on succes and -1 on failure.  				*/
/************************************************************************/

int get_rt_time(rt_time_struct *the_time)
{
#ifdef USE_REAL_TIME_VERSION
	/* Try sending the command */

        if (!send_rt_command(REPORT_TIME,0))
        {
                return -1;
        }

        /* Now get the register value */

        if (read_rt_data(the_time, sizeof(*the_time)) != sizeof(*the_time))
        {
                error(WARNING,"Error reading time. Not enough data\n");
		return -1;
        }
#else
        if (cbfp == NULL) return error(ERROR,"Clock device is not open.");

        if (fread(the_time, sizeof(rt_time_struct), 1, cbfp) == 0) return -1;
#endif

	return 0;

} /* get_rt_time() */

/************************************************************************/
/* chara_time_now()                                                     */
/*                                                                      */
/* Replacement for the old mbuff shared memory global pointer. Returns  */
/* time now or 0 on ERROR.                                              */
/************************************************************************/

CHARA_TIME chara_time_now(void)
{
        rt_time_struct the_time;

        if (get_rt_time(&the_time) != 0) return 0;

        return the_time.time_now;

} /* chara_time_now() */

/************************************************************************/
/* get_time_next_second()						*/
/*									*/
/* Uses one of the standard commands to get the	time of the next 	*/
/* second. Returns this time or -1 on error.				*/
/* If wait is TRUE will wait for it to change once first before 	*/
/* it returns to make sure it will be valid for a while. Times out      */
/* after 3 seconds.							*/
/************************************************************************/

CHARA_TIME get_time_next_second(int wait)
{
#ifndef USE_REAL_TIME_VERSION
	rt_time_struct  the_time;
#endif
	CHARA_TIME	time_next_second;
	CHARA_TIME	last_time_next_second;
	time_t		stop_time;
	time_t		now;

#ifdef USE_REAL_TIME_VERSION
	/* Try sending the command */

        if (!send_rt_command(GET_TIME_NEXT_SECOND,0)) return -1;

        if (read_rt_data(&last_time_next_second, sizeof(CHARA_TIME)) != 
		sizeof(CHARA_TIME)) return -1;

	if (!wait) return last_time_next_second;

	/* Now try for a few seconds to get the next one */

	time(&stop_time);
	stop_time += 3;
	do
	{
		if (!send_rt_command(GET_TIME_NEXT_SECOND,0)) return -1;

		if (read_rt_data(&time_next_second, sizeof(CHARA_TIME)) != 
			sizeof(CHARA_TIME)) return -1;

		time(&now);

	} while(now < stop_time && last_time_next_second==time_next_second);

	if (now >= stop_time) return -1;

#else

        if (cbfp == NULL) return error(ERROR,"Clock device is not open.");

        /* Work out the time of the next second */

        if (get_rt_time(&the_time) != 0) return -1;
        last_time_next_second = TICKS_PER_SECOND *
                                (the_time.time_now/TICKS_PER_SECOND + 1);

        /* Return now if that's what teh user wanted */

        if (!wait) return last_time_next_second;

        /* Now try for a few seconds to get the next one */

        time(&stop_time);
        stop_time += 3;
        do
        {
                if (get_rt_time(&the_time) != 0) return -1;
                time_next_second = TICKS_PER_SECOND *
                                (the_time.time_now/TICKS_PER_SECOND + 1);
                time(&now);

        } while(now < stop_time && last_time_next_second==time_next_second);

        if (now >= stop_time) return -1;

#endif

	return time_next_second;

} /* get_time_next_second() */

/************************************************************************/
/* set_rt_time()							*/
/*									*/
/* Uses one of the standard commands to send a value of type CHARA_TIME */
/* to the RT process. This value wil become the value of the RT 	*/
/* modules internal varriable time_next_second. When the next second	*/
/* tick occurs, or is detected to have been missed, the time is set to 	*/
/* this value. Warning.... race conditions can occurr, this should be 	*/
/* well before the next second tick is expected.			*/
/* Returns TRUE is all went OK, FALSE otherwise.			*/
/************************************************************************/

int set_rt_time(CHARA_TIME time_next_second)
{
#ifdef USE_REAL_TIME_VERSION
        /* 
	 * Send the time value to DATA IN FIFO 
	 * Note that this must be done first so the data
	 * is there when the command to use it is sent.
	 */

	if (write_rt_data(&time_next_second,sizeof(time_next_second)) != 
		sizeof(time_next_second))
	{
		return FALSE;
	}

	/* Now send the command */

        if (!send_rt_command(SET_TIME,0))
        {
                return FALSE;
        }

#else

        if (cbfp == NULL) return error(ERROR,"Clock device is not open.");

        if (ioctl(fileno(cbfp), CBDRIVER_SET_TIME, 
                (unsigned long int)time_next_second) != 0)
        {
                return FALSE;
        }

#endif

	return TRUE;

} /* set_rt_time() */

/************************************************************************/
/* sync_times()								*/
/*									*/
/* Attempts to set the time using the RT time based on RT clock.	*/
/* Uses the setgmt program. Note that this can mess up the local clock! */
/************************************************************************/

int sync_times(void)
{
	CHARA_TIME time_next_second;
	struct AP_TIME_STRUCT ap_time_next_second;
	char	command[81];
	rt_time_struct the_time;

	/* Find out the time of the next second */

	if ((time_next_second = get_time_next_second(TRUE)) == -1)
	{
		return error(ERROR,"Failed to get time of next second.");
	}

	/* Setup the command we will need to set the Linux Clock */

	ap_time_next_second = rt_to_ap_time(time_next_second);
	sprintf(command,"setgmt %d %d %d",
		ap_time_next_second.sign * ap_time_next_second.hrs,
		ap_time_next_second.min, ap_time_next_second.sec);

	/* Wait until the time comes ... with OFFSET */

	do
	{
		if (get_rt_time(&the_time) == -1)
		{
			return error(ERROR,"Failed to get RT time.");
		}
	} while(the_time.time_now < (time_next_second -  SYNC_OFFSET));

	system(command);
			
	return NOERROR;

} /* sync_times() */
