/************************************************************************/
/* clock_messages.c                                                     */
/*                                                                      */
/* Routines to respond to messages.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                  	*/
/* Date   : April 2002                                                  */
/************************************************************************/

#define SERVER

#include "../../../cliserv/include/cliserv.h"
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "../include/cliserv_clock.h"
#include "../../../include/chara_messages.h"

/************************************************************************/
/* setup_standard_clock_messages()					*/
/*									*/
/* Gues what... this sets up the standard clock messages!.		*/
/************************************************************************/

void setup_standard_clock_messages(void)
{
#ifdef USE_REAL_TIME_VERSION
	server_add_message_job(FLUSH_FIFOS, clock_flush_fifos);
	server_add_message_job(IN_SET, clock_in_set);
	server_add_message_job(CALL_SYNC_HANDLER, clock_call_sync_handler);
	server_add_message_job(MESS_SET_LPT, clock_set_lpt);
	server_add_message_job(MESS_GET_LPT, clock_get_lpt);
	server_add_message_job(SET_LPT_BIT, clock_set_lpt_bit);
	server_add_message_job(UNSET_LPT_BIT, clock_unset_lpt_bit);
	server_add_message_job(REINIT_CLOCK, clock_reinit_clock);
#endif
	server_add_message_job(MESS_CLEAR_LOST_COUNTS, clock_clear_lost_counts);
	server_add_message_job(RT_TIME, clock_rt_time);
	server_add_message_job(TIME_NEXT_SECOND, clock_time_next_second);
	server_add_message_job(SET_RT_TIME, clock_set_rt_time);
	server_add_message_job(SYNC_TIMES, clock_sync_times);

} /* setup_standard_clock_messages() */

#ifdef USE_REAL_TIME_VERSION
/************************************************************************/
/* clock_flush_fifos()							*/
/*									*/
/* Called by message FLUSH_DATA.					*/
/* No data to this message.						*/
/************************************************************************/

int clock_flush_fifos(struct smessage *message)
{

	/* Do we have the right number of arguments? */

	if (message->length != 0)
		error(ERROR,"Got FLUSH_DATA message with data. Data ignored.");

	return flush_data();

} /* clock_flush_fifos() */

/************************************************************************/
/* clock_in_set()							*/
/*									*/
/* Called by message IN_SET, no data fro this message.			*/
/************************************************************************/

int clock_in_set(struct smessage *message)
{
	struct smessage mess;
	unsigned char *args;
	int	in_or_out, port;
	unsigned char	result;

	/* Check data */

	if (message->length != 2*sizeof(unsigned char))
	{
		return error(ERROR,
			"IN_SET message contains wrong number of bytes.");
	}

	/* Get the data out */

	args = message->data;
	in_or_out = args[0];
	port = args[1];

	/* Go quiry the hardware */

	result = in_set(in_or_out, port);

	/* Send the message back the other way */

	mess.type = IN_SET;
	mess.length = sizeof(result);
	mess.data = (unsigned char *)&result;

	if (server_send_message(active_socket, &mess) != NOERROR)
		return error(ERROR,"Failed ot send IN_SET message.");

	/* That is all */

	return NOERROR;

} /* clock_in_set() */

/************************************************************************/
/* clock_call_sync_handler()						*/
/*									*/
/* On message CALL_SYNC_HANDLER  will call sync_times() fuinction.	*/
/************************************************************************/

int clock_call_sync_handler(struct smessage *message)
{
	if (message->length != 0)
		error(ERROR,
		"Got CALL_SYNC_HANDLER message with data. Data ignored.");

	error(MESSAGE,"Callng sync handler.");
	return call_sync_handler();

} /* clock_call_sync_handler() */

/************************************************************************/
/* clock_set_lpt()							*/
/*									*/
/* On message MESS_SET_LPT  will call set_lpt() fuinction.		*/
/************************************************************************/

int clock_set_lpt(struct smessage *message)
{
	unsigned char 	*value, new_value;
	struct smessage mess;

	if (message->length != sizeof(*value))
		return error(ERROR,
		"Got MESS_SET_LPT message with wrong data.");

	value = message->data;

	new_value = set_lpt(*value);
	mess.type = MESS_SET_LPT;
	mess.length = sizeof(new_value);
	mess.data = (unsigned char *)&new_value;

	if (server_send_message(active_socket, &mess) != NOERROR)
		return error(ERROR,"Failed to send MESS_SET_LPT message.");

	error(MESSAGE,"LPT set to 0x%02x.",*value);
	return NOERROR;

} /* clock_set_lpt() */

/************************************************************************/
/* clock_get_lpt()							*/
/*									*/
/* On message MESS_GET_LPT  will call set_lpt() fuinction.		*/
/************************************************************************/

int clock_get_lpt(struct smessage *message)
{
	struct smessage mess;
	int	lpt_value;
	unsigned char value;

	/* There should be no data */

	if (message->length != 0)
		error(ERROR,
		"Got MESS_GET_LPT message with data. Data ignored.");

	/* Find out what the port has in it */

	if ((lpt_value = get_lpt()) < 0)
		error(ERROR,"Failed to get LPT value.");
	value = lpt_value;

	/* Send back the result */

	mess.type = MESS_SET_LPT;
	mess.length = sizeof(value);
	mess.data = (unsigned char *)&value;

	if (server_send_message(active_socket, &mess) != NOERROR)
		return error(ERROR,"Failed ot send MESS_SET_LPT message.");

	/* That is all */

	return NOERROR;

} /* clock_get_lpt() */

/************************************************************************/
/* clock_set_lpt_bit()							*/
/*									*/
/* On message SET_LPT_BIT  will call set_lpt_bit() fuinction.		*/
/************************************************************************/

int clock_set_lpt_bit(struct smessage *message)
{
	unsigned char *bit, new_value;
	struct smessage mess;

	if (message->length != sizeof(*bit))
		return error(ERROR,
		"Got SET_LPT_BIT message with wrong data.");

	bit = message->data;

	new_value = set_bit_lpt(*bit);

	mess.type = MESS_SET_LPT;
	mess.length = sizeof(new_value);
	mess.data = (unsigned char *)&new_value;

	if (server_send_message(active_socket, &mess) != NOERROR)
		return error(ERROR,"Failed to send MESS_SET_LPT message.");

	error(MESSAGE,"SET LPT bit %d.",*bit);

	return NOERROR;

} /* clock_set_lpt_bit() */

/************************************************************************/
/* clock_unset_lpt_bit()						*/
/*									*/
/* On message UNSET_LPT_BIT  will call unset_lpt_bit() fuinction.	*/
/************************************************************************/

int clock_unset_lpt_bit(struct smessage *message)
{
	unsigned char *bit, new_value;
	struct smessage mess;

	if (message->length != sizeof(*bit))
		return error(ERROR,
		"Got SET_LPT_BIT message with wrong data.");

	bit = message->data;

	new_value = unset_bit_lpt(*bit);

	mess.type = MESS_SET_LPT;
	mess.length = sizeof(new_value);
	mess.data = (unsigned char *)&new_value;

	if (server_send_message(active_socket, &mess) != NOERROR)
		return error(ERROR,"Failed to send MESS_SET_LPT message.");

	error(MESSAGE,"UNSET LPT bit %d.",*bit);

	return NOERROR;

} /* clock_unset_lpt_bit() */

/************************************************************************/
/* clock_reinit_clock()                                                 */
/*                                                                      */
/* On message REINIT_CLOCK  will call initialize_clock() fuinction.     */
/************************************************************************/

int clock_reinit_clock(struct smessage *message)
{
        if (message->length != 0)
                return error(ERROR,
                "Got REINIT_CLOCK message with wrong data.");

        return initialize_clock();

} /* clock_reinit_clock() */

#endif

/************************************************************************/
/* clock_clear_lost_counts()						*/
/*									*/
/* Called by message MESS_CLEAR_LOST_COUNTS, no data fro this message.	*/
/************************************************************************/

int clock_clear_lost_counts(struct smessage *message)
{

	/* Do we have the right number of arguments? */

	if (message->length != 0)
		error(ERROR,
		"Got MESS_CLEAR_LOST_COUNTS message with data. Data ignored.");

	return clear_lost_counts();

} /* clock_clear_lost_counts() */

/************************************************************************/
/* clock_rt_time()							*/
/*									*/
/* Called by message RT_TIME, no data fro this message.			*/
/************************************************************************/

int clock_rt_time(struct smessage *message)
{
	struct smessage mess;
	rt_time_struct  the_time;

	/* There should be no data */

	if (message->length != 0)
		error(ERROR,
		"Got RT_TIME message with data. Data ignored.");

	/* Find out what time it is */

	if (get_rt_time(&the_time) != 0)
	{
		error(ERROR,"Failed to get time for RT side.");
	}

	/* Send back the result */

	mess.type = RT_TIME;
	mess.length = sizeof(the_time);
	mess.data = (unsigned char *)&the_time;

	if (server_send_message(active_socket, &mess) != NOERROR)
		return error(ERROR,"Failed ot send RT_TIME message.");

	/* That is all */

	return NOERROR;

} /* clock_rt_time() */

/************************************************************************/
/* clock_time_next_second()						*/
/*									*/
/* Called by message RT_TIME_NEXT_SECOND, no data for this message.	*/
/************************************************************************/

int clock_time_next_second(struct smessage *message)
{
	struct smessage mess;
	CHARA_TIME  time_next_second;

	/* There should be no data */

	if (message->length != 0)
		error(ERROR,
		"Got RT_TIME_NEXT_SECOND message with data. Data ignored.");

	/* Find out what time it is */

	if ((time_next_second = get_time_next_second(FALSE)) < 0)
	{
		error(ERROR,"Failed to get time of next second.");
	}

	/* Send back the result */

	mess.type = TIME_NEXT_SECOND;
	mess.length = sizeof(time_next_second);
	mess.data = (unsigned char *)&time_next_second;

	if (server_send_message(active_socket, &mess) != NOERROR)
		return error(ERROR,"Failed ot send TIME_NEXT_SECOND message.");

	error(MESSAGE,"Time next second = %ld",time_next_second);

	/* That is all */

	return NOERROR;

} /* clock_time_next_second() */

/************************************************************************/
/* clock_set_rt_time()							*/
/*									*/
/* Called by message SET_RT_TIME.					*/
/************************************************************************/

int clock_set_rt_time(struct smessage *message)
{
	CHARA_TIME *time_next_second;

	/* Check data */

	if (message->length != sizeof(CHARA_TIME))
	{
		return error(ERROR,
			"SET_RT_TIME message contains wrong number of bytes.");
	}

	/* Get the data out */

	time_next_second = (CHARA_TIME *)message->data;

	/* Go tell the hardware */

	if (!set_rt_time(*time_next_second))
	{
		return error(ERROR,"Failed to set RT clock.");
	}

	return NOERROR;

} /* clock_set_rt_time() */

/************************************************************************/
/* clock_sync_times()							*/
/*									*/
/* On message SYNC_TIMES  will call sync_times() fuinction.		*/
/************************************************************************/

int clock_sync_times(struct smessage *message)
{
	if (message->length != 0)
		error(ERROR,
		"Got RT_TIME message with data. Data ignored.");

	error(MESSAGE,"Syncing clock on server.");

	return sync_times();

} /* clock_sync_times() */
