/************************************************************************/
/* fast_clock_control_window.c                                          */
/*                                                                      */
/* Routine for controlling the basic CHARA clock functions.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : December 2004                                               */
/************************************************************************/

#include "../../slalib/slalib.h"
#include "../../cliserv/include/cliserv.h"
#include "../../include/chara_messages.h"
#include "cliserv_fast_clock.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

/* Globals */

static GtkWidget *fast_clock_control_window;
static GtkWidget *fast_time_label;
static GtkWidget *lost_ticks_label;
static GtkWidget *rate_label;
static GtkWidget *cal_label;
static GtkWidget *cal_entry;
static GtkWidget *rate_entry;
static GtkWidget *period_entry;
static char fast_clock_control_window_up = FALSE;
static guint fast_clock_control_background_id;

#define SYNC_OFFSET	50 

/************************************************************************/
/* create_fast_clock_control_window()					*/
/*									*/
/* ROutine for creating the clock control window. This should be self	*/
/* contained and give you all you need to control the basic clock 	*/
/* functionality. Argumentis the window name.				*/
/************************************************************************/

GtkWidget* create_fast_clock_control_window (GtkButton *button, gpointer *data)
{
	GtkWidget *abutton;
	GtkWidget *vbox;
	GtkWidget *hbox;

	struct control_window_data *the_data;

	/* Do nothing if we exist already */

	if (fast_clock_control_window_up) return fast_clock_control_window;

	/* Get the data */

	the_data = (struct control_window_data *)data;

	/* Create the window itself */

	fast_clock_control_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (fast_clock_control_window), 
		"fast_clock_control_window", fast_clock_control_window);
	gtk_window_set_title (GTK_WINDOW (fast_clock_control_window), 
		the_data->name);
	
	/* Make an hbox to put things in */
 
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (fast_clock_control_window), vbox);
	gtk_widget_show(vbox);

	/* fast Time label */

	fast_time_label = gtk_label_new ("FAST TIME: 00:00:00");
	gtk_widget_ref (fast_time_label);
	gtk_box_pack_start(GTK_BOX(vbox), fast_time_label, TRUE, TRUE, 0);
	gtk_widget_set_usize(fast_time_label, FAST_CLOCK_MAIN_WIDTH, 24);
	gtk_widget_show (fast_time_label);

	/* Lost ticks label */

	lost_ticks_label = gtk_label_new ("LOST TICKS:  000000");
	gtk_widget_ref (lost_ticks_label);
	gtk_box_pack_start(GTK_BOX(vbox), lost_ticks_label, TRUE, TRUE, 0);
	gtk_widget_set_usize(lost_ticks_label, FAST_CLOCK_MAIN_WIDTH, 24);
	gtk_widget_show (lost_ticks_label);

	/* rate label */

	rate_label = gtk_label_new ("RATE: ???");
	gtk_widget_ref (rate_label);
	gtk_box_pack_start(GTK_BOX(vbox), rate_label, TRUE, TRUE, 0);
	gtk_widget_set_usize(rate_label, FAST_CLOCK_MAIN_WIDTH, 24);
	gtk_widget_show (rate_label);

	/* cal label */

	cal_label = gtk_label_new ("CALIBRATION: ???");
	gtk_widget_ref (cal_label);
	gtk_box_pack_start(GTK_BOX(vbox), cal_label, TRUE, TRUE, 0);
	gtk_widget_set_usize(cal_label, FAST_CLOCK_MAIN_WIDTH, 24);
	gtk_widget_show (cal_label);

	/* Stuff for calibration */

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (vbox), hbox);
	gtk_widget_show(hbox);

	abutton = gtk_button_new_with_label ("CALIBRATE");
	gtk_widget_ref (abutton);
	gtk_signal_connect(GTK_OBJECT (abutton), "clicked",
		GTK_SIGNAL_FUNC (fast_clock_control_cal_callback), 
		(gpointer *)&(the_data->server));
	gtk_widget_show (abutton);
	gtk_box_pack_start(GTK_BOX(hbox), abutton, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (abutton), 1);
	gtk_widget_set_usize(abutton, FAST_CLOCK_MAIN_WIDTH/2, 24);

	cal_entry = gtk_entry_new ();
	gtk_widget_ref (cal_entry);
	gtk_signal_connect(GTK_OBJECT (cal_entry), "activate",
		GTK_SIGNAL_FUNC (fast_clock_control_cal_callback),
	       	(gpointer) &(the_data->server));
	gtk_widget_show (cal_entry);
	gtk_box_pack_start(GTK_BOX(hbox), cal_entry, TRUE, TRUE, 0);
	gtk_widget_set_usize(cal_entry, FAST_CLOCK_MAIN_WIDTH/2, 24);
	gtk_entry_set_text (GTK_ENTRY (cal_entry), "120");

	abutton = gtk_button_new_with_label ("STOP CALIBRATION");
	gtk_widget_ref (abutton);
	gtk_signal_connect(GTK_OBJECT (abutton), "clicked",
		GTK_SIGNAL_FUNC (fast_clock_control_stop_cal_callback), 
		(gpointer *)&(the_data->server));
	gtk_widget_show (abutton);
	gtk_box_pack_start(GTK_BOX(vbox), abutton, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (abutton), 1);
	gtk_widget_set_usize(abutton, FAST_CLOCK_MAIN_WIDTH, 24);

	/* Change rate stuff */

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (vbox), hbox);
	gtk_widget_show(hbox);

	abutton = gtk_button_new_with_label ("RATE");
	gtk_widget_ref (abutton);
	gtk_signal_connect(GTK_OBJECT (abutton), "clicked",
		GTK_SIGNAL_FUNC (fast_clock_control_rate_callback), 
		(gpointer *)&(the_data->server));
	gtk_widget_show (abutton);
	gtk_box_pack_start(GTK_BOX(hbox), abutton, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (abutton), 1);
	gtk_widget_set_usize(abutton, FAST_CLOCK_MAIN_WIDTH/2, 24);

	rate_entry = gtk_entry_new ();
	gtk_widget_ref (rate_entry);
	gtk_signal_connect(GTK_OBJECT (rate_entry), "activate",
		GTK_SIGNAL_FUNC (fast_clock_control_rate_callback),
	       	(gpointer) &(the_data->server));
	gtk_widget_show (rate_entry);
	gtk_box_pack_start(GTK_BOX(hbox), rate_entry, TRUE, TRUE, 0);
	gtk_widget_set_usize(rate_entry, FAST_CLOCK_MAIN_WIDTH/2, 24);
	gtk_entry_set_text (GTK_ENTRY (rate_entry), "???");

	/* Change period button */

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (vbox), hbox);
	gtk_widget_show(hbox);

	abutton = gtk_button_new_with_label ("PERIOD");
	gtk_widget_ref (abutton);
	gtk_signal_connect(GTK_OBJECT (abutton), "clicked",
		GTK_SIGNAL_FUNC (fast_clock_control_period_callback), 
		(gpointer *)&(the_data->server));
	gtk_widget_show (abutton);
	gtk_box_pack_start(GTK_BOX(hbox), abutton, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (abutton), 1);
	gtk_widget_set_usize(abutton, FAST_CLOCK_MAIN_WIDTH/2, 24);

	period_entry = gtk_entry_new ();
	gtk_widget_ref (period_entry);
	gtk_signal_connect(GTK_OBJECT (period_entry), "activate",
		GTK_SIGNAL_FUNC (fast_clock_control_period_callback),
	       	(gpointer) &(the_data->server));
	gtk_widget_show (period_entry);
	gtk_box_pack_start(GTK_BOX(hbox), period_entry, TRUE, TRUE, 0);
	gtk_widget_set_usize(period_entry, FAST_CLOCK_MAIN_WIDTH/2, 24);
	gtk_entry_set_text (GTK_ENTRY (period_entry), "???");

	/* CLEAR button */

	abutton = gtk_button_new_with_label ("CLEAR LOST TICKS");
	gtk_widget_ref (abutton);
	gtk_signal_connect(GTK_OBJECT (abutton), "clicked",
		GTK_SIGNAL_FUNC (fast_clock_control_clear_callback), 
		(gpointer *)&(the_data->server));
	gtk_widget_show (abutton);
	gtk_box_pack_start(GTK_BOX(vbox), abutton, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (abutton), 1);
	gtk_widget_set_usize(abutton, FAST_CLOCK_MAIN_WIDTH, 24);

	/* OK button */

	abutton = gtk_button_new_with_label ("OK");
	gtk_widget_ref (abutton);
	gtk_signal_connect(GTK_OBJECT (abutton), "clicked",
		GTK_SIGNAL_FUNC (fast_clock_control_ok_callback), NULL);
	gtk_widget_show (abutton);
	gtk_box_pack_start(GTK_BOX(vbox), abutton, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (abutton), 1);
	gtk_widget_set_usize(abutton, FAST_CLOCK_MAIN_WIDTH, 24);

	/* Now, we can show this window if we like */

	gtk_widget_show(fast_clock_control_window);

	/* Make sure we only get one version of this window */

	fast_clock_control_window_up = TRUE;

	/* Start routine for background processing */

	fast_clock_control_background_id = 
	gtk_timeout_add(1000,fast_clock_control_background, 
		(gpointer *)&(the_data->server));

	/* That should be all */

	return fast_clock_control_window;

} /* create_fast_clock_control_window() */

/************************************************************************/
/* fast_clock_control_ok_callback()					*/
/*									*/
/* Callback function for clock control OK button. Stop background	*/
/* processing and destroy clock control window.				*/
/************************************************************************/

void fast_clock_control_ok_callback(GtkButton *button, gpointer user_data)
{
	if (!fast_clock_control_window_up) return;

	gtk_widget_destroy(fast_clock_control_window);
	gtk_timeout_remove(fast_clock_control_background_id);
		fast_clock_control_window_up = FALSE;

} /* fast_clock_control_ok_callback() */

/************************************************************************/
/* fast_clock_control_background()					*/
/*									*/
/* Here's the background job for looking at clock outputs and so on.	*/
/************************************************************************/

gint fast_clock_control_background(gpointer data)
{
	struct smessage mess;
	int	*server;
	int	hrs,min,sec;
	char	s[80];
	int now;

	/* Do nothing if the window isn't there. */

	if (!fast_clock_control_window_up) return TRUE;

	/* Which server is speaking to us? */

	server = (int *)data;

	/* Fill in the data */

	now = fast_time[*server].time_now;
	if (now >= 0 && fast_clock_rate[*server] != 0)
	{
		now /= fast_clock_rate[*server];
		sec = now % 60;
		now /= 60;
		min = now % 60;
		hrs = now / 60;
		sprintf(s,"FAST TIME: %02d:%02d:%02d",hrs,min,sec);
		gtk_label_set_text((GtkLabel *)fast_time_label,s);

		sprintf(s,"LOST TICKS:  %d",  fast_time[*server].lost_ticks);
		gtk_label_set_text((GtkLabel *)lost_ticks_label,s);

		sprintf(s,"RATE:  %d",  fast_clock_rate[*server]);
		gtk_label_set_text((GtkLabel *)rate_label,s);

		sprintf(s,"CALIBRATION: %d",  fast_clock_cal_time[*server]);
		gtk_label_set_text((GtkLabel *)cal_label,s);
	}
	
	/* Update the current data */

	if (fast_clock_cal_time[*server] ==0)
	{
		mess.type = FAST_CLOCK_TIME;
		mess.length =0;
		mess.data = NULL;
		if (!send_message(*server, &mess))
		  create_message_window("Failed to send FAST_CLOCK_TIME.");

		mess.type = FAST_CLOCK_GET_RATE;
		mess.length =0;
		mess.data = NULL;
		if (!send_message(*server, &mess))
		  create_message_window("Failed to send FAST_CLOCK_GET_RATE.");
	}

	return TRUE;

} /* fast_clock_control_background() */

/************************************************************************/
/* fast_clock_control_clear_callback()					*/
/*									*/
/* Sends meeasge to clear lost counts.					*/
/************************************************************************/

void fast_clock_control_clear_callback(GtkButton *button, gpointer user_data)
{
	struct smessage mess;
	int	*server;

	server = (int *) user_data;

	mess.type = FAST_CLOCK_CLEAR_LOST_TICKS;
	mess.length =0;
	mess.data = NULL;
	if (!send_message(*server, &mess))
		create_message_window(
			"Failed to send FAST_CLOCK_CLEAR_LOST_TICKS.");

} /* fast_clock_control_clear_callback() */

/************************************************************************/
/* fast_clock_control_cal_callback()					*/
/*									*/
/* Sends a message to start calibration cycle.				*/
/************************************************************************/

void fast_clock_control_cal_callback(GtkWidget *widget, gpointer user_data)
{
	int	second;
	struct smessage mess;
	int	*server;
	char	*text;

	server = (int *)user_data;

	/* Get the text */

	text = gtk_entry_get_text(GTK_ENTRY(cal_entry));

	if (sscanf(text,"%d",&second) != 1)
	{
		create_message_window("Seconds format must be single number");
		return;
	}

	/* Send the message */

	mess.type = FAST_CLOCK_CALIBRATE;
	mess.length = sizeof(second);
	mess.data = (unsigned char *)&second;
	if (!send_message(*server, &mess))
		create_message_window("Failed to send FAST_CLOCK_CALIBRATE.");
	
} /* fast_clock_control_cal_callback() */

/************************************************************************/
/* fast_clock_control_stop_cal_callback()				*/
/*									*/
/* Sends message to stop calibration.					*/
/************************************************************************/

void fast_clock_control_stop_cal_callback(GtkButton *button, gpointer user_data)
{
	int	*server;

	server = (int *) user_data;

	send_ui_signal(*server, FAST_CLOCK_SIGNAL_STOP);

} /* fast_clock_control_stop_cal_callback() */

/************************************************************************/
/* fast_clock_control_rate_callback()					*/
/*									*/
/* Sends a message to set the rate.					*/
/************************************************************************/

void fast_clock_control_rate_callback(GtkWidget *widget, gpointer user_data)
{
	int	rate;
	struct smessage mess;
	int	*server;
	char	*text;

	server = (int *)user_data;

	/* Get the text */

	text = gtk_entry_get_text(GTK_ENTRY(rate_entry));

	if (sscanf(text,"%d",&rate) != 1)
	{
		create_message_window("Rate format must be single number");
		return;
	}

	/* Send the message */

	mess.type = FAST_CLOCK_SET_RATE;
	mess.length = sizeof(rate);
	mess.data = (unsigned char *)&rate;
	if (!send_message(*server, &mess))
		create_message_window("Failed to send FAST_CLOCK_SET_RATE.");
	
} /* fast_clock_control_rate_callback() */

/************************************************************************/
/* fast_clock_control_period_callback()					*/
/*									*/
/* Sends a message to set the period.					*/
/************************************************************************/

void fast_clock_control_period_callback(GtkWidget *widget, gpointer user_data)
{
	int	period;
	struct smessage mess;
	int	*server;
	char	*text;

	server = (int *)user_data;

	/* Get the text */

	text = gtk_entry_get_text(GTK_ENTRY(period_entry));

	if (sscanf(text,"%d",&period) != 1)
	{
		create_message_window("Period format must be single number");
		return;
	}

	/* Send the message */

	mess.type = FAST_CLOCK_SET_PERIOD;
	mess.length = sizeof(period);
	mess.data = (unsigned char *)&period;
	if (!send_message(*server, &mess))
		create_message_window("Failed to send FAST_CLOCK_SET_PERIOD.");
	
} /* fast_clock_control_period_callback() */

