/************************************************************************/
/* huT_messages.c							*/
/*                                                                      */
/* Message handling routines.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../include/chara_messages.h"
#include "../../cliserv/include/cliserv.h"
#include "../../cliserv/uiclient/uiclient.h"
#include "../../sockman/sockman.h"
#include "ui_huT_client.h"


char *huT_file_name = NULL;
int  huT_server = -1;
struct s_hut huT;
static bool send_huT = FALSE;
float huT_exp_times[NUM_HUT_EXP+1] = {0.0,
                1.0/2000.0,
                1.0/1000.0,
                1.0/500.0,
                1.0/250.0,
                1.0/125.0,
                1.0/60.0,
                1.0/30.0,
                1.0/15.0,
                2.0/15.0,
                4.0/15.0,
                8.0/15.0,
                16.0/15.0,
                32.0/15.0,
                64.0/15.0,
                128.0/15.0};

/************************************************************************/
/* open_huT_server_socket()						*/
/*									*/
/* Opens a socket so we can start sending server data to an huT    	*/
/* server. Returns server number or 					*/
/* -1 on error opening server.						*/
/* -2 on error adding standard jobs					*/
/* -3 on error adding local jobs					*/
/* -4 on error to send name and status messages				*/
/* -5 got no response from server					*/
/************************************************************************/

int open_huT_server_socket(char *name, bool send_huT_value)
{
	int	server;

	/* Try and open the socket */


	if ((server = open_server_by_name(name)) == -1)
	{
		return -1;
	}

	if (!set_up_standard_jobs(server)) 
	{
		close_server_socket(server);
		return -2;
	}
	
	/* Add all the local messages */

        if (!add_message_job(server, HUT_NOW, message_huT_now) ||
            !add_message_job(server, HUT_FILE_NAME, message_huT_file_name) ||
            !add_message_job(server, HUT_AOB_STATUS, message_huT_aob_status))
	 
	{
		close_server_socket(server);
		return -3;
	}

	/* Set up the initial values for teh global */

	huT.temp = 0.0;
	huT.rh = 0.0;
	huT.status = 0;
	huT.error = 0;

	/* Set global for sending HuT to clients */

	send_huT = send_huT_value;

	/* That is all */

	return server;

} /* open_huT_server_socket() */

/************************************************************************/
/* message_huT_now()							*/
/*									*/
/************************************************************************/

int message_huT_now(int server, struct smessage *mess)
{
	int	i;
	char	*p1,*p2;

        if (mess->length != sizeof(struct s_hut))
        {
		return error(ERROR, 
		"Receiced HUT_NOW with wrong amount of data.");
        }

	/* Copy it into the global structure */

	for (p1 = (char *) &huT, p2 = (char *)mess->data, i=0;
		i < sizeof(struct s_hut); i++) *p1++ = *p2++;

	/* Do we send it to all clients? */

	if (send_huT) return send_huT_all_clients();

        return NOERROR;

} /* message_huT_status() */

/************************************************************************/
/* message_huT_file_name()						*/
/*									*/
/************************************************************************/

int message_huT_file_name(int server, struct smessage *mess)
{
	int	i;
	char	*p1, *p2;

	if (huT_file_name != NULL) free(huT_file_name);
	huT_file_name = malloc(mess->length+2);

	for(p1 = (char *)mess->data, p2 = huT_file_name, i=0; 
		i<mess->length; i++) *p2++ = *p1++;
	*p2 = 0;

	return TRUE;

} /* message_huT_names() */

/************************************************************************/
/* message_huT_aob_status()                                             */
/************************************************************************/

int message_huT_aob_status(int server, struct smessage *mess)
{
        char s[1024], *p1, *p2;
        int  i;

        if (mess->length == 0) return NOERROR;

        for(i=0, p1 = s, p2 =(char *)mess->data;
                i < mess->length; i++, p1++, p2++) *p1 = *p2;
        *p2 = 0;

        message(system_window, s);

	return ui_send_message_all(mess);

} /* message_huT_aob_status() */

/************************************************************************/
/* get_huT_file_name()							*/
/*									*/
/* Go ask for the name of the huT data file.				*/
/* Value returned or NULL on error.					*/
/************************************************************************/

char *get_huT_file_name(void)
{
	struct smessage mess;
	time_t  start;

	if (huT_server == -1)
	{
		error(ERROR,"Weather server is not open.");
		return NULL;
	}

	/* prepare */

	if (huT_file_name != NULL) free(huT_file_name);
	huT_file_name = NULL;

	/* Send request */

	mess.type = HUT_FILE_NAME;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(huT_server, &mess))
	{
		error(ERROR, "Failed to send HUT_FILE_NAME.\n");
		return NULL;
	}

	/* wait for a response */

	start = time(NULL);
	while(time(NULL) < start+TIME_OUT)
	{
		process_server_socket(huT_server);
		if (huT_file_name != NULL) break;
	}
	if (time(NULL) >= start+TIME_OUT)
	{
		error(ERROR,"Failed to get weahter log file name.\n");
		return NULL;
	}

	return huT_file_name;

} /* get_huT_file_name() */

/************************************************************************/
/* hut_reset_comms()							*/
/*									*/
/* User callable routine to send the HUT_RESET_COMMS message.		*/
/************************************************************************/

int hut_reset_comms(int argc, char **argv)
{
	struct smessage mess;
	if (huT_server == -1)
	{
		return error(ERROR,"Weather server is not open.");
	}

	/* Send request */

	mess.type = HUT_RESET_COMMS;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(huT_server, &mess))
	{
		return error(ERROR, "Failed to send HUT_RESET_COMMS.\n");
	}

	return NOERROR;

} /* hut_reset_comms() */

/************************************************************************/
/* hut_heater_on()							*/
/*									*/
/* User callable routine to send the HUT_HEATER_ON message.		*/
/************************************************************************/

int hut_heater_on(int argc, char **argv)
{
	struct smessage mess;
	if (huT_server == -1)
	{
		return error(ERROR,"Weather server is not open.");
	}

	/* Send request */

	mess.type = HUT_HEATER_ON;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(huT_server, &mess))
	{
		return error(ERROR, "Failed to send HUT_HEATER_ON.\n");
	}

	return NOERROR;

} /* hut_heater_on() */

/************************************************************************/
/* hut_heater_off()							*/
/*									*/
/* User callable routine to send the HUT_HEATER_OFF message.		*/
/************************************************************************/

int hut_heater_off(int argc, char **argv)
{
	struct smessage mess;
	if (huT_server == -1)
	{
		return error(ERROR,"Weather server is not open.");
	}

	/* Send request */

	mess.type = HUT_HEATER_OFF;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(huT_server, &mess))
	{
		return error(ERROR, "Failed to send HUT_HEATER_OFF.\n");
	}

	return NOERROR;

} /* hut_heater_off() */

/************************************************************************/
/* hut_dehum_on()							*/
/*									*/
/* User callable routine to send the HUT_DEHUM_ON message.		*/
/************************************************************************/

int hut_dehum_on(int argc, char **argv)
{
	struct smessage mess;
	if (huT_server == -1)
	{
		return error(ERROR,"Weather server is not open.");
	}

	/* Send request */

	mess.type = HUT_DEHUM_ON;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(huT_server, &mess))
	{
		return error(ERROR, "Failed to send HUT_DEHUM_ON.\n");
	}

	return NOERROR;

} /* hut_dehum_on() */

/************************************************************************/
/* hut_dehum_off()							*/
/*									*/
/* User callable routine to send the HUT_DEHUM_OFF message.		*/
/************************************************************************/

int hut_dehum_off(int argc, char **argv)
{
	struct smessage mess;
	if (huT_server == -1)
	{
		return error(ERROR,"Weather server is not open.");
	}

	/* Send request */

	mess.type = HUT_DEHUM_OFF;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(huT_server, &mess))
	{
		return error(ERROR, "Failed to send HUT_DEHUM_OFF.\n");
	}

	return NOERROR;

} /* hut_dehum_off() */

/************************************************************************/
/* hut_auto_on()							*/
/*									*/
/* User callable routine to send the HUT_AUTO_ON message.		*/
/************************************************************************/

int hut_auto_on(int argc, char **argv)
{
	struct smessage mess;
	if (huT_server == -1)
	{
		return error(ERROR,"Weather server is not open.");
	}

	/* Send request */

	mess.type = HUT_AUTO_ON;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(huT_server, &mess))
	{
		return error(ERROR, "Failed to send HUT_AUTO_ON.\n");
	}

	return NOERROR;

} /* hut_auto_on() */

/************************************************************************/
/* hut_auto_off()							*/
/*									*/
/* User callable routine to send the HUT_AUTO_OFF message.		*/
/************************************************************************/

int hut_auto_off(int argc, char **argv)
{
	struct smessage mess;
	if (huT_server == -1)
	{
		return error(ERROR,"Weather server is not open.");
	}

	/* Send request */

	mess.type = HUT_AUTO_OFF;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(huT_server, &mess))
	{
		return error(ERROR, "Failed to send HUT_AUTO_OFF.\n");
	}

	return NOERROR;

} /* hut_auto_off() */

/************************************************************************/
/* send_huT_all_clients()						*/
/*									*/
/* Sends the huT status structure to all clients.			*/
/* Returns error level.							*/
/************************************************************************/

int send_huT_all_clients(void)
{
	struct smessage mess;

	mess.type = HUT_NOW;
	mess.length = sizeof(huT);
	mess.data = (unsigned char *) &huT;

	return ui_send_message_all(&mess);

} /* send_huT_all_clients() */
