/************************************************************************/
/* rpc_messages.c							*/
/*                                                                      */
/* Message handling routines.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../include/chara_messages.h"
#include "../../cliserv/include/cliserv.h"
#include "../../cliserv/uiclient/uiclient.h"
#include "ui_rpc_client.h"

struct srpc_ui_client rpc_info[MAX_SERVERS];

/************************************************************************/
/* open_rpc_server_socket()						*/
/*									*/
/* Opens a socket so we can start sending server data to an rpc    	*/
/* server. Returns server number or 					*/
/* -1 on error opening server.						*/
/* -2 on error adding standard jobs					*/
/* -3 on error adding local jobs					*/
/* -4 on error to send name and status messages				*/
/* -5 got no response from server					*/
/************************************************************************/

int open_rpc_server_socket(char *name)
{
	int	server;
	struct smessage mess;
        time_t   start;
	int	i;

	/* Try and open the socket */

	if ((server = open_server_by_name(name)) == -1)
	{
		return -1;
	}

	/* Set all names to NULLS */

	rpc_info[server].unit_name = NULL;
	for(i=0; i<NUM_PORTS; i++) rpc_info[server].port_names[i] = NULL;

	/* Now let's add our standard jobs to this server */

	if (!set_up_standard_jobs(server)) 
	{
		close_server_socket(server);
		return -2;
	}
	
	/* Add all the local messages */

        if (!add_message_job(server, RPC_STATUS, message_rpc_status) ||
            !add_message_job(server, RPC_NAMES, message_rpc_names)) 
	{
		close_server_socket(server);
		return -3;
	}

	/* Send out message to get the names and current status */

        mess.type = RPC_GET_NAMES;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
	{
		close_server_socket(server);
		return -4;
	}

        mess.type = RPC_GET_STATUS;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
	{
		close_server_socket(server);
		return -4;
	}

        /* Now let's wait a while and see if we get an answer. */

        start = time(NULL);
        while(time(NULL) < start+TIME_OUT && rpc_info[server].unit_name == 0)
        {
                process_server_socket(server);
        }

	if (rpc_info[server].unit_name == 0) 
	{
		close_server_socket(server);
		return -5;
	}

	/* That is all */

	return server;

} /* open_rpc_server_socket() */

/************************************************************************/
/* message_rpc_status()							*/
/*									*/
/************************************************************************/

int message_rpc_status(int server, struct smessage *mess)
{
        if (mess->length != sizeof(char))
        {
                return error(ERROR,
			"Receiced RPC_STATUS with wrong amount of data.");
        }

	rpc_info[server].status = (unsigned char) (*mess->data);

        return NOERROR;

} /* message_rpc_status() */

/************************************************************************/
/* message_rpc_names()							*/
/*									*/
/************************************************************************/

int message_rpc_names(int server, struct smessage *mess)
{
	char	s[256],*p1,*p2;
	int	i;

	/* We're hoping there's the right kind of data here */

	p1 = (char *)mess->data;

	if (rpc_info[server].unit_name != NULL) 
		free(rpc_info[server].unit_name);

	for(p2 = s; 
		*p1 != 0 && (p1-(char*)mess->data)<=mess->length; 
		p1++, p2++) *p2 = *p1;
	*p2 = 0;
	p1++;

	if ((rpc_info[server].unit_name = malloc(strlen(s))) == NULL)
		error(FATAL,"Run out of memory.");

	strcpy(rpc_info[server].unit_name, s);

	for(i=0; i<NUM_PORTS ; i++)
	{
		if (rpc_info[server].port_names[i] != NULL) 
			free(rpc_info[server].port_names[i]);

		for(p2 = s; 
			*p1 != 0 /*&& (p1-(char*)mess->data)<=mess->length*/; 
			p1++, p2++) *p2 = *p1;
		*p2 = 0;
		p1++;
	
		if ((rpc_info[server].port_names[i] =malloc(strlen(s))) == NULL)
			error(FATAL,"Run out of memory.");

		strcpy(rpc_info[server].port_names[i], s);
	}

	return NOERROR;

} /* message_rpc_names() */

/************************************************************************/
/* send_rpc_on()							*/
/* 									*/
/* Send the RPC_ON message to the server.				*/
/* Returns an error level.						*/
/************************************************************************/

int send_rpc_on(int server, int port)
{
	struct smessage	mess;
	unsigned char port_number;

	/* Make sure everything is OK */

	if (!server_socket_open(server))
		return error(ERROR,"Server socket is not open.");

	if (port < 1 || port > NUM_PORTS)
		return error(ERROR,"Port number %d is out of range.");


	/* Construct command */

	mess.type = RPC_ON;
	port_number = port;
	mess.length = sizeof(port_number);
	mess.data = &port_number;

	if (!send_message(server, &mess))
		return error(ERROR,"Failed to send RPC_ON message.");

	return NOERROR;

} /* send_rpc_on() */

/************************************************************************/
/* send_rpc_off()							*/
/* 									*/
/* Send the RPC_ON message to the server.				*/
/* Returns an error level.						*/
/************************************************************************/

int send_rpc_off(int server, int port)
{
	struct smessage	mess;
	unsigned char port_number;

	/* Make sure everything is OK */

	if (!server_socket_open(server))
		return error(ERROR,"Server socket is not open.");

	if (port < 1 || port > NUM_PORTS)
		return error(ERROR,"Port number %d is out of range.");


	/* Coffstruct command */

	mess.type = RPC_OFF;
	port_number = port;
	mess.length = sizeof(port_number);
	mess.data = &port_number;

	if (!send_message(server, &mess))
		return error(ERROR,"Failed to send RPC_OFF message.");

	return NOERROR;

} /* send_rpc_off() */
