/************************************************************************/
/* uishut_standard.c							*/
/*                                                                      */
/* Routines to respond to standard messages from server.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY client INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : August 2002							*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../cliserv/include/cliserv.h"
#include "../../include/chara_messages.h"
#include "../../cliserv/uiclient/uiclient.h"
#include "shutter.h"
#include "../chara_clock/include/cliserv_clock.h"
#include "uishut.h"

char	*shutter_types[NUMBER_SHUTTERS+1];
char    *shutter_status_types[5] = {"CLOSED", "OPEN  ", "MOVING", "INSANE", 
				(char *)NULL};
char	*shutter_standard_config_types[MAX_CONFIGS+1];
static int	shutter_status[NUMBER_SHUTTERS];
struct sshutter_conf shutter_conf;

int shut_server = -1;

/************************************************************************/
/* set_up_shutter_jobs()						*/
/*									*/
/* Puts functions to deal with standard messages into message que.	*/
/* Returns TRUE if all goes well, FALSE otherwise.			*/
/*									*/
/************************************************************************/

int set_up_shutter_jobs(void)
{

	if (shut_server == -1) return FALSE;

        if (!add_message_job(shut_server, 	
		STATUS_SHUTTER, message_shutter_status)) 
		return FALSE;
        if (!add_message_job(shut_server, 
		SHUTTER_CONF, message_shutter_conf)) 
		return FALSE;

	return TRUE;

} /* set_up_shutter_jobs() */

/************************************************************************/
/* message_shutter_status()						*/
/*									*/
/************************************************************************/

int message_shutter_status(int server, struct smessage *mess)
{
	char	s[256];
	struct sshutter_status *status;

        if (mess->length != sizeof(*status))
        {
		message(system_window,
			"Receiced STATUS_SHUTTER with wrong amount of data.");
		return FALSE;
        }

	status = (struct sshutter_status *)mess->data;

	/* Update the array for status */

	shutter_status[status->shutter] = status->status;

	/* Is it INSANE? */

	if (status->status == INSANE)
	{
		sprintf(s,"ERROR\nShutter %s is reported as INSANE!",
				shutter_conf.shutter_names[status->shutter]);
	        message(system_window,s);
	}

        return TRUE;

} /* message_shutter_status() */

/************************************************************************/
/* message_shutter_conf()						*/
/*									*/
/************************************************************************/

int message_shutter_conf(int server, struct smessage *mess)
{
	int	i;
	unsigned char	*p1, *p2;

	/* OK, we just copy the thing we got in the message */

	if (mess->length != sizeof(shutter_conf))
	{
	    message(system_window,"Wrong number of bytes in configuration.");
	    return FALSE;
	}

	for(i=0, p1 = (unsigned char *)&shutter_conf, p2 = mess->data;
		i < sizeof(shutter_conf); i++, p1++, p2++) *p1 = *p2;

	return TRUE;

} /* message_shutter_conf() */

/************************************************************************/
/* send_shutter_open()							*/
/* 									*/
/* Send the OPEN_SHUTTER message to the server.				*/
/* Returns TRUE or FALSE.						*/
/************************************************************************/

int send_shutter_open(unsigned char shutter)
{
	struct smessage	mess;

	/* Make sure everything is OK */

	if (!server_socket_open(shut_server)) return FALSE;

	if (shutter >= NUMBER_SHUTTERS) return FALSE;

	/* Construct command */

	mess.type = OPEN_SHUTTER;
	mess.length = sizeof(shutter);
	mess.data = &shutter;

	if (!send_message(shut_server, &mess))
		message(system_window,"Failed to send OPEN_SHUTTER message.");

	return TRUE;

} /* send_shutter_open() */

/************************************************************************/
/* send_shutter_close()							*/
/* 									*/
/* Send the CLOSE_SHUTTER message to the server.			*/
/* Returns an TRUE or FALSE.						*/
/************************************************************************/

int send_shutter_close(unsigned char shutter)
{
	struct smessage	mess;

	/* Make sure everything is OK */

	if (!server_socket_open(shut_server)) return FALSE;

	if (shutter >= NUMBER_SHUTTERS) return FALSE;

	/* Construct command */

	mess.type = CLOSE_SHUTTER;
	mess.length = sizeof(shutter);
	mess.data = &shutter;

	if (!send_message(shut_server, &mess))
		message(system_window,"Failed to send CLOSE_SHUTTER message.");

	return TRUE;

} /* send_shutter_close() */

/************************************************************************/
/* send_shutter_status()						*/
/* 									*/
/* Send the STATUS_SHUTTER message to the server.			*/
/* Returns an TRUE or FALSE.						*/
/************************************************************************/

int send_shutter_status(unsigned char shutter)
{
	struct smessage	mess;
	struct sshutter_status status;

	/* Make sure everything is OK */

	if (!server_socket_open(shut_server)) return FALSE;

	if (shutter >= NUMBER_SHUTTERS) return FALSE;

	/* Construct command */

	status.shutter = shutter;
	status.status  = 0;
	mess.type = STATUS_SHUTTER;
	mess.length = sizeof(status);
	mess.data = (unsigned char *)&status;

	if (!send_message(shut_server, &mess))
		message(system_window,"Failed to send STATUS_SHUTTER message.");

	return TRUE;

} /* send_shutter_status() */

/************************************************************************/
/* open_shutter_server()						*/
/* 									*/
/* Go ahead and try and open the shutter server.			*/
/* Returns 0 if all went well.						*/ 
/* -1 of failes to open server.						*/
/* -2 failed to setup standard jobs					*/
/* -3 Failed to setup shutter jobs					*/
/* -4 Failed to send initialize message					*/
/* -5 if no response from server					*/
/************************************************************************/

int open_shutter_server(void)
{
	int	err;
        struct smessage mess;
        time_t   start;
	char	s[256];
	int	i;

	if (server_socket_open(shut_server)) close_shutter_server(0, NULL);

	for(i=0; i<NUMBER_SHUTTERS; i++) shutter_status[i] = INSANE;

        if ((shut_server = open_server_by_name(SHUTTER_PROCESS_NAME)) 
		== -1) return -1;

	/* Now let's add our standard jobs to this server */

        if (!set_up_standard_jobs(shut_server)) 
        {
            close_server_socket(shut_server);
	    shut_server = -1;
	    return -2;
        }

	/* Now our own specific jobs */

	if (!set_up_shutter_jobs())
	{
	    close_server_socket(shut_server);
	    shut_server = -1;
	    return -3;
	}

        /* Send out requests for machine name and port names */

        *(shutter_conf.name) = 0;
        mess.type = SHUTTER_CONF;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(shut_server, &mess))
        {
		close_server_socket(shut_server);
		shut_server = -1;
                return -4;
        }

        /* Now let's wait a while and see if we get an answer. */

        start = time(NULL);
        while(time(NULL) < start+TIME_OUT && *(shutter_conf.name) == 0)
        {
                process_server_socket(shut_server);
        }

        if (*(shutter_conf.name) == 0)
        {
		close_server_socket(shut_server);
		shut_server = -1;
                return -5;
        }

	/* Fill out the shutter_types array and ask for status info */

	for(i=0; i<NUMBER_SHUTTERS; i++)
	{
		if (strlen(shutter_conf.shutter_names[i]))
			shutter_types[i] = shutter_conf.shutter_names[i];
		else
			shutter_types[i] = "NONE";

		send_shutter_status(i);
	}
	shutter_types[NUMBER_SHUTTERS] = NULL;

	/* Fill out the configuration types */

	for(i=0; i<shutter_conf.number_confs; i++)
	{
		shutter_standard_config_types[i] = shutter_conf.conf_names[i];
	}
	shutter_standard_config_types[i] = NULL;

	/* That should be it for now */

	message(system_window,"Opened connection with %s",shutter_conf.name);

	return 0;

} /* open_shutter_server() */

/************************************************************************/
/* call_open_shutter_server()						*/
/* 									*/
/* USer callable version of open_shutter_server.			*/
/************************************************************************/

int call_open_shutter_server(int argc, char **argv)
{
	if (open_shutter_server() != 0)
		return error(ERROR, "Failed to open shutter port.");

	return NOERROR;

} /* call_open_shutter_server() */

/************************************************************************/
/* close_shutter_server()						*/
/*									*/
/* Just calls close_server_socket().					*/
/************************************************************************/

int close_shutter_server(int argc, char **argv)
{
	close_server_socket(shut_server);
	shut_server = -1;

	return NOERROR;

} /* close_shutter_server() */

/************************************************************************/
/* get_shutter_number() 						*/
/*									*/
/* If argc>1 it tries to scan a shutter number from argv[1], otherwise	*/
/* polls the user for one. Returns the DL number or -1 if the user 	*/
/* exits using an ESCAPE key. Returns -2 if the delayline number is	*/
/* out of range.  							*/
/************************************************************************/

int get_shutter_number(int argc, char **argv)
{
	int shutter_number;

	/* Check out the command line */

	mvwaddstr(command_window,0,2,"                        ");
	wrefresh(command_window);
	if (argc > 1)
	{
	    if ((shutter_number = shutter_id(argv[1])) == -1)
	    {
	    	error(ERROR,"Unkown shutter %s.",argv[1]);
	    	return -2;
	    }
	}
	else
	{
		shutter_number = 0;
	        if (quick_edit("Shutter",shutter_types[shutter_number],
                        &shutter_number,shutter_types,ENUMERATED) == KEY_ESC)
                                return -1;
	}

	return shutter_number;

} /* get_shutter_number() */

/************************************************************************/
/* shutter_id()								*/
/*									*/
/* Given the string name for a shutter as an argument returns the	*/
/* shutter number for that shutter, or -1 if no name like that is	*/
/* found.								*/
/************************************************************************/

int shutter_id(char *name)
{
	int	id;

	if (!server_socket_open(shut_server)) return -1;

	for (id=0; id<NUMBER_SHUTTERS; id++)
	{
		if (strcmpi(name,shutter_types[id]) == 0)
		{
			return id;
		}
	}

	return -1;

} /* shutter_id() */


/************************************************************************/
/* shutter_open()							*/
/*									*/
/* User callable version of send_shutter_open.				*/
/************************************************************************/

int shutter_open(int argc, char **argv)
{
	char shutter;

	if (!server_socket_open(shut_server)) 
		return error(ERROR,"Shutter server not open.");

        /* get the detector channel */

        if ((shutter = get_shutter_number(argc,argv)) == -1) 
                return NOERROR;
        else if (shutter == -2)
                return ERROR;

	/* Now do it */

	if (!send_shutter_open(shutter)) return ERROR;

	return NOERROR;

} /* call_shutter_open() */

/************************************************************************/
/* shutter_close()							*/
/*									*/
/* User callable version of shutter_close.				*/
/************************************************************************/

int shutter_close(int argc, char **argv)
{
	char shutter;

        /* get the detector channel */

        if ((shutter = get_shutter_number(argc,argv)) == -1) 
                return NOERROR;
        else if (shutter == -2)
                return ERROR;

	/* Now do it */

	if (!send_shutter_close(shutter)) return ERROR;

	return NOERROR;

} /* call_shutter_close() */

/************************************************************************/
/* shutter_all_close()							*/
/*									*/
/* Closes all shutters.							*/
/************************************************************************/

int shutter_all_close(int argc, char **argv)
{
	char shutter;
	int	result;

	result = NOERROR;
	for(shutter = 0; shutter < NUMBER_SHUTTERS; shutter++)
	{
		if (!send_shutter_close(shutter))
		{
			result = ERROR;
		}
		usleep(100);
	}

	if (result != NOERROR) 
		return error(ERROR,"Failed to close all shutters.");

	return NOERROR;

} /* shutter_all_close() */

/************************************************************************/
/* get_shutter_status()							*/
/*									*/
/* Function callable from other functions to get shutter status.	*/
/* Return zero if all went well.					*/
/************************************************************************/

int get_shutter_status(int shutter, unsigned char *status)
{
        int     i;

	if (shutter < 0 || shutter >= NUMBER_SHUTTERS) return -1;

        /* Now do it */

        if (!send_shutter_status(shutter))
        {
                return -2;
        }

        for(i=0; i<20; i++) process_server_socket(shut_server);

        *status = shutter_status[shutter];

	return 0;

} /* get_shutter_status() */

/************************************************************************/
/* call_shutter_status()						*/
/*									*/
/* User callable version of send_shutter_status.			*/
/************************************************************************/

int call_shutter_status(int argc, char **argv)
{
	int shutter;
	unsigned char status;
	int	i;

        if ((shutter = get_shutter_number(argc,argv)) == -1) 
                return NOERROR;
        else if (shutter == -2)
                return ERROR;

	if (get_shutter_status(shutter, &status) != 0)
	{
		return error(ERROR,"Failed to get shutter status.");
	}

	message(system_window,"Shutter %s is %s",shutter_types[shutter],
				shutter_status_types[shutter_status[shutter]]);

	return NOERROR;

} /* call_shutter_status() */

/************************************************************************/
/* monitor_shutters()							*/
/*									*/
/* Continuously writes up the state of the shutters.			*/
/************************************************************************/

int monitor_shutters(int argc, char **argv)
{
	int	i;
	bool    loop;
	int     keystroke;
	unsigned char status;
        time_t   start;

	if (active_socket == -1)
	{
		active_window = sub_main_window;
		loop = TRUE;
		start = time(NULL);
		while(loop)
		{
			heading(heading_window,
		"SHUTTER POSITIONS - TYPE <ESC> TO EXIT OR A COMMAND","");

			/* Now loop through the shutters */

			werase(sub_main_window);
			message(system_window,CONT_TEXT);

			/* While there's no typing update display */

			while(!kbhit())
			{
			    background();

			    /* Do not do this too often */

			    if (time(NULL) < start + 1) continue;
			    start = time(NULL);

			    for(i=0; i< NUMBER_SHUTTERS; i++)
			    {
				background();

				/* Is this a connected shutter? */

				if (strcmpi(shutter_types[i],"NONE") != 0)
				{
				    if (!send_shutter_status(i))
				    {	
					return error(ERROR,
					"Error sending status message.");
				    }
	   
				    mvwprintw(sub_main_window,
				       i%7+1, i/7 * 20, "%s", 
					shutter_types[i]); 
				    mvwprintw(sub_main_window,
				       i%7+1, i/7 * 20 + 10, "%s",
				       shutter_status_types[shutter_status[i]]);
				}
			    }
			    wrefresh(sub_main_window);
			}

			/* Is the user trying to type a command ? */

		        keystroke = get_command();
#warning MOUSE STUFF HERE
			/* Was it KEY_ESC? */

			if (keystroke == KEY_ESC) 
			{
				loop = FALSE;
			}
			else
			{
				/* Is it a command ? */

				command_processor(keystroke);
			}
		}
		werase(system_window);
		wrefresh(system_window);
	}
	else
	{
		for(i=0; i< NUMBER_SHUTTERS; i++)
		{
			/* Is this a connected shutter? */

			if (strcmpi(shutter_types[i],"NONE") != 0)
			{
			    if (!send_shutter_status(i))
			    {	
				return error(ERROR,
				"Error sending status message.");
			    }

			    socket_print(active_socket, "%s : %s\n",
				shutter_types[i], 
				shutter_status_types[shutter_status[i]]);
			}
		}
	}
	
	return NOERROR;

} /* monitor_shutters() */

/************************************************************************/
/* shutter_config()							*/
/*									*/
/* This routine set's the shutters to some standard configuration.	*/
/* These configurations are in the array shutter_standard_config[][].	*/
/************************************************************************/

int shutter_config(int argc, char **argv)
{
	int	config_id;
	int	id;
	bool	delay;
	rt_time_struct now, stop;

	/* Let's see if the user had anything to say */

        if (argc > 1)
        {
		for (config_id=0; config_id<shutter_conf.number_confs; 
			config_id++)
		{
		    if (strcmpi(argv[1],
			shutter_standard_config_types[config_id])==0)
				break;
		}

		if (config_id >= shutter_conf.number_confs)
		{
		    return error(ERROR,"Unknown configuration %s.",argv[1]);
		}
        }
        else
        {
                config_id = 0;
                if (quick_edit("Shutter configuration",
			shutter_standard_config_types[config_id],
                        &config_id,shutter_standard_config_types,
			ENUMERATED) == KEY_ESC) return -1;
        }

	/* Now do it */

	for(id=0; id<NUMBER_SHUTTERS; id++)
	{
		/* Send the right command */

		switch (shutter_conf.conf[config_id][id])
		{
		  case CLOSED: message(system_window,"Closing shutter %s",
				       shutter_types[id]);
			       if (!send_shutter_close(id))
			               error(ERROR,"Failed to close shutter %s",
				       shutter_types[id]);
			       delay = TRUE;
			       break;

		  case OPEN: message(system_window,"Opening shutter %s",
				       shutter_types[id]);
			     if (!send_shutter_open(id))
				       error(ERROR,"Failed to open shutter %s",
				       shutter_types[id]);
			     delay = TRUE;
			     break;

		  default: delay = FALSE;
			   break;
		}

		/* Do a small delay if we have to */

		if (delay)
		{
			usleep(100);
		}
	}

	werase(system_window);
	wrefresh(system_window);

	return NOERROR;

} /* shutter_config() */

/************************************************************************/
/* alternate_ir_shutters()						*/
/* 									*/
/* Will alternate between IRB5 and IRB6 for a given number of seconds.	*/
/************************************************************************/

int alternate_ir_shutters(int argc, char **argv)
{
	int	seconds;
        time_t   start;
	int	irb5, irb6;
	bool 	loop;
	char	s[256];

	/* Check out the command line */

        if (argc > 1)
        {
                sscanf(argv[1],"%d",&seconds);
        }
        else
        {
		sprintf(s,"     2");
                if (quick_edit("TIme on each beam (S)",
                        s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

                sscanf(s,"%d",&seconds);
        }

	if ((irb5 = shutter_id("IRB5")) < 0)
		return error(ERROR,"Failed to identify IRB5");
	if ((irb6 = shutter_id("IRB6")) < 0)
		return error(ERROR,"Failed to identify IRB6");

	/* Go into main loop */

	loop = TRUE;
	while(loop)
	{
		/* Go to just Beam 5 */

	       if (!send_shutter_close(irb6)) 
			error(ERROR,"Failed to close shutter IRB6");
	       if (!send_shutter_open(irb5)) 
			error(ERROR,"Failed to open shutter IRB5");
		message(system_window,"BEAM 5");

		/* Now wait for the right number of seconds */

		for (start = time(NULL); time(NULL) < start+seconds;)
		{
			background();

			if (kbhit() && get_command() == KEY_ESC)
			{
				loop = FALSE;
				break;
			}
		}
		if (!loop) continue;
		
		/* Go to just Beam 6 */

	       if (!send_shutter_close(irb5)) 
			error(ERROR,"Failed to close shutter IRB5");
	       if (!send_shutter_open(irb6)) 
			error(ERROR,"Failed to open shutter IRB6");
		message(system_window,"BEAM 6");

		/* Now wait for the right number of seconds */

		for (start = time(NULL); time(NULL) < start+seconds;)
		{
			background();

			if (kbhit() && get_command() == KEY_ESC)
			{
				loop = FALSE;
				break;
			}
		}
	}

	/* Open both shutters */
		
	if (!send_shutter_open(irb5)) 
		error(ERROR,"Failed to open shutter IRB5");
        if (!send_shutter_open(irb6)) 
		error(ERROR,"Failed to open shutter IRB6");
	message(system_window,"BOTH IR BEAMS OPEN");

	return NOERROR;

} /* alternate_ir_shutters() */

/************************************************************************/
/* call_send_ping_shut()                                                */
/*                                                                      */
/* Sends a PING message to the server.                                  */
/************************************************************************/

int call_send_ping_shut(int argc, char **argv)
{
        return send_ping(shut_server);

} /* call_int send_ping_shut() */
