/************************************************************************/
/* uizaber_messages.c							*/
/*                                                                      */
/* Message handling routines.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../include/chara_messages.h"
#include "../../sockman/sockman.h"
#include "../../cliserv/include/cliserv.h"
#include "../../cliserv/uiclient/uiclient.h"
#include "uizaber.h"

struct SZABER_LIST zaber_list[MAX_SERVERS];
static int zaber_position[MAX_SERVERS][MAX_ZABER];
static bool zaber_position_changed[MAX_SERVERS][MAX_ZABER];
static bool bypass_zaber = FALSE;

/************************************************************************/
/* set_bypass_zaber()							*/
/*									*/
/************************************************************************/

void set_bypass_zaber(bool bypass)
{
	bypass_zaber = bypass;

} /* set_bypass_zaber() */

/************************************************************************/
/* open_zaber_server()							*/
/*									*/
/* Returns server number on exit or					*/
/* -1 if fails to open server.						*/
/************************************************************************/

int open_zaber_server(char *name)
{
	int server;
	struct smessage mess;
        time_t   start;
	int	i,j;

	if (bypass_zaber) return 0;

        /* Attempt to open the socket */

        if ((server = open_server_by_name(name)) == -1)
        {
		return -1;
        }

	/* Initialize position array */

	for(i=0; i<MAX_SERVERS; i++)
	for(j=0; j<MAX_ZABER; j++)
	{
		zaber_position[i][j] = 0;
		zaber_position_changed[i][j] = FALSE;
	}

        /* Setup up local message servers */

        if (!set_up_standard_jobs(server)) 
	{
		close_server_socket(server);
		return -2;
	}

	/* Local jobs */

        if (!add_message_job(server, ZABER_LIST, message_zaber_list)) 
	{
		close_server_socket(server);
		return -3;
	}

        if (!add_message_job(server, ZABER_UNIT_POSITION, 
		message_zaber_unit_position)) 
	{
		close_server_socket(server);
		return -3;
	}

        /* Send out requests for machine name and port names */

        mess.type = ZABER_GET_LIST;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
        {
		close_server_socket(server);
		return -4;
        }

        /* Now let's wait a while and see if we get an answer. */

        start = time(NULL);
        zaber_list[server].n = 0;
        while(time(NULL) < start+TIME_OUT && zaber_list[server].n == 0)
        {
                process_server_socket(server);
        }

        if (zaber_list[server].n == 0)
        {
		close_server_socket(server);
		return -5;
        }

	/* Now get current positions */

        mess.type = ZABER_ALL_UNIT_POSITION;
        mess.data = NULL;
        mess.length = 0;

        if (!send_message(server, &mess))
        {
		close_server_socket(server);
		return -4;
        }
	
	return server;

} /* open_zaber_server() */

/************************************************************************/
/* message_zaber_list()							*/
/*									*/
/************************************************************************/

int message_zaber_list(int server, struct smessage *mess)
{
	unsigned char *p1,*p2;
	int i;

	if (bypass_zaber) return TRUE;

	/* Is the length right? */

	if (mess->length != sizeof(zaber_list[server]))
	{
		error(ERROR,"Zaber list incorrect size.");
		return FALSE;
	}

	/* OK, copy the data over to the real list */

	p1 = mess->data;
	p2 = (unsigned char *)&(zaber_list[server]);
	for(i=0; i<sizeof(zaber_list[server]); i++) *p2++ = *p1++;

	return TRUE;

} /* message_zaber_list() */

/************************************************************************/
/* message_zaber_unit_position()					*/
/*									*/
/************************************************************************/

int message_zaber_unit_position(int server, struct smessage *mess)
{
	struct s_zaber_move_unit *move;
	int	zaber;

	if (bypass_zaber) return TRUE;

	/* Is the length right? */

	if (mess->length != sizeof(*move))
	{
		error(ERROR,"Zaber move incorrect size.");
		return FALSE;
	}

	move = (struct s_zaber_move_unit *)mess->data;

	for(zaber = 0; zaber < zaber_list[server].n; zaber++)
	{
		if (move->unit == zaber_list[server].zabers[zaber].axis_one)
			message(system_window,
			"Zaber %s unit %s axis one moved to %d.",
				zaber_list[server].name,
				zaber_list[server].zabers[zaber].name,
				move->position);
		else if (move->unit==zaber_list[server].zabers[zaber].axis_two)
			message(system_window,
			"Zaber %s unit %s axis two moved to %d.",
				zaber_list[server].name,
				zaber_list[server].zabers[zaber].name,
				move->position);
	}

	zaber_position[server][move->unit] = move->position;
	zaber_position_changed[server][move->unit] = TRUE;

	return TRUE;

} /* message_zaber_unit_position() */

/************************************************************************/
/* zaber_get_units_by_name()						*/
/*									*/
/* Goes through zaber list and tried to identify axis by name.		*/
/* Returns TURE if all goes well.					*/
/************************************************************************/

bool zaber_get_units_by_name(int server, 
			     char *name, 
			     int *axis_one, 
			     int *axis_two)
{
	int	i;

	if (bypass_zaber)
	{
		*axis_one = 1;
		*axis_two = 2;
		return TRUE;
	}

	for(i=0; 
	    i<zaber_list[server].n &&
		strcmpi(name, zaber_list[server].zabers[i].name) != 0;
	    i++);

	if (i >= zaber_list[server].n) return FALSE;
	
	*axis_one = zaber_list[server].zabers[i].axis_one;
	*axis_two = zaber_list[server].zabers[i].axis_two;

	return TRUE;

} /* zaber_get_units_by_name() */

/************************************************************************/
/* zaber_move_unit_absolute()						*/
/*									*/
/* Returns TRUE if it goes well.					*/
/************************************************************************/

bool zaber_move_unit_absolute(int server, int unit, int position)
{
	struct smessage mess;
	struct s_zaber_move_unit move;

	if (bypass_zaber)
	{
		zaber_position[server][unit] = position;
		zaber_position_changed[server][unit] = TRUE;
		return TRUE;
	}

	move.unit = unit;
	move.position = position;

        mess.type = ZABER_MOVE_ABSOLUTE;
        mess.length = sizeof(move);
        mess.data = (unsigned char *)&move;

        if (!send_message(server, &mess))
        {
		close_server_socket(server);
		return FALSE;
        }

	return TRUE;

} /* zaber_move_unit_absolute() */

/************************************************************************/
/* zaber_move_unit_relative()						*/
/*									*/
/* Returns TRUE if it goes well.					*/
/************************************************************************/

bool zaber_move_unit_relative(int server, int unit, int position)
{
	struct smessage mess;
	struct s_zaber_move_unit move;

	if (bypass_zaber)
	{
		zaber_position[server][unit] += position;
		zaber_position_changed[server][unit] = TRUE;
		return TRUE;
	}

	move.unit = unit;
	move.position = position;

        mess.type = ZABER_MOVE_RELATIVE;
        mess.length = sizeof(move);
        mess.data = (unsigned char *)&move;

        if (!send_message(server, &mess))
        {
		close_server_socket(server);
		return FALSE;
        }

	return TRUE;

} /* zaber_move_unit_relative() */

/************************************************************************/
/* zaber_move_set_point()						*/
/*									*/
/* Returns TRUE if it goes well.					*/
/************************************************************************/

bool zaber_move_set_point(int server, int unit)
{
	struct smessage mess;

	if (bypass_zaber) return zaber_move_unit_absolute(server, unit, 0);

        mess.type = ZABER_SET_POINT;
        mess.length = sizeof(unit);
        mess.data = (unsigned char *)&unit;

        if (!send_message(server, &mess))
        {
		close_server_socket(server);
		return FALSE;
        }

	return TRUE;

} /* zaber_move_set_point() */

/************************************************************************/
/* zaber_move_home()							*/
/*									*/
/* Returns TRUE if it goes well.					*/
/************************************************************************/

bool zaber_move_home(int server, int unit)
{
	struct smessage mess;

	if (bypass_zaber) return zaber_move_unit_absolute(server, unit, 0);

        mess.type = ZABER_GO_HOME;
        mess.length = sizeof(unit);
        mess.data = (unsigned char *)&unit;

        if (!send_message(server, &mess))
        {
		close_server_socket(server);
		return FALSE;
        }

	return TRUE;

} /* zaber_move_home() */

/************************************************************************/
/* zaber_init()								*/
/*									*/
/* Returns TRUE if it goes well.					*/
/************************************************************************/

bool zaber_init(int server)
{
	struct smessage mess;
	int	i;

	if (bypass_zaber)
	{
	    for(i=1; i<=MAX_ZABER; i++) zaber_move_unit_absolute(server, i, 0);
	    return TRUE;
	}

        mess.type = ZABER_INIT;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
        {
		close_server_socket(server);
		return FALSE;
        }

	return TRUE;

} /* zaber_init() */

/************************************************************************/
/* zaber_move_unit_absolute_wait()					*/
/*									*/
/* Calls zaber_move_unit_absolute_wait() but waits for move to happen.	*/
/************************************************************************/

bool zaber_move_unit_absolute_wait(int server, int unit, int position)
{
	time_t	start;

	zaber_position_changed[server][unit] = FALSE;

	if (!zaber_move_unit_absolute(server, unit, position))
		return FALSE;

	start = time(0);
	while(!zaber_position_changed[server][unit])
	{
                process_server_socket(server);
		if (time(0) > start+5) return FALSE;
	}

	return TRUE;

} /* zaber_move_unit_absolute_wait() */

/************************************************************************/
/* zaber_move_unit_relative_wait()					*/
/*									*/
/* Calls zaber_move_unit_relative_wait() but waits for move to happen.	*/
/************************************************************************/

bool zaber_move_unit_relative_wait(int server, int unit, int position)
{
	time_t	start;

	zaber_position_changed[server][unit] = FALSE;

	if (!zaber_move_unit_relative(server, unit, position))
		return FALSE;

	start = time(0);
	while(!zaber_position_changed[server][unit])
	{
		process_server_socket(server);
		if (time(0) > start+5) return FALSE;
	}

	return TRUE;

} /* zaber_move_unit_relative_wait() */

/************************************************************************/
/* get_zaber_position()							*/
/*									*/
/* Returns TRUE if it goes well.					*/
/* Return current position.						*/
/************************************************************************/

int get_zaber_position(int server, int unit)
{
	return (zaber_position[server][unit]);

} /* get_zaber_position() */
