/************************************************************************/
/* status_window.c 	                                                */
/*                                                                      */
/* Displays the UI status in a text window.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 2002                                                  */
/************************************************************************/
#define USE_GTK
#include "../include/cliserv.h"
#include "../../include/chara_messages.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include "../../charalib/chara.h"
#include "../../slalib/slalib.h"

/* Size of label for a single character */

#ifdef GTK2
#define LABEL_Y	14
#define LABEL_X	9
#else
#define LABEL_Y	14
#define LABEL_X	9
#endif

/* Globals */

static int start_has_been_run = FALSE;
static int ui_status_window_up = FALSE;
static GtkWidget *status_window;
static GtkWidget *status_labels[STATUS_LINES][STATUS_COLS];
static guint status_background_id;
static struct s_status *status;
static int status_server = -1;
static GtkWidget *ui_status_fixed = NULL;
static void (*ui_status_callback)(void) = NULL;

/************************************************************************/
/* create_status_window()						*/
/*									*/
/* Routine for creating the clock control window. This should be self	*/
/* contained and give you all you need to control the basic clock 	*/
/* functionality. Argumentis the window name.				*/
/* The data argument is a name to put on the window.			*/
/************************************************************************/

GtkWidget* create_status_window (GtkButton *button, gpointer *data)
{
	GtkWidget *ok_button;
	struct s_status_window *info;

	/* Do nothing if we exist already */

	if (ui_status_fixed != NULL) return status_window;

	/* Make sure we know what's going on */

        info = (struct s_status_window *)data;

	/* Create a new window */

	status_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (status_window), info->name);
	gtk_container_set_border_width (GTK_CONTAINER (status_window), 1);

	/* Fill it with the right things */

	fill_ui_status_page((GtkWidget *)status_window);

	/* OK button */

	ok_button = gtk_button_new_with_label ("OK");
	gtk_signal_connect(GTK_OBJECT (ok_button), "clicked",
	                    GTK_SIGNAL_FUNC (status_ok_callback), NULL);
	gtk_widget_ref(ok_button);
	gtk_widget_show (ok_button);
	gtk_fixed_put (GTK_FIXED (ui_status_fixed), ok_button, 0, 
			STATUS_LINES*LABEL_Y);
	gtk_widget_set_uposition (ok_button, 0, STATUS_LINES*LABEL_Y);
	gtk_widget_set_usize (ok_button, STATUS_COLS*LABEL_X, 1.5*LABEL_Y);

	/* Start routine for background processing */

	start_ui_status(button, (gpointer *)&(info->server));

	/* That should be all */

	ui_status_window_up = TRUE;
	return status_window;

} /* create_status_window() */

/************************************************************************/
/* status_ok_callback()							*/
/*									*/
/* Callback function for clock control OK button. Stop background	*/
/* processing and destroy clock control window.				*/
/************************************************************************/

void status_ok_callback(GtkButton *button, gpointer user_data)
{
	if (!ui_status_window_up) return;

	gtk_widget_destroy(status_window);
	stop_ui_status(button, user_data);
	ui_status_window_up = FALSE;
	ui_status_fixed = NULL;

} /* status_ok_callback() */

/************************************************************************/
/* status_background()							*/
/*									*/
/* Here's the background job for looking at clock outputs and so on.	*/
/************************************************************************/

gint status_background(gpointer data)
{
        struct smessage mess;

	/* Send message requesting status information */

	mess.type = UI_STATUS;
	mess.length = 0;
	mess.data = NULL;

	if (!send_message(status_server, &mess))
	{
		if (ui_status_window_up)
			status_ok_callback(NULL, NULL);
		else
			stop_ui_status(NULL, NULL);
	}

	return TRUE;

} /* status_background() */

/************************************************************************/
/* message_status()							*/
/*									*/
/* Reacts to a STATUS message comming from the server. Does nothing	*/
/* if we're not in status mode. Display text if we are.			*/
/************************************************************************/

int message_status(int server, struct smessage *mess)
{
	int 	i,j;
	char	s[2];

	/* If the winodw isn't open do nothing */

	if (ui_status_fixed == NULL) return NOERROR;
	
	/* Otherwise check that all is OK with the data */

	if (mess->length != sizeof(*status))
	{
		if (ui_status_window_up)
			status_ok_callback(NULL, NULL);
		else
			stop_ui_status(NULL, NULL);
		return NOERROR;
	}

	status = (struct s_status *) mess->data;

	/* Put the text in place */

	s[1] = 0;
        for (j=0; j<STATUS_LINES ; j++)
        for (i=0; i<STATUS_COLS ; i++)
        {
		s[0] = status->line[j][i];
		gtk_label_set_text((GtkLabel *)status_labels[j][i], s);
        }

	/* If there is a callback, use it */

	if (ui_status_callback != NULL) (*ui_status_callback)();

	return NOERROR;

} /* message_status() */

/************************************************************************/
/* fill_ui_status_page()						*/
/*									*/
/* Fill a container with what we need for status information.		*/
/************************************************************************/

void fill_ui_status_page(GtkWidget *container)
{
	int	i,j;

	/* Create a fixed area to put things in */

	ui_status_fixed = gtk_fixed_new ();
	gtk_widget_ref (ui_status_fixed);
	gtk_object_set_data_full (GTK_OBJECT (container), "fixed", 
			ui_status_fixed,
                        (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add (GTK_CONTAINER (container), ui_status_fixed);

        /* Add labels for text */

	for(j=0; j < STATUS_LINES; j++)
	for(i=0; i < STATUS_COLS; i++)
	{
        	status_labels[j][i] = gtk_label_new(" ");
		gtk_widget_ref(status_labels[j][i]);
		gtk_object_set_data_full (GTK_OBJECT (container), "X", 
			status_labels[j][i], (GtkDestroyNotify) 
			gtk_widget_unref);
		gtk_widget_show (status_labels[j][i]);
		gtk_fixed_put (GTK_FIXED(ui_status_fixed), status_labels[j][i], 
			i*LABEL_X, j*LABEL_Y);
		gtk_widget_set_usize (status_labels[j][i], 
			LABEL_X, LABEL_Y);
	}

	gtk_widget_show (ui_status_fixed);
	gtk_widget_show(container);

} /* fill_ui_status_page() */

/************************************************************************/
/* start_ui_status()							*/
/*									*/
/* Sets up background job we need to have a status window.		*/
/* Data points to integer giving the server name to use.		*/
/************************************************************************/

void start_ui_status(GtkButton *button, gpointer *data)
{
	/* Get the server */

	status_server = *((int *)data);

        /* Start routine for background processing */

        status_background_id = gtk_timeout_add(1000,
                                status_background, NULL);

        /* Run for the first time */

        status_background(NULL);

	start_has_been_run = TRUE;

} /* start_ui_status() */

/************************************************************************/
/* stop_ui_status()							*/
/*									*/
/* Removes the background job.						*/
/************************************************************************/

void stop_ui_status(GtkButton *button, gpointer *data)
{
	if (!start_has_been_run) return;

	gtk_timeout_remove(status_background_id);

} /* stop_ui_status() */

/************************************************************************/
/* set_ui_status_callback()						*/
/*									*/
/* Setup a callback function for when a status message is received.	*/
/* SHould be sset to NULL to turn it off.				*/
/* Removes the background job.						*/
/************************************************************************/

void set_ui_status_callback(void (*callback)(void))
{

	ui_status_callback = callback;

} /* set_ui_status_callback() */
