/************************************************************************/
/* error.c								*/
/*                                                                      */
/* Error message handling for the entire system. One function only.	*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY SERVER LIB   				*/
/*                 Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : Oct 2001							*/
/************************************************************************/

#define SERVER

/*
 * Include files
 */

#include "../include/cliserv.h"
#include "../../include/chara_messages.h"
#include <time.h>

static FILE *logfile = NULL;

/************************************************************************/
/* error()								*/
/*									*/
/* Handles all error messages. Errors sent to the logfile or stderr.	*/
/* If it's fatal we shut things down asap.				*/
/* Returns the error level it recieves.					*/
/************************************************************************/

int error(int level, char *fmt, ...)
{
	time_t	time_now;
	char	*now;
	char	err_string[9999];/* Contains sprinf formatted message */
	va_list	args;
	FILE	*output;

	/* do nothing if NOERROR */

	if (level == NOERROR) return level;

	/* Create the whole error string */

        va_start(args, fmt);
	vsprintf(err_string, fmt, args);

	/* Create a string for the time */

	time_now = time(NULL);
	now = asctime(localtime(&time_now));
	now[strlen(now)-1] = 0;

	/* Where do we send this? */

	if (logfile != NULL)
		output = logfile;
	else
		output = stderr;

	/* React to it */

	if (level == FATAL)
	{
		system("stty sane");
		fprintf(output,"A FATAL ERROR HAS OCCURED AT %s SHUT DOWN\n",
			now);
		fprintf(output,"\n");
		fprintf(output, err_string);
		close_server(0);
	}
	else if (level == ERROR)
	{
		fprintf(output,"%s ERROR: %s\n",now,err_string);
		if (active_socket != -1) 
			server_print_message(active_socket,
				"%s ERROR: %s",now,err_string);
	}
	else if (level == WARNING)
	{
		fprintf(output,"%s WARNING: %s\n",now,err_string);
		if (active_socket != -1) 
			server_print_message(active_socket,
				"%s WARNING: %s",now,err_string);
	}
	else if (level == MESSAGE)
	{
		fprintf(output,"%s MESSAGE: %s\n",now,err_string);
	}
	else
	{
		fprintf(output,"%s UNKNOWN ERROR: %s\n",now,err_string);
	}
	
	fflush(output);
	return level;
	
} /* error() */

/************************************************************************/
/* open_server_logfile()						*/
/*									*/
/* Opens a logfile for server messages. If the logfile is open all 	*/
/* messages will be appended to it.					*/
/* Returns TRUE if all went well, and FALSE otherwise.			*/
/************************************************************************/

bool open_server_logfile(char *logfilename)
{
	/* Close the old file if it is open */

	close_server_logfile();

	/* Try and open it in append mode */

	if ((logfile = fopen(logfilename, "a")) == NULL)
	{
		error(ERROR,"Failed to open logfile %s.", logfilename);
		logfile = NULL;
		return FALSE;
	}

	return TRUE;

} /* open_server_logfile() */

/************************************************************************/
/* close_server_logfile()						*/
/*									*/
/* Closes a logfile for server messages. 				*/
/************************************************************************/

void close_server_logfile(void)
{
	if (logfile == NULL) return;

	fclose(logfile);

} /* close_server_logfile() */
