/************************************************************************/
/* clock.c	 							*/
/*                                                                      */
/* Controller application routines for the CHARA real time clock.	*/
/* This version is for the 2.6 kernels and uses the cbdriver.ko module.	*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Oct 2009                                                    */
/************************************************************************/

#ifdef __CHARAUI__
#undef __CHARAUI__
#include "../charaui/src/charaui.h"
#else
#define SERVER
#include "../cliserv/include/cliserv.h"
#endif
#include "clock.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include <pthread.h>
#include "../../modules/cbdriver/cbdriver.h"
#include "../slalib/slalib.h"
#include "../charalib/chara.h"

/* Globals */

static FILE *cbfp = NULL;		/* File pointer for clock driver. */
static int max_clock_offset=32000;
static void (*clock_offset_callback)(void) = NULL;
static bool bypass_clock_driver = FALSE;
static int  bypass_clock_delay = DEFAULT_BYPASS_CLOCK_DELAY;

/*
 * Offset for setting the time in mS
 */

#define SYNC_OFFSET 15

/************************************************************************/
/* set_bypass_clock_delay()						*/
/*									*/
/* Sets global bypass_clock_delay.					*/
/* Returns an error level.						*/
/************************************************************************/

int set_bypass_clock_delay(int delay)
{
	if (cbfp != NULL)
	{
	    return error(ERROR,
		"The real clock driver is already going. Please close it.");
	}
	
	bypass_clock_delay = delay;

	return NOERROR;

} /* set_bypass_clock_delay() */


/************************************************************************/
/* set_bypass_clock_driver()						*/
/*									*/
/* Allows user to set local global bypass_clock_driver.			*/
/* If set to true, all functionality is there, well mostly, but the	*/
/* time is based on the local linux clock. This should make thins	*/
/* mostly transparent to the user program.				*/
/* Returns an error level.						*/
/************************************************************************/

int set_bypass_clock_driver(bool bypass)
{
	if (cbfp != NULL)
	{
	    return error(ERROR,
		"The real clock driver is already going. Please close it.");
	}
	
	bypass_clock_driver = bypass;

	return NOERROR;

} /* set_bypass_clock_driver() */

/************************************************************************/
/* open_clock_device()							*/
/* 									*/
/* OPens clock device. Returns TRUE if all went well, FALSE otherwise.	*/
/************************************************************************/

int open_clock_device(void)
{
	if (bypass_clock_driver)
	{
		return TRUE;
	}

	if ((cbfp = fopen(CB_NODE_NAME, "rb")) == NULL) 
	{
		return FALSE;
	}

	return TRUE;

} /* open_clock_device() */

/************************************************************************/
/* close_clock_device()							*/
/* 									*/
/* CLoses clock device.							*/
/************************************************************************/

void close_clock_device(void)
{
	if (bypass_clock_driver) return;

	if (cbfp == NULL) return;

	fclose(cbfp);

	cbfp = NULL;

} /* close_clock_device() */

/************************************************************************/
/* clear_lost_counts()						        */
/*									*/
/* Uses RT system command CLEAR_LOST_COUNTS to set lost_ticks=0 and	*/
/* lost_seconds = 0. Normally only used after you have set the clock.	*/
/************************************************************************/

int clear_lost_counts(int argc, char **argv)
{
	if (bypass_clock_driver) return NOERROR;

	if (cbfp == NULL) return error(ERROR,"Clock device is not open.");

	if (ioctl(fileno(cbfp), CBDRIVER_CLEAR) != 0)
	{
		return error(ERROR,"Failed to clear lost counts: %s",
			strerror(errno));
	}

	return NOERROR;

} /* clear_lost_counts() */

/************************************************************************/
/* get_rt_time()							*/
/*									*/
/* Uses an ioctl() call to get teh current time.			*/
/* Returns 0 on succes and -1 on failure.  				*/
/************************************************************************/

int get_rt_time(rt_time_struct *the_time)
{
	struct timeval time_now;
	struct timezone zone;
	time_t current_time;
	static long long int	last_run_us = 0;
	long long int	this_run_us = 0;

	if (bypass_clock_driver)
	{
		/* What is the time now in uS? */

		gettimeofday(&time_now, &zone);
		time(&current_time);
		this_run_us = ((long long int)1000000)*
				((long long int)current_time%86400) + 
			      	((long long int)time_now.tv_usec);

		/* Turn that into TICKS */

		the_time->time_now = (CHARA_TIME)(this_run_us /
					(1000000/TICKS_PER_SECOND));
		the_time->lost_ticks = 0;
		the_time->lost_seconds = 0;

		/* We shouldn't do this too often */

		if (this_run_us - last_run_us < bypass_clock_delay)
		{
			usleep(bypass_clock_delay);
		}
		
		last_run_us = this_run_us;

		return 0;
	}

	if (cbfp == NULL) return error(ERROR,"Clock device is not open.");

	if (fread(the_time, sizeof(rt_time_struct), 1, cbfp) == 0) return -1;

	return 0;

} /* get_rt_time() */

/************************************************************************/
/* chara_time_now()							*/
/*									*/
/* Replacement for the old mbuff shared memory global pointer. Returns	*/
/* time now or 0 on ERROR.						*/
/************************************************************************/

CHARA_TIME chara_time_now(void)
{
	rt_time_struct the_time;

	if (get_rt_time(&the_time) != 0) return 0;

	return the_time.time_now;

} /* chara_time_now() */

/************************************************************************/
/* get_time_next_second()						*/
/*									*/
/* Uses one of the standard commands to get the	time of the next 	*/
/* second. Returns this time or -1 on error.				*/
/* If wait is TRUE will wait for it to change once first before 	*/
/* it returns to make sure it will be valid for a while. Times out      */
/* after 3 seconds.							*/
/************************************************************************/

CHARA_TIME get_time_next_second(int wait)
{
	rt_time_struct	the_time;
	CHARA_TIME	time_next_second;
	CHARA_TIME	last_time_next_second;
	time_t		stop_time;
	time_t		now;

	if (!bypass_clock_driver && cbfp == NULL)
		return error(ERROR,"Clock device is not open.");

	/* Work out the time of the next second */

	if (get_rt_time(&the_time) != 0) return -1;
	last_time_next_second = TICKS_PER_SECOND *
				(the_time.time_now/TICKS_PER_SECOND + 1);

	/* Return now if that's what the user wanted */

	if (!wait) return last_time_next_second;

	/* Now try for a few seconds to get the next one */

	time(&stop_time);
	stop_time += 3;
	do
	{
		if (get_rt_time(&the_time) != 0) return -1;
		time_next_second = TICKS_PER_SECOND *
				(the_time.time_now/TICKS_PER_SECOND + 1);
		time(&now);

#ifdef __CHARAUI__
		background();
#endif

	} while(now < stop_time && last_time_next_second==time_next_second);

	if (now >= stop_time) return -1;

	return time_next_second;

} /* get_time_next_second() */

/************************************************************************/
/* set_rt_time()							*/
/*									*/
/* Uses one of the standard ioctl calls to send a value of type 	*/
/* CHARA_TIME to the driver. This value will become the value of the 	*/
/* drivers internal varriable time_next_second. When the next second	*/
/* tick occurs, or is detected to have been missed, the time is set to 	*/
/* this value. Warning.... race conditions can occurr, this should be 	*/
/* well before the next second tick is expected.			*/
/* Returns TRUE is all went OK, FALSE otherwise.			*/
/************************************************************************/

int set_rt_time(CHARA_TIME time_next_second)
{
	if (bypass_clock_driver) return TRUE;

	if (cbfp == NULL) return error(ERROR,"Clock device is not open.");

	if (ioctl(fileno(cbfp), CBDRIVER_SET_TIME, 
		(unsigned long int)time_next_second) != 0)
	{
		return FALSE;
	}

	return TRUE;

} /* set_rt_time() */

/************************************************************************/
/* rt_to_ap_time()							*/
/*									*/
/* Converts RT process time into an application time structure.		*/
/************************************************************************/

struct AP_TIME_STRUCT rt_to_ap_time(CHARA_TIME rt_time)
{
	ap_time_struct res;

	if (rt_time < 0)
	{
		rt_time *= (res.sign = -1);
	}
	else
	{
		res.sign = 1;
	}
		
	res.tick = rt_time % TICKS_PER_SECOND;
	rt_time -= res.tick;
	rt_time /= TICKS_PER_SECOND;

	res.sec = rt_time % 60;
	rt_time -= res.sec;
	rt_time /= 60;

	res.min = rt_time % 60;
	rt_time -= res.min;

	res.hrs = rt_time / 60;

	return res;

} /* rt_to_ap_time() */

/************************************************************************/
/* ap_to_rt_time()							*/
/*									*/
/* Converts an application time structure into a RT process time .	*/
/************************************************************************/

CHARA_TIME ap_to_rt_time(struct AP_TIME_STRUCT t)
{
	CHARA_TIME	res;

	res = t.tick + TICKS_PER_SECOND * (t.sec + 60 * (t.min + 60 * t.hrs));
	res *= t.sign;

	return res;

} /* ap_to_rt_time() */

/************************************************************************/
/* str_to_ap_time()							*/
/*									*/
/* Converts a string of the form [+-]HR:MN:SC into an application time	*/
/* structure.								*/
/************************************************************************/

struct AP_TIME_STRUCT str_to_ap_time(char *s)
{
	struct AP_TIME_STRUCT t;

	/* What is the sign? */

	while(*s == ' ' || *s == '\t') s++;
	if (*s == '-')
	{
		/* It's negative */

		t.sign = -1;
		*s = ' ';
	}
	else
	{
		/* It's positive */

		t.sign = 1;
	}

	switch(sscanf(s,"%d:%d:%d",&t.hrs,&t.min,&t.sec))
	{

		case 0: t.hrs = 0;

		case 1: t.min = 0;

		case 2: t.sec = 0;

		default: t.tick = 0;
			 break;
	}

	return t;
		
} /* str_to_ap_time() */

/************************************************************************/
/* ap_to_str_time()							*/
/*									*/
/* Converts a string of the form [+-]HR:MN:SC into an application time	*/
/* structure.								*/
/************************************************************************/

char *ap_to_str_time(char *s, struct AP_TIME_STRUCT t)
{
	/* What is the sign? */

	if (t.sign == -1)
	{
		/* It's negative */

		strcpy(s,"-");
	}
	else
	{
		/* It's positive */

		strcpy(s," ");
	}

	/* Print it */

	sprintf(s+1,"%02d:%02d:%02d",t.hrs,t.min,t.sec);

	return s;
		
} /* ap_to_str_time() */

/************************************************************************/
/* set_time()	 							*/
/*									*/
/* Chara UI callable function.						*/
/* An interface to the set_rt_time command.				*/
/************************************************************************/

#ifdef __CHARAUI__
int	set_time(int argc, char **argv)
{
	CHARA_TIME	time_next_second;
	ap_time_struct	new_time;
	char	str[80];
	char	*str_time;
        rt_time_struct now;
 
	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

	/* OK let's sort out the arguments */

	werase(system_window);
	wrefresh(system_window);
	if (argc > 1)
	{
		str_time = argv[1];
	}
	else
	{
		if (get_rt_time(&now) != 0) return ERROR;
		new_time = rt_to_ap_time(now.time_now);
		ap_to_str_time(str,new_time);
		if (quick_edit("Time at next second",str,str,NULL,STRING) == 
			KEY_ESC) return NOERROR;
		str_time = str;
	}

	new_time = str_to_ap_time(str_time);
	time_next_second = ap_to_rt_time(new_time);

	if (!set_rt_time(time_next_second))
	{
		return error(WARNING,"Failed to set time of next second.");
	}

	/* Wait for this to happen */

	sleep(1);

	/* Make sure linux clock is OK */

        return NOERROR;

} /* set_time() */
#endif

/************************************************************************/
/* sync_times()								*/
/*									*/
/* Attempts to set the time using the RT time based on RT clock.	*/
/* Uses the setgmt program. Note that this can mess up the local clock! */
/************************************************************************/

#ifdef __CHARAUI__
int sync_times(int argc, char **argv)
#else
int sync_times(void)
#endif
{
	CHARA_TIME time_next_second;
	struct AP_TIME_STRUCT ap_time_next_second;
	char	command[81];
	rt_time_struct the_time;
 
	if (bypass_clock_driver)
	{
#ifdef __CHARAUI__
		message(system_window,
			"We are using the Linux clock. Synching is redundant.");
#endif
		return NOERROR;
	}

#ifdef __CHARAUI__
	message(system_window,"Attempting to sync clocks... please wait.");
#else
	error(MESSAGE,"Attempting to sync clocks... please wait.");
#endif

	/* Find out the time of the next second */

	if ((time_next_second = get_time_next_second(TRUE)) == -1)
	{
#ifdef __CHARAUI__
		werase(system_window);
		wrefresh(system_window);
#endif
		return error(ERROR,"Failed to get time of next second.");
	}

	/* Setup the command we will need to set the Linux Clock */

	ap_time_next_second = rt_to_ap_time(time_next_second);
	sprintf(command,"setgmt %d %d %d",
		ap_time_next_second.sign * ap_time_next_second.hrs,
		ap_time_next_second.min, ap_time_next_second.sec);

	/* Wait until the time comes ... with OFFSET */

	do
	{
		if (get_rt_time(&the_time) == -1)
		{
#ifdef __CHARAUI__
			werase(system_window);
			wrefresh(system_window);
#endif
			return error(ERROR,"Failed to get RT time.");
		}
	} while(the_time.time_now < (time_next_second -  SYNC_OFFSET));

	system(command);
			
	/* Now make sure the background still works */

#ifdef __CHARAUI__
	reset_status_window_refresh(0, NULL);

	/* That is all */

	werase(system_window);
	wrefresh(system_window);
#endif
	return NOERROR;

} /* sync_times() */

/************************************************************************/
/* show_time()                                                          */
/*									*/
/* Display the time in the system_window. Mostly so a remote user can   */
/* see what the time is.                                                */
/************************************************************************/

#ifdef __CHARAUI__
int show_time(int argc, char **argv)
{
        rt_time_struct now;
        ap_time_struct t;
	time_t current_time;
	struct tm *loc_now;
	struct timeval time_now;
	struct timezone zone;
	int tdiff;
	int tlast=0,tnow;
	int last_sec = 0;
	int sec_length = 0;
	float n = -5.0,mean_sec = 0.0;

	do
	{
		zone.tz_minuteswest = 0;
	        zone.tz_dsttime = 0;
		if (get_rt_time(&now) != 0) 
			return error(ERROR,"Failed to get the time.");

		gettimeofday(&time_now, &zone);
		time(&current_time);
		loc_now = gmtime(&current_time);
		t = rt_to_ap_time(now.time_now);
		tdiff = 1000*loc_now->tm_sec + time_now.tv_usec/1000 -
			1000*t.sec - t.tick*(1000/TICKS_PER_SECOND);
		tnow = 1000000*loc_now->tm_sec + time_now.tv_usec;

		werase(system_window);
		message(system_window,
			"%02d:%02d:%02d CHARA-LOCAL = %d mS SEC = %d (%.1f) uS",
			t.hrs,t.min,t.sec, tdiff, sec_length, mean_sec/n);
		if (t.sec != last_sec)
		{
			sec_length = tnow - tlast;
			tlast = tnow;
			if (sec_length > 0.0)
			{
				if (n >= 0.0) mean_sec += (float)sec_length;
				n += 1.0;
			}
			last_sec = t.sec;
		}

		if (kbhit() && get_command() == KEY_ESC) break;

	} while(active_socket == -1);

	werase(system_window);
	wrefresh(system_window);

	return NOERROR;

} /* show_time() */
#endif

/************************************************************************/
/* sidereal_time()                                                      */
/*                                                                      */
/* Returns current sidereal time 0-2PI.                                 */
/* Uses date on computer but reall time from CHARA clock.               */
/* If returned value is negative, something went wrong.                 */
/************************************************************************/
                                                                                
double sidereal_time(void)
{
        double  mjd;
 
        mjd = modified_julian_date();
 
        if (mjd < 0) return -1;
 
        return (slaGmst(mjd) + CHARA_LONGITUDE);
 
} /* sidereal_time() */
 
/************************************************************************/
/* modified_julian_date()                                               */
/*                                                                      */
/* Returns the modified julian date, or negative number on error.       */
/* Result is in days plus fractions of days.                            */
/* -1.0 Slalib failed.                                                  */
/* -2.0 Clock read failed.                                              */
/************************************************************************/
 
double modified_julian_date(void)
{
        struct  tm      *gmt_now;
        time_t current_time;
        int     iy,im,id,j;
        double  mjd;
        rt_time_struct now;
 
        time(&current_time);
        gmt_now = gmtime(&current_time);
 
        iy = gmt_now->tm_year + 1900;
        if (iy < 1950) iy+=100;
        im = gmt_now->tm_mon+1;
        id = gmt_now->tm_mday;
 
        slaCaldj(iy,im,id,&mjd, &j);
 
        if (j != 0) return -1.0;
        if (get_rt_time(&now) != 0) return -2.0;
 
        mjd += ((double)now.time_now/(TICKS_PER_SECOND*60.0*60.0*24.0));
 
        return mjd;
 
} /* modified_julian_date() */

/************************************************************************/
/* clock_running()							*/
/*									*/
/* CHecks to see if local clock is running. Takes 1 second to do this.	*/
/* Returns TRUE if it is and FALSE if it isn't.				*/
/************************************************************************/

bool clock_running(void)
{
        rt_time_struct start,stop;

	get_rt_time(&start);
	usleep(10000);
	get_rt_time(&stop);

	return (start.time_now != stop.time_now);

} /* clock_running() */

/************************************************************************/
/* check_clock_offset()							*/
/*									*/
/* Calculates the difference between the real time clock and the local 	*/
/* clock. If this is larger than a given value set by 			*/
/* set_max_clock_offset() the function will call a callback routine set */
/* by set_clock_offset_callback().					*/
/************************************************************************/

int check_clock_offset(void)
{
        rt_time_struct now;
        ap_time_struct t;
	time_t current_time;
	struct timeval time_now;
	struct tm *loc_now;
	struct timezone zone;
	int tdiff;

	zone.tz_minuteswest = 0;
	zone.tz_dsttime = 0;
	if (get_rt_time(&now) != 0) 
		return error(ERROR,"Failed to get the time.");
	gettimeofday(&time_now, &zone);
	time(&current_time);
	loc_now = gmtime(&current_time);
	t = rt_to_ap_time(now.time_now);
	tdiff = 1000*loc_now->tm_sec + time_now.tv_usec/1000 -
		1000*t.sec - t.tick*(1000/TICKS_PER_SECOND);
	
	if ((tdiff > max_clock_offset || -1*tdiff > max_clock_offset) && 
		clock_offset_callback != NULL)
		(*clock_offset_callback)();

	return NOERROR;

} /* check_clock_offset() */

/************************************************************************/
/* set_max_clock_offset()						*/
/*									*/
/* Set the maximum clock offset allowed by the system.			*/
/************************************************************************/

void set_max_clock_offset(int max_offset)
{
	max_clock_offset = max_offset;

} /* set_max_clock_offset */

/************************************************************************/
/* set_clock_offset_callback()						*/
/*									*/
/* Set the clock offset callback function, which will be called if the 	*/
/* system clock and the realtime clock differ by more than 		*/
/* ticks max_clock_offset.						*/
/* Set to NULL to stop using this at all.				*/
/************************************************************************/

void set_clock_offset_callback(void (*offset_callback)(void))
{
	clock_offset_callback = offset_callback;

} /* set_clock_offset_callback() */
