/************************************************************************/
/* clock_messages.c                                                     */
/*                                                                      */
/* Routines to respond to messages.					*/
/* This version is for the 2.6 kernel drivers.				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                  	*/
/* Date   : April 2002                                                  */
/************************************************************************/

#ifdef __CHARAUI__
#undef __CHARAUI__
#include "../charaui/src/charaui.h"
#else
#define SERVER
#include "../cliserv/include/cliserv.h"
#endif
#include "../include/chara_messages.h"
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "./clock.h"

/************************************************************************/
/* setup_standard_clock_messages()					*/
/*									*/
/* Gues what... this sets up the standard clock messages!.		*/
/************************************************************************/

void setup_standard_clock_messages(void)
{

#ifdef __CHARAUI__
	ui_add_message_job(MESS_CLEAR_LOST_COUNTS, clock_clear_lost_counts);
	ui_add_message_job(RT_TIME, clock_rt_time);
	ui_add_message_job(TIME_NEXT_SECOND, clock_time_next_second);
	ui_add_message_job(SET_RT_TIME, clock_set_rt_time);
	ui_add_message_job(SYNC_TIMES, clock_sync_times);
#else
	server_add_message_job(MESS_CLEAR_LOST_COUNTS, clock_clear_lost_counts);
	server_add_message_job(RT_TIME, clock_rt_time);
	server_add_message_job(TIME_NEXT_SECOND, clock_time_next_second);
	server_add_message_job(SET_RT_TIME, clock_set_rt_time);
	server_add_message_job(SYNC_TIMES, clock_sync_times);
#endif

} /* setup_standard_clock_messages() */

/************************************************************************/
/* clock_clear_lost_counts()						*/
/*									*/
/* Called by message MESS_CLEAR_LOST_COUNTS, no data fro this message.	*/
/************************************************************************/

int clock_clear_lost_counts(struct smessage *message)
{

	/* Do we have the right number of arguments? */

	if (message->length != 0)
		error(ERROR,
		"Got MESS_CLEAR_LOST_COUNTS message with data. Data ignored.");

	return clear_lost_counts(0, NULL);

} /* clock_clear_lost_counts() */

/************************************************************************/
/* clock_rt_time()							*/
/*									*/
/* Called by message RT_TIME, no data fro this message.			*/
/************************************************************************/

int clock_rt_time(struct smessage *message)
{
	struct smessage mess;
	rt_time_struct  the_time;

	/* There should be no data */

	if (message->length != 0)
		error(ERROR,
		"Got RT_TIME message with data. Data ignored.");

	/* Find out what time it is */

	if (get_rt_time(&the_time) != 0)
	{
		error(ERROR,"Failed to get time for RT side.");
	}

	/* Send back the result */

	mess.type = RT_TIME;
	mess.length = sizeof(the_time);
	mess.data = (unsigned char *)&the_time;

#ifdef __CHARAUI__
	if (ui_send_message(client_socket, &mess) != NOERROR)
#else
	if (server_send_message(active_socket, &mess) != NOERROR)
#endif
		return error(ERROR,"Failed to send RT_TIME message.");

	/* That is all */

	return NOERROR;

} /* clock_rt_time() */

/************************************************************************/
/* clock_time_next_second()						*/
/*									*/
/* Called by message RT_TIME_NEXT_SECOND, no data for this message.	*/
/************************************************************************/

int clock_time_next_second(struct smessage *message)
{
	struct smessage mess;
	CHARA_TIME  time_next_second;

	/* There should be no data */

	if (message->length != 0)
		error(ERROR,
		"Got RT_TIME_NEXT_SECOND message with data. Data ignored.");

	/* Find out what time it is */

	if ((time_next_second = get_time_next_second(FALSE)) < 0)
	{
		error(ERROR,"Failed to get time of next second.");
	}

	/* Send back the result */

	mess.type = TIME_NEXT_SECOND;
	mess.length = sizeof(time_next_second);
	mess.data = (unsigned char *)&time_next_second;

#ifdef __CHARAUI__
	if (ui_send_message(client_socket, &mess) != NOERROR)
#else
	if (server_send_message(active_socket, &mess) != NOERROR)
#endif
		return error(ERROR,"Failed ot send TIME_NEXT_SECOND message.");

	/* That is all */

	return NOERROR;

} /* clock_time_next_second() */

/************************************************************************/
/* clock_set_rt_time()							*/
/*									*/
/* Called by message SET_RT_TIME.					*/
/************************************************************************/

int clock_set_rt_time(struct smessage *message)
{
	CHARA_TIME *time_next_second;

	/* Check data */

	if (message->length != sizeof(CHARA_TIME))
	{
		return error(ERROR,
			"SET_RT_TIME message contains wrong number of bytes.");
	}

	/* Get the data out */

	time_next_second = (CHARA_TIME *)message->data;

	/* Go tell the hardware */

	if (!set_rt_time(*time_next_second))
	{
		return error(ERROR,"Failed to set RT clock.");
	}

	/* Wait for this to take hold */

	sleep(1);

	/* Make sure linux clock is OK */

        return NOERROR;

} /* clock_set_rt_time() */

/************************************************************************/
/* clock_sync_times()							*/
/*									*/
/* On message SYNC_TIMES  will call sync_times() fuinction.		*/
/************************************************************************/

int clock_sync_times(struct smessage *message)
{
	if (message->length != 0)
		error(ERROR,
		"Got RT_TIME message with data. Data ignored.");

#ifdef __CHARAUI__
	return sync_times(0,NULL);
#else
	return sync_times();
#endif

} /* clock_sync_times() */
