#ifndef CPKHDEF
#define CPKHDEF

/*
**  - - - - - -
**   c p k . h
**  - - - - - -
**
**  Macros and prototype function declarations for compact
**  pointing kernel library.
**
**  Last revision:   27 January 2008
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

#include <math.h>

/* ====== */
/* Macros */
/* ====== */

/* Radian constants */
#define D90 1.5707963267948966
#define D180 3.141592653589793
#define D360 6.283185307179586

/* Tan z refraction constant, radians per hPa of pressure. */
#define PFAC 2.77e-7

/*
** Model term indices
**
** Adding a new term involves changes to:
**
**   cpk.h
**   model.c
**   sky2enc.c
**   enc2sky.c
**   equat.mod and/or altaz.mod
*/

enum {

   /* Equatorial */

       IH,             /* hour angle index error */
       ID,             /* declination index error */
       NP,             /* HA/dec nonperpendicularity */
       CH,             /* boresight/dec nonperpendicularity */
       ME,             /* polar axis vertical misalignment */
       MA,             /* polar axis left-right misalignment */
       FO,             /* fork flexure */
       DAF,            /* declination axis flop */

   /* Altazimuth */
       IA,             /* azimuth index error */
       IE,             /* elevation index error */
       NPAE,           /* az/el nonperpendicularity */
       CA,             /* boresight/el nonperpendicularity */
       AN,             /* azimuth axis tilt north */
       AW,             /* azimuth axis tilt west */
       HASA,	       /* Azimuth runout, sine component (THEO) */
       HASA2,	       /* Azimuth runout, harmonic component (THEO) */
       HASA3,	       /* Azimuth runout, harmonic component (THEO) */
       HASA4,	       /* Azimuth runout, harmonic component (THEO) */
       HASA5,	       /* Azimuth runout, harmonic component (THEO) */
       HASA6,	       /* Azimuth runout, harmonic component (THEO) */
       HASA7,	       /* Azimuth runout, harmonic component (THEO) */
       HASA8,	       /* Azimuth runout, harmonic component (THEO) */
       HACA,	       /* Azimuth runout, cosine component (THEO) */
       HACA2,	       /* Azimuth runout, 2nd cosine component (THEO) */
       HACA3,	       /* Azimuth runout, 2nd cosine component (THEO) */
       HESE,           /* elevation runout, sine component */
       HECE,	       /* elevation runout, cosine component (THEO) */
       HESA,	       /* Elevation first harmonic sin in azimuth (THEO) */
       HESA2,	       /* Elevation second harmonic sin in azimuth (THEO) */
       HESA3,	       /* Elevation third harmonic sin in azimuth (THEO) */
       HESA4,	       /* Elevation 4th harmonic sin in azimuth (THEO) */
       HESA5,	       /* Elevation 5th harmonic sin in azimuth (THEO) */
       HESA6,	       /* Elevation 6th harmonic sin in azimuth (THEO) */
       HESA7,	       /* Elevation 7th harmonic sin in azimuth (THEO) */
       HESA8,	       /* Elevation 8th harmonic sin in azimuth (THEO) */
       HESA9,	       /* Elevation 9th harmonic sin in azimuth (THEO) */
       HECA,	       /* Elevation first harmonic sin in azimuth (THEO) */
       HECA2,	       /* Elevation second harmonic sin in azimuth (THEO) */
       HECA3,	       /* Elevation third harmonic sin in azimuth (THEO) */
       PAA1,	       /* Azimuth scaling correction (THEO) */
       PEE1,	       /* ELevation scaling correction (THEO) */
       PEA1,	       /* ELevation scaling correction with azimuth (THEO) */
       TX,	       /* Tube Flexure (Tangent) (THEO) */
       HSSA,	       /* Left/Right by SIN Az (THEO) */
       HSSA2,	       /* Left/Right by SIN 2*Az (THEO) */
       HSSA3,	       /* Left/Right by SIN 3*Az (THEO) */
       HSCA,	       /* Left/Right by COS Az (THEO) */
       HSCA2,	       /* Left/Right by COS 2*Az (THEO) */
       HSCA3,	       /* Left/Right by COS 3*Az (THEO) */
       HSSE,	       /* Left/Right by SIN El (THEO) */
       HSSE2,	       /* Left/Right by SIN 2*El (THEO) */
       HSSE3,	       /* Left/Right by SIN 3*El (THEO) */
       HSCE,	       /* Left/Right by COS El (THEO) */
       HSCE2,	       /* Left/Right by COS 2*El (THEO) */
       HSCE3,	       /* Left/Right by COS 3*El (THEO) */

   /* Both */
       TF,             /* Hooke's Law vertical flexure */

   /* Number of supported terms */
       MAXT };


/* Coordinate systems */

typedef enum { ICRS = 1,     /* mean J2000 (almost) */
               CIRS = 2,     /* equator & CIO of date */
               AZEL_T = 3,   /* altazimuth, topocentric */
               AZEL_O = 4,   /* altazimuth, observed */
               ENC = 5       /* mount encoder demands */
             } COSYS;


/* Mount type codes */

typedef enum { MUNK = 0,     /* not known  */
               EQUAT = 1,    /* equatorial */
               ALTAZ = 2     /* altazimuth */
             } MOUNT;


/* Transformation parameters */

typedef struct {
   double elon;              /* longitude, east +ve (radians) */
   double sp, cp;            /* sin, cos of latitude */
   double xcip, ycip, zcip;  /* CIP vector */
   double xab, yab, zab;     /* aberration vector */
   double refr;              /* refraction quantity */
   double st, ct;            /* sin, cos of local ERA */
   MOUNT tmount;             /* mount type (EQUAT or ALTAZ) */
   double pco[MAXT];         /* pointing coefficients (radians) */
   double gc, gb;            /* guiding corrections (radians) */
} TRANSP;


/* =================== */
/* Function prototypes */
/* =================== */

#ifdef __cplusplus
extern "C" {
#endif

void cpkC2s ( double, double, double, double*, double* );

int cpkDome ( double, double, double, double, double, double,
              double, double, double, double, double*, double* );

int cpkEnc2sky ( TRANSP*, double, double, COSYS,
                 double[3], double[3], double[3], double[3] );

double cpkEqor ( double );

void cpkGuide ( double, double, TRANSP* );

int cpkLength ( char* );

double cpkMjd ( int, int, int );

int cpkModel ( char*, TRANSP* );

int cpkRefacc ( double, double, double, double, double, double,
                double, double, double* );

int cpkRot ( TRANSP*, double, double, COSYS, double* );

void cpkS2c ( double, double, double*, double*, double* );

void cpkS2v ( double, double, double* );

int cpkScomp ( char*, char* );

int cpkSky2enc ( int, TRANSP*, double[3], COSYS,
                 double[3], double[3], double[3], double*, double* );

double cpkTday ( int, int, double );

int cpkTpt ( TRANSP*, double[3], double, double, char* );

void cpkUpda ( double, double, double, double, TRANSP* );

void cpkUpdt ( double, TRANSP* );

void cpkV2s ( double*, double*, double* );

#ifdef __cplusplus
}
#endif

#endif
