#include "cpk.h"

int cpkDome ( double phi, double rdome,
              double xm, double ym, double zm,
              double p, double q, double r,
              double ta, double tb, double *az, double *el )

/*
**  - - - - - - - -
**   c p k D o m e
**  - - - - - - - -
**
**  Predict the azimuth and elevation of the dome aperture for an
**  asymmetrically mounted telescope.
**
**  Given:
**     phi       double    elevation of roll axis in radians
**     rdome     double    radius of dome
**     xm,ym,zm  double    offset of mount in dome
**     p         double    separation of roll and pitch axes
**     q         double    distance along pitch axis
**     r         double    separation of pitch axis and telescope
**     ta,tb     double    telescope roll/pitch in radians
**
**  Returned (arguments):
**     *az,*el   double    dome-aperture az/el in radians
**
**  Returned (function value)
**               int       0 = OK
**                        -1 = illegal rdome
**                        -2 = no solution
**
**  Notes:
**
**  1  The purpose of this function is to predict the point on the dome
**     (azimuth and elevation) through which the telescope's optical
**     axis passes.  This depends on where the telescope is pointed,
**     where within the dome the mount is located, the mount geometry,
**     and where on the mount the telescope is fixed.
**
**  2  The function supports all types of mount based on two axes at
**     right angles.  The longitude, or "roll", axis is the polar axis
**     in the case of an equatorial mount and the azimuth axis in the
**     case of an altazimuth mount.  The latitude, or "pitch", axis is
**     the declination axis in the case of an equatorial mount and the
**     elevation axis in the case of an altazimuth mount.
**
**  3  The telescope roll/pitch coordinate system matches minus hour
**     angle and declination in the equatorial case.  This means that it
**     is right-handed, longitude increasing anticlockwise as seen from
**     the positive pole.  It means also that zero roll for an
**     altazimuth occurs when the telescope is pointing south.  Thus in
**     the altazimuth case the telescope roll/pitch system matches
**     180-A,E.
**
**  4  For the roll/pitch values ta and tb, relatively low precision
**     will suffice.  For example the pointing model (corrections for
**     the effects of minor flexures and misalignments etc.) can safely
**     be ignored.
**
**  5  The type of mount is specified through the phi argument, which is
**     the elevation above the horizon of the north, or positive, roll
**     axis.  For an equatorial mount, phi is equal to the latitude;
**     for an altazimuth mount it is equal to pi/2.   Except in the
**     altazimuth case, the azimuth of the roll axis defines "north" for
**     the dome.  If the roll axis is not aligned north-south then an
**     appropriate adjustment needs to be made to the returned azimuth.
**
**  6  The enclosure is presumed to be hemispherical (or some other
**     portion of a sphere).  The radius of the sphere is specified
**     through the rdome argument.  Any desired units can be used as
**     long as all the other "length" arguments are in the same units.
**
**  7  The arguments xm,ym,zm specify the offset of the mount from the
**     center of the dome.  The "mount" in this context is that point
**     along the roll axis (and hence fixed in space) that lies nearest
**     to the pitch axis.  The x,y,z coordinate system is oriented
**     east,north,up.
**
**  8  The arguments p, q and r form a sequence of offsets linking the
**     roll axis, the pitch axis and the telescope's optical axis:
**
**     . p is the separation between the roll and pitch axes at their
**       closest approach.  For most GEMs (German equatorial mounts) and
**       altazimuth mounts, the two axes intersect and hence p is zero.
**       Occasionally (some horseshoe mounts for example) the two axes
**       do not intersect and p is non-zero.
**
**     . q is the distance along the pitch axis to where the telescope
**       assembly turns, starting from the point on the pitch axis
**       closest to the roll axis.  For most altazimuths and for
**       equatorial forks etc., q is zero, but for GEMs q is a
**       substantial distance.
**
**     . r is the separation between the pitch and optical axes.  It is
**       usually zero.  Very occasionally, for example where a second
**       telescope is mounted on the "side" of the main telescope, the
**       pitch axis and optical axis do not intersect and so r is non-
**       zero.
**
**  9  The sign conventions for p, q and r are most easily described
**     by considering the altazimuth and equatorial cases separately,
**     though the two cases are in fact equivalent.
**
**     . Altazimuth:  with the telescope horizontal and pointing south,
**       p is positive to the north, q is positive to the east and r is
**       positive upwards.
**
**     . Equatorial:  with the telescope pointing at zero HA,Dec, p is
**       positive towards HA 12h, Dec 0d, q is positive towards 6h east,
**       Dec 0d, and r is positive towards Dec +90d.
**
** 10  The offsets p and q are at right angles, and the offsets q and r
**     are at right angles, but the angle between the offsets p and r
**     varies with the pitch angle.
**
** 11  The units of rdome, xm, ym, zm, p, q, and r must all be the same.
**
** 12  The dome azimuth/elevation coordinate system follows the normal
**     convention.  Azimuth increases clockwise from zero in the north,
**     through 90 degrees (pi/2 radians) in the east.  The value
**     returned is in the range zero to 2pi.  At the zenith, zero is
**     returned.
**
** 13  Example:
**
**     . The mount is a GEM and is at latitude +36.18 deg.
**
**     . The dome is 3.8 meters in diameter.
**
**     . The optical axis and the declination axis intersect.
**
**     . The declination axis and the polar axis intersect.
**
**     . When the mount is set to zero HA,Dec, the telescope is east of
**       the pier and the counterweight is west of the pier.
**
**     . The distance from the polar axis along the declination axis to
**       the optical axis is 505 mm.
**
**     . The point at which the polar axis and declination axis
**       intersect is 35 mm west of, 370 mm north of, and 1250 mm above
**       the center of the dome.
**
**     . The telescope is pointing at a star 10 minutes west of the
**       meridian, at declination +37.9 degrees, and is east of the
**       pier.
**
**     The cpkDome arguments are as follows:
**
**       phi           0.6315   elevation of polar axis (radians)
**       rdome      1900.0      radius of dome
**       xm          -35.0      offset of...
**       ym          370.0      ...mount from...
**       zm         1250.0      ...dome
**       p             0.0      separation of mount axes
**       q           505.0      distance of telescope along Dec axis
**       r             0.0      separation of Dec and optical axes
**       ta           -0.0436   telescope roll (i.e. -HA in radians)
**       tb            0.6615   telescope pitch (i.e. Dec in radians)
**
** 14  The telescope roll/pitch are mechanical rather than celestial, so
**     that above/below pole and east/west of the pier cases are
**     distinguished.  When the mount is in the "below the pole"
**     configuration, the pitch value tb will lie outside the range
**     +/- pi/2.  Though rare for most mount types, in the GEM case this
**     will happen half the time because of the pier reversal necessary
**     as targets cross the meridian.  For the telescope in the previous
**     note, to acquire the same star but with the telescope west of the
**     pier would require ta = 3.098 and tb = 2.480 (radians).
**
**  Last revision:   27 January 2008
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

{
   double sp, cp, sa, ca, sb, cb, y, xmo, ymo, zmo,
          xdo, ydo, zdo, x, z, xs, ys, zs, sdt, tm2, w, f;


/* Validate dome radius. */
   if ( rdome <= 0.0 ) return -1;

/* Functions of roll-axis elevation. */
   sp = sin ( phi );
   cp = cos ( phi );

/* Functions of roll/pitch. */
   sa = sin ( ta );
   ca = cos ( ta );
   sb = sin ( tb );
   cb = cos ( tb );

/* Mount to optical center, roll/pitch frame. */
   y = p + r * sb;
   xmo = q * ca - y * sa;
   ymo = q * sa + y * ca;
   zmo = r * cb;

/* Dome to optical center, east-north-up frame. */
   xdo = xm + xmo;
   ydo = ym + ymo * sp + zmo * cp;
   zdo = zm - ymo * cp + zmo * sp;

/* Telescope az/el unit vector, east-north-up frame. */
   x = sa * cb;
   y = - ca * cb;
   z = sb;
   xs = x;
   ys = y * sp + z * cp;
   zs = - y * cp + z * sp;

/* Solve for distance from optical center to dome aperture. */
   sdt = xs * xdo + ys * ydo + zs * zdo;
   tm2 = xdo * xdo + ydo * ydo + zdo * zdo;
   w = sdt * sdt - tm2 + rdome * rdome;
   if ( w < 0.0 ) return -2;
   f = - sdt + sqrt ( w );

/* Vector from dome center to dome aperture. */
   x = xdo + f * xs;
   y = ydo + f * ys;
   z = zdo + f * zs;

/* To spherical coordinates. */
   *az = x != 0.0 || y != 0.0 ? atan2 ( x, y ) : 0.0;
   if ( *az < 0.0 ) *az += D360;
   *el = atan2 ( z, sqrt ( x * x + y * y ) );

   return 0;
}
