#include "cpk.h"

int cpkEnc2sky ( TRANSP *pars, double a, double b, COSYS to,
                 double vobs[3], double vtopo[3],
                 double vdate[3], double vicrs[3] )

/*
**  - - - - - - - - - - -
**   c p k E n c 2 s k y
**  - - - - - - - - - - -
**
**  Evaluate a specified part of the chain of transformations involved
**  in determining where a telescope is pointing:
**
**
**            ENC             encoder demands: where the mount was aimed
**             |
**       pointing model
**             |
**           AZEL_O           observed (refracted) Az,El
**             |
**         refraction
**             |
**           AZEL_T           topocentric (unrefracted) Az,El
**             |
**       Earth rotation
**             |
**            CIRS            RA,Dec of date (RA is wrt CIO)
**             |
**     precession-nutation
**         aberration
**             |
**            ICRS            astrometric RA,Dec
**
**
**  The starting point is always the demanded encoder readings ENC.  The
**  sky directions AZEL_O, AZEL_T, CIRS and ICRS are returned, but
**  optionally stopping short at the specified place.
**
**  Given:
**     pars   TRANSP*  transformation parameters
**     a        d      roll angle demand (radians)
**     b        d      pitch angle demand (radians)
**     to     COSYS    final system (AZEL_O, AZEL_T, CIRS or ICRS)
**
**  Returned:
**     vobs    d[3]    AZEL_O vector (i.e. observed Az,El)
**     vtopo   d[3]    AZEL_T vector (i.e. topocentric Az,El)
**     vdate   d[3]    CIRS vector (i.e. CIO-based RA,Dec of date)
**     vicrs   d[3]    ICRS vector (similar to J2000 mean RA,Dec)
**
**  Returned (function value):
**              i      0 = OK
**                  else = illegal final system
**
**  Notes:
**
**  1  The transformation parameters pars must be set up before the
**     call.  See the functions cpkModel, cpkUpda and cpkUpdt.
**
**  2  The accuracy of the first part of the transformation, namely the
**     pointing corrections, depends on the size of the model
**     coefficients.  For typical values, the error compared with a
**     rigorous implementation is unlikely to be as large as
**     1 arcsecond.
**
**  3  Using the functions cpkUpda etc., the accuracy of the remaining
**     AZEL_O to ICRS portion is about 1 arcsecond for dates between
**     1950 and 2100, elevations above 25 deg and V color.
**
**  4  The inverse transformations are performed by the function
**     cpkSky2enc.  The two functions are not precisely complementary,
**     but for typical pointing coefficients should deliver consistency
**     at the 1 arcsecond level except close to the pole of the
**     mounting.
**
**  5  All results are right-handed 3-vectors with axes as follows:
**
**                CIRS          AZEL_O
**                ICRS          AZEL_T
**
**       x     RA=0h,Dec=0d      south
**       y     RA=6h,Dec=0d      east
**       z        Dec=90d       zenith
**
**  6  The returned vectors are not precisely unit vectors in all cases.
**
**  7  The encoder demand angles are HA,Dec for the case of an
**     equatorial mount and Az,El for an altazimuth mount.  Hour angle
**     is zero on the meridian and west-positive.  Azimuth is zero in
**     the north and increases clockwise as seen from above.  The
**     angles may exceed normal ranges, and represent actual mechanical
**     attitude.
**
**  Last revision:   27 January 2008
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

{
   double ta, tb, a1, b1, x, y, z, r, coz, tc, tn,
          xe, ze, cd, sq, cq, w, x1, tx, ty, y1, tf, z1;



/* First correct the index errors. */
   if ( pars->tmount == EQUAT ) {
      ta = pars->pco[IH];
      tb = pars->pco[ID];
   } else {
      ta = - pars->pco[IA];
      tb = pars->pco[IE];
   }
   a1 = a + ta;
   b1 = b + tb;

/* Encoder readings to internal (right-handed) form. */
   if ( pars->tmount == EQUAT ) {
      a1 = -a1;
   } else {
      a1 = D180-a1;
   }

/* Now apply other encoder corrections. */
   if ( pars->tmount == EQUAT ) {
      ta = 0.0;
      tb = pars->pco[FO]*cos(a1);
   } else {
      ta = pars->pco[HASA]*sin(a1); /* MODIFIED BY THEO */
      ta += pars->pco[HASA2]*sin(2.0*a1); /* MODIFIED BY THEO */
      ta += pars->pco[HASA3]*sin(3.0*a1); /* MODIFIED BY THEO */
      ta += pars->pco[HASA4]*sin(4.0*a1); /* MODIFIED BY THEO */
      ta += pars->pco[HASA5]*sin(5.0*a1); /* MODIFIED BY THEO */
      ta += pars->pco[HASA6]*sin(6.0*a1); /* MODIFIED BY THEO */
      ta += pars->pco[HASA7]*sin(7.0*a1); /* MODIFIED BY THEO */
      ta += pars->pco[HASA8]*sin(8.0*a1); /* MODIFIED BY THEO */
      ta += pars->pco[HACA]*cos(a1); /* MODIFIED BY THEO */
      ta += pars->pco[HACA2]*cos(2.0*a1); /* MODIFIED BY THEO */
      ta += pars->pco[HACA3]*cos(3.0*a1); /* MODIFIED BY THEO */
      ta += pars->pco[HSSA] * sin(a1)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSSA2] * sin(2.0*a1)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSSA3] * sin(3.0*a1)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSCA] * cos(a1)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSCA2] * cos(2.0*a1)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSCA3] * cos(3.0*a1)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSSE] * sin(b)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSSE2] * sin(2.0*b)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSSE3] * sin(3.0*b)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSCE] * cos(b)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSCE2] * cos(2.0*b)/cos(b); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[HSCE3] * cos(3.0*b)/cos(b); /* THIS CODE ADDED BY THEO */
      tb = pars->pco[HESE]*sin(b1); 
      tb += pars->pco[HECE]*cos(b1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA]*sin(a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA2]*sin(2.0*a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA3]*sin(3.0*a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA4]*sin(4.0*a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA5]*sin(5.0*a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA6]*sin(6.0*a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA7]*sin(7.0*a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA8]*sin(8.0*a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA9]*sin(9.0*a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HECA]*cos(a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HECA2]*cos(2.0*a1); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HECA3]*cos(3.0*a1); /* THIS CODE ADDED BY THEO */
      ta += pars->pco[PAA1] * a1; /* THIS CODE ADDED BY THEO */
      tb += pars->pco[PEE1] * b1; /* THIS CODE ADDED BY THEO */
      tb += pars->pco[PEA1] * a1; /* THIS CODE ADDED BY THEO */
      tb -= pars->pco[TX]/tan(b); /* THIS CODE ADDED BY THEO */
   }
   tb += pars->gb;
   a1 += ta;
   b1 += tb;

/* To Cartesian. */
   cpkS2c ( a1, b1, &x, &y, &z );

/* Cosine of pitch angle. */
   r = sqrt(x*x+y*y);

/* Collimation and nonperpendicularity corrections. */
   if ( pars->tmount == EQUAT ) {

   /* Equatorial:  estimate cosine of observed ZD. */
      coz = z*pars->sp + x*pars->cp;

   /* Collim/nonperp corrections. */
      tc =   pars->pco[DAF]*coz
           - pars->pco[CH]
           - pars->gc;
      tn = - pars->pco[NP];

   } else {

   /* Altazimuth collim/nonperp terms. */
      tc = pars->pco[CA]
         + pars->gc;
      tn = pars->pco[NPAE];

   /* Functions of parallactic angle (with zenith/pole precautions). */
      xe = x*pars->sp + z*pars->cp;
      ze = -x*pars->cp + z*pars->sp;
      cd = sqrt ( xe*xe + y*y);
      if ( cd == 0.0 || r == 0.0 ) {
         sq = 0.0;
         cq = 1.0;
      } else {
         sq = ( -y*pars->cp / cd ) / r;
         cq = ( cd*pars->sp - xe*ze*pars->cp / cd ) / r;
      }

   /* Apply altazimuth rotating collimation corrections. */
      tc += ( - pars->pco[ID]*sq - pars->pco[CH]*cq );
      tb =  (   pars->pco[ID]*cq - pars->pco[CH]*sq );
      w = r != 0.0 ? tb*z/r : 0.0;
      x -= w*x;
      y -= w*y;
      z += tb*r;

   }
   tc += z*tn;

/* Reverse if beyond the pole. */
   if ( fabs(b1) > D90 ) tc = -tc;

/* Apply the corrections. */
   w = r != 0.0 ? tc/r : 0.0;
   x1 = x;
   x -= w*y;
   y += w*x1;

/* Apply mount misalignment corrections. */
   if ( pars->tmount == EQUAT ) {
      tx = pars->pco[ME];
      ty = - pars->pco[MA];
   } else {
      tx = pars->pco[AN];
      ty = pars->pco[AW];
   }
   x1 = x;
   y1 = y;
   x -= tx*z;
   y -= ty*z;
   z -= - tx*x1 - ty*y1;

/* If equatorial, rotate into altaz. */
   if ( pars->tmount == EQUAT ) {
      x1 = pars->sp*x - pars->cp*z;
      z = pars->cp*x + pars->sp*z;
      x = x1;
   }

/* Apply vertical flexure corrections. */
   tf = pars->pco[TF];
   w = tf*z;
   z -= tf*(x*x+y*y);
   x += w*x;
   y += w*y;

/* Observed x,y,z. */
   vobs[0] = x;
   vobs[1] = y;
   vobs[2] = z;

/* Stop now if final system has been reached. */
   if ( to == AZEL_O ) return 0;

/* Approximate derefraction, fixed below about 6 deg elevation. */
   z -= pars->refr / ( z > 0.1 ? z : 0.1 );

/* Topocentric result. */
   w = sqrt ( x*x + y*y + z*z );
   vtopo[0] = x /= w;
   vtopo[1] = y /= w;
   vtopo[2] = z /= w;

/* Stop now if final system has been reached. */
   if ( to == AZEL_T ) return 0;

/* Rotate into equatorial coordinates. */
   x1 =   x*pars->sp + z*pars->cp;
   z  = - x*pars->cp + z*pars->sp;

/* Terrestrial to celestial. */
   x = x1*pars->ct - y*pars->st;
   y = x1*pars->st + y*pars->ct;

/* CIRS result. */
   vdate[0] = x;
   vdate[1] = y;
   vdate[2] = z;

/* Stop now if final system has been reached. */
   if ( to == CIRS ) return 0;

/* Precession-nutation. */
   x1 =   pars->zcip*x                + pars->xcip*z;
   y1 =                             y + pars->ycip*z;
   z1 = - pars->xcip*x - pars->ycip*y + pars->zcip*z;

/* Aberration, giving ICRS (loosely mean J2000) result. */
   vicrs[0] = x1 - pars->xab;
   vicrs[1] = y1 - pars->yab;
   vicrs[2] = z1 - pars->zab;

/* Verify destination and exit. */
   return ( to == ICRS ) ? 0 : -1;

}
