#include "cpk.h"

double cpkEqor ( double date )

/*
**  - - - - - - - -
**   c p k E q o r
**  - - - - - - - -
**
**  Equation of the origins.
**
**  Given:
**     date     d        Modified Julian Date (JD-2400000.5)
**
**  Returned (function value):
**              d        equation of the origins (radians)
**
**  Notes:
**
**    1  The date is TDB in principle, but it is perfectly acceptable to
**       use UTC.
**
**    2  The equation of the origins (EO) is the accumulated precession-
**       nutation since J2000.  It is the difference between Greenwich
**       sidereal time and Earth rotation angle and, equivalently,
**       between the equinox based apparent right ascension and the CIO-
**       based CIRS right ascension:
**
**          EO = ERA - GST
**             = RA_CIRS - RA_apparent
**
**    3  This function need be called only occasionally, say once per
**       slew or even once per night, and only if support for geocentric
**       apparent place is required.  The value returned can then be
**       used to transform apparent RA,Dec target coordinates into the
**       CIRS coordinates used by CPK:  simply add EO to the apparent RA
**       and leave the Dec as it is.
**
**    4  The series used is accurate to about 0.25 arcsec during the
**       21st century.
**
**  Last revision:   20 December 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

{
   double t, f, d, om;



/* Days since J2000. */
   t = date - 51544.5;

/* Delaunay arguments D, D and Omega (radians). */
   f = 1.627905234 + 0.230895720*t;
   d = 5.198466741 + 0.212768710*t;
   om = 2.18243920 - 0.000924218*t;

/* Equation of the origins. */
   return (-612193452.6e-15 - 5.1e-15*t)*t
              +765355e-10*sin(om)
               +58585e-10*sin(2.0*(f-d+om))
               +10126e-10*sin(2.0*(f+om))
                -9228e-10*sin(2.0*om);

}
