#include "cpk.h"
#include <stdio.h>

int cpkModel ( char *file, TRANSP *pars )

/*
**  - - - - - - - - -
**   c p k M o d e l
**  - - - - - - - - -
**
**  Read the telescope pointing model from a file.
**
**  The file must conform strictly to the format produced by the
**  OUTMOD command in the TPOINT package, except that certain records
**  and fields are ignored (Notes 1,2).
**
**  Given:
**     file   char[]    the name of the file to be read
**
**  Returned:
**     pars   TRANSP*   pco[MAXT] array initialized and tmount set
**
**  Returned (function value):
**              int       status:  0 = OK
**                                -1 = unable to open input file
**                                -2 = I/O error or premature EOF
**                                -3 = unrecognized record
**
**  Notes:
**
**  1  The file containing the pointing model must, as far as the basic
**     layout is concerned, be exactly as written by TPOINT's OUTMOD
**     command (which indeed is the ideal way of generating the file).
**     However, unlike TPOINT's INMOD command, the present function does
**     not support format changes and the insertion of comment records.
**     The first two records, which are ignored, may be used to record
**     comments;  however, both must be present, as must the END marker.
**     In the term records, the term name must start in column 3, and
**     the coefficient value must not start before column 11.
**
**  2  The order of the term records is ignored.  Only selected TPOINT
**     terms are supported;  if absent they are set to zero;  unknown
**     terms cause an error status.
**
**  3  The supported terms are as follows.  Each appears as a radian
**     value in the specified pars->pco[] element.
**
**        IH         hour angle index error
**        ID         declination index error
**        TF         tube flexure
**        DAF        declination axis flop
**        FO         fork flexure
**        NP         HA/dec nonperp
**        CH         boresight/dec nonperp
**        ME         polar axis misalignment up-down
**        MA         polar axis misalignment left-right on sky
**        IA         azimuth index error
**        IE         elevation index error
**        HESE       elevation centering, sine
**	  HESA2	     elevation second harmonic SIN in Azimuth (THEO)
**        NPAE       Az/El nonperp
**        CA         boresight/el nonperp
**        AN         azimuth axis tilt north
**        AW         azimuth axis tilt west
**        TX	     Alternative tube flexure
**
**  4  Here is an example file:
**
**           "Comment 1"
**           "Comment 2"
**           "  IA       -256.0000"
**           "  IE       +380.0000"
**           "  HESE      -24.5000"
**           "  TF        +27.2400"
**           "  NPAE      -13.7700"
**           "  CA        +18.1000"
**           "  AN        -32.0500"
**           "  AW        +17.0001"
**           "END"
**            ^
**          col 1
**
**     Note the two initial records, which must be present but which are
**     not interpreted, and the mandatory END record.  Each of the
**     remaining records defines a term and the corresponding
**     coefficient value (in arcseconds).  For example, the record
**
**           "  IA       -256.0000"
**
**     defines a term called IA with a value of -256 arcsec.  The
**     additional flags and numbers found in records output by the
**     TPOINT command OUTDAT are ignored.
**
**  5  If any invalid record is detected, the pointing model is reset
**     and the rest of the input file is ignored.
**
**  6  The guiding corrections are reset to zero whenever this function
**     is called.
**
**  Called:  cpkGuide, cpkScomp
**
**  Last revision:   27 January 2008
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

#define MORE 1
#define LSTRNG 200                           /* length of buf string */
#define LNAME 9                              /* length of name string */
#define AS2R 4.848136811095359935899141e-6   /* arcsec to radians */

{
   int i;
   FILE* fp;
   char buf[LSTRNG], name[LNAME];
   double val;



/*
** Initialize the pointing model
** -----------------------------
*/

/* Coefficients all zero. */
   for ( i = 0; i < MAXT; i++ ) {
      pars->pco[i] = 0.0;
   }

/* Unknown mount type. */
   pars->tmount = MUNK;

/* Reset the guiding corrections. */
   cpkGuide ( 0.0, 0.0, pars );

/*
** Open the model file
** -------------------
*/

   if ( ( fp = fopen ( file, "r" ) ) == NULL ) return -1;

/*
** Read and ignore the caption record
** ----------------------------------
*/

   if ( fgets ( buf, LSTRNG, fp ) == NULL ) return -2;

/*
** Read and ignore the method, statistics and refraction record
** ------------------------------------------------------------
*/

   if ( fgets ( buf, LSTRNG, fp ) == NULL ) return -2;

/*
** Term records
** ------------
*/

   while ( MORE ) {

   /* Read one record. */
      if ( fgets ( buf, LSTRNG, fp ) == NULL ) return -2;

   /* OK exit if end marker (mandatory). */
      if ( ! cpkScomp ( buf, "END" ) ) {
         fclose ( fp );
         return 0;
      }

   /* Extract and decode the term name and amplitude. */
      if ( sscanf ( buf+2, "%8s %lf", name, &val ) != 2 ) {

      /* If suspect, error-exit. */
         fclose ( fp );
         return -3;

      } else {

      /* Record the coefficient value (also identify the mount). */
         val*= AS2R;
         if ( ! cpkScomp ( name, "IH" ) ) {
            pars->pco[IH] = val;
            pars->tmount = EQUAT;
         }
         if ( ! cpkScomp ( name, "ID" ) ) pars->pco[ID] = val;
         if ( ! cpkScomp ( name, "TF" ) ) pars->pco[TF] = val;
         if ( ! cpkScomp ( name, "DAF" ) ) pars->pco[DAF] = val;
         if ( ! cpkScomp ( name, "FO" ) ) pars->pco[FO] = val;
         if ( ! cpkScomp ( name, "NP" ) ) pars->pco[NP] = val;
         if ( ! cpkScomp ( name, "CH" ) ) pars->pco[CH] = val;
         if ( ! cpkScomp ( name, "ME" ) ) pars->pco[ME] = val;
         if ( ! cpkScomp ( name, "MA" ) ) pars->pco[MA] = val;
         if ( ! cpkScomp ( name, "IA" ) ) {
            pars->pco[IA] = val;
            pars->tmount = ALTAZ;
         }
         if ( ! cpkScomp ( name, "IE" ) ) pars->pco[IE] = val;
         if ( ! cpkScomp ( name, "HASA" ) ) pars->pco[HASA] = val; /* THEO */
         if ( ! cpkScomp ( name, "HASA2" ) ) pars->pco[HASA2] = val; /* THEO */
         if ( ! cpkScomp ( name, "HASA3" ) ) pars->pco[HASA3] = val; /* THEO */
         if ( ! cpkScomp ( name, "HASA4" ) ) pars->pco[HASA4] = val; /* THEO */
         if ( ! cpkScomp ( name, "HASA5" ) ) pars->pco[HASA5] = val; /* THEO */
         if ( ! cpkScomp ( name, "HASA6" ) ) pars->pco[HASA6] = val; /* THEO */
         if ( ! cpkScomp ( name, "HASA7" ) ) pars->pco[HASA7] = val; /* THEO */
         if ( ! cpkScomp ( name, "HASA8" ) ) pars->pco[HASA8] = val; /* THEO */
         if ( ! cpkScomp ( name, "HACA" ) ) pars->pco[HACA] = val; /* THEO */
         if ( ! cpkScomp ( name, "HACA2" ) ) pars->pco[HACA2] = val; /* THEO */
         if ( ! cpkScomp ( name, "HACA3" ) ) pars->pco[HACA3] = val; /* THEO */
         if ( ! cpkScomp ( name, "HESE" ) ) pars->pco[HESE] = val;
         if ( ! cpkScomp ( name, "HECE" ) ) pars->pco[HECE] = val;
         if ( ! cpkScomp ( name, "HESA" ) ) pars->pco[HESA] = val; /* THEO */
         if ( ! cpkScomp ( name, "HESA2" ) ) pars->pco[HESA2] = val; /* THEO */
         if ( ! cpkScomp ( name, "HESA3" ) ) pars->pco[HESA3] = val; /* THEO */
         if ( ! cpkScomp ( name, "HESA4" ) ) pars->pco[HESA4] = val; /* THEO */
         if ( ! cpkScomp ( name, "HESA5" ) ) pars->pco[HESA5] = val; /* THEO */
         if ( ! cpkScomp ( name, "HESA6" ) ) pars->pco[HESA6] = val; /* THEO */
         if ( ! cpkScomp ( name, "HESA7" ) ) pars->pco[HESA7] = val; /* THEO */
         if ( ! cpkScomp ( name, "HESA8" ) ) pars->pco[HESA8] = val; /* THEO */
         if ( ! cpkScomp ( name, "HESA9" ) ) pars->pco[HESA9] = val; /* THEO */
         if ( ! cpkScomp ( name, "HECA" ) ) pars->pco[HECA] = val; /* THEO */
         if ( ! cpkScomp ( name, "HECA2" ) ) pars->pco[HECA2] = val; /* THEO */
         if ( ! cpkScomp ( name, "HECA3" ) ) pars->pco[HECA3] = val; /* THEO */
         if ( ! cpkScomp ( name, "NPAE" ) ) pars->pco[NPAE] = val;
         if ( ! cpkScomp ( name, "CA" ) ) pars->pco[CA] = val;
         if ( ! cpkScomp ( name, "AN" ) ) pars->pco[AN] = val;
         if ( ! cpkScomp ( name, "AW" ) ) pars->pco[AW] = val;
         if ( ! cpkScomp ( name, "PAA1" ) ) pars->pco[PAA1] = val; /* THEO */
         if ( ! cpkScomp ( name, "PEE1" ) ) pars->pco[PEE1] = val; /* THEO */
         if ( ! cpkScomp ( name, "PEA1" ) ) pars->pco[PEA1] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSSA" ) ) pars->pco[HSSA] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSSA2" ) ) pars->pco[HSSA2] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSSA3" ) ) pars->pco[HSSA3] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSCA" ) ) pars->pco[HSCA] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSCA2" ) ) pars->pco[HSCA2] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSCA3" ) ) pars->pco[HSCA3] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSSE" ) ) pars->pco[HSSE] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSSE2" ) ) pars->pco[HSSE2] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSSE3" ) ) pars->pco[HSSE3] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSCE" ) ) pars->pco[HSCE] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSCE2" ) ) pars->pco[HSCE2] = val; /* THEO */
         if ( ! cpkScomp ( name, "HSCE3" ) ) pars->pco[HSCE3] = val; /* THEO */
         if ( ! cpkScomp ( name, "TX" ) ) pars->pco[TX] = val; /* THEO */

      }
   }
}
