#include "cpk.h"

static void atmt ( double, double, double, double, double, double,
                   double, double, double, double, double, double,
                   double*, double*, double* );
static void atms ( double, double, double, double, double,
                   double*, double* );

int cpkRefacc ( double wlmu, double phpa, double tdc, double rhpc,
                double phi, double hm, double tlr, double etopo,
                double *padj )

/*
**  - - - - - - - - - -
**   c p k R e f a c c
**  - - - - - - - - - -
**
**  Optional high-accuracy refraction:  returns an adjusted pressure
**  which when used by CPK produces improved refraction predictions.
**
**  Given:
**     wlmu     d      wavelength in micrometers
**     phpa     d      ambient pressure in HPa (= mb)
**     tdc      d      ambient temperature in deg C
**     rhpc     d      relative humidity (percent)
**     phi      d      geodetic latitude (radians)
**     hm       d      height above sea level (meters)
**     tlr      d      tropospheric lapse rate (K/m)
**     etopo    d      topocentric elevation (radians)
**
**  Returned:
**     padj     d*     adjusted pressure
**
**  Returned (function value):
**              i      0 = OK
**                  else = illegal argument
**
**  Notes:
**
**  1  The ambient pressure is as measured at the telescope (QFE) and
**     is smaller at high-altitude sites.  It is NOT the equivalent
**     pressure at sea level (QNH), which is always around 1013 mb.
**
**  2  The "topocentric" elevation is that for the line of sight to the
**     target but in vacuo.  The telescope tracks the "observed"
**     elevation, slightly greater than the topocentric elevation
**     because of refraction.
**
**  3  The pressure that is returned, padj, will typically be close to
**     to the ambient reading, phpa, but includes an adjustment to
**     compensate for CPK's own simple (but extremely fast) refraction
**     prediction.  The adjustment takes into account temperature,
**     humidity and wavelength (also site location and topospheric lapse
**     rate.
**
**  4  This function is CPU-intensive, and the real-time system would
**     usually call it relatively infrequently (at 1 Hz, say, instead of
**     at servo rates) or use it to populate a look-up table.
**
**  5  The radio refraction is chosen by specifying wlmu > 100 microns
**     and is taken to be independent of wavelength.  Because the
**     algorithm takes no account of the ionosphere, the accuracy
**     deteriorates at low frequencies, below about 30 MHz.  The upper
**     frequency limit is set by water absorption features that start to
**     become important above Ka band.  In the optical the wavelength
**     predicts the atmospheric dispersion, which is a large effect -
**     over 1% of the total refraction between 0.35 and 0.9 microns.
**
**  6  A suggested value for the tlr argument is 0.0065.  The
**     refraction is significantly affected by tlr, and if studies
**     of the local atmosphere have been carried out a better tlr
**     value may be available.  The sign of the supplied tlr value
**     is ignored.
**
**  7  The refraction predictions use the method of Hohenkerk and
**     Sinclair (NAO Technical Notes 59 and 63, subsequently adopted
**     in the Explanatory Supplement, 1992 edition - see section 3.281)
**     with following improvements:
**
**     .  Murray's values for the gas constants have been used
**        (Vectorial Astrometry, Adam Hilger, 1983).
**
**     .  A better model for Ps(T) has been adopted (taken from
**        Gill, Atmosphere-Ocean Dynamics, Academic Press, 1982).
**        More accurate expressions for Pwo have been adopted from the
**        same reference.  It should be noted that the relative humidity
**        rhpc is formally defined in terms of "mixing ratio" rather
**        than pressures or densities as is often stated.  It is the
**        mass of water per unit mass of dry air divided by that for
**        saturated air at the same temperature and pressure.
**
**     .  The formula for the water vapour pressure, given the
**        saturation pressure and the relative humidity, is from
**        Crane (1976), expression 2.5.5.
**
**     .  The radio refractivity model is from J.M.Rueger, "Refractive
**        Index Formulae for Electronic Distance Measurement with Radio
**        and Millimetre Waves", in Unisurv Report S-68 (2002), School
**        of Surveying and Spatial Information Systems, University of
**        New South Wales, Sydney, Australia.
**
**     .  The optical refractivity for dry air is from Resolution 3 of
**        the International Association of Geodesy adopted at the XXIIth
**        General Assembly in Birmingham, UK, 1999.
**
**  8  The results bring CPK into close agreement with the rigorous
**     raytracing method used, for all positive elevations.  Negative
**     elevations (relevant to high-elevation sites only) the agreement
**     begins to fall off.  However, it is unlikely that even the
**     raytracing results are useful in this region.
**
**  Last revision:   23 May 2009
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

#define OPTIC ( wlmu <= 100.0 )

#define NITS 6        /* number of iterations to get observed ZD */
#define EPS 1e-8      /* convergence criterion */
#define ISMAX 16384   /* numerical integration: max number of strips */
#define GCR 8314.32   /* universal gas constant */
#define DMD 28.9644   /* molecular weight of dry air */
#define DMW 18.0152   /* molecular weight of water vapour */
#define S 6378120.0   /* mean Earth radius (m) */
#define DELTA 18.36   /* expnt of temp dependence of water vapour P */
#define HT 11000.0    /* height of tropopause (m) */
#define HS 80000.0    /* upper limit for refractive effects (m) */

{

/* Arguments for the internal functions atmt and atms */
   double robs;   /* height of observer from center of Earth (m) */
   double gamm2;  /* GAMMA minus 2 */
   double delm2;  /* DELTA minus 2 */
   double rt;     /* height of tropopause from center of Earth (m) */
   double tt;     /* temperature at the tropopause (K) */
   double dnt;    /* refractive index at the tropopause */
   double gamal;  /* constant of the atmospheric model = g*md/r */
   double c1,c2,c3,c4,c5,c6;

/* The refraction integrand */
#define refi(DN,RDNDR) ((RDNDR)/(DN+RDNDR));

   int it, is, k, n, i, j;
   double tlra, tk, rh, ztopo, zobs, wlsq, gb, a, gamma, psat,
          pwo, w, tempo, dn0, rdndr0, f0, rdndrt, ft, dnts, rdndrp,
          fts, rs, dns, rdndrs, fs, sk0, zt, zts, zs, refold, z0,
          zrange, fb, ff, fo, fe, h, r, sz, rg, dr, tg, dn, rdndr,
          t, f, refp, reft, ref, refa, dz0, z, dz1;



/* Ignore sign of lapse rate. */
   tlra = fabs ( tlr );

/* Crudely validate arguments. */
   if ( wlmu  <     0.2                      ||
        phpa  <     0.0   || phpa  >  1500.0 ||
        tdc   <  -100.0   || tdc   >   100.0 ||
        rhpc  <     0.0   || rhpc  >   100.0 ||
        phi   <  -D90     || phi   >   D90   ||
        hm    < -1000.0   || hm    >    HT   ||
        tlra  <     0.002 || tlra  >  0.02   ||
        etopo <     0.0   || etopo >   D90      ) return -1;

/* Temperature in Kelvins. */
   tk = 273.15 + tdc;

/* Humidity as a fraction. */
   rh = rhpc / 100.0;

/* Topocentric zenith distance. */
   ztopo = D90 - etopo;

/* Set up model atmosphere parameters defined at the observer. */
   wlsq = wlmu * wlmu;
   gb = 9.784 * ( 1.0 - 0.0026 * cos ( 2.0 * phi ) - 2.8e-7 * hm );
   a = ( OPTIC ) ?
         ( ( 287.6155 + 1.62887 / wlsq + 0.01360 / ( wlsq * wlsq ) )
                 * 273.15 / 1013.25 ) * 1e-6
       :
         77.6890e-6;
   gamal = gb * DMD / GCR;
   gamma = gamal / tlra;
   gamm2 = gamma - 2.0;
   delm2 = DELTA - 2.0;
   psat = pow ( 10.0, ( 0.7859 + 0.03477 * tdc ) /
                         ( 1.0 + 0.00412 * tdc ) ) *
                ( 1.0 + phpa * ( 4.5e-6 + 6e-10 * tdc * tdc ) );
   pwo = ( phpa > 0.0 ) ?
         rh * psat / ( 1.0 - ( 1.0 - rh ) * psat / phpa ) :
         0.0;
   w = pwo * ( 1.0 - DMW / DMD ) * gamma / ( DELTA - gamma );
   c1 = a * ( phpa + w ) / tk;
   c2 = ( a * w + ( OPTIC ? 11.2684e-6 : 6.3938e-6 ) * pwo ) / tk;
   c3 = ( gamma - 1.0 ) * tlra * c1 / tk;
   c4 = ( DELTA - 1.0 ) * tlra * c2 / tk;
   c5 = OPTIC ? 0.0 : 375463e-6 * pwo / tk;
   c6 = c5 * delm2 * tlra / ( tk * tk );

/* Conditions at the observer. */
   robs = S + hm;
   atmt ( robs, tk, tlra, gamm2, delm2, c1, c2, c3, c4, c5, c6, robs,
          &tempo, &dn0, &rdndr0 );
   f0 = refi ( dn0, rdndr0 );

/* Conditions at the tropopause in the troposphere. */
   rt = S + HT;
   atmt ( robs, tk, tlra, gamm2, delm2, c1, c2, c3, c4, c5, c6, rt,
          &tt, &dnt, &rdndrt );
   ft = refi ( dnt, rdndrt );

/* Conditions at the tropopause in the stratosphere. */
   atms ( rt, tt, dnt, gamal, rt, &dnts, &rdndrp );
   fts = refi ( dnts, rdndrp );

/* Conditions at the stratosphere limit. */
   rs = S + HS;
   atms ( rt, tt, dnt, gamal, rs, &dns, &rdndrs );
   fs = refi ( dns, rdndrs );

/* Variable initialization to avoid compiler warning. */
   reft = 0.0;

/* Iteratively estimate the refraction. */
   zobs = ztopo;
   for ( it = 0; it < NITS; it++ ) {

   /* Quantities that depend on zobs. */
      sk0 = dn0 * robs * sin ( zobs );
      zt = asin ( sk0 / ( rt * dnt ) );
      zts = asin ( sk0 / ( rt * dnts ) );
      zs = asin ( sk0 / ( rs * dns ) );

   /*
   ** Integrate the refraction integral in two parts;  first in the
   ** troposphere (k=1), then in the stratosphere (k=2).
   */

      for ( k = 1; k <= 2; k++ ) {

      /* Init. prev. refraction to guarantee at least two iterations. */
         refold = 1.0;

      /* Start off with 8 strips. */
         is = 8;

      /* Start z, z range, and start and end values. */
         if ( k == 1 ) {
            z0 = zobs;
            zrange = zt - z0;
            fb = f0;
            ff = ft;
         } else {
            z0 = zts;
            zrange = zs - z0;
            fb = fts;
            ff = fs;
         }

      /* Sums of odd and even values. */
         fo = 0.0;
         fe = 0.0;

      /* First time through the loop we have to do every point. */
         n = 1;

      /* Start of iteration loop (terminates at specified precision). */
         for ( ; ; ) {

         /* Strip width */
            h = zrange / (double) is;

         /* Init. distance from Earth center for quadrature pass. */
            r = ( k == 1 ) ? robs : rt;

         /* One pass (no need to compute evens after first time). */
            for ( i = 1; i < is; i += n ) {

            /* Sine of observed zenith distance. */
               sz = sin ( z0 + h * (double) i );

            /* Find r (to nearest meter, maximum four iterations). */
               if ( sz > 1e-20 ) {
                  w = sk0 / sz;
                  rg = r;
                  j = 0;
                  do {
                     if ( k == 1 ) {
                        atmt ( robs, tk, tlra, gamm2, delm2,
                               c1, c2, c3, c4, c5, c6, rg,
                               &tg, &dn, &rdndr );
                     } else {
                        atms ( rt, tt, dnt, gamal, rg, &dn, &rdndr );
                     }
                     dr = ( rg * dn - w ) / ( dn + rdndr );
                     rg -= dr;
                  } while ( fabs ( dr ) > 1.0 && j++ <= 4 );
                  r = rg;
               }

            /* Find refractive index and integrand at r. */
               if ( k == 1 ) {
                  atmt ( robs, tk, tlra, gamm2, delm2,
                         c1, c2, c3, c4, c5, c6, r,
                         &t, &dn, &rdndr );
               } else {
                  atms ( rt, tt, dnt, gamal, r, &dn, &rdndr );
               }
               f = refi ( dn, rdndr );

            /* Accumulate odd and (first time only) even values. */
               if ( n == 1 && i % 2 == 0 ) {
                  fe += f;
               } else {
                  fo += f;
               }
            }

         /* Evaluate the integrand using Simpson's Rule. */
            refp = h * ( fb + 4.0 * fo + 2.0 * fe + ff ) / 3.0;

         /* Save troposphere component. */
            if ( k == 1 ) reft = refp;

         /* If requested precision reached (or can't be), terminate. */
            if ( fabs ( refp - refold ) <= EPS || is >= ISMAX ) break;

         /* Not yet: prepare for the next iteration. */
            refold = refp; /* save current value for convergence test */
            is += is;      /* double the number of strips */
            fe += fo;      /* sum of all = sum of evens next time */
            fo = 0.0;      /* reset odds accumulator */
            n = 2;         /* skip even values next time */
         }
      }

   /* Total refraction. */
      ref = reft + refp;

   /* Observed zenith distance. */
      zobs = ztopo - ref;

   }

/* The refraction constant. */
   refa = PFAC * phpa;

/* The accurate dz value. */
   dz0 = zobs != 0.0 ? sin ( ref ) / sin ( zobs ) : refa;

/* The CPK dz value. */
   z = sin ( etopo );
   dz1 = refa / ( z > 0.1 ? z : 0.1 );

/* The adjusted pressure. */
   *padj = phpa * dz0 / dz1;

   return 0;

}

/*--------------------------------------------------------------------*/

static void atmt ( double robs, double tk, double tlra, double gamm2,
                   double delm2, double c1, double c2, double c3,
                   double c4, double c5, double c6, double r,
                   double *t, double *dn, double *rdndr )
/*
**  - - - - -
**   a t m t
**  - - - - -
**
**  Internal function used by cpkRefacc:
**
**    refractive index and derivative with respect to height for the
**    troposphere.
**
**  Given:
**    robs    double   height of observer from center of the Earth (m)
**    tk      double   temperature at the observer (K)
**    tlra    double   tlra           )
**    gamm2   double   gamma minus 2  ) see ES
**    delm2   double   delta minus 2  )
**    c1      double   useful term    )
**    c2      double   useful term    )
**    c3      double   useful term    ) see source of
**    c4      double   useful term    ) cpkRefacc main function
**    c5      double   useful term    )
**    c6      double   useful term    )
**    r       double   current distance from the center of the Earth (m)
**
**  Returned:
**    *t      double   temperature at r (K)
**    *dn     double   refractive index at r
**    *rdndr  double   r * rate the refractive index is changing at r
**
**  Note that in the optical/IR case c5 and c6 are zero.
*/

#define WMIN 100.0
#define WMAX 320.0

{
   double w, tt0, tt0gm2, tt0dm2;

   w = tk - tlra * ( r - robs );
   w = w > WMIN ? ( w < WMAX ? w : WMAX ) : WMIN;
   tt0 = w / tk;
   tt0gm2 = pow ( tt0, gamm2 );
   tt0dm2 = pow ( tt0, delm2 );
   *t = w;
   *dn = 1.0 + ( c1 * tt0gm2 - ( c2 - c5 / w ) * tt0dm2 ) * tt0;
   *rdndr = r * ( - c3 * tt0gm2 + ( c4 - c6 / tt0 ) * tt0dm2 );
}

/*--------------------------------------------------------------------*/

static void atms ( double rt, double tt, double dnt, double gamal,
                   double r, double *dn, double *rdndr )
/*
**  - - - - -
**   a t m s
**  - - - - -
**
**  Internal function used by cpkRefacc:
**
**   refractive index and derivative with respect to height for the
**   stratosphere.
**
**  Given:
**    rt      double   height of tropopause from center of the Earth (m)
**    tt      double   temperature at the tropopause (k)
**    dnt     double   refractive index at the tropopause
**    gamal   double   constant of the atmospheric model = g*md/r
**    r       double   current distance from the center of the Earth (m)
**
**  Returned:
**    *dn     double   refractive index at r
**    *rdndr  double   r * rate the refractive index is changing at r
**
*/

{
   double b, w;

   b = gamal / tt;
   w = ( dnt - 1.0 ) * exp ( - b * ( r - rt ) );
   *dn = 1.0 + w;
   *rdndr = - r * b * w;
}

/*--------------------------------------------------------------------*/
