#include "cpk.h"

int cpkRot ( TRANSP *pars, double a, double b, COSYS sky, double *pa )

/*
**  - - - - - - -
**   c p k R o t
**  - - - - - - -
**
**  Rotator angle required to stabilize view of sky.
**
**  Given:
**     pars   TRANSP*  transformation parameters
**     a        d      roll angle demand (radians)
**     b        d      pitch angle demand (radians)
**     sky    COSYS    coordinate system (ICRS, CIRS, AZEL_T or AZEL_O)
**
**  Returned (argument):
**     pa       d*     rotator angle (radians)
**
**  Returned (function value):
**              i      0 = OK
**                  else = illegal coordinate system
**
**  Notes:
**
**  1  The transformation parameters pars must be set up before the
**     call.  See the functions cpkModel, cpkUpda and cpkUpdt.
**
**  2  The sign convention for the rotator angle pa is such that when
**     pa increases it drives the projection of the rotator on the sky
**     counter-clockwise.
**
**  Called:  cpkEnc2sky
**
**  Last revision:   9 January 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

#define DB 1e-6     /* pitch increment for sensing field orientation */

{
   double* v;
   double vobs[3], vtopo[3], vdate[3], vicrs[3], x0, y0, z0, x1, y1, z1;



/* Identify which coordinates to use. */
   switch ( sky ) {
   case ICRS:
      v = vicrs;
      break;
   case CIRS:
      v = vdate;
      break;
   case AZEL_T:
      v = vtopo;
      break;
   case AZEL_O:
      v = vobs;
      break;
   default:
      return -1;
   }

/* Obtain sky coordinates for two sample points. */
   cpkEnc2sky ( pars, a, b-DB, sky, vobs, vtopo, vdate, vicrs );
   x0 = v[0];
   y0 = v[1];
   z0 = v[2];
   cpkEnc2sky ( pars, a, b+DB, sky, vobs, vtopo, vdate, vicrs );
   x1 = v[0];
   y1 = v[1];
   z1 = v[2];

/* Determine their relative position angle. */
   *pa = atan2 ( x1*y0 - y1*x0,
                 z1 * ( x0*x0 + y0*y0 ) - z0 * ( x1*x0 + y1*y0 ) );

   return 0;

}
