#include "cpk.h"

int cpkSky2enc ( int jbp, TRANSP *pars, double v[3], COSYS from,
                 double vdate[3], double vtopo[3], double vobs[3],
                 double *adem, double *bdem )

/*
**  - - - - - - - - - - -
**   c p k S k y 2 e n c
**  - - - - - - - - - - -
**
**  Evaluate a specified part of the chain of transformations involved
**  in pointing a telescope:
**
**
**            ICRS            star catalog RA,Dec + proper motion
**             |
**         aberration
**     precession-nutation
**             |
**            CIRS            RA,Dec of date (RA is wrt CIO)
**             |
**       Earth rotation
**             |
**           AZEL_T           topocentric (unrefracted) Az,El
**             |
**         refraction
**             |
**           AZEL_O           observed (refracted) Az,El
**             |
**       pointing model
**             |
**            ENC             encoder demands: where to aim the mount
**
**
**  The starting point may be any of the points in the chain marked as
**  ICRS, CIRS, AZEL_T and AZEL_O, and the returned directions comprise
**  all those below that point.  A common use would be to start with
**  star catalog position (marked as ICRS) and to use the final ENC
**  predicted encoder readings to command the mount, discarding all the
**  intervening results.
**
**  Given:
**     jbp     int     0 = normal attitude, else = beyond pole
**     pars   TRANSP*  transformation parameters
**     v       d[3]    starting vector
**     from   COSYS    starting system (ICRS, CIRS, AZEL_T or AZEL_O)
**
**  Conditionally returned:
**     vdate   d[3]    CIRS vector (i.e. CIO-based RA,Dec of date)
**     vtopo   d[3]    AZEL_T vector (i.e. topocentric Az,El)
**     vobs    d[3]    AZEL_O vector (i.e. observed Az,El)
**
**  Always returned:
**     a        d*     roll angle demand (radians)
**     b        d*     pitch angle demand (radians)
**
**  Returned (function value):
**              i      0 = OK
**                  else = illegal type
**
**  Notes:
**
**  1  The jbp argument is almost always false for an altaz, may
**     occasionally be true for a fork or horseshoe equatorial and
**     will be true half the time for mounts that have to be reversed
**     at the meridian such as a GEM.  When true, the "pitch" coordinate
**     (i.e. elevation or declination) will be outside the range +/-pi/2
**     and the "roll" coordinate (i.e. azimuth or hour angle) will be
**     pi different from the nominal value.
**
**  2  The transformation parameters pars must be set up before the
**     call.  See the functions cpkModel, cpkUpda and cpkUpdt.
**
**  3  The transformation neglects proper motion, parallax and light
**     deflection.  Using the functions cpkUpda etc., the accuracy of
**     the ICRS to AZEL_O portion is about 1 arcsecond for dates between
**     1950 and 2100, elevations above 25 deg and V color.
**
**  4  The accuracy of the remainder of the transformation, namely the
**     pointing corrections, depends on the size of the model
**     coefficients.  For typical values, the additional error is
**     unlikely to be as large as 1 arcsecond, except close to the pole
**     of the mounting.
**
**  5  The inverse transformations are performed by the function
**     cpkEnc2sky.  The two are not precisely complementary, but for
**     typical pointing coefficients should deliver consistency at the
**     1 arcsecond level.
**
**  6  All except the ENC results are 3-vectors.  The vector axes are as
**     follows:
**
**                ICRS          AZEL_T
**                CIRS          AZEL_O
**
**       x     RA=0h,Dec=0d      south
**       y     RA=6h,Dec=0d      east
**       z        Dec=90d       zenith
**
**  7  The starting vector v must be a unit vector except in the
**     AZEL_O case.  The returned vectors vdate, vtopo and vobs are not
**     in all cases guaranteed to be unit vectors.  However, the vectors
**     remain close to unit length, so that the successive returned
**     vectors can be passed back into the function without
**     renormalizing.
**
**  8  Which of the conditional vectors are returned depends on the
**     choice of coordinate system for the starting vector:
**
**                    CIRS   AZEL_T   AZEL_O     v
**
**        ICRS          Y       Y        Y
**        CIRS          N       Y        Y
**       AZEL_T         N       N        Y
**       AZEL_O         N       N        N
**
**        from
**
**  9  The returned demand angles are HA,Dec for the case of an
**     equatorial mount and Az,El for an altazimuth mount.  Hour angle
**     is zero on the meridian and west-positive.  Azimuth is zero in
**     the north and increases clockwise as seen from above.  The angles
**     may exceed normal ranges, and represent actual mechanical
**     attitude.
**
**  Called:  cpkC2s
**
**  Last revision:   5 March 2008
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

{
   double x, y, z, coz, x1, y1, tf, w, tx, ty, r, tc, tn,
          xe, ze, cd, sq, cq, tb, a, b, ta;



/* Unpack the given vector. */
   x = v[0];
   y = v[1];
   z = v[2];

/* Join the transformation chain at the right place. */
   switch ( from ) {

   case ICRS:

   /* Aberration. */
      x1 = x + pars->xab;
      y1 = y + pars->yab;
      z  = z + pars->zab;

   /* Precession-nutation. */
      x = pars->zcip*x1                 - pars->xcip*z;
      y =                            y1 - pars->ycip*z;
      z = pars->xcip*x1 + pars->ycip*y1 + pars->zcip*z;

   /* Return the CIRS vector. */
      vdate[0] = x;
      vdate[1] = y;
      vdate[2] = z;

   case CIRS:

   /* Celestial to terrestrial. */
      x1 =   pars->ct*x + pars->st*y;
      y  = - pars->st*x + pars->ct*y;

   /* Rotate into horizon coordinates. */
      x = pars->sp*x1 - pars->cp*z;
      z = pars->cp*x1 + pars->sp*z;

   /* Return the topocentric vector. */
      vtopo[0] = x;
      vtopo[1] = y;
      vtopo[2] = z;

   case AZEL_T:

   /* Approximate refraction, fixed below about 6 deg elevation. */
      z += pars->refr / ( z > 0.1 ? z : 0.1 );

   /* Normalize and return the observed vector. */
      w = sqrt ( x*x + y*y + z*z );
      vobs[0] = x /= w;
      vobs[1] = y /= w;
      vobs[2] = z /= w;

   case AZEL_O:
      break;

   default:
      return -1;
   }

/* Save the observed cos(ZD). */
   coz = z;

/* Vertical flexure terms. */
   tf = pars->pco[TF];
   w = tf*z;
   z += tf*(x*x+y*y);
   x -= w*x;
   y -= w*y;

/* Rotate into mount coordinates. */
   if ( pars->tmount == EQUAT ) {
      x1 =  pars->sp*x + pars->cp*z;
      z = - pars->cp*x + pars->sp*z;
      x = x1;

   /* Mount misalignment terms: equatorial. */
      tx = pars->pco[ME];
      ty = - pars->pco[MA];

   } else {

   /* Mount misalignment terms: altazimuth. */
      tx = pars->pco[AN];
      ty = pars->pco[AW];
   }
   x1 = x;
   y1 = y;
   x += tx*z;
   y += ty*z;
   z += - tx*x1 - ty*y1;

/* Cosine of pitch angle. */
   r = sqrt(x*x+y*y);

/* Collimation and nonperpendicularity terms. */
   if ( pars->tmount == EQUAT ) {

   /* Equatorial collim/nonperp terms. */
      tc =   pars->pco[DAF]*coz
           - pars->pco[CH]
           - pars->gc;
      tn = - pars->pco[NP];

   } else {

   /* Altazimuth collim/nonperp terms. */
      tc = pars->pco[CA]
         + pars->gc;
      tn = pars->pco[NPAE];

   /* Functions of parallactic angle (with zenith/pole precautions). */
      xe = x*pars->sp + z*pars->cp;
      ze = -x*pars->cp + z*pars->sp;
      cd = sqrt ( xe*xe + y*y);
      if ( cd == 0.0 || r == 0.0 ) {
         sq = 0.0;
         cq = 1.0;
      } else {
         sq = ( -y*pars->cp / cd ) / r;
         cq = ( cd*pars->sp - xe*ze*pars->cp / cd ) / r;
      }

   /* Apply altazimuth rotating collimation terms. */
      tc += ( - pars->pco[ID]*sq - pars->pco[CH]*cq );
      tb =  (   pars->pco[ID]*cq - pars->pco[CH]*sq );
      w = r != 0.0 ? tb*z/r : 0.0;
      x += w*x;
      y += w*y;
      z -= tb*r;

   }
   tc += z*tn;
   if ( jbp ) tc = -tc;
   w = r != 0.0 ? tc/r : 0.0;
   x1 = x;
   x += w*(y-w*x);
   y -= w*(x1+w*y);

/* To spherical. */
   cpkC2s ( x, y, z, &a, &b );

/* Deal with beyond pole case. */
   if ( jbp ) {
      a += D180;
      b = D180 - b;
   }

/* Encoder terms. */
   tb = pars->gb;
   if ( pars->tmount == EQUAT ) {
      tb += pars->pco[FO]*cos(a);
      w = fmod ( -a, D360 );
      if ( fabs(w) >= D180 ) w += w < 0.0 ? D360 : -D360;
      ta = pars->pco[IH];
      tb += pars->pco[ID];
   } else {
      ta = -1.0*pars->pco[HASA]*sin(a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HASA2]*sin(2.0*a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HASA3]*sin(3.0*a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HASA4]*sin(4.0*a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HASA5]*sin(5.0*a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HASA6]*sin(6.0*a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HASA7]*sin(7.0*a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HASA8]*sin(8.0*a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HACA]*cos(a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HACA2]*cos(2.0*a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HACA3]*cos(3.0*a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSSA] * sin(a)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSSA2] * sin(2.0*a)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSSA3] * sin(3.0*a)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSCA] * cos(a)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSCA2] * cos(2.0*a)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSCA3] * cos(3.0*a)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSSE] * sin(b)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSSE2] * sin(2.0*b)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSSE3] * sin(3.0*b)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSCE] * cos(b)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSCE2] * cos(2.0*b)/cos(b); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[HSCE3] * cos(3.0*b)/cos(b); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESE]*sin(b);
      tb += pars->pco[HECE]*cos(b); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA]*sin(a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA2]*sin(2.0*a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA3]*sin(3.0*a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA4]*sin(4.0*a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA5]*sin(5.0*a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA6]*sin(6.0*a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA7]*sin(7.0*a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA8]*sin(8.0*a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HESA9]*sin(9.0*a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HECA]*cos(a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HECA2]*cos(2.0*a); /* THIS CODE ADDED BY THEO */
      tb += pars->pco[HECA3]*cos(3.0*a); /* THIS CODE ADDED BY THEO */
      ta -= pars->pco[PAA1] * a; /* THIS CODE ADDED BY THEO */
      tb += pars->pco[PEE1] * b; /* THIS CODE ADDED BY THEO */
      tb += pars->pco[PEA1] * a; /* THIS CODE ADDED BY THEO */
      tb -= pars->pco[TX]/tan(b); /* THIS CODE ADDED BY THEO */
      w = fmod ( D180-a, D360 );
      if ( w < 0.0 ) w += D360;
      ta -= pars->pco[IA]; /* MODIFIED BY THEO */
      tb += pars->pco[IE];
   }
   *adem = w - ta;
   *bdem = b - tb;

   return 0;

}
