#include <stdio.h>
#include "cpk.h"

/*
**  - - - - - - -
**   t _ s t a r
**  - - - - - - -
**
**  Demonstrate how to point a telescope in a given place at a given
**  time at a given star.  The result is accurate to about 1 arcsec.
**  The reverse transformation (encoders to sky) is demonstrated also.
**
**  Site latitude and longitude     N 35d 12' 36"  W 111d 37' 12"
**  Star RA,Dec (ICRS, ~J2000)      05h 14m 32.27s  -08d 12' 05.9"
**  Date and time                   2006 December 28, 04h 05m 12s UT
**  Air pressure                    766 hPa
**
**  USNO website predictions:
**
**                                 Rigel
**
**                    Apparent Topocentric Positions
**                      Local Zenith and True North
**
**                          FLAGSTAFF, ARIZONA
**            Location:  W11137'12.0", N3512'36.0",  2300m
**              (Longitude referred to Greenwich meridian)
**
**     Date        Time                Zenith               Azimuth
**          (UT1)                     Distance              (E of N)
**               h  m   s                '   "               '   "
**  2006 Dec 28 04:05:12.0            53 09 53.8          138 17 15.0
**
**                                  = 36.83506 el       = 138.28750
**
**  Latest revision:   29 September 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
**
*/

int main ( )

#define D2PI 6.283185307179586     /* 2pi */
#define R2D (360.0/D2PI)           /* Radians to degrees */

{
   TRANSP pars;
   int j;
   double phi, elong, r2000, d2000, ut1, press,
          vicrs[3], vdate[3], vtopo[3], vobs[3],
          adem, bdem, a, b, cg, bg, pa;


/* Convert latitude and longitude to radians. */
   phi = cpkTday ( 35, 12, 36.0 ) * D2PI / 15.0;
   elong = - cpkTday ( 111, 37, 12.0 ) * D2PI / 15.0;

   printf("Phi = %.4f Elong = %.4f\n", phi/M_PI*180.0, elong/M_PI*180.0);

/* Convert RA,Dec to radians. */
   r2000 = cpkTday ( 5, 14, 32.27 ) * D2PI;
   d2000 = - cpkTday ( 8, 12, 5.9 ) * D2PI / 15.0;

/* UT1 MJD. */
   ut1 = cpkMjd ( 2006, 12, 28 ) + cpkTday ( 4, 5, 12.0 );

/* Pressure (hPa). */
   press = 766.0;

/* Report RA,Dec. */
   printf ( "\nCatalog:\n\n" );
   printf ( "  RA = %11.5f, Dec = %+9.5f  ICRS\n",
            r2000*R2D, d2000*R2D );

/* ------------------------------------ */
/* Initialize transformation parameters */
/* ------------------------------------ */

/* Read pointing model from file. */
   j = cpkModel ( "model.mod", &pars );
   switch ( j ) {
   case 0:
      break;
   case -1:
      printf ( "Unable to open model file!\n" );
      return -1;
   case -2:
      printf ( "I/O error or premature EOF!\n" );
      return -1;
   case -3:
      printf ( "Unsupported term!\n" );
      return -1;
   default:
      printf ( "Unknown error!\n" );
      return -1;
   }

/* Aberration, precession, nutation, refraction and Earth rotation. */
   cpkUpda ( ut1, press, elong, phi, &pars );

/* ---------------------------------- */
/* Test downstream/upstream functions */
/* ---------------------------------- */

/* J2000 RA,Dec to vector. */
   cpkS2v ( r2000, d2000, vicrs );

/* Downstream transformations. */
   printf ( "\nDownstream:\n\n" );
   cpkSky2enc ( 0, &pars, vicrs, ICRS,
                vdate, vtopo, vobs, &adem, &bdem );
   cpkV2s ( vdate, &a, &b );
   a *= R2D;
   b *= R2D;
   if ( a < 0.0 ) a += 360.0;
   printf ( "  RA = %11.5f, Dec = %+9.5f  date\n", a, b );
   cpkC2s ( -vtopo[0], vtopo[1], vtopo[2], &a, &b );
   a *= R2D;
   b *= R2D;
   if ( a < 0.0 ) a += 360.0;
   printf ( "  Az = %11.5f, El  = %9.5f  topo\n", a, b );
   cpkC2s ( -vobs[0], vobs[1], vobs[2], &a, &b );
   a *= R2D;
   b *= R2D;
   if ( a < 0.0 ) a += 360.0;
   printf ( "  Az = %11.5f, El  = %9.5f  obs\n", a, b );
   printf ( "\n" );
   printf ( "  A  = %+11.5f, B   = %+9.5f  enc\n", adem*R2D, bdem*R2D );

   printf ( "\nSame with guiding:\n\n" );
   cg = 0.001/R2D;
   bg = 0.002/R2D;
   printf ( "  dC = %+11.5f, dB  = %+9.5f\n\n", cg*R2D, bg*R2D );
   cpkGuide ( cg, bg, &pars );
   cpkSky2enc ( 0, &pars, vicrs, ICRS,
                vdate, vtopo, vobs, &adem, &bdem );
   printf ( "  A  = %+11.5f, B   = %+9.5f  enc\n", adem*R2D, bdem*R2D );

/* Upstream transformations. */
   printf ( "\nUpstream:\n\n" );
   printf ( "  A  = %+11.5f, B   = %+9.5f  enc\n", adem*R2D, bdem*R2D );
   printf ( "\n" );
   cpkEnc2sky ( &pars, adem, bdem, ICRS, vobs, vtopo, vdate, vicrs );
   cpkC2s ( -vobs[0], vobs[1], vobs[2], &a, &b );
   a *= R2D;
   b *= R2D;
   if ( a < 0.0 ) a += 360.0;
   printf ( "  Az = %11.5f, El  = %9.5f  obs\n", a, b );
   cpkC2s ( -vtopo[0], vtopo[1], vtopo[2], &a, &b );
   a *= R2D;
   b *= R2D;
   if ( a < 0.0 ) a += 360.0;
   printf ( "  Az = %11.5f, El  = %9.5f  topo\n", a, b );
   cpkV2s ( vdate, &a, &b );
   a *= R2D;
   b *= R2D;
   if ( a < 0.0 ) a += 360.0;
   printf ( "  RA = %11.5f, Dec = %+9.5f  date\n", a, b );
   printf ( "\n" );
   cpkV2s ( vicrs, &a, &b );
   a *= R2D;
   b *= R2D;
   if ( a < 0.0 ) a += 360.0;
   printf ( "  RA = %11.5f, Dec = %+9.5f  ICRS\n", a, b );
   printf ( "Delta= %11.5f        %9.5f Degrees\n", r2000*R2D-a, d2000*R2D-b);
   printf ( "Delta= %11.5f        %9.5f Arcsec\n",
            (r2000*R2D - a)*3600.0, (d2000*R2D-b)*3600.0 );

/* Rotator angle. */
   printf ( "\nRotator:\n\n" );
   cpkRot ( &pars, adem, bdem, ICRS, &pa );
   pa *= R2D;
   if ( pa < 0.0 ) pa += 360.0;
   printf ( "  PA = %+11.5f\n", pa );

   return 0;

}

/*-----------------------------------------------------------------------*/
