#include "cpk.h"
#include <stdio.h>

static void Dr2tf ( int, double, char*, int[4] );
static void Dr2af ( int, double, char*, int[4] );
static void Dd2tf ( int, double, char*, int[4] );

int cpkTpt ( TRANSP *pars, double vobs[3], double a, double b,
              char *string )

/*
**  - - - - - - -
**   c p k T p t
**  - - - - - - -
**
**  Format a pointing observation for TPOINT analysis.
**
**  Given:
**     pars   TRANSP*  transformation parameters
**     vobs    d[3]    AZEL_O vector (i.e. observed Az,El)
**     a        d*     roll angle demand (radians)
**     b        d*     pitch angle demand (radians)
**
**  Returned:
**     string   c*     formatted record (null terminated)
**
**  Returned (function value):
**              i      length of record (not counting the null)
**
**  Notes:
**
**    1  No validation of the arguments is performed.
**
**    2  The transformation parameters pars must be set up before the
**       call.  See the functions cpkModel and cpkUpda.  (The parts used
**       are the mount type and the functions of latitude.)
**
**    3  It is the caller's responsibility to provide a character
**       array big enough to receive the record.  The recommended length
**       is 81 characters.
**
**  Called:  cpkC2s
**
**  Last revision:   12 December 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

#define D2R 0.0174532925199432957692369  /* degrees to radians */

{
   int n, nf[4];
   char s;
   double x, y, z, r, d, az, el;



/* Initialize offset within string. */
   n = 0;

/* Mount type? */
   if ( pars->tmount == EQUAT ) {

   /* Equatorial:  express observed Az,El as RA,Dec. */
      x = vobs[0]*pars->sp + vobs[2]*pars->cp;
      y = vobs[1];
      z  = - vobs[0]*pars->cp + vobs[2]*pars->sp;
      cpkC2s ( x, y, z, &r, &d );

   /* Format star HA,Dec to TPOINT record (Format 1). */
      if ( r < 0.0 ) r += D360;
      Dr2tf ( 4, r, &s, nf );
      if ( nf[0] > 23 ) {
         nf[0] = 0;
         nf[1] = 0;
         nf[2] = 0;
         nf[3] = 0;
      }
      n += sprintf ( string+n, "%2.2i %2.2i %2.2i.%4.4i ",
                               nf[0], nf[1], nf[2], nf[3] );
      Dr2af ( 3, d, &s, nf );
      if ( nf[0] < 100 ) n += sprintf ( string+n, " " );
      n += sprintf ( string+n,
                     " %c%2.2i %2.2i %2.2i.%3.3i   ",
                     s, nf[0], nf[1], nf[2], nf[3] );

   /* Continue, using encoder HA,Dec. */
      r = -a;
      if ( r < 0.0 ) r += D360;
      Dr2tf ( 4, r, &s, nf );
      n += sprintf ( string+n, "%2.2i %2.2i %2.2i.%4.4i ",
                               nf[0], nf[1], nf[2], nf[3] );
      Dr2af ( 3, b, &s, nf );
      if ( nf[0] < 100 ) n += sprintf ( string+n, " " );
      n += sprintf ( string+n,
                     " %c%2.2i %2.2i %2.2i.%3.3i   00 00",
                     s, nf[0], nf[1], nf[2], nf[3] );

   } else {

   /* Altazimuth:  get observed Az,El. */
      cpkC2s ( -vobs[0], vobs[1], vobs[2], &az, &el );
      if ( az < 0.0 ) az += D360;

   /* Write TPOINT record (Format 4). */
      n += sprintf ( string+n, "%10.6f %10.6f %10.6f %10.6f",
                               az/D2R, el/D2R, a/D2R, b/D2R );

   }

   return n;

}

/*--------------------------------------------------------------------*/

static void Dr2tf ( int ndp, double angle, char *sign, int ihmsf[4] )

/*
**  - - - - - -
**   D r 2 t f
**  - - - - - -
**
**  Convert an angle in radians to hours, minutes, seconds.
**
**  Given:
**     ndp       int          number of decimal places of seconds
**     angle     double       angle in radians
**
**  Returned:
**     sign      char*        '+' or '-'
**     ihmsf     int[4]       hours, minutes, seconds, fraction
**
**  Called:  Dd2tf
**
**  Last revision:   3 January 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/
{

/* Scale then use days to h,m,s function. */
   Dd2tf ( ndp, angle / D360, sign, ihmsf );

}

/*--------------------------------------------------------------------*/

static void Dr2af ( int ndp, double angle, char *sign, int idmsf[4] )

/*
**  - - - - - -
**   D r 2 a f
**  - - - - - -
**
**  Convert an angle in radians to degrees, arcminutes, arcseconds.
**
**  Given:
**     ndp       int          number of decimal places of arcseconds
**     angle     double       angle in radians
**
**  Returned:
**     sign      char*        '+' or '-'
**     idmsf     int[4]       degrees, arcminutes, arcseconds, fraction
**
**  Called:  Dd2tf
**
**  Last revision:   3 January 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/
{

/* Scale then use days to h,m,s function. */
   Dd2tf ( ndp, angle * 15.0 / D360, sign, idmsf );

}

/*--------------------------------------------------------------------*/

static void Dd2tf ( int ndp, double days, char *sign, int ihmsf[4] )

/*
**  - - - - - -
**   D d 2 t f
**  - - - - - -
**
**  Convert an interval in days into hours, minutes, seconds.
**
**  (double precision)
**
**  Given:
**     ndp       int      number of decimal places of seconds
**     days      double   interval in days
**
**  Returned:
**     *sign     char     '+' or '-'
**     ihmsf     int[4]   hours, minutes, seconds, fraction
**
**  Last revision:   3 January 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

#define gmax(A,B) ((A)>(B)?(A):(B))
#define dint(A) ((A)<0.0?ceil(A):floor(A))
#define dnint(A) ((A)<0.0?ceil((A)-0.5):floor((A)+0.5))

{
   double rs, rm, rh, a, ah, am, as, af;


/* Handle sign. */
   *sign = (char) ( ( days < 0.0 ) ?  '-' : '+' );

/* Field units in terms of least significant figure. */
   rs = pow ( 10.0, (double) gmax ( ndp, 0 ) );
   rs = dint ( rs );
   rm = rs * 60.0;
   rh = rm * 60.0;

/* Round interval and express in smallest units required. */
   a = rs * 86400.0 * fabs ( days );
   a = dnint ( a );

/* Separate into fields. */
   ah = a / rh;
   ah = dint ( ah );
   a  = a - ah * rh;
   am = a / rm;
   am = dint ( am );
   a  = a - am * rm;
   as = a / rs;
   as = dint ( as );
   af = a - as * rs;

/* Return results */
   ihmsf[0] = (int) ah;
   ihmsf[1] = (int) am;
   ihmsf[2] = (int) as;
   ihmsf[3] = (int) af;

}

/*--------------------------------------------------------------------*/
