#include "cpk.h"

void cpkUpda ( double ut1, double press, double elong, double phi,
               TRANSP *pars )

/*
**  - - - - - - - -
**   c p k U p d a
**  - - - - - - - -
**
**  Update the astrometric portion of the transformation parameters.
**
**  Given:
**     ut1      d        UT1 as Modified Julian Date (JD-2400000.5)
**     press    d        air pressure (hPa)
**     elong    d        longitude (radians, E +ve)
**     phi      d        latitude (radians)
**     pars   TRANSP*    transformation parameters
**
**  Returned:
**     pars   TRANSP*    updated
**
**  Notes:
**
**    1  No validation of the arguments is performed.
**
**    2  The main purpose of this function is to update the aberration,
**       precession and nutation quantities, which change very slowly.
**       In addition, the refraction is renewed to take account of any
**       change in air pressure and, for completeness, the rapidly
**       changing Earth rotation component is also updated.
**
**    3  This function need be called only occasionally, say once per
**       slew or even once per night.  However, during tracking,
**       continuous updates to the transformation parameters are
**       required, using the cpkUpdt function, in order to account for
**       Earth rotation.
**
**  Called:  cpkUpdt
**
**  Last revision:   21 March 2008
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

{
   double t, om, x, sl, c;



/* Site coordinates. */
   pars->elon = elong;
   pars->sp = sin(phi);
   pars->cp = cos(phi);

/* Days since J2000. */
   t = ut1 - 51544.5;

/* Longitude of Moon's ascending node. */
   om = 2.182 - 9.242e-4*t;

/* CIP coordinates. */
   x = 2.66e-7*t - 33.2e-6*sin(om);
   pars->xcip = x;
   pars->ycip = -8.14e-14*t*t + 44.6e-6*cos(om);
   pars->zcip = 1.0 - x*x/2.0;

/* Sun's geometric longitude. */
   sl = 4.895 + 1.72021e-2*t;

/* Aberration vector. */
   c = cos(sl);
   pars->xab = 0.99e-4*sin(sl);
   pars->yab = -0.91e-4*c;
   pars->zab = -0.40e-4*c;

/* Refraction constant. */
   pars->refr = PFAC*press;

/* Local ERA. */
   cpkUpdt ( ut1, pars );

}
