/************************************************************************/
/* esp_client.c								*/
/*                                                                      */
/* Message handling routines.						*/
/* WORKS FOR SINGLE ESP UNIT AT A TIME.					*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar (but mostly taken from espgtk)          */
/* Date   : May 2007                                                    */
/************************************************************************/

#include "../cliserv/include/cliserv.h"
#include "../include/chara_messages.h"
#include "../charalib/chara.h"
#include "espclient.h"

/* Globals */

int esp_num_axes[MAX_SERVERS];
struct AXIS esp_stage[MAX_SERVERS][ESP_MAX_STAGE];
char *esp_port_names[MAX_SERVERS][ESP_MAX_STAGE];
float esp_port_min[MAX_SERVERS][ESP_MAX_STAGE];
float esp_port_max[MAX_SERVERS][ESP_MAX_STAGE];
char *esp_unit_name[MAX_SERVERS];
unsigned char esp_status[MAX_SERVERS];
char esp_power_status_bit[MAX_SERVERS];
char esp_motion_status_bit[MAX_SERVERS];
#warning Some of these should be arrays
char esp_update_power_status[MAX_SERVERS];
char esp_update_motion_status[MAX_SERVERS];
char esp_update_position[MAX_SERVERS][ESP_MAX_STAGE];
char esp_update_velocity[MAX_SERVERS][ESP_MAX_STAGE];
char esp_update_acceleration[MAX_SERVERS][ESP_MAX_STAGE];
char esp_update_minmax[MAX_SERVERS][ESP_MAX_STAGE];

#warning Axis always starts with 0.

#warning Beam always starts on 1

/************************************************************************/
/* open_esp_client()							*/
/*									*/
/* Guess what this does.						*/
/* Assumes that init_jobs has been called. It will call it anyway,	*/
/* which should do no harm if it has already been called.		*/
/* Returns server index or -1 if it fails.				*/
/************************************************************************/

int open_esp_client(char *server_name)
{
        struct smessage mess;
	time_t	 start;
	int	i;
	int	esp_server;

	/* This is for safety and should be harmless */

	init_jobs();

	/*
	 * Attempt to open the socket
	 */

	if ((esp_server = open_server_by_name(server_name)) == -1) return -1;

	/* Fill in the jobs messages we need */

        if (!add_message_job(esp_server, SERVER_STOPPING, 
			message_esp_server_stopping) ||
		!add_message_job(esp_server, ESP_STATUS, message_esp_status) || 
		!add_message_job(esp_server, ESP_NAMES, message_esp_names) || 
		!add_message_job(esp_server, ESP_MINMAX, message_esp_minmax) || 
		!add_message_job(esp_server, ESP_POWER_STATUS, 
			message_esp_power) || 
		!add_message_job(esp_server, ESP_MOTION_DONE_STATUS, 
			message_esp_motion_done) ||
		!add_message_job(esp_server, ESP_POSITION, 
			message_esp_position) || 
		!add_message_job(esp_server, ESP_VELOCITY, 
			message_esp_velocity) || 
		!add_message_job(esp_server,ESP_ACCELERATION, 
			message_esp_acceleration) || 
		!add_message_job(esp_server,CENTRAL_SCRUTINIZER_LOG,
			message_do_nothing) ||
        	!add_message_job(esp_server,CENTRAL_SCRUTINIZER_STOP_JOB_QUEUE,
			message_do_nothing))
	{
		close_esp_client(esp_server);
		return -3;
	}

	/*
	 * Reset the unit name.
	 */
	esp_unit_name[esp_server]=NULL;

	/*
	 * Send out requests for machine name and port names
	 */

        mess.type = ESP_GET_NAMES;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(esp_server, &mess)) return -2;

	/*
	 * Now let's wait a while and see if we get an answer.
	 */

	start = time(NULL);
	while(time(NULL) < start+TIME_OUT && esp_unit_name[esp_server] == NULL)
		process_server_socket(esp_server);

	if (esp_unit_name[esp_server] == NULL) return -1;

	/* Initialize things */

	esp_update_power_status[esp_server] = FALSE;
	esp_update_motion_status[esp_server] = FALSE;
	for(i=0; i<esp_num_axes[esp_server]; i++)
	{
		esp_update_position[esp_server][i] = FALSE;
		esp_update_velocity[esp_server][i] = FALSE;
		esp_update_acceleration[esp_server][i] = FALSE;
		esp_update_minmax[esp_server][i] = FALSE;
		esp_stage[esp_server][i].num = i+1;
		esp_get_position(esp_server, i);
		esp_get_minmax(esp_server, i);
		esp_get_velocity(esp_server, i);
		esp_get_acceleration(esp_server, i);
	}
	esp_get_power_status(esp_server);
	esp_get_motion_status(esp_server);

	/* All should be well */

	return esp_server;

} /* open_esp_client() */

/************************************************************************/
/* close_esp_client()							*/
/*									*/
/************************************************************************/

void close_esp_client(int esp_server)
{
	int	i;

	if (!server_socket_open(esp_server)) return;

	if (server_socket_open(esp_server))
	{
		close_server_socket(esp_server);
		esp_num_axes[esp_server] = 0;
		esp_unit_name[esp_server] = 0;
		esp_status[esp_server] = 0;
		esp_power_status_bit[esp_server]= 0;
		esp_motion_status_bit[esp_server] = 0;
		esp_update_power_status[esp_server] = FALSE;
		esp_update_motion_status[esp_server] = FALSE;
		for(i=0; i<ESP_MAX_STAGE; i++)
		{
			esp_update_position[esp_server][i] = FALSE;
			esp_update_velocity[esp_server][i] = FALSE;
			esp_update_acceleration[esp_server][i] = FALSE;
			esp_update_minmax[esp_server][i] = FALSE;
		}
	}

} /* close_esp_client() */

/************************************************************************/
/* message_esp_server_stopping()					*/
/*									*/
/************************************************************************/

int message_esp_server_stopping(int server, struct smessage *mess)
{
	close_esp_client(server);

        return TRUE;

} /* message_esp_server_stopping() */

/************************************************************************/
/* message_esp_status()							*/
/*									*/
/************************************************************************/

int message_esp_status(int server, struct smessage *mess)
{
        if (mess->length != sizeof(char)) return FALSE;

	esp_status[server] = (unsigned char) (*mess->data);

        return TRUE;

} /* message_esp_status() */

/************************************************************************/
/* message_esp_position()						*/
/*									*/
/************************************************************************/

int message_esp_position(int server, struct smessage *mess)
{
	struct AXIS *astage;

        if (mess->length != sizeof(*astage)) return FALSE;

	astage = (struct AXIS *)mess->data;

	if (astage->num <= 0 || astage->num > esp_num_axes[server])
		return FALSE;

	esp_stage[server][astage->num-1].real_position=astage->real_position;
	esp_stage[server][astage->num-1].beam=astage->beam;

	esp_update_position[server][astage->num-1]=TRUE;

        return TRUE;

} /* message_esp_position() */

/************************************************************************/
/* message_esp_velocity()						*/
/*									*/
/************************************************************************/

int message_esp_velocity(int server, struct smessage *mess)
{
	struct AXIS *astage;

        if (mess->length != sizeof(*astage)) return FALSE;

	astage = (struct AXIS *)mess->data;

	if (astage->num <= 0 || astage->num > esp_num_axes[server])
		return FALSE;

	esp_stage[server][astage->num-1].velocity=astage->velocity;

	esp_update_velocity[server][astage->num-1]=TRUE;

        return TRUE;

} /* message_esp_velocity() */

/************************************************************************/
/* message_esp_acceleration()						*/
/*									*/
/************************************************************************/

int message_esp_acceleration(int server, struct smessage *mess)
{
	struct AXIS *astage;

        if (mess->length != sizeof(*astage))
        {
		return FALSE;
        }

	astage = (struct AXIS *)mess->data;

	if (astage->num <= 0 || astage->num > esp_num_axes[server])
		return FALSE;

	esp_stage[server][astage->num-1].acceleration=astage->acceleration;

	esp_update_acceleration[server][astage->num-1]=TRUE;

        return TRUE;

} /* message_esp_acceleration() */

/************************************************************************/
/* message_esp_power()							*/
/* 									*/
/* Receives the power status bit from the server.			*/
/*									*/
/************************************************************************/

int message_esp_power(int server, struct smessage *mess)
{
        if (mess->length != sizeof(char))
        {
		return TRUE;
        }

	/*
	 * Put the reply into a global
	 */
	esp_power_status_bit[server] = *mess->data;

	esp_update_power_status[server] = TRUE;

        return TRUE;

} /* message_esp_power() */

/************************************************************************/
/* message_esp_motion_done()						*/
/*									*/
/************************************************************************/

int message_esp_motion_done(int server, struct smessage *mess)
{

        if (mess->length != sizeof(char))
        {
		return FALSE;
        }

	/*
	 * Put the reply into a global
	 */

	esp_motion_status_bit[server] = *mess->data;

	esp_update_motion_status[server]=TRUE;

        return TRUE;

} /* message_esp_motion() */

/************************************************************************/
/* message_esp_minmax()							*/
/*									*/
/************************************************************************/

int message_esp_minmax(int server, struct smessage *mess)
{
	struct AXIS *astage;

        if (mess->length != sizeof(*astage)) return FALSE;

	/*
	 * Put the reply into a global
	 */

	astage = (struct AXIS *)mess->data;

	if (astage->num <= 0 || astage->num > esp_num_axes[server])
		return FALSE;

	esp_port_min[server][astage->num-1]=astage->min;
	esp_port_max[server][astage->num-1]=astage->max;

	esp_update_minmax[server][astage->num-1] = TRUE;

        return TRUE;

} /* message_esp_minmax() */

/************************************************************************/
/* message_esp_names()							*/
/*									*/
/************************************************************************/

int message_esp_names(int server, struct smessage *mess)
{
	char	s[256],*p1,*p2;
	int	i, count;

	/* We're hoping there's the right kind of data here */

	p1 = (char *)mess->data;

	if (esp_unit_name[server] != NULL) free(esp_unit_name[server]);

	for(p2 = s; 
		*p1 != 0 && (p1-(char*)mess->data)<=mess->length; 
		p1++, p2++) *p2 = *p1;
	*p2 = 0;
	p1++;

	if ((esp_unit_name[server] = malloc(strlen(s))) == NULL) return FALSE;

	strcpy(esp_unit_name[server], s);

	count=0;
	
	for(i=0; i<6 ; i++)
	{
		if (esp_port_names[server][i] != NULL) 
			free(esp_port_names[server][i]);

		for(p2 = s; 
			*p1 != 0 && ((p1-(char*)mess->data)<=mess->length-1); 
			p1++, p2++) *p2 = *p1;
		*p2 = 0;
		p1++;

	
		if ((esp_port_names[server][i] = 
			malloc(strlen(s))) == NULL) return 
			FALSE;

		strcpy(esp_port_names[server][i], s);

		if (strcmp(s,"")) count++;

	}


	/*
	 * Set the global variable with the number of axes
	 */

	esp_num_axes[server]=count;

	if (esp_num_axes[server] > ESP_MAX_STAGE) return FALSE;

	return TRUE;

} /* message_esp_names() */

/************************************************************************/
/* send_esp_on()							*/
/* 									*/
/* Send the ESP_ON message to the server.				*/
/* Returns an error level.						*/
/************************************************************************/

int send_esp_on(int esp_server, int axis)
{
	struct smessage	mess;
	struct AXIS astage;

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Construct command */

	mess.type = ESP_ON;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

	if (!send_message(esp_server, &mess)) return FALSE;

	return TRUE;

} /* send_esp_on() */

/************************************************************************/
/* send_esp_off()							*/
/* 									*/
/* Send the ESP_OFF message to the server.				*/
/* Returns an error level.						*/
/************************************************************************/

int send_esp_off(int esp_server, int axis)
{
	struct smessage	mess;
	struct AXIS astage;

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Construct command */

	mess.type = ESP_OFF;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

	if (!send_message(esp_server, &mess)) return FALSE;

	return TRUE;

} /* send_esp_off() */

/************************************************************************/
/* send_esp_move_absolute()						*/
/* 									*/
/* Send the ESP_MOVE_ABS message to the server.				*/
/* Returns an error level.						*/
/************************************************************************/

int send_esp_move_absolute(int esp_server, int axis, double position)
{
	struct smessage	mess;
	struct AXIS astage;

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;
	astage.position = position;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Construct command */

	mess.type = ESP_MOVE_ABS;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

	if (!send_message(esp_server, &mess)) return FALSE;

	return TRUE;

} /* send_esp_move_absolute() */

/************************************************************************/
/* send_esp_set_beam()                                                  */
/*                                                                      */
/* Send the ESP_SET_BEAM message to the server.                         */
/* Returns an error level.                                              */
/* Beam goes from 1 to 6.						*/
/************************************************************************/

int send_esp_set_beam(int esp_server, int axis, int beam)
{
        struct smessage mess;
	struct AXIS astage;

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;

	if (beam < 1 || beam > NUM_BEAMS) return FALSE;
	astage.beam = beam;

        /* Make sure everything is OK */

        if (!server_socket_open(esp_server)) return FALSE;

        /* Construct command */

        mess.type = ESP_SET_BEAM;
        mess.data = (unsigned char *)&astage;
        mess.length = sizeof(astage);

        if (!send_message(esp_server, &mess)) return FALSE;

        return TRUE;

} /* send_esp_set_beam() */

/************************************************************************/
/* send_esp_set_velocity()						*/
/* 									*/
/* Send the SET_VELOCITY message to the server.				*/
/* Returns an error level.						*/
/************************************************************************/

int send_esp_set_velocity(int esp_server, int axis, double velocity)
{
	struct smessage	mess;
	struct AXIS astage;

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;
	astage.velocity = velocity;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Construct command */

	mess.type = ESP_SET_VELOCITY;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

	if (!send_message(esp_server, &mess)) return FALSE;

	return TRUE;

} /* send_esp_set_velocity() */

/************************************************************************/
/* send_esp_set_acceleration()						*/
/* 									*/
/* Send the SET_ACCERLATION message to the server.			*/
/* Returns an error level.						*/
/************************************************************************/

int send_esp_set_acceleration(int esp_server, int axis, double acceleration)
{
	struct smessage	mess;
	struct AXIS astage;

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;
	astage.acceleration = acceleration;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Construct command */

	mess.type = ESP_SET_ACCELERATION;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

	if (!send_message(esp_server, &mess)) return FALSE;

	return TRUE;

} /* send_esp_set_acceleration() */

/************************************************************************/
/* send_esp_move_relative()						*/
/* 									*/
/* Send the ESP_MOVE_REL message to the server.				*/
/* Returns an error level.						*/
/************************************************************************/

int send_esp_move_relative(int esp_server, int axis, double relative_step_size)
{
	struct smessage	mess;
	struct AXIS astage;

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;
	astage.relative_step_size = relative_step_size;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Construct command */

	mess.type = ESP_MOVE_REL;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

	if (!send_message(esp_server, &mess)) return FALSE;

	return TRUE;

} /* send_esp_move_absolute() */

/************************************************************************/
/* send_esp_move_home()							*/
/* 									*/
/* Send the ESP_MOVE_HOME message to the server.			*/
/* Returns an error level.						*/
/************************************************************************/

int send_esp_move_home(int esp_server, int axis)
{
	struct smessage	mess;
	struct AXIS astage;

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Construct command */

	mess.type = ESP_MOVE_HOME;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

	if (!send_message(esp_server, &mess)) return FALSE;

	return TRUE;

} /* send_esp_move_home() */

/************************************************************************/
/* send_esp_text()							*/
/* 									*/
/* Send the ESP_TEXT message to the server.				*/
/* Returns an error level.						*/
/************************************************************************/

int send_esp_text(int esp_server, char *entry_text)
{
	struct smessage	mess;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Construct command */

	mess.type = ESP_TEXT;
	mess.length = strlen(entry_text);
	mess.data = (unsigned char *)entry_text;

	if (!send_message(esp_server, &mess)) return FALSE;

	return TRUE;

} /* send_esp_text() */

/************************************************************************/
/* esp_get_position()							*/
/*									*/
/* Tries to get the current position of a given axis.			*/
/* Result is placed in relevant esp_stage array.			*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_get_position(int esp_server, int axis)
{
        struct smessage mess;
	struct AXIS astage;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Does the axis make sense? */

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;

	/* Send the request */

	mess.type = ESP_GET_POSITION;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

        return send_message(esp_server, &mess);

} /* esp_get_position() */

/************************************************************************/
/* esp_wait_position()							*/
/*									*/
/* Same as esp_get_position but waits for a replay.			*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_wait_position(int esp_server, int axis)
{
	time_t	 start;

	/* Start off not knowing */

	esp_update_position[esp_server][axis] = FALSE;

	/* Send the message */

	if (!esp_get_position(esp_server,axis)) return FALSE;

	/* Now let's wait a while and see if we get an answer. */

	start = time(NULL);
	while(time(NULL) < start+TIME_OUT && 
	    !esp_update_position[esp_server][axis])
		process_server_socket(esp_server);

	/* Did that work? */

	return esp_update_position[esp_server][axis];

} /* esp_wait_position() */

/************************************************************************/
/* esp_get_motion_status()						*/
/*									*/
/* Tries to get the current motion status for all axes.			*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_get_motion_status(int esp_server)
{
        struct smessage mess;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Send the request */

	mess.type = ESP_GET_MOTION_DONE_STATUS;
	mess.data = NULL;
	mess.length = 0;

        return send_message(esp_server, &mess);

} /* esp_get_motion_status() */

/************************************************************************/
/* esp_wait_motion_status()						*/
/*									*/
/* Same as esp_get_motion_status but will wait for a respose.		*/
/* Result is placed in relevant global.					*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_wait_motion_status(int esp_server)
{
	time_t	 start;

	/* We start off knowing nothing */

	esp_update_motion_status[esp_server] = FALSE;

	/* Send the message */

	if (!esp_get_motion_status(esp_server)) return FALSE;

	/* Now let's wait a while and see if we get an answer. */

	start = time(NULL);
	while(time(NULL) < start+TIME_OUT &&
  	    !esp_update_motion_status[esp_server])
		process_server_socket(esp_server);

	/* Did that work? */

	return esp_update_motion_status[esp_server];

} /* esp_wait_motion_status() */

/************************************************************************/
/* esp_get_power_status()						*/
/*									*/
/* Tries to get the current power status for all axes.			*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_get_power_status(int esp_server)
{
        struct smessage mess;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Send the request */

	mess.type = ESP_GET_POWER_STATUS;
	mess.data = NULL;
	mess.length = 0;

        return send_message(esp_server, &mess);

} /* esp_get_power_status() */

/************************************************************************/
/* esp_wait_power_status()						*/
/*									*/
/* Same as esp_get_power_status() but waits for a response.		*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_wait_power_status(int esp_server)
{
	time_t	 start;

	/* We start off knowing nothing */

	esp_update_power_status[esp_server] = FALSE;

	/* Send the request */

	if (!esp_get_power_status(esp_server)) return FALSE;

	/* Now let's wait a while and see if we get an answer. */

	start = time(NULL);
	while(time(NULL) < start+TIME_OUT && 
	    !esp_update_power_status[esp_server])
		process_server_socket(esp_server);

	/* Did that work? */

	return esp_update_power_status[esp_server];

} /* esp_wait_power_status() */

/************************************************************************/
/* esp_get_minmax()							*/
/*									*/
/* Tries to get the current minmax of a given axis.			*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_get_minmax(int esp_server, int axis)
{
        struct smessage mess;
	struct AXIS astage;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Does the axis make sense? */

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;

	/* Send the request */

	mess.type = ESP_GET_MINMAX;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

        return send_message(esp_server, &mess);

} /* esp_get_minmax() */

/************************************************************************/
/* esp_wait_minmax()							*/
/*									*/
/* Same as esp_get_minmax() but waits for a response.			*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_wait_minmax(int esp_server, int axis)
{
	time_t	 start;

	/* We start off knowing nothing */

	esp_update_minmax[esp_server][axis] = FALSE;

	/* Send the request */

	if (!esp_get_minmax(esp_server, axis)) return FALSE;

	/* Now let's wait a while and see if we get an answer. */

	start = time(NULL);
	while(time(NULL) < start+TIME_OUT && !esp_update_minmax[esp_server][axis])
		process_server_socket(esp_server);

	/* Did that work? */

	return esp_update_minmax[esp_server][axis];

} /* esp_wait_minmax() */

/************************************************************************/
/* esp_get_velocity()							*/
/*									*/
/* Tries to get the current velocity of a given axis.			*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_get_velocity(int esp_server, int axis)
{
        struct smessage mess;
	struct AXIS astage;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Does the axis make sense? */

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;

	/* Send the request */

	mess.type = ESP_GET_VELOCITY;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

        return send_message(esp_server, &mess);

} /* esp_get_velocity() */

/************************************************************************/
/* esp_wait_velocity()							*/
/*									*/
/* Same as esp_get_veolicty() but waits for a response.			*/
/* Result is placed in relevant esp_stage array.			*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_wait_velocity(int esp_server, int axis)
{
	time_t	 start;

	/* We start off knowing nothing */

	esp_update_velocity[esp_server][axis] = FALSE;

	/* Send the request */

        if (!esp_get_velocity(esp_server, axis)) return FALSE;

	/* Now let's wait a while and see if we get an answer. */

	start = time(NULL);
	while(time(NULL) < start+TIME_OUT &&
	    !esp_update_velocity[esp_server][axis])
		process_server_socket(esp_server);

	/* Did that work? */

	return esp_update_velocity[esp_server][axis];

} /* esp_wait_velocity() */

/************************************************************************/
/* esp_get_acceleration()						*/
/*									*/
/* Tries to get the current acceleration of a given axis.		*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_get_acceleration(int esp_server, int axis)
{
        struct smessage mess;
	struct AXIS astage;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	/* Does the axis make sense? */

	if (axis < 0 || axis >= esp_num_axes[esp_server]) return FALSE;
	astage.num = axis+1;

	/* Send the request */

	mess.type = ESP_GET_ACCELERATION;
	mess.data = (unsigned char *)&astage;
	mess.length = sizeof(astage);

        return send_message(esp_server, &mess);

} /* esp_get_acceleration() */

/************************************************************************/
/* esp_wait_acceleration()						*/
/*									*/
/* Same as esp_get_acceleration() but waits for a response.		*/
/* Returns TRUE if it goes well and FALSE if it doesn't.		*/
/************************************************************************/

int esp_wait_acceleration(int esp_server, int axis)
{
	time_t	 start;

	/* We start off knowing nothing */

	esp_update_acceleration[esp_server][axis] = FALSE;

	/* Send the request */

        if (!esp_get_acceleration(esp_server, axis)) return FALSE;

	/* Now let's wait a while and see if we get an answer. */

	start = time(NULL);
	while(time(NULL) < start+TIME_OUT &&
	    !esp_update_acceleration[esp_server][axis])
		process_server_socket(esp_server);

	/* Did that work? */

	return esp_update_acceleration[esp_server][axis];

} /* esp_wait_acceleration() */

/************************************************************************/
/* esp_update()								*/
/*									*/
/* Sends out messages to try and update all axes.			*/
/* Returns TRUE if all goes well, FALSE otherwise.			*/
/************************************************************************/

int esp_update(int esp_server)
{
	int	axis;

	/* Make sure everything is OK */

	if (!server_socket_open(esp_server)) return FALSE;

	if (!esp_get_motion_status(esp_server)) return FALSE;
	if (!esp_get_power_status(esp_server)) return FALSE;
	for (axis = 0; axis < esp_num_axes[esp_server]; axis++)
	{
		if (!esp_get_position(esp_server, axis)) return FALSE;
		if (!esp_get_minmax(esp_server, axis)) return FALSE;
		if (!esp_get_velocity(esp_server, axis)) return FALSE;
		if (!esp_get_acceleration(esp_server, axis)) return FALSE;
	}

	return TRUE;

} /* esp_update() */
