/************************************************************************/
/* esp_ui_client.c							*/
/*                                                                      */
/* UI interface for ESP library routines.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar (but mostly taken from espgtk)          */
/* Date   : May 2007                                                    */
/************************************************************************/

#include <math.h>
#include "../charaui/src/charaui.h"
#include "../cliserv/include/cliserv.h"
#include "../include/chara_messages.h"
#include "../charalib/chara.h"
#include "espclient.h"
#include "espuiclient.h"

char *esp_types[NUM_ESP_TYPES+1] = {"BS   ", "BC1  ", "BC2  ", "BC3  ",
				    "PERI1", 
			   	    "PERI2", "LDC1 ", "LDC2 ", "RETRO",
				    NULL};

int ui_esp_servers[NUM_ESP_TYPES] = {-1,-1,-1,-1,-1,-1,-1,-1,-1};

/************************************************************************/
/* ui_open_esp_client()							*/
/*									*/
/* Calles routine to open connection to an ESP server.			*/
/* Returns server, or -1. SHOULD NOT BE USED AS A UI FUNCTION, IT SHOULD*/
/* BE CALLED BY A UI FUNCTION IN THE LOCAL PROGRAM.			*/
/************************************************************************/

int ui_open_esp_client(int argc, char **argv)
{
	int	unit;
	char	name[81],*p;
	int	server;

	/* Check out the command line */

	clean_command_line();
        if (argc > 1)
        {
		strcpy(name, argv[1]);
		while(strlen(name) != strlen(esp_types[0]))
			strcat(name," ");
                for (unit=0; unit<NUM_ESP_TYPES; unit++)
                {
                        if (strcmpi(name,esp_types[unit]) == 0)
                        {
                                break;
                        }
                }

                if (unit >= NUM_ESP_TYPES)
                {
                        return error(ERROR,"Unknown ESP unit %s.",argv[1]);
                }
        }
        else
	{
		unit = 0;
                if (quick_edit("ESP Server",esp_types[unit],
                        &unit,esp_types,ENUMERATED) == KEY_ESC)
                                return NOERROR;
		strcpy(name, esp_types[unit]);
	}

	/* Strip off leading and trailing spaces */

	for(p = name; *p == ' ' && *p != 0; p++);
	for(; *p != ' ' && *p != 0; p++);
	*p = 0;
	for(p = name; *p == ' ' && *p != 0; p++);

	/* Send the command */

	if ((server = open_esp_client(p)) == -1)
	{
		return error(ERROR, "Failed to open ESP unit %s.", p);
	}

	/* Set the globals for the servers */

	ui_esp_servers[unit] = server;

	return server;

} /* ui_open_esp_client() */

/************************************************************************/
/* call_close_esp_client()						*/
/*									*/
/* UI callable version of close_esp_client().				*/
/************************************************************************/

int call_close_esp_client(int argc, char **argv)
{
	int	server;

	if ((server = esp_get_server(argc, argv)) == -1) return NOERROR;

	close_esp_client(server);

	return NOERROR;

} /* call_close_esp_client() */

/************************************************************************/
/* call_send_esp_on()							*/
/*									*/
/* UI callable version of send_esp_on().				*/
/************************************************************************/

int call_send_esp_on(int argc, char **argv)
{
	int	axis;
	int	server;

	if(!esp_get_server_and_axis(argc, argv, &server, &axis)) return NOERROR;

	if (!send_esp_on(server, axis))
		return error(ERROR,"Failed to turn ON axis %d on ESP unit %s.",
			axis+1, esp_unit_name);

	return NOERROR;

} /* call_send_esp_on() */

/************************************************************************/
/* call_send_esp_off()							*/
/*									*/
/* UI callable version of send_esp_off().				*/
/************************************************************************/

int call_send_esp_off(int argc, char **argv)
{
	int	axis;
	int	server;

	if(!esp_get_server_and_axis(argc, argv, &server, &axis)) return NOERROR;

	if (!send_esp_off(server, axis))
		return error(ERROR,"Failed to turn OFF axis %d on ESP unit %s.",
			axis+1, esp_unit_name);

	return NOERROR;

} /* call_send_esp_off() */

/************************************************************************/
/* call_send_esp_move_absolute()					*/
/*									*/
/* UI callable version of send_esp_move_absolute().			*/
/************************************************************************/

int call_send_esp_move_absolute(int argc, char **argv)
{
	int	axis;
	double  position;
	char    s[81], m[81];
	int	server;

	if(!esp_get_server_and_axis(argc, argv, &server, &axis)) return NOERROR;

	if (argc > 3)
        {
                sscanf(argv[3],"%lf",&position);
        }
        else
        {
		if (!esp_wait_position(server, axis))
			return error(ERROR, 
			   "Failed to get axis %d position for ESP unit %s.",
			   axis+1, esp_unit_name[server]);
		sprintf(m,"%s abosolute position", esp_unit_name[server]);
		sprintf(s,"%.3f", esp_stage[server][axis].real_position);
                if (quick_edit(m,s,s,NULL,FLOAT) == KEY_ESC)
                        return -1;
                sscanf(s,"%lf",&position);
        }

	if (!send_esp_move_absolute(server, axis, position))
		return error(ERROR,
		  "Failed to send absolute position axis %d unit %s.",
			axis+1, esp_unit_name[server]);

	return NOERROR;

} /* call_send_esp_move_absolute() */

/************************************************************************/
/* call_send_esp_set_beam()						*/
/*									*/
/* UI callable version of send_set_beam()				*/
/************************************************************************/

int call_send_esp_set_beam(int argc, char **argv)
{
	int	axis;
	int	beam;
	char    s[81], m[81];
	int	server;

	if(!esp_get_server_and_axis(argc, argv, &server, &axis)) return NOERROR;

	if (argc > 3)
        {
                sscanf(argv[3],"%d",&beam);
        }
        else
        {
		if (!esp_wait_position(server, axis))
			return error(ERROR, 
			   "Failed to get axis %d position for ESP unit %s.",
			   axis+1, esp_unit_name[server]);
		sprintf(m,"%s beam", esp_unit_name[server]);
		sprintf(s,"%d", esp_stage[server][axis].beam);
                if (quick_edit(m,s,s,NULL,FLOAT) == KEY_ESC)
                        return -1; sscanf(s,"%d",&beam);
        }

	if (!send_esp_set_beam(server, axis, beam))
		return error(ERROR,
		  "Failed to send beam %d axis %d unit %s.",
			beam, axis+1, esp_unit_name[server]);

	return NOERROR;

} /* call_send_esp_set_beam() */

/************************************************************************/
/* call_send_esp_set_velocity()						*/
/*									*/
/* UI callable version of send_set_velocity().				*/
/************************************************************************/

int call_send_esp_set_velocity(int argc, char **argv)
{
	int	axis;
	double  velocity;
	char    s[81], m[81];
	int	server;

	if(!esp_get_server_and_axis(argc, argv, &server, &axis)) return NOERROR;

	if (argc > 3)
        {
                sscanf(argv[3],"%lf",&velocity);
        }
        else
        {
		if (!esp_wait_velocity(server,axis))
			return error(ERROR, 
			   "Failed to get axis %d velocity for ESP unit %s.",
			   axis+1, esp_unit_name[server]);
		sprintf(m,"%s abosolute velocity", esp_unit_name[server]);
		sprintf(s,"%.3f", esp_stage[server][axis].velocity);
                if (quick_edit(m,s,s,NULL,FLOAT) == KEY_ESC)
                        return -1;
                sscanf(s,"%lf",&velocity);
        }

	if (!send_esp_set_velocity(server, axis, velocity))
		return error(ERROR,
		  "Failed to send absolute velocity axis %d unit %s.",
			axis+1, esp_unit_name[server]);

	return NOERROR;

} /* call_send_esp_set_velocity() */

/************************************************************************/
/* call_send_esp_set_acceleration()					*/
/*									*/
/* UI callable version of send_set_acceleration()			*/
/************************************************************************/

int call_send_esp_set_acceleration(int argc, char **argv)
{
	int	axis;
	double  acceleration;
	char    s[81], m[81];
	int	server;

	if(!esp_get_server_and_axis(argc, argv, &server, &axis)) return NOERROR;

	if (argc > 3)
        {
                sscanf(argv[3],"%lf",&acceleration);
        }
        else
        {
		if (!esp_wait_acceleration(server, axis))
			return error(ERROR, 
			  "Failed to get axis %d acceleration for ESP unit %s.",
			  axis+1, esp_unit_name[server]);
		sprintf(m,"%s abosolute acceleration", esp_unit_name[server]);
		sprintf(s,"%.3f", esp_stage[server][axis].acceleration);
                if (quick_edit(m,s,s,NULL,FLOAT) == KEY_ESC)
                        return -1;
                sscanf(s,"%lf",&acceleration);
        }

	if (!send_esp_set_acceleration(server, axis, acceleration))
		return error(ERROR,
		  "Failed to send absolute acceleration axis %d unit %s.",
			axis+1, esp_unit_name[server]);

	return NOERROR;

} /* call_send_esp_set_acceleration() */

/************************************************************************/
/* call_send_esp_move_relative()					*/
/*									*/
/* UI callable version of send_move_relative()				*/
/************************************************************************/

int call_send_esp_move_relative(int argc, char **argv)
{
	int	axis;
	double  rel_pos;
	char    s[81], m[81];
	int	server;

	if(!esp_get_server_and_axis(argc, argv, &server, &axis)) return NOERROR;

	if (argc > 3)
        {
                sscanf(argv[3],"%lf",&rel_pos);
        }
        else
        {
		sprintf(m,"%s relative position", esp_unit_name[server]);
		sprintf(s,"%.3f", 0.0);
                if (quick_edit(m,s,s,NULL,FLOAT) == KEY_ESC)
                        return -1;
                sscanf(s,"%lf",&rel_pos);
        }

	if (!send_esp_move_relative(server, axis, rel_pos))
		return error(ERROR,
		  "Failed to send absolute relative position axis %d unit %s.",
			axis+1, esp_unit_name[server]);


	return NOERROR;

} /* call_send_esp_move_relative() */

/************************************************************************/
/* call_send_esp_move_home()						*/
/*									*/
/* UI callable version of send_move_home()				*/
/************************************************************************/

int call_send_esp_move_home(int argc, char ** argv)
{
	int	axis;
	int	server;

	if(!esp_get_server_and_axis(argc, argv, &server, &axis)) return NOERROR;

	if (!send_esp_move_home(server, axis))
		return error(ERROR,
		  "Failed to send axis %d unit %s home.",
			axis+1, esp_unit_name);

	return NOERROR;

} /* call_send_esp_move_home() */

/************************************************************************/
/* call_send_esp_text()							*/
/*									*/
/* UI callable version of send_esp_text().				*/
/************************************************************************/

int call_send_esp_text(int argc, char ** argv)
{
	char   m[81], text[81];
	int    server;

	if ((server = esp_get_server(argc, argv)) == -1) return NOERROR;

	if (argc > 2)
        {
		strcpy(text, argv[2]);
        }
        else
        {
		*text = 0;
                sprintf(m,"Text to send to unit %s", esp_unit_name[server]);
                if (quick_edit(m,text,text,NULL,STRING) == KEY_ESC)
                        return -1;
        }

	if (!send_esp_text(server, text))
		return error(ERROR,
		  "Failed to send text to unit %s.", esp_unit_name[server]);

	return NOERROR;

} /* call_send_esp_text() */

/************************************************************************/
/* show_esp_stage()							*/
/*									*/
/* Let user see what is happening inside a stage structure.		*/
/************************************************************************/

int show_esp_stage(int argc, char **argv)
{
	int	axis;
	char	s[256];
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
#endif
	int	loop;
	int	keystroke;
	int    server;

	no_socket();

	if(!esp_get_server_and_axis(argc, argv, &server, &axis)) return NOERROR;

	sprintf(s,"%s Stage %d",esp_unit_name[server], axis+1);
	heading(heading_window,s,"");
	active_window = sub_main_window;
	loop = TRUE;
	while(loop)
	{
		esp_update(server);
		background();
		process_message_socket();
	
		werase(sub_main_window);
                message(system_window,CONT_TEXT);

		mvwprintw(sub_main_window,0,0,"Position      = %8.3f",
			esp_stage[server][axis].position);
		mvwprintw(sub_main_window,1,0,"Rel Step Size = %8.3f",
			esp_stage[server][axis].relative_step_size);
		mvwprintw(sub_main_window,2,0,"Real Postition= %8.3f",
			esp_stage[server][axis].real_position);
		mvwprintw(sub_main_window,3,0,"Velocity      = %8.3f",
			esp_stage[server][axis].velocity);
		mvwprintw(sub_main_window,4,0,"Acceleration  = %8.3f",
			esp_stage[server][axis].acceleration);
		mvwprintw(sub_main_window,5,0,"Minimum pos   = %8.3f",
			esp_stage[server][axis].min);
		mvwprintw(sub_main_window,6,0,"Maximum pos   = %8.3f",
			esp_stage[server][axis].max);

		mvwprintw(sub_main_window,0,40,"Num          = %d",
			esp_stage[server][axis].num);
		mvwprintw(sub_main_window,1,40,"Beam         = %d",
			esp_stage[server][axis].beam);
		mvwprintw(sub_main_window,2,40,"Power        = %d",
			esp_stage[server][axis].beam);
		mvwprintw(sub_main_window,3,40,"Motion Done  = %d",
			esp_stage[server][axis].motion_done);
		mvwprintw(sub_main_window,4,40,"Status       = 0x%x",
			esp_status[server]);
		mvwprintw(sub_main_window,5,40,"Power Status = 0x%x",
			esp_power_status_bit[server]);
		mvwprintw(sub_main_window,6,40,"Motion Status= 0x%x",
			esp_motion_status_bit[server]);

		wrefresh(sub_main_window);

		/* Was a key hit ? */
 
		if (kbhit())
		{
			keystroke = get_command();

			/* Was it a mouse click? */
#ifdef __NCURSES_H
			if (keystroke == KEY_MOUSE)
			{
				getmouse(&mouse);

				if (mouse.bstate == BUTTON1_CLICKED)
				{
					if (mouse.y == SYSTEM_BEGIN_Y &&
                                               mouse.x<(int)strlen(CONT_TEXT)+2)
					{
						keystroke = KEY_ESC;
					}
				}
			}
#endif															     
			/* Was it KEY_ESC? */

			if (keystroke == KEY_ESC)
			{
				loop = FALSE;
			}
			else
			{
				/* Is it a command ? */
															     
				command_processor(keystroke);
			}
		}
	}
        werase(system_window);
        wrefresh(system_window);

	return NOERROR;

} /* show_esp_stage() */

/************************************************************************/
/* esp_get_server()							*/
/*									*/
/* Command lines stuff to get which server is required.			*/
/* Sets arguments. Returns server if things go well, -1 otherwise.	*/
/************************************************************************/

int esp_get_server(int argc, char ** argv)
{
	char	name[81];
	int 	unit;
	int	server;

	clean_command_line();

	/* Which server do we want to talk to? */

        if (argc > 1)
        {
		strcpy(name, argv[1]);
		while(strlen(name) != strlen(esp_types[0]))
			strcat(name," ");
                for (unit=0; unit<NUM_ESP_TYPES; unit++)
                {
                        if (strcmpi(name,esp_types[unit]) == 0)
                        {
                                break;
                        }
                }

                if (unit >= NUM_ESP_TYPES)
                {
			error(ERROR,"Unkown ESP unit %s.",argv[1]);
                        return -1;
                }
        }
        else
	{
		unit = 0;
                if (quick_edit("ESP Server",esp_types[unit],
                        &unit,esp_types,ENUMERATED) == KEY_ESC)
                                return -1;
		strcpy(name, esp_types[unit]);
	}
	server = ui_esp_servers[unit];

	if (!server_socket_open(server))
	{
		error(ERROR,"ESP server %s socket is not open.",
			esp_types[unit]);
		return -1;
	}

	return server;

} /* esp_get_server() */

/************************************************************************/
/* esp_get_server_and_axis()						*/
/*									*/
/* Command lines stuff to get which server and axis is required.	*/
/* Sets arguments. Returns TRUE if things go well.			*/
/************************************************************************/

bool esp_get_server_and_axis(int argc, char ** argv, int *server, int *axis)
{
	char	s[81];
	char    m[81];

	clean_command_line();

	if ((*server = esp_get_server(argc, argv)) == -1) return FALSE;

	if (argc > 2)
        {
                sscanf(argv[2],"%d",axis);
        }
        else
        {
                sprintf(s,"1");
		sprintf(m,"%s axis number", esp_unit_name[*server]);
                if (quick_edit(m,s,s,NULL,INTEGER) == KEY_ESC)
                        return FALSE;
                sscanf(s,"%d",axis);
        }

	(*axis)--;

	if (*axis < 0 || *axis >= esp_num_axes[*server])
	{
		error(ERROR,"Axis %d out of range for esp unit %s.",
			axis+1, esp_unit_name[*server]);
		return FALSE;
	}

	return TRUE;

} /* esp_get_server_and_axis() */

/************************************************************************/
/* esp_fts()								*/
/*									*/
/* For FTS experiment. Moves a stage back a forwards a bunch of times.	*/
/************************************************************************/

int esp_fts(int argc, char **argv)
{
	double start, stop;
	double	center, length;
	bool	forward = TRUE;
	char	s[256];
	int	num = 0;
	bool 	loop = TRUE;
	int	axis;
	int	server;
	double	position;
	double	target = 0.0;

	if(!esp_get_server_and_axis(argc, argv, &server, &axis)) return NOERROR;

	if (argc > 3)
        {
                sscanf(argv[3],"%lf",&center);
        }
        else
        {
		sprintf(s,"4.15");
                if (quick_edit("Scan center",s,s,NULL,FLOAT) == KEY_ESC)
                        return -1;
                sscanf(s,"%lf",&center);
        }

	if (argc > 4)
        {
                sscanf(argv[4],"%lf",&length);
        }
        else
        {
		sprintf(s,"500.0");
                if (quick_edit("Scan Amplitude (um)",s,s,NULL,FLOAT) == KEY_ESC)
                        return -1;
                sscanf(s,"%lf",&length);
        }

	start = center - length/1000.0;
	stop  = center + length/1000.0;

	/* OK, let's do this. */

	while(loop)
	{

		clean_command_line();

		/* WHich way to we go? */

		if (forward)
		{
			if (!send_esp_move_absolute(server, axis, start))
			return error(ERROR,
			  "Failed to send absolute position axis %d unit %s.",
			axis+1, esp_unit_name[server]);
			target = start;
		}
		else
		{
			if (!send_esp_move_absolute(server, axis, stop))
			return error(ERROR,
			  "Failed to send absolute position axis %d unit %s.",
			axis+1, esp_unit_name[server]);
			target = stop;
		}

		forward = !forward;

		num++;

		position = 0.0;
		do
		{
			background();

			if (!esp_wait_position(server, axis))
			   return error(ERROR, 
			   "Failed to get axis %d position for ESP unit %s.",
			   axis+1, esp_unit_name[server]);

			position = esp_stage[server][axis].real_position;

			message(system_window,"Num = %d Position = %f", 
				num, position);

			usleep(250000);

			if (kbhit())
			{
				if (get_command() == KEY_ESC) loop = FALSE;
			}

		} while(loop && fabs(position - target) > 0.01);

		if (kbhit())
		{
			if (get_command() == KEY_ESC) loop = FALSE;
		}

		sleep(2);

		background();
	}

	message(system_window,"Completed %d scans.", num);

	return NOERROR;

} /* esp_fts() */
