/************************************************************************/
/* fft.c 								*/
/*                                                                      */
/* FFT routines.							*/
/* Downloaded from Dan Kegel's site. From DRPN and modified by me	*/
/* to be more ANSII compilant and possibly a bit faster....		*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/* Except of course the bit I stole from Dan Kegel.....			*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Dan Kegel with mods by Theo ten Brummelaar			*/
/* Date   : May 1998                                                    */
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "fft.h"

/*
 * GLOBALS
 */

/* Following globals control use of block floating point. */

static int bfp_do;		/* set to 1 to enable, 0 to disable. */
static int bfp_scale_ct;	/* set to 0 before string of bfp operations */

/* To control some setup */

static int fft_has_been_setup = FALSE;

/* Here's the number of samples in the arrays. */

static int n;

/*
 * For now, keep a single static lookup table.
 * Rewrite it whenever the fft size changes.
 */

static int tab_fn = -1;
static float *tab_fcos, *tab_fsin;
static int tab_in = -1;
static short *tab_icos, *tab_isin;
static int bitrev_tabsize = -1;
static int *bitrev_tab = NULL;

/*
 * MACROS
 */

/* Block floating point support.  Scan thru array; if any value has
 * absolute value greater than 1<<FRAC_BITS, divide whole array by two and
 * increment the scale count.
 * To disable, set bfp_do to 0.
 * Really, this restricts us to FRAC_BITS bits of fraction, but that's life.
 * I wonder how people really do this?
 */

#define BFP_SCALE(n,xr,xi,j) \
    if (bfp_do) {						\
	for (j= (n); --j>=0; ) {				\
	   int t;						\
	   t = xr[j]; if (t < 0) t=-t; if (t > (1<<FRAC_BITS)) break;	\
	   t = xi[j]; if (t < 0) t=-t; if (t > (1<<FRAC_BITS)) break;	\
	}							\
	if (j >= 0) {						\
	    /* Yep, need to scale data. */			\
	    bfp_scale_ct++;					\
	    for (j= (n); --j>=0;) {				\
		xr[j] /= 2;					\
		xi[j] /= 2;					\
	    }							\
	}							\
    }

/*
 * The symbol BFP_DELAY_DIVIDES only applies to inverse fft's.
 * If false, we divide by 2 at each column of the inverse fft (as normal).
 * If true, we simply decrement bfp_shift_ct to flag the caller that the
 * result should be divided by 2.
 * In an informal test on a single radar echo, this reduces peak error
 * by a factor of four, and root mean squared error by a factor of 2.
 * (It also caused the median error to jump from zero to 1 or 2, but
 * we won't talk about that!)
 * THEO: MAYBE WE SHOULD FIX IT ONE WAY OR ANOTHER......
 */

#define BFP_DELAY_DIVIDES 1

#define TAB_BITS FRAC_BITS
#define TAB_UNITY (1<<TAB_BITS)
#define TAB_HALF  (1<<(TAB_BITS-1))

/*
 * Macro to convert floating point number to integer format.
 * Result is a fixed-point number with TAB_BITS of fraction.
 */

#define ftoi(f) ((short)(f * TAB_UNITY))

/************************************************************************/
/* isincos_table()							*/
/* Fill a lookup table for integers with the values of sin and cos.	*/
/* n is size of table.							*/
/* Table covers range [0..pi).						*/
/************************************************************************/

static void isincos_table(int n, short *isintab, short *icostab)
{
    int i;
    for (i=0; i<n; i++) {
	icostab[i] = ftoi(cos((M_PI * i) / (float) n));
	isintab[i] = ftoi(sin((M_PI * i) / (float) n));
    }

}

/************************************************************************/
/* fsincos_table()							*/
/* Fill a lookup table for floats with the values of sin and cos.	*/
/* n is size of table.							*/
/* Table covers range [0..pi).						*/
/************************************************************************/

static void fsincos_table(int n, float *fsintab, float *fcostab)
{
    int i;
    for (i=0; i<n; i++) {
	fcostab[i] = cos((M_PI * i) / (float) n);
	fsintab[i] = sin((M_PI * i) / (float) n);
    }
}

/************************************************************************/
/* tab_fsetup()								*/
/*									*/
/* Setups floating point sin and cos tables. Allocates memory and 	*/
/* deallocates old arrays.						*/
/************************************************************************/

static void tab_fsetup(int n)
{
    if (tab_fn != n) {
	if (tab_fn != -1) {
	    free(tab_fcos);
	    free(tab_fsin);
	}
	tab_fcos = (float *)malloc(sizeof(float) * n);
	tab_fsin = (float *)malloc(sizeof(float) * n);

	fsincos_table(n, tab_fsin, tab_fcos);
    }
    tab_fn = n;
}

/************************************************************************/
/* tab_isetup()								*/
/*									*/
/* Setups floating point sin and cos tables. Allocates memory and 	*/
/* deallocates old arrays.						*/
/************************************************************************/

static void tab_isetup(int n)
{
    if (tab_in != n) {
	if (tab_in != -1) {
	    free(tab_icos);
	    free(tab_isin);
	}
	tab_icos = (short *)malloc(sizeof(short) * n);
	tab_isin = (short *)malloc(sizeof(short) * n);

	isincos_table(n, tab_isin, tab_icos);
    }
    tab_in = n;
}

/* 
 * Returns TRUE if all OK, false otherwise.
 */

/************************************************************************/
/* setup_fft()								*/
/*									*/
/* Performs setup operations for FFT routines. Must be called before    */
/* any FFT call!							*/ 
/************************************************************************/

void setup_fft(int log_n)
{
	/* Set up some globals. */

	bfp_do = 1;
	bfp_scale_ct = 0;

	/* Setup the sin/cos tables */

	n = 1<<log_n;
	tab_isetup(n/2);
	tab_fsetup(n/2);

	/* Setup the bit reversal tables */

	bitrev_setup(n);

	/* That should be all */

	fft_has_been_setup = TRUE;

}

/************************************************************************/
/* cfft_dit()								*/
/*									*/
/* Complex FFT, decimation-in-time, in place.				*/
/* Each of rbuf and ibuf is a (2 ^ log_n)-long array of 16 bit signed 	*/
/* fractions. Inputs are in bit-reversed order, outputs are in order.	*/
/* 									*/
/* See Oppenheim & Schafer, "Digital Signal Processing", 1975, p. 318;  */
/* the particular dataflow graph implemented is that of figure 6.10, 	*/
/* which uses bit-reversed input, and accesses coefficients in normal   */
/* order.								*/
/*									*/
/************************************************************************/

void cfft_dit(int log_n, short *xr, short *xi)
{
    int j, k;
    int stage;

    if (!fft_has_been_setup) return;

    /* For each of the log_n stages of the FFT */
    for (stage=1; stage<=log_n; stage++) {
	int n_dft;
	int n_butterfly;
	int twiddle;
	short ar, br, ai, bi;

	/* If this macro defined, scale x by 1/2 if |x[j]|>1 for any j */
	BFP_SCALE(n, xr, xi, j);

	n_dft = 1 << (log_n - stage);		/* # of dft's */
	n_butterfly = 1 << (stage-1);		/* # of butterflies per dft */

	/* For each (dft_size)-point dft making up this stage: */
	for (j=0; j < n_dft; j++) {

	    /* For each butterfly making up this dft: */
	    for (k=twiddle=0; k<n_butterfly; k++, twiddle += n_dft) {
		int index1, index2;
		short wr, wi;

		if (twiddle >= n/2) twiddle = 0;
		wr = tab_icos[twiddle];
		wi = tab_isin[twiddle];

		/* Calculate the indices of the two locations this butterfly
		 * operates upon.
		 */
		index1 = j * 2 * n_butterfly + k;
		index2 = index1 + n_butterfly;

		/* Grab old values & apply twiddle factor to one input value. */
		ar = xr[index1];
		ai = xi[index1];
		br =(TAB_HALF+(int)wr*xr[index2] - wi*xi[index2]) >> TAB_BITS;
		bi =(TAB_HALF+(int)wr*xi[index2] + wi*xr[index2]) >> TAB_BITS;

		/* Do butterfly. */
		xr[index1] = ar + br;
		xi[index1] = ai + bi;
		xr[index2] = ar - br;
		xi[index2] = ai - bi;
	    }
	}
    }
}

/************************************************************************/
/* cfft_dif()								*/
/*									*/
/* Complex FFT, decimation-in-frequency, in place.			*/
/* Each of rbuf and ibuf is a (2 ^ log_n)-long array of 16 bit signed 	*/
/* fractions. Inputs are in order, outputs are in bit-reversed order.	*/
/* 									*/
/* Routine will divide by two in the first butterfly if divide_stages[0]*/
/* is TRUE, etc.							*/
/************************************************************************/

void cfft_dif(int log_n, short *xr, short *xi, short *divide_stages)
{
    int stage;
    int j;

    if (!fft_has_been_setup) return;

    /* For each of the log_n stages of the FFT */
    for (stage=log_n; stage>=1; stage--) {
	int k;
	int n_dft;
	int n_butterfly;
	int twiddle;

	/* If this macro defined, scale x by 1/2 if |x[j]|>1 for any j */
	BFP_SCALE(n, xr, xi, j);

	n_dft = 1 << (log_n - stage);		/* # of dft's */
	n_butterfly = 1 << (stage-1);		/* # of butterflies per dft */

#define cfft_dif_stage(SHIFT) \
	/* For each (dft_size)-point dft making up this stage: */	\
	for (j=0; j < n_dft; j++) {					\
	    /* For each butterfly making up this dft: */		\
	     for (k=twiddle=0; k<n_butterfly; k++, twiddle += n_dft) {	\
		int index1, index2;					\
		short xr1, xi1, xr2, xi2;				\
		short wr, wi;						\
		if (twiddle >= n/2) twiddle = 0;			\
		wr = tab_icos[twiddle];					\
		wi = tab_isin[twiddle];					\
		/* Calculate adr of locations this butterfly operates upon. */ \
		index1 = j * 2 * n_butterfly + k;			\
		index2 = index1 + n_butterfly;				\
		xr1 = xr[index1];					\
		xi1 = xi[index1];					\
		xr2 = xr[index2];					\
		xi2 = xi[index2];					\
		/* Do butterfly.  Different from a DIT bfly. */		\
		xr[index1]= (xr1 + xr2)>>SHIFT;				\
		xi[index1]= (xi1 + xi2)>>SHIFT;				\
		xr[index2]=((1<<(TAB_BITS+SHIFT-1))			\
		    +(int)(xr1-xr2)*wr - (xi1-xi2)*wi)>>(TAB_BITS+SHIFT);\
		xi[index2]=((1<<(TAB_BITS+SHIFT-1))			\
		    +(int)(xr1-xr2)*wi + (xi1-xi2)*wr)>>(TAB_BITS+SHIFT);\
	      }								\
	}

	/* Do inner two loops of fft; divide by 2 if desired. */
	if (divide_stages && *divide_stages++) {
	    cfft_dif_stage(1);
	} else {
	    cfft_dif_stage(0);
	}
    }
}

/************************************************************************/
/* cfft_dif_float32()							*/
/*									*/
/* Complex FFT, decimation-in-frequency, in place.			*/
/* Each of rbuf and ibuf is a (2 ^ log_n)-long array of 32 bit signed 	*/
/* floats. Inputs are in order, outputs are in bit-reversed order.	*/
/* 									*/
/* Routine will divide by two in the first butterfly if divide_stages[0]*/
/* is TRUE, etc.							*/
/************************************************************************/

void cfft_dif_float32(int log_n, float *xr, float *xi)
{
    int stage;
    int j;

    if (!fft_has_been_setup) return;

    /* For each of the log_n stages of the FFT */
    for (stage=log_n; stage>=1; stage--) {
	int k;
	int n_dft;
	int n_butterfly;
	int twiddle;

	n_dft = 1 << (log_n - stage);		/* # of dft's */
	n_butterfly = 1 << (stage-1);		/* # of butterflies per dft */

	/* For each (dft_size)-point dft making up this stage: */
	for (j=0; j < n_dft; j++) {

	    /* For each butterfly making up this dft: */
	    for (k=twiddle=0; k<n_butterfly; k++, twiddle += n_dft) {
		int index1, index2;
		float xr1, xi1, xr2, xi2;
		float wr, wi;

		if (twiddle >= n/2) twiddle = 0;
		wr = tab_fcos[twiddle];
		wi = tab_fsin[twiddle];
		/* printf("stage=%d j=%d k=%d: wr=%d, wi=%d, twiddle=%d\n", stage, j, k, wr, wi, twiddle); */

		/* Calculate the indices of the two locations this butterfly
		 * operates upon.
		 */
		index1 = j * 2 * n_butterfly + k;
		index2 = index1 + n_butterfly;

		/* Grab old values. */
		xr1 = xr[index1];
		xi1 = xi[index1];
		xr2 = xr[index2];
		xi2 = xi[index2];

		/* Do butterfly.  Note this is different from a DIT bfly. */
		xr[index1] = xr1 + xr2;
		xi[index1] = xi1 + xi2;
		xr[index2] = (xr1-xr2)*wr - (xi1-xi2)*wi;
		xi[index2] = (xr1-xr2)*wi + (xi1-xi2)*wr;
	    }
	}
    }
}

/************************************************************************/
/* cifft_dif()								*/
/*									*/
/* Inverse of Complex FFT, decimation-in-frequency, in place.		*/
/* Each of rbuf and ibuf is a (2 ^ log_n)-long array of 16 bit signed 	*/
/* fractions. Inputs are in bit-reversed order, outputs are in order.	*/
/*									*/
/* Divides data by N, unless divide_stages is non-NULL, in which case it*/
/* only divides by 2 at stage i s.t. divide_stages[i]==1.		*/
/*									*/
/* Same as decimation-in-time forward transform, but goes thru stages 	*/
/* in opposite order, and uses 1/original twiddle factors (i.e. uses 	*/
/* the complex conjugate of the original twiddle factors; you could 	*/
/* equivalently take the complex conjugate of the input (and output?) 	*/
/* data & use the original twiddle factors!)				*/
/************************************************************************/

void cifft_dif(int log_n, short *xr, short *xi, short *divide_stages)
{
    int j, k;
    int stage;

    if (!fft_has_been_setup) return;

    /* For each of the log_n stages of the FFT */
    for (stage=1; stage<=log_n; stage++) {
	int n_dft;
	int n_butterfly;
	int twiddle;
	short ar, br, ai, bi;

	/* If this macro defined, scale x by 1/2 if |x[j]|>1 for any j */
	BFP_SCALE(n, xr, xi, j);

	n_dft = 1 << (log_n - stage);		/* # of dft's */
	n_butterfly = 1 << (stage-1);		/* # of butterflies per dft */

#define cifft_dif_stage(SHIFT)						\
	/* For each (dft_size)-point dft making up this stage: */	\
	for (j=0; j < n_dft; j++) {					\
	    /* For each butterfly making up this dft: */		\
	    for (k=twiddle=0; k<n_butterfly; k++, twiddle += n_dft) {	\
		int index1, index2;					\
		short wr, wi;						\
		if (twiddle >= n/2) twiddle = 0;			\
		wr = tab_icos[twiddle];					\
		wi = -tab_isin[twiddle];/* negative exp on twiddle */	\
		/* Calculate adr of locations this butterfly operates upon. */ \
		index1 = j * 2 * n_butterfly + k;			\
		index2 = index1 + n_butterfly;				\
		/* Grab old values, apply twiddle to 1 input value. */	\
		ar = xr[index1];					\
		ai = xi[index1];					\
		br =(TAB_HALF+ wr*xr[index2] - wi*xi[index2]) >> TAB_BITS; \
		bi =(TAB_HALF+ wr*xi[index2] + wi*xr[index2]) >> TAB_BITS; \
		/* Do butterfly already. */				\
		xr[index1] = (ar + br)>>SHIFT;				\
		xi[index1] = (ai + bi)>>SHIFT;				\
		xr[index2] = (ar - br)>>SHIFT;				\
		xi[index2] = (ai - bi)>>SHIFT;				\
	    }								\
	} /* end of macro cifft_dif_stage */

	{
	    int do_div = (divide_stages==NULL || *divide_stages++!=0);
	    int delay_any_div = (BFP_DELAY_DIVIDES && bfp_do);

	    /* If we want to divide by 2 while doing the fft stage, do so. */
	    if (do_div && !delay_any_div) {
		cifft_dif_stage(1);
	    } else {
		cifft_dif_stage(0);
	    }
	    /* If we need to divide by 2 later, say so. */
	    if (do_div && delay_any_div)
		bfp_scale_ct--;
	}
    }
}

/************************************************************************/
/* cifft_dif_float32()							*/
/*									*/
/* Inverse of Complex FFT, decimation-in-frequency, in place.		*/
/* Each of rbuf and ibuf is a (2 ^ log_n)-long array of 32 bit signed 	*/
/* float. Inputs are in bit-reversed order, outputs are in order.	*/
/*									*/
/* Divides data by N, unless divide_stages is non-NULL, in which case it*/
/* only divides by 2 at stage i s.t. divide_stages[i]==1.		*/
/*									*/
/* Same as decimation-in-time forward transform, but goes thru stages 	*/
/* in opposite order, and uses 1/original twiddle factors (i.e. uses 	*/
/* the complex conjugate of the original twiddle factors; you could 	*/
/* equivalently take the complex conjugate of the input (and output?) 	*/
/* data & use the original twiddle factors!)				*/
/************************************************************************/

void cifft_dif_float32(int log_n, float *xr, float *xi)
{
    int j, k;
    int stage;

    if (!fft_has_been_setup) return;

    /* For each of the log_n stages of the FFT */
    for (stage=1; stage<=log_n; stage++) {
	int n_dft;
	int n_butterfly;
	int twiddle;
	float ar, br, ai, bi;

	n_dft = 1 << (log_n - stage);		/* # of dft's */
	n_butterfly = 1 << (stage-1);		/* # of butterflies per dft */

	/* For each (dft_size)-point dft making up this stage: */
	for (j=0; j < n_dft; j++) {

	    /* For each butterfly making up this dft: */
	    for (k=twiddle=0; k<n_butterfly; k++, twiddle += n_dft) {
		int index1, index2;

		float wr, wi;

		if (twiddle >= n/2) twiddle = 0;
		wr = tab_fcos[twiddle];
		wi = -tab_fsin[twiddle];	/* negative exp on twiddle */

		/* Calculate the indices of the two locations this butterfly
		 * operates upon.
		 */
		index1 = j * 2 * n_butterfly + k;
		index2 = index1 + n_butterfly;

		/* Grab old values & apply twiddle factor to one input value. */
		ar = xr[index1];
		ai = xi[index1];
		br = wr*xr[index2] - wi*xi[index2];
		bi = wr*xi[index2] + wi*xr[index2];

		/* Do butterfly.  Scale down by factor of 2. */
		xr[index1] = (ar + br)/2;
		xi[index1] = (ai + bi)/2;
		xr[index2] = (ar - br)/2;
		xi[index2] = (ai - bi)/2;
	    }
	}
    }
}

/************************************************************************/
/* bitrev()								*/
/* Stupid bitreversal routine.  Works for n <= 8192.			*/
/************************************************************************/

static int bitrev(int i, int n)
{
    n >>= 1;
    return
	((i & 1<<0) ? (n>>0) : 0) +
	((i & 1<<1) ? (n>>1) : 0) +
	((i & 1<<2) ? (n>>2) : 0) +
	((i & 1<<3) ? (n>>3) : 0) +
	((i & 1<<4) ? (n>>4) : 0) +
	((i & 1<<5) ? (n>>5) : 0) +
	((i & 1<<6) ? (n>>6) : 0) +
	((i & 1<<7) ? (n>>7) : 0) +
	((i & 1<<8) ? (n>>8) : 0) +
	((i & 1<<9) ? (n>>9) : 0) +
	((i & 1<<10) ? (n>>10) : 0) +
	((i & 1<<11) ? (n>>11) : 0) +
	((i & 1<<12) ? (n>>12) : 0) +
	((i & 1<<13) ? (n>>13) : 0);
}

/************************************************************************/
/* bitrev_setup()							*/
/*									*/
/* Build static lookup table with bit reversal indices for 		*/
/* array of size n.							*/
/************************************************************************/

void bitrev_setup(int n)
{
    int i;

    if (n == bitrev_tabsize) return;
    if (bitrev_tab != NULL) free(bitrev_tab);
    bitrev_tab = (int *)malloc(n * sizeof(int));
    bitrev_tabsize = n;

    for (i=0; i<n; i++)
	bitrev_tab[i] = bitrev(i,n);
}

/************************************************************************/
/* bitrev_int16()							*/
/*									*/
/* Do bit reversal on n sized short in array.				*/
/************************************************************************/

void bitrev_int16(int n, short *pin, short *pout)
{
    int i;
    int *tab;

    if (!fft_has_been_setup) return;

    tab = bitrev_tab;
    for (i=0; i<n; i++)
	pout[*tab++] = *pin++;
}

/************************************************************************/
/* bitrev_float32()							*/
/*									*/
/* Do bit reversal on n sized float in array.				*/
/************************************************************************/

void bitrev_float32(int n, float *pin, float *pout)
{
    int i;
    int *tab;

    if (!fft_has_been_setup) return;

    tab = bitrev_tab;
    for (i=0; i<n; i++)
	pout[*tab++] = *pin++;
}
