/************************************************************************/
/* pid.c								*/
/*									*/
/* Calculates the response of a quadratic lag filter.			*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.						*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Jan 1995     						*/ 
/*									*/
/* Changes:								*/
/*									*/
/* 6/26/96 Fixed math for this rouinte, hope it's right this time!.	*/
/*									*/
/************************************************************************/

#include "../nrc/nrc.h"
#ifdef NOT_AN_APPLE
#warning YAY
#include <values.h>
#endif
#include <math.h>
#include "filter.h"

float	process_pid(float *params, float input);

/************************************************************************/
/* pid()								*/
/*									*/
/* Set's up the quadratic lag filter setup.				*/
/************************************************************************/

float	pid(digital_filter *filter, float dt, float p, float i, float d)
{
	float	f6db;

	/* Check out what the -6db point is */

	if (d == 0.0)
	{
		f6db = MAXFLOAT;
	}
	else
	{
		f6db = (sqrt(3.0) + sqrt(3.0 + 4.0 * d * i))/(4.0 * d * M_PI);
	}

	/* 
	 * Set up the parameters:
 	 * 1. P
	 * 2. 1+2D/dT
	 * 3. D/dT
	 * 4. 1/(1+Idt+D/dT)
	 * 5. i(n-1)
	 * 6. o(n-1)
 	 * 7. o(n-2)
	 */

	filter->dt = dt;
	filter->n_params = 7;
	filter->params = vector(1,7);
	
	filter->params[1] = p;
	filter->params[3] = d/dt;
	filter->params[2] = 1 + 2.0*filter->params[3];
	filter->params[4] = 1.0 / (1.0 + i * dt + filter->params[3]);
	filter->params[5] = 0.0;
	filter->params[6] = 0.0;
	filter->params[7] = 0.0;

	/* Set the processing function */

	filter->process = process_pid;

	return f6db;

} /* pid() */

/************************************************************************/
/* process_pid(float *params, float input)				*/
/*									*/
/* Does the processing for the quadratic lag filter.			*/
/************************************************************************/

float	process_pid(float *params, float input)
{
	float	output;

#ifdef DEBUG
	{
		int i;

		printf("process_pid(*params,%f)\n",input);
		for(i=1;i<=7;i++) printf("params[%d] = %f\n",i,params[i]);
	}
#endif

	output = params[4] * (params[1] * (input - params[5]) +
			      params[2] * params[6] - params[3] * params[7]);

	params[5] = input;
	params[7] = params[6];
	params[6] = output;

	return output;

} /* process_pid() */

/************************************************************************/
/* pid_response()							*/
/*									*/
/* Calculates the response of a PID filter.				*/
/************************************************************************/

fcomplex pid_response(float freq, float p, float i, float d)
{
        fcomplex s;
        fcomplex enumerator;
        fcomplex denominator;
        fcomplex ci;
        fcomplex response;

        enumerator.r = p;
        enumerator.i = 0.0;

        ci.r = i;
        ci.i = 0.0;

	if (freq == 0)
	{
		response.r = p;
		response.i = 0.0;
	}
	else
	{
		s.r = 0.0;
		s.i = 2.0 * M_PI * freq;

		denominator = Cadd(Cdiv(ci,s),RCmul(d,s));
		denominator.r += 1.0;

		response = Cdiv(enumerator,denominator);
	}
	
	return response;

} /* pid_response() */
