/************************************************************************/
/* quadratic_lag.c							*/
/*									*/
/* Calculates the response of a quadratic lag filter.			*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.						*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Jan 1995     						*/ 
/************************************************************************/

#include <math.h>
#include "../nrc/nrc.h"
#include "filter.h"

float	process_quadratic_lag(float *params, float input);

/************************************************************************/
/* quadratic_lag()							*/
/*									*/
/* Set's up the quadratic lag filter setup.				*/
/************************************************************************/

float	quadratic_lag(digital_filter *filter, float dt, float res, float damp)
{
	float	damp2;
	float	f6db;
	float	wnT, wnT2;

	damp2 = damp*damp;
	f6db = res * (1.0 - 2.0*damp2 + sqrt(2.0 - 4.0*damp2*(1.0-damp2)));

	/* 
	 * Set up the parameters:
 	 * 1. 1/A
	 * 2. B
	 * 3. C
	 * 4. Last output
	 * 5. output two samples ago
	 */

	filter->dt = dt;
	filter->n_params = 5;
	filter->params = vector(1,5);
	
	wnT = 1.0 / (2.0 * M_PI * res * dt);
	wnT2 = wnT * wnT;
	wnT = damp * wnT;

	filter->params[1] = 1.0/(wnT2 + 2.0*wnT + 1);
	filter->params[2] = 2.0 * (wnT2 + wnT);
	filter->params[3] = wnT2;
	filter->params[4] = filter->params[5] = 0.0;

	/* Set the processing function */

	filter->process = process_quadratic_lag;

	return f6db;

} /* quadratic_lag() */

/************************************************************************/
/* process_quadratic_lag(float *params, float input)			*/
/*									*/
/* Does the processing for the quadratic lag filter.			*/
/************************************************************************/

float	process_quadratic_lag(float *params, float input)
{
	float	output;

	output = params[1] * (input + params[2] * params[4] - 
			params[3] * params[5]);

	params[5] = params[4];
	params[4] = output;

	return output;

} /* process_quadratic_lag() */

/************************************************************************/
/* quadratic_lag_response()						*/
/*									*/
/* Calculates theoretical response of quadratic lag.			*/
/************************************************************************/

fcomplex quadratic_lag_response(float freq, float resonance, float damp)
{
	fcomplex unity;
	fcomplex sdivwn;
	fcomplex denominator;
	fcomplex response;

        unity.r = 1.0;
        unity.i = 0.0;

	sdivwn.r = 0.0;
	sdivwn.i = freq /resonance;

	denominator = Cadd(Cmul(sdivwn,sdivwn), RCmul(2.0*damp,sdivwn));
	denominator.r += 1.0;

	response = Cdiv(unity,denominator);

	return response;

} /* quadratic_lag_response() */
