/************************************************************************/
/* simple_lag.c								*/
/*									*/
/* Calculates the response of a simple lag filter.			*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.						*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Jan 1995     						*/ 
/************************************************************************/

#include <math.h>
#include "../nrc/nrc.h"
#include "filter.h"

float	process_simple_lag(float *params, float input);

/************************************************************************/
/* simple_lag()								*/
/*									*/
/* Set's up the simple lag filter setup.				*/
/************************************************************************/

float	simple_lag(digital_filter *filter, float dt, float tau)
{
	float	f6db;

	/* Check out what the -6db point is */

	f6db = sqrt(3) / (2.0 * M_PI * tau);
#ifdef DEBUG
	printf("For simple_lag(filter,%f) f-6db = %f\n",tau,f6db);
#endif

	/* 
	 * Set up the parameters:
 	 * 1. dt/(dt+tau)
	 * 2. tau/(dt+tau)
	 * 3. Last output
	 */

	filter->dt = dt;
	filter->n_params = 3;
	filter->params = vector(1,3);
	
	filter->params[1] = dt / (tau + dt);
	filter->params[2] = tau / (tau + dt);
	filter->params[3] = 0.0;

	/* Set the processing function */

	filter->process = process_simple_lag;

	return f6db;

} /* simple_lag() */

/************************************************************************/
/* process_simple_lag(float *params, float input)			*/
/*									*/
/* Does the processing for the simple lag filter.			*/
/************************************************************************/

float	process_simple_lag(float *params, float input)
{
	params[3] = params[1] * input + params[2] * params[3];

	return params[3];

} /* process_simple_lag() */

/************************************************************************/
/* simple_lag_response()						*/
/*									*/
/* Calculates the theoretical response of a simple lag.			*/
/************************************************************************/

fcomplex simple_lag_response(float freq, float tau)
{
	fcomplex unity;
	fcomplex denominator;
	fcomplex response;

        unity.r = 1.0;
        unity.i = 0.0;

	denominator.r = 1.0;
	denominator.i = tau * freq * 2.0 * M_PI;

	response = Cdiv(unity,denominator);

	return response;

} /* simple_lag_response() */
