/************************************************************************/
/* calibration.c	 						*/
/*									*/
/* Routines for doing calibration of data.				*/
/************************************************************************/
/*                                        	                      	*/
/* Center for High Angular Resolution Astronomy				*/
/*									*/
/* ZERNIKE CODE LIBRARY.						*/
/*									*/
/* Telephone: 404-651-2932						*/
/* Fax      : 404-651-1384						*/
/* Internet : theo@@chara.gsu.edu					*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : July 2004    						*/
/************************************************************************/

#include <math.h>
#include <float.h>
#include <nrc.h>
#ifdef NOT_AN_APPLE_
#include <values.h>
#endif
#include "interferometry.h"

/************************************************************************/
/* calibrate_v()							*/
/*									*/
/* Does the simple minded calibration thing.				*/
/************************************************************************/

void   calibrate_v(double lambda, double dlambda,  /* Observing band (m)*/
	    double c1size, double c1sizeerr, /* Radian */
            double c1v, double c1err, 
		double c1t, double c1b, /* Cal1 V, time and baseline (m)*/
            double ov, double oerr, double ot, /* Obj V, and time */
	    double c2size, double c2sizeerr, /* Radian */
            double c2v, double c2err, 
		double c2t, double c2b,
	double *v, double *err) /* Cal1 V, time and baseline (m)*/
{
	double	m, c;
	double	t;
	double  dtdc1v, dtdc2v, terr;
	double	c1vx, c2vx, x, var_vx2;

	/* First we adjust for the size of the calibrator */

	c1vx = v_broadband(UNIFORM_DISK, c1b, lambda, dlambda, c1size);
	c1err /= c1vx;

	c2vx = v_broadband(UNIFORM_DISK, c2b, lambda, dlambda, c2size);
	c2err /= c2vx;

	/* Now properly add the uncertainty from Cdiam (Log 8 Page 40&64) */

	if (c1size > 0.0)
	{
	     x = (M_PI * c1size * c1b) / lambda;
	     var_vx2 = ((8.0 * bessj1(x) * bessj(2, x))/x) * (c1sizeerr/c1size);
	     var_vx2 = var_vx2 * var_vx2;
	     c1err = sqrt(c1err*c1err + 
		((c1v*c1v)/(c1vx * c1vx * c1vx * c1vx)) * 
		(var_vx2 / (4.0 * c1vx * c1vx)));
	}

	if (c2size > 0.0)
	{
	    x = (M_PI * c2size * c2b) / lambda;
	    var_vx2 = ((8.0 * bessj1(x) * bessj(2, x))/x) * (c2sizeerr/c2size);
	    var_vx2 = var_vx2 * var_vx2;
	    c2err = sqrt(c2err*c2err + 
		((c2v*c2v)/(c2vx * c2vx * c2vx * c2vx)) * 
		(var_vx2 / (4.0 * c2vx * c2vx)));
	}

	/* Now find the system visibility */

	c1v /= c1vx;
	c2v /= c2vx;

	/* Now do the interpolation */

	if (c1t == c2t)
	{
		m = 0.0;
		c = c1v;
	}
	else
	{
		m = (c1v - c2v)/(c1t - c2t);
		c = c2v - m * c2t;
	}
	t = m * ot + c;

	/* and the vis extimator is....*/

	*v = ov / t;

	/* Error propogation, see log 7 page 127 */

	if (c1t != c2t)
	{
		dtdc1v = (ot - c2t)/(c1t - c2t);
		dtdc2v = 1.0 + (c2t - ot)/(c1t - c2t);
	}
	else
	{
		dtdc1v = 0.0;
		dtdc2v = 0.0;
	}
	terr = sqrt(dtdc1v * dtdc1v * c1err * c1err +
		    dtdc2v * dtdc2v * c2err * c2err);
	*err = (*v) * sqrt((oerr*oerr)/(ov * ov) + (terr*terr)/(t*t));

} /* calibrate_v() */

/************************************************************************/
/* calibrate_v2()							*/
/*									*/
/* Does the simple minded calibration thing for v2.			*/
/************************************************************************/

void   calibrate_v2(double lambda, double dlambda,  /* Observing band (m)*/
	    double c1size, double c1sizeerr, /* Radian */
            double c1v2, double c1err, 
		double c1t, double c1b, /* Cal1 V, time and baseline (m)*/
            double ov2, double oerr, double ot, /* Obj V, and time */
	    double c2size, double c2sizeerr, /* Radian */
            double c2v2, double c2err, 
		double c2t, double c2b,
	double *v2, double *err) /* Cal1 V, time and baseline (m)*/
{
	double	m, c;
	double	t;
	double  dtdc1v2, dtdc2v2, terr;
	double	c1vx2, c2vx2, x, var_vx2;

	/* First we adjust for the size of the calibrator */

	c1vx2 = v_broadband(UNIFORM_DISK, c1b, lambda, dlambda, c1size);
	c1vx2 *= c1vx2;
	c1err /= c1vx2;

	c2vx2 = v_broadband(UNIFORM_DISK, c2b, lambda, dlambda, c2size);
	c2vx2 *= c2vx2;
	c2err /= c2vx2;

	/* Now properly add the uncertainty from Cdiam (Log 8 Page 40&64) */

	if (c1size > 0.0)
	{
	    x = (M_PI * c1size * c1b) / lambda;
	    var_vx2 = ((8.0 * bessj1(x) * bessj(2, x))/x) * (c1sizeerr/c1size);
	    var_vx2 = var_vx2 * var_vx2;
	    c1err = sqrt(c1err*c1err + 
		((c1v2*c1v2)/(c1vx2 * c1vx2 * c1vx2 * c1vx2)) * var_vx2);
	}

	if (c2size > 0.0)
	{
	    x = (M_PI * c2size * c2b) / lambda;
	    var_vx2 = ((8.0 * bessj1(x) * bessj(2, x))/x) * (c2sizeerr/c2size);
	    var_vx2 = var_vx2 * var_vx2;
	    c2err = sqrt(c2err*c2err + 
		((c2v2*c2v2)/(c2vx2 * c2vx2 * c2vx2 * c2vx2)) * var_vx2);
	}

	/* System vis */

	c1v2 /= c1vx2;
	c2v2 /= c2vx2;

	/* Now do the interpolation */

	if (c1t == c2t)
	{
		m = 0.0;
		c = c1v2;
	}
	else
	{
		m = (c1v2 - c2v2)/(c1t - c2t);
		c = c2v2 - m * c2t;
	}
	t = m * ot + c;

	/* That should be all */

	*v2 = ov2 / t;

	/* Error propogation, see log 7 page 127 */

	if (c1t != c2t)
	{
		dtdc1v2 = (ot - c2t)/(c1t - c2t);
		dtdc2v2 = 1.0 + (c2t - ot)/(c1t - c2t);
	}
	else
	{
		dtdc1v2 = 0.0;
		dtdc2v2 = 0.0;
	}
	terr = sqrt(dtdc1v2 * dtdc1v2 * c1err * c1err +
		    dtdc2v2 * dtdc2v2 * c2err * c2err);
	*err = (*v2) * sqrt((oerr*oerr)/(ov2 * ov2) + (terr*terr)/(t*t));

} /* calibrate_v2() */

/************************************************************************/
/* calibrate_closure()							*/
/*									*/
/* Does simple minded thing for closure.				*/
/************************************************************************/

void   calibrate_closure(
            double c1cp, double c1err, double c1t, /* Cal1 CP and time */
            double ocp, double oerr, double ot, /* Obj CP and time */
            double c2cp, double c2err, double c2t, /* Cal2 CP and time */
            double *cp, double *err) /* Cal1 cp */
{
	double	m, c;
	double	t;
	double	dtdc1cp, dtdc2cp, terr;

	/* Do the interpolation */

	if (c1t == c2t)
	{
		m = 0.0;
		c = c1cp;
	}
	else
	{
		m = (c1cp - c2cp)/(c1t - c2t);
		c = c2cp - m * c2t;
	}
	t = m * ot + c;

	/* That should be all */

	*cp = ocp - t;

	/* Error propogation, see Log 7 page 129 */

	if (c1t != c2t)
	{
		dtdc1cp = (ot - c2t)/(c1t - c2t);
		dtdc2cp = 1.0 + (c2t - ot)/(c1t - c2t);
	}
	else
	{
		dtdc1cp = 0.0;
		dtdc2cp = 0.0;
	}
	terr = sqrt(dtdc1cp * dtdc1cp * c1err * c1err +
		    dtdc2cp * dtdc2cp * c2err * c2err);
	*err = sqrt(oerr*oerr + terr*terr);

} /* calibrate_cp() */
