/************************************************************************/
/* gaussian_approx_track_loss.c						*/
/*									*/
/* Routine to approximate point of tracking loss.			*/
/* Uses Gaussian approximation.						*/
/************************************************************************/
/*                                        	                      	*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : October 1995							*/
/************************************************************************/

#include <math.h>
#include <nrc.h>
#include "interferometry.h"

static float local_n;
static float local_v;
static float local_m;
static float local_s;
static int   local_npix;

static float approx_tracking_loss_func(float v, 
				       float n, 
				       int npix, 
				       float s, 
				       float m);

static float zbrent_approx_tracking_loss_n(float n);

static float zbrent_approx_tracking_loss_m(float m);

#define BETA 3
#define BETA2 9
#define BETA4 81

static float approx_tracking_loss_func(float v, 
				       float n, 
				       int npix, 
				       float s, 
				       float m)
{
	float result;
	float n4,n3,n2;
	float v2;
	float s4,s2;

	n2 = n*n;
	n3 = n*n2;
	n4 = n2*n2;

	v2 = v*v;

	s2 = s*s;
	s4 = s2*s2;

	result = v2 - 8.0*BETA2/(m * n2) * 
		 (
			n + npix*s2 + 
			sqrt(m*(n2+npix*npix*s4+2.0*npix*s2*n))/BETA
		 );

#ifdef DEBUG
	printf("approx_tracking_loss_func(%f, %f, %d, %f, %f) = %f\n",
		v,n,npix,s,m,result);
#endif
	return result;

} /* approx_tracking_loss_func() */

static float zbrent_approx_tracking_loss_n(float n)
{
	return approx_tracking_loss_func(local_v,
		n, local_npix, local_s, local_m);
}

static float zbrent_approx_tracking_loss_m(float m)
{
	float result;

        result = approx_tracking_loss_func(local_v,
                local_n, local_npix, local_s, m);
#ifdef DEBUG
	printf("zbrent_approx_tracking_loss_m(%f) = %f\n",m,result);
#endif 

	return result;
}

/************************************************************************/
/* tracking_loss_n()							*/
/*									*/
/* Uses the gaussian tracking loss probability approximation to find 	*/
/* photon detection rate at which tracking failure will start to 	*/
/* occur.								*/
/************************************************************************/

float tracking_loss_n(float v,
		      int   npix,
		      float s,
		      float   m)
{
	local_v = v;
	local_npix = npix;
	local_s = s;
	local_m = m;

	return zbrent(zbrent_approx_tracking_loss_n,1e-4,1e10,0.01);
}

/************************************************************************/
/* tracking_loss_m()							*/
/*									*/
/* Uses the gaussian tracking loss probability approximation to find 	*/
/* the number of integrations at which tracking failure will start to 	*/
/* occur.								*/
/************************************************************************/

float tracking_loss_m(float v,
		      float n,
                      int   npix,
                      float s)
{
        local_v = v;
	local_n = n;
        local_npix = npix;
        local_s = s;

        return zbrent(zbrent_approx_tracking_loss_m,1e-20,1e20,0.01);
}

/************************************************************************/
/* tracking_loss_v()							*/
/*									*/
/* Uses the gaussian tracking loss probability approximation to find 	*/
/* the visibility at which tracking failure will start to occur.	*/
/************************************************************************/

float tracking_loss_v( float n,
                      int   npix,
                      float s,
		      float m)
{
	float v;

	v = -1.0*approx_tracking_loss_func(0.0, n, npix, s, m);

	return sqrt(v);

} /* tracking_loss_v() */
