/************************************************************************/
/* interferometry.h							*/
/*                                                                      */
/* Header file for Interferometry library.				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : August 1998	                                                */
/************************************************************************/

#ifndef _INTERFEROMETRY_ /** Let's make sure of only one pass through */
#define _INTERFEROMETRY_

/* Some macros */

#define UNIFORM_DISK 0
#define GAUSSIAN     1
#define LIMB_DARK_MP 2
#define LIMB_DARK_LIN 3
#define LIMB_DARK_POW 4
#define LIMB_DARK     5

long int random_seed;

/* gaussian_approx_track_loss.c */

float tracking_loss_n(float v, int npix, float s, float m);
float tracking_loss_m(float v, float n, int npix, float s);
float tracking_loss_v(float n, int npix, float s, float m);

/* photon_models.c */

float expected_photon_count(float mag, float R, float tau, float bandwidth, float dqe_sys, float dqe_det);
float photon_count_2_mag(float n, float R, float tau, float bandwidth, float dqe_sys, float dqe_det);
float detect_counts(float phase, float v, float n, float s, float *n1, float *n2);
float fringe_value(float pos, /* micron */
                   float v,
                   float lambda, /* micron */
                   float bw,     /* micron */
		   float phase);
float detect_fringe_value(float n, /* Mean photon count */
		   float gain, /* of camera */
                   float sig, /* Read noise */
                   float pos, /* micron */
                   float v,
                   float lambda, /* micron */
                   float bw,     /* micron */
		   float phase);
float calc_v_trans(float I1, float I2);
void climb_fringe_value(float pos1, /* micron */
                   float pos2,
                   float pos3,
                   float v12,
                   float v23,
                   float v31,
                   float lambda, /* micron */
                   float bw,     /* micron */
                   float phase12,
                   float phase23,
                   float phase31,
                   float *pix1,
                   float *pix2,
                   float *pix3);
void detect_climb_fringe_value(float n,
		   float gain,
		   float sig,
		   float pos1, /* micron */
		   float pos2,
		   float pos3,
                   float v12,
		   float v23,
		   float v31,
                   float lambda, /* micron */
                   float bw,     /* micron */
		   float phase12,
		   float phase23,
		   float phase31,
                   float *pix1,
                   float *pix2,
                   float *pix3);
float calc_climb_v_trans(float I1, float I2, float I3);
float *piston_error(float r0, float v, float dt, int n);
float *scintillation(float f0, float dt, int n);
void check_random_seed(void);

/* tango_twiss_integral.c */

float tango_twiss_strehl_tilt(float dr0);
float tango_twiss_strehl_no_tilt(float dr0);
float tango_twiss_vis_tilt(float dr0);
float tango_twiss_vis_no_tilt(float dr0);
void set_up_tango_twiss_integral(void);
float tango_twiss_integral_value(float d, float a, int s);
float tango_twiss_integral_func(float u);
float zbrent_strehl_to_vis_tilt(float strehl);
float zbrent_strehl_to_vis_no_tilt(float strehl);
float zbrent_strehl_to_dr0_tilt(float strehl);
float zbrent_strehl_to_dr0_no_tilt(float strehl);
float zbrent_funct_tilt_strehl(float dr0);
float zbrent_funct_no_tilt_strehl(float dr0);
int set_up_cheb_strehl_to_vis(void);
void clean_up_cheb_strehl_to_vis(void);
float cheb_strehl_to_vis_tilt(float strehl, int m);
float cheb_strehl_to_vis_no_tilt(float strehl, int m);
float corr_t_buscher_eq8(float v, float tdivt0);
float int_func_corr_buscher_eq8(float x);

/* tracking_loss_prob.c */

double rician_prob_density(double Z, double Lambda, double sigma2);
double mean_rician(double Lambda, double sigma2);
double var_rician(double Lambda, double sigma2);
double Lambda_rician(double v, double n);
double sigma2_rician(double n, int npix, double s);
int joint_prob_density(double Lambda, double sigma2, int m, double **sig_noise, double **noise, double **Z);
double fft_pow(double *fft, int num, int m);
void normalize_prob_dens(double *f, int num, double dx);
double prob_track_err(double v, double n, int npix, double s, int m, int b);
float zbrent_funct_prob_to_mag(float mag);
double tracking_loss_prob_to_mag(double prob, int nchan, double R, double tau, double bandwidth, double dqe_sys, double dqe_det, double v, int npix, double s, int m, int b);

/* vis2_snr.c */

float vis2_snr_susi_no_noise(float v, float n);
float vis2_snr_susi_noise(float v, float n, float s);
float vis2_snr_susi_known_noise(float v, float n, float s);
float vis2_snr_TT_no_noise(float v, float n);
float vis2_snr_TT_noise(float v, float n, float s);
float vis2_snr_TT_known_noise(float v, float n, float s);
float vis2_snr_beletic(float v, float n, float s, float npix);

/* visibility_functions.c */

float v_uniform_disk(float baseline, float wavelength, float theta);
float v2_binary(float baseline, float wavelength, float theta1,
                float theta2, float sep, float r);
float v_gaussian(float baseline, float wavelength, float theta);
float v_broadband(
        int profile_type,
        float baseline,float wavelength,float bandwidth,float theta);
float v_broadband_int_funct(float wavelength);
float axisymmetric_direct_calculation(float (*intensity)(float), float u);
float axisymmetric_direct_int_funct(float theta);
float axisymmetric_direct_tot_int_funct(float theta);
float   limb_dark_mp(float b0, float b, float p);
void setup_v_limb_dark_mp(float p);
float v_limb_dark_mp(float baseline, float wavelength, float theta);
float v_limb_dark_mp_int_funct(float theta);
float   limb_dark_lin(float b0, float b, float p);
void setup_v_limb_dark_lin(float p);
float v_limb_dark_lin(float baseline, float wavelength, float theta);
float v_limb_dark_lin_int_funct(float theta);
float   limb_dark_pow(float b0, float b, float p);
void setup_v_limb_dark_pow(float p);
float v_limb_dark_pow(float baseline, float wavelength, float theta);
void setup_v_limb_dark(float mu);
float   limb_dark(float b0, float b, float p);
float v_limb_dark(float baseline, float wavelength, float theta);
void vis_phase_binary(float rho, /* In radians */
                      float theta, /* In radians */
                      float r, /* 0 - 1 */
                      float theta_p, /* In radians */
                      float theta_s, /* In radians */
                      float u, /* In meters */
                      float v, /* In meters */
                      float wavelength, /* In meters */
                      float *vis, /* 0-1 */
                      float *phase /* In radians */);
void vis_closure_phase_binary(float rho, /* In radians */
                      float theta, /* In radians */
                      float r, /* 0 - 1 */
                      float theta_p, /* In radians */
                      float theta_s, /* In radians */
                      float u12, /* In meters */
                      float v12, /* In meters */
                      float u23, /* In meters */
                      float v23, /* In meters */
                      float u31, /* In meters */
                      float v31, /* In meters */
                      float wavelength /* In meters */,
                      float *vis12, /* -1 to 1 */
                      float *vis23, /* -1 to 1 */
                      float *vis31,
                      float *cp); /* -1 to 1 */

/* calibration.c */

void   calibrate_v(double lambda, double dlambda,  /* Observing band (m)*/
            double c1size, double c1sizeerr, /* Radian */
            double c1v, double c1err,
            double c1t, double c1b, /* Cal1 V, time and baseline (m)*/
            double ov, double oerr, double ot, /* Obj V, and time */
            double c2size, double c2sizeerr, /* Radian */
            double c2v, double c2err,
            double c2t, double c2b,
            double *v, double *err); /* Cal1 V, time and baseline (m)*/

void   calibrate_v2(double lambda, double dlambda,  /* Observing band (m)*/
            double c1size, double c1sizeerr, /* Radian */
            double c1v2, double c1err,
            double c1t, double c1b, /* Cal1 V, time and baseline (m)*/
            double ov2, double oerr, double ot, /* Obj V, and time */
            double c2size, double c2sizeerr, /* Radian */
            double c2v2, double c2err,
            double c2t, double c2b,
            double *v2, double *err); /* Cal1 V, time and baseline (m)*/

void   calibrate_closure(
            double c1cp, double c1err, double c1t, /* Cal1 CP and time */
            double ocp, double oerr, double ot, /* Obj CP and time */
            double c2cp, double c2err, double c2t, /* Cal2 CP and time */
            double *cp, double *err); /* Cal1 cp */

#endif /* Ensures only one pass through header file */
