/************************************************************************/
/* photon_models.c							*/
/*									*/
/* Routines for modeling things with real photons.			*/
/*									*/
/************************************************************************/
/*                                        	                      	*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 1995							*/
/************************************************************************/

#include <math.h>
#include <nrc.h>
#include "interferometry.h"
#include <time.h>
#ifdef NOT_AN_APPLE_
#include <values.h>
#endif

/************************************************************************/
/* expected_photon_count()						*/
/*									*/
/* Given aperture size, bandwidth, sample time etc this routine returns */
/* the expected average photon count from a star of given magnitude.	*/
/************************************************************************/

float	expected_photon_count(
	float	mag,		/* Magnitude of star */
	float	R,		/* aperture radius in meters */
	float	tau,		/* Sample time in seconds */
	float	bandwidth,	/* optical bandwidth in nanometers */
	float	dqe_sys,	/* DQE of optical system */
	float	dqe_det)	/* DQE of detector */
	
{
	float	result;

	result = R * R * M_PI * tau * 10.0 * bandwidth * dqe_sys * dqe_det *
		 pow(10.0,(7.0-mag/2.5));

	return result;

} /* expected_photon_count() */

/************************************************************************/
/* photon_count_2_mag()                                                 */
/*                                                                      */
/* Given aperture size, bandwidth, sample time etc this routine returns */
/* the expected magnitude of a star at a curtain count rate.    	*/
/************************************************************************/

float   photon_count_2_mag(
        float   n,              /* Photon Count */
        float   R,              /* aperture radius in meters */
        float   tau,            /* Sample time in seconds */
        float   bandwidth,      /* optical bandwidth in nanometers */
        float   dqe_sys,        /* DQE of optical system */
        float   dqe_det)        /* DQE of detector */

{
        float   result;

        result = 2.5*(7.0-log10(n/(R * R * M_PI * tau * 10.0 * 
			bandwidth * dqe_sys * dqe_det)));

        return result;

} /* photon_count_2_mag() */

/************************************************************************/
/* detect_counts()							*/
/* 									*/
/* Simulates the counts recieved in the two detectors either side of a	*/
/* beam splitter. Returns total count.					*/
/************************************************************************/

float detect_counts(float phase,float v,float n,float s,float *n1,float *n2)
{
	float k1,k2,r1,r2,I1,I2;
	float vcos;

	vcos = v * cos(phase);

	I1 = n * (1.0 + vcos);
	I2 = n * (1.0 - vcos);

	check_random_seed();
	k1 = poidev(I1, &random_seed);
	k2 = poidev(I2, &random_seed);

	if (s != 0.0)
	{
		r1 = gasdev(&random_seed) * s;
		r2 = gasdev(&random_seed) * s;
	}
	else
	{
		r1 = 0.0;
		r2 = 0.0;
	}

	*n1 = k1 + r1;
	*n2 = k2 + r2;

#ifdef DEBUG
	printf("detect_counts(%f,%f,%f,%f,%f,%f)\n",phase,v,n,s,*n1,*n2);
	printf("k1 = %f k2 = %f r1 = %f r2 = %f\n",k1,k2,r1,r2);
#endif

	return (*n1+*n2);

} /* detect_counts() */

/************************************************************************/
/* fringe_value()							*/
/*									*/
/* Calculates the fringe value given a set of parameters.		*/
/* Bandpass assumed square.						*/
/************************************************************************/

float fringe_value(float pos, /* micron */
		   float v,
		   float lambda, /* micron */
		   float bw	 /* micron */,
		   float phase)  /* Rad */
{
	float	frg, envel;
	float	wn,wn1,wn2,dwn,x;

	wn = 1.0/lambda;
	wn1 = 1.0/(lambda - bw/2.0);
	wn2 = 1.0/(lambda + bw/2.0);
	dwn = fabs(wn1-wn2);

	if (pos == 0.0)
	{
		envel = 1.0;
	}
	else
	{
		x = M_PI * pos * dwn;	
		envel = sin(x)/x;
	}

	frg = 1.0 + v * envel * cos(2.0*M_PI*pos*wn+phase);

#ifdef DEBUG
	printf("%f\t%f\t%f\n", pos, envel, frg);
#endif

	return frg;

} /* fringe_value() */

/************************************************************************/
/* detect_fringe_value()						*/
/*									*/
/* Uses fringe_value to calculate a number of photons detected. 	*/
/* Includes photon and read noise.					*/
/************************************************************************/

float detect_fringe_value(float	n, /* Mean photon count */
		   float gain, /* Gain of camera */
		   float sig, /* Read noise */
		   float pos, /* micron */
		   float v,
		   float lambda, /* micron */
		   float bw,	 /* micron */
		   float phase) /* Rad */
{
	n = n * fringe_value(pos, v, lambda, bw, phase);

	/* Calculate the count */

	check_random_seed();
	return (gain * poidev(n, &random_seed) + gasdev(&random_seed) * sig);

} /* detect_fringe_value() */

/************************************************************************/
/* calc_v_trans()							*/
/*									*/
/* Calculate the visibility transfer function given the two intensities.*/
/* Return 1.0 if they are both zero.					*/
/************************************************************************/

float calc_v_trans(float I1, float I2)
{ 

	if (I1 + I2 <= 0.0) return 1.0;

	return (2.0 * sqrt(I1 * I2) / (I1 + I2));

} /* calc_v_trans() */

/************************************************************************/
/* climb_fringe_value()							*/
/*									*/
/* Calculates the fringe value given a set of parameters.		*/
/* Bandpass assumed square.						*/
/* This version for CLIMB like combiner, see log book 5 page 232.	*/
/* Also see log book 6 page 9. Note that the phase for 31 is subtracted */
/* because it goes in the opposite sense to the other two to force 	*/
/* closure.								*/
/* This routine assumes equal and perfect beam balance.			*/
/* See log book 7 page 39						*/
/************************************************************************/

void climb_fringe_value(float pos1, /* micron */
                   float pos2, /* extra path in this beam */
                   float pos3,
                   float v12,
                   float v23,
                   float v31,
                   float lambda, /* micron */
                   float bw,     /* micron */
                   float phase12,
                   float phase23,
                   float phase31,
		   float *pix1,
		   float *pix2,
		   float *pix3)
{
	float	envel12, envel23, envel31;
	float	wn,wn1,wn2,dwn,x;
	float	frg12, frg23, frg31;
	float	t12, t23, t31;

	wn = 1.0/lambda;
	wn1 = 1.0/(lambda - bw/2.0);
	wn2 = 1.0/(lambda + bw/2.0);
	dwn = fabs(wn1-wn2);

	/* The sign of pos in the envelope doesn't really matter */

	if ((pos1 - pos2) == 0.0)
	{
		envel12 = 1.0;
	}
	else
	{
		x = M_PI * (pos1 - pos2) * dwn;	
		envel12 = sin(x)/x;
	}

	if ((pos2 - pos3) == 0.0)
	{
		envel23 = 1.0;
	}
	else
	{
		x = M_PI * (pos2 - pos3) * dwn;	
		envel23 = sin(x)/x;
	}

	if ((pos3 - pos1) == 0.0)
	{
		envel31 = 1.0;
	}
	else
	{
		x = M_PI * (pos3 - pos1) * dwn;	
		envel31 = sin(x)/x;
	}

	/* Now the signs matter */

	frg12 = v12 * envel12 * cos(2.0*M_PI*(pos1 - pos2)*wn+phase12);
	frg23 = v23 * envel23 * cos(2.0*M_PI*(pos2 - pos3)*wn+phase23);
	frg31 = v31 * envel31 * cos(2.0*M_PI*(pos1 - pos3)*wn-phase31);

	t12 = 0.0;
	t23 = 1.0;
	t31 = 0.0;
	*pix1 = 1.0 - t23 * frg23;

	t12 = 1.0/sqrt(2.0);
	t23 = 0.5;
	t31 = t12;
	*pix2 = 1.0 - t12 * frg12 + t23 * frg23 - t31 * frg31;
	*pix3 = 1.0 + t12 * frg12 + t23 * frg23 + t31 * frg31;

} /* climb_fringe_value() */

/************************************************************************/
/* detect_climb_fringe_value()						*/
/*									*/
/* Uses fringe_value to calculate a number of photons detected. 	*/
/* Includes photon and read noise.					*/
/* This version for CLIMB like combiner, see log book 5 page 232.	*/
/************************************************************************/

void detect_climb_fringe_value(float n,
                   float gain,
                   float sig,
                   float pos1, /* micron */
                   float pos2,
                   float pos3,
                   float v12,
                   float v23,
                   float v31,
                   float lambda, /* micron */
                   float bw,     /* micron */
                   float phase12,
                   float phase23,
                   float phase31,
		   float *pix1,
		   float *pix2,
		   float *pix3)
{
	float n1, n2, n3;

	climb_fringe_value(pos1, pos2, pos3, v12, v23, v31, 
			lambda, bw, phase12, phase23, phase31,
			 pix1, pix2, pix3);

	n1 = n * (*pix1);
	n2 = n * (*pix2);
	n3 = n * (*pix3);

	/* Calculate the count */

	check_random_seed();
	*pix1 = (gain * poidev(n1, &random_seed) + gasdev(&random_seed) * sig);
	*pix2 = (gain * poidev(n2, &random_seed) + gasdev(&random_seed) * sig);
	*pix3 = (gain * poidev(n3, &random_seed) + gasdev(&random_seed) * sig);

} /* detect_climb_fringe_value() */

/************************************************************************/
/* calc_climb_v_trans()							*/
/*									*/
/* Calculate visibility transfer function for CLIMB like beam combiner.	*/
/* See log book 5 page 232.						*/
/* Return 1.0 if they are both zero.					*/
/* Returns the version for I1,I2, so you need to be careful how it's	*/
/* called.								*/
/************************************************************************/

float calc_climb_v_trans(float I1, float I2, float I3)
{ 

	if (I1 + I2 + I3 <= 0.0) return 1.0;

	return (2.0 * sqrt(I1 * I2) / (I1 + I2 + I3));

} /* calc_climb_v_trans() */

/************************************************************************/
/* piston_error()							*/
/* 									*/
/* Creates a vector of piston error with the correct spectrum.		*/
/* Memory allocated in this function. Returns a vector [1..n].		*/
/* Result is positon in microns.					*/
/************************************************************************/

float *piston_error(float r0, float v, float dt, int n)
{
	int	i,j;
	float	*pos;
	float	*f;
	float	*theta;
	float	*psamp;
	float	t;
	float	norm;

	/* Allocate the memory */

	pos = vector(1, n);
	f = vector(1, 2*n);
	theta = vector(1, 2*n);
	psamp = vector(1, 2*n);

	/* Do the work */

	check_random_seed();
	for(i=1; i<=2*n ;i++)
	{
		f[i] = (float)i/(2.0* (float)n * dt);
		theta[i] = ran3(&random_seed) * 2.0*M_PI;
		psamp[i] = pow(f[i], (-11.0/6.0));
	}

	norm = sqrt(0.046 * f[1]) * pow(r0, (-5.0/6.0)) * pow(v, (11.0/6.0)) *
		(0.5 / (2.0 * M_PI));

	/* Now create the positions - see IPAC log book 1 */

	for(i=1; i<=n; i++)
	{
		t = (float)i * dt;
		pos[i] = 0.0;
		for(j=1; j<=2*n ;j++)
		{
			pos[i] += (psamp[j] * cos(2.0*M_PI*f[j]*t - theta[j]));
		}

		pos[i] *= norm;
	}

	/* Free memory */

	free_vector(f, 1, 2*n);
	free_vector(theta, 1, 2*n);
	free_vector(psamp, 1, 2*n);

	/* That is all */

	return pos;

} /* piston_error() */

/************************************************************************/
/* scintillation()							*/
/* 									*/
/* Creates a vector of scintillation with the correct spectrum (f0/f).	*/
/* Memory allocated in this function. Returns a vector [1..n].		*/
/* Result is a multiplier.						*/
/* This is clearly nonesense unfortunately as the PS model is terrible.	*/
/************************************************************************/

float *scintillation(float f0, float dt, int n)
{
	int	i,j;
	float	*mult;
	float	*f;
	float	*theta;
	float	*psamp;
	float	t;
	float	a,b;

	/* Allocate the memory */

	mult = vector(1, n);
	f = vector(1, 2*n);
	theta = vector(1, 2*n);
	psamp = vector(1, 2*n);

	/* Do the work */

	check_random_seed();
	a = -1.0/(f0*f0);
	b = -2.0 * a * f0;
	for(i=1; i<=2*n ;i++)
	{
		f[i] = (float)i/(2.0* (float)n * dt);
		theta[i] = ran3(&random_seed) * 2.0 * M_PI;
		if (f[i] > f0)
			psamp[i] = sqrt(f0/f[i]);
		else
			psamp[i] = sqrt((a*f[i]*f[i] + b*f[i]));
	}

	/* Now create the multipliers */

	for(i=1; i<=n; i++)
	{
		t = (float)i * dt;
		mult[i] = 0.0;
		for(j=1; j<=2*n ;j++)
		{
			mult[i] += (psamp[j] * cos(2.0*M_PI*f[j]*t - theta[j]));
		}
		mult[i] = mult[i] + 1.0;
	}

	/* Free memory */

	free_vector(f, 1, 2*n);
	free_vector(theta, 1, 2*n);
	free_vector(psamp, 1, 2*n);

	/* That is all */

	return mult;

} /* scintillation() */

/************************************************************************/
/* check_random_seed()							*/
/*									*/
/* Tries to create a new random seed if we need one.			*/
/************************************************************************/

void check_random_seed(void)
{
	if (random_seed != 0) return;

	random_seed = -1 * time((time_t *)0)%(1<<(sizeof(int)*8-1));

} /* check_random_seed() */
