/************************************************************************/
/* tango_twiss_integral.c 						*/
/*									*/
/* A bunch of routines for calculating and approximating Tango/Twiss	*/
/* formulation for Strehl and Visibility.				*/
/************************************************************************/
/*                                        	                      	*/
/* Center for High Angular Resolution Astronomy				*/
/*									*/
/* ZERNIKE CODE LIBRARY.						*/
/*									*/
/* Telephone: 404-651-2932						*/
/* Fax      : 404-651-1384						*/
/* Internet : theo@chara.gsu.edu					*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : March 1995    						*/
/************************************************************************/

#include <math.h>
#include <nrc.h>
#include "interferometry.h"

/* Local definitions */

#define TOL 0.01

/*
 * Local prototypes
 */

float tango_twiss_integral_value( float d, float a, int s);
float tango_twiss_integral_func(float u);
float zbrent_funct_tilt_strehl(float dr0);
float zbrent_funct_no_tilt_strehl(float dr0);

/* Local Globals */

static float dr0 = 1;
static float alpha = 0;
static int do_strehl = 0;
static float norm = 1.0;
static float local_strehl;

/************************************************************************/
/* tango_twiss_strehl_tilt()						*/
/*									*/
/* Calculates strehl including tilt using tango/twiss formula.		*/
/************************************************************************/

float	tango_twiss_strehl_tilt(float dr0)
{
	return tango_twiss_integral_value(dr0,0.0,1);

} /* tango_twiss_strehl_tilt() */

/************************************************************************/
/* tango_twiss_strehl_no_tilt()                                         */
/*                                                                      */
/* Calculates strehl without tilt using tango/twiss formula.            */
/************************************************************************/

float   tango_twiss_strehl_no_tilt(float dr0)
{
        return tango_twiss_integral_value(dr0,1.0,1);

} /* tango_twiss_strehl_tilt() */

/************************************************************************/
/* tango_twiss_vis_tilt()                                               */
/*                                                                      */
/* Calculates vis including tilt using tango/twiss formula.             */
/************************************************************************/

float   tango_twiss_vis_tilt(float dr0)
{
        return sqrt(tango_twiss_integral_value(dr0,0.0,0));

} /* tango_twiss_vis_tilt() */

/************************************************************************/
/* tango_twiss_vis_no_tilt()                                            */
/*                                                                      */
/* Calculates vis without tilt using tango/twiss formula.               */
/************************************************************************/

float   tango_twiss_vis_no_tilt(float dr0)
{
        return sqrt(tango_twiss_integral_value(dr0,1.0,0));

} /* tango_twiss_vis_tilt() */

/************************************************************************/
/* set_up_tango_twiss_integral() 					*/
/*									*/
/************************************************************************/

void set_up_tango_twiss_integral(void)
{
	norm = 1.0;
        norm = tango_twiss_integral_value(0.0,0.0,1);

} /* set_up_tango_twiss_integral() */

/************************************************************************/
/* tango_twiss_integral_value()						*/
/*									*/
/* Calculates integral in the Noll paper				*/
/************************************************************************/

float tango_twiss_integral_value( float d, float a, int s)
{

	alpha = a;
	dr0 = d;
	do_strehl = s;

	return (1.6976527 * qromo(tango_twiss_integral_func,0.0,1.0,midpnt) / 
			norm); 

} /* integral_value() */

/************************************************************************/
/* tango_twiss_integral_func()						*/
/*									*/
/* Calculates function inside integral of Noll papaer.			*/
/************************************************************************/

float tango_twiss_integral_func(float u)
{
	/* Internals */

	float	multiplier;
	float	result;

	if (do_strehl)
		multiplier = -3.44;
	else
		multiplier = -6.88;

	result = (u * acos(u) - u * sqrt(1 - u*u)) *
		 exp(multiplier * pow(dr0*u,1.66667) * 
			(1.0 - alpha * pow(u,0.33333333)));

	return result;

} /* tango_twiss_integral_func() */

/************************************************************************/
/* zbrent_strehl_to_vis_tilt() 						*/
/*									*/
/* Given strehl returns visibility using zbrent including tilt.		*/
/************************************************************************/

float zbrent_strehl_to_vis_tilt(float strehl)
{
	float	dr0;

        /* Make sure this is reasonable ... */

        if (strehl > 1.0 || strehl < 0.0)
                nrerror("zbrent_strehl_to_vis_tilt: strehl out of range.");

        if (strehl == 1.0) return 1.0;

        if (strehl == 0.0) return 0.0;

	/* Find the dr0 for this strehl */
	
	dr0 = zbrent_strehl_to_dr0_tilt(strehl);
#ifdef DEBUG
	printf("Inside zbrent_strehl_to_vis_tilt(%f), dr0 = %f\n",strehl,dr0);
#endif

	/* Return the visibility for this strehl */

	return tango_twiss_vis_tilt(dr0);

} /* zbrent_strehl_to_vis_tilt() */

/************************************************************************/
/* zbrent_strehl_to_vis_no_tilt()                                       */
/*                                                                      */
/* Given strehl returns visibility using zbrent including tilt.         */
/************************************************************************/

float zbrent_strehl_to_vis_no_tilt(float strehl)
{
        float   dr0;

        /* Make sure this is reasonable ... */

        if (strehl > 1.0 || strehl < 0.0)
                nrerror("zbrent_strehl_to_vis_no_tilt: strehl out of range.");

        if (strehl == 1.0) return 1.0;

        if (strehl == 0.0) return 0.0;

        /* Find the dr0 for this strehl */

        dr0 = zbrent_strehl_to_dr0_no_tilt(strehl);

        /* Return the visibility for this strehl */

        return tango_twiss_vis_no_tilt(dr0);

} /* zbrent_strehl_to_vis_no_tilt() */

/************************************************************************/
/* zbrent_strehl_to_dr0_tilt()						*/
/*									*/
/* Using the zbrent method find dr0 from strehl in case including tilt. */
/************************************************************************/

float zbrent_strehl_to_dr0_tilt(float strehl)
{
	float	dr01,dr02;
	float	s1,s2;

	/* Make sure this is reasonable ... */

	if (strehl > 1.0 || strehl < 0.0)
		nrerror("zbrent_strehl_to_dr0_tilt: strehl out of range.");

	if (strehl == 1.0) return 1.0;

	if (strehl == 0.0) return 0.0;

	/* First we must make sure we have brackited the root */

	local_strehl = strehl;
	dr01 = 0.0;
	s1 = zbrent_funct_tilt_strehl(dr01);

	for(dr02 = 1.0; dr02 <=1000.0; dr02 *= 10.0)
	{
		s2 = zbrent_funct_tilt_strehl(dr02);

		if (s1 * s2 < 0.0) break;
	}

	if (dr02 >= 1000.0)
	{
		nrerror("zbrent_strehl_to_dr0_tilt: could not bracket root.");
	}

	return zbrent(zbrent_funct_tilt_strehl, dr01, dr02, TOL);

} /* zbrent_strehl_to_dr0_tilt() */
		
/************************************************************************/
/* zbrent_strehl_to_dr0_no_tilt()					*/
/*									*/
/* Using the zbrent method find dr0 from strehl in case with no tilt.   */
/************************************************************************/

float zbrent_strehl_to_dr0_no_tilt(float strehl)
{
	float	dr01,dr02;
	float	s1;

        /* Make sure this is reasonable ... */

        if (strehl > 1.0 || strehl < 0.0)
                nrerror("zbrent_strehl_to_dr0_tilt: strehl out of range.");

        if (strehl == 1.0) return 1.0;

        if (strehl == 0.0) return 0.0;

	/* First we must make sure we have brackited the root */

	local_strehl = strehl;
	dr01 = 0.0;
	s1 = zbrent_funct_no_tilt_strehl(dr01);

	for(dr02 = 1.0; dr02 <=1000.0; dr02 *= 10.0)
	{
		if (s1 * zbrent_funct_no_tilt_strehl(dr02) < 0.0) break;
	}

	if (dr02 >= 1000.0)
	{
	    nrerror("zbrent_strehl_to_dr0_no_tilt: could not bracket root.");
	}

	return zbrent(zbrent_funct_no_tilt_strehl, dr01, dr02, TOL);

} /* zbrent_strehl_to_dr0_no_tilt() */

/************************************************************************/
/* zbrent_funct_tilt_strehl() 						*/
/*									*/
/* Function for use in find D/r0 from strehl with tilt included.	*/
/************************************************************************/

float zbrent_funct_tilt_strehl(float dr0)
{
	return  (tango_twiss_strehl_tilt(dr0) - local_strehl);
}

/************************************************************************/
/* zbrent_funct_no_tilt_strehl()                                        */
/*                                                                      */
/* Function for use in find D/r0 from strehl with tilt not included.    */
/************************************************************************/

float zbrent_funct_no_tilt_strehl(float dr0)
{
        return (tango_twiss_strehl_no_tilt(dr0) - local_strehl);
}

/*
 * THE FOLLOWING CODE IS GENERATED AUTOMATICALLY BY THE PROGRAMME CHEBFT
 */

/* Globals */

#define CHEB_N 50
static float *c_tilt;
static float *c_no_tilt;

int set_up_cheb_strehl_to_vis(void)
{
	/* Allocate memory */

	c_tilt = vector(0,CHEB_N-1);
	c_no_tilt = vector(0,CHEB_N-1);

	/* Fill in the numbers... */

	c_tilt[0] = 1.105996e+00;
	c_tilt[1] = 4.472671e-01;
	c_tilt[2] = -1.838589e-02;
	c_tilt[3] = 2.681697e-02;
	c_tilt[4] = -1.387563e-02;
	c_tilt[5] = 8.479696e-03;
	c_tilt[6] = -5.673173e-03;
	c_tilt[7] = 4.269271e-03;
	c_tilt[8] = -3.148762e-03;
	c_tilt[9] = 2.662088e-03;
	c_tilt[10] = -2.101627e-03;
	c_tilt[11] = 1.631667e-03;
	c_tilt[12] = -1.411343e-03;
	c_tilt[13] = 1.264596e-03;
	c_tilt[14] = -1.063851e-03;
	c_tilt[15] = 8.484406e-04;
	c_tilt[16] = -7.393055e-04;
	c_tilt[17] = 7.776430e-04;
	c_tilt[18] = -5.954335e-04;
	c_tilt[19] = 3.567326e-04;
	c_tilt[20] = -7.220373e-04;
	c_tilt[21] = 3.788075e-04;
	c_tilt[22] = -3.697103e-04;
	c_tilt[23] = 4.351402e-04;
	c_tilt[24] = -2.165649e-04;
	c_tilt[25] = 2.631603e-04;
	c_tilt[26] = -2.988327e-04;
	c_tilt[27] = 3.779519e-04;
	c_tilt[28] = -1.121888e-04;
	c_tilt[29] = 1.004597e-04;
	c_tilt[30] = -3.429402e-04;
	c_tilt[31] = 1.287466e-04;
	c_tilt[32] = -3.759040e-05;
	c_tilt[33] = 3.308604e-04;
	c_tilt[34] = -5.449832e-05;
	c_tilt[35] = 9.743482e-05;
	c_tilt[36] = -9.320206e-05;
	c_tilt[37] = 1.198868e-04;
	c_tilt[38] = -1.121906e-04;
	c_tilt[39] = 1.027089e-05;
	c_tilt[40] = -1.825270e-04;
	c_tilt[41] = -7.941252e-06;
	c_tilt[42] = -4.438954e-05;
	c_tilt[43] = 1.395062e-04;
	c_tilt[44] = -1.112549e-05;
	c_tilt[45] = -5.978468e-05;
	c_tilt[46] = -5.085892e-05;
	c_tilt[47] = 1.232224e-06;
	c_tilt[48] = -1.037227e-04;
	c_tilt[49] = -2.494610e-05;


	c_no_tilt[0] = 1.061421e+00;
	c_no_tilt[1] = 4.614999e-01;
	c_no_tilt[2] = 2.209326e-03;
	c_no_tilt[3] = 1.186567e-02;
	c_no_tilt[4] = -1.136641e-02;
	c_no_tilt[5] = 8.748218e-03;
	c_no_tilt[6] = -6.432175e-03;
	c_no_tilt[7] = 4.810369e-03;
	c_no_tilt[8] = -3.517852e-03;
	c_no_tilt[9] = 2.672122e-03;
	c_no_tilt[10] = -2.106652e-03;
	c_no_tilt[11] = 1.702354e-03;
	c_no_tilt[12] = -1.397638e-03;
	c_no_tilt[13] = 1.162692e-03;
	c_no_tilt[14] = -9.644549e-04;
	c_no_tilt[15] = 8.562983e-04;
	c_no_tilt[16] = -7.555053e-04;
	c_no_tilt[17] = 6.483074e-04;
	c_no_tilt[18] = -5.731923e-04;
	c_no_tilt[19] = 5.515717e-04;
	c_no_tilt[20] = -4.974582e-04;
	c_no_tilt[21] = 3.824081e-04;
	c_no_tilt[22] = -4.043681e-04;
	c_no_tilt[23] = 2.856978e-04;
	c_no_tilt[24] = -3.842844e-04;
	c_no_tilt[25] = 3.224429e-04;
	c_no_tilt[26] = -2.171289e-04;
	c_no_tilt[27] = 2.364118e-04;
	c_no_tilt[28] = -2.305489e-04;
	c_no_tilt[29] = 2.326197e-04;
	c_no_tilt[30] = -1.440824e-04;
	c_no_tilt[31] = 1.228440e-04;
	c_no_tilt[32] = -1.864992e-04;
	c_no_tilt[33] = 1.868898e-04;
	c_no_tilt[34] = -8.037432e-05;
	c_no_tilt[35] = 1.205376e-04;
	c_no_tilt[36] = -1.400915e-04;
	c_no_tilt[37] = 6.745441e-05;
	c_no_tilt[38] = -7.048649e-05;
	c_no_tilt[39] = 6.018663e-05;
	c_no_tilt[40] = -1.266231e-04;
	c_no_tilt[41] = 8.652986e-05;
	c_no_tilt[42] = -1.641891e-05;
	c_no_tilt[43] = 3.253325e-05;
	c_no_tilt[44] = -4.292444e-05;
	c_no_tilt[45] = 3.484682e-05;
	c_no_tilt[46] = -4.611665e-05;
	c_no_tilt[47] = -6.023469e-06;
	c_no_tilt[48] = -1.392721e-05;
	c_no_tilt[49] = -3.198069e-06;


	return CHEB_N;

} /* set_up_cheb_strehl_to_vis() */

void clean_up_cheb_strehl_to_vis(void)
{
	/* De-allocate memory */

	free_vector(c_tilt,0,CHEB_N-1);
	free_vector(c_no_tilt,0,CHEB_N-1);

} /* clean_up_cheb_strehl_to_vis() */

float cheb_strehl_to_vis_tilt(float strehl, int m)
{
	if (strehl > 0.8) return strehl;

	if (m > CHEB_N) m = CHEB_N;

	return chebev(0.0,1.0,c_tilt, m, strehl);

} /* cheb_strehl_to_vis_tilt() */

float cheb_strehl_to_vis_no_tilt(float strehl, int m)
{
	if (strehl > 0.8) return strehl;

	if (m > CHEB_N) m = CHEB_N;

	return chebev(0.0,1.0,c_no_tilt, m, strehl);

} /* cheb_strehl_to_vis_no_tilt() */

/* 
 * Also here are the buscher calculations
 */

float int_func_corr_buscher_eq8(float x);

static float corr_buscher_eq8_temp;

/************************************************************************/
/* corr_t_buscher_eq8()                                                 */
/*                                                                      */
/* Returns the correlation (Vis squared) according to equation 8 of     */
/* Buscher's optimization of interferometer paper MNRAS 235, 1203-1226  */
/************************************************************************/

float corr_t_buscher_eq8(float v, float tdivt0)
{
        float   result;

        if (tdivt0 == 0.0) return v;

        /* Start by stetting things up */

        corr_buscher_eq8_temp = tdivt0;

        result = 2.0 * v*v/tdivt0 * qromo(int_func_corr_buscher_eq8,
                                        0.0,tdivt0,midpnt);

        return result;

} /* corr_t_buscher_eq8() */

float int_func_corr_buscher_eq8(float x)
{
        float result;

        if (x == 0.0) return 0.0;

        result = (1.0 - x/corr_buscher_eq8_temp) * exp(-1.0*pow(x,5.0/3.0));

#ifdef DEBUG
        printf("int_func_corr_buscher_eq8(%f) = %f\n",x,result);
#endif

        return result;
}
