/************************************************************************/
/* tracking_loss_prob.c							*/
/*									*/
/* Routines for fringe tracking loss probability calculations		*/
/* Some are taken from NRC but changed to double precision.		*/
/* Some are taken from Peter Lawson's code and changed to ANSII C.	*/
/*									*/
/************************************************************************/
/*                                        	                      	*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 1995							*/
/************************************************************************/

#include <math.h>
#include <nrc.h>
#include "interferometry.h"
#ifdef NOT_AN_APPLE_
#include <values.h>
#endif

/*
 * Some local definitions.
 */

#define N_PROB  64
#define MAX_PROB 8192
#define USE_LOG

/************************************************************************/
/* rician_prob_density()						*/
/*									*/
/* Caclulates Rician probablitly density function.			*/
/************************************************************************/

double	rician_prob_density(double Z, 	  /* Value for which prob is wanted */
                   double Lambda,/* Parameter 1 of distn */
                   double sigma2) /*Parameter 1 of distn */
{
	double	result;

	result = Z/sigma2 * exp((Z*Z + Lambda*Lambda)/(-2.0*sigma2)) * 
			dbessi0(Lambda*Z/sigma2);

#ifdef DEBUG
	printf("# rician_prob_density(%lf,%lf,%lf)=%lf\n",Z,Lambda,sigma2,
			result);
	fflush(stdout);
#endif

	return result;

} /* rician_prob_density() */

/************************************************************************/
/* mean_rician()							*/
/*									*/
/* Returns the mean value of a rician distribution.			*/
/************************************************************************/

double mean_rician(double Lambda, /* Parameter 1 of distn */
                   double sigma2) /*Parameter 1 of distn */
{
	double mean;
	double rho;

	rho = Lambda*Lambda / (2.0 * sigma2);
	mean = exp(-1.0*rho/2.0) * 
		sqrt(M_PI*sigma2/2.0) *
		((1.0+rho) * dbessi0(rho/2.0) + rho * dbessi1(rho/2.0));

#ifdef DEBUG
        printf("# mean_rician(%lf,%lf)=%lf\n",Lambda,sigma2,
                        mean);
	printf("dbessi0(%lf/2.0) = %lf dbessi1(%lf/2.0) = %lf\n",
		rho, dbessi0(rho/2.0), rho, dbessi1(rho/2.0));
        fflush(stdout);
#endif

	return mean;

} /* mean_rician() */

/************************************************************************/
/* var_rician()                                                         */
/*                                                                      */
/* Returns the variance value of a rician distribution.                 */
/************************************************************************/

double var_rician( double Lambda,/* Parameter 1 of distn */
                   double sigma2) /*Parameter 1 of distn */
{
	double mean;
	double var;

	mean = mean_rician(Lambda, sigma2);

	var = Lambda*Lambda + 2.0*sigma2 - mean*mean;

#ifdef DEBUG
        printf("# var_rician(%lf,%lf)=%lf\n",Lambda,sigma2,
                        var);
        fflush(stdout);
#endif

	return var;

} /* var_rician() */

/************************************************************************/
/* Lambda_rician()							*/
/*									*/
/* Returns visibility estimator Lambda given photon count and visibility*/
/************************************************************************/

double	Lambda_rician(double v,	/* Visibility */
		      double n)	/* Average photon count */
{
	double result;

	result = n*v/2.0;

#ifdef DEBUG
	printf("# Lambda_rician(%lf, %lf) = %lf\n",v ,n, result);
	fflush(stdout);
#endif

	return result;

} /* Lambda_rician() */

/************************************************************************/
/* sigma2_rician()							*/
/*									*/
/* Returns the sigma squared estimate for rician distribution based on 	*/
/* visibility read noise pixels and photon counts.			*/
/************************************************************************/

double	sigma2_rician(double n, int npix, double s)
{
	double	result;

	result = (n + npix*s*s)/2.0;

#ifdef DEBUG
	printf("# sigma2_rician(%lf, %d, %lf) = %lf\n", n, npix, s, result);
	fflush(stdout);
#endif

	return result;

} /* sigma2_rician() */

/************************************************************************/
/* joint_prob_density()							*/
/* 									*/
/* Calculates the joint probability density function as described 	*/
/* by Peter Lawson JOSA-A, 12, 366.					*/
/* Results pu tin location result, Z's put in array Z both locally 	*/
/* allocated.								*/
/************************************************************************/

int joint_prob_density(double Lambda, double sigma2, int m,
		double **sig_noise, double **noise, double **Z)
{
	double	orig_max_Z,max_Z, inc_Z;
	double	last_p,p;
	int	i;
	double	sig_noise_scale;
	double	noise_scale;
	int	num; /* Number of data points used */

	/* Make m is reasonable */

	if (m > 1000)
	{
		fprintf(stderr,"Warning: joint_prob_density: m > 1000!\n");
		m = 1000;
	}

	/* 
	 * First we need to find the Z value for which the
	 * rician distribution is very small.
	 */

	orig_max_Z = 0.0;
	p = -1.0;
	do 
	{
		last_p = p;
		p = rician_prob_density(orig_max_Z, Lambda, sigma2);
		orig_max_Z += 1.0;

	} while(p > last_p || p > 0.000001);

	/* Make this m times as large to cover full range after expansion */

	max_Z = m*orig_max_Z;

	/* Choose an approriate size for FFT's */

	for(num = 1; num < m*N_PROB; num = num <<1);

	/* But let's not make it too big!!!! */

	if (num > MAX_PROB) num = MAX_PROB; 

	/* Allocate memory */

	*Z = dvector(0, num-1);
	*sig_noise = dvector(0, num-1);
	*noise = dvector(0, num-1);

#ifdef DEBUG
	printf("#joint_prob_density(%lf, %lf, %d, sig_noise, noise, Z)\n",
		Lambda,sigma2,m);
	printf("#Maximum Z value found %lf (P = %lf)\n",max_Z,p);
	printf("# num = %d\n",num);
#endif

	/* OK, now set up the Z array and calculate the rician distributions */

	inc_Z = max_Z/(num-1);

	for(i=0; i<num; i++)
	{
		(*Z)[i] = i*inc_Z;

		if ((*Z)[i] < orig_max_Z)
		{
			(*sig_noise)[i] = 
				rician_prob_density((*Z)[i], Lambda, sigma2);
			(*noise)[i] = 
				rician_prob_density((*Z)[i], 0.0, sigma2);
		}
		else
		{
			(*sig_noise)[i] = (*noise)[i] = 0.0;
		}
	}

	/* If m is one that is all we need to do..... */

	if (m == 1) return num;

	/* Otherwise we do an FFT */

	drealft((*sig_noise)-1, num, 1);
	drealft((*noise)-1, num, 1);

	/* Now raise this to the appropriate power of m */

	sig_noise_scale = fft_pow(*sig_noise, num, m);
	noise_scale = fft_pow(*noise, num, m);
	
	/* And.. inverse transform them */

	drealft((*sig_noise)-1, num, -1);
	drealft((*noise)-1, num, -1);

	/* Now we force this to be an average by dividing Z by m */

	for(i=0;i<num;i++) (*Z)[i] /= (double)m;
	inc_Z /= (double)m;

	/* Normalize the buggers .... */

	normalize_prob_dens(*sig_noise, num, inc_Z);
	normalize_prob_dens(*noise, num, inc_Z);

	/* That is it.... */

	return num;

} /* joint_prob_density() */

/************************************************************************/
/* fft_pow()								*/
/*									*/
/* Raises a fourier transform in fft length num as created by drealft()	*/
/* To the power m. Returns a scaling factor used.			*/
/************************************************************************/

double fft_pow(double *fft, int num, int m)
{
	double	scale;
	int	i,j;
	double	r, theta;
	double	two_pi = 2.0*M_PI;

#ifdef DEBUG
	printf("# fft_pow(fft, %d, %d)...\n",num, m);
	printf("# BEFORE....\n");
	printf("# fft[ 0] = %lf\t0.0\n",fft[0]);
	for(i=2;i<num-1;i+=2) 
		printf("# fft[%2d] = %lf\t%lf\n",i/2,fft[i],fft[i+1]);
	printf("# fft[%2d] = %lf\t0.0\n",num/2,fft[num-1]);
#endif

#ifdef USE_LOG /* Do this using logs to ensure we do not overflow */
#ifdef DEBUG
	printf("#Using LOG domain for calculations\n");
#endif

	/* Find scale factor based on DC term... */

	if (fft[0] > 0.0)
	{
		scale = log(fft[0])*m;
		fft[0] = 1.0;
	}
	else
	{
		scale = 0.0;
		fft[0] = 0.0;
	}

#ifdef DEBUG
	printf("# Scale = %lf\n",scale);
#endif

	/* Now we need to do the Nyquist term */

	if (fft[1] > 0.0)
		fft[1] = exp(log(fft[0])*m - scale);
	else
		fft[1] = 0.0;

	/* and then run through the rest of them */

	for(i=2; i<num-1; i+=2)
	{
		j = i+1;

		/* Go to cylindrical coords */

		r = sqrt(fft[i]*fft[i]+fft[j]*fft[j]);

		if (fft[j]+fft[i] == 0.0) 
			theta = 0.0;
		else
			theta = atan2(fft[j],fft[i]);

		while (theta < 0.0) theta+=two_pi;

		/* Raise it to the mth power with scaling */

		if (r > 0.0)
			r = exp(log(r)*m-scale);
		else
			r = 0.0;

		theta *= (double)m;
		theta = fmod(theta, two_pi);

		/* Now go back to cartesian coords */

		fft[i] = r * cos(theta);
		fft[j] = r * sin(theta);
	}

#else	/* Do it the direct way. This may cause under/overflow */
#ifdef DEBUG
	printf("#Using LINEAR domain for calculations\n");
#endif
	scale = 0.0;

	/* Do DC term */

        if (fft[0] > 0.0)
                fft[0] = pow(fft[0],(double)m);
        else
                fft[0] = 0.0;

	/* Do Nyquist term */

        if (fft[1] > 0.0)
                fft[1] = pow(fft[1],(double)m);
        else
                fft[1] = 0.0;

	/* Do all the other terms */
	
        for(i=2; i<num-1; i+=2)
        {
                j = i+1;

                /* Go to cylindrical coords */

                r = sqrt(fft[i]*fft[i]+fft[j]*fft[j]);

		if (fft[j]+fft[i] == 0.0) 
			theta = 0.0;
		else
			theta = atan2(fft[j],fft[i]);

                while (theta < 0.0) theta += two_pi;

		if (r > 0.0)
		{
			r = pow(r,(double)m);
		}
		else
		{
			r = 0.0;
		}

                theta *= (double)m;
		theta = fmod(theta, two_pi);

                /* Now go back to cartesian coords */

                fft[i] = r * cos(theta);
                fft[j] = r * sin(theta);
        }
#endif

#ifdef DEBUG
	printf("# AFTER....\n");
	printf("# fft[ 0] = %lf\t0.0\n",fft[0]);
	for(i=2;i<num-1;i+=2) 
		printf("# fft[%2d] = %lf\t%lf\n",i/2,fft[i],fft[i+1]);
	printf("# fft[%2d] = %lf\t0.0\n",num/2,fft[num-1]);
#endif
	return scale;

} /* fft_pow() */

/************************************************************************/
/* normalize_prob_dens()						*/
/*									*/
/* Normalize and smooth a probability density function.			*/
/************************************************************************/

void normalize_prob_dens(double *f, int num, double dx)
{
	double	min,norm;
	int i;

#ifdef DEBUG
	printf("# normalize_prob_dens(f, %d)\n",num);
#endif

	/* First loop through and get rid of the ripple .... */

	min = 0.0;
	for(i=0; i<num; i++)
	{
		/* Look for the minimum */
	
		if (f[i] < min) min = f[i];

		/* If this is negative it's ripple, clean it up */

		if (f[i] < 0.0)
		{
			f[i] = 0.0;
			if (i < num-2) f[i+1] = 0.0; /* second part of ripple */
		}
	}

	/* Now normalize the thing */

	norm = dsimpson(num, f, dx);

#ifdef DEBUG
	printf("# min  = %lf\n",min);
	printf("# norm = %lf\n",norm);
#endif

	for(i=0; i<num; i++) f[i] /= norm;

	/* Now see if the ripple was bad.... */

	if (min/norm < -0.000001)
	{
/* This is getting anoying! 
		fprintf(stderr,"Warning: normalize_prob_dens: Ripple %lf\n",
			min/norm);
*/
	}

} /* normalize_prob_dens() */

/************************************************************************/
/* prob_track_err()							*/
/*									*/
/* Calculates probability of tracking loss given V, N, npix, S and m	*/
/* as set out in paper by Lawson and extend to CCD's by me.		*/
/************************************************************************/

double prob_track_err(double v, double n, int npix, double s, int m, int b)
{
	double	Lambda, sigma2; 	/* info on Rician distributions */
	double	*sig_noise,*noise,*Z;		/* The dists themselves */
	int	n_prob_dens;		/* Number of parts to dist. */
	int 	i;			/* For looping */
	double	sub_integral;
	double	integral;
	double	inc_Z;
	double	result;

	/* Find the Lambda and variance */

	Lambda = Lambda_rician(v,n);
	sigma2    = sigma2_rician(n, npix, s);

	/* Calculate the probability density functions */

	n_prob_dens = joint_prob_density(Lambda,sigma2,m,&sig_noise,&noise,&Z);

	/* Set up Z incriment for simpson routine */

	inc_Z = Z[1] - Z[0];
	
#ifdef DEBUG
	printf("prob_track_err(%lf,%lf,%d,%lf,%d,%d)\n",v,n,npix,s,m,b);
	printf("Lambda = %lf\n",Lambda);
	printf("sigma2 = %lf\n",sigma2);
	printf("n_prob_dens = %d\n",n_prob_dens);
	printf("inc_Z = %lf\n",inc_Z);
#endif

	/* Set up sig & noise array by multiplying by internal integral */

	for(i=0;i<n_prob_dens; i++)
	{
#ifdef DEBUG
		printf("#%d I1 = %lf ",i,sub_integral);
#endif
		/* Calculate the integral */

		if (sig_noise[i] > 0.0)
		{
			sub_integral = dsimpson(i+1, noise, inc_Z);

			if (b > 1)
			{
				sub_integral = pow(sub_integral,(double)b);
			}

			sig_noise[i] *= sub_integral;
		}
#ifdef DEBUG
		printf("#I1^%d = %lf I2 = %lf\n",b,sub_integral,sig_noise[i]);
#endif
	}

	/* Now do the integral for the final result */

	integral = dsimpson(n_prob_dens, sig_noise, inc_Z);

	/* We do not want any silly huge numbers */

	if (isnan(integral)) 
	{
		integral = 1.0;
	}

	/* OK work out the result..... */

	result = 1.0 - integral;

	/* 
	 * Kludge time......
	 * This number should never be exactly zero or one because of the 
	 * numerical nature of the calculation. Therefore if it is zero 
	 * something weird has happened. We therefore force it to be 1.0, 
 	 * a reasonable guess is that tracking failure will not occur.
  	 * Also, this number should never be outside the range 0.0 to 1.0
	 */

	if (result <= MINDOUBLE || result >= 1.0) 
	{
		result = 0.0;
	}

	/* Free memory */

	free_dvector(sig_noise,0,n_prob_dens-1);
	free_dvector(noise,0,n_prob_dens-1);
	free_dvector(Z,0,n_prob_dens-1);

	/* That is all */

	return result;

} /* prob_track_err() */

/************************************************************************/
/* tracking_loss_prob_to_mag()						*/
/* 									*/
/* Given aperture size, DQEs, sample times etc this routine works out	*/
/* the stellar magnitude that produces a given probability of tracking  */
/* loss. It uses the zbrent routine and could very well take a LONG	*/
/* time to run. 							*/
/************************************************************************/

/*
 * Some local variables
 */

static double local_prob;
static int    local_nchan;
static double local_R;
static double local_tau;
static double local_bandwidth;
static double local_dqe_sys;
static double local_dqe_det;
static double local_v;
static int local_npix;
static double local_s;
static int local_m;
static int local_b;

/*
 * A function that will be used in the NRC zbrent() routine....
 */

float zbrent_funct_prob_to_mag(float mag)
{
	float	n;
	float 	prob;

	n = expected_photon_count(mag, local_R, local_tau, 
		local_bandwidth, local_dqe_sys, local_dqe_det);

	prob = pow(prob_track_err(local_v, n, local_npix, 
		local_s, local_m, local_b),(double) local_nchan);

#ifdef DEBUG
	printf("mag = %f n= %f prob= %f local_prob = %f return value = %f\n",mag,n,prob,local_prob,prob-local_prob);
	fflush(stdout);
#endif

	return (prob - local_prob);

} /* zbrent_funct_prob_to_mag() */

double tracking_loss_prob_to_mag(
		double prob,
		int    nchan,
		double R,
		double tau,
		double bandwidth,
		double dqe_sys,
		double dqe_det,
		double v,
		int    npix,
		double s,
		int    m,
		int    b)
{
	/* Set up globals */ 

	local_prob = prob;
	local_nchan = nchan;
	local_R = R;
	local_tau = tau;
	local_bandwidth = bandwidth;
	local_dqe_sys = dqe_sys;
	local_dqe_det = dqe_det;
	local_v = v;
	local_npix = npix;
	local_s = s;
	local_m = m;
	local_b = b;

	/* OK, do the dirty work..... */

	return zbrent(zbrent_funct_prob_to_mag, 0.0, 12, 0.001);

} /* tracking_loss_prob_to_mag() */
