/************************************************************************/
/* clock.c	 							*/
/*                                                                      */
/* Controller application routines for the CHARA real time clock.	*/
/* This version is for using RT Linux and is largely obsolete.		*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : May 1998                                                    */
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "clock.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include "../../slalib/slalib.h"
#include "../../charalib/chara.h"
#ifndef KERNEL_OTHER
#include <mbuff.h>
#endif

/* Globals */

int      fd_comm;                /* Command channel file handle */
int      fd_data_out;            /* Data out channel file handle */
int      fd_data_in;             /* Data in channel file handle */
int      fd_err;                 /* Error channel file handle */
CHARA_TIME *chara_time_now = NULL;      /* Shared memory clock time */
CHARA_TIME *chara_time_next_second = NULL;      /* Shared memory clock time */
static 	 int max_clock_offset=32000;
static void (*clock_offset_callback)(void) = NULL;
static bool bypass_clock_driver = FALSE;
static int  bypass_clock_delay = DEFAULT_BYPASS_CLOCK_DELAY;

/*
 * Offset for setting the time in mS
 */

#define SYNC_OFFSET 15

/************************************************************************/
/* set_bypass_clock_delay()                                             */
/*                                                                      */
/* Sets global bypass_clock_delay.                                      */
/* Returns an error level.                                              */
/************************************************************************/

int set_bypass_clock_delay(int delay)
{
	bypass_clock_delay = delay;

	return NOERROR;

} /* set_bypass_clock_delay() */

/************************************************************************/
/* set_bypass_clock_driver()                                            */
/*                                                                      */
/* Allows user to set local global bypass_clock_driver.                 */
/* If set to true, all functionality is there, well mostly, but the     */
/* time is based on the local linux clock. This should make thins       */
/* mostly transparent to the user program.                              */
/* Returns an error level.                                              */
/************************************************************************/

int set_bypass_clock_driver(bool bypass)
{
	bypass_clock_driver = bypass;

	return NOERROR;

} /* set_bypass_clock_driver() */

/************************************************************************/
/* open_rt_fifos()							*/
/* 									*/
/* Opens all FIFOs for comms with RT module. Returns TRUE if all OK,	*/
/* FALSE otherwise.							*/
/* Sends no text to stderr.						*/
/************************************************************************/

char open_rt_fifos(void)
{
	if (bypass_clock_driver) return TRUE;

	if ((fd_comm = open(COMM_FIFO_DEV, O_WRONLY)) < 0) 
	{
		return FALSE;
	}

	if ((fd_data_out = open(DATA_OUT_FIFO_DEV, O_RDONLY)) < 0) 
	{
		return FALSE;
	}

	if ((fd_data_in = open(DATA_IN_FIFO_DEV, O_WRONLY)) < 0) 
	{
		return FALSE;
	}

	if ((fd_err = open(ERR_FIFO_DEV, O_RDONLY)) < 0) 
	{
		return FALSE;
	}

	return TRUE;

} /* open_rt_fifos() */

/************************************************************************/
/* close_rt_fifos()							*/
/* 									*/
/* CLoses all FIFOs for comms with RT module.				*/
/************************************************************************/

void close_rt_fifos(void)
{
	if (!bypass_clock_driver)
	{
		close(fd_comm);
		close(fd_data_out);
		close(fd_data_in);
		close(fd_err);
	}

} /* close_rt_fifos() */

/************************************************************************/
/* send_rt_command()							*/
/*									*/
/* Attempts to send a command into the command FIFO. Returns TRUE if	*/
/* it went well and FALSE otherwise.					*/
/* Sends no text to stderr.						*/
/************************************************************************/

char send_rt_command(unsigned int command, unsigned char argument)
{
	rt_command_struct c;

	if (bypass_clock_driver) return TRUE;

	c.comm = command;
	c.data = argument;

	if (write(fd_comm,&c,sizeof(c)) != sizeof(c))
	{
		error(WARNING,
		"Failed to write command %d:%d : %s",
				command,argument,strerror(errno));
		return FALSE;
	}

	return TRUE;

} /* send_rt_command() */

/************************************************************************/
/* get_rt_error_message()						*/
/*									*/
/* Polls the ERR_FIFO and see's if there is anything in there to get.	*/
/* If there is nothing there, it returns 0. If there is something	*/
/* there is grabs it up till the first NULL it sees and places		*/
/* it into the string s (assumed large enough...) and returns the	*/
/* length of the string.						*/
/************************************************************************/

int get_rt_error_message(char *s)
{
	char	*ps;
	fd_set	err_fds;	/* fdset for select() function */
        struct timeval tv;      /* Time to wait */

	if (bypass_clock_driver) return 0;

	/* Set up for the select function call */

	FD_ZERO(&err_fds);
	FD_SET(fd_err, &err_fds);
	tv.tv_sec = 0;           /* Wait no more than 1 usec */
        tv.tv_usec = 1;

	select(FD_SETSIZE, &err_fds, NULL, NULL, &tv);			

	/* OK, let's get the string */

	if (FD_ISSET(fd_err, &err_fds)) 
	{
		/* Get the message */

		for (ps = s; read(fd_err , ps, 1) == 1 && *ps != 0; ps++);
		return strlen(s);
	}
	else
	{
		return 0;
	}

} /* get_rt_error_message() */

/************************************************************************/
/* read_rt_data()							*/
/*									*/
/* Polls the DATA_OUT_FIFO and see's if there is anything in there to 	*/
/* get.									*/
/* If there is nothing there, it returns 0. If there is something	*/
/* there it tries to read size bytes into the buffer.			*/
/* It's a lot like read() but it doesn't hang around too long.		*/
/************************************************************************/

int read_rt_data(void *buf, size_t count)
{
	fd_set	data_fds;	/* fdset for select() function */
        struct timeval tv;      /* Time to wait */

	if (bypass_clock_driver) return 0;

	/* Set up for the select function call */

	FD_ZERO(&data_fds);
	FD_SET(fd_data_out, &data_fds);
	tv.tv_sec = 0;           /* Wait no more than 10 usec */
        tv.tv_usec = 10;

	select(FD_SETSIZE, &data_fds, NULL, NULL, &tv);			

	/* OK, let's get the string */

	if (FD_ISSET(fd_data_out, &data_fds)) 
	{
		return read(fd_data_out , buf, count);
	}
	else
	{
		return 0;
	}

} /* read_rt_data() */

/************************************************************************/
/* write_rt_data()							*/
/*									*/
/* Sends data to the DATA IN FIFO, ie to the RT process.		*/
/* Some user command must be used to make the RT process do something   */
/* with it at the other end. Returns number of bytes actually sent.	*/
/* You need to be carefull not to send too much at a time as the FIFO	*/
/* buffers are not large.						*/
/* It's a lot like write() but it doesn't hang around too long.		*/
/************************************************************************/

int write_rt_data(void *buf, size_t count)
{
	
	if (bypass_clock_driver) return count;

	return write(fd_data_in,buf,count);

} /* write_rt_data() */

/************************************************************************/
/* flush_rt_data_in()							*/
/*									*/
/* Calls the system RT function FLUSH_DATA, which tries to empty	*/
/* the DATA_IN_FIFO of the RT process. 					*/
/* Returns the number of btes actually flushed.				*/
/* Note that since this RT system call puts the number of bytes it	*/
/* flushed, you should almost always call flush_rt_data_out() before	*/
/* using this function.							*/
/************************************************************************/

int	flush_rt_data_in(void)
{
	CHARA_TIME    count;

	if (bypass_clock_driver) return 0;
	
	/* OK try and flush the DATA_IN buffer */

        if (!send_rt_command(FLUSH_DATA,0))
        {
                return 0;
        }

	read_rt_data(&count,sizeof(count));

	return (int)count;

} /* flush_rt_data_in() */

/************************************************************************/
/* flush_rt_data_out()							*/
/*									*/
/* Tries to read any existing 						*/
/* in the DATA_OUT_FIFO until it is empty. This function sometimes	*/
/* helpful to clear up locked data transfers.				*/
/* Returns the number of bytes actually flushed.			*/
/* Usually used with flush_rt_data_in().				*/
/************************************************************************/

int	flush_rt_data_out(void)
{
	unsigned char c;
	int    count;

	if (bypass_clock_driver) return 0;

	/* Empty the incoming buffer stream */

	count = 0;
	while(read_rt_data(&c, sizeof(c)) != 0) count++;

	return count;

} /* flush_rt_data_out() */

/************************************************************************/
/* flush_data()								*/
/*									*/
/* Gives UI acces to the flush_rt_data() function.			*/
/* Ensures all FIFOs are empty.                                         */
/* Note that this removes everything from the ERROR stream also so if   */
/* there are any errors there they will be flushed too. Only looks for  */
/* up to 1000 errors in case something terrible is wrong.               */
/* Chara UI callable function.                                          */
/************************************************************************/

int flush_data(int argc, char **argv)
{
        int count_in,count_out,count_error;
        char s[256];

	if (bypass_clock_driver) return NOERROR;

        count_out = flush_rt_data_out();
        count_in = flush_rt_data_in();
        for(count_error=0; count_error<1000; count_error++)
        {
                if (get_rt_error_message(s) == 0) break;
        }
        if (count_error >= 1000) 
                error(WARNING,"More than 1000 errors flushed!");

        message(system_window,
        "Bytes Flushed: DATA_IN_FIFO = %d DATA_OUT_FIFO = %d ERROR_FIFO = %d",
                        count_in,count_out,count_error);

        return NOERROR;

} /* flush_data() */

/************************************************************************/
/* clear_lost_counts()						        */
/*									*/
/* Uses RT system command CLEAR_LOST_COUNTS to set lost_ticks=0 and	*/
/* lost_seconds = 0. Normally only used after you have set the clock.	*/
/************************************************************************/

int clear_lost_counts(int argc, char **argv)
{
	if (bypass_clock_driver) return NOERROR;

        if (!send_rt_command(CLEAR_LOST_COUNTS,0))
        {
                return error(ERROR,"Failed to clear counters.");
        }

	return NOERROR;

} /* clear_lost_counts() */

/************************************************************************/
/* in_set()								*/
/*									*/
/* Uses RT system command TTL_CONTROL in order to poll the value	*/
/* of the input TTL pins. Arguments can be 1, 2 or 3, depending on	*/
/* which TTL input you want to check. Returns TRUE if TTL input is	*/
/* set and FLASE if it is not. Returns -1 on error.			*/
/************************************************************************/

int	in_set(int in_or_out, int port)
{
	char	result;

	if (bypass_clock_driver) return 0;

	/* Send the right command */

	if (in_or_out == TTL_IN)
	{
		switch(port)
		{
		case 1: result = send_rt_command(TTL_CONTROL,IN_1);
			break;

		case 2: result = send_rt_command(TTL_CONTROL,IN_2);
			break;

		case 3: result = send_rt_command(TTL_CONTROL,IN_3);
			break;
	
		default: return -1;
		}
	}
	else
	{
		switch(port)
		{
		case 1: result = send_rt_command(TTL_CONTROL,OUT_1);
			break;

		case 2: result = send_rt_command(TTL_CONTROL,OUT_2);
			break;

		default: return -1;
		}
	}

	if (!result) 
	{
		return -2;
	}

	/* Get the result */

	if (read_rt_data(&result,sizeof(result)) != sizeof(result)) 
	{
		return -3;
	}

	/* That is all */

	if (result)
		return TRUE;
	else
		return FALSE;

} /* in_set() */

static char    *port_types[] = {
                                "out_1",
                                "out_2",
                                NULL
                          };

static char    *port_value_types[] = {
                                "clear",
                                "set  ",
                                NULL
                          };

/************************************************************************/
/* out_set()								*/
/*									*/
/* Uses RT system command TTL_CONTROL in order to control the value	*/
/* of the output TTL pins. 						*/
/* Expects the port number as first argument and the value to set it	*/
/* too as the second argument. If no arguments it polls the user.	*/
/************************************************************************/

int out_set(int argc, char **argv)
{
	int	port,value;
	unsigned char	argument;

	if (bypass_clock_driver) return NOERROR;

	/* OK let's sort out the arguments */

	werase(system_window);
	wrefresh(system_window);
	if (argc > 1)
	{
		sscanf(argv[1],"%d",&port);
	}
	else
	{
		port = 0;
		clean_command_line();
		if (quick_edit("Output port",port_types[port],&port,
			port_types,ENUMERATED) == KEY_ESC) return NOERROR;
		if (port == 0) 
			port = 1;
		else
			port = 2;
	}

	werase(system_window);
	wrefresh(system_window);
	if (argc > 2)
	{
		sscanf(argv[2],"%d",&value);
	}
	else
	{
		value = 0;
		clean_command_line();
		if (quick_edit("Port action",port_value_types[value],&value,
			port_value_types,ENUMERATED) == KEY_ESC) return NOERROR;
	}

	if (port == 1 && value == 0)
	{
		argument = CLEAR_OUT_1;
	}
	else if (port == 1 && value == 1)
	{
		argument = SET_OUT_1;
	}
	else if (port == 2 && value == 0)
	{
		argument = CLEAR_OUT_2;
	}
	else if (port == 2 && value == 1)
	{
		argument = SET_OUT_2;
	}
	else
	{
		return error(WARNING,"out_set(): Invalid data %d,%d.",
			port,value);
	}

	/* OK, and send the command */

	if (!send_rt_command(TTL_CONTROL,argument))
	{
		return error(WARNING,"out_set(): Failed to send command.");
	}

	/* Get the result */

	if (read_rt_data(&argument,sizeof(argument)) != sizeof(argument)) 
	{
		return error(MESSAGE,"out_set(): Failed to read result.");
	}

	if (argument != value)
	{
		return error(WARNING,"out_set(): Port %d reports seting of %d not %d.",
			port,argument,value);
	}

	return NOERROR;

} /* out_set() */

/************************************************************************/
/* send_command() 							*/
/*									*/
/* Sends the command and argument to the RT process.			*/
/* Chara UI callable function.						*/
/************************************************************************/

int	send_command(int argc, char **argv)
{
	unsigned int comm;
	unsigned char data;
	int	value;
	char	str[4];

	if (bypass_clock_driver) return NOERROR;

	/* OK let's sort out the arguments */

	werase(system_window);
	wrefresh(system_window);
	if (argc > 1)
	{
		sscanf(argv[1],"%ud",&value);
	}
	else
	{
		strcpy(str,"000");
		if (quick_edit("Command",str,str,NULL,INTEGER) == KEY_ESC) 
		{
			return NOERROR;
		}
		sscanf(str,"%ud",&value);
	}
	comm = value;

	if (argc > 2)
	{
		sscanf(argv[2],"%ud",&value);
	}
	else
	{
		strcpy(str,"000");
		if (quick_edit("Argument",str,str,NULL,INTEGER) == KEY_ESC) 
		{
			return NOERROR;
		}
		sscanf(str,"%ud",&value);
	}
	data = value;

        if (!send_rt_command(comm,data))
        {
                return ERROR;
        }
	return NOERROR;

} /* send_command() */

/************************************************************************/
/* error_status() 							*/
/*									*/
/* Looks for an error in the ERR_FIFO, if there is one report it to	*/
/* the console in an ERROR window.					*/
/************************************************************************/

int error_status(void)
{
	char	*p,s[4096];

	if (bypass_clock_driver) return NOERROR;

	if (get_rt_error_message(s) == 0) return NOERROR;

	/* Remove any new lines from this */

	for(p=s; *p != 0; p++) if (*p == '\n' || *p == '\r') *p = 0;

	error(MESSAGE,"Message from RT process: %s",s);

	return NOERROR;
}

/************************************************************************/
/* get_lpt_register()							*/
/*									*/
/* Uses one of the standard commands to get a register from the		*/
/* printer port.							*/
/************************************************************************/

unsigned char get_lpt_register(int reg)
{
	unsigned char c;

	if (bypass_clock_driver) return 0;

	/* Check the argument makes sense */

	if (reg != CONTROL_REG && reg != STATUS_REG)
	{
		error(WARNING,"get_lpt_register(): Bad register.");
		return 0;
	}

	/* Try sending the command */

        if (!send_rt_command(REPORT_LPT_REGISTER,reg))
        {
                return 0;
        }

        /* Now get the register value */

        if (read_rt_data(&c, sizeof(c)) != sizeof(c))
        {
                return error(WARNING,"Error reading register. No data\n");
        }

	return c;

} /* get_lpt_register() */

/************************************************************************/
/* get_rt_time()							*/
/*									*/
/* Uses one of the standard commands to get a structure from the	*/
/* RT process containing the time, lost ticks and lost seconds.		*/
/* Returns 0 on succes and -1 on failure.  				*/
/************************************************************************/

int get_rt_time(rt_time_struct *the_time)
{
	struct timeval time_now;
	struct timezone zone;
	time_t current_time;
	static long long int    last_run_us = 0;
	long long int   this_run_us = 0;

	if (bypass_clock_driver)
	{
		/* What is the time now in uS? */

		gettimeofday(&time_now, &zone);
		time(&current_time);
		this_run_us = ((long long int)1000000)*
				((long long int)current_time%86400) +
				((long long int)time_now.tv_usec);

		/* Turn that into TICKS */

		the_time->time_now = (CHARA_TIME)(this_run_us /
												(1000000/TICKS_PER_SECOND));
		the_time->lost_ticks = 0;
		the_time->lost_seconds = 0;

		/* We shouldn't do this too often */

		if (this_run_us - last_run_us < bypass_clock_delay)
		{
			usleep(bypass_clock_delay);
		}

		last_run_us = this_run_us;
	}
	else
	{
		/* Try sending the command */

		if (!send_rt_command(REPORT_TIME,0))
		{
			return -1;
		}

		/* Now get the register value */

		if (read_rt_data(the_time, sizeof(*the_time)) !=
			sizeof(*the_time))
		{
			error(WARNING,"Error reading time. Not enough data\n");
			return -1;
		}
	}

	return 0;

} /* get_rt_time() */

/************************************************************************/
/* get_time_next_second()						*/
/*									*/
/* Uses one of the standard commands to get the	time of the next 	*/
/* second. Returns this time or -1 on error.				*/
/* If wait is TRUE will wait for it to change once first before 	*/
/* it returns to make sure it will be valid for a while. Times out      */
/* after 3 seconds.							*/
/************************************************************************/

CHARA_TIME get_time_next_second(int wait)
{
	CHARA_TIME	time_next_second;
	CHARA_TIME	last_time_next_second;
	time_t		stop_time;
	time_t		now;

	if (bypass_clock_driver) return 0;

	/* Try sending the command */

        if (!send_rt_command(GET_TIME_NEXT_SECOND,0)) return -1;

        if (read_rt_data(&last_time_next_second, sizeof(CHARA_TIME)) != 
		sizeof(CHARA_TIME)) return -1;

	if (!wait) return last_time_next_second;

	/* Now try for a few seconds to get the next one */

	time(&stop_time);
	stop_time += 3;
	do
	{
		if (!send_rt_command(GET_TIME_NEXT_SECOND,0)) return -1;

		if (read_rt_data(&time_next_second, sizeof(CHARA_TIME)) != 
			sizeof(CHARA_TIME)) return -1;

		time(&now);

		background();

	} while(now < stop_time && last_time_next_second==time_next_second);

	if (now >= stop_time) return -1;

	return time_next_second;

} /* get_time_next_second() */

/************************************************************************/
/* set_rt_time()							*/
/*									*/
/* Uses one of the standard commands to send a value of type CHARA_TIME */
/* to the RT process. This value wil become the value of the RT 	*/
/* modules internal varriable time_next_second. When the next second	*/
/* tick occurs, or is detected to have been missed, the time is set to 	*/
/* this value. Warning.... race conditions can occurr, this should be 	*/
/* well before the next second tick is expected.			*/
/* Returns TRUE is all went OK, FALSE otherwise.			*/
/************************************************************************/

int set_rt_time(CHARA_TIME time_next_second)
{
	if (bypass_clock_driver) return TRUE;

        /* 
	 * Send the time value to DATA IN FIFO 
	 * Note that this must be done first so the data
	 * is there when the command to use it is sent.
	 */

	if (write_rt_data(&time_next_second,sizeof(time_next_second)) != 
		sizeof(time_next_second))
	{
		return FALSE;
	}

	/* Now send the command */

        if (!send_rt_command(SET_TIME,0))
        {
                return FALSE;
        }

	return TRUE;

} /* set_rt_time() */

/************************************************************************/
/* rt_to_ap_time()							*/
/*									*/
/* Converts RT process time into an application time structure.		*/
/************************************************************************/

struct AP_TIME_STRUCT rt_to_ap_time(CHARA_TIME rt_time)
{
	ap_time_struct res;

	if (rt_time < 0)
	{
		rt_time *= (res.sign = -1);
	}
	else
	{
		res.sign = 1;
	}
		
	res.tick = rt_time % TICKS_PER_SECOND;
	rt_time -= res.tick;
	rt_time /= TICKS_PER_SECOND;

	res.sec = rt_time % 60;
	rt_time -= res.sec;
	rt_time /= 60;

	res.min = rt_time % 60;
	rt_time -= res.min;

	res.hrs = rt_time / 60;

	return res;

} /* rt_to_ap_time() */

/************************************************************************/
/* ap_to_rt_time()							*/
/*									*/
/* Converts an application time structure into a RT process time .	*/
/************************************************************************/

CHARA_TIME ap_to_rt_time(struct AP_TIME_STRUCT t)
{
	CHARA_TIME	res;

	res = t.tick + TICKS_PER_SECOND * (t.sec + 60 * (t.min + 60 * t.hrs));
	res *= t.sign;

	return res;

} /* ap_to_rt_time() */

/************************************************************************/
/* str_to_ap_time()							*/
/*									*/
/* Converts a string of the form [+-]HR:MN:SC into an application time	*/
/* structure.								*/
/************************************************************************/

struct AP_TIME_STRUCT str_to_ap_time(char *s)
{
	struct AP_TIME_STRUCT t;

	/* What is the sign? */

	while(*s == ' ' || *s == '\t') s++;
	if (*s == '-')
	{
		/* It's negative */

		t.sign = -1;
		*s = ' ';
	}
	else
	{
		/* It's positive */

		t.sign = 1;
	}

	switch(sscanf(s,"%d:%d:%d",&t.hrs,&t.min,&t.sec))
	{

		case 0: t.hrs = 0;

		case 1: t.min = 0;

		case 2: t.sec = 0;

		default: t.tick = 0;
			 break;
	}

	return t;
		
} /* str_to_ap_time() */

/************************************************************************/
/* ap_to_str_time()							*/
/*									*/
/* Converts a string of the form [+-]HR:MN:SC into an application time	*/
/* structure.								*/
/************************************************************************/

char *ap_to_str_time(char *s, struct AP_TIME_STRUCT t)
{
	/* What is the sign? */

	if (t.sign == -1)
	{
		/* It's negative */

		strcpy(s,"-");
	}
	else
	{
		/* It's positive */

		strcpy(s," ");
	}

	/* Print it */

	sprintf(s+1,"%02d:%02d:%02d",t.hrs,t.min,t.sec);

	return s;
		
} /* ap_to_str_time() */

/************************************************************************/
/* set_time()	 							*/
/*									*/
/* Chara UI callable function.						*/
/* An interface to the set_rt_time command.				*/
/************************************************************************/

int	set_time(int argc, char **argv)
{
	CHARA_TIME	time_next_second;
	ap_time_struct	new_time;
	char	str[80];
	char	*str_time;
        rt_time_struct now;

	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

	/* OK let's sort out the arguments */

	werase(system_window);
	wrefresh(system_window);
	if (argc > 1)
	{
		str_time = argv[1];
	}
	else
	{
		if (get_rt_time(&now) != 0) return ERROR;
		new_time = rt_to_ap_time(now.time_now);
		ap_to_str_time(str,new_time);
		if (quick_edit("Time at next second",str,str,NULL,STRING) == 
			KEY_ESC) return NOERROR;
		str_time = str;
	}

	new_time = str_to_ap_time(str_time);
	time_next_second = ap_to_rt_time(new_time);

	if (!set_rt_time(time_next_second))
	{
		return error(WARNING,"Failed to set time of next second.");
	}

	/* Wait for this to happen */

	sleep(1);

	/* Make sure linux clock is OK */

        return NOERROR;

} /* set_time() */

/************************************************************************/
/* set_control_reg() 							*/
/*									*/
/* Chara UI callable function.						*/
/* Interface to the WRITE_LPT_CONTROL function of the system. Let's you	*/
/* modify the LPT control register. THIS IS A DANGEROUS THING TO DO!    */
/************************************************************************/

int	set_control_reg(int argc, char **argv)
{
	char	str[80];
	char	*str_reg;
        int reg;

	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

	/* OK let's sort out the arguments */

	werase(system_window);
	wrefresh(system_window);
	if (argc > 1)
	{
		str_reg = argv[1];
	}
	else
	{
		sprintf(str,"%x",get_lpt_register(CONTROL_REG));
		if (quick_edit("Control register",str,str,NULL,STRING) == 
			KEY_ESC) return NOERROR;
		str_reg = str;
	}

	sscanf(str_reg,"%x",&reg);

        if (!send_rt_command(WRITE_LPT_CONTROL,(unsigned char)reg))
        {
                return error(WARNING,"Couldn't send the command.");
        }

	return NOERROR;

} /* set_control_reg() */

/************************************************************************/
/* sync_times()								*/
/*									*/
/* Attempts to set the time using the RT time based on RT clock.	*/
/* Uses the setgmt program. Note that this can mess up the local clock! */
/************************************************************************/

int sync_times(int argc, char **argv)
{
	CHARA_TIME time_next_second;
	struct AP_TIME_STRUCT ap_time_next_second;
	char	command[81];
	rt_time_struct the_time;

	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

	message(system_window,"Attempting to sync clocks... please wait.");

	/* Find out the time of the next second */

	if ((time_next_second = get_time_next_second(TRUE)) == -1)
	{
		werase(system_window);
		wrefresh(system_window);
		return error(ERROR,"Failed to get time of next second.");
	}

	/* Setup the command we will need to set the Linux Clock */

	ap_time_next_second = rt_to_ap_time(time_next_second);
	sprintf(command,"setgmt %d %d %d",
		ap_time_next_second.sign * ap_time_next_second.hrs,
		ap_time_next_second.min, ap_time_next_second.sec);

	/* Wait until the time comes ... with OFFSET */

	do
	{
		if (get_rt_time(&the_time) == -1)
		{
			werase(system_window);
			wrefresh(system_window);
			return error(ERROR,"Failed to get RT time.");
		}
	} while(the_time.time_now < (time_next_second -  SYNC_OFFSET));

	system(command);
			
	/* Now make sure the background still works */

	reset_status_window_refresh(0, NULL);

	/* That is all */

	werase(system_window);
	wrefresh(system_window);
	return NOERROR;

} /* sync_times() */

/************************************************************************/
/* show_time()                                                          */
/*									*/
/* Display the time in the system_window. Mostly so a remote user can   */
/* see what the time is.                                                */
/************************************************************************/

int show_time(int argc, char **argv)
{
        rt_time_struct now;
        ap_time_struct t;
	time_t current_time;
	struct timeval time_now;
	struct tm *loc_now;
	struct timezone zone;
	int tdiff;
	int tlast=0,tnow;
	int last_sec = 0;
	int sec_length = 0;
	float n = -5.0,mean_sec = 0.0;

	do
	{
		zone.tz_minuteswest = 0;
	        zone.tz_dsttime = 0;
		if (get_rt_time(&now) != 0) 
			return error(ERROR,"Failed to get the time.");
		gettimeofday(&time_now, &zone);
		time(&current_time);
		loc_now = gmtime(&current_time);
		t = rt_to_ap_time(now.time_now);
		tdiff = 1000*loc_now->tm_sec + time_now.tv_usec/1000 -
			1000*t.sec - t.tick*(1000/TICKS_PER_SECOND);
		tnow = 1000000*loc_now->tm_sec + time_now.tv_usec;

		werase(system_window);
		message(system_window,
			"%02d:%02d:%02d CHARA-LOCAL = %d mS SEC = %d (%.1f) uS",
			t.hrs,t.min,t.sec, tdiff, sec_length, mean_sec/n);
		if (t.sec != last_sec)
		{
			sec_length = tnow - tlast;
			tlast = tnow;
			if (sec_length > 0.0)
			{
				if (n >= 0.0) mean_sec += (float)sec_length;
				n += 1.0;
			}
			last_sec = t.sec;
		}

		if (kbhit() && get_command() == KEY_ESC) break;

	} while(active_socket == -1);

	werase(system_window);
	wrefresh(system_window);

	return NOERROR;

} /* show_time() */

/************************************************************************/
/* call_sync_handler()                                                  */
/*                                                                      */
/* Sends command to real time side to forcibly call the sync handler.   */
/* Mostly used when the prio_sched get's confused.                      */
/************************************************************************/

int call_sync_handler(int argc, char **argv)
{
	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

        if (!send_rt_command(SYNC_HANDLER,0))
        {
                return error(WARNING,"Couldn't send the command.");
        }

        return NOERROR;

} /* call_sync_handler() */

/************************************************************************/
/* set_lpt()								*/
/*									*/
/* Alows user to set the printer port output to a specific value.	*/
/************************************************************************/

int set_lpt(int argc, char **argv)
{
	int	i;
	unsigned char value;
	char	str[81];

	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

	/* Check command line */

	if (argc > 1)
	{
		sscanf(argv[1],"%d",&i);
	}
	else
	{
                clean_command_line();
		sprintf(str,"000");
		if (quick_edit("Printer port value",str,str,NULL,INTEGER) == 
			KEY_ESC) return NOERROR;
		sscanf(str,"%d",&i);
	}

	/* Is the value reasonable */


	if (i < 0 || i > 255)
		return error(ERROR,"Must have a value between 0 and 255.");

	/* Send the command */
	
	
	value = i;
        if (!send_rt_command(SET_LPT,value))
        {
                return error(WARNING,"Couldn't send the command.");
        }

        return call_get_lpt(0,NULL);

} /* set_lpt() */

/************************************************************************/
/* get_lpt()                                                            */
/*                                                                      */
/* Returns the current setting of printer port, or -1 on command        */
/* error, -2 on data error.                                             */
/************************************************************************/

int get_lpt(void)
{
        unsigned char value;

	if (bypass_clock_driver)return 0;

        if (!send_rt_command(GET_LPT,value)) return -1;
        if (read_rt_data(&value, sizeof(value)) != sizeof(value)) return -2;
        return (int)value;

} /* get_lpt() */

/************************************************************************/
/* call_get_lpt()                                                       */
/*                                                                      */
/* Alows user to read the current printer port output value.            */
/************************************************************************/

int call_get_lpt(int argc, char **argv)
{
        int  value;

	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}


        if ((value = get_lpt()) < 0)
                return error(ERROR,"Failed to get parallel port setting.");

        message(system_window,"Printer port = 0x%x (%d)",value,value);

        return NOERROR;

} /* call_get_lpt() */

/************************************************************************/
/* get_lpt_data()                                                       */
/*                                                                      */
/* Returns the current setting of printer data port, or -1 on command   */
/* error, -2 on data error.                                             */
/************************************************************************/

int get_lpt_data(void)
{
        unsigned char value;

	if (bypass_clock_driver)return 0;

        if (!send_rt_command(GET_LPT_DATA,value)) return -1;
        if (read_rt_data(&value, sizeof(value)) != sizeof(value)) return -2;
        return (int)value;

} /* get_lpt_data() */

/************************************************************************/
/* call_get_lpt_data()                                                  */
/*                                                                      */
/* Alows user to read the current printer data port output value.       */
/************************************************************************/

int call_get_lpt_data(int argc, char **argv)
{
        int  value;

	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

        if ((value = get_lpt_data()) < 0)
                return error(ERROR,"Failed to get parallel port setting.");

        message(system_window,"Printer port = 0x%x (%d)",value,value);

        return NOERROR;

} /* call_get_lpt_data() */

/************************************************************************/
/* set_bit_lpt()							*/
/*									*/
/* Alows user to set a bit in the printer port output.			*/
/************************************************************************/

int set_bit_lpt(int argc, char **argv)
{
	int	i;
	unsigned char bit;
	char	str[81];

	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

	/* Check command line */

	if (argc > 1)
	{
		sscanf(argv[1],"%d",&i);
	}
	else
	{
                clean_command_line();
		sprintf(str,"0");
		if (quick_edit("Printer port bit",str,str,NULL,INTEGER) == 
			KEY_ESC) return NOERROR;
		sscanf(str,"%d",&i);
	}

	/* Is the value reasonable */


	if (i < 0 || i > 7)
		return error(ERROR,"Must have a value between 0 and 7.");

	/* Send the command */
	
	
	bit = i;
        if (!send_rt_command(SET_BIT_LPT,bit))
        {
                return error(WARNING,"Couldn't send the command.");
        }

        return call_get_lpt(0,NULL);

} /* set_bit_lpt() */

/************************************************************************/
/* unset_bit_lpt()							*/
/*									*/
/* Alows user to unset a bit in the printer port output.		*/
/************************************************************************/

int unset_bit_lpt(int argc, char **argv)
{
	int	i;
	unsigned char bit;
	char	str[81];

	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

	/* Check command line */

	if (argc > 1)
	{
		sscanf(argv[1],"%d",&i);
	}
	else
	{
                clean_command_line();
		sprintf(str,"0");
		if (quick_edit("Printer port bit",str,str,NULL,INTEGER) == 
			KEY_ESC) return NOERROR;
		sscanf(str,"%d",&i);
	}

	/* Is the value reasonable */


	if (i < 0 || i > 7)
		return error(ERROR,"Must have a value between 0 and 7.");

	/* Send the command */
	
	
	bit = i;
        if (!send_rt_command(UNSET_BIT_LPT,bit))
        {
                return error(WARNING,"Couldn't send the command.");
        }

        return call_get_lpt(0,NULL);

} /* unset_bit_lpt() */

/************************************************************************/
/* initialize_clock()							*/
/*									*/
/* ALlows user to force a clock reinitialization.			*/
/************************************************************************/

int initialize_clock(int argc, char **argv)
{
	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

        if (!send_rt_command(INITIALIZE_CLOCK,0))
        {
                return error(WARNING,"Couldn't send the command.");
        }

	message(system_window,"Clock has been initialized.");

        return NOERROR;

} /* initialize_clock() */

/************************************************************************/
/* list_commands()                                                      */
/*                                                                      */
/* Asks RT side for a list of commands.                                 */
/************************************************************************/

int list_commands(int argc, char **argv)
{
	if (bypass_clock_driver)
	{
		message(system_window,
		    "We are using the Linux clock - command ignored.");
		return NOERROR;
	}

        if (!send_rt_command(LIST_COMMANDS,0))
        {
                return error(WARNING,"Couldn't send the command.");
        }

        return NOERROR;

} /* list_commands() */

/************************************************************************/
/* sidereal_time()                                                      */
/*                                                                      */
/* Returns current sidereal time 0-2PI.                                 */
/* Uses date on computer but reall time from CHARA clock.               */
/* If returned value is negative, something went wrong.                 */
/************************************************************************/
                                                                                
double sidereal_time(void)
{
        double  mjd;
 
        mjd = modified_julian_date();
 
        if (mjd < 0) return -1;
 
        return (slaGmst(mjd) + CHARA_LONGITUDE);
 
} /* sidereal_time() */
 
/************************************************************************/
/* modified_julian_date()                                               */
/*                                                                      */
/* Returns the modified julian date, or negative number on error.       */
/* Result is in days plus fractions of days.                            */
/* -1.0 Slalib failed.                                                  */
/* -2.0 Clock read failed.                                              */
/************************************************************************/
 
double modified_julian_date(void)
{
        struct  tm      *gmt_now;
        time_t current_time;
        int     iy,im,id,j;
        double  mjd;
        rt_time_struct now;
 
        time(&current_time);
        gmt_now = gmtime(&current_time);
 
        iy = gmt_now->tm_year + 1900;
        if (iy < 1950) iy+=100;
        im = gmt_now->tm_mon+1;
        id = gmt_now->tm_mday;
 
        slaCaldj(iy,im,id,&mjd, &j);
 
        if (j != 0) return -1.0;
        if (get_rt_time(&now) != 0) return -2.0;
 
        mjd += ((double)now.time_now/(TICKS_PER_SECOND*60.0*60.0*24.0));
 
        return mjd;
 
} /* modified_julian_date() */

/************************************************************************/
/* clock_running()							*/
/*									*/
/* CHecks to see if local clock is running. Takes 1 second to do this.	*/
/* Returns TRUE if it is and FALSE if it isn't.				*/
/************************************************************************/

bool clock_running(void)
{
        rt_time_struct start,stop;

	get_rt_time(&start);
	usleep(10000);
	get_rt_time(&stop);

	return (start.time_now != stop.time_now);

} /* clock_running() */

/************************************************************************/
/* alloc_chara_time_now()						*/
/*									*/
/* Setup shared memory for the current time.				*/
/* returns TRUE if all goes well, FALSE otherwise.			*/
/************************************************************************/

bool alloc_chara_time_now(void)
{
#ifndef KERNEL_OTHER
	if (chara_time_now != NULL) return TRUE;

        chara_time_now = (CHARA_TIME *) mbuff_alloc("shared_time_now",
                                2 * sizeof(*chara_time_now));
	if (chara_time_now != NULL)
	{
		chara_time_next_second = chara_time_now + 1;
		return TRUE;
	}
	
	return FALSE;
#else
	return TRUE;
#endif

} /* alloc_chara_time_now() */

/************************************************************************/
/* free_chara_time_now()						*/
/*									*/
/* Setup shared memory for the current time.				*/
/************************************************************************/

void free_chara_time_now(void)
{
#ifndef KERNEL_OTHER
	if (chara_time_now == NULL) return;

	mbuff_free("shared_time_now", (void *)chara_time_now);
#endif

} /* free_chara_time_now() */

/************************************************************************/
/* check_clock_offset()							*/
/*									*/
/* Calculates the difference between the real time clock and the local 	*/
/* clock. If this is larger than a given value set by 			*/
/* set_max_clock_offset() the function will call a callback routine set */
/* by set_clock_offset_callback().					*/
/************************************************************************/

int check_clock_offset(void)
{
        rt_time_struct now;
        ap_time_struct t;
	time_t current_time;
	struct timeval time_now;
	struct tm *loc_now;
	struct timezone zone;
	int tdiff;

	zone.tz_minuteswest = 0;
	zone.tz_dsttime = 0;
	if (get_rt_time(&now) != 0) 
		return error(ERROR,"Failed to get the time.");
	gettimeofday(&time_now, &zone);
	time(&current_time);
	loc_now = gmtime(&current_time);
	t = rt_to_ap_time(now.time_now);
	tdiff = 1000*loc_now->tm_sec + time_now.tv_usec/1000 -
		1000*t.sec - t.tick*(1000/TICKS_PER_SECOND);
	
	if ((tdiff > max_clock_offset || -1*tdiff > max_clock_offset) && 
		clock_offset_callback != NULL)
		(*clock_offset_callback)();

	return NOERROR;

} /* check_clock_offset() */

/************************************************************************/
/* set_max_clock_offset()						*/
/*									*/
/* Set the maximum clock offset allowed by the system.			*/
/************************************************************************/

void set_max_clock_offset(int max_offset)
{
	max_clock_offset = max_offset;

} /* set_max_clock_offset */

/************************************************************************/
/* set_clock_offset_callback()						*/
/*									*/
/* Set the clock offset callback function, which will be called if the 	*/
/* system clock and the realtime clock differ by more than 		*/
/* ticks max_clock_offset.						*/
/* Set to NULL to stop using this at all.				*/
/************************************************************************/

void set_clock_offset_callback(void (*offset_callback)(void))
{
	clock_offset_callback = offset_callback;

} /* set_clock_offset_callback() */
