/************************************************************************/
/* dalsa.h								*/
/*                                                                      */
/* Header file for the Dalsa Camera Driver.				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar based on code by Laszlo Sturmann        */
/* Date   : January 2000						*/
/************************************************************************/

#ifndef __DALSA__
#define __DALSA__

/************************************************************************/
/* The stuff for both MODULES and User Interfaces			*/
/************************************************************************/

/*
 * The commands
 */

#define GET_PARAMS		 (FIRST_DALSA_COMMAND+0)
#define SET_PARAMS		 (FIRST_DALSA_COMMAND+1)
#define ARM			 (FIRST_DALSA_COMMAND+2)
#define GET_FRAME		 (FIRST_DALSA_COMMAND+3)
#define ZERO_BIAS		 (FIRST_DALSA_COMMAND+4)
#define SET_BIAS		 (FIRST_DALSA_COMMAND+5)
#define GET_BIAS		 (FIRST_DALSA_COMMAND+6)
#define SET_FULL_BIAS		 (FIRST_DALSA_COMMAND+7)
#define SET_BB			 (FIRST_DALSA_COMMAND+8)
#define GET_BB			 (FIRST_DALSA_COMMAND+9)
#define SET_BB_BIAS		 (FIRST_DALSA_COMMAND+10)
#define FRAME_RATE		 (FIRST_DALSA_COMMAND+11)
#define DALSA_MAX_COMMAND	 (FIRST_DALSA_COMMAND+12)

/* 
 * Some macros
 */

#define MIN_EXPOSURE		1
#define MAX_EXPOSURE		2000

/* 
 * Structure for tell Dalsa what settings to use.
 */

struct dalsa_struct {
     	int ccd_size;
     	int exposure;
	int status;
	int	ulx,uly,lrx,lry; /* Bounding box */
	unsigned char data_ready;
	unsigned char *dma_buf;
	unsigned char bias_ready;
	unsigned char full_bias;
	unsigned char bb_bias;
	unsigned char *bias_buf;
	int data_size;
	int bb_size;
     	unsigned char binning;
	int seconds;
	int num_frames;
};

typedef struct dalsa_struct DALSA_PARAM;

/************************************************************************/
/* The stuff for only MODULES						*/
/************************************************************************/

#ifdef MODULE

/* Macros */

#define Kbyte 			1024
#define DALSA_DMA_BUFF_SIZE   	(64*Kbyte) /* Enough for 256x256 */
#define DALSA_DMA     		5
#define DALSA_DMA_MODE 		0x15 /* DEMAND, AUTOINC, AUTOINI, WRITE, CH5 */
#define DALSA_IRQ_NUM    		5
#define DALSA_IRQ_CLOCK    		10
#define DALSA_BASE        	0x308
#define DALSA_COM_REG     	DALSA_BASE+4
#define DALSA_STAT_REG    	DALSA_BASE+4

#define DALSA_CMD_REG 		DALSA_BASE+3
#define DALSA_COUNTER_0 	DALSA_BASE
#define DALSA_COUNTER_1 	DALSA_BASE+1
#define DALSA_COUNTER_2 	DALSA_BASE+2
#define DALSA_RESET 		DALSA_BASE+6
#define DALSA_ENABLE_DMA	DALSA_BASE+7
#define DALSA_BINNING 		128

/* Prototypes */

int init_module(void);
void dalsa_prepare_dma(void);
void dalsa_irq_handler(void);
int dalsa_set_handler(void (*handler)(void));
void dalsa_unset_handler(void);
int dalsa_arm(void);
int dalsa_get_params(unsigned char arg);
int dalsa_set_params(unsigned int ccd_size, 
	unsigned int exposure, 
	unsigned char binning);
int dalsa_get_frame(unsigned char arg);
int dalsa_call_set_params(unsigned char arg);
int dalsa_call_arm(unsigned char arg);
void dalsa_current_params(DALSA_PARAM *current_params);
void dalsa_current_bb(unsigned char *box);
int dalsa_zero_bias(unsigned char arg);
int dalsa_set_bias(unsigned char arg);
int dalsa_get_bias(unsigned char arg);
int dalsa_set_full_bias(unsigned char arg);
int dalsa_set_bb(unsigned char arg);
int dalsa_get_bb(unsigned char arg);
int dalsa_set_bb_bias(unsigned char arg);
int dalsa_measure_frame_rate(unsigned char arg);
void dalsa_measure_frame_rate_handler(void);
void cleanup_module(void);

#else

/************************************************************************/
/* This stuff only for user interfaces					*/
/************************************************************************/

#include <math.h>

/* Globals */

extern char    *default_display_machine;
extern char *bias_types[3];

/* dalsa_comms.c */

int set_params(unsigned int ccd_size, 
	       unsigned int exposure, 
	       unsigned char binning);
int get_params(DALSA_PARAM *dalsa_param);
int arm(void);
unsigned char *get_frame(void);
int set_bias(unsigned char *new_bias, DALSA_PARAM dalsa_param);
unsigned char *get_bias(void);
int set_bb(int ulx, int uly, int lrx, int lry);
unsigned char *get_bb(void);

/* dalsa_control.c */

int set_ccd(int argc, char **argv);
bool wait_for_frame(DALSA_PARAM *dalsa_param);
int call_set_params(int argc, char **argv);
int single_frame(int argc, char **argv);
int display_frame(char *display, DALSA_PARAM dalsa_param, unsigned char *buffer);
int clear_window(int argc, char **argv);
void dalsa_close(void);
int movie(int argc, char **argv);
int background_movie(int argc, char **argv);
int movie_background_job(void);
int zero_bias(int argc, char **argv);
int calculate_bias(int argc, char **argv);
int display_bias(int argc, char **argv);
int set_full_bias(int argc, char **argv);
int set_bb_bias(int argc, char **argv);
int call_set_bb(int argc, char **argv);
int single_bb(int argc, char **argv);
int display_bb(char *display, DALSA_PARAM dalsa_param, unsigned char *buffer);
int bb_movie(int argc, char **argv);
int bb_background_movie(int argc, char **argv);
int bb_movie_background_job(void);
int show_dalsa_structure(int argc, char **argv);
int measure_frame_rate(int argc, char **argv);
int save_frame(int argc, char **argv);
int save_bb(int argc, char **argv);

/* dalsa_spect.c */

float *bb_power_spec(bool no_fft);
float *display_power_spectrum(Window win, int width, int height, bool no_fft);
int call_display_power_spec(int argc, char **argv);
int power_spec_bias(int argc, char **argv);
int toggle_ps_bias(int argc, char **argv);
int set_num_pix_zero(int argc, char **argv);
int set_spec_row(int argc, char **argv);
int data_flat(int argc, char **argv);
int toggle_data_flat(int argc, char **argv);
int calc_data_dark(int argc, char **argv);
int toggle_data_dark(int argc, char **argv);
int toggle_auto_range(int argc, char **argv);
int set_spect_cutoff(int argc, char **argv);

#endif
#endif
