/************************************************************************/
/* dalsa_comms.c							*/
/*                                                                      */
/* Routines to read/write info to the realtime process.  		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Sept 1998							*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../clock/clock.h"
#include "../../nsimpleX/nsimpleX.h"
#include "dalsa.h"

/************************************************************************/
/* set_params()								*/
/* 									*/
/* Attempts to set the parameters for Dalsa CCD 			*/
/* Returns -1 if it fails to send the command, -2 if it fails to	*/
/* send the data.							*/
/* Returns -3 if it is not ready to arm.				*/
/* Returns 0 if it worked.						*/
/************************************************************************/

int set_params(unsigned int ccd_size, /* 256 OR 128 */
              unsigned int exposure, /* In millisceconds */
              unsigned char binning) /* TRUE or FALSE */
{
	DALSA_PARAM	dalsa_param;

	/* First, get the parameters */

	if (get_params(&dalsa_param) != 0) return -3;

	/* Are we ready? */

	if (!dalsa_param.data_ready)
	{
		error(ERROR,"Data not ready so we can not set params.");
		return -3;
	}

	/* Send the data itself */

	if (write_rt_data(&ccd_size,sizeof(ccd_size)) != sizeof(ccd_size)) 
		return -2;
	if (write_rt_data(&exposure,sizeof(exposure)) != sizeof(exposure)) 
		return -2;
	if (write_rt_data(&binning,sizeof(binning)) != sizeof(binning)) 
		return -2;

	/* Send the command */

	if (!send_rt_command(SET_PARAMS, 0)) return -1;

	return 0;

} /* set_params() */

/************************************************************************/
/* arm()								*/
/* 									*/
/* Attempts to arm the Dalsa CCD for a single frame.			*/
/* Returns TRUE if it fails to send the command.			*/
/* Returns FALSE if it worked.						*/
/************************************************************************/

int arm(void)
{
	/* Send the command */

	if (!send_rt_command(ARM, 0)) return FALSE;

	return TRUE;

} /* arm() */

/************************************************************************/
/* get_params()								*/
/* 									*/
/* Tries to get the paramers. Returns -1 if it fails to send the 	*/
/* command and -2 if it fails to get the data. Returns 0 if it works.	*/
/************************************************************************/

int get_params(DALSA_PARAM *dalsa_param)
{
	DALSA_PARAM new_dalsa_param;

	/* Send the command */

	if (!send_rt_command(GET_PARAMS, 0)) return -1;

	/* Get the data itself */

	if (read_rt_data(&new_dalsa_param,sizeof(new_dalsa_param)) != 
			sizeof(new_dalsa_param)) 
	{
		return -2;
	}

	dalsa_param->ccd_size = new_dalsa_param.ccd_size;
	dalsa_param->exposure = new_dalsa_param.exposure;
	dalsa_param->status = new_dalsa_param.status;
	dalsa_param->ulx = new_dalsa_param.ulx;
	dalsa_param->uly = new_dalsa_param.uly;
	dalsa_param->lrx = new_dalsa_param.lrx;
	dalsa_param->lry = new_dalsa_param.lry;
	dalsa_param->data_ready = new_dalsa_param.data_ready;
	dalsa_param->dma_buf = new_dalsa_param.dma_buf;
	dalsa_param->bias_ready = new_dalsa_param.bias_ready;
	dalsa_param->full_bias = new_dalsa_param.full_bias;
	dalsa_param->bb_bias = new_dalsa_param.bb_bias;
	dalsa_param->bias_buf = new_dalsa_param.bias_buf;
	dalsa_param->data_size = new_dalsa_param.data_size;
	dalsa_param->bb_size = new_dalsa_param.bb_size;
	dalsa_param->binning = new_dalsa_param.binning;
	dalsa_param->seconds = new_dalsa_param.seconds;
	dalsa_param->num_frames = new_dalsa_param.num_frames;

	return 0;

} /* get_params() */

/************************************************************************/
/* get_frame()								*/
/*									*/
/* Attempts to get a frame of data from the realtime side.		*/
/* Will always first get the parameters to make sure it knows how many	*/
/* pixels to get and if the system is ready for this or not.		*/
/* Returns a pointer to memory with the data in it which is allocated	*/
/* inside this function. The caller function should free this memory.	*/
/* Returns NULL if things do not go well.				*/
/************************************************************************/

unsigned char *get_frame(void)
{
	unsigned char *buf;
	DALSA_PARAM	dalsa_param;

	/* First, get the parameters */

	if (get_params(&dalsa_param) != 0) return NULL;

	/* Are we ready? */

	if (!dalsa_param.data_ready)
	{
		error(ERROR,"Data not ready for frame.");
		return NULL;
	}

	/* OK, allocate the memory */

	if ((buf = malloc(dalsa_param.data_size)) == NULL) return NULL;

	/* Send the command */

	if (!send_rt_command(GET_FRAME, 0))
	{
		free(buf);
		return NULL;
	}

	/* Get the data */

	if (read_rt_data(buf,dalsa_param.data_size) != dalsa_param.data_size) 
	{
		free(buf);
		return NULL;
	}

	/* That is all */

	return buf;

} /* get_frame() */

/************************************************************************/
/* set_bias()								*/
/*									*/
/* Sends new bias data to the realtime side. Be carefull to send the	*/
/* right length of data down there!					*/
/* Returns -1 if it fails to send the command, -2 if it fails to	*/
/* send the data.							*/
/* Returns -3 if it cannot get current parameters.			*/
/* Returns -4 if data size is wrong.					*/
/* Returns 0 if it worked.						*/
/************************************************************************/

int set_bias(unsigned char *new_bias, DALSA_PARAM dalsa_param)
{
	DALSA_PARAM	cur_dalsa_param;

	/* First, get the parameters */

	if (get_params(&cur_dalsa_param) != 0) return -3;

	if (cur_dalsa_param.data_size != dalsa_param.data_size)
	{
		return -4;
	}

	/* Put the data into the FIFO */

	if (write_rt_data(new_bias,dalsa_param.data_size) != 
		dalsa_param.data_size) return -2;

	/* Send the command */

	if (!send_rt_command(SET_BIAS, 0)) return -1;

	return 0;

} /* set_bias() */

/************************************************************************/
/* get_bias()								*/
/*									*/
/* Attempts to get bias data from the realtime side.			*/
/* Will always first get the parameters to make sure it knows how many	*/
/* pixels to get and if the system is ready for this or not.		*/
/* Returns a pointer to memory with the data in it which is allocated	*/
/* inside this function. The caller function should free this memory.	*/
/* Returns NULL if things do not go well.				*/
/************************************************************************/

unsigned char *get_bias(void)
{
	unsigned char *buf;
	DALSA_PARAM	dalsa_param;

	/* First, get the parameters */

	if (get_params(&dalsa_param) != 0) 
	{
		error(ERROR,"Failed to get params.");
		return NULL;
	}

	/* Are we ready? */

	if (!dalsa_param.bias_ready)
	{
		error(ERROR,"Bias not ready.");
		return NULL;
	}

	/* OK, allocate the memory */

	if ((buf = malloc(dalsa_param.data_size)) == NULL) 
	{
		error(ERROR,"Failed to get memory.");
		return NULL;
	}

	/* Send the command */

	if (!send_rt_command(GET_BIAS, 0))
	{
		error(ERROR,"Failed to send command.");
		free(buf);
		return NULL;
	}

	/* Get the data */

	if (read_rt_data(buf,dalsa_param.data_size) != dalsa_param.data_size) 
	{
		error(ERROR,"Failed to get data.");
		free(buf);
		return NULL;
	}

	/* That is all */

	return buf;

} /* get_bias() */

/************************************************************************/
/* set_bb()								*/
/* 									*/
/* Attempts to set the bounding box for Dalsa CCD 			*/
/* Returns -1 if it fails to send the command, -2 if it fails to	*/
/* send the data.							*/
/* Returns 0 if it worked.						*/
/************************************************************************/

int set_bb(int ulx, int uly, int lrx, int lry)
{
	/* Send the data itself */

	if (write_rt_data(&ulx,sizeof(ulx)) != sizeof(ulx)) 
		return -2;
	if (write_rt_data(&uly,sizeof(uly)) != sizeof(uly)) 
		return -2;
	if (write_rt_data(&lrx,sizeof(lrx)) != sizeof(lrx)) 
		return -2;
	if (write_rt_data(&lry,sizeof(lry)) != sizeof(lry)) 
		return -2;

	/* Send the command */

	if (!send_rt_command(SET_BB, 0)) return -1;

	return 0;

} /* set_bb() */

/************************************************************************/
/* get_bb()								*/
/*									*/
/* Attempts to get data from inside bounding box from the realtime side.*/
/* Will always first get the parameters to make sure it knows how many	*/
/* pixels to get and if the system is ready for this or not.		*/
/* Returns a pointer to memory with the data in it which is allocated	*/
/* inside this function. The caller function should free this memory.	*/
/* Returns NULL if things do not go well.				*/
/************************************************************************/

unsigned char *get_bb(void)
{
	unsigned char *buf;
	DALSA_PARAM	dalsa_param;
	int	i;

	/* First, get the parameters */

	if (get_params(&dalsa_param) != 0) 
	{
		error(ERROR,"Failed to get params.");
		return NULL;
	}

	/* Are we ready? */

	if (!dalsa_param.data_ready)
	{
		error(ERROR,"Data not ready.");
		return NULL;
	}

	/* OK, allocate the memory */

	if ((buf = malloc(dalsa_param.bb_size)) == NULL) 
	{
		error(ERROR,"Failed to get memory.");
		return NULL;
	}

	/* Send the command */

	if (!send_rt_command(GET_BB, 0))
	{
		error(ERROR,"Failed to send command.");
		free(buf);
		return NULL;
	}

	/* Get the data */

	if ((i=read_rt_data(buf,dalsa_param.bb_size)) != dalsa_param.bb_size) 
	{
		error(ERROR,"Failed to get data (%d instead of %d bytes).",
				i, dalsa_param.bb_size);
		free(buf);
		return NULL;
	}

	/* That is all */

	return buf;

} /* get_bb() */
