/************************************************************************/
/* dalsa_control.c							*/
/*                                                                      */
/* Functions to read and do stuff with the Dalsa camera.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar         		                */
/* Date   : Feb 2000 							*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../charalib/chara.h"
#include "../clock/clock.h"
#include "../../nsimpleX/nsimpleX.h"
#include "../../nrc/nrc.h"
#include "dalsa.h"

static int ccd_size = -1;
char    *default_display_machine = NULL;
static XImage *dalsaImage = NULL;
static Window	dalsaWindow;  /* Window for displaying things */
static time_t start_time;
static float	frames=0;
static bool	background_movie_running = FALSE;
static XImage *bbImage = NULL;
static Window	bbWindow;  /* Window for displaying things */
static time_t bb_start_time;
static	int	last_bb_size = 1;
static float	bb_frames=0;
static bool	bb_movie_running = FALSE;
static int	pixmult = 1;
static	int	last_pixmult = 1;
char *bias_types[3] = {"OFF","ON ", NULL};
static int min_frame, max_frame;
static int min_x, max_x;
static int min_y, max_y;

/************************************************************************/
/* set_ccd()								*/
/*									*/
/* Sets the global ccd_size once and for all so you don't have to	*/
/* do it for each function.						*/
/************************************************************************/

int set_ccd(int argc, char **argv)
{
	char	s[80];
	int	new_size;

	/* Has it already been done ? */

	if (ccd_size != -1)
	{
		sprintf(s,"CCD size has already been set to %d.",ccd_size);
		if (!ask_yes_no(s,"Do you want to reset it?"))
			return NOERROR;
	}

	/* Go get the data from the command line */

	if (argc > 1)
	{
		sscanf(argv[1],"%d",&new_size);
	}
	else
	{
		sprintf(s,"%4d",256);
		if (quick_edit("CCD size",
                        s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

                sscanf(s,"%d",&new_size);
	}

	/* Is it reasonable */

	if (new_size != 256 && new_size != 128)
	{
		return error(ERROR,"CCD size must be either 256 or 128.");
	}

	ccd_size = new_size;

	message(system_window,
		"Change will not take effect until you set parameters.");

	return NOERROR;

} /* set_ccd() */

/************************************************************************/
/* wait_for_frame()							*/
/*									*/
/* Waits for the frame to be ready. It will time out after a while	*/
/* and return FASLE. Returns TRUE if all went well.			*/
/************************************************************************/

bool wait_for_frame(DALSA_PARAM *dalsa_param)
{
	time_t stop_time;
	time_t start_time;

        time(&start_time);
	stop_time = start_time + MAX_EXPOSURE/1000 + 1;

	while(TRUE)
	{
		if (get_params(dalsa_param) != 0)
		{
			error(ERROR,"Failed to get Dalsa parameters.");
			return FALSE;
		}

		if (dalsa_param->data_ready) break;

		if (time(NULL) >= stop_time)
		{	
			error(ERROR,"Timed out wait for frame data.");
			return FALSE;
		}
	
		background();
	}
		
	return TRUE;

} /* wait_for_frame() */

/************************************************************************/
/* call_set_params()							*/
/*									*/
/* Tries to set parameters on the Dalsa camera and get a single fame.   */
/************************************************************************/

int call_set_params(int argc, char **argv)
{
	char	s[81];
	int	exposure;
	unsigned char binning;
	DALSA_PARAM dalsa_param;

	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	if (get_params(&dalsa_param) != 0)
	{
		error(ERROR,"Failed to get Dalsa parameters.");
		return FALSE;
	}

	/* Now look at the arguments */

	if (argc > 1)
	{
		sscanf(argv[1],"%d",&exposure);
	}
	else
	{
		sprintf(s,"%5d",dalsa_param.exposure);
		if (quick_edit("Exposure in milliseconds",
                        s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

                sscanf(s,"%d",&exposure);
	}

	if (argc > 2)
	{
		binning = (*argv[2] == 'Y' || *argv[2] == 'y');
	}
	else
	{
		binning = ask_yes_no("You can bin the chip into 2x2 pixels.",
				"Is this what you want to do?");
	}

	/* First we check that there is a frame there to get. */

	if (!wait_for_frame(&dalsa_param)) return ERROR;

	/* Now set it up */

	if (set_params(ccd_size,exposure,binning) != 0)
	{
		return error(ERROR,"Failed to set parameters.");
	}

	return NOERROR;

} /* call_set_parameters() */

/************************************************************************/
/* single_frame()							*/
/*									*/
/* Tries to arm the Dalsa camera and get a single fame. This will	*/
/* be displayed on the X terminal using display_frame.			*/
/************************************************************************/

int single_frame(int argc, char **argv)
{
	DALSA_PARAM dalsa_param;
	unsigned char *buffer; /* Data area for images */

	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	/* First we check that there is a frame there to get. */

	if (!wait_for_frame(&dalsa_param)) return ERROR;

	/* Are the parameters valid? */

	if (dalsa_param.exposure < 0)
	{
		return error(ERROR,"Set up the parameters first.");
	}

	/* Get the frame */

	if (!arm())
	{
		return error(ERROR,"Failed to arm CCD.");
	}

	if (!wait_for_frame(&dalsa_param)) return ERROR;

	if ((buffer = get_frame()) == NULL)
	{
		return error(ERROR,"Failed to get frame data.");
	}

	if (argc > 1)
	{
		return display_frame(argv[1],dalsa_param, buffer);
	}
	else
	{
		return display_frame(NULL,dalsa_param,buffer);
	}

} /* single_frame() */

/************************************************************************/
/* display_frame()							*/
/*									*/
/* Tries to display the frame on the X term. If no window exists it	*/
/* will create one for you.						*/
/* Also two globals, min_frame and max_frame, are set to the minimum	*/
/* and maximum of this frame.						*/
/************************************************************************/

int display_frame(char *display, DALSA_PARAM dalsa_param, unsigned char *buffer)
{
	int	col, row;
	int	colors;
        char    display_name[81];
	unsigned char *picture, *buf;
	int	pix;
	int	i,j;
	float	**values;

	/* Which machine do we display this on ? */

	if ((dalsaImage == NULL && bbImage == NULL) || !display_is_open)
	{
		if (display != NULL)
		{
			strcpy(display_name,display); 
		}
		else
		{
			if (default_display_machine == NULL)
			{
				sprintf(display_name,"%s:0.0",
					get_machine_name(display_name));
			}
			else
			{
				sprintf(display_name,"%s:0.0",
					default_display_machine);
			}
			if (quick_edit("Display name",display_name,
			display_name,NULL,STRING) == KEY_ESC) return NOERROR;
		}
		clean_command_line();
	}

	/* Setup the row and col variables */

	row = col = dalsa_param.ccd_size;
	if (dalsa_param.binning)
	{
		pix = dalsa_param.ccd_size/2;
	}
	else
	{	
		pix = dalsa_param.ccd_size;
	}

        /* Try and open the display */

	if (dalsaImage != NULL) 
	{
		XDestroyImage(dalsaImage);
		dalsaImage = NULL;
	}
	else
	{
		if (!display_is_open)
		{
			if (strcmpi(display_name,"null") == 0)
			{
				if ((colors = initX(NULL)) == 0) 
				{
					return error(ERROR,
						"Failed to open display.");
				}
			}
			else
			{
				if ((colors = initX(display_name)) == 0) 
				{
					return error(ERROR,
						"Failed to open display %s.",
						display_name);
				}
			}
		}
		dalsaWindow = openWindow("DALSA",50,50,col,row);
	}

	/* Allocate memory */

	values = matrix(1,pix,1,pix);

	/* Now fill in the colors */

	buf = buffer;
	min_frame = 256;
	max_frame = 0;
	max_x = max_y = 0;
	min_x = min_y = 0;

	/* Build the picture */

	for(j=1; j<= pix; j++)
	for(i=1; i<= pix; i++)
	{
		if (i != pix-1)
		{
			if (*buf > max_frame) 
			{
				max_frame = *buf;
				max_x = i;
				max_y = j;
			}
			if (*buf < min_frame) 
			{
				min_frame = *buf;
				min_x = i;
				min_y = j;
			}
		}

		values[i][j] = (float)(*buf++);
	}

/* Save image as a test.
	{
		FILE *fp;

		fp = fopen("test.dat", "w");
		buf = buffer;
		for(j=1; j<= pix; j++)
		for(i=1; i<= pix; i++)
			fprintf(fp,"%f %d\n", values[i][j], *buf++);
		fclose(fp);
	}
*/
		
	/* Display it */

	picture = make_picture(pix,pix,pixmult,values,STD);
	dalsaImage = XCreateImage(theDisplay,theVisual,
			theDepth,ZPixmap,0, picture,pix,pix,theBitmapPad,0);
	XPutImage(theDisplay,dalsaWindow,theGC,dalsaImage,0,0,0,0,
			pix,pix);
	XMapWindow(theDisplay, dalsaWindow);
	XFlush(theDisplay);

	/* Clean up memory */

	free(buffer);
	free_matrix(values,1,pix,1,pix);

	/* And go */

	return NOERROR;

} /* display_frame() */

/************************************************************************/
/* clear_window()							*/
/*									*/
/* Cleans up X windows stuff.						*/
/************************************************************************/

int clear_window(int argc, char **argv)
{

	if (background_movie_running || bb_movie_running)
		return error(ERROR,"Can not clear while movie running.");

	if (dalsaImage != NULL)
	{
		XDestroyImage(dalsaImage);
		dalsaImage = NULL;
		XDestroyWindow(theDisplay, dalsaWindow);
	}

	if (bbImage != NULL)
	{
		XDestroyImage(bbImage);
		bbImage = NULL;
		XDestroyWindow(theDisplay, bbWindow);
	}

	quitX();
	return NOERROR;

} /* clear_window() */

/************************************************************************/
/* dalsa_close()							*/
/*									*/
/* Called by UI when closing.						*/
/************************************************************************/

void dalsa_close(void)
{
	clear_window(0,NULL);

} /* dalsa_close() */

/************************************************************************/
/* movie()								*/
/*									*/
/* Run a movie of this stuff. Done by repeated calls to single_frame.   */
/************************************************************************/

int movie(int argc, char **argv)
{
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
#endif
	time_t	last_socket_update;
	char	socket_command[81];
	int	key;

	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	/* Are we already doing this in the background? */

	if (background_movie_running)
	{
		return error(ERROR,"Already running movie in the background.");
	}

	/* Tell background user what's up */

	if (active_socket != -1)
	{
		socket_print(active_socket,"Type <q> to exit.\n");
	}

	/* Now start up the main loop */

        time(&start_time);
	frames = 0;
	werase(system_window);
	mvwaddstr(system_window,0,0,CONT_TEXT);
	time(&last_socket_update);
	while(TRUE)
	{
		/* Display the frame */

		if (single_frame(0,NULL) != NOERROR) break;
		frames++;
		mvwprintw(system_window,1,0,
			"%5.1f F/S %3d (%3d,%3d) Max %3d (%3d,%3d) Min",
			frames/((float)time(NULL) - start_time),
			max_frame, max_x, max_y,
			min_frame, min_x, min_y);
		wrefresh(system_window);

		/* Tell socket user what's up */

		key = -1;
		if (active_socket != -1)
		{
			if (time(0) > last_socket_update)
			{
			    last_socket_update = time(0);
			    socket_print(active_socket,
			      "%5.1f F/S %3d (%3d,%3d) Max %3d (%3d,%3d) Min\n",
			      frames/((float)time(NULL) - start_time),
			      max_frame, max_x, max_y,
			      min_frame, min_x, min_y);
			}

			/* Did the socket user hit a key? */

			if ( char_waiting(active_socket))
                        {
                                if (socket_scan(active_socket, "%s", 
                                        socket_command) <= EOF || 
                                        *socket_command == 'q') 
                                {
                                        key = KEY_ESC;
                                }
                                else
                                {
                                        key = *socket_command;
                                }
                        }
		}

		/* Did someone type a key? */
		
		if (kbhit()) key = get_command();

		if (key != -1)
		{
#ifdef __NCURSES_H
			if (key == KEY_MOUSE)
			{
				getmouse(&mouse);

				if (mouse.bstate == BUTTON1_CLICKED)
				{
					if (mouse.y == SYSTEM_BEGIN_Y &&
						mouse.x <  strlen(CONT_TEXT)+2)
					{
						key = KEY_ESC;
					}
				}
                	}
#endif
			if (key == KEY_ESC) break;
		}

		/* Run a background job */

		background();
	}
	werase(system_window);
	wrefresh(system_window);

	return NOERROR;

} /* movie() */

/************************************************************************/
/* background_movie()							*/
/*									*/
/* Run a movie of this stuff. Done by repeated calls to single_frame.   */
/* This version does most of the work as a background process and so	*/
/* may be slow. It does, however, allow you to do other things while	*/
/* the movie is displayed.						*/
/* First call toggles it on, the second one off.			*/
/************************************************************************/

int background_movie(int argc, char **argv)
{
	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	if (background_movie_running)
	{
		background_del(movie_background_job);
		background_movie_running = FALSE;
		message(system_window,"Average frame rate = %.1f",
			frames/((float)time(NULL) - start_time));
	}
	else
	{
		background_add(movie_background_job);
		background_movie_running = TRUE;
		time(&start_time);
		frames = 0;
		if (!arm()) 
		{
			background_del(movie_background_job);
			background_movie_running = FALSE;
			return error(ERROR,"Failed to arm CCD.");
		}
	}
	
	return NOERROR;

} /* background_movie() */

/************************************************************************/
/* movie_background_job()						*/
/*									*/
/* Background job used by background_movie process.			*/
/* If a window does not already exist it will use the default window.	*/
/************************************************************************/

int movie_background_job(void)
{
	DALSA_PARAM dalsa_param;
	unsigned char *buffer;
	char	s[81],display[81];

	/* Is there a frame to get and display? */

	if (get_params(&dalsa_param) != 0)
	{
		error(ERROR,"Failed to get Dalsa parameters.");
		return FALSE;
	}

	if (!dalsa_param.data_ready) return NOERROR;

	/* Go get the frame */

	if ((buffer = get_frame()) == NULL)
	{
		return error(ERROR,"Failed to get frame data.");
	}

	/* display it */

	if (default_display_machine == NULL)
	{
		sprintf(display,"%s:0.0",get_machine_name(s));
	}
	else
	{
		sprintf(display,"%s:0.0",default_display_machine);
	}
	display_frame(display, dalsa_param, buffer);
	frames++;

	/* Re-arm the board */

	if (!arm()) 
	{
		background_del(movie_background_job);
		background_movie_running = FALSE;
		return error(ERROR,"Failed to re-arm board.");
	}

	return NOERROR;

} /* movie_background_job() */

/************************************************************************/
/* zero_bias()								*/
/*									*/
/* Sends command to zero the current bias.				*/
/************************************************************************/

int zero_bias(int argc, char **argv)
{
        if (!send_rt_command(ZERO_BIAS, 0)) 
		return error(ERROR,"Failed to zero bias.");

	return NOERROR;

} /* zero_bias() */

/************************************************************************/
/* calculate_bias()							*/
/*									*/
/* Tries to calculate and install a new BIAS frame.			*/
/* You must ensure that no realtime process is doing anything.		*/
/************************************************************************/

int calculate_bias(int argc, char **argv)
{
	float	*bias;
	unsigned char *buffer, *new_bias;
	DALSA_PARAM dalsa_param;
	int	i,j,n;
	char	s[81];
	float	*p,min,max,average;

	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	/* Look at the command line */

	if (argc > 1)
	{
		sscanf(argv[1],"%d",&n);
	}
	else
	{
		sprintf(s,"%5d",10);
		if (quick_edit("Number of frames to use",
                        s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

                sscanf(s,"%d",&n);
	}

	/* Is this reasonable */

	if (n >= 50)
	{
		sprintf(s,"You have asked for a lot of frames (%d)",n);
		if (!ask_yes_no(s,"Is this really what you want?")) 
			return NOERROR;
	}

	/* First we check that there is a frame there to get. */

	if (!wait_for_frame(&dalsa_param)) return ERROR;

	/* Are the parameters valid? */

	if (dalsa_param.exposure < 0)
	{
		return error(ERROR,"Set up the parameters first.");
	}

	/* Zero the current bias */

        if (!send_rt_command(ZERO_BIAS, 0)) 
		return error(ERROR,"Failed to zero bias.");

	/* Get some memory */

	if ((bias = malloc(dalsa_param.data_size * sizeof(float))) ==  NULL)
	{
		return error(ERROR,"Ran out of memory.");
	}

	if ((new_bias = malloc(dalsa_param.data_size)) ==  NULL)
	{
		return error(ERROR,"Ran out of memory.");
	}

	/* Get the frame */

	for(j=0; j<=dalsa_param.data_size; j++) *(bias+j) = 0.0;
	for(i=0; i< n; i++)
	{
		if (!arm())
		{
			free(bias);
			free(new_bias);
			return error(ERROR,"Failed to arm CCD.");
		}

		if (!wait_for_frame(&dalsa_param)) 
		{
			free(bias);
			free(new_bias);
			return ERROR;
		}

		if ((buffer = get_frame()) == NULL)
		{
			free(bias);
			free(new_bias);
			return error(ERROR,"Failed to get frame data.");
		}

		for(j=0;j<=dalsa_param.data_size;j++) *(bias+j) += *(buffer+j);

		free(buffer);

		message(system_window,"Process frame %d.",i+1);
	}

	min = 256.0;
	max = 0.0;
	average = 0.0;
	for(j=0; j<=dalsa_param.data_size; j++) 
	{
		p = bias+j;
		*p /= (float)n;

		if (*p > max) max = *p;
		if (*p < min) min = *p;
		average += *p;
		*(new_bias+j) = (unsigned char)(*p + 0.5);
	}
	average /= dalsa_param.data_size;
	
	/* Now send it one down */

	if (set_bias(new_bias, dalsa_param) != 0)
	{
		free(bias);
		free(new_bias);
		return error(ERROR,"Failed to send new bias.");
	}
	message(system_window,
		"Bias frame minimum = %.1f, maximum = %.1f and mean = %.1f.",
		min,max,average);

	free(bias);
	free(new_bias);
	return NOERROR;

} /* calculate_bias() */

/************************************************************************/
/* display_bias_frame()							*/
/*									*/
/* Get's the current bias data from the real time side so it can	*/
/* be displayed on the X terminal using display_frame.			*/
/************************************************************************/

int display_bias(int argc, char **argv)
{
	DALSA_PARAM dalsa_param;
	unsigned char *buffer; /* Data area for images */

	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	/* First we check that there is a frame there to get. */

	if (get_params(&dalsa_param) != 0)
	{
		error(ERROR,"Failed to get Dalsa parameters.");
		return FALSE;
	}

	/* Are the parameters valid? */

	if (dalsa_param.exposure < 0)
	{
		return error(ERROR,"Set up the parameters first.");
	}

	if (!dalsa_param.bias_ready)
	{
		return error(ERROR,"Bias data not ready.");
	}


	/* Go get the data */

	if ((buffer = get_bias()) == NULL)
	{
		return error(ERROR,"Failed to get bias data.");
	}

	if (argc > 1)
	{
		return display_frame(argv[1],dalsa_param, buffer);
	}
	else
	{
		return display_frame(NULL,dalsa_param,buffer);
	}

} /* display_bias() */

/************************************************************************/
/* set_full_bias()							*/
/*									*/
/* Let's the user either turn on or turn off the full biasing mode.	*/
/************************************************************************/

int set_full_bias(int argc, char **argv)
{
	DALSA_PARAM dalsa_param;
	int	res;
	char	s[80];

        mvwaddstr(command_window,0,2,"                        ");
        wrefresh(command_window);
        if (argc > 1)
        {
		strcpy(s,argv[1]);
		while (strlen(s) < strlen(bias_types[OFF])) strcat(s," ");
		if (strcmpi(s,bias_types[ON])==0)
			res = TRUE;
		else if (strcmpi(s,bias_types[OFF])==0)
			res = FALSE;
		else
			return error(ERROR,"Unknown setting %s",argv[1]);

        }
        else
        {
		if (get_params(&dalsa_param) != 0)
		{
			error(ERROR,"Failed to get Dalsa parameters.");
			return FALSE;
		}
		if (dalsa_param.full_bias)
			res = ON;
		else
			res = FALSE;

                if (quick_edit("Full bias",bias_types[res],
                        &res,bias_types,ENUMERATED) == KEY_ESC)
                                return -1;
        }

        if (!send_rt_command(SET_FULL_BIAS, (unsigned char) res)) 
		return error(ERROR,"Failed to zero bias.");

	return NOERROR;

} /* set_full_bias() */

/************************************************************************/
/* set_bb_bias()							*/
/*									*/
/* Let's the user either turn on or turn off the bb biasing mode.	*/
/************************************************************************/

int set_bb_bias(int argc, char **argv)
{
	DALSA_PARAM dalsa_param;
	int	res;
	char	s[80];

        mvwaddstr(command_window,0,2,"                        ");
        wrefresh(command_window);
        if (argc > 1)
        {
		strcpy(s,argv[1]);
		while (strlen(s) < strlen(bias_types[OFF])) strcat(s," ");
		if (strcmpi(s,bias_types[ON])==0)
			res = TRUE;
		else if (strcmpi(s,bias_types[OFF])==0)
			res = FALSE;
		else
			return error(ERROR,"Unknown setting %s",argv[1]);

        }
        else
        {
		if (get_params(&dalsa_param) != 0)
		{
			error(ERROR,"Failed to get Dalsa parameters.");
			return FALSE;
		}
		if (dalsa_param.bb_bias)
			res = ON;
		else
			res = FALSE;

                if (quick_edit("BB bias",bias_types[res],
                        &res,bias_types,ENUMERATED) == KEY_ESC)
                                return -1;
        }

        if (!send_rt_command(SET_BB_BIAS, (unsigned char) res)) 
		return error(ERROR,"Failed to zero bias.");

	return NOERROR;

} /* set_bb_bias() */

/************************************************************************/
/* call_set_bb()							*/
/*									*/
/* Tries to set bounding box on the Dalsa camera.   			*/
/************************************************************************/

int call_set_bb(int argc, char **argv)
{
	DALSA_PARAM dalsa_param;
	char	s[81];
	int	ulx,uly,lrx,lry;
	int	i,pixmult;
	int	rows;

	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	if (get_params(&dalsa_param) != 0)
	{
		error(ERROR,"Failed to get Dalsa parameters.");
		return FALSE;
	}

	/* If there are no arguments, maybe we can use the mouse */

	if (argc <= 1 && active_socket == -1 && dalsaImage != NULL)
	{
	        if (dalsa_param.binning)
		{
			pixmult = 2;
		        rows = dalsa_param.ccd_size / 2;
		}
		else
		{
			pixmult = 1;
			rows = dalsa_param.ccd_size;
		}
		message(system_window,
                        "Left click mouse on upper left corner.");
                if (!get_mouse(dalsaWindow, pixmult, &ulx, &uly))
                {
                        werase(system_window);
                        wrefresh(system_window);
                        return error(ERROR,"Failed to get upper left corner.");
                }

                /* Wait for a bit */

                working();
                for(i=0; i<50; i++) background();
                not_working();

                message(system_window,
                        "Left click mouse on lower right corner.");
                if (!get_mouse(dalsaWindow, pixmult, &lrx, &lry))
                {
                        werase(system_window);
                        wrefresh(system_window);
                        return error(ERROR,"Failed to get upper left corner.");
                }

               /* 
                * The get_mouse function does things for NRC arrays
                * and reverses the sense of y to make zero at the bottom.
                * We need to undo this for this routine.
                */
	
                ulx -= 1;
                lrx -= 1;
                uly = rows - uly;
                lry = rows - lry;
	}
	else
	{
		if (argc > 1)
		{
			sscanf(argv[1],"%d",&ulx);
		}
		else
		{
			sprintf(s,"%5d",dalsa_param.ulx);
			if (quick_edit("Upper left X",
				s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

			sscanf(s,"%d",&ulx);
		}

		if (argc > 2)
		{
			sscanf(argv[2],"%d",&uly);
		}
		else
		{
			sprintf(s,"%5d",dalsa_param.uly);
			if (quick_edit("Upper left Y",
				s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

			sscanf(s,"%d",&uly);
		}

		if (argc > 3)
		{
			sscanf(argv[3],"%d",&lrx);
		}
		else
		{
			sprintf(s,"%5d",dalsa_param.lrx);
			if (quick_edit("Low right X",
				s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

			sscanf(s,"%d",&lrx);
		}

		if (argc > 4)
		{
			sscanf(argv[4],"%d",&lry);
		}
		else
		{
			sprintf(s,"%5d",dalsa_param.lry);
			if (quick_edit("Lower right Y",
				s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

			sscanf(s,"%d",&lry);
		}

	}

	if (set_bb(ulx,uly,lrx,lry) < 0)
	{
		return error(ERROR,"Failed to set bounding box.");
	}

	message(system_window,"New Bounting box: (%d,%d) - (%d,%d)",
			ulx,uly,lrx,lry);

	return NOERROR;

} /* call_set_bb() */

/************************************************************************/
/* single_bb()								*/
/*									*/
/* Tries to arm the Dalsa camera and get a single fame. This will	*/
/* be displayed on the X terminal using display_frame.			*/
/************************************************************************/

int single_bb(int argc, char **argv)
{
	DALSA_PARAM dalsa_param;
	unsigned char *buffer; /* Data area for images */
	char s[80];

	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

        /* Go get the data from the command line */

	if (argc > 1)
	{
		sscanf(argv[1],"%d",&pixmult);
	}
	else
	{
		if (bbImage == NULL)
		{
			sprintf(s,"%4d",5);
			if (quick_edit("Pixel multiplier",
				s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

			sscanf(s,"%d",&pixmult);
		}
	}

	/* First we check that there is a frame there to get. */

	if (!wait_for_frame(&dalsa_param)) return ERROR;

	/* Are the parameters valid? */

	if (dalsa_param.exposure < 0)
	{
		return error(ERROR,"Set up the parameters first.");
	}

	/* Get the frame */

	if (!arm())
	{
		return error(ERROR,"Failed to arm CCD.");
	}

	if (!wait_for_frame(&dalsa_param)) return ERROR;

	if ((buffer = get_bb()) == NULL)
	{
		return error(ERROR,"Failed to get bb data.");
	}

	if (argc > 2)
	{
		return display_bb(argv[2],dalsa_param, buffer);
	}
	else
	{
		return display_bb(NULL,dalsa_param,buffer);
	}

} /* single_bb() */

/************************************************************************/
/* display_bb()								*/
/*									*/
/* Tries to display the bb on the X term. If no window exists it	*/
/* will create one for you.						*/
/* Also two globals, min_frame and max_frame, are set to the minimum	*/
/* and maximum of this frame.						*/
/************************************************************************/

int display_bb(char *display, DALSA_PARAM dalsa_param, 
		unsigned char *buffer)
{
	int	col, row;
	int	colors;
        char    display_name[81];
	unsigned char *picture, *buf;
	float	**values;
	int	i,j;

	/* Which machine do we display this on ? */

	if ((bbImage == NULL && dalsaImage == NULL) || !display_is_open)
	{
		if (display != NULL)
		{
			strcpy(display_name,display); 
		}
		else
		{
			if (default_display_machine == NULL)
			{
				sprintf(display_name,"%s:0.0",
					get_machine_name(display_name));
			}
			else
			{
				sprintf(display_name,"%s:0.0",
					default_display_machine);
			}
			if (quick_edit("Display name",display_name,
			display_name,NULL,STRING) == KEY_ESC) return NOERROR;
		}
		clean_command_line();
	}

	/* Setup the row and col variables */

	col = dalsa_param.lrx - dalsa_param.ulx + 1;
	row = dalsa_param.lry - dalsa_param.uly + 1;

	/* If the box size has changed we gotta redo the window */

	if (dalsa_param.bb_size != last_bb_size || pixmult != last_pixmult) 
	{
		clear_window(0,NULL);
		last_bb_size = dalsa_param.bb_size;
		last_pixmult = pixmult;
	}

        /* Try and open the display */

	if (bbImage != NULL) 
	{
		XDestroyImage(bbImage);
		bbImage = NULL;
	}
	else
	{
		if (!display_is_open)
		{
			if (strcmpi(display_name,"null") == 0)
			{
				if ((colors = initX(NULL)) == 0) 
				{
					return error(ERROR,
						"Failed to open display.");
				}
			}
			else
			{
				if ((colors = initX(display_name)) == 0) 
				{
					return error(ERROR,
						"Failed to open display %s.",
						display_name);
				}
			}
		}
		bbWindow = openWindow("DALSA BB",60+dalsa_param.ccd_size,50,
					col*pixmult,row*pixmult);
	}

	/* Allocate memory */

	values = matrix(1,col,1,row);

	/* Now fill in the values */

	buf = buffer;
	min_frame = 256;
	max_frame = 0;
	max_x = max_y = 0;
	min_x = min_y = 0;
	for(j=row; j>=1; j--)
	for(i=1; i<=col; i++)
	{
		values[i][j] = (*buf++)>>1;
		if (*buf > max_frame) 
		{
			max_frame = *buf;
			max_x = i;
			max_y = j;
		}
		if (*buf < min_frame) 
		{
			min_frame = *buf;
			min_x = i;
			min_y = j;
		}
	}

	/* Create a picture */

	picture = make_picture(col,row,pixmult,values,STD);

	/* Display it */

	bbImage = XCreateImage(theDisplay,theVisual,
			theDepth,ZPixmap,0, picture,col*pixmult,row*pixmult, 
			theBitmapPad,col*pixmult);
	XPutImage(theDisplay,bbWindow,theGC,bbImage,0,0,0,0,
			col*pixmult,row*pixmult);
	XMapWindow(theDisplay, bbWindow);
	XFlush(theDisplay);

	/* Clean up memory */

	free(buffer);
	free_matrix(values,1,col,1,row);

	/* And go */

	return NOERROR;

} /* display_bb() */

/************************************************************************/
/* bb_movie()								*/
/*									*/
/* Run a movie of this stuff. Done by repeated calls to display_bb.     */
/************************************************************************/

int bb_movie(int argc, char **argv)
{
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
#endif
	time_t	last_socket_update;
	char	socket_command[81];
	int	key;

	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	/* Are we already doing this in the background? */

	if (bb_movie_running)
	{
		return error(ERROR,"Already running movie in the background.");
	}

	/* Tell background user what's up */

	if (active_socket != -1)
	{
		socket_print(active_socket,"Type <q> to exit.\n");
	}

        time(&bb_start_time);
	bb_frames = 0;
	werase(system_window);
	mvwaddstr(system_window,0,0,CONT_TEXT);
	time(&last_socket_update);
	while(TRUE)
	{
		/* Display the frame */

		if (single_bb(0,NULL) != NOERROR) break;
		bb_frames++;
		mvwprintw(system_window,1,0,
			"%5.1f F/S %3d (%3d,%3d) Max %3d (%3d,%3d) Min",
			bb_frames/((float)time(NULL) - bb_start_time),
			max_frame, max_x, max_y,
			min_frame, min_x, min_y);
		wrefresh(system_window);

                /* Tell socket user what's up */

                key = -1;
                if (active_socket != -1)
                {
                        if (time(0) > last_socket_update)
                        {
                            last_socket_update = time(0);
                            socket_print(active_socket,
                              "%5.1f F/S %3d (%3d,%3d) Max %3d (%3d,%3d) Min\n",
                              frames/((float)time(NULL) - start_time),
                              max_frame, max_x, max_y,
                              min_frame, min_x, min_y);
                        }

                        /* Did the socket user hit a key? */

                        if ( char_waiting(active_socket))
                        {
                                if (socket_scan(active_socket, "%s",
                                        socket_command) <= EOF ||
                                        *socket_command == 'q')
                                {
                                        key = KEY_ESC;
                                }
                                else
                                {
                                        key = *socket_command;
                                }
                        }
                }

                /* Did someone type a key? */

                if (kbhit()) key = get_command();

                if (key != -1)
                {
#ifdef __NCURSES_H
                        if (key == KEY_MOUSE)
                        {
                                getmouse(&mouse);

                                if (mouse.bstate == BUTTON1_CLICKED)
                                {
                                        if (mouse.y == SYSTEM_BEGIN_Y &&
                                                mouse.x <  strlen(CONT_TEXT)+2)
                                        {
                                                key = KEY_ESC;
                                        }
                                }
                        }
#endif
                        if (key == KEY_ESC) break;
                }

		/* Run a background job */

		background();
	}
	werase(system_window);
	wrefresh(system_window);

	return NOERROR;

} /* bb_movie() */

/************************************************************************/
/* bb_background_movie()						*/
/*									*/
/* Run a movie of BB. Done by repeated calls to single_bb.   		*/
/* This version does most of the work as a background process and so	*/
/* may be slow. It does, however, allow you to do other things while	*/
/* the movie is displayed.						*/
/* First call toggles it on, the second one off.			*/
/************************************************************************/

int bb_background_movie(int argc, char **argv)
{
	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	if (bb_movie_running)
	{
		background_del(bb_movie_background_job);
		bb_movie_running = FALSE;
		message(system_window,"Average frame rate = %.1f",
			bb_frames/((float)time(NULL) - bb_start_time));
	}
	else
	{
		background_add(bb_movie_background_job);
		bb_movie_running = TRUE;
		time(&bb_start_time);
		bb_frames = 0;
		if (!arm()) 
		{
			background_del(bb_movie_background_job);
			bb_movie_running = FALSE;
			return error(ERROR,"Failed to arm CCD.");
		}
	}
	
	return NOERROR;

} /* bb_background_movie() */

/************************************************************************/
/* bb_movie_background_job()						*/
/*									*/
/* Background job used by background_movie process.			*/
/* If a window does not already exist it will use the default window.	*/
/************************************************************************/

int bb_movie_background_job(void)
{
	DALSA_PARAM dalsa_param;
	unsigned char *buffer;
	char	s[81],display[81];

	/* Is there a bb to get and display? */

	if (get_params(&dalsa_param) != 0)
	{
		error(ERROR,"Failed to get Dalsa parameters.");
		return FALSE;
	}

	if (!dalsa_param.data_ready) return NOERROR;

	/* Go get the bb */

	if ((buffer = get_bb()) == NULL)
	{
		return error(ERROR,"Failed to get bb data.");
	}

	/* display it */

	if (default_display_machine == NULL)
	{
		sprintf(display,"%s:0.0",get_machine_name(s));
	}
	else
	{
		sprintf(display,"%s:0.0",default_display_machine);
	}
	display_bb(display, dalsa_param, buffer);
	bb_frames++;

	/* Re-arm the board */

	if (!arm()) 
	{
		background_del(movie_background_job);
		background_movie_running = FALSE;
		return error(ERROR,"Failed to re-arm board.");
	}

	return NOERROR;

} /* bb_movie_background_job() */

/************************************************************************/
/* show_dalsa_structure()						*/
/*									*/
/* Displays the dalsa data in the sub main window.			*/
/* Also allows commands to be typed.					*/
/************************************************************************/

int show_dalsa_structure(int argc, char **argv)
{  
	DALSA_PARAM dalsa_param;
	bool 	loop;
        int     keystroke;
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
#endif

	if (active_socket == -1)
	{

		active_window = sub_main_window;
		loop = TRUE;
		while(loop)
		{
			heading(heading_window,"DALSA STRUCTURE","");

			background();
			
			if (get_params(&dalsa_param) != 0)
			{
				error(ERROR,"Failed to get Dalsa parameters.");
				return FALSE;
			}

			werase(sub_main_window);
                        message(system_window,CONT_TEXT);

			mvwaddstr(sub_main_window,0,0,"CCD SIZE   : ");
			wprintw(sub_main_window,"   %8d",dalsa_param.ccd_size);

			mvwaddstr(sub_main_window,1,0,"EXPOSURE   : ");
			wprintw(sub_main_window,"   %8d",dalsa_param.exposure);

			mvwaddstr(sub_main_window,2,0,"STATUS     : ");
			wprintw(sub_main_window,"     0x%04x",
					dalsa_param.status);

			mvwaddstr(sub_main_window,3,0,"BB UL      : ");
			wprintw(sub_main_window,"  (%3d,%3d)",
				dalsa_param.ulx,dalsa_param.uly);

			mvwaddstr(sub_main_window,4,0,"BB LR      : ");
			wprintw(sub_main_window,"  (%3d,%3d)",
				dalsa_param.lrx,dalsa_param.lry);

			mvwaddstr(sub_main_window,5,0,"BB SIZE    : ");
			wprintw(sub_main_window,"  %9d",dalsa_param.bb_size);

			mvwaddstr(sub_main_window,6,0,"SECONDS    : ");
			wprintw(sub_main_window,"  %9d",dalsa_param.seconds);

			mvwaddstr(sub_main_window,0,40,"DATA READY : ");
			waddstr(sub_main_window,
			dalsa_param.data_ready?"        YES":"         NO");

			mvwaddstr(sub_main_window,1,40,"BUFFER     : ");
			wprintw(sub_main_window," 0x%08x",dalsa_param.dma_buf);

			mvwaddstr(sub_main_window,2,40,"BIAS READY : ");
			waddstr(sub_main_window,
			dalsa_param.bias_ready?"        YES":"         NO");

			mvwaddstr(sub_main_window,3,40,"BIAS       : ");
			wprintw(sub_main_window," 0x%08d",dalsa_param.bias_buf);

			mvwaddstr(sub_main_window,4,40,"DATA SIZE  : ");
			wprintw(sub_main_window,"   %8d",
				dalsa_param.data_size);

			mvwaddstr(sub_main_window,5,40,"BINNING    : ");
			waddstr(sub_main_window,
			dalsa_param.binning?"        YES":"         NO");

			mvwaddstr(sub_main_window,6,40,"FRAMES     : ");
			wprintw(sub_main_window,"  %9d",dalsa_param.num_frames);

			wrefresh(sub_main_window);

                        /* Was a key hit ? */

                        if (kbhit())
                        {
                                keystroke = get_command();

                                /* Was it a mouse click? */
#ifdef __NCURSES_H
                                if (keystroke == KEY_MOUSE)
                                {
                                        getmouse(&mouse);

                                        if (mouse.bstate == BUTTON1_CLICKED)
                                        {
                                                if (mouse.y == SYSTEM_BEGIN_Y &&
                                                mouse.x <  strlen(CONT_TEXT)+2)
                                                {
                                                        keystroke = KEY_ESC;
                                                }
                                        }
                                }

#endif
                                /* Was it KEY_ESC? */

                                if (keystroke == KEY_ESC) 
                                {
                                        loop = FALSE;
                                }
                                else
                                {
                                        /* Is it a command ? */

                                        command_processor(keystroke);
                                }
                        }
                }
                werase(system_window);
                wrefresh(system_window);
        }
	else
	{
		if (get_params(&dalsa_param) != 0)
		{
			error(ERROR,"Failed to get Dalsa parameters.");
			return FALSE;
		}

		socket_print(active_socket,"CCD SIZE   : ");
		socket_print(active_socket,"   %8d\n",dalsa_param.ccd_size);

		socket_print(active_socket,"EXPOSURE   : ");
		socket_print(active_socket,"   %8d\n",dalsa_param.exposure);

		socket_print(active_socket,"STATUS     : ");
		socket_print(active_socket,"     0x%04x\n",dalsa_param.status);

		socket_print(active_socket,"BB UL      : ");
		socket_print(active_socket,"  (%3d,%3d)\n",
			dalsa_param.ulx,dalsa_param.uly);

		socket_print(active_socket,"BB LR      : ");
		socket_print(active_socket,"  (%3d,%3d)\n",
			dalsa_param.lrx,dalsa_param.lry);

		socket_print(active_socket,"BB SIZE    : ");
		socket_print(active_socket,"  %9d\n",dalsa_param.bb_size);

		socket_print(active_socket,"DATA READY : ");
		socket_print(active_socket,"%s\n",
			dalsa_param.data_ready?"        YES":"         NO");

		socket_print(active_socket,"BUFFER     : ");
		socket_print(active_socket," 0x%08x\n",dalsa_param.dma_buf);

		socket_print(active_socket,"BIAS READY : ");
		socket_print(active_socket,"%s\n",
			dalsa_param.bias_ready?"        YES":"         NO");

		socket_print(active_socket,"BIAS       : ");
		socket_print(active_socket," 0x%08d\n",dalsa_param.bias_buf);

		socket_print(active_socket,"DATA SIZE  : ");
		socket_print(active_socket,"   %8d\n",dalsa_param.data_size);

		socket_print(active_socket,"BINNING    : ");
		socket_print(active_socket,"%s\n",
			dalsa_param.binning?"        YES":"         NO");
	}

	return NOERROR;

} /* show_dalsa_structure(active_socket,) */

/************************************************************************/
/* measure_frame_rate()							*/
/*									*/
/* Calls the realtime function dalsa_measure_frame_rate().		*/
/************************************************************************/

int measure_frame_rate(int argc, char **argv)
{
	char	s[80];
	int	seconds;

        /* Has the ccd size been set yet? */

        if (ccd_size == -1)
        {
                return error(ERROR,"You must first set the CCD size.");
        }

        /* Go get the data from the command line */

        if (argc > 1)
        {
                sscanf(argv[1],"%d",&seconds);
        }
        else
        {
                sprintf(s,"%4d",10);
                if (quick_edit("Number of seconds",
                                s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;

                sscanf(s,"%d",&seconds);
        }

	/* Is it reasonable ? */

	if (seconds < 0 || seconds > 255)
	{
		return error(ERROR,
			"Number of seconds must be between 0 and 255.");
	}

	/* Send the command */

        if (!send_rt_command(FRAME_RATE, (unsigned char) seconds)) 
		return error(ERROR,"Failed to setup frame measurement.");

	return NOERROR;

} /* measure_frame_rate() */

/************************************************************************/
/* save_frame()								*/
/*									*/
/* Get's a frames and saves them to a file. The file will consist    	*/
/* of a series of unsigned bytes. The first two are the x and y sizes	*/
/* then comes the data in the frame itself.				*/
/************************************************************************/

int	save_frame(int argc, char **argv)
{
	char		s[81];
	char		t[181];
	char		temp[81];
	char		*filename,*p;
	bool		found_dir;
	DALSA_PARAM 	dalsa_param;
	unsigned char 	*buffer;
	FILE		*output;
	unsigned char	x,y;
	int		frames;
	int		c;
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
#endif

        /* Check out the command line */

        if (argc > 1)
        {
                filename = argv[1];
        }
        else
        {
                filename = s;
                sprintf(s,"        frame.dat");
                if (quick_edit("Filename",s,s,NULL,STRING) == KEY_ESC) 
                        return NOERROR;
        }

        while(*filename==' ' && *filename != 0) filename++;
        if (*filename == 0)
        {
                return error(ERROR,"Null file name.");
        }

        /* OK, if no directory specified, put it in data directory. */

        for(found_dir = FALSE, p = filename; !found_dir && *p != 0; p++)
        {
                if (*p == '/') found_dir = TRUE;
        }

        if (!found_dir)
        {
                sprintf(temp,"%s%s",get_data_directory(t),filename);
                filename = temp;
        }

	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	/* OK, now we try and save the data */

	if (fopen(filename,"r") != NULL)
	{
		sprintf(s,"The file %s already exists.",filename);
		if (!ask_yes_no(s,"Shall we overwrite it?")) 		
		{
			return NOERROR;
		}
	}

	if ((output = fopen(filename,"w")) == NULL)
	{
		return error(ERROR,"Failed to open file %s.", filename);
	}

	/* First we check that there is a frame there to get. */

	if (!wait_for_frame(&dalsa_param)) return ERROR;

	/* Are the parameters valid? */

	if (dalsa_param.exposure < 0)
	{
		return error(ERROR,"Set up the parameters first.");
	}

	if (dalsa_param.binning)
	{
		x = y = dalsa_param.ccd_size / 2;
	}
	else
	{
		x = y = dalsa_param.ccd_size;
	}

	if (fwrite(&x, sizeof(unsigned char), 1, output) != 1 ||
	    fwrite(&y, sizeof(unsigned char), 1, output) != 1)
	{
		fclose(output);
		return error(ERROR,"Failed to write data to %s.",filename);
	}

	/* Start looping */

	frames = 1;
	while(TRUE)
	{
		message(system_window,"%s Frame %d",CONT_TEXT,frames++);

		/* Get the frame */

		if (!arm())
		{
			werase(system_window);
			wrefresh(system_window);
			return error(ERROR,"Failed to arm CCD.");
		}

		if (!wait_for_frame(&dalsa_param)) 
		{
			werase(system_window);
			wrefresh(system_window);
			return ERROR;
		}

		if ((buffer = get_frame()) == NULL)
		{
			werase(system_window);
			wrefresh(system_window);
			fclose(output);
			return error(ERROR,"Failed to get frame data.");
		}

		if (fwrite(buffer, sizeof(unsigned char), 
			dalsa_param.data_size, output) != dalsa_param.data_size)
		{
			fclose(output);
			free(buffer);
			werase(system_window);
			wrefresh(system_window);
			return error(ERROR,
				"Failed to write data to %s.",filename);
		}
		free(buffer);

		if (kbhit())
		{
			c = get_command();
#ifdef __NCURSES_H
			if (c == KEY_MOUSE)
			{
				getmouse(&mouse);

				if (mouse.bstate == BUTTON1_CLICKED)
				{
					if (mouse.y == SYSTEM_BEGIN_Y &&
						mouse.x <  strlen(CONT_TEXT)+2)
					{
						c = KEY_ESC;
					}
				}
                	}
#endif
			if (c  == KEY_ESC) break;
		}
	}
			
	werase(system_window);
	wrefresh(system_window);
	fclose(output);
	return NOERROR;

} /* save_frame() */
	
/************************************************************************/
/* save_bb()								*/
/*									*/
/* Get's subframes and saves them to a file. The file will consist 	*/
/* of a series of unsigned bytes. The first two are the x and y sizes	*/
/* then comes the data in the frame itself.				*/
/************************************************************************/

int	save_bb(int argc, char **argv)
{
	char		s[81];
	char		t[181];
	char		temp[81];
	char		*filename,*p;
	bool		found_dir;
	DALSA_PARAM 	dalsa_param;
	unsigned char 	*buffer;
	FILE		*output;
	unsigned char	x,y;
	int		frames;
	int		c;
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
#endif

        /* Check out the command line */

        if (argc > 1)
        {
                filename = argv[1];
        }
        else
        {
                filename = s;
                sprintf(s,"        frame.dat");
                if (quick_edit("Filename",s,s,NULL,STRING) == KEY_ESC) 
                        return NOERROR;
        }

        while(*filename==' ' && *filename != 0) filename++;
        if (*filename == 0)
        {
                return error(ERROR,"Null file name.");
        }

        /* OK, if no directory specified, put it in data directory. */

        for(found_dir = FALSE, p = filename; !found_dir && *p != 0; p++)
        {
                if (*p == '/') found_dir = TRUE;
        }

        if (!found_dir)
        {
                sprintf(temp,"%s%s",get_data_directory(t),filename);
                filename = temp;
        }

	/* Has the ccd size been set yet? */

	if (ccd_size == -1)
	{
		return error(ERROR,"You must first set the CCD size.");
	}

	/* OK, now we try and save the data */

	if (fopen(filename,"r") != NULL)
	{
		sprintf(s,"The file %s already exists.",filename);
		if (!ask_yes_no(s,"Shall we overwrite it?")) 		
		{
			return NOERROR;
		}
	}

	if ((output = fopen(filename,"w")) == NULL)
	{
		return error(ERROR,"Failed to open file %s.", filename);
	}

	/* First we check that there is a frame there to get. */

	if (!wait_for_frame(&dalsa_param)) return ERROR;

	/* Are the parameters valid? */

	if (dalsa_param.exposure < 0)
	{
		return error(ERROR,"Set up the parameters first.");
	}

	x = dalsa_param.lrx - dalsa_param.ulx + 1;
	y = dalsa_param.lry - dalsa_param.uly + 1;

	if (fwrite(&x, sizeof(unsigned char), 1, output) != 1 ||
	    fwrite(&y, sizeof(unsigned char), 1, output) != 1)
	{
		fclose(output);
		return error(ERROR,"Failed to write data to %s.",filename);
	}

	/* Start looping */

	frames = 1;
	while(TRUE)
	{
		message(system_window,"%s Frame %d",CONT_TEXT,frames++);

		/* Get the frame */

		if (!arm())
		{
			werase(system_window);
			wrefresh(system_window);
			return error(ERROR,"Failed to arm CCD.");
		}

		if (!wait_for_frame(&dalsa_param)) 
		{
			werase(system_window);
			wrefresh(system_window);
			return ERROR;
		}

		if ((buffer = get_bb()) == NULL)
		{
			fclose(output);
			werase(system_window);
			wrefresh(system_window);
			return error(ERROR,"Failed to get frame data.");
		}

		if (fwrite(buffer, sizeof(unsigned char), 
			dalsa_param.bb_size, output) != dalsa_param.bb_size)
		{
			fclose(output);
			free(buffer);
			werase(system_window);
			wrefresh(system_window);
			return error(ERROR,
				"Failed to write data to %s.",filename);
		}
		free(buffer);

		if (kbhit())
		{
			c = get_command();
#ifdef __NCURSES_H
			if (c == KEY_MOUSE)
			{
				getmouse(&mouse);

				if (mouse.bstate == BUTTON1_CLICKED)
				{
					if (mouse.y == SYSTEM_BEGIN_Y &&
						mouse.x <  strlen(CONT_TEXT)+2)
					{
						c = KEY_ESC;
					}
				}
                	}
#endif
			if (c  == KEY_ESC) break;
		}
	}
			
	werase(system_window);
	wrefresh(system_window);
	fclose(output);
	return NOERROR;

} /* save_bb() */
