/************************************************************************/
/* dalsa_spec.c								*/
/*                                                                      */
/* Routines for looking at the dalsa data in the spectrograph.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 			                */
/* Date   : 1998-1999                					*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../charalib/chara.h"
#include "../../nrc/nrc.h"
#include "../../nsimpleX/nsimpleX.h"
#include "dalsa.h"

static XImage *ps_image = NULL;
static XImage *data_image = NULL;
static float ps_bias[128];
static float data_flat_field[256];
static float data_dark[256];
static bool debias_ps = FALSE;
static bool flat_data = FALSE;
static bool dark_data = FALSE;
static int num_pix_zero = 0;
static int spec_row = 0;
static bool auto_range_display = FALSE;
static float spect_cutoff=1.5;

/************************************************************************/
/* bb_power_spec()							*/
/*									*/
/* Get's the current bounding box, get's 				*/
/* the row of data, calculates the power				*/
/* spectrum of that row.						*/
/* Returns this power spectrum.						*/
/* FFT always done in 256 sized array so result always 0-128 long.	*/
/* If no_fft set to TRUE no FFT is done and the result is 256 long.	*/
/* Returns NULL if something goes wrong.				*/
/* IF the no_fft flag is TRUE it simply returns the data untouched.	*/
/************************************************************************/

float *bb_power_spec(bool no_fft)
{
	float	*ps;
	float	data[256];
        DALSA_PARAM dalsa_param;
	int	start_i,i,j;
	unsigned char *bb;
	int	size;
	float	total;

	/* First of all, wait for the frame */

        if (!wait_for_frame(&dalsa_param)) return NULL;

	/* What size is the thing? */

	size = dalsa_param.lrx - dalsa_param.ulx + 1;
	start_i = (256-size)/2;

	/* Go get the bounding box */

	if ((bb = get_bb()) == NULL) return NULL;

	/* OK, Create the data */

	for(i=0; i<start_i; i++) data[i] = 0;
	for(i=start_i+size; i<256; i++) data[i] = 0;
	if (flat_data)
	{
	    total = 0.0;
	    if (dark_data)
	    {
		for(i=0; i<size; i++) 
		{
		    j = start_i + i;
		    if ((data[j] = *(bb+spec_row*size+i) - data_dark[j]) < 0.0)
			data[j] = 0.0;
		    total += data[j];
		}
	    }
	    else
	    {
		for(i=0; i<size; i++) 
		{
		    total += (data[start_i+i] = *(bb+spec_row*size+i));
		}
	    }
		

	    for(i=0; i<size; i++) 
	    {
		j = i + start_i;
		if (data_flat_field[j] == 0)
		{
		    data[j] = 0.0;
		}
		else
		{
		    data[j] = (float)data[j]/
			      (total*data_flat_field[j])-1.0;
		}
	    }
	}
	else
	{
	    if (dark_data) for(i=0; i<size; i++) 
	    {
		j = start_i + i;
	    	if ((data[j] = (float)*(bb+spec_row*size+i) - data_dark[j])<0.0)
			data[j] = 0.0;
	    }
	    else for(i=0; i<size; i++) 
	    {
	    	data[i+start_i] = (float)*(bb+spec_row*size+i);
	    }
	}
	
	free(bb);
	
	/* If we are not doing an FFT just return the data */

	if (no_fft)
	{
		ps = vector(0,255);
		for(i=0;i<256;i++) ps[i] = data[i];
		return ps;
	}

	/* Do the FFT on the data */

	realft(data-1, 256, 1);

	/* Now pull out the power spectrum (forcing first few to be zero */

	ps = vector(0,128);
	ps[0] = data[0] * data[0];
	ps[128] = data[1]*data[1];
	for(i=1;i<128;i++)
	{
		j = i<<1;
		ps[i] = data[j]*data[j] + data[j+1] * data[j+1];
	}
	if (debias_ps)
	{
		for(i=0;i<129;i++) ps[i] -= ps_bias[i];
	}
	if (num_pix_zero > 0) for(i=0; i<num_pix_zero; i++) ps[i] = 0.0;

	return ps;
	
} /* bb_power_spec() */

/************************************************************************/
/* display_power_spec()							*/
/*									*/
/* Calls bb_power_spec and then displays the result in a window.	*/
/* Returns the power spectrum or NULL on error.				*/
/* If the bias is set to NULL it is ignored, if set to point to		*/
/* a vector of numbers these will be subtracted before display. This 	*/
/* only happens if you are doing thepower spectrum. no_fft bool like	*/
/* bb_power_spec call.							*/
/************************************************************************/

float *display_power_spectrum(Window win, int width, 
		int height, bool no_fft)
{
	float *ps;
	char  *picture;
	float	min,max;

	/* First, get the power spectrum */

	if ((ps = bb_power_spec(no_fft)) == NULL) return NULL;

	/* Create the picture */

	if (no_fft)
	{
		if (auto_range_display)
		{
			min = 0.0;
			max = 0.0;
		}
		else if (flat_data)
		{
			max = 0.5;
			min = -0.5;
		}
		else
		{
			max = 256;
			min = 0.0;
		}
		if (data_image != NULL) XDestroyImage(data_image);
		if ((picture=plot_funct(width,height,ps-1,256,&min,&max))==NULL)
		{
			free_vector(ps,0,255);
			return NULL;
		}
		data_image = XCreateImage(theDisplay,theVisual, 
					theDepth,ZPixmap,0, 
					picture, width, height, 8, width);
		XPutImage(theDisplay,win,theGC,data_image,0,0,0,0,width,height);
	}
	else
	{
		min = 0.0;
		max = 0.0;
		if (ps_image != NULL) XDestroyImage(ps_image);
		if ((picture=plot_funct(width,height,ps-1,129,&min,&max))==NULL)
		{
			free_vector(ps,0,128);
			return NULL;
		}
		ps_image = XCreateImage(theDisplay,theVisual, 
					theDepth,ZPixmap,0, 
					picture, width, height, 8, width);
		XPutImage(theDisplay, win, theGC,ps_image,0,0,0,0,width,height);
	}

	/* Now display it */

	XMapWindow(theDisplay, win);
	XFlush(theDisplay);

	return ps;

} /* display_power_spectrum() */

/************************************************************************/
/* call_display_power_spec()						*/
/*									*/
/* User callable version of display_power_spec.				*/
/************************************************************************/


int call_display_power_spec(int argc, char **argv)
{
	Window  ps_win;
	Window  data_win;
	char	display_name[81];
	float	*ps;
        DALSA_PARAM dalsa_param;
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
#endif
        char    socket_command[81];
        int     key;

	/* Warn the user about setting the row. */

	if (spec_row == 0) 
		error(MESSAGE,
		"You may want to set the row to something other than 0.");

	/* Create the windows */

	if (display_is_open)
	{
		ps_win = openWindow("PS",theWidth-525,480, 512, 100);
		data_win = openWindow("DATA",theWidth-525,610, 512, 100);
	}

	message(system_window,CONT_TEXT);
	if (active_socket != -1) 
	{
		socket_print(active_socket,"Type <q> to exit.\n");
	}

	while(1)
	{
		if (!display_is_open)
		{
			if (argc > 1)
			{
				strcpy(display_name,argv[1]);
			}
			else
			{
				if (default_display_machine == NULL)
				{
					sprintf(display_name,"%s:0.0",
						get_machine_name(display_name));
				}
				else
				{
					sprintf(display_name,"%s:0.0",
						default_display_machine);
				}
				if (quick_edit("Display name",display_name,
				display_name,NULL,STRING) == KEY_ESC) 
					return NOERROR;
			}

			if (strcmpi(display_name,"null") == 0)
			{
				if (initX(NULL) == 0) 
				{
					return error(ERROR,
						"Failed to open display.");
				}
			}
			else
			{
				if (initX(display_name) == 0) 
				{
					return error(ERROR,
						"Failed to open display %s.",
						display_name);
				}
			}

			/* ReCreate the windows */

			ps_win = openWindow("PS",theWidth-525,480,512,100);
			data_win = openWindow("DATA",theWidth-525,610,512,100);

		}
		
		background();

		if (!wait_for_frame(&dalsa_param)) return ERROR;
	
		/* Are the parameters valid? */

		if (dalsa_param.exposure < 0)
		{
			return error(ERROR,"Set up the parameters first.");
		}

		/* Get the frame */

		if (!arm())
		{
			return error(ERROR,"Failed to arm CCD.");
		}

		if ((ps=display_power_spectrum(ps_win,512,100,FALSE))
			==NULL)
		{
			clean_command_line();
			XDestroyWindow(theDisplay, ps_win);
			XDestroyWindow(theDisplay, data_win);
			werase(system_window);
			wrefresh(system_window);
			return error(ERROR,"Failed to display power spectrum.");
		}
		free_vector(ps,0,128);

		if ((ps=display_power_spectrum(data_win,512,100,TRUE))
			==NULL)
		{
			clean_command_line();
			XDestroyWindow(theDisplay, ps_win);
			XDestroyWindow(theDisplay, data_win);
			werase(system_window);
			wrefresh(system_window);
			return error(ERROR,"Failed to display power spectrum.");
		}
		free_vector(ps,0,256);

		key = -1;
		if (active_socket != -1 && char_waiting(active_socket))
		{
			if (socket_scan(active_socket,"%s", socket_command) 
				<= EOF || *socket_command == 'q')
			{
				key = KEY_ESC;
			}
			else
			{
				key = -1;
			}
		}

		if (kbhit()) key = get_command();

		if (key != -1)
		{
#ifdef __NCURSES_H
			if (key == KEY_MOUSE)
			{
				getmouse(&mouse);

				if (mouse.bstate == BUTTON1_CLICKED)
				{
					if (mouse.y == SYSTEM_BEGIN_Y &&
						mouse.x <  strlen(CONT_TEXT)+2)
					{
						key = KEY_ESC;
					}
				}
			}
#endif
			if (key == KEY_ESC) 
			{
				clean_command_line();
				XDestroyWindow(theDisplay, ps_win);
				XDestroyWindow(theDisplay, data_win);
				werase(system_window);
				wrefresh(system_window);
				return NOERROR;
			}

			command_processor(key);
		}

	}

} /* call_display_power_spec() */

/************************************************************************/
/* power_spec_bias()							*/
/*									*/
/* calculates the power spectrum bias data.				*/
/************************************************************************/

int power_spec_bias(int argc, char **argv)
{
	int	i,j;
	float	*ps;
	float	n;
	char	s[81];
        DALSA_PARAM dalsa_param;

	/* Analyses the command line */

	if (argc > 1)
	{
		sscanf(argv[1],"%f",&n);
	}
	else
	{
		sprintf(s,"    10");
		if (quick_edit("Number of frames to average",s, s,NULL,INTEGER)
			== KEY_ESC) return NOERROR;
		sscanf(s,"%f",&n);
                clean_command_line();
	}

	/* First we set it to zero */

	for(i=0; i< 129; i++) ps_bias[i] = 0;

	/* Now we cycle through things and make the average */

	for(i=0; i<n; i++)
	{
		if (!wait_for_frame(&dalsa_param)) return ERROR;
	
		/* Are the parameters valid? */

		if (dalsa_param.exposure < 0)
		{
			return error(ERROR,"Set up the parameters first.");
		}

		/* Get the frame */

		if (!arm())
		{
			return error(ERROR,"Failed to arm CCD.");
		}

		message(system_window,"Frame = %d",i+1);

		if ((ps = bb_power_spec(FALSE)) == NULL)
		{
			for(i=0; i< 129; i++) ps_bias[i] = 0;
			return error(ERROR,"Failed... Bias zero.");
		}

		for(j=0; j<129; j++) ps_bias[j] += ps[j];

		free_vector(ps,0,128);

		background();
	}

	for(i=0; i<129; i++) ps_bias[i] /= n;

	werase(system_window);
	wrefresh(system_window);

	return NOERROR;

} /* power_spec_bias() */

/************************************************************************/
/* toggle_ps_bias()							*/
/*									*/
/* Toggle on/off whether to use the PS bias or not.			*/
/************************************************************************/

int toggle_ps_bias(int argc, char **argv)
{
	debias_ps = !debias_ps;

	if (debias_ps)
	{
		message(system_window,"Using power spectrum bias frame.");
	}
	else
	{
		message(system_window,"Not using power spectrum bias frame.");
	}
	
	return NOERROR;

} /* toggle_ps_bias() */

/************************************************************************/
/* set_num_pix_zero()							*/
/*									*/
/* Sets the number of pixel to force to zero in the power spectrum.	*/
/************************************************************************/

int set_num_pix_zero(int argc, char **argv)
{
	int	new_num_pix_zero;
	char	s[81];

	if (argc > 1)
	{
		sscanf(argv[1],"%d",&new_num_pix_zero);
	}
	else
	{
		sprintf(s,"%3d",num_pix_zero);
		if (quick_edit("Number of pixels to zero",s, s,NULL,INTEGER) 
			== KEY_ESC) return NOERROR;
		sscanf(s,"%d",&new_num_pix_zero);
                clean_command_line();
	}

	if (num_pix_zero < 0 || num_pix_zero > 128)
	    return error(ERROR,"Number of pixelks must be bewteen 0 and 128.");

	num_pix_zero = new_num_pix_zero;

	return NOERROR;

} /* set_num_pix_zero() */

/************************************************************************/
/* set_spec_row()							*/
/*									*/
/* Set's which row to look for the spectrum. If an argument is given	*/
/* that's what is used. If no argument it looks for the brightest	*/
/* pixel and asks if you want to use that row.				*/
/* Always checks to see if this operation is actually possible.		*/
/************************************************************************/

int set_spec_row(int argc, char **argv)
{
        DALSA_PARAM dalsa_param;
	unsigned char *bb,max,max_y,*p;
	int	size_x, size_y;
	int	i,j;
	int	row;
	char	s[81];

	/* First of all, wait for the frame */

        if (!wait_for_frame(&dalsa_param)) 
		return error(ERROR,"Failed to get Dalsa parameters");

	/* How big is the bounding box? */

	size_x = dalsa_param.lrx - dalsa_param.ulx + 1;
	size_y = dalsa_param.lry - dalsa_param.uly + 1;

	/* Analyses the command line */

	if (argc > 1)
	{
		sscanf(argv[1],"%d",&row);
	}
	else
	{
		/* Go get the bounding box */

		if ((bb = get_bb()) == NULL) 
			return error(ERROR,"Failed to get bounding box.");

		/* Find the maximum pixel */

		max_y = 0;
		for(max = 0, p = bb, j=0; j<size_y; j++)
		for(i=0; i<size_x; i++)
		{
			if (*p > max)
			{
				max = *p;
				max_y = j;
			}
			p++;
		}

		sprintf(s,"%3d",max_y);
		if (quick_edit("Row with maximum pixel",s, s,NULL,INTEGER) 
			== KEY_ESC) return NOERROR;
		sscanf(s,"%d",&row);
                clean_command_line();
	}

	/* OK, is the answer reasonable? */

	if (row < 0 || row > size_y)
	{
		return error(ERROR,"Row value outside of bounding box.");
	}

	/* Set the new value */

	spec_row = row;

	return NOERROR;

} /* set_spec_row() */

/************************************************************************/
/* data_flat()								*/
/*									*/
/* Calculates the flat fied for the data.				*/
/************************************************************************/

int data_flat(int argc, char **argv)
{
	int	i,j;
	float	*ps;
	float	n;
	char	s[81];
        DALSA_PARAM dalsa_param;
	float	total;

	/* Analyses the command line */

	if (argc > 1)
	{
		sscanf(argv[1],"%f",&n);
	}
	else
	{
		sprintf(s,"    20");
		if (quick_edit("Number of frmaes to average",s, s,NULL,INTEGER)
			== KEY_ESC) return NOERROR;
		sscanf(s,"%f",&n);
                clean_command_line();
	}

	/* First we set it to zero */

	for(i=0; i< 256; i++) data_flat_field[i] = 0;

	/* Now we cycle through things and make the average */

	flat_data = FALSE;
	for(i=0; i<n; i++)
	{
		if (!wait_for_frame(&dalsa_param)) return ERROR;
	
		/* Are the parameters valid? */

		if (dalsa_param.exposure < 0)
		{
			return error(ERROR,"Set up the parameters first.");
		}

		/* Get the frame */

		if (!arm())
		{
			return error(ERROR,"Failed to arm CCD.");
		}

		message(system_window,"Frame = %d",i+1);

		if ((ps = bb_power_spec(TRUE)) == NULL)
		{
			for(i=0; i< 256; i++) data_flat_field[i] = 0;
			return error(ERROR,"Failed... Flat field zero.");
		}

		for(j=0; j<256; j++) data_flat_field[j] += ps[j];

		free_vector(ps,0,255);

		background();
	}
	flat_data = TRUE;
	message(system_window,"Using data flat field frame.");

	total = 0;
	for(i=0; i<256; i++) 
	{
		total += data_flat_field[i];
	}
	for(i=0; i<256; i++) 
	{
		if (data_flat_field[i] <= n*spect_cutoff)
			data_flat_field[i] = 0.0;
		else
			data_flat_field[i] /= total;
	}

	werase(system_window);
	wrefresh(system_window);

	return NOERROR;

} /* data_flat() */

/************************************************************************/
/* toggle_data_flat()							*/
/*									*/
/* Toggle on/off whether to use the PS bias or not.			*/
/************************************************************************/

int toggle_data_flat(int argc, char **argv)
{
	flat_data = !flat_data;

	if (flat_data)
	{
		message(system_window,"Using data flat field frame.");
	}
	else
	{
		message(system_window,"Not using data flat field frame.");
	}
	
	return NOERROR;

} /* toggle_data_flat() */

/************************************************************************/
/* calc_data_dark()							*/
/*									*/
/* Calculates the dark for the data.					*/
/************************************************************************/

int calc_data_dark(int argc, char **argv)
{
	int	i,j;
	float	*ps;
	float	n;
	char	s[81];
	bool	old_flat_data;
        DALSA_PARAM dalsa_param;

	/* Analyses the command line */

	if (argc > 1)
	{
		sscanf(argv[1],"%f",&n);
	}
	else
	{
		sprintf(s,"    20");
		if (quick_edit("Number of frmaes to average",s, s,NULL,INTEGER)
			== KEY_ESC) return NOERROR;
		sscanf(s,"%f",&n);
                clean_command_line();
	}

	/* First we set it to zero */

	for(i=0; i< 256; i++) data_dark[i] = 0;

	/* Now we cycle through things and make the average */

	old_flat_data = flat_data;
	dark_data = FALSE;
	flat_data = FALSE;
	for(i=0; i<n; i++)
	{
		if (!wait_for_frame(&dalsa_param)) return ERROR;
	
		/* Are the parameters valid? */

		if (dalsa_param.exposure < 0)
		{
			return error(ERROR,"Set up the parameters first.");
		}

		/* Get the frame */

		if (!arm())
		{
			return error(ERROR,"Failed to arm CCD.");
		}

		message(system_window,"Frame = %d",i+1);

		if ((ps = bb_power_spec(TRUE)) == NULL)
		{
			for(i=0; i< 256; i++) data_dark[i] = 0;
			return error(ERROR,"Failed... Dark zero.");
		}

		for(j=0; j<256; j++) data_dark[j] += ps[j];

		free_vector(ps,0,255);

		background();
	}
	for(i=0; i< 256; i++) data_dark[i] /= (float)n;
	dark_data = TRUE;
	flat_data = old_flat_data;
	message(system_window,"Using data dark frame.");

	werase(system_window);
	wrefresh(system_window);

	return NOERROR;

} /* calc_data_dark() */

/************************************************************************/
/* toggle_data_dark()							*/
/*									*/
/* Toggle on/off whether to use the PS bias or not.			*/
/************************************************************************/

int toggle_data_dark(int argc, char **argv)
{
	dark_data = !dark_data;

	if (dark_data)
	{
		message(system_window,"Using data dark frame.");
	}
	else
	{
		message(system_window,"Not using data dark frame.");
	}
	
	return NOERROR;

} /* toggle_data_dark() */

/************************************************************************/
/* toggle_auto_range()							*/
/*									*/
/* Toggle on/off whether to auto range the display.			*/
/************************************************************************/

int toggle_auto_range(int argc, char **argv)
{
	auto_range_display = !auto_range_display;

	if (auto_range_display)
	{
		message(system_window,"Auto display range ON.");
	}
	else
	{
		message(system_window,"Auto display range OFF.");
	}
	
	return NOERROR;

} /* toggle_auto_range() */

/************************************************************************/
/* set_spect_cutoff()							*/
/*									*/
/* Sets the number of pixel to force to zero in the power spectrum.	*/
/************************************************************************/

int set_spect_cutoff(int argc, char **argv)
{
	float	new_spect_cutoff;
	char	s[81];

	if (argc > 1)
	{
		sscanf(argv[1],"%f",&new_spect_cutoff);
	}
	else
	{
		sprintf(s,"%5f",spect_cutoff);
		if (quick_edit("Spectrum intensity cutoff",s, s,NULL,FLOAT) 
			== KEY_ESC) return NOERROR;
		sscanf(s,"%f",&new_spect_cutoff);
                clean_command_line();
	}

	if (spect_cutoff < 0 || spect_cutoff > 254)
	    return error(ERROR,"Spectrum cutoff must be bewteen 0 and 254.");

	spect_cutoff = new_spect_cutoff;

	return NOERROR;

} /* set_spect_cutoff() */
