/************************************************************************/
/* fast_clock_comms.c							*/
/*                                                                      */
/* Routines to talk to fast clock mirror.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Dec 2004							*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../charalib/chara.h"
#include "../../include/chara_messages.h"
#include "../clock/clock.h"
#include "fast_clock.h"

/************************************************************************/
/* get_fast_clock_time()                                                */
/*                                                                      */
/* Uses one of the standard commands to get a the        		*/
/* fast clock time. Returns 0 on succes and -1 on failure.              */
/************************************************************************/
                                                                                
int get_fast_clock_time(fast_clock_time_struct *now)
{
        /* Try sending the command */
                                                                                
        if (!send_rt_command(GET_TIME,0)) return -1;
                                                                                
        /* Now get the register value */
                                                                                
        if (read_rt_data(now, sizeof(*now)) != sizeof(*now))
        {
                return -1;
        }
                                                                                
        return 0;
                                                                                
} /* get_fast_clock_time() */

/************************************************************************/
/* fast_clock_calibrate()						*/
/*									*/
/* Sends the command to the real time side to calibrate the fast clock.	*/
/* The fast clock will count ticks for the requested period and work	*/
/* out a correct rate for the clock.					*/
/* Returns -1 on bad seconds parameter, -2 on command failure		*/
/* -3 on user abort and -4 timeout and the new period if things go well.*/
/************************************************************************/

int fast_clock_calibrate(int seconds)
{
	time_t	last_now, now, stop;
	int	stage, period;
	struct smessage mess;
	int	dt;
	bool key_hit;
	int	key = 0;
	char	socket_command[80];

	/* Is the time reasonable? */

	if (seconds <= 0 || seconds > 255) return -1;

        /* Try sending the command */
                                                                                
        if (!send_rt_command(CALIBRATE,(unsigned char)seconds))
		return -2;

	/* When should this be finished? */

	stop = time(0) + seconds + 1;
	last_now = 0;
	clear_client_signal(FAST_CLOCK_SIGNAL_STOP);
	while((now = time(0)) < stop)
	{
		background();
		process_message_socket();
		process_command_socket();

		if ((stage = fast_clock_calibrate_get_stage()) < 0)
		{
			werase(system_window);
			wrefresh(system_window);
			if (client_socket != -1)
			{
				dt = 0;
				mess.type = FAST_CLOCK_CALIBRATE_INFO;
				mess.length= sizeof(dt);
				mess.data = (unsigned char *)&dt;
				ui_send_message(client_socket, &mess);
			}
			return -2;
		}

		if (stage >= CAL_STAGE_DONE) break;

		key_hit = FALSE;
		if (kbhit()) 
		{
			key = get_command();
			key_hit = TRUE;
		}
		else if (client_signal_set(FAST_CLOCK_SIGNAL_STOP))
		{
			clear_client_signal(FAST_CLOCK_SIGNAL_STOP);
			key = KEY_ESC;
			key_hit = TRUE;
		}
		else if (active_socket != -1 && char_waiting(active_socket))
                {
                    if (socket_scan(active_socket,"%s",socket_command) <= EOF ||                        *socket_command == 'q')
                    {
			key = KEY_ESC;
			key_hit = TRUE;
                    }
                    else
                    {
                        key = *socket_command;
                    }
                    key_hit = TRUE;
                                                                                
                    /* Flush any other socket activity */
                                                                                
                    flush_sockets();
                }

		if (key_hit && key == KEY_ESC) 
		{
			werase(system_window);
			wrefresh(system_window);
			if (client_socket != -1)
			{
				dt = 0;
				mess.type = FAST_CLOCK_CALIBRATE_INFO;
				mess.length= sizeof(dt);
				mess.data = (unsigned char *)&dt;
				ui_send_message(client_socket, &mess);
			}
			return -3;
		}

		if (now > last_now)
		{
			last_now = now;
			message(system_window, 
				"Fast clock calibration in progress %d.",
				dt = stop-now);

			if (client_socket != -1)
			{
				mess.type = FAST_CLOCK_CALIBRATE_INFO;
				mess.length= sizeof(dt);
				mess.data = (unsigned char *)&dt;
				ui_send_message(client_socket, &mess);
			}
		}
	}
	werase(system_window);
	wrefresh(system_window);
	
	if (client_socket != -1)
	{
		dt = 0;
		mess.type = FAST_CLOCK_CALIBRATE_INFO;
		mess.length= sizeof(dt);
		mess.data = (unsigned char *)&dt;
		ui_send_message(client_socket, &mess);
	}

	/* Did we time out ? */

	if (now >= stop) return -4;
		
	/* Get the new period and send it back down */

	if ((period = fast_clock_get_period()) < 0) return -2;
	if (fast_clock_set_period(period) != 0) return -2;

	return period;

} /* fast_clock_calibrate() */

/************************************************************************/
/* fast_clock_calibrate_stage()						*/
/*									*/
/* Asks the real time side what the current calibration stage is.	*/
/*  Returns that or negative number on failure.				*/
/************************************************************************/

int fast_clock_calibrate_get_stage(void)
{
	int	stage;

        if (!send_rt_command(CALIBRATE_GET_STAGE,0)) return -1;

        if (read_rt_data(&stage, sizeof(stage)) != sizeof(stage))
        {
                return -2;
        }
                                                                                
	return stage;

} /* fast_clock_calibrate_stage() */

/************************************************************************/
/* fast_clock_get_period()						*/
/*									*/
/* Asks the real time side what the current period is. Returns that	*/
/* or  negative number on failure.					*/
/************************************************************************/

int fast_clock_get_period(void)
{
	int	period;

        if (!send_rt_command(GET_PERIOD,0)) return -1;

        if (read_rt_data(&period, sizeof(period)) != sizeof(period))
        {
                return -2;
        }
                                                                                
	return period;

} /* fast_clock_get_period() */

/************************************************************************/
/* fast_clock_set_period()						*/
/*									*/
/* Send command to set the period of the fast clock.			*/
/* Returns 0 if all goes well.						*/
/************************************************************************/

int fast_clock_set_period(int period)
{
        if (write_rt_data(&period,sizeof(period)) != sizeof(period)) return -1;

        if (!send_rt_command(SET_PERIOD, 0)) return -2;

	return 0;

} /* fast_clock_set_period() */

/************************************************************************/
/* fast_clock_get_rate()						*/
/*									*/
/* Asks the real time side what the current rate is. Returns that	*/
/* or  negative number on failure.					*/
/************************************************************************/

int fast_clock_get_rate(void)
{
	int	rate;

        if (!send_rt_command(GET_RATE,0)) return -1;

        if (read_rt_data(&rate, sizeof(rate)) != sizeof(rate))
        {
                return -2;
        }
                                                                                
	return rate;

} /* fast_clock_get_rate() */

/************************************************************************/
/* fast_clock_set_rate()						*/
/*									*/
/* Send command to set the rate of the fast clock.			*/
/* Returns 0 if all goes well.						*/
/************************************************************************/

int fast_clock_set_rate(int rate)
{
	/* Is this reasonable? */

	if (rate <= 1000 > 20000) return -1;

	/* Do it. */

        if (write_rt_data(&rate,sizeof(rate)) != sizeof(rate)) return -2;
        if (!send_rt_command(SET_RATE,0)) return -3;

	/* And reset all the globals */

	FAST_CLOCK_RATE = rate;
	FAST_CLOCK_TICKS_PER_CHARA_TICK = (int)((float)rate/
                                (float)TICKS_PER_SECOND + 0.5);
        FAST_CLOCK_TICKS_PER_SECOND = rate;
        FAST_CLOCK_TICKS_PER_MINUTE = FAST_CLOCK_TICKS_PER_SECOND*60L;
        FAST_CLOCK_TICKS_PER_HOUR = FAST_CLOCK_TICKS_PER_MINUTE*60L;
        FAST_CLOCK_TICKS_PER_DAY = FAST_CLOCK_TICKS_PER_HOUR*24L;

	return 0;

} /* fast_clock_set_rate() */
