/************************************************************************/
/* fast_clock_control.c							*/
/*                                                                      */
/* Routines to control fast clock mirror.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Dec 2004							*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../charalib/chara.h"
#include "../../include/chara_messages.h"
#include "../clock/clock.h"
#include "fast_clock.h"

CHARA_TIME FAST_CLOCK_RATE = DEFAULT_FAST_CLOCK_RATE;
CHARA_TIME FAST_CLOCK_TICKS_PER_CHARA_TICK = DEFAULT_FAST_CLOCK_RATE/
						TICKS_PER_SECOND;
CHARA_TIME FAST_CLOCK_TICKS_PER_SECOND = DEFAULT_FAST_CLOCK_RATE;
CHARA_TIME FAST_CLOCK_TICKS_PER_MINUTE = DEFAULT_FAST_CLOCK_RATE*60L;
CHARA_TIME FAST_CLOCK_TICKS_PER_HOUR = DEFAULT_FAST_CLOCK_RATE*3600L;
CHARA_TIME FAST_CLOCK_TICKS_PER_DAY = DEFAULT_FAST_CLOCK_RATE*3600L*24L;

/************************************************************************/
/* fast_clock_clear_lost_counts()					*/
/*									*/
/* Sends command to RT side to clear fast clock clock lost counts.	*/
/************************************************************************/

int fast_clock_clear_lost_counts(int argc, char **argv)
{
	if (!send_rt_command(CLEAR_LOST_TICKS,0)) return -1;

	return NOERROR;

} /* fast_clock_clear_lost_counts() */

/************************************************************************/
/* call_fast_clock_calibrate()						*/
/*									*/
/* User callable version of fast_clock_calibrate().			*/
/************************************************************************/

int call_fast_clock_calibrate(int argc, char **argv)
{
	char	s[80];
	int	seconds, period, old_period;

	if (argc > 1)
	{
		sscanf(argv[1],"%d", &seconds);
	}
	else
	{
		strcpy(s,"         60");
                if (quick_edit("Length of time for caibration (seconds)",s,s,
			NULL,INTEGER) == KEY_ESC)
                {
                        return NOERROR;
                }
                sscanf(s,"%d",&seconds);
	}

	if ((old_period = fast_clock_get_period()) < 0)
		return error(ERROR,"Failed to get current period.");

	switch(period = fast_clock_calibrate(seconds))
	{
		case -1: return error(ERROR,
		"Calibration failed: Invalid number of seconds (0-255).");
			 break;

		case -2: return error(ERROR,
				"Calibration failed: Failed to send command.");
			 break;

		case -3: return error(ERROR,
				"Calibration failed: Aborted by user.");
			 break;

		case -4: return error(ERROR,
				"Calibration failed: Timed out.");
			 break;

		default: break;
	}

	message(system_window,"Fast clock period changed from %d to %d.",
			old_period, period);

	return NOERROR;

} /* call_fast_clock_calibrate() */

/************************************************************************/
/* call_fast_clock_set_period()						*/
/*									*/
/* User callable version of fast_clock_set_period().			*/
/************************************************************************/

int call_fast_clock_set_period(int argc, char **argv)
{
	char	s[80];
	int	period;

	if (argc > 1)
	{
		sscanf(argv[1],"%d", &period);
	}
	else
	{
		if ((period = fast_clock_get_period()) < 0)
			return error(ERROR,"Failed to get current period.");
		sprintf(s,"     %d",period);
                if (quick_edit("Fast clock period",s,s,NULL,INTEGER) == KEY_ESC)
                {
                        return NOERROR;
                }
                sscanf(s,"%d",&period);
	}

	if (fast_clock_set_period(period) != 0)
		return error(ERROR,"Failed to set new period.");

	return NOERROR;

} /* call_fast_clock_set_period() */

/************************************************************************/
/* call_fast_clock_set_rate()						*/
/*									*/
/* User callable version of fast_clock_set_rate().			*/
/************************************************************************/

int call_fast_clock_set_rate(int argc, char **argv)
{
	char	s[80];
	int	rate;

	if (argc > 1)
	{
		sscanf(argv[1],"%d", &rate);
	}
	else
	{
		if ((rate = fast_clock_get_rate()) < 0)
			return error(ERROR,"Failed to get current rate.");
		sprintf(s,"     %d",rate);
                if (quick_edit("Fast clock rate (Hz)",
			s,s,NULL,INTEGER) == KEY_ESC) return NOERROR;
                sscanf(s,"%d",&rate);
	}

	if (rate <= 1000 || rate > 20000)
		return error(ERROR,"Request rate out of range 1000 to 20000.");

	if (fast_clock_set_rate(rate) != 0)
		return error(ERROR,"Failed to set new rate.");

	return NOERROR;

} /* call_fast_clock_set_rate() */

/************************************************************************/
/* fast_clock_sync()							*/
/*									*/
/* User callable version of fast_clock_sync().				*/
/************************************************************************/

int fast_clock_sync(int argc, char **argv)
{
	int	rate;

	/* Get the current rate */

	if ((rate = fast_clock_get_rate()) < 0)
			return error(ERROR,"Failed to get current rate.");

	/* Reset the rate to the same rate. */

	if (fast_clock_set_rate(rate) != 0)
		return error(ERROR,"Failed to set new rate.");

	return NOERROR;

} /* fast_clock_sync() */
