/************************************************************************/
/* fast_clock_messages.c						*/
/*                                                                      */
/* Routines to control fast clock mirror messages.			*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Dec 2004							*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../charalib/chara.h"
#include "../../include/chara_messages.h"
#include "fast_clock.h"

/************************************************************************/
/* setup_standard_fast_clock_messages()					*/
/*									*/
/* Adds standard message hanlders for teh fast clock mirror.		*/
/************************************************************************/

void setup_standard_fast_clock_messages(void)
{
        /* Initialize the message jobs */
 
        ui_add_message_job(FAST_CLOCK_CLEAR_LOST_TICKS,
		message_fast_clock_clear_lost_counts);
        ui_add_message_job(FAST_CLOCK_CALIBRATE,
		message_fast_clock_calibrate);
        ui_add_message_job(FAST_CLOCK_SET_PERIOD,
		message_fast_clock_set_period);
        ui_add_message_job(FAST_CLOCK_SET_RATE,
		message_fast_clock_set_rate);
        ui_add_message_job(FAST_CLOCK_GET_RATE,
		message_fast_clock_get_rate);
        ui_add_message_job(FAST_CLOCK_TIME,
		message_fast_clock_time);

} /* setup_standard_fast_clock_messages() */

/************************************************************************/
/* message_fast_clock_clear_lost_counts()				*/
/************************************************************************/

int message_fast_clock_clear_lost_counts(struct smessage *message)
{
	if (message->length != 0)
        {
                return error(ERROR,
		"Wrong data for FAST_CLOCK_CLEAR_LOST_TICKS message.");
        }

	return fast_clock_clear_lost_counts(0, NULL);

} /* message_fast_clock_clear_lost_counts() */

/************************************************************************/
/* message_fast_clock_calibrate()					*/
/************************************************************************/

int message_fast_clock_calibrate(struct smessage *message)
{
	int	*seconds;
	int	old_period, period;

	if (message->length != sizeof(*seconds))
        {
                return error(ERROR,
		"Wrong data for FAST_CLOCK_CALIBRATE message.");
        }
 
        seconds = (int *)message->data;

	if ((old_period = fast_clock_get_period()) < 0)
                return error(ERROR,"Failed to get current period.");
 
        switch(period = fast_clock_calibrate(*seconds))
        {
		case -1: return error(ERROR,
		"Calibration failed: Invalid number of seconds (0-255).");
			 break;

		case -2: return error(ERROR,
				"Calibration failed: Failed to send command.");
			 break;

		case -3: return error(ERROR,
				"Calibration failed: Aborted by user.");
			 break;

		case -4: return error(ERROR,
				"Calibration failed: Timed out.");
			 break;

		default: break;
        }
 
        ui_print_message(client_socket, 
			"Fast clock period changed from %d to %d.",
                        old_period, period);

	return NOERROR;

} /* message_fast_clock_calibrate() */

/************************************************************************/
/* message_fast_clock_set_period()					*/
/************************************************************************/

int message_fast_clock_set_period(struct smessage *message)
{
	int	*period;

	if (message->length != sizeof(*period))
        {
                return error(ERROR,
		"Wrong data for FAST_CLOCK_SET_PERIOD message.");
        }
 
        period = (int *)message->data;

	if (fast_clock_set_period(*period) != 0)
                return error(ERROR,"Failed to set new period.");

	return NOERROR;

} /* message_fast_clock_set_period() */

/************************************************************************/
/* message_fast_clock_set_rate()					*/
/*									*/
/* Message to set fast clock rate.					*/
/************************************************************************/

int message_fast_clock_set_rate(struct smessage *message)
{
	int	*rate;

	if (message->length != sizeof(*rate))
        {
                return error(ERROR,
		"Wrong data for FAST_CLOCK_SET_RATE message.");
        }
 
        rate = (int *)message->data;

	if (*rate <= 1000 || *rate > 20000)
                return error(ERROR,"Request rate out of range 1000 to 20000.");
 
        if (fast_clock_set_rate(*rate) != 0)
                return error(ERROR,"Failed to set new rate.");

	return NOERROR;

} /* message_fast_clock_set_rate() */

/************************************************************************/
/* message_fast_clock_get_rate()					*/
/*									*/
/* Message to get fast clock rate.					*/
/************************************************************************/

int message_fast_clock_get_rate(struct smessage *message)
{
	int	rate;
	struct smessage mess;

	if (message->length != 0)
        {
                return error(ERROR,
		"Wrong data for FAST_CLOCK_GET_RATE message.");
        }
 
        if ((rate = fast_clock_get_rate()) <= 0)
                return error(ERROR,"Failed to get current rate.");

	mess.type = FAST_CLOCK_SET_RATE;
        mess.length= sizeof(rate);
        mess.data = (unsigned char *)&rate;
        ui_send_message(client_socket, &mess);

	return NOERROR;

} /* message_fast_clock_get_rate() */

/************************************************************************/
/* message_fast_clock_time()						*/
/*									*/
/* Message to get fast clock time.					*/
/************************************************************************/

int message_fast_clock_time(struct smessage *message)
{
	fast_clock_time_struct	now;
	struct smessage mess;

	if (message->length != 0)
        {
                return error(ERROR,
		"Wrong data for FAST_CLOCK_TIME message.");
        }
 
        if (get_fast_clock_time(&now) != 0)
                return error(ERROR,"Failed to get current time.");

	mess.type = FAST_CLOCK_TIME;
        mess.length= sizeof(now);
        mess.data = (unsigned char *)&now;
        ui_send_message(client_socket, &mess);

	return NOERROR;

} /* message_fast_clock_time() */

/************************************************************************/
/* message_fast_clock_sync()						*/
/*									*/
/* Message to force a fast clock sync to chara clock.			*/
/************************************************************************/

int message_fast_clock_sync(struct smessage *message)
{
	if (message->length != 0)
        {
                return error(ERROR,"Wrong data for FAST_CLOCK_SYNC message.");
        }
 
	return fast_clock_sync(0, NULL);

} /* message_fast_clock_sync() */
