/************************************************************************/
/* extra_alloc.c							*/
/*                                                                      */
/* Extra memory allocation and deallocation routines left out from NRC.	*/
/* Also contains some useful comparison functions for qsort.		*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar (But largly modified NRC routines)      */
/* Date   : 31st July 1996                                              */
/************************************************************************/

#include <stdlib.h>
#include "nrc.h"

#define NR_END 1
#define FREE_ARG char*

/************************************************************************/
/* fcvector()								*/
/*									*/
/* Allocate an fcomplex vector with subscript range v[nl..nh]  		*/
/************************************************************************/

fcomplex *fcvector(long nl, long nh)
{
  fcomplex *v;

  v=(struct FCOMPLEX*)malloc((size_t) ((nh-nl+1+NR_END)*
		sizeof(struct FCOMPLEX)));
  if (!v) nrerror("allocation failure in fcvector()");
  return v-nl+NR_END;
}

/************************************************************************/
/* fcmatrix()								*/
/*									*/
/* Allocate an fcomplex matrix with subscript m[nrl..nrh][ncl..nch] 	*/
/************************************************************************/

fcomplex **fcmatrix(long nrl, long nrh, long ncl, long nch)
{
  long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  fcomplex **m;

  /* allocate pointers to rows */

  m=(struct FCOMPLEX **) malloc((size_t)((nrow+NR_END)*
		sizeof(struct FCOMPLEX*)));
  if (!m) nrerror("allocation failure 1 in fcmatrix()");
  m += NR_END;
  m -= nrl;

  /* allocate rows and set pointers to them */
  m[nrl]=(struct FCOMPLEX *) malloc((size_t)((nrow*ncol+NR_END)*
		sizeof(struct FCOMPLEX)));
  if (!m[nrl]) nrerror("allocation failure 2 in fcmatrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

  /* return pointer to array of pointers to rows */
  return m;
}

/************************************************************************/
/* dcvector()                                                           */
/*                                                                      */
/* Allocate an dcomplex vector with subscript range v[nl..nh]           */
/************************************************************************/

dcomplex *dcvector(long nl, long nh)
{
  dcomplex *v;

  v=(struct DCOMPLEX*)malloc((size_t) ((nh-nl+1+NR_END)*
                sizeof(struct DCOMPLEX)));
  if (!v) nrerror("allocation failure in dcvector()");
  return v-nl+NR_END;
}

/************************************************************************/
/* dcmatrix()                                                           */
/*                                                                      */
/* Allocate an dcomplex matrix with subscript m[nrl..nrh][ncl..nch]     */
/************************************************************************/

dcomplex **dcmatrix(long nrl, long nrh, long ncl, long nch)
{
  long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  dcomplex **m;

  /* allocate pointers to rows */

  m=(struct DCOMPLEX **) malloc((size_t)((nrow+NR_END)*
                sizeof(struct DCOMPLEX*)));
  if (!m) nrerror("allocation failure 1 in dcmatrix()");
  m += NR_END;
  m -= nrl;

  /* allocate rows and set pointers to them */
  m[nrl]=(struct DCOMPLEX *) malloc((size_t)((nrow*ncol+NR_END)*
                sizeof(struct DCOMPLEX)));
  if (!m[nrl]) nrerror("allocation failure 2 in dcmatrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

  /* return pointer to array of pointers to rows */
  return m;
}

/************************************************************************/
/* free_fcvector()							*/
/*									*/
/* Free an fcomplex vector allocated with fcvector() 			*/
/************************************************************************/

void free_fcvector(fcomplex *v, long nl, long nh)
{
	free((FREE_ARG) (v+nl-NR_END));
}

/************************************************************************/
/* free_fcmatrix()							*/
/*									*/
/* Free an fcomplex matrix allocated by fcmatrix() 			*/
/************************************************************************/

void free_fcmatrix(fcomplex **m, long nrl, long nrh, long ncl, long nch)
{
  free((FREE_ARG) (m[nrl]+ncl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
}

/************************************************************************/
/* free_dcvector()                                                      */
/*                                                                      */
/* Free an dcomplex vector allocated with dcvector()                    */
/************************************************************************/

void free_dcvector(dcomplex *v, long nl, long nh)
{
        free((FREE_ARG) (v+nl-NR_END));
}

/************************************************************************/
/* free_dcmatrix()                                                      */
/*                                                                      */
/* Free an dcomplex matrix allocated by dcmatrix()                      */
/************************************************************************/

void free_dcmatrix(dcomplex **m, long nrl, long nrh, long ncl, long nch)
{
  free((FREE_ARG) (m[nrl]+ncl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
}

/************************************************************************/
/* comapr_long_int()							*/
/*									*/
/* qsort compar function for long integers.				*/
/************************************************************************/

int compar_long_int(const void *a, const void *b)
{
	long int *first, *second;

	first = (long int *)a;
	second = (long int *)b;

	return (int)(*first - *second);

} /* compar_long_int() */

/************************************************************************/
/* comapr_int()								*/
/*									*/
/* qsort compar function for integers.					*/
/************************************************************************/

int compar_int(const void *a, const void *b)
{
	int *first, *second;

	first = (int *)a;
	second = (int *)b;

	return (int)(*first - *second);

} /* compar_int() */

/************************************************************************/
/* comapr_float()							*/
/*									*/
/* qsort compar function for float.					*/
/************************************************************************/

float compar_float(const void *a, const void *b)
{
	float *first, *second;

	first = (float *)a;
	second = (float *)b;

	if (*first < *second) 
		return -1;
	else if (*first > *second)
		return 1;
	else return 0;

} /* compar_float() */

/************************************************************************/
/* comapr_double()							*/
/*									*/
/* qsort compar function for double.					*/
/************************************************************************/

double compar_double(const void *a, const void *b)
{
	double *first, *second;

	first = (double *)a;
	second = (double *)b;

	if (*first < *second) 
		return -1;
	else if (*first > *second)
		return 1;
	else return 0;

} /* compar_double() */
