/************************************************************************/
/* simpson.c								*/
/*                                                                      */
/* Routines to do a Simpson's rule integration. These routines largely	*/
/* taken from Peter Lawson (at Cambridge University England) but to	*/
/* a great extent modified by myself.					*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Peter Lawson & Theo ten Brummelaar                          */
/* Date   : 31st July 1996                                              */
/************************************************************************/

#include "nrc.h"

/************************************************************************/
/* simpson()                                                            */
/*                                                                      */
/* An extended Simpson Rule as shown on p.116 of Num. Rec. Eq. 4.1.12   */
/************************************************************************/

float simpson(int num, float *y, float dx)
{
        float sum;
        int i;

        if (num < 5)
        {
                sum = 0.0;
                for(i=0;i<=num;i++) sum += y[i];
        }
        else
        {
                sum = (5.0/12.0) * y[0] + (13.0/12.0) * y[1];

                for (i=2; i<num-2; i++)
                {
                        sum += y[i];
                }

                sum += (13.0/12.0) * y[num-2];
                sum += (5.0/12.0)  * y[num-1];
        }

        sum *= dx;

        return sum;

} /* simpson() */

/************************************************************************/
/* do_simpson()								*/
/*									*/
/* A wrapper for the simpson() function, where you give the function	*/
/* the limits of integration and the number of calculations along the	*/
/* way.									*/
/************************************************************************/

float do_simpson(float (*funct)(float), float a, float b, int num)
{
	float   dx;
	float	*y;
	float	result;
	int	i;

	/* Work out range of values */

	dx = (b - a)/(num-1.0);

	/* Allocate memory */

	y = vector(0,num);

	/* Calculate the values */

	for(i=0; i<num; i++)
	{
		y[i] = (*funct)(a + dx * i);
	}

	/* Do the integration */

	result = simpson(num, y, dx);
	
	/* Clean memory and go */

	free_vector(y, 0, num);

	return result;

} /* do_simpson() */

/************************************************************************/
/* dsimpson()                                                            */
/*                                                                      */
/* An extended Simpson Rule as shown on p.116 of Num. Rec. Eq. 4.1.12   */
/* Double precision version.						*/
/************************************************************************/

double dsimpson(int num, double *y, double dx)
{
        double sum;
        int i;

        if (num < 5)
        {
                sum = 0.0;
                for(i=0;i<=num;i++) sum += y[i];
        }
        else
        {
                sum = (5.0/12.0) * y[0] + (13.0/12.0) * y[1];

                for (i=2; i<num-2; i++)
                {
                        sum += y[i];
                }

                sum += (13.0/12.0) * y[num-2];
                sum += (5.0/12.0)  * y[num-1];
        }

        sum *= dx;

        return sum;

} /* dsimpson() */

/************************************************************************/
/* do_dsimpson()							*/
/*									*/
/* A wrapper for the dsimpson() function, where you give the function	*/
/* the limits of integration and the number of calculations along the	*/
/* way.									*/
/************************************************************************/

double do_dsimpson(double (*funct)(double), double a, double b, int num)
{
	double   dx;
	double	*y;
	double	result;
	int	i;

	/* Work out range of values */

	dx = (b - a)/(num-1.0);

	/* Allocate memory */

	y = dvector(0,num);

	/* Calculate the values */

	for(i=0; i<num; i++)
	{
		y[i] = (*funct)(a + dx * i);
	}

	/* Do the integration */

	result = dsimpson(num, y, dx);
	
	/* Clean memory and go */

	free_dvector(y, 0, num);

	return result;

} /* do_simpson() */
