/************************************************************************/
/* build_airy.c								*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : June 1998                                                   */
/************************************************************************/

#include <math.h>
#include "../nrc/nrc.h"
#include "nsimpleX.h"

/************************************************************************/
/* build_airy()								*/
/*									*/
/* Calculates an Airy function for a given wavelength and pixel		*/
/* scaling.  It then inserts that into a matrix and returns it.		*/
/*									*/
/* build_airy() gets its filter wavelength value from a lookup table	*/
/* whose location is defined by means of a shell variable,		*/
/* NSIMPLEX_FILTER.  The lookup table contains one line per filter,	*/
/* three fields per line.  The first field is the text description of	*/
/* the filter, the second field the center wavelength in microns, and	*/
/* the third field the width of the filter bandpass.  The field		*/
/* separator is two commas in a row -- ",,".  Comments are denoted by	*/
/* use of the hash symbol, "#".						*/
/************************************************************************/

float **build_airy(int xpix, int ypix, 
	   float scale, char *filter, float tel_diam, float tel_obsc)
{
	float	**airy;
	int	i,j;
	double	lambda;
	double	dx,dy,dist;
	double   v;
	double	mid_x,mid_y;
	double	temp;
	double  bndpss;

	/* Which wavelength are we looking it? */
#ifdef OLD_WAY

	if (*filter == 'I')
	{
		lambda = 0.90e-6;
	}
	else if (*filter == 'R')
	{
		lambda = 0.70e-6;
	}
	else if (*filter == 'V')
	{
		lambda = 0.55e-6;
	}
	else if (*filter == 'B')
	{
		lambda = 0.44e-6;
	}
	else if (*filter == 'U')
	{
		lambda = 0.365e-6;
	}
	else
	{
		/* Try and read a number out of the string */

		if (sscanf(filter,"%lf",&lambda) != 1)
		{
			fprintf(stderr,
			"calculate_strehl(): Unknown filter %s, asumming I.\n",
				filter);
			lambda = 0.90e-6;
		}
	}
#else

	lambda = get_filter_value(filter,&bndpss);
#endif

	/* First we construct an Airy disk of the right size and shape */

	airy = matrix(1,xpix,1,ypix);
	temp = M_PI/lambda * tel_diam;
	mid_x = (float)xpix/2.0;
	mid_y = (float)ypix/2.0;
	scale /= RADIANS_PER_ARCSEC;
#ifdef DEBUG
	printf("Lambda = %f um\n",lambda*1e6);
	printf("temp = %f\n",temp);
	printf("mid_x = %f\n",mid_x);
	printf("mid_y = %f\n",mid_y);
	printf("scale = %f\n",scale);
	fflush(stdout);
#endif

	for(i=1; i<=xpix; i++)
	{
		for(j=1; j<=ypix; j++)
		{
			/* Find distance in radians from center */

			dx = mid_x - (float)i;
			dy = mid_y - (float)j;
			dist = dx*dx + dy*dy;
			dist = sqrt(dist)/scale;

			v = temp * dist;

			if (tel_obsc > 0.0)
			{
				airy[i][j] = airyobsc_intensity(v, tel_obsc); 
			}
			else
			{
				airy[i][j] = airy_intensity(v); 
			}
#ifdef DEBUG2
			printf("airy[%d][%d] = %f, dist = %f\n",
				i,j,airy[i][j],dist);
			fflush(stdout);
#endif
		}
	}

	/* That is all */

	return airy;

} /* build_airy() */

/************************************************************************/
/* airy_intensity() 							*/
/*									*/
/* Returns the Airy disk intensity.					*/
/************************************************************************/

float	airy_intensity(float v)
{
	float	airy;

	if (v != 0.0)
		airy = 2.0 * bessj1(v) / v;
	else
		airy = 1.0;

	airy *= airy;

	return airy;

} /* airy_intensity() */
	
/************************************************************************/
/* airyobsc_intensity() 						*/
/*									*/
/* Returns the obscurred Airy disk intensity.				*/
/************************************************************************/

float	airyobsc_intensity(float v, float e)
{
	float	airy;
	float	airyv;
	float	airyve;

	airyv = sqrt(airy_intensity(v));
	airyve = sqrt(airy_intensity(v*e));

	airy = (airyv - e * e * airyve)/(1.0-e*e);
	airy *= airy;

	return airy;

} /* airyobsc_intensity() */
