/************************************************************************/
/* calculate_fft.c 							*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : June 1998                                                   */
/************************************************************************/

#include "../nrc/nrc.h"
#include "nsimpleX.h"

/************************************************************************/
/* calculate_fft()							*/
/*									*/
/* Creates the FFT of an image. 	 				*/
/* The fill parameter specifies what to place outside the aperture, 	*/
/* and can take the values MEAN, MIN, MAX or ZERO.			*/
/* Returned matrix is fcomplex fft[1..npix][1..npix].			*/
/* Parameter imbedp2 sets the size of the matrix used to perform FFT.	*/
/* It is the power of 2 to use, ie 4 uses a 16x16 matrix.		*/
/************************************************************************/

fcomplex **calculate_fft(float **data, int nx, int ny, int imbedp2, int fill)
{
	int	imbed;
	int	imbed2;
	float	*aperture;
	float	*ap;
	int	i,j;
	unsigned int *nn;
	fcomplex **fft;
	int	ix,iy,offset_x,offset_y;
	float	fill_value;
	int	pos_x,pos_y;

	/* Get real values for npix and imbed */

	imbed = 1<<imbedp2;
	imbed2 = imbed >> 1;

	if (imbed > 1024)
	{
		fprintf(stderr,
"calculate_fft(): Using VERY large arrays for image...  this will take forever.\n");
	}

	if (nx > imbed || ny > imbed)
	{
		fprintf(stderr,
			"calculate_fft(): Number of pixels greater than array.\n");
		return NULL;
	}

	/* Allocate memory */

        aperture = vector(1,2*imbed*imbed);
	fft = fcmatrix(1,nx,1,ny);
	nn = lvector(1,2);

	/* Calculate offset */

	offset_x = (imbed - nx)>>1;
	offset_y = (imbed - ny)>>1;

	/* Calculate fill value */

	switch(fill)
	{
		case ZERO: fill_value = 0.0;
			   break;

		case MIN: fill_value = 
				min_image_intensity(data,nx,ny,&pos_x,&pos_y);
			  break;

		case MAX: fill_value = 
				max_image_intensity(data,nx,ny,&pos_x,&pos_y);
			  break;

		case MEAN: fill_value = total_image(data, nx, ny)/
					((float)nx * (float)ny);
			  break;
	
		default: fprintf(stderr,"calculate_fft(): Unknown fill factor.\n");
			  exit(-3);
	}

	/* Generate the aperture field */

	ap = &(aperture[1]);
        for(i=1;i<=imbed;i++)
        {
                for(j=1;j<=imbed;j++)
                {
			if ((ix = i-offset_x) < 1 || ix > nx ||
			    (iy = j-offset_y) < 1 || iy > ny)
			{
				*ap++ = fill_value;
				*ap++ = 0.0;
			}
			else
			{
				*ap++ = data[ix][iy];
				*ap++ = 0.0;
			}
		}
	}

	/* Set up for and do FFT */

	nn[1] = imbed;
	nn[2] = imbed;

 	fourn((float*)aperture, nn, 2, 1); 

	/* Now pull out the fft .... */

	ap = &(aperture[1]);

        for(i=1; i<=imbed; i++)
        {
                for(j=1; j<=imbed ; j++)
                {
			if (i > 0 && i <= nx && j > 0 && j <= ny)
			{
				fft[i][j].r = *ap;
				fft[i][j].i = *(ap+1);
			}
			ap++;
			ap++;
                }
        }

	/* Clean up memory */

	free_vector(aperture,1,2*imbed*imbed);
	free_lvector(nn,1,2);

	/* That is all */
	
	return fft;

} /* calculate_fft() */

/************************************************************************/
/* calculate_ifft()							*/
/*									*/
/* Creates the IFFT of an image. Result is assumed to be real.  	*/
/* Returned matrix is float data[1..npix][1..npix].			*/
/* Parameter imbedp2 sets the size of the matrix used to perform iFFT.	*/
/* It is the power of 2 to use, ie 4 uses a 16x16 matrix.		*/
/************************************************************************/

float **calculate_ifft(fcomplex **fft, int nx, int ny, int imbedp2)
{
	int	imbed;
	int	imbed2;
	float	*aperture;
	float	*ap;
	int	i,j;
	unsigned long *nn;
	float **data;
	int	ix,iy,offset_x,offset_y;
	int	pos_x,pos_y;
	float	divider;

	/* Get real values for npix and imbed */

	imbed = 1<<imbedp2;
	imbed2 = imbed >> 1;

	if (imbed > 1024)
	{
		fprintf(stderr,
"calculate_ifft(): Using VERY large arrays for image...  this will take forever.\n");
	}

	if (nx > imbed || ny > imbed)
	{
		fprintf(stderr,
			"calculate_ifft(): Number of pixels greater than array.\n");
		return NULL;
	}

	/* Allocate memory */

        aperture = vector(1,2*imbed*imbed);
	data = matrix(1,nx,1,ny);
	nn = lvector(1,2);

	/* Calculate offset */

	offset_x = (imbed - nx)>>1;
	offset_y = (imbed - ny)>>1;

	/* Generate the aperture field */

	ap = &(aperture[1]);
        for(i=1;i<=imbed;i++)
        {
                for(j=1;j<=imbed;j++)
                {
			if (i > 0 && i <= nx && j > 0 && j <= ny)
			{
				*ap++ = fft[i][j].r;
				*ap++ = fft[i][j].i;
			}
			else
			{
				*ap++ = 0.0;
				*ap++ = 0.0;
			}
		}
	}

	/* Set up for and do FFT */

	nn[1] = imbed;
	nn[2] = imbed;

 	fourn((float*)aperture, nn, 2, -1); 

	/* Now pull out the ifft .... */

	ap = &(aperture[1]);
	divider = (float)imbed * (float)imbed;
        for(i=1; i<=imbed; i++)
        {
                for(j=1; j<=imbed ; j++)
                {
			if ((ix = i-offset_x) > 0 && ix <= nx &&
			    (iy = j-offset_y) > 0 && iy <= ny)
			{
				data[ix][iy] = sqrt(*ap * *ap + 
						*(ap+1) * *(ap+1))/divider;
			}
			ap++;
			ap++;
                }
        }

	/* Clean up memory */

	free_vector(aperture,1,2*imbed*imbed);
	free_lvector(nn,1,2);

	/* That is all */
	
	return data;

} /* calculate_ifft() */
