/************************************************************************/
/* fitgaussian.c	 						*/
/*                                                                      */
/* Routines to fit gaussians to things.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Feb 2004                                                    */
/************************************************************************/

#include "../nrc/nrc.h"
#include "nsimpleX.h"
#include <math.h>

/************************************************************************/
/* fit_2D_gaussian()							*/
/*									*/
/* Tries to fit a 2D gaussian to a matrix of floats in NRC		*/
/* format.								*/
/* The input matrix is float image[1..nx][1..ny].			*/
/* Returns the RMS of the fit.						*/
/* If this return value is 0.0 it means, usually, that a fit was not 	*/
/* possible.								*/
/************************************************************************/

float fit_2D_gaussian(float **pict, int nx, int ny, float *amp, 
			float *x0, float *y0, float *c, float *fwhm, 
			float per_rms_err, int maxit, int show)
{
	int      nvar;
	float    *var;   /* 1..nvar */
	int      ndata;
	float    *data;  /* 1..ndata */
	int      nparam;
	float    **param; /* 1..ndata 1..nparam */
	float    rmsres;
	float    maxres;
	float	 max,min;
	int	 i,j,k;

	/* Allocate memory */

	nvar = 4;
	ndata = nx * ny;
	nparam = 2;
	var = vector(1,nvar);
	data = vector(1, ndata);
	param = matrix(1,ndata,1,nparam);

	/*
	 * Fill in the stuff to the appropriate bits  and
	 * get first guess for the fit.
	 */

	max = -1e32;
	min = 1e32;
	for(k=1,i=1; i<=nx;i++)
	for(j=1; j<=ny; j++)
	{
		data[k] = pict[i][j];
		param[k][1] = (float)i;
		param[k][2] = (float)j;

		if (data[k] > max)
		{
			max = data[k];
			var[2] = (float)i;
			var[3] = (float)j;
		}
		if (data[k] < min) min = data[k];
		k++;
	}
	var[1] = max;

	/* We need a first guess for the final variable */

	var[4] = calculate_fwhm(pict,  nx, ny, 1.0)/2.354820045;

#ifdef DEBUG
	printf("First Guess:\n");
	printf("Amp    = %f\n",var[1]);
        printf("X0     = %f\n",var[2]);
        printf("Y0     = %f\n",var[3]);
        printf("C      = %f\n",var[4]);
        printf("FWHM   = %f\n",var[4]*2.354820045);
#endif

	/* We do no fitting if things look bad */

	if (max-min < 1e-10)
	{
		*amp = var[1];
		*x0 = var[2];
		*y0 = var[3];
		*c = var[4];
		*fwhm = *c * 2.354820045;
		return 0.0;
	}

	/* Do the fit */

	non_linear_fit(nvar, var, ndata, data, nparam, param, 
                       TRUE, gassian_2D_func, &rmsres, &maxres,
                       per_rms_err, maxit, show);

	/* Send the results back to the caller function */

	*amp = var[1];
	*x0 = var[2];
	*y0 = var[3];
	*c = var[4];
	*fwhm = *c * 2.354820045;

	/* Release the memory */

	free_vector(var, 1,nvar);
	free_vector(data, 1, ndata);
	free_matrix(param, 1,ndata,1,nparam);

	/* That is all */

	return rmsres;

} /* fit_2D_gaussian() */

/************************************************************************/
/* gassian_2D_func()                                                    */
/*                                                                      */
/* Calculates function value and derivatives.                           */
/* param[1] is X , param[2] is y.					*/
/* var[1] is amplotide, var[2] is X0, var[3] is Y0 and var[4] is C.	*/
/************************************************************************/
 
void    gassian_2D_func(float *ftn, float *var, float *param)
{
	float	x,y,c2,t;

	x = param[1] - var[2];
	y = param[2] - var[3];
	c2 = var[4] * var[4];
	t = x*x + y*y;

	ftn[1] = exp(t/(-2.0*c2));
        ftn[0] = var[1] * ftn[1];
	ftn[2] = ftn[0] * x / c2;
	ftn[3] = ftn[0] * y / c2;
	ftn[4] = ftn[0] * t / (c2*var[4]);

} /* gassian_2D_funct() */

/************************************************************************/
/* fit_1D_gaussian()							*/
/*									*/
/* Trys to fit a gaussian to a power spectrum.				*/
/* Returns the RMS of the fit.						*/
/* If this return value is 0.0 it means, usually, that a fit was not 	*/
/* possible.								*/
/************************************************************************/

float fit_1D_gaussian(int ndata, /* Number of data points */
		  float *x, /* x[1..ndat] */
		  float *y, /* y[1..ndat] */
		  float *amp, float *mean, float *stddev, /* Results */
		  float per_rms_err, int maxit, int show)
{
	float	*var;
	float	**param;
	float	*data;
	float	rmsres, maxres;
	float	max;
	int	i, max_pos;

	/* Search for a good first guess */

	max = -1e32;
	max_pos = ndata/2;
	for(i=1; i<=ndata; i++)
	{
		if (y[i] > max)
		{
			max = y[i];
			max_pos = i;
		}
	}

	/* These help get a good first guess */

	*amp = max;
	*mean = x[max_pos];

	/* Now go look for a good guess for the stddev */

	for(i=max_pos; i<=ndata && y[i] > max/2; i++);
	*stddev = fabs(x[i] - x[max_pos]);
	
	/* Allocate the memory */

	var = vector(1,3);
	param = matrix(1, ndata, 1, 1);
	data = y;

	/* Fill things up */

	var[1] = *amp;
	var[2] = *mean;
	var[3] = *stddev;
	for(i=1; i<=ndata; i++) param[i][1] = x[i];

	/* Do the fit */

	i = non_linear_fit(3, var, ndata, data, 1, param, TRUE,
		       gauss_1D_func, &rmsres, &maxres, 
                       per_rms_err, maxit, show);

	/* Extract the result */

	*amp = var[1];
	*mean = var[2];
	*stddev = var[3];

	/* Free memory */

	free_vector(var, 1, 3);
	free_matrix(param, 1, ndata, 1, 1);

	/* That is all */

	return rmsres;

} /* fit_1D_gaussian() */

/************************************************************************/
/* gauss_1D_func()							*/
/*									*/
/* Function required by 1D_fit_gaussian().				*/
/************************************************************************/

void    gauss_1D_func(float *ftn, float *var, float *param)
{
	float	x,t,c2;

	x = param[1] - var[2];
	c2 = var[3] * var[3];
	t = x*x;

	ftn[1] = exp(t/(-2.0*c2));
        ftn[0] = var[1] * ftn[1];
	ftn[2] = ftn[0] * x / c2;
	ftn[3] = ftn[0] * t / (c2*var[3]);

} /* 1D_gauss_func() */

/************************************************************************/
/* fit_1D_gaussian_with_floor()						*/
/*									*/
/* Trys to fit a gaussian to a function with a floor.			*/
/* Returns the RMS of the fit.						*/
/* If this return value is 0.0 it means, usually, that a fit was not 	*/
/* possible.								*/
/************************************************************************/

float fit_1D_gaussian_with_floor(int ndata, /* Number of data points */
		  float *x, /* x[1..ndat] */
		  float *y, /* y[1..ndat] */
		  float *floor, float *amp, /* Results */
		  float *mean, float *stddev, /* Results */
		  float per_rms_err, int maxit, int show)
{
	float	*var;
	float	**param;
	float	*data;
	float	rmsres, maxres;
	float	min,max;
	int	i, max_pos;

	/* Search for a good first guess */

	max = -1e32;
	min = 1e32;
	max_pos = ndata/2;
	for(i=1; i<=ndata; i++)
	{
		if (y[i] < min) min = y[i];
		if (y[i] > max)
		{
			max = y[i];
			max_pos = i;
		}
	}

	/* These help get a good first guess */

	*floor = min;
	*amp = max - min;
	*mean = x[max_pos];
	*floor = 0.0;

	/* Now go look for a good guess for the stddev */

	for(i=max_pos; i<=ndata && y[i] > max/2; i++);
	*stddev = fabs(x[i] - x[max_pos]);
	
	/* Allocate the memory */

	var = vector(1,4);
	param = matrix(1, ndata, 1, 1);
	data = y;

	/* Fill things up */

	var[1] = *amp;
	var[2] = *mean;
	var[3] = *stddev;
	var[4] = *floor;
	for(i=1; i<=ndata; i++) param[i][1] = x[i];

	/* Do the fit */

	i = non_linear_fit(4, var, ndata, data, 1, param, TRUE,
		       gauss_1D_func_with_floor, &rmsres, &maxres, 
                       per_rms_err, maxit, show);

	/* Extract the result */

	*amp = var[1];
	*mean = var[2];
	*stddev = var[3];
	*floor = var[4];

	/* Free memory */

	free_vector(var, 1, 4);
	free_matrix(param, 1, ndata, 1, 1);

	/* That is all */

	return rmsres;

} /* fit_1D_gaussian() */

/************************************************************************/
/* gauss_1D_func_with_floor()						*/
/*									*/
/* Function required by 1D_fit_gaussian_with_floor().			*/
/************************************************************************/

void    gauss_1D_func_with_floor(float *ftn, float *var, float *param)
{
	float	x,t,c2;

	x = param[1] - var[2];
	c2 = var[3] * var[3];
	t = x*x;

	ftn[1] = exp(t/(-2.0*c2));
        ftn[0] = var[4] + var[1] * ftn[1];
	ftn[2] = ftn[0] * x / c2;
	ftn[3] = ftn[0] * t / (c2*var[3]);
	ftn[4] = 1.0;

} /* 1D_gauss_funcwoth_floor() */

/************************************************************************/
/* fit_fringe_envelope()						*/
/*									*/
/* Tries to fit a fringe envelope function to some data.		*/
/* Returns the RMS of the fit.						*/
/* If this return value is 0.0 it means, usually, that a fit was not 	*/
/* possible.								*/
/* See page 327 of log book 4.						*/
/************************************************************************/

float fit_fringe_envelope(int ndata, /* Number of data points */
		  float *x, /* x[1..ndat] */
		  float *y, /* y[1..ndat] */
		  float *v, float *x0, float *dwn, /* Results */
		  float per_rms_err, int maxit, int show)
{
	float	*var;
	float	**param;
	float	*data;
	float	rmsres, maxres;
	float	max;
	int	i, max_pos;

	/* Search for a good first guess */

	max = -1e32;
	max_pos = 0;
	for(i=1; i<=ndata; i++)
	{
		if (y[i] > max)
		{
			max = y[i];
			max_pos = i;
		}
	}

	/* These help get a good first guess */

	*v = max;
	*x0 = x[max_pos];

	/* Now try and guess the width */

	for(i=max_pos; i<= ndata && y[i] > max/2.0;i++);
	*dwn = fabs(1.88 / (M_PI * (*x0 - x[i])));

	/* Allocate the memory */

	var = vector(1,3);
	param = matrix(1, ndata, 1, 1);
	data = y;

	/* Fill things up */

	var[1] = *v;
	var[2] = *x0;
	var[3] = *dwn;
	for(i=1; i<=ndata; i++) param[i][1] = x[i];

	/* Do the fit */

	i = non_linear_fit(3, var, ndata, data, 1, param, TRUE,
		       fit_fringe_envelope_func, &rmsres, &maxres, 
                       per_rms_err, maxit, show);

	/* Extract the result */

	*v = var[1];
	*x0 = var[2];
	*dwn = var[3];

	/* Free memory */

	free_vector(var, 1, 3);
	free_matrix(param, 1, ndata, 1, 1);

	/* That is all */

	return rmsres;

} /* fit_fringe_envelope() */

/************************************************************************/
/* fit_fringe_envelope_func()                                           */
/*                                                                      */
/* Function for envelope fitting, single fringe version.                */
/* See page 326 log book 4.                                             */
/************************************************************************/

void fit_fringe_envelope_func(float *ftn, float *var, float *param)
{
        float   arg, dwn, v, x, x0;

        /* Assign these easier to understand variables */

        v = var[1];
        x0 = var[2];
        dwn = var[3];
        x = param[1] - x0;
        arg = M_PI * dwn * x;

        /* Calculate the envelope function */

        if (arg == 0.0)
	{
                ftn[0] = v;
                ftn[1] = 0.0;
                ftn[2] = 0.0;
                ftn[3] = 0.0;
	}
        else
	{
                ftn[1] = sin(arg)/arg;
		ftn[0] = v * ftn[1];
		ftn[3] = v * (arg * cos(arg) - sin(arg))/(arg*dwn);
		ftn[2] = -1.0 * ftn[3] * dwn/x;
	}

        /* Deal with negative bits */

        if (ftn[0] < 0.0)
        {
                ftn[0] *= -1.0;
                ftn[1] *= -1.0;
                ftn[2] *= -1.0;
                ftn[3] *= -1.0;
        }
        else if (ftn[0] == 0.0)
        {
                ftn[1] = ftn[2] = ftn[3] = 0.0;
        }

} /* fit_fringe_envelope_func() */
