/************************************************************************/
/* get_filter_value.c		       					*/
/*									*/
/* Takes as input a text string (the filter description) and visits a 	*/
/* lookup table to determine the center wavelength and FWHM.		*/
/* Part of the nsimpleX library.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 626-796-5405						*/
/* Fax      : 626-796-6717						*/
/* email    : theo@chara-array.org					*/
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Nils Turner							*/
/* Date   : 2004-02-11	   						*/
/************************************************************************/

#include <stdio.h>
#include <string.h>
#include "../nrc/nrc.h"
#include "nsimpleX.h"

/************************************************************************/
/* get_filter_value()							*/
/*									*/
/* Returns a filter effective wavelength (and effective bandpass,	*/
/* "fwidth") given an input filter description text string.  The values	*/
/* for wavelength and bandpass are extracted from a lookup table stored	*/
/* in a file.  The file location is given by the shell environment	*/
/* variable, "NSIMPLEX_FILTER".  If NSIMPLEX_FILTER is not defined by	*/
/* the shell, it warns the user and sets the values to those of	the	*/
/* Johnson "I" band.							*/
/*                                                                      */
/* The syntax for setting the environment variable is:                  */
/*                                                                      */
/* export NSIMPLEX_FILTER=/ctrscrut/chara/etc/filter_info               */
/*                                                                      */
/* for bourne-like shells (inserted into the .profile or .bash_profile  */
/* file), and:                                                          */
/*                                                                      */
/* setenv NSIMPLEX_FILTER "/ctrscrut/chara/etc/filter_info"             */
/*                                                                      */
/* for c-shell-like shells (inserted into the .login file).  The name   */
/* of the file and the path can be arbitrarily changed, just make sure  */
/* the new values are correct in the above entries in the profile or    */
/* login file.                                                          */
/************************************************************************/

double get_filter_value(
	char	*filter,
	double	*fwidth)
{
	char	filter_info_path[81];
	FILE	*filter_file;
	double	center_wavelength;
	char	line[81];
	int	i;
	char	*tmp;
	
	/* Set up the default values of the center wavelength and width */
	center_wavelength = 0.878e-6;
	*fwidth = 0.2176e-6;

	if (getenv("NSIMPLEX_FILTER") != NULL)
	{
		strcpy(filter_info_path,getenv("NSIMPLEX_FILTER"));
		filter_file = fopen(filter_info_path,"r");
		if (filter_file == NULL)
		{
			fprintf(stderr,"Unable to open %s,\n",
				filter_info_path);
			fprintf(stderr,"  setting wavelength and %s\n",
				"bandpass to default values.");
			return center_wavelength;
		}
	}
	else
	{
		fprintf(stderr,"The shell environment variable %s\n",
			"NSIMPLEX_FILTER is not set.  See the man page");
		fprintf(stderr,"for get_filter_value(3) for details on %s\n",
			"how to set the environment variable.");
		fprintf(stderr,"For now, setting wavelength and bandpass %s\n",
			"to default values.");
		return center_wavelength;
	}

	while (fgets(line,80,filter_file) != NULL)
	{
		/* Make sure there are no newlines in the string. */
		i = 0;
		while ((line[i] != '\n') && (i < 80))
		{
			i++;
		}
		line[i] = '\0';

		/* Parse the line using ",," as a delimiter */
		tmp = strtok(line,",,");
		if (!strcmp(filter,tmp))
		{
			tmp = strtok(NULL,",,");
			if (tmp != NULL)
			{
				center_wavelength = atof(tmp);
				if (center_wavelength == 0.0)
				{
				fprintf(stderr,"%s %s %s\n",
					filter,
					"entry of filter lookup table is",
					"malformed, setting wavelength");
				fprintf(stderr,"  to default value.\n");
					center_wavelength = 0.878;
				}
				center_wavelength *= 1.0e-6;
			}
			else
			{
				fprintf(stderr,"%s %s %s\n",
					filter,
					"entry of filter lookup table is",
					"malformed, setting wavelength");
				fprintf(stderr,"  and bandpass to %s\n",
					"default values.");
				fclose(filter_file);
				return center_wavelength;
			}
			tmp = strtok(NULL,",,");
			if (tmp != NULL)
			{
				*fwidth = atof(tmp);
				if (*fwidth == 0.0)
				{
				fprintf(stderr,"%s %s %s\n",
					filter,
					"entry of filter lookup table is",
					"malformed, setting bandpass");
				fprintf(stderr,"  to default value.\n");
					*fwidth = 0.2176;
				}
				*fwidth *= 1.0e-6;
			}
			else
			{
				fprintf(stderr,"%s %s %s\n",
					filter,
					"entry of filter lookup table is",
					"malformed, setting bandpass");
				fprintf(stderr,"  to default value.\n");
			}
			fclose(filter_file);
			return center_wavelength;
		}
	}
	fprintf(stderr,"%s entry not found in filter lookup table, %s\n",
		filter,"setting wavelength");
	fprintf(stderr,"  and bandpass to default values.\n");

	fclose(filter_file);
	return center_wavelength;
} /* get_filter_value() */
