/************************************************************************/
/* image_math.c								*/
/*                                                                      */
/* Some functions to do some maths on images				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : August 1st 1996						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include "../nrc/nrc.h"
#ifdef NOT_AN_APPLE
#include <values.h>
#endif
#include "nsimpleX.h"

/************************************************************************/
/* subtract_images()							*/
/*									*/
/* Returns the difference between two images.				*/
/* It allocates memory for the result and assumes both inputs have	*/
/* the same dimensions.							*/
/* Return value is the result.						*/
/*									*/
/************************************************************************/

float	**subtract_images(float **image1,
			  float **image2,
			  int npix_x,
			  int npix_y)
{
	float **result;
	int	i,j;

	result = matrix(1,npix_x,1,npix_y);

	for(i=1; i<= npix_x; i++)
	for(j=1; j<= npix_y; j++)
	{
		result[i][j] = image1[i][j] - image2[i][j];
	}
	
	return result;

} /* subtract_images() */

/************************************************************************/
/* normalize_image() 							*/
/*									*/
/* Normalizes an image by dividing by the maximum value.		*/
/* The result is put in same memory location as original image.		*/
/* It also finds the x and y position of the maximum value.		*/
/*									*/
/************************************************************************/

void 	normalize_image(float **image,
			int npix_x,
			int npix_y,
			int *max_x,
			int *max_y)
{

	int i,j;
	float	max;

        max = max_image_intensity(image, npix_x, npix_y, max_x, max_y);

        for(i=1; i<= npix_x; i++)
        for(j=1; j<= npix_y; j++)
        {
                image[i][j] /= max;
        }

} /* normalize_image() */

/************************************************************************/
/* smooth_image() 							*/
/*									*/
/* Performs a BFI smoothing of an image array.				*/
/* If "weights" is NULL, it uses even weighting.			*/
/* The weighting matrix MUST be square and have an odd pixel dimension	*/
/* (the "spix" parameter), i.e. 3x3, 5x5, etc.				*/
/************************************************************************/

void	smooth_image(float 	**image,	/* The image */
		     int	npix_x,		/* Number of pixels X axis */
		     int	npix_y,		/* Number of pixels Y axis */
		     float	**weights,	/* Weighting matrix */
		     int	spix)		/* weights dimension */
{
	int	i,j,k,l,x,y;
	int	mid;
	float	**smooth;
	float	norm;

	/* The spix MUST be odd */

	if (spix%2 == 0)
	{
		nrerror(
	"smooth_image: Number of pixel is weight matrix must be odd.\n");
	}

	/* Allocate memory */

	mid = spix/2+1;
	smooth=matrix(1,npix_x,1,npix_y);

	/* Work out the normalization */

	if (weights == NULL)
	{
		norm = spix*spix;
	}
	else
	{
		norm = 0.0;
		for(k=1; k<= spix; k++)
		for(l=1; l<= spix; l++)
		{
			norm += weights[k][l];
		}
	}

	/* Set up for smoothing operation */

	for(i=1; i<=npix_x; i++)
	for(j=1; j<=npix_y; j++)
	{
		smooth[i][j] = image[i][j];
		image[i][j]  = 0.0;
	}

	/* OK do it */

	for(i=1; i<=npix_x; i++)
	for(j=1; j<=npix_y; j++)
	{
		for(k=1; k<= spix; k++)
		for(l=1; l<= spix; l++)
		{
			x = i + k - mid;
			y = j + l - mid;

			if (x >= 1 && x <= npix_x && y >= 1 && y <= npix_y)
			{
				if (weights != NULL)
				{
					image[i][j] += 
						(weights[k][l]*smooth[x][y]);
				} 
				else
				{
					image[i][j] += smooth[x][y];
				}
			}
		}
		image[i][j] /= norm;
			
	}

	/* Free up memory and return */

	free_matrix(smooth,1,npix_x,1,npix_y);

} /* smooth_image() */

/************************************************************************/
/* shift_matrix() 							*/
/*									*/
/* Performs an even parity shift of a square matrix.			*/
/* Data "rolled off" one edge "reappears" on the opposite edge.		*/
/************************************************************************/

void	shift_matrix(float **data, 
		     int npix_x, 
		     int npix_y, 
		     int x_shift, 
		     int y_shift)
{
	int	i,j,k,l;
	float	**orig;

	/* OK, allocate memory for doing this */

	orig = matrix(1,npix_x,1,npix_y);

	/* 
	 * Load the data into this area, 
 	 * The original may have been a submatrix and so
	 * we must ensure result goes into the original memory locations.
	 */

	for(i=1; i<=npix_x; i++)
	for(j=1; j<=npix_y; j++)
	{
		orig[i][j] = data[i][j];
	}

	/* Now do that rotation thang .... */

	for(i=1; i<=npix_x; i++)
	for(j=1; j<=npix_y; j++)
	{
		k = (i+x_shift)%npix_x;
		while (k < 1) k += npix_x;

		l = (j+y_shift)%npix_y;
		while (l < 1) l += npix_y;

		data[k][l] = orig[i][j];
	}

	/* Clean up and go */

	free_matrix(orig,1,npix_x,1,npix_y);

}
/************************************************************************/
/* shift_vector() 							*/
/*									*/
/* Performs an even parity shift of a vector.				*/
/* Data "rolled off" one edge "reappears" on the opposite edge.		*/
/* Assumes that vector goes from 1 to npix.				*/
/************************************************************************/

void	shift_vector(float *data, 
		     int npix, 
		     int shift)
{
	int	i,k;
	float	*orig;

	/* OK, allocate memory for doing this */

	orig = vector(1,npix);

	/* Load the data into this area. */

	for(i=1; i<=npix; i++) orig[i] = data[i];

	/* Now do that rotation thang .... */

	for(i=1; i<=npix; i++)
	{
		k = (i+shift)%npix;
		while (k < 1) k += npix;

		data[k] = orig[i];
	}

	/* Clean up and go */

	free_vector(orig,1,npix);

} /* shift_vector() */

/************************************************************************/
/* sum_images()	 							*/
/*									*/
/* Adds one image into another ...					*/
/* It can be called recursively to keep a running sum.			*/
/************************************************************************/

void	sum_images( float **result, 
		    float **image, 
		    int npix_x,
		    int npix_y )
{
	int i,j;

	for(i=1;i<=npix_x;i++)	
	for(j=1;j<=npix_y;j++)
	{
		result[i][j] += image[i][j];
	}

} /* sum_images() */

/************************************************************************/
/* max_image_intensity()						*/
/*									*/
/* Finds the maximum value of the "image".				*/
/* It also finds the x and y position of the maximum value.		*/
/*									*/
/************************************************************************/

float	max_image_intensity( float **image, 
			     int npix_x, 
			     int npix_y, 
			     int *pos_x, 
			     int *pos_y)
{
	int	i,j;
	float	max;

	/* Find the maximum */

#ifdef DEBUG
	printf("Finding maximum.\n");
#endif

	max = -MAXFLOAT;
	for(i=1;i<=npix_x;i++)
	for(j=1;j<=npix_y;j++)
	{
		if (image[i][j] > max) 
		{
			*pos_x = i;
			*pos_y = j;
			max = image[i][j];
		}
	}

	/* That is all */

#ifdef DEBUG
	printf("Result is %f.\n",max);
#endif
	return max;

} /* max_image_intensity() */

/************************************************************************/
/* min_image_intensity()						*/
/*									*/
/* Finds the minimum value of the "image".				*/
/* It also finds the x and y positions of the minimum value.		*/
/*									*/
/************************************************************************/

float	min_image_intensity( float **image, 
			     int npix_x, 
			     int npix_y, 
			     int *pos_x, 
			     int *pos_y)
{
	int	i,j;
	float	min;

	/* Find the maximum */

#ifdef DEBUG
	printf("Finding minimum.\n");
#endif

	min = MAXFLOAT;
	for(i=1;i<=npix_x;i++)
	for(j=1;j<=npix_y;j++)
	{
		if (image[i][j] < min) 
		{
			*pos_x = i;
			*pos_y = j;
			min = image[i][j];
		}
	}

	/* That is all */

#ifdef DEBUG
	printf("Result is %f.\n",min);
#endif
	return min;

} /* min_image_intensity() */

/************************************************************************/
/* total_image() 							*/
/* 									*/
/* Returns the sum of all the pixels in the "image" matrix.		*/
/************************************************************************/

float	total_image( float **image, 
			     int npix_x, 
			     int npix_y)
{
	float	result;
	int	i,j;

	result = 0.0;
	for (i=1; i<= npix_x; i++)
	for (j=1; j<= npix_y; j++)
	{
		result += image[i][j];
	}

	return result;

} /* total_image() */

/************************************************************************/
/* scale_image() 							*/
/* 									*/
/* Multiplies all the pixels in the "image" by "scale".			*/
/************************************************************************/

void	scale_image(float **image, 
			float scale,
			int npix_x, 
			int npix_y)
{
	int	i,j;

	for (i=1; i<= npix_x; i++)
	for (j=1; j<= npix_y; j++)
	{
		image[i][j] *= scale;
	}

} /* scale_image() */

/************************************************************************/
/* copy_image() 							*/
/* 									*/
/* Creates a copy of an image matrix.  					*/
/************************************************************************/

float **copy_image( float **image, 
			     int npix_x, 
			     int npix_y)
{
	int	i,j;
	float	**copy;
	
	copy = matrix(1,npix_x,1,npix_y);

	for (i=1; i<= npix_x; i++)
	for (j=1; j<= npix_y; j++)
	{
		copy[i][j] = image[i][j];
	}

	return copy;

} /* copy_image() */

/************************************************************************/
/* centroid_image()							*/
/*									*/
/* Performs a simple centroiding around a given zone on an image.	*/
/* Returns the value of the image at that position.			*/
/************************************************************************/

float centroid_image(float **data, int nx, int ny, /* Data array size */
		    int gx, int gy,        /* First guess of position */
		    int	size,	                  /* Search area size */
		    float *px, float *py)       /* Resulting position */
{
	int	i,j;
	int	start_x,stop_x;
	int	start_y, stop_y;
	float	total,subtotal,mean;

	/* Work out our starting position... */

	if ((start_x = gx - size/2) < 1) start_x = 1;
	if ((stop_x = start_x + size) > nx) stop_x = nx;
	if ((start_y = gy - size/2) < 1) start_y = 1;
	if ((stop_y = start_y + size) > ny) stop_y = ny;

#ifdef DEBUG
	printf("centroid():\n\n");
	printf("X range is %d to %d\n",start_x,stop_x);
	printf("Y range is %d to %d\n",start_y,stop_y);
#endif

	/* OK, let's do the X thing */

	for(mean = 0.0, total = 0.0, i=start_x; i<=stop_x; i++)
	{
		/* Work out the total intensity in one column */

		for(subtotal=0.0, j=start_y; j<=stop_y; j++)
		{
			subtotal += data[i][j];
		}

		total += subtotal;
		mean += (i * subtotal);
	}

	*px = mean/total;

	/* OK, let's do the Y thing */

	for(mean = total = 0, j=start_y; j<=stop_y; j++)
	{
		/* Work out the total intensity in one column */

		for(subtotal=0, i=start_x; i<=stop_x; i++)
		{
			subtotal += data[i][j];
		}

		total += subtotal;
		mean += (j * subtotal);
	}

	*py = mean/total;

	/* That is all */

	return data[(int)(*px+0.5)][(int)(*py+0.5)];

} /* centroid_image() */

/************************************************************************/
/* local_max_image_intensity()						*/
/*									*/
/* Finds the local maximum around a given point.			*/
/* Returns the value of the image at that position.			*/
/************************************************************************/

float local_max_image_intensity(float **image, int nx, int ny, 
			int gx, int gy,   /* First guess of position */
			int size,	         /* Search area size */
			int *xpos, int *ypos)  /* Resulting position */
{
	int	i,j;
	int	start_x,stop_x;
	int	start_y, stop_y;
	float	maximum;

	/* Work out our starting position... */

	if ((start_x = gx - size/2) < 1) start_x = 1;
	if ((stop_x = start_x + size) > nx) stop_x = nx;
	if ((start_y = gy - size/2) < 1) start_y = 1;
	if ((stop_y = start_y + size) > ny) stop_y = ny;


	maximum = -MAXFLOAT;
	for(i=start_x; i<=stop_x; i++)
	for(j=start_y; j<=stop_y; j++)
	{
		if (image[i][j] > maximum)
		{
			maximum = image[i][j];
			*xpos = i;
			*ypos = j;
		}
	}

	return maximum;

} /* local_max_image_intensity() */
