/************************************************************************/
/* makepicture.c							*/
/*									*/
/* Creates the picture data for a displayed image.			*/
/* Y axis inverted so it looks correct on the screen.			*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : August 1994   						*/
/************************************************************************/

#include <stdlib.h>
#include <math.h>
#include "../nrc/nrc.h"
#ifdef NOT_AN_APPLE
#include <values.h>
#endif
#include "nsimpleX.h"

//#define DEBUG

/************************************************************************/
/* make_scaled_picture()						*/
/*									*/
/* Given a 2D array of floats 1..nx and 1..ny representing a picture	*/
/* this function builds a 2D array of chars 0..nx-1, 0..ny-1 ready for	*/
/* the display. The Y-axis is reversed so it looks right on the screen.	*/
/* Returns a pointer to the picture itself.  				*/
/* If min and max are set to zero they are calculated localy, otherwise */
/* The set values are used.						*/
/************************************************************************/

char	*make_scaled_picture(int nx, 
		   int ny, 
		   int pixmult, 
		   float **values,
		   char disp_type,
		   float *mmin,
		   float *mmax)
{
  return make_xyscaled_picture(nx, ny, pixmult, pixmult, values,
			       disp_type, mmin, mmax);
} /* make_scaled_picture() */

/************************************************************************/
/* make_xyscaled_picture()						*/
/*									*/
/* Like make_scaled_picture, but has separate x and y scalings.		*/
/************************************************************************/

char	*make_xyscaled_picture(int nx, 
		   int ny, 
  	           int xpixmult, int ypixmult, 
		   float **values,
		   char disp_type,
		   float *mmin,
		   float *mmax)
{
	int	i,j,k,l;
	int	pixel;
	float	min, max, step;
	float	value;
	float	**new_values;
	char *picture;
	char	*p8;
	short int *p16;
	int *p32;
	size_t  size;
#ifdef DEBUG
	int	maxi,mini;
#endif

	/* Allocate memory for the picture */

	size = nx * xpixmult * ny * ypixmult * theBitmapBytes;

#ifdef DEBUG
	printf("theBitmapBytes = %d\n",theBitmapBytes);
	printf("Memory allocation size = %ld\n",size);
#endif

	if ((picture = (char *)malloc(size)) == NULL)
	{
		fprintf(stderr,"Ran out of memory in make_picture().\n");
		return NULL;
	}

	p8 = (char *)picture;
	p16 = (short int *)picture;
	p32 = (int *)picture;

	if (disp_type == EXP || disp_type == LOG)
		new_values = matrix(1,nx,1,ny);
	else
		new_values = values;

	/* Find the maximum and minimum of the data */

	if (disp_type != STD)
	{
		
		if (*mmin == 0.0 && *mmax == 0.0)
		{
			*mmax = -1.0*MAXFLOAT;
			*mmin = MAXFLOAT;

			for(i=1;i<=nx;i++)
			{
			    for(j=1;j<=ny;j++)
			    {
				if (values[i][j] > *mmax) *mmax = values[i][j];
				if (values[i][j] < *mmin) *mmin = values[i][j];
			    }
			}
		}
		min = *mmin;
		max = *mmax;
	}
	else
	{
		max = *mmax = NUM_COLORS-1;
		min = *mmin = 0;
	}

#ifdef DEBUG
	printf("Min = %f\nMax = %f\n",min,max);
#endif

	/* Now do the conversion */

	if (disp_type == EXP || disp_type == LOG)
	{
	        for(i=1;i<=nx;i++)
		{
			for(j=1;j<=ny;j++)
			{
				switch(disp_type)
				{
					case LOG: new_values[i][j] =
						      log(values[i][j]-min+1);
						  break;

					case EXP: new_values[i][j] =
						      exp(values[i][j]/ max);
						  break;

					default : break;
				}
			}
		}
		switch(disp_type)
		{
			case LOG: min=0.0;
				  max = log(max-min+1);
				  break;

			case EXP: min = exp(min / max);
				  max = exp(max / max);
				  break;

			default:  break;
		}
	}

	/* Now allocate the colors ..... */

	step = (max - min)/(float)(NUM_COLORS -1);

#ifdef DEBUG
	printf("Min = %f\nMax = %f\n",min,max);
	printf("Step = %f\n",step);
	maxi = -1;
	mini = 200;
#endif
	switch(theDepthBytes)
	{
	    case 1: for(j=ny; j>0; j--)
		{
		    for(i=1;i<=nx;i++)
		    {
			if (min != max)
			{
				value = new_values[i][j] - min;
			}
			else
			{
				value = 0;
			}

			if (min != max)
				pixel = value/step;
			else
				pixel = 0;
#ifdef DEBUG
			if (pixel < mini) mini = pixel;
			if (pixel > maxi) maxi = pixel;
#endif
			if (pixel < 0) pixel = 0;
			if (pixel > NUM_COLORS-1) pixel = NUM_COLORS-1;

			for(k=0;k<xpixmult;k++)
			for(l=0;l<ypixmult;l++)
			{
				*(p8+nx*xpixmult*l+k) = GreyScale[pixel];
			}
			/* MJI Comments. This is the x loop, so we want to 
			 add the xpixmult in here...*/
			p8 += xpixmult;
		    }
		    /* MJI comment. Now we are in the y loop. One rebinned
		       pixel has a width of xpixmult*nx, and we have already moved
		       xpixmult*nx elements through the array. So we have only another
		       ypixmult-1 rows to go. */
		    p8 += ((ypixmult-1)*xpixmult*nx);
		}
		break;

	    case 2: for(j=ny; j>0; j--)
		{
		    for(i=1;i<=nx;i++)
		    {
			if (min != max)
			{
				value = new_values[i][j] - min;
			}
			else
			{
				value = 0;
			}

			if (min != max)
				pixel = value/step;
			else
				pixel = 0;
#ifdef DEBUG
			if (pixel < mini) mini = pixel;
			if (pixel > maxi) maxi = pixel;
#endif
			if (pixel < 0) pixel = 0;
			if (pixel > NUM_COLORS-1) pixel = NUM_COLORS-1;

			for(k=0;k<xpixmult;k++)
			for(l=0;l<ypixmult;l++)
			{
				*(p16+nx*xpixmult*l+k) = GreyScale[pixel];
			}

			p16 += xpixmult;
		    }
		    p16 += ((ypixmult-1)*xpixmult*nx);
		}
		break;

	    default: for(j=ny; j>0; j--)
		{
		    for(i=1;i<=nx;i++)
		    {
			if (min != max)
			{
				value = new_values[i][j] - min;
			}
			else
			{
				value = 0;
			}

			if (min != max)
				pixel = value/step;
			else
				pixel = 0;
#ifdef DEBUG
			if (pixel < mini) mini = pixel;
			if (pixel > maxi) maxi = pixel;
#endif
			if (pixel < 0) pixel = 0;
			if (pixel > NUM_COLORS-1) pixel = NUM_COLORS-1;

			for(k=0;k<xpixmult;k++)
			for(l=0;l<ypixmult;l++)
			{
				*(p32+nx*xpixmult*l+k) = GreyScale[pixel];
			}

			p32 += xpixmult;
		    }
		    p32 += ((ypixmult-1)*xpixmult*nx);
		}
		break;
	}
#ifdef DEBUG
	printf("mini = %d\nmaxi = %d\n",mini,maxi);
#endif

	/* Clean up memory */

	if (disp_type == EXP || disp_type == LOG) 
	{
		free_matrix(new_values,1,nx,1,ny);
	}

	/* That's it */

	return picture;

} /* make_xyscaled_picture() */

/************************************************************************/
/* make_picture()							*/
/*									*/
/* Calls make_scaled_picture with min and max forced to be zero.	*/
/************************************************************************/

char	*make_picture(int nx, 
		   int ny, 
		   int pixmult, 
		   float **values,
		   char disp_type)
{
	float	min,max;

	min = max = 0.0;
	return make_scaled_picture(nx,ny,pixmult,values,disp_type,&min,&max);

} /* make_picture() */

/************************************************************************/
/* make_ppm_P3()							*/
/*									*/
/* Given a pointer to the start of a picture and its size, it writes a	*/
/* pure ASCII PPM file.							*/
/************************************************************************/

void	make_ppm_P3(char *filename, 
		    int nx, 
		    int ny, 
		    int pixmult,
		    char *picture)
{
	int	i,j;	/* Looping */
	FILE	*output;
	unsigned char	*p8;
	unsigned short int *p16;
	unsigned int *p32;
	unsigned char	color;

	/* Open the data file */

	if ((output = fopen(filename,"w")) == NULL)
	{
		fprintf(stderr,"Could not open %s for writing\n",filename);
		return;
	}

	/* Magic number */

	fprintf(output,"P3\n");

	/* Height and width */

	fprintf(output,"%d\n%d\n",ny*pixmult,nx*pixmult);

	/* Maximum color componant */

	fprintf(output,"%d\n",65535);

	/* Pixels going out .... */

	switch(theDepthBytes)
	{
		case 1: p8 = (unsigned char *)picture;
			for(i=0;i<nx*pixmult;i++)
			for(j=0;j<ny*pixmult;j++)
			{
				for(color=0; color<NUM_COLORS; color++)
				{
					if (GreyScale[color] == *p8) break;
				}
				if (color < NUM_COLORS)
				{
					fprintf(output,"%u %u %u\n",
					    color*65535/(NUM_COLORS-1),
					    color*65535/(NUM_COLORS-1),
					    color*65535/(NUM_COLORS-1));
					p8++;
					continue;
				}
				for(color=0; color<NUM_OTHER_COLORS; color++)
				{
					if (other_colors[color] == *p8) break;
				}
				if (color < NUM_OTHER_COLORS)
				{
					fprintf(output,"%u %u %u\n",
						other_colors_rgb[color][0],
						other_colors_rgb[color][1],
						other_colors_rgb[color][2]);
					p8++;
					continue;
				}
				color = NUM_COLORS-1;
				fprintf(output,"%u %u %u\n",
				    color*65535/(NUM_COLORS-1),
				    color*65535/(NUM_COLORS-1),
				    color*65535/(NUM_COLORS-1));
				p8++;
			}
			break;

		case 2: p16 = (unsigned short int *)picture;
			for(i=0;i<nx*pixmult;i++)
			for(j=0;j<ny*pixmult;j++)
			{
				for(color=0; color<NUM_COLORS; color++)
				{
					if (GreyScale[color] == *p16) break;
				}
				if (color < NUM_COLORS)
				{
					fprintf(output,"%u %u %u\n",
					    color*65535/(NUM_COLORS-1),
					    color*65535/(NUM_COLORS-1),
					    color*65535/(NUM_COLORS-1));
					p16++;
					continue;
				}
				for(color=0; color<NUM_OTHER_COLORS; color++)
				{
					if (other_colors[color] == *p16) break;
				}
				if (color < NUM_OTHER_COLORS)
				{
					fprintf(output,"%u %u %u\n",
						other_colors_rgb[color][0],
						other_colors_rgb[color][1],
						other_colors_rgb[color][2]);
					p16++;
					continue;
				}
				color = NUM_COLORS-1;
				fprintf(output,"%u %u %u\n",
				    color*65535/(NUM_COLORS-1),
				    color*65535/(NUM_COLORS-1),
				    color*65535/(NUM_COLORS-1));
				p16++;
			}
				break;

		default: p32 = (unsigned char *)picture;
			for(i=0;i<nx*pixmult;i++)
			for(j=0;j<ny*pixmult;j++)
			{
				for(color=0; color<NUM_COLORS; color++)
				{
					if (GreyScale[color] == *p32) break;
				}
				if (color < NUM_COLORS)
				{
					fprintf(output,"%u %u %u\n",
					    color*65535/(NUM_COLORS-1),
					    color*65535/(NUM_COLORS-1),
					    color*65535/(NUM_COLORS-1));
					p32++;
					continue;
				}
				for(color=0; color<NUM_OTHER_COLORS; color++)
				{
					if (other_colors[color] == *p32) break;
				}
				if (color < NUM_OTHER_COLORS)
				{
					fprintf(output,"%u %u %u\n",
						other_colors_rgb[color][0],
						other_colors_rgb[color][1],
						other_colors_rgb[color][2]);
					p32++;
					continue;
				}
				color = NUM_COLORS-1;
				fwrite(&color, sizeof(char), 1, output);
				fwrite(&color, sizeof(char), 1, output);
				fwrite(&color, sizeof(char), 1, output);
				fprintf(output,"%u %u %u\n",
				    color*65535/(NUM_COLORS-1),
				    color*65535/(NUM_COLORS-1),
				    color*65535/(NUM_COLORS-1));
				p32++;
				break;
			}
	}

	fclose(output);
}

/************************************************************************/
/* make_ppm_P6()							*/
/*									*/
/* Given a pointer to the start of a picture and its size, it writes a	*/
/* mixed ASCII/binary PPM file.						*/
/************************************************************************/

void    make_ppm_P6(char *filename, 
		    int nx, 
		    int ny, 
		    int pixmult,
		    char *picture)
{
        int     i,j,k;    /* Looping */
        FILE    *output;
	unsigned char	*p8;
	unsigned short int *p16;
	unsigned int *p32;
	unsigned char	color;

        /* Open the data file */

        if ((output = fopen(filename,"wb")) == NULL)
        {
                fprintf(stderr,"Could not open %s for writing\n",filename);
                return;
        }

        /* Magic number */

        fprintf(output,"P6\n");

        /* Height and width */

        fprintf(output,"%d\n%d\n",ny*pixmult,nx*pixmult);

        /* Maximum color componant */

        fprintf(output,"%d\n",NUM_COLORS-1);

        /* Pixels going out .... */

	switch(theDepthBytes)
        {
		case 1: p8 = (unsigned char *)picture;
			for(i=0;i<nx*pixmult;i++)
			for(j=0;j<ny*pixmult;j++)
			{
				for(color=0; color<NUM_COLORS; color++)
				{
					if (GreyScale[color] == *p8) break;
				}
				if (color < NUM_COLORS)
				{
					fwrite(&color, sizeof(char), 1, output);
					fwrite(&color, sizeof(char), 1, output);
					fwrite(&color, sizeof(char), 1, output);
					p8++;
					continue;
				}
				for(k=0; k<NUM_OTHER_COLORS; k++)
				{
					if (other_colors[k] == *p8) break;
				}
				if (k < NUM_OTHER_COLORS)
				{
					color = (unsigned char)(
					    (float)other_colors_rgb[k][0]/
					    65535.0*(float)(NUM_COLORS-1));
					fwrite(&color, sizeof(char), 1, output);
					color = (unsigned char)(
					    (float)other_colors_rgb[k][0]/
					    65535.0*(float)(NUM_COLORS-1));
					fwrite(&color, sizeof(char), 1, output);
					color = (unsigned char)(
					    (float)other_colors_rgb[k][0]/
					    65535.0*(float)(NUM_COLORS-1));
					fwrite(&color, sizeof(char), 1, output);
					p8++;
					continue;
				}
				color = NUM_COLORS-1;
				fwrite(&color, sizeof(char), 1, output);
				fwrite(&color, sizeof(char), 1, output);
				fwrite(&color, sizeof(char), 1, output);
				p8++;
			}
			break;

		case 2: p16 = (unsigned short int *)picture;
			for(i=0;i<nx*pixmult;i++)
			for(j=0;j<ny*pixmult;j++)
			{
				for(color=0; color<NUM_COLORS; color++)
				{
					if (GreyScale[color] == *p16) break;
				}
				if (color < NUM_COLORS)
				{
					fwrite(&color, sizeof(char), 1, output);
					fwrite(&color, sizeof(char), 1, output);
					fwrite(&color, sizeof(char), 1, output);
					p16++;
					continue;
				}
				for(k=0; k<NUM_OTHER_COLORS; k++)
				{
					if (other_colors[k] == *p16) break;
				}
				if (k < NUM_OTHER_COLORS)
				{
					color = (unsigned char)(
					    (float)other_colors_rgb[k][0]/
					    65535.0*(float)(NUM_COLORS-1));
					fwrite(&color, sizeof(char), 1, output);
					color = (unsigned char)(
					    (float)other_colors_rgb[k][0]/
					    65535.0*(float)(NUM_COLORS-1));
					fwrite(&color, sizeof(char), 1, output);
					color = (unsigned char)(
					    (float)other_colors_rgb[k][0]/
					    65535.0*(float)(NUM_COLORS-1));
					fwrite(&color, sizeof(char), 1, output);
					p16++;
					continue;
				}
				color = NUM_COLORS-1;
				fwrite(&color, sizeof(char), 1, output);
				fwrite(&color, sizeof(char), 1, output);
				fwrite(&color, sizeof(char), 1, output);
				p16++;
			}
			break;

		default: p32 = (unsigned int *)picture;
			for(i=0;i<nx*pixmult;i++)
			for(j=0;j<ny*pixmult;j++)
			{
				for(color=0; color<NUM_COLORS; color++)
				{
					if (GreyScale[color] == *p32) break;
				}
				if (color < NUM_COLORS)
				{
					fwrite(&color, sizeof(char), 1, output);
					fwrite(&color, sizeof(char), 1, output);
					fwrite(&color, sizeof(char), 1, output);
					p32++;
					continue;
				}
				for(k=0; k<NUM_OTHER_COLORS; k++)
				{
					if (other_colors[k] == *p32) break;
				}
				if (k < NUM_OTHER_COLORS)
				{
					color = (unsigned char)(
					    (float)other_colors_rgb[k][0]/
					    65535.0*(float)(NUM_COLORS-1));
					fwrite(&color, sizeof(char), 1, output);
					color = (unsigned char)(
					    (float)other_colors_rgb[k][1]/
					    65535.0*(float)(NUM_COLORS-1));
					fwrite(&color, sizeof(char), 1, output);
					color = (unsigned char)(
					    (float)other_colors_rgb[k][2]/
					    65535.0*(float)(NUM_COLORS-1));
					fwrite(&color, sizeof(char), 1, output);
					p32++;
					continue;
				}
				color = NUM_COLORS-1;
				fwrite(&color, sizeof(char), 1, output);
				fwrite(&color, sizeof(char), 1, output);
				fwrite(&color, sizeof(char), 1, output);
				p32++;
			}
			break;
        }
        fclose(output);

} /* make_ppm_P6() */

/************************************************************************/
/* make_ascii_pict()							*/
/*									*/
/* Given a pointer to the start of a picture and its size, it writes a	*/
/* pure ASCII file containing the dimensions as first two data points.	*/
/************************************************************************/

void	make_ascii_pict(char *filename, 
		    int nx, 
		    int ny, 
		    char *picture)
{
	int	i,j;	/* Looping */
	FILE	*output;
	unsigned char	*p8;
	unsigned short int *p16;
	unsigned int *p32;
	unsigned char	color;

	/* Open the data file */

	if ((output = fopen(filename,"w")) == NULL)
	{
		fprintf(stderr,"Could not open %s for writing\n",filename);
		return;
	}

	/* Height and width */

	fprintf(output,"%d\t%d\n",ny,nx);

	/* Pixels going out .... */

	for(i=0;i<nx;i++)
	for(j=0;j<ny;j++)
	{
        switch(theDepthBytes)
        {
                case 1: p8 = (unsigned char *)picture;
                        for(i=0;i<nx;i++)
                        for(j=0;j<ny;j++)
                        {
                                for(color=0; color<NUM_COLORS; color++)
                                {
                                        if (GreyScale[color] == *p8) break;
                                }
                                p8++;
                                fprintf(output,"%u\n",color);
                        }
                        break;
                                                                                
                case 2: p16 = (unsigned short int *)picture;
                        for(i=0;i<nx;i++)
                        for(j=0;j<ny;j++)
                        {
                                for(color=0; color<NUM_COLORS; color++)
                                {
                                        if (GreyScale[color] == *p16) break;
                                }
                                p16++;
                                fprintf(output,"%u\n",color);
                        }
                        break;
                                                                                
                default: p32 = (unsigned int *)picture;
                        for(i=0;i<nx;i++)
                        for(j=0;j<ny;j++)
                        {
                                for(color=0; color<NUM_COLORS; color++)
                                {
                                        if (GreyScale[color] == *p32) break;
                                }
                                p32++;
                                fprintf(output,"%u\n",color);
                        }
                        break;
        }

	}

	fclose(output);

} /* make_ascii_pict() */

/************************************************************************/
/* make_2D_gnuplot_file() 						*/
/* 									*/
/* Creates a file that GNUPLOT can read and plot.			*/
/* The vectors x and y contain the values along the x and y axis.  	*/
/************************************************************************/

void make_2D_gnuplot_file(float **data, 
			  int nx, 
			  float *x, 
			  int ny, 
			  float *y, 
			  char *filename)
{
	int	i,j;
	float	f;
	FILE	*output;

	/* Open the file */

	if ((output = fopen(filename,"wb")) == NULL)
	{
		printf("make_2D_gnuplot_file(): Failed to open file %s\n",
				filename);
		return;
	}

	/* Add the number of columns */

	f = nx;
	if (fwrite(&f, sizeof(float), 1, output) != 1)
	{
		fprintf(stderr,
			"make_2D_gnuplot_file(): Failed to write data.\n");
		fclose(output);
		return;
	}

	/* Now add the X positions for these columns */

	for(i=1; i<= nx; i++)
	{
		f = x[i];
		if (fwrite(&f, sizeof(float), 1, output) != 1)
		{
		    fprintf(stderr,
			"make_2D_gnuplot_file(): Failed to write data.\n");
		    fclose(output);
		    return;
		}
	}

	/* Now add the Y positions and the actual data */

	for(j=1; j<= ny; j++)
	{
		f = y[j];
		if (fwrite(&f, sizeof(float), 1, output) != 1)
		{
		    printf("make_2D_gnuplot_file(): Failed to write data.\n");
		    fclose(output);
		    return;
		}

		for(i=1; i<= nx; i++)
		{
			f = data[i][j];
			if (fwrite(&f, sizeof(float), 1, output) != 1)
			{
			    printf(
			    "make_2D_gnuplot_file(): Failed to write data.\n");
			    fclose(output);
			    return;
			}
		}
	}

	fclose(output);

} /* make_2D_gnuplot_file() */
