/************************************************************************/
/* overlay_line.c							*/
/*									*/
/* Modifies an existing picture by overlaying a line on the picture 	*/
/* data.  								*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Chad Ogden		                                        */
/* Date   : March 24,  2004                                             */
/************************************************************************/

#include <stdlib.h>
#include <math.h>
#include "../nrc/nrc.h"
#ifdef NOT_AN_APPLE
#include <values.h>
#endif
#include "nsimpleX.h"

/************************************************************************/
/* overlay_line()							*/
/*									*/
/* nx = picture width in pixels (including any pixel multiplier)	*/
/* ny = picture height in pixels (including any pixel multiplier)	*/
/* *picture = pointer to picture data returned by make_picture()	*/
/* x = line starting point x coordinate, 0..nx-1, 0 at left		*/
/* y = line starting point y coordinate, 0..ny-1, 0 at top (+ down)	*/
/* length = line length including starting pixel, - values draw the 	*/
/*	line in the - axis direction					*/
/* width = line width including starting pixel, - values widen the	*/
/* 	line in the  perp. axis direction.				*/
/* horiz = is line horizontal (xaxis)? if >0 yes, if <=0 vertical(yaxis)*/
/* Draws a line over existing picture data pointed to by *picture.	*/
/*									*/
/************************************************************************/

void overlay_line(int nx, int ny, char *picture, int x, int y, int length, 
		int width, int horiz, int red, int green, int blue)
{
	//local variables
	int i,j,k;
	char *p8;
	unsigned short int *p16;
        unsigned int *p32;
	XColor color;
	int	plot_color;

	color.red = red;
	color.green = green;
	color.blue = blue;
	color.flags = DoRed | DoGreen | DoBlue;
	if (!XAllocColor(theDisplay,theCmap,&color))
		plot_color = WHITE;
	else
		plot_color = color.pixel;


	//check to see if the origin is within the data, if not, force it.	
	
	if(x < 0) x=0;
	if(x >= nx) x = nx-1;
	if(y < 0) y=0;
	if(y >= ny) y = ny-1;

	//check to see if the length is within the data, if not, force it.
	
	if( length==0 ) length=1;  // 1 pixel minimum length.
	
	else if(length > 0)
	{
		if(horiz>0) //horizontal line
		{
			// if it's going to run outside the data, shorten it.
		
			if( (x + (length-1)) >= nx) length = nx-x;
		}

		else //vertical line
		{
			// if it's going to run outside the data, shorten it.
		
			if( (y + (length-1)) >= ny) length = ny-y;

		}
	}
	
	else //length < 0
	{
		if(horiz>0) //horizontal line
		{
			// if it's going to run outside the data, shorten it.
		
			if( (x + (length+1)) < 0) length = -(x+1);
		}

		else //vertical line
		{
			// if it's going to run outside the data, shorten it.
		
			if( (y + (length+1)) < 0) length = -(y+1);

		}
	}

	//check to see if the width is within the data, if not, force it.
	
	if( width==0 ) width=1;  // 1 pixel minimum length.
	
	else if(width > 0)
	{
		if(horiz>0) //horizontal line
		{
			// if it's going to run outside the data, shorten it.
		
			if( (y + (width-1)) >= ny) width = ny-y;
		}

		else //vertical line
		{
			// if it's going to run outside the data, shorten it.
		
			if( (x + (width-1)) >= nx) width = nx-x;

		}
	}
	
	else //width < 0
	{
		if(horiz>0) //horizontal line
		{
			// if it's going to run outside the data, shorten it.
		
			if( (y + (width+1)) < 0) width = -(y+1);
		}

		else //vertical line
		{
			// if it's going to run outside the data, shorten it.
		
			if( (x + (width+1)) < 0) width = -(x+1);

		}
	}

	// step through the array and change the data.
	
	switch(theDepthBytes)
	{
	    case 1:

		// loop for horizontal lines

		if (horiz > 0)
		{
		    for(j=0; j<abs(width); j++)
		    {
			p8 = picture + (y+j)*nx + x;	
			for(i=0; i<abs(length); i++)
			{
				//change pixel color
			
				*p8++ = plot_color;
			}
		    }
		}

	        // loop for vertical lines

		else
		{
		    for(i=0; i<abs(width); i++)
		    {
			p8 = picture + y*nx + x + i;	
			for(j=0; j<abs(length); j++)
			{
				//change pixel color

				*p8 = plot_color;
				p8 += nx;
			}
		    }
		}
		break;

	    case 2:

		// loop for horizontal lines

		if (horiz > 0)
		{
		    for(j=0; j<abs(width); j++)
		    {
			p16 = (unsigned short int *)picture + (y+j)*nx + x;	
			for(i=0; i<abs(length); i++)
			{
				//change pixel color
			
				*p16++ = plot_color;
			}
		    }
		}

	        // loop for vertical lines

		else
		{
		    for(i=0; i<abs(width); i++)
		    {
			p16 = (unsigned short int *)picture + y*nx + x + i;	
			for(j=0; j<abs(length); j++)
			{
				//change pixel color

				*p16 = plot_color;
				p16 += nx;
			}
		    }
		}
		break;

	    default:

		// loop for horizontal lines

		if (horiz > 0)
		{
		    for(j=0; j<abs(width); j++)
		    {
			p32 = (unsigned int *)picture + (y+j)*nx + x;	
			for(i=0; i<abs(length); i++)
			{
				//change pixel color
			
				*p32++ = plot_color;
			}
		    }
		}

	        // loop for vertical lines

		else
		{
		    for(i=0; i<abs(width); i++)
		    {
			p32 = (unsigned int *)picture + y*nx + x + i;	
			for(j=0; j<abs(length); j++)
			{
				//change pixel color

				*p32 = plot_color;
				p32 += nx;
			}
		    }
		}
		break;

	}

	return;

} /* end overlay_line()*/


/************************************************************************/
/* overlay_rectangle()							*/
/*									*/
/* nx = picture width in pixels (including any pixel multiplier)	*/
/* ny = picture height in pixels (including any pixel multiplier)	*/
/* *picture = pointer to picture data returned by make_picture()	*/
/* ulx = rectangle starting point x coordinate, 0..nx-1, 0 at left	*/
/* uly = rectangle starting point y coordinate, 0..ny-1, 0 at top	*/ 
/*	(+ down)							*/
/* lrx = lower right corner x coordinate				*/
/* lry = lower right corner y coordinate				*/
/* width = line width including starting pixel, - values widen the	*/
/* 	line in the outward from the rectangle. + values widen inward.	*/
/*									*/
/* Draws a rectangle over existing picture data pointed to by *picture.	*/
/*									*/
/************************************************************************/

void overlay_rectangle(int nx, int ny, char *picture, int ulx, int uly, 
		int lrx, int lry, int width, int red, int green, int blue)
{
	// draw the top line

	overlay_line(nx,ny,picture,ulx,uly,(lrx-ulx+1),-width, 1, 
			red, green, blue);
	
	// draw the bottom line

	overlay_line(nx,ny,picture,ulx,lry,(lrx-ulx+1),width, 1, 
			red, green, blue);
	
	// draw the left side line

	overlay_line(nx,ny,picture,ulx,uly,(lry-uly+1),width, 0, 
			red, green, blue);

	// draw the right side line

	overlay_line(nx,ny,picture,lrx,uly,(lry-uly+1),-width, 0, 
			red, green, blue);

	return;
	
}
	
