/************************************************************************/
/* ople_fringe_track.c							*/
/*                                                                      */
/* Routines for sending fringe tracking targets to the OPLE VME rack.	*/
/* These should be more or less stand alone routines and not		*/
/* require other libraries in teh CHARA system to work.			*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : March 2010							*/
/************************************************************************/

/*
 * Include files
 */

#include "opleft.h"
#include <netdb.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <resolv.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/tcp.h>

#define OPLE_FT_MACHINE "vme1.chara-array.org"
#define OPLE_FT_PORT 	6001

/* 
 * Local globals
 */

static int	ople_ft_socket = -1;
static double	fringe_tracker_targets[NUM_DELAY_LINES];

/************************************************************************/
/* open_ople_fringe_tracking_socket()					*/
/*									*/
/* Tries to open a socket for talking to the fringe tracking targets	*/
/* on the OPLE VME computer system.					*/
/* Returns:								*/
/* 0 - All went well.							*/
/* -1 - Failed to resolve the vme1 machine.				*/
/* -2 - Failed to create a socket 					*/
/* -3 - Failed to turn off nangle.					*/
/* -4 - Failed to connect to socket.					*/
/************************************************************************/

int open_ople_fringe_tracking_socket(void)
{
        struct sockaddr_in server; 	/* Server address */
        struct hostent *hp;     	/* Pointer to host data */
	int	nagle;
	int	i;

	if (ople_ft_socket != -1) close(ople_ft_socket);

        /* First see if we can identify the host. */

        if((hp = gethostbyname(OPLE_FT_MACHINE)) == 0) return -1;

        /* Create a socket */

        if ((ople_ft_socket = socket(AF_INET, SOCK_STREAM, 0)) < 0) return -2;

	/* We need to disable Nagle */

	nagle = 1;
	if (setsockopt(ople_ft_socket, IPPROTO_TCP,TCP_NODELAY, 
		(char *)&nagle, sizeof(nagle) ))
	{
		close(ople_ft_socket);
		ople_ft_socket = -1;
		return -3;
	}

        /* Create server address */

        server.sin_family = AF_INET;
        memcpy((char *)&server.sin_addr, (char *)hp->h_addr, hp->h_length);
        server.sin_port = htons(OPLE_FT_PORT);

        /* Let's try and connect now.... */

        if (connect(ople_ft_socket, (struct sockaddr *)&server, 
		sizeof(server)) < 0)
        {
                close(ople_ft_socket);
		ople_ft_socket = -1;
                return -4;
        }

	for(i=0; i<NUM_DELAY_LINES; i++) fringe_tracker_targets[i] = 0.0;
	send_fringe_tracking_targets();

	/* All is well it seems */

	return 0;

} /* open_ople_fringe_tracking_socket() */

/************************************************************************/
/* close_ople_fringe_tracking_socket()					*/
/*									*/
/* Closes the socket to the VME rack frgine tracking targets.		*/
/************************************************************************/

void close_ople_fringe_tracking_socket(void)
{
	if (ople_ft_socket != -1)
	{
		close(ople_ft_socket);
		ople_ft_socket = -1;
	}

} /* close_ople_fringe_tracking_socket() */

/************************************************************************/
/* current_fringe_tracking_targets()					*/
/*									*/
/* Copies current fringe tracking target to an array.			*/
/* Numbers are in Metre and not corrected for differential dispersion.	*/
/* Returns 0 if all goes well, and -1 if it doesn't.			*/
/************************************************************************/

void current_fringe_tracking_targets(double targets[6])
{
	int i;

	for ( i=0; i<NUM_DELAY_LINES; i++)
	{
		targets[i] = fringe_tracker_targets[i];
	}

} /* current_fringe_tracking_targets() */

/************************************************************************/
/* set_fringe_tracking_targets()					*/
/*									*/
/* Set fringe tracking targets in Metres. Order of carts is:		*/
/* S1 - S2 - E1 - E2 - W1 - W2						*/
/* In this function the current targets are set to the user targets.	*/
/* Numbers are in Metre and not corrected for differential dispersion.	*/
/* Returns 0 if all goes well, and -1 if it doesn't.			*/
/************************************************************************/

int set_fringe_tracking_targets(double targets[6])
{
	int	i;

	for ( i=0; i<NUM_DELAY_LINES; i++)
	{
		fringe_tracker_targets[i] = targets[i];
	}

	return send_fringe_tracking_targets();
	
} /* set_fringe_tracking_targets() */

/************************************************************************/
/* incriment_fringe_tracking_targets()					*/
/*									*/
/* Set fringe tracking targets in Metres. Order of carts is:		*/
/* S1 - S2 - E1 - E2 - W1 - W2						*/
/* In this function the user sent array is added to the current		*/
/* targets.								*/
/* Numbers are in Metre and not corrected for differential dispersion.	*/
/* Returns 0 if all goes well, and -1 if it doesn't.			*/
/************************************************************************/

int incriment_fringe_tracking_targets(double targets[6])
{
	int	i;

	for ( i=0; i<NUM_DELAY_LINES; i++)
	{
		fringe_tracker_targets[i] += targets[i];
	}

	return send_fringe_tracking_targets();
	
} /* incriment_fringe_tracking_targets() */

/************************************************************************/
/* send_fringe_tracking_targets()					*/
/*									*/
/* Try and send the current targets to the VME rack. 			*/
/* Returns 0 if all goes well, and -1 if it doesn't.			*/
/************************************************************************/

int send_fringe_tracking_targets(void)
{
	
	int	i,j;
	double	targets[NUM_DELAY_LINES];
	char	*c1, *c2;

	/* Copy with reverse byte order */

	for(i=0; i<NUM_DELAY_LINES; i++)
	{
		for(c1 = (char *)(targets+i),
		    c2 = ((char *)(fringe_tracker_targets+i))+sizeof(double)-1,
		    j =0;
		    j < sizeof(double);
		    j++, c1++, c2--) *c1 = *c2;
	}

	if (write(ople_ft_socket, targets, NUM_DELAY_LINES * sizeof(double)) !=
			NUM_DELAY_LINES * sizeof(double)) return -1;

	return 0;

} /* send_fringe_tracking_targets() */		
