/************************************************************************/
/* picom_control.c							*/
/*                                                                      */
/* User-level routines to talk to PICOMOTOR Multi-Axis Driver box.	*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Nils Turner							*/
/* Date   : 22 February 2001						*/
/************************************************************************/

#include "../charaui/src/charaui.h"
#include "picom.h"

/************************************************************************/
/* call_pico_idn()							*/
/*                                                                      */
/* Call pico_idn and display the reply.					*/
/************************************************************************/

int call_pico_idn(int argc, char **argv)
{
	char	s[256];

	if (pico_idn(s))
	{
		message(system_window,s);
		return NOERROR;
	}
	else
	{
		return error(ERROR,"Picomotor port is not open.");
	}

} /* call_pico_idn() */

/************************************************************************/
/* call_pico_reset()							*/
/*                                                                      */
/* Call pico_reset and display status information.			*/
/************************************************************************/

int call_pico_reset(int argc, char **argv)
{
        if (pico_reset())
        {
		message(system_window,"NF 8732 successfully reset.");
		return NOERROR;
        }
	else
	{
		return error(ERROR,"NF 8732 not reset.");
	}
} /* call_pico_reset() */

/************************************************************************/
/* call_pico_go_to_local()						*/
/*                                                                      */
/* Call pico_go_to_local and display status infomation.			*/
/************************************************************************/

int call_pico_go_to_local(int argc, char **argv)
{
        if (pico_go_to_local())
        {
		message(system_window,"NF 8732 in local mode.");
		return NOERROR;
        }
	else
	{
		return error(ERROR,"NF 8732 in unknown mode.");
	}
} /* call_pico_go_to_local() */

/************************************************************************/
/* call_pico_set_channel()						*/
/*                                                                      */
/* Call pico_set_channel and display status information.		*/
/************************************************************************/

int call_pico_set_channel(int argc, char **argv)
{
       	char	s[256];
	int	channel;
	int	status;
	int	slot;
	int	connector;
	int	axis;

	/* Handle command line arguments */

	if (argc > 1)
	{
		sscanf(argv[1],"%d",&channel);
	}
	else
	{
		channel = 111;
		sprintf(s,"%5d",channel);

		clean_command_line();
		if (quick_edit("Channel",s,s,NULL,INTEGER)==KEY_ESC)
		{
			return NOERROR;
		}

		channel = atoi(s);
	}

	status = pico_set_channel(channel);
	if (status == 0)
	{
		return error(ERROR,"Port not open.");
	}
	else if (status < 0)
	{
		return error(ERROR,"Channel out of range.");
	}
	else
	{
		slot = channel/100;
		connector = (channel - 100*slot)/10;
		axis = channel - 100*slot - 10*connector;
		message(system_window,"Slot: %d Connector: %d Axis: %d",slot,
			connector,axis);
		return NOERROR;
	}
} /* call_pico_set_channel() */

/************************************************************************/
/* call_pico_get_channel()						*/
/*                                                                      */
/* Call pico_get_channel and display the reply				*/
/************************************************************************/

int call_pico_get_channel(int argc, char **argv)
{
	int	channel;
	int	slot;
	int	connector;
	int	axis;

	channel = pico_get_channel();
	if (channel)
	{
		slot = channel/100;
		connector = (channel - 100*slot)/10;
		axis = channel - 100*slot - 10*connector;
		message(system_window,"Slot: %d Connector: %d Axis: %d",slot,
			connector,axis);
		return NOERROR;
	}
	else
	{
		return error(ERROR,"Picomotor port is not open.");
	}
} /* call_pico_get_channel() */

/************************************************************************/
/* call_pico_set_frequency()						*/
/*                                                                      */
/* Call pico_set_frequency and display the reply.			*/
/************************************************************************/

int call_pico_set_frequency(int argc, char **argv)
{
       	char	s[256];
	double	frequency;
	int	status;

	/* Handle command line arguments */

	if (argc > 1)
	{
		sscanf(argv[1],"%lf",&frequency);
	}
	else
	{
		frequency = INITFREQ;
		sprintf(s,"%5f",frequency);

		clean_command_line();
		if (quick_edit("Frequency",s,s,NULL,FLOAT)==KEY_ESC)
		{
			return NOERROR;
		}

		frequency = atof(s);
	}

	status = pico_set_frequency(frequency);
	if (status == 0)
	{
		return error(ERROR,"Port not open.");
	}
	else if (status < 0)
	{
		return error(ERROR,"Frequency out of range.");
	}
	else
	{
		pico_get_frequency(s);
		message(system_window,"Pulse frequency is %s",s);
		return NOERROR;
	}
} /* call_pico_set_frequency() */

/************************************************************************/
/* call_pico_init_frequency()						*/
/*                                                                      */
/* Call pico_init_frequency and display the reply.			*/
/************************************************************************/

int call_pico_init_frequency(int argc, char **argv)
{
       	char	s[256];
	double	frequency;
	int	status;

	/* Handle command line arguments */

	if (argc > 1)
	{
		sscanf(argv[1],"%lf",&frequency);
	}
	else
	{
		frequency = INITFREQ;
		sprintf(s,"%5f",frequency);

		clean_command_line();
		if (quick_edit("Frequency",s,s,NULL,FLOAT)==KEY_ESC)
		{
			return NOERROR;
		}

		frequency = atof(s);
	}

	status = pico_init_frequency(frequency);
	if (status == 0)
	{
		return error(ERROR,"Port not open.");
	}
	else if (status < 0)
	{
		return error(ERROR,"Frequency out of range.");
	}
	else
	{
		pico_get_frequency(s);
		message(system_window,"Pulse frequency is %s",s);
		return NOERROR;
	}
} /* call_pico_init_frequency() */

/************************************************************************/
/* call_pico_get_frequency()						*/
/*                                                                      */
/* Call pico_get_frequency and display the reply.			*/
/************************************************************************/

int call_pico_get_frequency(int argc, char **argv)
{
	char	s[256];

	if (pico_get_frequency(s))
	{
		message(system_window,"Pulse frequency is %s",s);
		return NOERROR;
	}
	else
	{
		return error(ERROR,"Picomotor port is not open.");
	}
} /* call_pico_get_frequency() */

/************************************************************************/
/* call_pico_op_complete()						*/
/*                                                                      */
/* Call pico_op_complete and display the reply				*/
/************************************************************************/

int call_pico_op_complete(int argc, char **argv)
{
	if (pico_op_complete())
	{
		message(system_window,"Operation complete");
	}
	else
	{
		message(system_window,"Operation not complete");
	}

	return NOERROR;

} /* call_pico_op_complete() */

/************************************************************************/
/* call_pico_run_one_axis()						*/
/*                                                                      */
/* Calls pico_run_one_axis.						*/
/************************************************************************/

int call_pico_run_one_axis(int argc, char **argv)
{
	int		pulses;
	int		channel;
	int		dirc;
	char		s[256];
	PseudoConn	motor;

	pico_init_pseudo_conn(&motor);

	if (argc > 3)
	{
		sscanf(argv[1],"%d",&pulses);
		sscanf(argv[2],"%d",&channel);
		sscanf(argv[3],"%d",&dirc);
	}
	else
	{
		pulses = 10;
		sprintf(s,"%5d",pulses);

		clean_command_line();
		if (quick_edit("Pulses",s,s,NULL,INTEGER)==KEY_ESC)
		{
			return NOERROR;
		}

		pulses = atoi(s);

		channel = 111;
		sprintf(s,"%5d",channel);

		clean_command_line();
		if (quick_edit("Channel",s,s,NULL,INTEGER)==KEY_ESC)
		{
			return NOERROR;
		}

		channel = atoi(s);

		dirc = 1;
		sprintf(s,"%5d",dirc);

		clean_command_line();
		if (quick_edit("Direction",s,s,NULL,INTEGER)==KEY_ESC)
		{
			return NOERROR;
		}

		dirc = atoi(s);
	}

	motor.slot_a = channel/100;
	motor.conn_a = (channel - motor.slot_a*100)/10;
	motor.axis_a = channel - motor.slot_a*100 - motor.conn_a*10;
	motor.dirc_a = dirc;

	pico_run_one_axis(pulses,&motor);

	return NOERROR;

} /* call_pico_run_one_axis() */

/************************************************************************/
/* call_pico_run_two_axis()						*/
/*                                                                      */
/* Calls pico_run_two_axis.						*/
/************************************************************************/

int call_pico_run_two_axis(int argc, char **argv)
{
	int		pulses;
	int		channel_a;
	int		direction_a;
	int		channel_b;
	int		direction_b;
	char		s[256];
	PseudoConn	motor;

	pico_init_pseudo_conn(&motor);

	if (argc > 5)
	{
		sscanf(argv[1],"%d",&pulses);
		sscanf(argv[2],"%d",&channel_a);
		sscanf(argv[3],"%d",&direction_a);
		sscanf(argv[4],"%d",&channel_b);
		sscanf(argv[5],"%d",&direction_b);
	}
	else
	{
		pulses = 10;
		sprintf(s,"%d",pulses);

		clean_command_line();
		if (quick_edit("Pulses",s,s,NULL,INTEGER)==KEY_ESC)
		{
			return NOERROR;
		}

		pulses = atoi(s);

		channel_a = 111;
		sprintf(s,"%5d",channel_a);

		clean_command_line();
		if (quick_edit("Channel A",s,s,NULL,INTEGER)==KEY_ESC)
		{
			return NOERROR;
		}

		channel_a = atoi(s);

		direction_a = 1;
		sprintf(s,"%5d",direction_a);

		clean_command_line();
		if (quick_edit("Direction A",s,s,NULL,INTEGER)==KEY_ESC)
		{
			return NOERROR;
		}

		direction_a = atoi(s);

		channel_b = 111;
		sprintf(s,"%5d",channel_b);

		clean_command_line();
		if (quick_edit("Channel B",s,s,NULL,INTEGER)==KEY_ESC)
		{
			return NOERROR;
		}

		channel_b = atoi(s);

		direction_b = 1;
		sprintf(s,"%5d",direction_b);

		clean_command_line();
		if (quick_edit("Direction B",s,s,NULL,INTEGER)==KEY_ESC)
		{
			return NOERROR;
		}

		direction_b = atoi(s);
	}

	motor.slot_a = channel_a/100;
	motor.conn_a = (channel_a - motor.slot_a*100)/10;
	motor.axis_a = channel_a - motor.slot_a*100 - motor.conn_a*10;
	motor.dirc_a = direction_a;
	motor.slot_b = channel_b/100;
	motor.conn_b = (channel_b - motor.slot_b*100)/10;
	motor.axis_b = channel_b - motor.slot_b*100 - motor.conn_b*10;
	motor.dirc_b = direction_b;

	pico_run_two_axis(pulses,&motor);

	return NOERROR;

} /* call_pico_run_two_axis() */

/************************************************************************/
/* pico_control_single_mirror()						*/
/*                                                                      */
/* For controling a mirror for which there is only one motor. 		*/
/* Returns ERRORLEVEL.							*/
/************************************************************************/

int pico_control_single_mirror(char *name, int pulses, int chan)
{
	char		s[256];
	PseudoConn	motor_up;
	PseudoConn	motor_down;
	bool		loop;
	int		response;
	int		total;

	pico_init_pseudo_conn(&motor_up);
	pico_init_pseudo_conn(&motor_down);

	motor_up.slot_a = chan/100;
	motor_up.conn_a = (chan - motor_up.slot_a*100)/10;
	motor_up.axis_a = chan - motor_up.slot_a*100 - motor_up.conn_a*10;
	motor_up.dirc_a = FORWARD;

	motor_down.slot_a = motor_up.slot_a;
	motor_down.conn_a = motor_up.conn_a;
	motor_down.axis_a = motor_up.axis_a;
	motor_down.dirc_a = BACKWARD;

	/* OK here we go. */

	active_window = sub_main_window;
	werase(sub_main_window);
	sprintf(s,"CONTROLING MIRROR %s",name);
	heading(heading_window,s,"");
	
	total = 0;
	mvwprintw(sub_main_window,1,30,"k   - Up");
	mvwprintw(sub_main_window,2,30,"j   - Down");
	mvwprintw(sub_main_window,3,30,"p   - Pulses (%4d)",pulses);
	mvwprintw(sub_main_window,4,30,"ESC - Quit function");
	loop = TRUE;
	while(loop)
	{
		mvwprintw(sub_main_window,5,30,"POS = %9d",total);
		wrefresh(sub_main_window);

		/* Is there a socket user? */

		if (active_socket != -1)
		{
			socket_print(active_socket,"k   - Up\n");
			socket_print(active_socket,"j   - Down\n");
			socket_print(active_socket,"p   - Pulses (%4d)\n",
				pulses);
			socket_print(active_socket,"q   - Quit function\n");
			socket_print(active_socket,"POS = %9d\n",total);
		}

		/* Wait for a user to respond */

		if (active_socket == -1)
		{
			while(!kbhit()) background();
		}
		else
		{
			while(!kbhit() && !char_waiting(active_socket)) 
				background();

		}

		/* Get the user's reposne */

		if (kbhit())
		{
			response = get_command();
		}
		else
		{
			if (socket_scan(active_socket,"%s",s) <= 0 || *s == 'q')
				response = KEY_ESC;
			else
				response = *s;
		}

		/* React to user's request */

		switch(response)
		{
			case KEY_UP:
			case 'k': pico_run_one_axis(pulses,&motor_up);
			 	  total += pulses;
				  break;
		
			case KEY_DOWN:
			case 'j': pico_run_one_axis(pulses,&motor_down);
			 	  total -= pulses;
				  break;
		
			case 'p': sprintf(s,"%5d",pulses);
				  clean_command_line();
				  if (quick_edit("Pusles",s,s,NULL,INTEGER)!=
					KEY_ESC) sscanf(s,"%d",&pulses);
				  mvwprintw(sub_main_window,3,30,
					"p   - Pulses (%4d)",pulses);
				  wrefresh(sub_main_window);
				  clean_command_line();
				  break;

			case KEY_ESC: loop = FALSE;
				  break;
		}
	}

	if (active_socket != -1 && active_menuwindow != NULL)
	{
		touchwin(active_menuwindow);
		wrefresh(active_menuwindow);
        }

	return NOERROR;

} /* pico_control_single_mirror() */

/************************************************************************/
/* call_pico_control_single_mirror()					*/
/*                                                                      */
/* Calls pico_control_single_mirror.					*/
/************************************************************************/

int call_pico_control_single_mirror(int argc, char **argv)
{
	char	name[80];
	char	s[256];
	int	pulses;
	int	chan;

	no_socket();

	if (argc > 1)
		strcpy(name,argv[1]);
	else
	{
		sprintf(name,"MIRROR");
		if (quick_edit("Mirror name",name,name,NULL,STRING) ==
					KEY_ESC) return NOERROR;
	}

	if (argc > 2)
	{
		sscanf(argv[2],"%d",&pulses);
	}
	else
	{
		sprintf(s,"   100");
		if (quick_edit("Number of pulses",s,s,NULL,INTEGER) ==
					KEY_ESC) return NOERROR;
		sscanf(s,"%d",&pulses);
	}

	if (argc > 3)
	{
		sscanf(argv[3],"%d",&chan);
	}
	else
	{
		sprintf(s,"   111");
		if (quick_edit("Azimuth chanel",s,s,NULL,INTEGER) ==
					KEY_ESC) return NOERROR;
		sscanf(s,"%d",&chan);
	}
	return pico_control_single_mirror(name, pulses, chan);

} /* call_pico_control_single_mirror() */

/************************************************************************/
/* pico_control_uncoupled_mirror()					*/
/*                                                                      */
/* For controling a mirror for which the two motors not coupled. That 	*/
/* is, they control perpendicular axis.					*/
/* Returns ERRORLEVEL.							*/
/************************************************************************/

int pico_control_uncoupled_mirror(char *name, int pulses, int az, int el)
{
	char		s[256];
	PseudoConn	motor_up;
	PseudoConn	motor_down;
	PseudoConn	motor_left;
	PseudoConn	motor_right;
	bool		loop;
	int		total_az,total_el;
	int		response;

	pico_init_pseudo_conn(&motor_up);
	pico_init_pseudo_conn(&motor_down);
	pico_init_pseudo_conn(&motor_left);
	pico_init_pseudo_conn(&motor_right);

	motor_up.slot_a = el/100;
	motor_up.conn_a = (el - motor_up.slot_a*100)/10;
	motor_up.axis_a = el - motor_up.slot_a*100 - motor_up.conn_a*10;
	motor_up.dirc_a = FORWARD;

	motor_down.slot_a = motor_up.slot_a;
	motor_down.conn_a = motor_up.conn_a;
	motor_down.axis_a = motor_up.axis_a;
	motor_down.dirc_a = BACKWARD;

	motor_left.slot_a = az/100;
	motor_left.conn_a = (az - motor_left.slot_a*100)/10;
	motor_left.axis_a =az-motor_left.slot_a*100-motor_left.conn_a*10;
	motor_left.dirc_a = FORWARD;

	motor_right.slot_a = motor_left.slot_a;
	motor_right.conn_a = motor_left.conn_a;
	motor_right.axis_a = motor_left.axis_a;
	motor_right.dirc_a = BACKWARD;

	/* OK here we go. */

	active_window = sub_main_window;
	werase(sub_main_window);
	sprintf(s,"CONTROLING MIRROR %s",name);
	heading(heading_window,s,"");
	
	total_az = total_el = 0;
	mvwprintw(sub_main_window,0,30,"k   - Up");
	mvwprintw(sub_main_window,1,30,"j   - Down");
	mvwprintw(sub_main_window,2,30,"h   - Left");
	mvwprintw(sub_main_window,3,30,"l   - Right");
	mvwprintw(sub_main_window,4,30,"p   - Pulses (%4d)",pulses);
	mvwprintw(sub_main_window,5,30,"ESC - Quit function");
	loop = TRUE;
	while(loop)
	{
		mvwprintw(sub_main_window,6,30,"AZ = %9d",total_az);
		mvwprintw(sub_main_window,7,30,"EL = %9d",total_el);
		wrefresh(sub_main_window);

		if (active_socket != -1)
		{
			socket_print(active_socket,"k   - Up\n");
			socket_print(active_socket,"j   - Down\n");
			socket_print(active_socket,"h   - Left\n");
			socket_print(active_socket,"l   - Right\n");
			socket_print(active_socket,"p   - Pulses (%4d)\n",
				pulses);
			socket_print(active_socket,"q   - Quit function\n");
			socket_print(active_socket,"AZ = %9d\n",total_az);
			socket_print(active_socket,"EL = %9d\n",total_el);
		}

		/* Wait for a user to respond */

		if (active_socket == -1)
		{
			while(!kbhit()) background();
		}
		else
		{
			while(!kbhit() && !char_waiting(active_socket)) 
				background();

		}

		/* Get the user's reposne */

		if (kbhit())
		{
			response = get_command();
		}
		else
		{
			if (socket_scan(active_socket,"%s",s) <= 0 || *s == 'q')
				response = KEY_ESC;
			else
				response = *s;
		}

		switch(response)
		{
			case KEY_UP:
			case 'k': pico_run_one_axis(pulses,&motor_up);
			 	  total_el += pulses;
				  break;
		
			case KEY_DOWN:
			case 'j': pico_run_one_axis(pulses,&motor_down);
			 	  total_el -= pulses;
				  break;
		
			case KEY_LEFT:
			case 'h': pico_run_one_axis(pulses,&motor_left);
			 	  total_az -= pulses;
				  break;
		
			case KEY_RIGHT:
			case 'l': pico_run_one_axis(pulses,&motor_right);
			 	  total_az += pulses;
				  break;

			case 'p': sprintf(s,"%5d",pulses);
				  clean_command_line();
				  if (quick_edit("Pusles",s,s,NULL,INTEGER)!=
					KEY_ESC) sscanf(s,"%d",&pulses);
				  mvwprintw(sub_main_window,4,30,
					"p   - Pulses (%4d)",pulses);
				  wrefresh(sub_main_window);
				  clean_command_line();
				  break;

			case KEY_ESC: loop = FALSE;
				  break;
		}
	}

	if (active_socket != -1 && active_menuwindow != NULL)
	{
		touchwin(active_menuwindow);
		wrefresh(active_menuwindow);
        }

	return NOERROR;

} /* pico_control_uncoupled_mirror() */

/************************************************************************/
/* call_pico_control_uncoupled_mirror()					*/
/*                                                                      */
/* Calls pico_control_uncoupled_mirror.					*/
/************************************************************************/

int call_pico_control_uncoupled_mirror(int argc, char **argv)
{
	char	name[80];
	char	s[256];
	int	pulses;
	int	az;
	int	el;

	no_socket();

	if (argc > 1)
		strcpy(name,argv[1]);
	else
	{
		sprintf(name,"MIRROR");
		if (quick_edit("Mirror name",name,name,NULL,STRING) ==
					KEY_ESC) return NOERROR;
	}

	if (argc > 2)
	{
		sscanf(argv[2],"%d",&pulses);
	}
	else
	{
		sprintf(s,"   100");
		if (quick_edit("Number of pulses",s,s,NULL,INTEGER) ==
					KEY_ESC) return NOERROR;
		sscanf(s,"%d",&pulses);
	}

	if (argc > 3)
	{
		sscanf(argv[3],"%d",&az);
	}
	else
	{
		sprintf(s,"   111");
		if (quick_edit("Azimuth chanel",s,s,NULL,INTEGER) ==
					KEY_ESC) return NOERROR;
		sscanf(s,"%d",&az);
	}

	if (argc > 4)
	{
		sscanf(argv[3],"%d",&el);
	}
	else
	{
		sprintf(s,"   111");
		if (quick_edit("Elevation chanel",s,s,NULL,INTEGER) ==
					KEY_ESC) return NOERROR;
		sscanf(s,"%d",&el);
	}

	return pico_control_uncoupled_mirror(name, pulses, az, el);

} /* call_pico_control_uncoupled_mirror() */

/************************************************************************/
/* pico_control_coupled_mirror()					*/
/*                                                                      */
/* For controling a mirror for which the two motors are coupled. That 	*/
/* is, they control non-perpendicular axis in a triangle.		*/
/* UP and DOWN are not really going to be UP and DOWN.			*/
/* Returns ERRORLEVEL.							*/
/************************************************************************/

int pico_control_coupled_mirror(char *name, int pulses, int one, int two)
{
	char		s[256];
	PseudoConn	motor_up;
	PseudoConn	motor_down;
	PseudoConn	motor_left;
	PseudoConn	motor_right;
	bool		loop;
	int		total_az,total_el;
	int		response;

	pico_init_pseudo_conn(&motor_up);
	pico_init_pseudo_conn(&motor_down);
	pico_init_pseudo_conn(&motor_left);
	pico_init_pseudo_conn(&motor_right);

	motor_up.slot_a = two/100;
	motor_up.conn_a = (two - motor_up.slot_a*100)/10;
	motor_up.axis_a = two - motor_up.slot_a*100 - motor_up.conn_a*10;
	motor_up.dirc_a = FORWARD;
	motor_up.slot_b = one/100;
	motor_up.conn_b = (one - motor_up.slot_b*100)/10;
	motor_up.axis_b = one - motor_up.slot_b*100 - motor_up.conn_b*10;
	motor_up.dirc_b = FORWARD;

	motor_down.slot_a = two/100;
	motor_down.conn_a = (two - motor_down.slot_a*100)/10;
	motor_down.axis_a = two - motor_down.slot_a*100 - motor_down.conn_a*10;
	motor_down.dirc_a = BACKWARD;
	motor_down.slot_b = one/100;
	motor_down.conn_b = (one - motor_down.slot_b*100)/10;
	motor_down.axis_b = one - motor_down.slot_b*100 - motor_down.conn_b*10;
	motor_down.dirc_b = BACKWARD;

	motor_left.slot_a = two/100;
	motor_left.conn_a = (two - motor_left.slot_a*100)/10;
	motor_left.axis_a = two - motor_left.slot_a*100 - motor_left.conn_a*10;
	motor_left.dirc_a = FORWARD;
	motor_left.slot_b = one/100;
	motor_left.conn_b = (one - motor_left.slot_b*100)/10;
	motor_left.axis_b = one - motor_left.slot_b*100 - motor_left.conn_b*10;
	motor_left.dirc_b = BACKWARD;

	motor_right.slot_a = two/100;
	motor_right.conn_a = (two - motor_right.slot_a*100)/10;
	motor_right.axis_a = two-motor_right.slot_a*100 - motor_right.conn_a*10;
	motor_right.dirc_a = BACKWARD;
	motor_right.slot_b = one/100;
	motor_right.conn_b = (one - motor_right.slot_b*100)/10;
	motor_right.axis_b = one-motor_right.slot_b*100 - motor_right.conn_b*10;
	motor_right.dirc_b = FORWARD;

	/* OK here we go. */

	active_window = sub_main_window;
	werase(sub_main_window);
	sprintf(s,"CONTROLING MIRROR %s",name);
	heading(heading_window,s,"");
	
	total_az = total_el = 0;
	mvwprintw(sub_main_window,0,30,"k   - Up");
	mvwprintw(sub_main_window,1,30,"j   - Down");
	mvwprintw(sub_main_window,2,30,"h   - Left");
	mvwprintw(sub_main_window,3,30,"l   - Right");
	mvwprintw(sub_main_window,4,30,"p   - Pulses (%4d)",pulses);
	mvwprintw(sub_main_window,5,30,"ESC - Quit function");
	loop = TRUE;
	while(loop)
	{
		mvwprintw(sub_main_window,6,30,"AZ = %9d",total_az);
		mvwprintw(sub_main_window,7,30,"EL = %9d",total_el);
		wrefresh(sub_main_window);

		if (active_socket != -1)
		{
			socket_print(active_socket,"k   - Up\n");
			socket_print(active_socket,"j   - Down\n");
			socket_print(active_socket,"h   - Left\n");
			socket_print(active_socket,"l   - Right\n");
			socket_print(active_socket,"p   - Pulses (%4d)\n",
				pulses);
			socket_print(active_socket,"q   - Quit function\n");
			socket_print(active_socket,"AZ = %9d\n",total_az);
			socket_print(active_socket,"EL = %9d\n",total_el);
		}

		/* Wait for a user to respond */

		if (active_socket == -1)
		{
			while(!kbhit()) background();
		}
		else
		{
			while(!kbhit() && !char_waiting(active_socket)) 
				background();

		}

		/* Get the user's reposne */

		if (!kbhit())
		{
			response = get_command();
		}
		else
		{
			if (socket_scan(active_socket,"%s",s) <= 0 || *s == 'q')
				response = KEY_ESC;
			else
				response = *s;
		}

		switch(response)
		{
			case KEY_UP:
			case 'k': pico_run_two_axis(pulses,&motor_up);
			 	  total_el += pulses;
				  break;
		
			case KEY_DOWN:
			case 'j': pico_run_two_axis(pulses,&motor_down);
			 	  total_el -= pulses;
				  break;
		
			case KEY_LEFT:
			case 'h': pico_run_two_axis(pulses,&motor_left);
			 	  total_az -= pulses;
				  break;
		
			case KEY_RIGHT:
			case 'l': pico_run_two_axis(pulses,&motor_right);
			 	  total_az += pulses;
				  break;

			case 'p': sprintf(s,"%5d",pulses);
				  clean_command_line();
				  if (quick_edit("Pusles",s,s,NULL,INTEGER)!=
					KEY_ESC) sscanf(s,"%d",&pulses);
				  mvwprintw(sub_main_window,4,30,
					"p   - Pulses (%4d)",pulses);
				  wrefresh(sub_main_window);
				  clean_command_line();
				  break;

			case KEY_ESC: loop = FALSE;
				  break;
		}
	}

	if (active_socket != -1 && active_menuwindow != NULL)
	{
		touchwin(active_menuwindow);
		wrefresh(active_menuwindow);
        }

	return NOERROR;

} /* pico_control_coupled_mirror() */

/************************************************************************/
/* call_pico_control_coupled_mirror()					*/
/*                                                                      */
/* Calls pico_control_uncoupled_mirror.					*/
/************************************************************************/

int call_pico_control_coupled_mirror(int argc, char **argv)
{
	char	name[80];
	char	s[256];
	int	pulses;
	int	one;
	int	two;

	no_socket();

	if (argc > 1)
		strcpy(name,argv[1]);
	else
	{
		sprintf(name,"MIRROR");
		if (quick_edit("Mirror name",name,name,NULL,STRING) ==
					KEY_ESC) return NOERROR;
	}

	if (argc > 2)
	{
		sscanf(argv[2],"%d",&pulses);
	}
	else
	{
		sprintf(s,"   100");
		if (quick_edit("Number of pulses",s,s,NULL,INTEGER) ==
					KEY_ESC) return NOERROR;
		sscanf(s,"%d",&pulses);
	}

	if (argc > 3)
	{
		sscanf(argv[3],"%d",&one);
	}
	else
	{
		sprintf(s,"   111");
		if (quick_edit("Chanel one",s,s,NULL,INTEGER) ==
					KEY_ESC) return NOERROR;
		sscanf(s,"%d",&one);
	}

	if (argc > 4)
	{
		sscanf(argv[3],"%d",&two);
	}
	else
	{
		sprintf(s,"   111");
		if (quick_edit("Chanel two",s,s,NULL,INTEGER) ==
					KEY_ESC) return NOERROR;
		sscanf(s,"%d",&two);
	}

	return pico_control_coupled_mirror(name, pulses, one, two);

} /* call_pico_control_coupled_mirror() */

/************************************************************************/
/* control_mirror()							*/
/*									*/
/* Routine for controling a mirror. Calls pico_control_coupled_mirror()	*/
/* or pico_control_uncoupled_mirror() depending on arguments.		*/
/************************************************************************/

int control_mirror(char *name, 
		   int  *pulses,
		   char **names, 
		   int *type, 
		   int *axis_one, 
		   int *axis_two)
{
	int i;

	/* If there's a name there try and find it. */

	if (name != NULL)
	{
		for(i=0; names[i] != NULL; i++)
		{
			if (strcmpi(name, names[i]) == 0) break;
		}
		if (names[i] == NULL)
			return error(ERROR,"Unknown mirror %s",name);
	}
	else
        {
                i = 0;
                if (quick_edit("Mirror",names[i],
                        &i,names,ENUMERATED) == KEY_ESC) return -1;
        }

	/* Call the apropriate routine */

	switch(type[i])
	{
		case COUPLED: 
			return pico_control_coupled_mirror(names[i],pulses[i], 
						axis_one[i], axis_two[i]);

		case UNCOUPLED:
			return pico_control_uncoupled_mirror(names[i],pulses[i], 
						axis_one[i], axis_two[i]);

		case SINGLE:
			return pico_control_single_mirror(names[i], pulses[i], 
						axis_one[i]);

		default: return error(ERROR,"Unknown mirror type for %s.",
					names[i]);
	}
		
} /* control_mirror() */
