/************************************************************************/
/* pop_client.c								*/
/*                                                                      */
/* General routines for talking to the POP server.			*/
/* Much code taken from Dave Berger's code.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar (but mostly taken from popgtk)          */
/* Date   : Oct 2007							*/
/************************************************************************/

#include "../cliserv/include/cliserv.h"
#include "../include/chara_messages.h"
#include "../charalib/chara.h"
#include "popclient.h"

/* Globals */

struct POP pops[N_POPS];
int  pop_server = -1;
static int got_pop_status = -1;

/************************************************************************/
/* open_pop_client()							*/
/*									*/
/* Guess what this does.						*/
/* Assumes that init_jobs has been called. It will call it anyway,	*/
/* which should do no harm if it has already been called.		*/
/* Returns TRUE or FALSE, depending on how things go. TRUE is good.	*/
/************************************************************************/

int open_pop_client(void)
{
	/* Close it if it is open */

	close_pop_client();

	/* This is for safety and should be harmless */

	init_jobs();

	/*
	 * Attempt to open the socket
	 */

	if ((pop_server = open_server_by_name("pop")) == -1) return FALSE;

	/* Fill in the jobs messages we need */

        if (!add_message_job(pop_server, SERVER_STOPPING, 
			message_pop_server_stopping)) return FALSE;

        if (!add_message_job(pop_server, POP_STATUS, message_pop_status)) 
		return FALSE;

	/*
	 * Send out requests for status on all lines
	 */

	if (!get_all_pop_status())
	{
		close_pop_client();
		pop_server = -1;
		return FALSE;
	}

	/* All should be well */

	return TRUE;

} /* open_pop_client() */

/************************************************************************/
/* close_pop_client()							*/
/*									*/
/************************************************************************/

void close_pop_client(void)
{
	if (server_socket_open(pop_server))
	{
		close_server_socket(pop_server);
		pop_server = -1;
	}

} /* close_pop_client() */

/************************************************************************/
/* message_pop_server_stopping()					*/
/*									*/
/************************************************************************/

int message_pop_server_stopping(int server, struct smessage *mess)
{
	close_pop_client();

        return TRUE;

} /* message_pop_server_stopping() */

/************************************************************************/
/* message_pop_status()							*/
/*									*/
/************************************************************************/

int message_pop_status(int server, struct smessage *mess)
{
        struct POP *spop;
	int	id;

        if (mess->length != sizeof(*spop)) return FALSE;

        spop = (struct POP *)mess->data;

	id=spop->id;

        pops[id].id=id;
        strcpy(pops[id].name, spop->name);
        pops[id].status         = spop->status;
        pops[id].position       = spop->position;
        pops[id].status_limit   = spop->status_limit;
        pops[id].status_motor   = spop->status_motor;
        pops[id].status_led     = spop->status_led;
        pops[id].status_timeout = spop->status_timeout;
        pops[id].status_local   = spop->status_local;
	got_pop_status = id;

        return TRUE;

} /* message_pop_status() */

/************************************************************************/
/* get_pop_status()							*/
/*									*/
/* Ask for and wait for the status of a single POP device.		*/
/* Returns TURE if things went well, and FALSE otherwise.		*/
/************************************************************************/

int get_pop_status(int id)
{
	struct smessage mess;
        struct POP spop;
	time_t start;

	/* Is this a real POP? */

	if (id < 0 || id >= N_POPS) return FALSE;

	/* Send out the request */

	got_pop_status = 0;
	spop.id = id;
	mess.type = POP_SEND_STATUS;
	mess.data = (unsigned char *)&spop;
	mess.length = sizeof(spop);

	if (!send_message(pop_server, &mess)) return FALSE;

	/* Wait for a response */

	start = time(NULL);
	while(got_pop_status != id)
	{
		process_server_socket(pop_server);

		if (time(NULL) > start+TIME_OUT) return FALSE;
	}

	return TRUE;

} /* get_pop_status() */

/************************************************************************/
/* get_all_pop_status()							*/
/*									*/
/* Ask for and wait for the status of a all POP devices.		*/
/* Returns TURE if things went well, and FALSE otherwise.		*/
/************************************************************************/

int get_all_pop_status(void)
{
	int	i;

	for (i=0; i<N_POPS; i++)
	{
		if (!get_pop_status(i)) return FALSE;
	}

	return TRUE;

} /* get_all_pop_status() */

/************************************************************************/
/* send_pop_in								*/
/*									*/
/* Sends POP_IN message.						*/
/* Return TRUE if things go well, FALSE otherwise.			*/
/************************************************************************/

int send_pop_in(int id)
{
	struct POP spop;
	struct smessage mess;

	/* Is it a real POP? */
	
	if (id < 0 || id >= N_POPS) return FALSE;

	/* Send the message */

	spop.id = id;
	mess.type = POP_IN;
	mess.data = (unsigned char *)&spop;
        mess.length = sizeof(spop);

        return send_message(pop_server, &mess);

} /* send_pop_in() */

/************************************************************************/
/* send_pop_out								*/
/*									*/
/* Sends POP_OUT message.						*/
/* Return TRUE if things go well, FALSE otherwise.			*/
/************************************************************************/

int send_pop_out(int id)
{
	struct POP spop;
	struct smessage mess;

	/* Is it a real POP? */
	
	if (id < 0 || id >= N_POPS) return FALSE;

	/* Send the message */

	spop.id = id;
	mess.type = POP_OUT;
	mess.data = (unsigned char *)&spop;
        mess.length = sizeof(spop);

        return send_message(pop_server, &mess);

} /* send_pop_out() */

/************************************************************************/
/* send_pop_off								*/
/*									*/
/* Sends POP_OFF message.						*/
/* Return TRUE if things go well, FALSE otherwise.			*/
/************************************************************************/

int send_pop_off(int id)
{
	struct POP spop;
	struct smessage mess;

	/* Is it a real POP? */
	
	if (id < 0 || id >= N_POPS) return FALSE;

	/* Send the message */

	spop.id = id;
	mess.type = POP_OFF;
	mess.data = (unsigned char *)&spop;
        mess.length = sizeof(spop);

        return send_message(pop_server, &mess);

} /* send_pop_off() */

/************************************************************************/
/* send_pop_led_red							*/
/*									*/
/* Sends POP_LED_RED message.						*/
/* Return TRUE if things go well, FALSE otherwise.			*/
/************************************************************************/

int send_pop_led_red(int id)
{
	struct POP spop;
	struct smessage mess;

	/* Is it a real POP? */
	
	if (id < 0 || id >= N_POPS) return FALSE;

	/* Send the message */

	spop.id = id;
	mess.type = POP_LED_RED;
	mess.data = (unsigned char *)&spop;
        mess.length = sizeof(spop);

        return send_message(pop_server, &mess);

} /* send_pop_led_red() */

/************************************************************************/
/* send_pop_led_green							*/
/*									*/
/* Sends POP_LED_GREEN message.						*/
/* Return TRUE if things go well, FALSE otherwise.			*/
/************************************************************************/

int send_pop_led_green(int id)
{
	struct POP spop;
	struct smessage mess;

	/* Is it a real POP? */
	
	if (id < 0 || id >= N_POPS) return FALSE;

	/* Send the message */

	spop.id = id;
	mess.type = POP_LED_GREEN;
	mess.data = (unsigned char *)&spop;
        mess.length = sizeof(spop);

        return send_message(pop_server, &mess);

} /* send_pop_led_green() */

/************************************************************************/
/* send_pop_led_yellow							*/
/*									*/
/* Sends POP_LED_YELLOW message.					*/
/* Return TRUE if things go well, FALSE otherwise.			*/
/************************************************************************/

int send_pop_led_yellow(int id)
{
	struct POP spop;
	struct smessage mess;

	/* Is it a real POP? */
	
	if (id < 0 || id >= N_POPS) return FALSE;

	/* Send the message */

	spop.id = id;
	mess.type = POP_LED_YELLOW;
	mess.data = (unsigned char *)&spop;
        mess.length = sizeof(spop);

        return send_message(pop_server, &mess);

} /* send_pop_led_yellow() */

/************************************************************************/
/* send_pop_led_off							*/
/*									*/
/* Sends POP_LED_OFF message.						*/
/* Return TRUE if things go well, FALSE otherwise.			*/
/************************************************************************/

int send_pop_led_off(int id)
{
	struct POP spop;
	struct smessage mess;

	/* Is it a real POP? */
	
	if (id < 0 || id >= N_POPS) return FALSE;

	/* Send the message */

	spop.id = id;
	mess.type = POP_LED_OFF;
	mess.data = (unsigned char *)&spop;
        mess.length = sizeof(spop);

        return send_message(pop_server, &mess);

} /* send_pop_led_off() */
