/************************************************************************/
/* fits_read.c								*/
/*									*/
/* Simplified and hacked FITS file reader and writer routines.  	*/
/* Routines for reading things in FITs files.				*/
/* Sometimes you need to define BYTE_SWAP for these to work well.	*/
/************************************************************************/
/*                                        	                      	*/
/* Center for High Angular Resolution Astronomy				*/
/*									*/
/* Telephone: 404-651-2932						*/
/* Fax      : 404-651-1384						*/
/* Internet : theo@chara.gsu.edu					*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : April 1997   						*/
/************************************************************************/

#include "rwfits.h"
#include "../nrc/nrc.h"

/************************************************************************/
/* read_fits() 								*/
/*                                                                      */
/* Reads a fits file and returns the FITS_DATA structure or NULL 	*/
/* on error.								*/
/*                                                                      */
/************************************************************************/

fits_data read_fits(char *filename)
{
	FILE *input;
	fits_data result;
	char	line[FITS_LINE_SIZE+1];
	unsigned char block[FITS_BLOCK_SIZE];
	char *p,*s;
	int	i,j,nlines;
	char loop;
	int	bitpix,naxis,naxis1,naxis2;
	float	bscale, bzero;
	char	comment[FITS_LINE_SIZE];
	int1	 byte1; /* BITPIX = 8 One byte signed int */
	int2	 byte2; /* BITPIX = 16 Two byte signed int */
	int4  	 byte4; /* BITPIX = 32 Four byte signed int */
	real4	 float4; /* BITPIX = -32 Four byte signed float */
	real8	 float8; /* BITPIX = -64 Eight byte signed float */
#ifdef DEBUG
	float	min,max;
#endif

	/* Set up defaults */

	result.nheader = 0;
	result.header  = NULL;
	result.nx = 0;
	result.ny = 0;
	result.data = NULL;

	/* Try and open the file */

	if ((input = fopen(filename,"rb")) == NULL)
	{
		fprintf(stderr,
			"read_fits(): Failed to open file %s.\n",filename);
		return result;
	}

	/* Allocate some space for the list of headers */

	result.header = (char **)malloc((sizeof(char *))*(MAXHEADER+1))-1;
	mem_test(result.header);

	/* Now loop through until we find read in the header */

	loop = TRUE;
/*
	nlines = 0; THIS USED TO BE HERE.
*/
	while (loop)
	{
		nlines = 0; /* But Lewis thinks it should be here */

		/* Read in one block of header */

		if (!(loop = read_fits_block(block,input)))
		{
			fprintf(stderr,
			"read_fits(): Reached data end before header end.\n");
			continue;
		}

		p = (char *)block;
		while(++nlines <= (FITS_BLOCK_SIZE/FITS_LINE_SIZE))
		{
			/* Copy a line into the line buffer */

			s = line;
			for(i=0;i<FITS_LINE_SIZE;i++) *s++ = *p++;
			line[FITS_LINE_SIZE] = 0;
				
			/* Is this the end of the header ? */

			if (*line == 'E' &&
			    *(line+1) == 'N' &&
			    *(line+2) == 'D') 
			{
				loop = FALSE;
				break;
			}

			/* Otherwise let's add it to the list */

			++result.nheader;
			result.header[result.nheader] = 
				(char *)malloc(FITS_LINE_SIZE+2);
			mem_test(result.header[result.nheader]);
			strcpy(result.header[result.nheader],line);

			/* Is the memory full */

			if (result.nheader >= MAXHEADER)
			{
			   fprintf(stderr,"read_fits(): Header memory full.\n");
			   break;
			}
		}
	}

	/* Ensure this is of the SIMPLE 'T' type */

	if (!(i = find_key(result, "SIMPLE")))
	{
		fprintf(stderr,"read_fits(): %s has no SIMPLE key.\n",
			filename);
		fclose(input);
		return result;
	}

	for(s = result.header[i]+FITS_KEY_SIZE+1; *s != 'T' && *s != 0; s++);
	if (*s != 'T')
	{
		fprintf(stderr,"read_fits(): %s is not a SIMPLE T file.\n",
			filename);
		fclose(input);
		return result;
	}

	/* Now read in some of the header info required */

	bitpix = read_fits_header_int(result, "BITPIX", comment);
	naxis = read_fits_header_int(result, "NAXIS", comment);
	naxis1 = read_fits_header_int(result, "NAXIS1", comment);
	naxis2 = read_fits_header_int(result, "NAXIS2", comment);
	bscale = read_fits_header_float(result, "BSCALE", comment);
	bzero = read_fits_header_float(result, "BZERO", comment);
#ifdef DEBUG
	printf("BITPIX = %d\n",bitpix);
	printf("NAXIS  = %d\n",naxis);
	printf("NAXIS1 = %d\n",naxis1);
	printf("NAXIS2 = %d\n",naxis2);
	printf("BSCALE = %f\n",bscale);
	printf("BZERO  = %f\n",bzero);
#endif
	/* Can we deal with this ? */
	
	if (bitpix!=8 && bitpix!=16 && bitpix!=32 && bitpix!=-32 && bitpix!=-64)
	{
		fprintf(stderr,
			"read_fits(): Bitpix must be 8, 16, 32, -32 or -64.\n");
		fclose(input);
		return result;
	}

	if (naxis != 2)
	{
		fprintf(stderr,"read_fits(): Sorry only 2x2 data for now.\n");
		fclose(input);
		return result;
	}

	if (bscale < 0.0)
	{
		fprintf(stderr,"read_fits(): bscale = %f rediculous.\n",bscale);
		fclose(input);
		return result;
	}
	if (bscale == 0.0) bscale = 1.0;

	/* Allocate memory for the data array */

	result.nx = naxis1; /* Number of columns */
	result.ny = naxis2; /* Number of rows */
	result.data = matrix(1,naxis1,1,naxis2);
	

	/* OK read in the data */

#ifdef DEBUG
	min = 1e30;
	max = -1e30;
#endif
	for(j=1; j<= naxis2; j++)
	for(i=1; i<= naxis1; i++)
	{
		if (bitpix == 8)
		{
			if (fread(&byte1, 1, 1, input) != 1)
			{
				fprintf(stderr,
					"read_fits(): Ran out of data.\n");
				free_matrix(result.data,1,naxis1,1,naxis2);
				result.nx = 0;
				result.ny = 0;
				return result;
			}

			result.data[i][j] = byte1;
#ifdef DEBUG1
			printf("byte1 = %d\n", byte1);
			printf("result = %f\n", result.data[i][j]);
#endif
		}
		else if (bitpix == 16)
		{
			if (fread(&byte2, 2, 1, input) != 1)
			{
				fprintf(stderr,
					"read_fits(): Ran out of data.\n");
				free_matrix(result.data,1,naxis1,1,naxis2);
				result.nx = 0;
				result.ny = 0;
				return result;
			}

#ifdef BYTE_SWAP
			reverse_byte_order(&byte2, 2);
#endif
			result.data[i][j] = byte2;
#ifdef DEBUG1
			printf("byte2 = %d\n", byte2);
			printf("result = %f\n", result.data[i][j]);
#endif
		}
		else if (bitpix == 32)
		{
			if (fread(&byte4, 4, 1, input) != 1)
			{
				fprintf(stderr,
					"read_fits(): Ran out of data.\n");
				free_matrix(result.data,1,naxis1,1,naxis2);
				result.nx = 0;
				result.ny = 0;
				return result;
			}

#ifdef BYTE_SWAP
			reverse_byte_order(&byte4, 4);
#endif
			result.data[i][j] = byte4;
#ifdef DEBUG1
			printf("byte4 = %d\n", byte4);
			printf("result = %f\n", result.data[i][j]);
#endif
		}
		else if (bitpix == -32)
		{
			if (fread(&float4, 4, 1, input) != 1)
			{
				fprintf(stderr,
					"read_fits(): Ran out of data.\n");
				free_matrix(result.data,1,naxis1,1,naxis2);
				result.nx = 0;
				result.ny = 0;
				return result;
			}

#ifdef BYTE_SWAP
			reverse_byte_order(&float4, 4);
#endif
			result.data[i][j] = float4;
#ifdef DEBUG1
			printf("float4 = %f\n", float4);
			printf("result = %f\n", result.data[i][j]);
#endif
		}
		else if (bitpix == -64)
		{
			if (fread(&float8, 8, 1, input) != 1)
			{
				fprintf(stderr,
					"read_fits(): Ran out of data.\n");
				free_matrix(result.data,1,naxis1,1,naxis2);
				result.nx = 0;
				result.ny = 0;
				return result;
			}

#ifdef BYTE_SWAP
			reverse_byte_order(&float8, 8);
#endif
			result.data[i][j] = float8;
#ifdef DEBUG1
			printf("float8 = %lf\n", float8);
			printf("result = %f\n", result.data[i][j]);
#endif
		}
		else	
		{
			fprintf(stderr,"read_fits(): Unknown data type.\n");
			free_matrix(result.data,1,naxis1,1,naxis2);
			result.nx = 0;
			result.ny = 0;
			return result;
		}

		result.data[i][j] = file_to_physical(result.data[i][j]);

#ifdef DEBUG
		if (result.data[i][j] > max) max = result.data[i][j];
		if (result.data[i][j] < min) min = result.data[i][j];
#endif
	}
#ifdef DEBUG
	printf("Data maximum = %f (%f)\n",max,physical_to_file(max));
	printf("Data minimum = %f (%f)\n",min,physical_to_file(min));
#endif

	/* Close the file and leave */

	fclose(input);

	return result;

} /* read_fits() */

/************************************************************************/
/* read_fits_block()							*/
/*									*/
/* Reads in one 2880 byte block from a fits file.			*/
/* Returns TRUE if all was OK and FALSE otherwise.			*/
/* Assumes memory has been allocated for block of data. 		*/
/************************************************************************/

unsigned char read_fits_block(unsigned char *block, FILE *input)
{
	/* Try and read in the block of data */

	if (fread(block, FITS_BLOCK_SIZE, 1, input) != 1)
	{
	    fprintf(stderr,"read_fits_block(): Failed to read entire block.\n");
	    return FALSE;
	}

	return TRUE;

} /* read_fits_block() */

/************************************************************************/
/* read_fits_header_int()						*/
/* 									*/
/* Get a specific integer value from a fits data header.		*/
/* Returns 0 if none found.						*/
/* Also reads in comment. Comment blank if no number found.		*/
/************************************************************************/

int read_fits_header_int(fits_data data, char *keyword, char *comment)
{
	int i;
	int	result;
	char	*s1;

	/* Did we find it ? */

	if (!(i = find_key(data, keyword)))
	{
		fprintf(stderr,
			"read_fits_header_int(): Keyword %s not found.\n",
			keyword);
		*comment = 0;
		return 0;
	}

	/* OK read in the value */

	sscanf(data.header[i]+FITS_KEY_SIZE+1,"%d",&result);

	/* Now find the comment field */

	for(s1 = data.header[i]+FITS_KEY_SIZE+1; 
		*s1 != '/' && *s1 != 0; s1++); 

	/* And copy it, stripping leading spaces */

	if (*s1++ == 0) 
	{
		*comment = 0;
	}

	else
	{
		while(*s1 == ' ') s1++;
		strcpy(comment,s1);
	}


	/* That is all */

	return result;

} /* read_fits_header_int() */

/************************************************************************/
/* read_fits_header_float()						*/
/* 									*/
/* Get a specific float value from a fits data header.			*/
/* Returns 0 if none found.						*/
/* Also reads in comment. Comment blank if no number found.		*/
/************************************************************************/

float read_fits_header_float(fits_data data, char *keyword, char *comment)
{
	int i;
	float	result;
	char	*s1;

	/* Did we find it ? */

	if (!(i= find_key(data, keyword)))
	{
		fprintf(stderr,
			"read_fits_header_float(): Keyword %s not found.\n",
			keyword);
		*comment = 0;
		return 0;
	}

	/* OK read in the value */

	sscanf(data.header[i]+FITS_KEY_SIZE+1,"%f",&result);

	/* Now find the comment field */

	for(s1 = data.header[i]+FITS_KEY_SIZE+1; 
		*s1 != '/' && *s1 != 0; s1++); 

	/* And copy it, stripping leading spaces */

	if (*s1++ == 0) 
	{
		*comment = 0;
	}

	else
	{
		while(*s1 == ' ') s1++;
		strcpy(comment,s1);
	}


	/* That is all */

	return result;

} /* read_fits_header_float() */

/************************************************************************/
/* read_fits_header_text()						*/
/* 									*/
/* Get a specific text value from a fits data header.			*/
/* Returns NULL string  if none found.					*/
/* Also reads in comment.						*/
/* Returns pointer to resulting string stored in text array.  		*/
/* Does not allocate memory for string.					*/
/************************************************************************/

char *read_fits_header_text(fits_data data, char *keyword,char *s,char *comment)
{
	int i;
	char	*s1,*s2;
	int 	length;

	/* Did we find it ? */

	if (!(i = find_key(data, keyword)))
	{
		fprintf(stderr,
			"read_fits_header_text(): Keyword %s not found.\n",
			keyword);
		*comment = 0;
		*s = 0;
		return 0;
	}

	/* OK find begining of text field */

	for(s1 = data.header[i]+FITS_KEY_SIZE+1; *s1 != '\'' && *s1 != 0; s1++);
	if (*s1 == 0)
	{
                fprintf(stderr,
                        "read_fits_header_text(): Text for %s not found.\n",
                        keyword);
                *s = 0;
        }
	else
	{
		s1++;  /* First character beyong the quote */

		/* Calculate length of string */

		for(s2=s1, length=1; 
			*s2 != '/' && *s2 != '\'' && *s2 != 0; length++,s2++);

		/* Copy the string across */

		for(s2=s; *s1 != '\'' && *s1 != 0;s1++,s2++)
		{
			*s2 = *s1;
		}

		/* Strip trailing white space */

		s2--;
		while(*s2 == ' ') s2--;
		*(s2+1) = 0;
	}

	/* Now find the comment field */

	for(; *s1 != '/' && *s1 != 0; s1++); 

	/* And copy it, stripping leading spaces */

	if (*s1++ == 0) 
	{
		*comment = 0;
	}

	else
	{
		while(*s1 == ' ') s1++;
		strcpy(comment,s1);
	}


	/* That is all */

	return s;

} /* read_fits_header_text() */

/************************************************************************/
/* read_fits_datasec()							*/
/*									*/
/* Looks for a text field in the fits header called DATASEC and tries 	*/
/* read what is in it. The field should contain a string something like */
/* [1:256,1:256], and the idea is to get the numbers inside. Returns	*/
/* TRUE if it found all four numbers and FALSE if it didn't, or if there*/
/* was no filed of the right type.					*/
/************************************************************************/

int read_fits_datasec(fits_data data, int *xmin, int *xmax, int *ymin, int *ymax)
{
	char	field_text[81];
	char	comment[81];

	/* Try and get the text field. */

	if (!read_fits_header_text(data,"DATASEC",field_text,comment))
	{
		fprintf(stderr,"read_fits_datasec(): No DATASEC field.\n");
		return FALSE;
	}

	/* Now try and read the numbers */

	if (sscanf(field_text,"[%d:%d,%d:%d]",xmin,xmax,ymin,ymax) != 4)
	{
		fprintf(stderr,"read_fits_datasec(): DATASEC field bad.\n");
		return FALSE;
	}

	/* That is all */

	return TRUE;

} /* read_fits_datasec() */

/************************************************************************/
/* read_fits_ccdsec()							*/
/*									*/
/* Looks for a text field in the fits header called CCDSEC and tries 	*/
/* read what is in it. The field should contain a string something like */
/* [1:256,1:256], and the idea is to get the numbers inside. Returns	*/
/* TRUE if it found all four numbers and FALSE if it didn't, or if there*/
/* was no filed of the right type.					*/
/************************************************************************/

int read_fits_ccdsec(fits_data data, int *xmin, int *xmax, int *ymin, int *ymax)
{
	char	field_text[81];
	char	comment[81];

	/* Try and get the text field. */

	if (!read_fits_header_text(data,"CCDSEC",field_text,comment))
	{
		fprintf(stderr,"read_fits_ccdsec(): No CCDSEC field.\n");
		return FALSE;
	}

	/* Now try and read the numbers */

	if (sscanf(field_text,"[%d:%d,%d:%d]",xmin,xmax,ymin,ymax) != 4)
	{
		fprintf(stderr,"read_fits_ccdsec(): CCDSEC field bad.\n");
		return FALSE;
	}

	/* That is all */

	return TRUE;

} /* read_fits_ccdsec() */

/************************************************************************/
/* read_fits_biassec()							*/
/*									*/
/* Looks for a text field in the fits header called BIASSEC and tries 	*/
/* read what is in it. The field should contain a string something like */
/* [1:256,1:256], and the idea is to get the numbers inside. Returns	*/
/* TRUE if it found all four numbers and FALSE if it didn't, or if there*/
/* was no filed of the right type.					*/
/************************************************************************/

int read_fits_biassec(fits_data data, int *xmin, int *xmax, int *ymin, int *ymax)
{
	char	field_text[81];
	char	comment[81];

	/* Try and get the text field. */

	if (!read_fits_header_text(data,"BIASSEC",field_text,comment))
	{
		fprintf(stderr,"read_fits_biassec(): No BIASSEC field.\n");
		return FALSE;
	}

	/* Now try and read the numbers */

	if (sscanf(field_text,"[%d:%d,%d:%d]",xmin,xmax,ymin,ymax) != 4)
	{
		fprintf(stderr,"read_fits_biassec(): BIASSEC field bad.\n");
		return FALSE;
	}

	/* That is all */

	return TRUE;

} /* read_fits_biassec() */
