/************************************************************************/
/* fits_util.c								*/
/*									*/
/* Simplified and hacked FITS file reader and writer routines.  	*/
/* Utility functions for FITs stuff.					*/
/************************************************************************/
/*                                        	                      	*/
/* Center for High Angular Resolution Astronomy				*/
/*									*/
/* Telephone: 404-651-2932						*/
/* Fax      : 404-651-1384						*/
/* Internet : theo@chara.gsu.edu					*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : April 1997   						*/
/************************************************************************/

#include "rwfits.h"
#include "../nrc/nrc.h"

/************************************************************************/
/* copy_fits()								*/
/*									*/
/* Creates and returns a copy of a fits file.				*/
/* ALl memory is allocated for the header and data.			*/
/************************************************************************/

fits_data copy_fits(fits_data data)
{
	int i,j;
	fits_data copy;

	/* Set up defaults */

	copy.nheader = 0;
	copy.header  = NULL;
	copy.nx = 0;
	copy.ny = 0;
	copy.data = NULL;

	/* Allocate some space for the list of headers */

	copy.header = (char **)malloc((sizeof(char **))*(MAXHEADER+1))-1;
	mem_test(copy.header);

	/* OK do the header */

	copy.nheader = data.nheader;
	for(i=1; i<= copy.nheader; i++)
	{
		copy.header[i] = (char *)malloc(strlen(data.header[i]+1));
		mem_test(copy.header[i]);
		strcpy(copy.header[i], data.header[i]);
	}

	/* Allocate and fill the rest */

	copy.nx = data.nx;
	copy.ny = data.ny;
	copy.data = matrix(1,copy.nx,1,copy.ny);
	for(i=1; i<= copy.nx; i++)
	for(j=1; j<= copy.ny; j++)
	{
		copy.data[i][j] = data.data[i][j];
	}

	/* That should be about it */

	return copy;

} /* copy_fits() */

/************************************************************************/
/* free_fits_data(fits_data data)					*/
/* 									*/
/* Frees memory used by a fits data structure.				*/
/* Will create a problem if the structure was not ever allocated.	*/
/************************************************************************/

void free_fits_data(fits_data data)
{
	int	i;

	free_matrix(data.data, 1, data.nx, 1, data.ny);
	for(i=1; i<= data.nheader; i++) 
	{
		free(data.header[i]);
	}
	free(data.header+1);

	data.nheader = 0;
	data.nx = 0;
	data.ny = 0;

} /* free_fits_data() */

/************************************************************************/
/* print_fits_header() 							*/
/*									*/
/* Prints out a fits header dumby!					*/
/************************************************************************/

void print_fits_header(fits_data data)
{
	int i;

	for(i=1; i<= data.nheader; i++) printf("%s\n",data.header[i]);

} /* print_fits_header() */

/************************************************************************/
/* format_key() 							*/
/*									*/
/* Forces all letters in key to be upper case.				*/
/* Forces length of key to be exactly FITS_KEY_SIZE long.		*/
/* Expects memory allocation to be right though!!!			*/
/************************************************************************/

void format_key(char *oldkey, char *newkey)
{
	char *s;

	/* Make a copy of the key and force it to 8 characters */

	strncpy(newkey, oldkey, FITS_KEY_SIZE);
	newkey[FITS_KEY_SIZE] = 0;
	while(strlen(newkey) < FITS_KEY_SIZE) strcat(newkey," ");

	/* Now let's make it an upper case if neccessary */

	for(s= newkey; *s != 0; s++)
	{
		if (*s >= 'a' && *s <= 'z') *s = *s - 'a' + 'A';
	}

} /* format_key() */

/************************************************************************/
/* find_key()								*/
/*									*/
/* Returns the position of a KEY in a header. Returns 0 if fails.	*/
/************************************************************************/

int find_key(fits_data data, char *keyword)
{
	int i;
	char	found;
	char	line[FITS_LINE_SIZE+1];
	char	key[FITS_KEY_SIZE+1];

	format_key(keyword, key);

#ifdef DEBUG1
	printf("Looking for key \'%s\'\n",key);
#endif
	/* Loop through and try to find the correct header entry */

	for(found = FALSE, i=1; !found && i<= data.nheader; i++)
	{
		/* Copy the header line */

		strcpy(line,data.header[i]);

		/* Force it to be the correct length */

		line[FITS_KEY_SIZE] = 0;
#ifdef DEBUG1
		printf("Testing for key \'%s\'\n",line);
#endif

		/* Is this the right keyword ? */

		if (!strcmp(line,key)) found = i;
	}

	return found;

} /* find_key() */

/************************************************************************/
/* reverse_byte_order()							*/
/*									*/
/* Reverses the byte order from the pointer location including the	*/
/* given number of bytes.						*/
/************************************************************************/

void reverse_byte_order(void *p, int nbytes)
{
	unsigned char *c1,*c2;
	int	i;
	unsigned char *np;

	/* Allocate enough memory to do this */

#ifdef DEBUG2
	printf("nbytes = %d\n",nbytes);
	fflush(stdout);
#endif
	np = (unsigned char *)malloc(nbytes);
	mem_test(np);

	/* Make a copy of the bytes */

	for(c1 = p, c2 = np, i=0; i<nbytes; i++,c1++,c2++) 
	{
		*c2 = *c1;
	}

	/* Now copy it back in reverse order */

	for(c1 = p, c2 = np+nbytes-1, i=0; i<nbytes; i++, c1++, c2--) 
	{
		*c1 = *c2;
	}

#ifdef DEBUG2
	for(c1 = p, c2 = np, i=0; i<nbytes; i++,c1++,c2++) 
	{
		printf("%d %d\n",*c2,*c1);
	}
	printf("\n");
	fflush(stdout);
#endif

	/* Release the temp memory space */

	free(np);

} /* reverse_byte_order() */

/************************************************************************/
/* fits_header_type()							*/
/*									*/
/* Tries to work out what the data type is for a given fits entry.  	*/
/* Returns INT, FLOAT or TEXT if it works it out, UNKNOWN otherwise.	*/
/************************************************************************/

int fits_header_type(fits_data data, char *keyword)
{
	int	position;
	int	is_a_number;
	int	has_a_point;
	char	*p;

	/* First, lets find out which entry it is. */

	if  ((position = find_key(data, keyword)) == 0)
	{
		fprintf(stderr,"fits_header_type(): Key %s does not exist.\n",
				keyword);
		return UNKNOWN;
	}

	/* OK, let's see if we can work this out. */

	is_a_number = FALSE;
	has_a_point = FALSE; 

	for (p = data.header[position]+FITS_KEY_SIZE+1;
	     *p != '/' && *p != 0; 
	     p++)
	{
		if (*p >= '0' && *p <= '9')
		{
			is_a_number = TRUE;
		}
		else if (*p == '.')
		{
			has_a_point = TRUE;
		}
		else if (*p == '\'')
		{
			return TEXT;
		}
	}

	if (is_a_number) 
	{
		if (has_a_point)
			return FLOAT;
		else
			return INT;
	}

	return UNKNOWN;

} /* fits_header_type() */
