/************************************************************************/
/* fits_write.c								*/
/*									*/
/* Simplified and hacked FITS file reader and writer routines.  	*/
/* Routines for writing things to FITs files.				*/
/************************************************************************/
/*                                        	                      	*/
/* Center for High Angular Resolution Astronomy				*/
/*									*/
/* Telephone: 404-651-2932						*/
/* Fax      : 404-651-1384						*/
/* Internet : theo@chara.gsu.edu					*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : April 1997   						*/
/************************************************************************/

#include "rwfits.h"
#include "../nrc/nrc.h"

/************************************************************************/
/* write_fits()								*/
/*                                                                      */
/* Writes a SIMPLE fits file. Only supports 2x2 float at the moment.	*/
/*                                                                      */
/************************************************************************/

void write_fits(fits_data data, char *filename) 
{
	FILE *output;
	char	line[FITS_LINE_SIZE+1];
	char	*s;
	int	i,j;
	int	naxis,naxis1,naxis2;
	unsigned char byte;
	int	nline;
	int	nbyte;
	int	bitpix;
	float	bscale;
	float	bzero;
	float	value;
#ifdef DEBUG
	float	min,max;
#endif

	/* Ensure this is of the SIMPLE 'T' type */

	if (!(i = find_key(data, "SIMPLE"))) /* No SIMPLE T exists */
	{

		print_fits_header(data);
		printf("Forcing %s\n",SIMPLE_T_STR);
		if (++data.nheader >= MAXHEADER)
		{
			fprintf(stderr,"write_fits(): Header Full.\n");
			return;
		}

		/* Shunt up all the other header info */

		for (i = data.nheader; i > 0; i--) 
		{
			data.header[i] = data.header[i-1];
		}

		data.header[1] = (char *)malloc(strlen(SIMPLE_T_STR)+1);
		mem_test(data.header[1]);
		strcpy(data.header[1],SIMPLE_T_STR);
	}
	else
	{
		for(s=data.header[i]+FITS_KEY_SIZE+1; *s!='T' && *s != 0; s++);
		if (*s != 'T')
		{
			fprintf(stderr,
				"write_fits(): %s is not a SIMPLE T file.\n",
				filename);
			return;
		}
		free(data.header[i]);
		data.header[i] = (char *)malloc(strlen(SIMPLE_T_STR+1));
		mem_test(data.header[i]);
		strcpy(data.header[i],SIMPLE_T_STR);
	}

	/* Force bitpix to -32, floating point save */

	bitpix = -32;
	write_fits_header_int(&data, "BITPIX", bitpix, "Bits per pixel.");

	/* Force bscale and bzero to 1 and 0 */

	bscale = 1.0;
	write_fits_header_float(&data, "BSCALE", bscale, 
		"PHYSICAL = BSCALE*FILE+BZERO.");
	bzero = 0.0;
	write_fits_header_float(&data, "BZERO", bzero, 
		"PHYSICAL = BSCALE*FILE+BZERO.");

	/* Write out some of the header info required */

	write_fits_header_int(&data, "NAXIS", 2, "Number of Axis");
	write_fits_header_int(&data, "NAXIS1", data.nx, "Number of columns");
	write_fits_header_int(&data, "NAXIS2", data.ny, "Number of Rows");
	naxis = 2.0;
	naxis1 = data.nx;
	naxis2 = data.ny;

	/* Try and open the file */

	if ((output = fopen(filename,"wb")) == NULL)
	{
		fprintf(stderr,
			"write_fits(): Failed to open file %s.\n",filename);
		return;
	}

	/* Now loop through until we write out the header */

	for(nline=0, i=1; i<=data.nheader; i++)
	{

		/* Copy a line into the data buffer */

		strcpy(line, data.header[i]);
		while(strlen(line) < FITS_LINE_SIZE) 
		{
			strcat(line," ");
		}

		/* Write out this line */

		if (fwrite(line, FITS_LINE_SIZE, 1, output)!= 1)
		{
			fprintf(stderr,"write_fits(): Error writing header.\n");
			fclose(output);
			return;
		}
		nline++;
	}

	/* We have reached the end of the header */

	strcpy(line,"END");
	while(strlen(line) < FITS_LINE_SIZE) 
	{
		strcat(line," ");
	}

	if (fwrite(line, FITS_LINE_SIZE, 1, output)!= 1)
	{
		fprintf(stderr,"write_fits(): Error writing header.\n");
		fclose(output);
		return;
	}
	nline++;

	/* Now we need to pad out the buffer area */

	*line = 0;
	while(strlen(line) < FITS_LINE_SIZE)
	{
		strcat(line," ");
	}

	while(nline%(FITS_BLOCK_SIZE/FITS_LINE_SIZE))
	{
		if (fwrite(line, FITS_LINE_SIZE, 1, output)!= 1)
		{
			fprintf(stderr,"write_fits(): Error writing header.\n");
			fclose(output);
			return;
		}
		nline++;
	}

#ifdef DEBUG
	min = MAXFLOAT;
	max = -1.0*min;
	for(j=1; j<= naxis2; j++)
	for(i=1; i<= naxis1; i++)
	{
		if (data.data[i][j] < min) min = data.data[i][j];
		if (data.data[i][j] > max) max = data.data[i][j];
	}
	printf("Minimum data point found = %f\n",min);
	printf("Maximum data point found = %f\n",max);
#endif

	/* OK write out the data */

	nbyte = 0;
	for(j=1; j<= naxis2; j++)
	for(i=1; i<= naxis1; i++)
	{
#ifdef DEBUG
		printf("data[%d][%d] = %f\n",i,j,data.data[i][j]);
#endif
		value = data.data[i][j];
#ifdef BYTE_SWAP
		reverse_byte_order(&value, 4);
#endif
#ifdef DEBUG
		printf("value = %f\n",value);
		fflush(stdout);
#endif

		if (fwrite(&value, 4, 1, output) != 1)
		{
			fprintf(stderr,"write_fits(): Error writing byte.\n");
			data.nx = 0;
			data.ny = 0;
		}
#ifdef DEBUG
		printf("A");
		fflush(stdout);
#endif

			nbyte+=4;
	}

	/* Force even number of FITS_BLOCKs */

#ifdef DEBUG
	printf("B\n");
	fflush(stdout);
#endif
	byte = 0;
	while(nbyte%FITS_BLOCK_SIZE)
	{
#ifdef DEBUG
		printf("%d %d\n",nbyte,nbyte%FITS_BLOCK_SIZE);
		fflush(stdout);
#endif
		if (fwrite(&byte, 1, 1, output)!= 1)
		{
			fprintf(stderr,"write_fits(): Error writing byte.\n");
			fclose(output);
			return;
		}
		nbyte++;
	}
#ifdef DEBUG
	printf("B\n");
	fflush(stdout);
#endif

	/* Close the file and leave */

	fclose(output);

} /* write_fits() */

/************************************************************************/
/* write_fits_block()							*/
/*									*/
/* Writes one 2880 byte block to a fits file.				*/
/* Returns TRUE if all was OK and FALSE otherwise.			*/
/************************************************************************/

unsigned char write_fits_block(unsigned char *block, FILE *input)
{
	/* Try and write  the block of data */

	if (fwrite(block, FITS_BLOCK_SIZE, 1, input) != 1)
	{
	    fprintf(stderr,
		"write_fits_block(): Failed to write entire block.\n");
	    return FALSE;
	}

	return TRUE;

} /* write_fits_block() */

/************************************************************************/
/* write_fits_header_int()						*/
/*									*/
/* Changes a current header entry or makes a new header integer entry.	*/
/************************************************************************/

void write_fits_header_int(fits_data *data, char *keyword, 
				int value, char *comment)
{
	char	new_entry[256];
	char	key[FITS_KEY_SIZE+1];
	int 	i;
	
	format_key(keyword, key);

	/* Did we find the right place ? */

	if ((i = find_key(*data, keyword)))
	{
		/* YES, deallocate old header string */

		free(data->header[i]);
	}
	else
	{
		/* NO, create a new space for it */

		if ((i=data->nheader+1) > MAXHEADER)
		{
		    fprintf(stderr,"write_fits_header_int(): Header full.\n");
		    return;
		}
		data->nheader++;

	}

	/* OK, the new entry will be created */

	sprintf(new_entry,"%8s= %d / %s",key,value,comment);

	/* It must not be longer than FITS_LINE_SIZE */

	new_entry[FITS_LINE_SIZE] = 0;

	/* Allocate memory and copy it */

	data->header[i] = (char *)malloc(strlen(new_entry)+1);
	mem_test(data->header[i]);

	strcpy(data->header[i], new_entry);

	/* That is all */

} /* write_fits_header_int() */

/************************************************************************/
/* write_fits_header_float()						*/
/*									*/
/* Changes a current header entry or makes a new header float entry.	*/
/************************************************************************/

void write_fits_header_float(fits_data *data, char *keyword, 
				float value, char *comment)
{
	char	key[FITS_KEY_SIZE+1];
	char	new_entry[256];
	int 	i;
	
	format_key(keyword, key);

	/* Did we find the right place ? */

	if ((i = find_key(*data, keyword)))
	{
		/* YES, deallocate old header string */

		free(data->header[i]);
	}
	else
	{
		/* NO, create a new space for it */

		if ((i=data->nheader+1) > MAXHEADER)
		{
		    fprintf(stderr,"write_fits_header_float(): Header full.\n");
		    return;
		}
		data->nheader++;

	}

	/* OK, the new entry will be created */

	sprintf(new_entry,"%8s= %f / %s",key,value,comment);

	/* It must not be longer than FITS_LINE_SIZE */

	new_entry[FITS_LINE_SIZE] = 0;

	/* Allocate memory and copy it */

	data->header[i] = (char *)malloc(strlen(new_entry)+1);
	mem_test(data->header[i]);

	strcpy(data->header[i], new_entry);

	/* That is all */

} /* write_fits_header_float() */

/************************************************************************/
/* write_fits_header_text()						*/
/*									*/
/* Changes a current header entry or makes a new header text entry.	*/
/************************************************************************/

void write_fits_header_text(fits_data *data, char *keyword, 
				char *value, char *comment)
{
	char	key[FITS_KEY_SIZE+1];
	char	new_entry[256];
	char	new_value[256];
	int 	i;
	
	format_key(keyword, key);

	/* Did we find the right place ? */

	if ((i = find_key(*data, keyword)))
	{
		/* YES, deallocate old header string */

		free(data->header[i]);
	}
	else
	{
		/* NO, create a new space for it */

		if ((i=data->nheader+1) > MAXHEADER)
		{
		    fprintf(stderr,"write_fits_header_text(): Header full.\n");
		    return;
		}
		data->nheader++;

	}

	/* Ensure the entry isn't too long */

	sprintf(new_value,"%s",value);
	new_value[FITS_LINE_SIZE-FITS_KEY_SIZE-4] = 0;

	/* OK, the new entry will be created */

	sprintf(new_entry,"%8s=\'%s\' / %s",key,new_value,comment);

	/* It must not be longer than FITS_LINE_SIZE */

	new_entry[FITS_LINE_SIZE] = 0;

	/* Allocate memory and copy it */

	data->header[i] = (char *)malloc(strlen(new_entry)+1);
	mem_test(data->header[i]);

	strcpy(data->header[i], new_entry);

	/* That is all */

} /* write_fits_header_text() */
