/************************************************************************/
/* calculate_ps.c 							*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : June 1998                                                   */
/************************************************************************/

#include "../nrc/nrc.h"
#include <stdlib.h>
#include "simpleX.h"

/************************************************************************/
/* calculate_ps()							*/
/*									*/
/* Creates apower spectrum from an image.  				*/
/* The fill parameter specifies what to place outside the aperture, 	*/
/* and can take the values MEAN, MIN, MAX or ZERO.			*/
/* Returned matrix is float image[1..npix][1..npix].			*/
/* Parameter imbedp2 sets the size of the matrix used to perform FFT.	*/
/* It is the power of 2 to use, ie 4 uses a 16x16 matrix.		*/
/************************************************************************/

float **calculate_ps(float **data, int nx, int ny, int imbedp2, int fill)
{
	int	imbed;
	int	imbed2;
	float	*aperture;
	float	*ap;
	int	i,j;
	unsigned long int *nn;
	float	**ps;
	int	ix,iy,offset_x,offset_y;
	float	fill_value;
	int	pos_x,pos_y;

	/* Get real values for npix and imbed */

	imbed = 1<<imbedp2;
	imbed2 = imbed >> 1;

	if (imbed > 1024)
	{
		fprintf(stderr,
"calculate_ps(): Using VERY large arrays for image...  this will take forever.\n");
	}

	if (nx > imbed || ny > imbed)
	{
		fprintf(stderr,
			"calculate_ps(): Number of pixels greater than array.\n");
		return NULL;
	}

	/* Allocate memory */

        aperture = vector(1,2*imbed*imbed);
	ps = matrix(1,nx,1,ny);
	nn = lvector(1,2);

	/* Calculate offset */

	offset_x = (imbed - nx)>>1;
	offset_y = (imbed - ny)>>1;

	/* Calculate fill value */

	switch(fill)
	{
		case ZERO: fill_value = 0.0;
			   break;

		case MIN: fill_value = 
				min_image_intensity(data,nx,ny,&pos_x,&pos_y);
			  break;

		case MAX: fill_value = 
				max_image_intensity(data,nx,ny,&pos_x,&pos_y);
			  break;

		case MEAN: fill_value = total_image(data, nx, ny)/
					((float)nx * (float)ny);
			  break;
	
		default: fprintf(stderr,"calculate_ps(): Unknown fill factor.\n");
			  exit(-3);
	}

	/* Generate the aperture field */

	ap = &(aperture[1]);
        for(i=1;i<=imbed;i++)
        {
                for(j=1;j<=imbed;j++)
                {
			if ((ix = i-offset_x) < 1 || ix > nx ||
			    (iy = j-offset_y) < 1 || iy > ny)
			{
				*ap++ = fill_value;
				*ap++ = 0.0;
			}
			else
			{
				*ap++ = data[ix][iy];
				*ap++ = 0.0;
			}
		}
	}

	/* Set up for and do FFT */

	nn[1] = imbed;
	nn[2] = imbed;

 	fourn((float*)aperture, nn, 2, 1); 

	/* Now pull out the image .... */

	ap = &(aperture[1]);

        for(i=1; i<=imbed; i++)
        {
                for(j=1; j<=imbed ; j++)
                {
			/*
			 * Find the equivalent image position 
			 * including the shift to get image in center 
			 */

			if ((ix = (i+imbed2)%imbed) == 0) ix = imbed;
			if ((iy = (j+imbed2)%imbed) == 0) iy = imbed;

			/* Are we inside the image area ? */

			if ((ix -= offset_x) > 0 && ix <= nx &&
			    (iy -= offset_y) > 0 && iy <= ny)
			{
				ps[ix][iy] = *ap * *ap + *(ap+1) * *(ap+1);
			}
			ap++;
			ap++;
                }
        }

	/* Clean up memory */

	free_vector(aperture,1,2*imbed*imbed);
	free_lvector(nn,1,2);

	/* That is all */
	
	return ps;

} /* calculate_ps() */
