/************************************************************************/
/* convolve.c	 							*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : June 1998                                                   */
/************************************************************************/

#include "../nrc/nrc.h"
#include "simpleX.h"
#include <math.h>
#include <stdlib.h>

/************************************************************************/
/* convolve()								*/
/*									*/
/* Convolves two matrices of the SAME SIZE.				*/
/* The fill parameter specifies what to place outside the aperture, 	*/
/* and can take the values MEAN, MIN, MAX or ZERO.			*/
/* Returned matrix is float image[1..npix][1..npix].			*/
/* It is the power of 2 to use, ie 4 uses a 16x16 matrix.		*/
/************************************************************************/

float **convolve(float **data1, float **data2,
		int nx, int ny, int fill)
{
	int	imbed;
	int	imbed2;
	int	imbedp2;
	float	*aperture1;
	float	*ap1;
	float	*aperture2;
	float	*ap2;
	int	i,j;
	unsigned long int *nn;
	float	**result;
	int	ix,iy,offset_x,offset_y;
	float	fill_value1;
	float	fill_value2;
	int	pos_x,pos_y;
	float	real,image;
	float	divider;

	/* Get real values for npix and imbed */

	if ((i=nx) < ny) i = ny;
	imbedp2 = 1;
	while (1<<imbedp2 < i) imbedp2++;
	imbed = 1<<imbedp2;
	imbed2 = imbed >> 1;
#ifdef DEBUG
	printf("convolve(): Using imbedp2 = %d imbed = %d\n",imbedp2,imbed);
	fflush(stdout);
#endif

	if (imbed > 1024)
	{
		fprintf(stderr,
"convolve(): Using VERY large arrays for image...  this will take forever.\n");
	}

	/* Allocate memory */

        aperture1 = vector(1,2*imbed*imbed);
        aperture2 = vector(1,2*imbed*imbed);
	result = matrix(1,nx,1,ny);
	nn = lvector(1,2);

	/* Calculate offset */

	offset_x = (imbed - nx)>>1;
	offset_y = (imbed - ny)>>1;

	/* Calculate fill value */

	switch(fill)
	{
		case ZERO: fill_value1 = 0.0;
			   fill_value2 = 0.0;
			   break;

		case MIN: fill_value1 = 
				min_image_intensity(data1,nx,ny,&pos_x,&pos_y);
			  fill_value2 = 
				min_image_intensity(data2,nx,ny,&pos_x,&pos_y);
			  break;

		case MAX: fill_value1 = 
				max_image_intensity(data1,nx,ny,&pos_x,&pos_y);
			  fill_value2 = 
				max_image_intensity(data2,nx,ny,&pos_x,&pos_y);
			  break;

		case MEAN: fill_value1 = total_image(data1,nx,ny)/
					((float)nx * (float)ny);
			   fill_value2 = total_image(data2,nx,ny)/
					((float)nx * (float)ny);
			  break;
	
		default: fprintf(stderr,"convolve(): Unknown fill factor.\n");
			  exit(-3);
	}

	/* Generate the aperture field */

	ap1 = &(aperture1[1]);
	ap2 = &(aperture2[1]);
        for(i=1;i<=imbed;i++)
        {
                for(j=1;j<=imbed;j++)
                {
			if ((ix = i-offset_x) < 1 || ix > nx ||
			    (iy = j-offset_y) < 1 || iy > ny)
			{
				*ap1++ = fill_value1;
				*ap1++ = 0.0;
				*ap2++ = fill_value2;
				*ap2++ = 0.0;
			}
			else
			{
				*ap1++ = data1[ix][iy];
				*ap1++ = 0.0;
				*ap2++ = data2[ix][iy];
				*ap2++ = 0.0;
			}
		}
	}

	/* Set up for and do FFT */

	nn[1] = imbed;
	nn[2] = imbed;

 	fourn((float*)aperture1, nn, 2, 1); 
 	fourn((float*)aperture2, nn, 2, 1); 

	/* Now we multiply the FFTs */

	ap1 = &(aperture1[1]);
	ap2 = &(aperture2[1]);
        for(i=1;i<=imbed;i++)
        {
                for(j=1;j<=imbed;j++)
                {
			real = *ap1 * *ap2 - *(ap1+1) * *(ap2+1);
			image = *ap1 * *(ap2+1) + *ap2 * *(ap1+1);
			*ap1++ = real;
			*ap1++ = image;
			ap2++;
			ap2++;
		}
	}
	
	/* Now inverse transform */

 	fourn((float*)aperture1, nn, 2, -1); 

	/* Now pull out the image .... */

	ap1 = &(aperture1[1]);
	
	divider = imbed * imbed;
        for(i=1; i<=imbed; i++)
        {
                for(j=1; j<=imbed ; j++)
                {
                        /*
                         * Find the equivalent image position 
                         * including the shift to get image in center 
                         */

                        if ((ix = (i+imbed2)%imbed) == 0) ix = imbed;
                        if ((iy = (j+imbed2)%imbed) == 0) iy = imbed;

			/* Are we inside the image area ? */

			if ((ix -= offset_x) > 0 && ix <= nx &&
			    (iy -= offset_y) > 0 && iy <= ny)
			{
				result[ix][iy] = sqrt(*ap1 * *ap1 + 
						*(ap1+1) * *(ap1+1))/divider;
			}
			ap1++;
			ap1++;
                }
        }

	/* Clean up memory */

	free_vector(aperture1,1,2*imbed*imbed);
	free_vector(aperture2,1,2*imbed*imbed);
	free_lvector(nn,1,2);

	/* That is all */
	
	return result;

} /* convolve() */
