/************************************************************************/
/* makepicture.c							*/
/*									*/
/* Creates the picture data for a displayed image.			*/
/* Y axis inverted so it looks correct on the screen.			*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : August 1994   						*/
/************************************************************************/

#include <stdlib.h>
#include <math.h>
#include "../nrc/nrc.h"
#ifdef NOT_AN_APPLE
#include <values.h>
#endif
#include "simpleX.h"

/************************************************************************/
/* make_picture()							*/
/*									*/
/* Given a 2D array of floats 1..nx and 1..ny representing a picture	*/
/* this function builds a 2D array of chars 0..nx-1, 0..ny-1 ready for	*/
/* the display. Y axis is reversed.					*/
/* Returns pointer to the picture itself.  				*/
/************************************************************************/

char	*make_picture(int	nx, 
		   int ny, 
		   int pixmult, 
		   float **values,
		   char disp_type)
{
	int	i,j,k,l;
	int	pixel;
	float	max,min,step;
	float	value;
	float	**new_values;
	char *picture;
	char	*p;
	size_t  size;
#ifdef DEBUG
	int	maxi,mini;
#endif

	/* Allocate memory for the picture */

	size = nx * pixmult * ny * pixmult;

#ifdef DEBUG
	printf("Memory allocation size = %ld\n",size);
#endif

	if ((picture = (char *)malloc(size)) == NULL)
	{
		fprintf(stderr,"Ran out of memory in make_picture().\n");
		return NULL;
	}

	p = picture;
	if (disp_type == EXP || disp_type == LOG)
		new_values = matrix(1,nx,1,ny);
	else
		new_values = values;

	/* Find the maximum and minimum of the data */

	if (disp_type != STD)
	{
		max = -1.0*MAXFLOAT;
		min = MAXFLOAT;

		for(i=1;i<=nx;i++)
		{
			for(j=1;j<=ny;j++)
			{
				if (values[i][j] > max) max = values[i][j];
				if (values[i][j] < min) min = values[i][j];
			}
		}
	}
	else
	{
		max = NUM_COLORS-1;
		min = 0;
	}

#ifdef DEBUG
	printf("Min = %f\nMax = %f\n",min,max);
#endif

	/* Now do the conversion */

	if (disp_type == EXP || disp_type == LOG)
	{
	        for(i=1;i<=nx;i++)
		{
			for(j=1;j<=ny;j++)
			{
				switch(disp_type)
				{
					case LOG: new_values[i][j] =
							log(values[i][j]-min+1);
						  break;

					case EXP: new_values[i][j] =
							exp(values[i][j]/max);
						  break;

					default : break;
				}
			}
		}
		switch(disp_type)
		{
			case LOG: min=0.0;
				  max = log(max-min+1);
				  break;

			case EXP: min = exp(min/max);
				  max = exp(max/max);
				  break;

			default:  break;
		}
	}

	/* Now allocate the colors ..... */

	step = (max - min)/(float)(NUM_COLORS -1);

#ifdef DEBUG
	printf("Min = %f\nMax = %f\n",min,max);
	printf("Step = %f\n",step);
	maxi = -1;
	mini = 200;
#endif
	for(j=ny; j>0; j--)
	{
		for(i=1;i<=nx;i++)
		{
			if (min != max)
			{
				value = new_values[i][j] - min;
			}
			else
			{
				value = 0;
			}

			if (min != max)
				pixel = value/step;
			else
				pixel = 0;
#ifdef DEBUG
			if (pixel < mini) mini = pixel;
			if (pixel > maxi) maxi = pixel;
#endif
			if (pixel < 0) pixel = 0;
			if (pixel > NUM_COLORS-1) pixel = NUM_COLORS-1;

			for(k=0;k<pixmult;k++)
			for(l=0;l<pixmult;l++)
			{
				*(p+nx*pixmult*l+k) = GreyScale[pixel];
			}
			p += pixmult;
		}

		p += ((pixmult-1)*pixmult*nx);
	}
#ifdef DEBUG
	printf("mini = %d\nmaxi = %d\n",mini,maxi);
#endif

	/* Clean up memory */

	if (disp_type == EXP || disp_type == LOG) 
	{
		free_matrix(new_values,1,nx,1,ny);
	}

	/* That's it */

	return picture;

} /* make_picture() */

/************************************************************************/
/* make_ppm_P3()							*/
/*									*/
/* Given a pointer to the start of a picture and the sizes writes a	*/
/* pure ascii PPM file.							*/
/************************************************************************/

void	make_ppm_P3(char *filename, 
		    int nx, 
		    int ny, 
		    char *picture)
{
	int	i,j;	/* Looping */
	FILE	*output;

	/* Open the data file */

	if ((output = fopen(filename,"w")) == NULL)
	{
		fprintf(stderr,"Could not open %s for writing\n",filename);
		return;
	}

	/* Magic number */

	fprintf(output,"P3\n");

	/* Height and width */

	fprintf(output,"%d\n%d\n",ny,nx);

	/* Maximum color componant */

	fprintf(output,"%d\n",255);

	/* Pixels going out .... */

	for(i=0;i<nx;i++)
	for(j=0;j<ny;j++)
	{
		fprintf(output,"%u %u %u\n",*picture,*picture,*picture);
		picture++;
	}

	fclose(output);

} /* make_ppm_P3() */

/************************************************************************/
/* make_ppm_P6()							*/
/*									*/
/* Given a pointer to the start of a picture and the sizes writes a	*/
/* mixed ascii/binary PPM file.						*/
/************************************************************************/

void    make_ppm_P6(char *filename, 
		    int nx, 
		    int ny, 
		    char *picture)
{
        int     i,j;    /* Looping */
        FILE    *output;

        /* Open the data file */

        if ((output = fopen(filename,"wb")) == NULL)
        {
                fprintf(stderr,"Could not open %s for writing\n",filename);
                return;
        }

        /* Magic number */

        fprintf(output,"P6\n");

        /* Height and width */

        fprintf(output,"%d\n%d\n",ny,nx);

        /* Maximum color componant */

        fprintf(output,"%d\n",255);

        /* Pixels going out .... */

        for(i=0;i<nx;i++)
        for(j=0;j<ny;j++)
        {
		fwrite(picture, sizeof(char), 1, output);
		fwrite(picture, sizeof(char), 1, output);
		fwrite(picture, sizeof(char), 1, output);
        	picture++;
        }

        fclose(output);

} /* make_ppm_P6() */

/************************************************************************/
/* make_ascii_pict()							*/
/*									*/
/* Given a pointer to the start of a picture and the sizes writes a	*/
/* pure ascii file containing the dimmensions as first two data points  */
/************************************************************************/

void	make_ascii_pict(char *filename, 
		    int nx, 
		    int ny, 
		    char *picture)
{
	int	i,j;	/* Looping */
	FILE	*output;

	/* Open the data file */

	if ((output = fopen(filename,"w")) == NULL)
	{
		fprintf(stderr,"Could not open %s for writing\n",filename);
		return;
	}

	/* Height and width */

	fprintf(output,"%d\t%d\n",ny,nx);

	/* Pixels going out .... */

	for(i=0;i<nx;i++)
	for(j=0;j<ny;j++)
	{
		fprintf(output,"%u\n",*picture);
		picture++;
	}

	fclose(output);

} /* make_ascii_pict() */

/************************************************************************/
/* make_2D_gnuplot_file() 						*/
/* 									*/
/* Creates a file that GNIPLOT can read and plot.			*/
/* the vectors x and y contain the values along the x and y axis.  	*/
/************************************************************************/

void make_2D_gnuplot_file(float **data, 
			  int nx, 
			  float *x, 
			  int ny, 
			  float *y, 
			  char *filename)
{
	int	i,j;
	float	f;
	FILE	*output;

	/* Open the file */

	if ((output = fopen(filename,"wb")) == NULL)
	{
		printf("make_2D_gnuplot_file(): Failed to open file %s\n",
				filename);
		return;
	}

	/* Add the number of columns */

	f = nx;
	if (fwrite(&f, sizeof(float), 1, output) != 1)
	{
		fprintf(stderr,
			"make_2D_gnuplot_file(): Failed to write data.\n");
		fclose(output);
		return;
	}

	/* Now add the X positions for these columns */

	for(i=1; i<= nx; i++)
	{
		f = x[i];
		if (fwrite(&f, sizeof(float), 1, output) != 1)
		{
		    fprintf(stderr,
			"make_2D_gnuplot_file(): Failed to write data.\n");
		    fclose(output);
		    return;
		}
	}

	/* Now add the Y positions and the actual data */

	for(j=1; j<= ny; j++)
	{
		f = y[j];
		if (fwrite(&f, sizeof(float), 1, output) != 1)
		{
		    printf("make_2D_gnuplot_file(): Failed to write data.\n");
		    fclose(output);
		    return;
		}

		for(i=1; i<= nx; i++)
		{
			f = data[i][j];
			if (fwrite(&f, sizeof(float), 1, output) != 1)
			{
			    printf(
			    "make_2D_gnuplot_file(): Failed to write data.\n");
			    fclose(output);
			    return;
			}
		}
	}

	fclose(output);

} /* make_2D_gnuplot_file() */
