/************************************************************************/
/* messages.c 		                                                */
/*                                                                      */
/* Some message handlers and other related routines. Note some message	*/
/* handlers will not be here since they are specific to certain		*/
/* windows.								*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : May 2003                                                    */
/************************************************************************/

#include "../include/chara_messages.h"
#include "../cliserv/include/cliserv.h"
#include "sockman.h"
#include "../charalib/chara.h"

/* Some static globals */

static int socket_manager_server = -1;
static struct s_process_sockets process = {0, 0, 0, 0, "", "", ""};
static char *names = NULL;
static char *long_names = NULL;
static char ping_worked = FALSE;
static char no_such_process = FALSE;
static char use_non_socket_manager_version = FALSE;

/* Some functions are only used in here so are locally declared */

int set_socket_manager_messages(void);
int message_set_process(int server, struct smessage *mess);
int message_list_names(int server, struct smessage *mess);
int message_list_long_names(int server, struct smessage *mess);
int message_local_message(int server, struct smessage *mess);
int message_no_such_process(int server, struct smessage *mess);

#define SOCKMAN_TIME_OUT	30

//#define TEST

/************************************************************************/
/* socket_manager_set_version()						*/
/*									*/
/* So programs can decide if they are using the socket manager or not.	*/
/************************************************************************/

void socket_manager_set_version(char non_socket)
{
	use_non_socket_manager_version = non_socket;

} /* socket_manager_set_version() */

/************************************************************************/
/* set_socket_manager_messages()					*/
/*									*/
/* Routine to create pointers to all the messages we have.		*/
/* It also opens the socket and initializes it. The idea is this is	*/
/* called each time you wish to speak to this socket.			*/
/* We assume here that the init_jobs() call has happened already.	*/
/* Returns TRUE if all goes well, FALSE otherwise.			*/
/************************************************************************/

int set_socket_manager_messages(void)
{
	char	command[512];
	int	i;

	/* First, we try and process any remaining messages */

	if (!process_server_socket_all_messages(socket_manager_server))
	{
		close_socket_manager_server();
	}

	/* Open the socket if we need to */

	if (!server_socket_open(socket_manager_server))
	{
	    /* Yes, we need to so we will try three times */

	    for (i=0; i<3 && !server_socket_open(socket_manager_server); i++)
	    {
	    	if ((socket_manager_server = open_server_socket(
		    SOCKET_MANAGER_MACHINE, SOCKET_MANAGER_PORT,
		    "SOCKET SERVER")) == -1)
	        {
		    /* Wait a little then try again */

		    sleep(1);
		}
	    }
		
	    if (!server_socket_open(socket_manager_server))
	    {
		/* 
		 * Socket manager seems to be dead!
		 * So let's try and start it up ourselves.
		 */
    
		sprintf(command,
		   "rsh %s /usr/local/bin/socket_manager &",
		   SOCKET_MANAGER_MACHINE);
		system(command);
    
		/* Wait and see if this worked */
    
		sleep(1);

		if ((socket_manager_server = open_server_socket(
		   SOCKET_MANAGER_MACHINE, SOCKET_MANAGER_PORT,
		    "SOCKET SERVER")) == -1) return FALSE;
	    }

            if (!add_message_job(socket_manager_server, 
		SOCKET_MANAGER_SET_PROCESS, 
		message_set_process)) return FALSE;
            if (!add_message_job(socket_manager_server, 
		SOCKET_MANAGER_LIST_NAMES, 
		message_list_names)) return FALSE;
            if (!add_message_job(socket_manager_server,
		SOCKET_MANAGER_LIST_LONG_NAMES, 
		message_list_long_names)) return FALSE;
            if (!add_message_job(socket_manager_server,
		SOCKET_MANAGER_NO_SUCH_PROCESS, 
		message_no_such_process)) return FALSE;
	}
	
	return TRUE;

} /* set_socket_manager_messages() */

/************************************************************************/
/* message_set_process() 	                                        */
/*                                                                      */
/* Will react to SOCKET_MANAGER_SET_PROCESS message from server         */
/************************************************************************/

int message_set_process(int server, struct smessage *mess)
{
	struct s_process_sockets *new_process;

	/* Is this the right kind of data? */

	if (mess->length != sizeof(*new_process))
	{
		return NOERROR;
	}

	/* Transfer the data */

	new_process = (struct s_process_sockets *)mess->data;

	strcpy(process.name, new_process->name);
	strcpy(process.machine, new_process->machine);
	process.pid = new_process->pid;
	process.commands = new_process->commands;
	process.data = new_process->data;
	process.messages = new_process->messages;
	strcpy(process.restart_command, new_process->restart_command);

	return NOERROR;

} /* message_set_process() */

/************************************************************************/
/* message_list_names() 	                                        */
/*                                                                      */
/* Will react to SOCKET_MANAGER_LIST_NAMES message from serve r         */
/************************************************************************/

int message_list_names(int server, struct smessage *mess)
{
	char	*new_names;

	/* Is this the right kind of data? */

	if (mess->length <= 0) return NOERROR;

	/* Free memory if we have to */

	if (names != NULL) free(names);

	/* Allocate new memory */

	if ((names = malloc(mess->length)) == NULL) return NOERROR;

	/* Transfer the data */

	new_names = (char *)mess->data;
	strcpy(names, new_names);

	return NOERROR;

} /* message_list_names() */

/************************************************************************/
/* message_list_long_names() 	                                        */
/*                                                                      */
/* Will react to SOCKET_MANAGER_LIST_NAMES message from serve r         */
/************************************************************************/

int message_list_long_names(int server, struct smessage *mess)
{
	char	*new_long_names;

	/* Is this the right kind of data? */

	if (mess->length <= 0) return NOERROR;

	/* Free memory if we have to */

	if (long_names != NULL) free(long_names);

	/* Allocate new memory */

	if ((long_names = malloc(mess->length)) == NULL) return NOERROR;

	/* Transfer the data */

	new_long_names = (char *)mess->data;
	strcpy(long_names, new_long_names);

	return NOERROR;

} /* message_list_long_names() */

/************************************************************************/
/* message_no_such_process() 	                                        */
/*                                                                      */
/* Will react to SOCKET_NO_SUCH_PROCESS message from server             */
/************************************************************************/

int message_no_such_process(int server, struct smessage *mess)
{
	no_such_process = TRUE;

	return NOERROR;

} /* message_no_such_process() */

/************************************************************************/
/* socket_manager_get_process()						*/
/*									*/
/* Talks to socket manager and tries to get the information on a	*/
/* given process. Returns						*/
/* 	0 if all goes well						*/
/*	1 if we fail to reach the server at all				*/
/*	2 if we fail to send the message.   				*/
/*	3 if process does not seem to exist.				*/
/* 	4 if name does not match.					*/
/************************************************************************/

int socket_manager_get_process(char *name,struct s_process_sockets *new_process)
{
	time_t	start;
	struct smessage mess;
	int	ret;

	/* We should try the non socket version first. */

	if ((ret = socket_manager_non_socket_get_process(name, new_process))
		 == 0)
	{
		return ret;
	}
	else if (ret == 5)
	{
		return 3;
	}

	/* 
	 * If that failed, try the older socket based version
	 * First we try and open the port
	 */

	if (!set_socket_manager_messages()) return 1;

	/* Set the current process to NULL */

	*process.name = -1;
	no_such_process = FALSE;

	/* Now send out our request */

	mess.type = SOCKET_MANAGER_GET_PROCESS;
	mess.data = (unsigned char *)name;
	mess.length = strlen(name)+1;

	if (!send_message(socket_manager_server, &mess))
	{
		close_socket_manager_server();
		return 2;
	}

	/* Wait for a response */

	start = time(NULL);
	while(time(NULL) < start+SOCKMAN_TIME_OUT && 
		*process.name == -1 && !no_such_process)
	{
		process_server_socket(socket_manager_server);
	}

	if (no_such_process || *process.name == -1)
	{
		close_socket_manager_server();
		return 3;
	}

	/* Is the name right? */

	if (strcmp(process.name, name) != 0)
	{
		close_socket_manager_server();
		return 4;
	}

	/* Copy result */

	strcpy(new_process->name, process.name);
	strcpy(new_process->machine, process.machine);
	new_process->pid = process.pid;
	new_process->commands = process.commands;
	new_process->data = process.data;
	new_process->messages = process.messages;
	strcpy(new_process->restart_command, process.restart_command);

	/* Close the socket connection */

#ifdef ALWAYS_CLOSE_SOCKET
	close_socket_manager_server();
#endif

	return 0;

} /* socket_manager_get_process() */

/************************************************************************/
/* socket_manager_set_process()						*/
/*									*/
/* Talks to socket manager and tries to set the information on a	*/
/* given process. Returns						*/
/* 	0 if all goes well						*/
/*	1 if we fail to reach the server at all				*/
/*	2 if we fail to send the message.   				*/
/*	3 if process already exists.					*/
/* 	4 if name etc does not match.					*/
/*	5 Name or machine has white space and is not cool.		*/
/*	6 restart command is more than one line.			*/
/*	7 Either the name, machine or PID is missing			*/
/************************************************************************/

int socket_manager_set_process(struct s_process_sockets new_process)
{
	time_t	start;
	struct smessage mess;
	struct s_process_sockets temp_process;
	char	filename[234], s[234];
	FILE	*fp;

	if (has_white_space(new_process.name) ||
	    has_white_space(new_process.machine)) return 5;

	if (has_new_line(new_process.restart_command)) return 6;

	if (strlen(new_process.name) == 0 || strlen(new_process.machine) == 0 ||
		new_process.pid <= 0 ) return 7;

	/* Always check if it exists with the non socket version */

	if (socket_manager_non_socket_get_process(new_process.name,
			&temp_process) == 0) return 3;

	/* Use the non socket version if required and possible */

	if (use_non_socket_manager_version && 
	    strcmp(temp_process.name, new_process.name) == 0)
	{
	    /* OK, does the information match? */

	    if (strcmp(temp_process.machine, new_process.machine) == 0 ||
		    temp_process.commands == new_process.commands ||
		    temp_process.data == new_process.data ||
		    temp_process.messages == new_process.messages)
	    {
		/* Right, we can save the relevant data */
			
		sprintf(filename, "%s/socket_manager/%s",
			get_etc_directory(s), new_process.name);

		/* Is it possible to do this? */

		if ((fp = fopen(filename, "w")) != NULL)
		{
			fprintf(fp, "%d %s",
				new_process.pid,
				new_process.restart_command);
			fclose(fp);
			sprintf(s,"chmod a+r %s", filename);
			system(s);
			return 0;
		}
	    }
	}

	/* 
	 * Otherwise we use the socket version.
	 * First we try and open the port
	 */

	if (!set_socket_manager_messages()) return 1;

	/* Set the current process to NULL */

	*process.name = 0;

	/* Now send out our request */

	mess.type = SOCKET_MANAGER_SET_PROCESS;
	mess.data = (unsigned char *)&new_process;
	mess.length = sizeof(new_process);

	if (!send_message(socket_manager_server, &mess))
	{
                close_socket_manager_server();
		return 2;
	}

	/* Wait for a response */

	start = time(NULL);
	while(time(NULL) < start+SOCKMAN_TIME_OUT && *process.name == 0)
        {
                process_server_socket(socket_manager_server);
        }

	if (*process.name == 0)
	{
                close_socket_manager_server();
		return 3;
	}

	/* Is the name right? */

	if (strcmp(process.name, new_process.name) != 0)
	{
                close_socket_manager_server();
		return 4;
	}

	/* Close the socket connection */

#ifdef ALWAYS_CLOSE_SOCKET
	close_socket_manager_server();
#endif

	return 0;

} /* socket_manager_set_process() */

/************************************************************************/
/* socket_manager_remove_process()					*/
/*									*/
/* Talks to socket manager and tries to remove the information on a	*/
/* given process. Returns						*/
/* 	0 if all goes well						*/
/*	1 if we fail to reach the server at all				*/
/*	2 if we fail to send the message.   				*/
/*	3 if process does not seem to exist.				*/
/* 	4 if name does not match.					*/
/************************************************************************/

int socket_manager_remove_process(char *name)
{
	time_t	start;
	struct smessage mess;
	struct s_process_sockets temp_process;
	char	filename[234], s[234];

	/* Always check if it exists with the non socket version */

	if (socket_manager_non_socket_get_process(name, &temp_process) == 0)
	{
	    /* If we find a PID it should be running.... */

	    if(temp_process.pid != -1)
	    {
		/* We need to remove the file with PID and restart command */

		sprintf(filename, "%s/socket_manager/%s",
			get_etc_directory(s), name);
		unlink(filename);

		return 0;
	    }
	}

	/* First we try and open the port */

	if (!set_socket_manager_messages()) return 1;

	/* Set the current process to NULL */

	*process.name = 0;
	no_such_process = FALSE;

	/* Now send out our request */

	mess.type = SOCKET_MANAGER_REMOVE_PROCESS;
	mess.data = (unsigned char *)name;
	mess.length = strlen(name)+1; 

	if (!send_message(socket_manager_server, &mess)) 
	{
                close_socket_manager_server();
		return 2;
	}

	/* Wait for a response */

	start = time(NULL);
	while(time(NULL) < start+SOCKMAN_TIME_OUT && *process.name == 0 &&
		!no_such_process)
        {
                process_server_socket(socket_manager_server);
        }

	if (*process.name == 0 || no_such_process)
	{
                close_socket_manager_server();
		return 3;
	}

	/* Is the name right? */

	if (strcmp(process.name, name) != 0 ||
	    strlen(process.machine) != 0 ||
		process.commands != -1 ||
		process.data != -1 ||
		process.messages != -1)
	{
		printf("%s %d %d %d\n",
			process.name, process.commands,
			process.data, process.messages);
                close_socket_manager_server();
		return 4;
	}

	/* Close the socket connection */

#ifdef ALWAYS_CLOSE_SOCKET
	close_socket_manager_server();
#endif

	return 0;

} /* socket_manager_remove_process() */

/************************************************************************/
/* socket_manager_list_names()						*/
/*									*/
/* Talks to socket manager and tries to set the information on a	*/
/* given process. Returns list of names or NULL on error.		*/
/************************************************************************/

char *socket_manager_list_names(void)
{
	time_t	start;
	struct smessage mess;

	/* First the socket version. First we try and open the port */

	if (!set_socket_manager_messages()) return NULL;

	/* Set the current to NULL */

	if (names != NULL) free(names);
	names = NULL;

	/* Now send out our request */

	mess.type = SOCKET_MANAGER_LIST_NAMES;
	mess.data = NULL;
	mess.length = 0;

	if (!send_message(socket_manager_server, &mess))
	{
                close_socket_manager_server();
		return names;
	}

	/* Wait for a response */

	start = time(NULL);
	while(time(NULL) < start+SOCKMAN_TIME_OUT && names == NULL)
        {
                process_server_socket(socket_manager_server);
        }

	/* Close up and exit */

#ifdef ALWAYS_CLOSE_SOCKET
	close_socket_manager_server();
#endif
	/* Always try non socket version as well */

#warning Non socket version goes here. Add its list of names here.

	return names;

} /* socket_manager_list_names() */

/************************************************************************/
/* socket_manager_list_long_names()					*/
/*									*/
/* Talks to socket manager and tries to set the information on a	*/
/* given process. Returns list of long_names or NULL on error.		*/
/************************************************************************/

char *socket_manager_list_long_names(void)
{
	time_t	start;
	struct smessage mess;

	/* Always try socket version. First we try and open the port */

	if (!set_socket_manager_messages()) return NULL;

	/* Set the current to NULL */

	if (long_names != NULL) free(long_names);
	long_names = NULL;

	/* Now send out our request */

	mess.type = SOCKET_MANAGER_LIST_LONG_NAMES;
	mess.data = NULL;
	mess.length = 0;

	if (!send_message(socket_manager_server, &mess))
	{
                close_socket_manager_server();
		return long_names;
	}

	/* Wait for a response */

	start = time(NULL);
	while(time(NULL) < start+SOCKMAN_TIME_OUT && long_names == NULL)
        {
                process_server_socket(socket_manager_server);
        }

	/* Close up and exit */

#ifdef ALWAYS_CLOSE_SOCKET
	close_socket_manager_server();
#endif
	/* Always try non socket version as well */

#warning Non socket version goes here. Add its list of names here.

	return long_names;

} /* socket_manager_list_long_names() */

/************************************************************************/
/* socket_manager_ping_process()					*/
/*									*/
/* Tries to open a socket to a process and ping it to see if it's alive.*/
/* Returns 								*/
/* 	0 if it is alive						*/
/*	1 if it does not exist						*/
/*	2 if it fails to send the message				*/
/*	3 if it does not respond.					*/
/************************************************************************/

int     socket_manager_ping_process(struct s_process_sockets process)
{
	int	server;
	struct smessage mess;
	int	i;
	time_t	start;

	/* Start off saying it does not work */

	ping_worked = FALSE;

	/* First we try and open a link to the server */

	if ((server = open_server_socket(process.machine, process.messages,
		process.name)) == -1) return 1;

	/* Now make sure all messages are set to say we got something */

	for(i=FIRST_STANDARD_MESSAGE; i< MAX_JOBS; i++)
	{
		add_message_job(server, i, message_local_message);
	}

	/* Now send out a PING */
	
	mess.type = PING;
	mess.data = NULL;
	mess.length = 0;
	if (!send_message(server, &mess))
	{
                close_socket_manager_server();
		return 2;
	}

	/* OK, wait for a response */

	start = time(NULL);
	while(time(NULL) < start+SOCKMAN_TIME_OUT && !ping_worked)
        {
                process_server_socket(server);
        }

	close_server_socket(server);

	/* Did it work? */

	if (!ping_worked) return 3;

	return 0;

} /* socket_manager_ping_process() */

/************************************************************************/
/* message_local_message() 	                                        */
/*                                                                      */
/* Will react to any message from server			        */
/************************************************************************/

int message_local_message(int server, struct smessage *mess)
{
	ping_worked = TRUE;
	return NOERROR;

} /* message_local_message() */

/************************************************************************/
/* has_white_space()							*/
/*									*/
/* Searchs a string. If it finds white space returns TRUE, FALSE 	*/
/* otherwise.								*/
/************************************************************************/

int has_white_space(char *s)
{
	for(; *s != 0; s++)
	{
		if (*s == ' ' || *s == '\n' || *s == '\r' || *s == '\t')
			return TRUE;
	}

	return FALSE;

} /* has_white_space() */

/************************************************************************/
/* has_new_line()							*/
/*									*/
/* Searchs a string. If it finds new line returns TRUE, FALSE 		*/
/* otherwise.								*/
/************************************************************************/

int has_new_line(char *s)
{
	for(; *s != 0; s++)
	{
		if (*s == '\n' || *s == '\r') return TRUE;
	}

	return FALSE;

} /* has_new_line() */

/************************************************************************/
/* socket_manager_stop_process()					*/
/*									*/
/* Tries to stop a process gracefully.					*/
/* Returns 								*/
/* 	0 if it all went well						*/
/*	1 if there was no machine or pid.				*/
/*	2 if the process still seems to exist in sockman.		*/
/*	3 if there is a command socket which should be used instead.	*/
/************************************************************************/

int     socket_manager_stop_process(struct s_process_sockets process)
{
	char	command[512];
	struct s_process_sockets new_process;

	/* Does it have a command socket? */

	if (process.commands > 0) return 3;

	/* Check we have a machine and PID */

	if (*process.machine == 0 || process.pid <= 0) return 1;

	/* Assemble the command */

	sprintf(command, "rsh %s kill -2 %d &",process.machine, process.pid);

	/* And try and do it */

	system(command);

	/* Give some time for this to happen */

	sleep(1);

	/* Now make sure it has been removed from the list */

	if (socket_manager_get_process(process.name, &new_process) != 3)
		return 2;

	/* All is done */

	return 0;

} /* socket_manager_stop_process() */

/************************************************************************/
/* socket_manager_kill_process()					*/
/*									*/
/* Tries to kill a process gracefully.					*/
/* Returns 								*/
/* 	0 if it all went well						*/
/*	1 if there was no machine or pid.				*/
/************************************************************************/

int     socket_manager_kill_process(struct s_process_sockets process)
{
	char	command[512];

	/* Check we have a machine and PID */

	if (*process.machine == 0 || process.pid <= 0) return 1;

	/* Assemble the command */

	sprintf(command, "rsh %s kill -9 %d &", process.machine, process.pid);

	/* And try and do it */

	system(command);

	/* All is done */

	return 0;

} /* socket_manager_kill_process() */

/************************************************************************/
/* socket_manager_restart_process()					*/
/*									*/
/* Tries to restart a process gracefully.				*/
/* Returns 								*/
/* 	0 if it all went well						*/
/*	1 if there was no machine.					*/
/*	2 if there was no restart command.				*/
/************************************************************************/

int     socket_manager_restart_process(struct s_process_sockets process)
{
	char	command[512];

	/* Check we have a machine and PID */

	if (*process.machine == 0) return 1;

	if (*process.restart_command == 0) return 2;

	/* Assemble the command */

	sprintf(command, "rsh %s \"%s\" &", process.machine, 
			process.restart_command);

	/* And try and do it */

	system(command);

	/* All is done */

	return 0;

} /* socket_manager_restart_process() */

/************************************************************************/
/* close_socket_manager_server()					*/
/*									*/
/* Close the socket manager server.					*/
/************************************************************************/

void close_socket_manager_server(void)
{
	close_server_socket(socket_manager_server);
	socket_manager_server = -1;

} /* close_socket_manager_server() */

/************************************************************************/
/* socket_manager_non_socket_get_process()				*/
/*									*/
/* Non socket version of get process. Returns:				*/
/* 	0 if all goes well						*/
/*	1 if we fail to reach the server or file at all			*/
/*	3 if process does not seem to exist.				*/
/*	5 process exists but has no PID.				*/
/************************************************************************/

int socket_manager_non_socket_get_process(char *name,
		struct s_process_sockets *new_process)
{
	FILE	*fp;
	char	filename[234], s[234], *p;

	/* Start off with no information */

	*new_process->name = 0;
	*new_process->machine = 0;
	new_process->pid = -1;
	new_process->commands = -1;
	new_process->data = -1;
	new_process->messages = -1;
	*new_process->restart_command = 0;

	 /* Assemble the filename for the list of servers. */

	sprintf(filename, "%s/socket_manager/socket_manager.list",
		get_etc_directory(s));

	/* Try and open it */

	if ((fp = fopen(filename, "r")) == NULL) return 1;

	/* Cycle though and search for the name */

	while((p = fgets(s, 234, fp)) != (char *)NULL)
	{	
		if (sscanf(s, "%s %s %d %d %d", 
			new_process->name,
			new_process->machine,
			&(new_process->commands),
			&(new_process->data),
			&(new_process->messages)) != 5) continue;

#ifdef TEST
		printf("%s %s %s %d %d %d\r\n", 
			name,
			new_process->name,
                        new_process->machine,
                        new_process->commands,
                        new_process->data,
                        new_process->messages);
		fflush(stdout);
		sleep(1);
#endif

		if (strcmp(name, new_process->name) == 0) break;
	}
	fclose(fp);

	/* Did we find it? */

	if (p == (char *)NULL) return 3;

	/* OK, go try and find the PID and restart command */

	sprintf(filename, "%s/socket_manager/%s", get_etc_directory(s), name);

	/* Does this file exist? */

	if ((fp = fopen(filename, "r")) != NULL)
	{
		while(fgets(s, 234, fp) != (char *)NULL)
		{
			if (*p == '#') continue;
		}
		fclose(fp);
				
		if (sscanf(s, "%d", &(new_process->pid)) == 1)
		{
			for(p = s; *p != ' ' && *p != '\t';p++);
			for(; *p == ' ' || *p == '\t' ||
				*p == '\n' || *p == '\r' ;p++);
			strcpy(new_process->restart_command, p);
			return 0;
		}
	}

	new_process->pid = -1;
	*new_process->restart_command = 0;

	return 5;

} /* socket_manager_non_socket_get_process() */
