/************************************************************************/
/* command.c								*/
/*                                                                      */
/*	This file is the command processor and assorted ancillary	*/
/*	functions :							*/
/*									*/
/*	command_processor() : well... yes				*/
/*	get_user_input() : Gets menu command from user			*/
/*	call_function() : Function caller				*/
/*									*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/************************************************************************/
/* command_processor()							*/
/* Process 4.2								*/
/*									*/
/* The command processor will accept a line of text from the user (not	*/
/* more than 77 characters due to the one-line window that it operates	*/
/* in). If it is a function name it will call the function and return a	*/
/* NULL. If it is a menu name it will return a pointer to that menu.	*/
/* The arguments of the command line will be passed on to the function	*/
/* caller if it is a function name.					*/
/*									*/
/* This function calls the line_edit function.				*/
/*									*/
/************************************************************************/

struct	smenu *command_processor(char first_char)
{
	/* Internal Variables */

	struct	smenu *return_value;	/* returned to calling function */
	int	function_pt;		/* index to function array */
	char	*first_word;		/* in the command line */
	char	*args;			/* of the command line */
	char	command_line[LENGTH_CL+1];	/* includes space for NULL */
	static char	last_command_line[LENGTH_CL+1];	
	struct sscripts *script_pt;	/* A pointer to a script */

	/* Initialise */

	command_line[0] = first_char;
	command_line[1] = (char)NULL;

	/* call the line editor */

	if (line_edit(command_window,0,2,command_line,LENGTH_CL,
		STRING,check_command,TRUE) == KEY_ESC) 
	{
		clean_command_line();
		return (struct smenu *)NULL;
	}

	/* If the first letter is '!' add this to the old command string */

	if (*command_line == '!')
	{
		strncat(last_command_line,command_line+1,
			LENGTH_CL+1-strlen(last_command_line));
		strcpy(command_line,last_command_line);
		mvwaddstr(command_window,0,2,command_line);
		touchwin(command_window);
		wrefresh(command_window);
	}
	else
	{
		strcpy(last_command_line,command_line);	
	}

	/* split off first word from the argument list */

	first_word = split_string(command_line,&args);

	/* is it NULL? */

	if (*first_word == (char)NULL) return (struct smenu *)NULL;

	/* Is it a function name? */

	if ((function_pt = find_function(first_word)) != (int)NULL)
	{
		call_function(function_pt, args);
		clean_command_line();
		return (struct smenu *)NULL;
	}

	/* is it a menu name? */
	
	if ((return_value = find_menu(first_word)) != (struct smenu *)NULL)
	{
		/* ensure that "main" can't be called from command line */

		if (return_value == find_menu("main"))
		{
		      error(WARNING,WARN_1);
		      return_value = (struct smenu *)NULL;
		}
		clean_command_line();
		return return_value;
	}

	/* Is it a script ? */
	
	if ((script_pt = find_script(first_word)) != (struct sscripts *)NULL)
	{
		call_script(script_pt);
		clean_command_line();
		return (struct smenu *)NULL;
	}

	/* If we get to here then it's an error */

	/* Command is unknown */

	clean_command_line();
	error(WARNING,WARN_2,first_word);
	return (struct smenu *)NULL;

} /* command_processor [4.2] */	

/************************************************************************/
/* get_user_input()							*/
/* Process 4.1								*/
/*									*/
/* Awaits a menu command keystroke from user. Any other key will call 	*/
/* command line processor and put the character on the command line.	*/
/* Control characters (etc) are ignored.				*/
/* Returns the menu command key or NULL if command processor was 	*/
/* invoked.								*/ 
/*									*/
/************************************************************************/

int	get_user_input(WINDOW *menuwindow, struct smenu **nextmenu)
{
	/* Internal variables */

	int	command;	/* Command requested by user */
	bool	loop;		/* Loop variable used by while loop */
#ifdef __NCURSES_H
	MEVENT	mouse;		/* A mouse event. */
#endif
	
	/* Initialisation */

	loop = TRUE;
	*nextmenu = NULL;

	while (loop)
	{
		command = get_command();

#ifdef __NCURSES_H
		if (command == KEY_MOUSE)
		{
			getmouse(&mouse);

			if (mouse.bstate == BUTTON1_CLICKED)
			{
				/* OK, what key did the mouse represent. */
		
				if (mouse.y == (MENU_BEGIN_Y + 6) &&
					mouse.x <  strlen(MENU_INFO1)+2)
				{
					command = KEY_HELP;
				}
				else if (mouse.y == (MENU_BEGIN_Y + 7) &&
					mouse.x <  strlen(MENU_INFO2)+2)
				{
					command = KEY_MENU_UP;
				}
				else if (mouse.y == (MENU_BEGIN_Y + 8) &&
					mouse.x <  strlen(MENU_INFO3)+2)
				{
					command = KEY_RISE;
				}
				else if (mouse.y == (MENU_BEGIN_Y + 9) &&
					mouse.x <  strlen(MENU_INFO5)+2)
				{
					command = KEY_SWAP;
				}
				else if (mouse.y >= MENU_BEGIN_Y &&
					 mouse.y <= MENU_BEGIN_Y+9 &&
					 mouse.x > 39)
				{
					command = KEY_F(1) + 
						  mouse.y-MENU_BEGIN_Y;
				}
				else
				{
					continue;
				}
			}
			else continue;
		}
#endif
		/* We want the numerals to also act as function keys */

		switch(command)
		{
			case '1': command = KEY_F(1);
				  break;
			case '2': command = KEY_F(2);
				  break;
			case '3': command = KEY_F(3);
				  break;
			case '4': command = KEY_F(4);
				  break;
			case '5': command = KEY_F(5);
				  break;
			case '6': command = KEY_F(6);
				  break;
			case '7': command = KEY_F(7);
				  break;
			case '8': command = KEY_F(8);
				  break;
			case '9': command = KEY_F(9);
				  break;
			case '0': command = KEY_F(10);
				  break;
			default:  break;
		}

		/* Now, what do we do with it? */

		switch(command)
		{
			case KEY_HELP:
			case KEY_MENU_UP:
			case KEY_SWAP:
			case KEY_RISE:
			case KEY_F(1) :
			case KEY_F(2) :
			case KEY_F(3) :
			case KEY_F(4) :
			case KEY_F(5) :
			case KEY_F(6) :
			case KEY_F(7) :
			case KEY_F(8) :
			case KEY_F(9) : 
			case KEY_F(10) : loop = FALSE;
					 werase(system_window);
					 wrefresh(system_window);
					 break;

			default : if((command >= 'a' && command <= 'z')
					||
				     (command >= 'A' && command <= 'Z')
					|| command == '!')
				  {
					werase(system_window);
					wrefresh(system_window);
					*nextmenu = command_processor(command);
					if(*nextmenu != (struct smenu *)NULL)
					{
						return MENU;
					}
					loop = FALSE;
					command = (int)NULL;
				  }

				  break;

		} /* end switch */

	} /* end while */

	return command;

} /* get_user_input [4.1] */

/************************************************************************/
/* check_command()							*/
/*									*/
/* Always returns TRUE.							*/
/*									*/
/************************************************************************/

bool check_command(char *command_line)
{
	return TRUE;

} /* check_command() */

/************************************************************************/
/* call_function()							*/
/* Process 4.4								*/
/*									*/
/* Function caller. It recieves an index into the function array	*/
/* and a string of arguments. It then constructs int argc and		*/
/* char **argv and calls the required function.				*/
/*									*/
/* One future addition may be action on the error level returned	*/
/* by the called function. At present only returns the error level of 	*/
/* returned by the function.						*/
/*									*/
/************************************************************************/

/*
 * A local define
 */

#define	MAX_ARGS	10

int	call_function(int function_pt, char *args)
{
	/* Internal Variables */

	int	argc;			/* Number of arguments */
	char	*argv[MAX_ARGS];	/* Array of argument strings */
	char	*rest;			/* Rest of string */

	/* Initialisation */

	argc = 1;
	argv[0] = functions[function_pt].name;

	/* Assemble the rest of the argument list */

	while(argc < MAX_ARGS && *args != (char)NULL)
	{
		argv[argc++] = split_string(args, &rest);
		args = rest;
	}

	/* Call the function */

	return (*(functions[function_pt].function))(argc,argv);

}  /* call_function [4.4] */
