/************************************************************************/
/* testuiesp.c								*/
/*                                                                      */
/* Test program for ESP libraries.					*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar    		                        */
/* Date: May 2007							*/
/************************************************************************/

#include "../charaui/src/charaui.h"
#include "../cliserv/include/cliserv.h"
#include "../include/chara_messages.h"
#include "../charalib/chara.h"
#include "../esp_client/espclient.h"
#include "../esp_client/espuiclient.h"


int time_status(void);		/* Example background job below */
int esp_background(void);		/* Background job tests data socket */
void esp_open(void);
void esp_close(void);
static int esp_server;

static char unit_name[256];

int main(int argc, char **argv)
{
	/* Check command line */

	if (argc != 2)
	{
		error(FATAL,"Usage: %s ESP_NAME.", argv[0]);
		exit(-1);
	}
	strcpy(unit_name, argv[1]);

	/* Title page. */

	ui_clear_screen();
	put_line("");
	center_line("CHARA ESP INTERFACE TEST ROUTINE");
	put_line("");
	center_line("The CHARA Array");
	center_line("Center for High Angular Resolution Astronomy");
	center_line("Mount Wilson Observatory, CA 91001, USA");
	put_line("");
	center_line("Telephone: 1-626-796-5405");
	center_line("Fax: 1-626-796-6717");
	center_line("email: theo@chara.gsu.edu");
	center_line("WWW: http://www.chara.gsu.edu");
	put_line("");
	center_line("(C) This executable program is copyright.");
	wait_for_title();

	/* Initialize the user interface */

	TITLE = "ESP VERSION 0.0";
	initialise_ui("esp_menus.ini","esp_help.ini","testuiesp");
	set_user_open_function(esp_open);
	set_user_close_function(esp_close);

	/* Setup background job(s) */

	background_add(time_status);
	background_add(esp_background);

	/* Let's go! */

	start_ui(); /* Should never return from here. */

	exit(0);
}

/************************************************************************/
/* esp_open()								*/
/************************************************************************/

void esp_open(void)
{
	int 	argc;
	char	*argv[2];

	/* Try and open an ESP unit */

	message(system_window,"Opening socket to ESP unit.");
	argc = 2;
	argv[0] = "espo";
	argv[1] = unit_name;
	if ((esp_server = ui_open_esp_client(argc, argv)) == -1)
	{
		error(FATAL,"Failed to open unit %s.\n", unit_name);
	}
	werase(system_window);
	wrefresh(system_window);

	/* Start the background */

        background_start(0, NULL);

} /* esp_open() */

/************************************************************************/
/* esp_close()								*/
/************************************************************************/

void esp_close(void)
{
	close_esp_client(esp_server);

} /* esp_open() */

/************************************************************************/
/* time_status()							*/
/* 									*/
/* Displays the time in the status window.				*/
/************************************************************************/

int time_status(void)
{
	time_t current_time;
	struct tm *now;

	time(&current_time);
	now = localtime(&current_time);
	wstandout(status_window);
	mvwaddstr(status_window,0,0,"Time : ");
	wstandend(status_window);
	wprintw(status_window,"%2d:%02d:%02d",
		now->tm_hour,now->tm_min,now->tm_sec);

	return NOERROR;

} /* time_status() */


/************************************************************************/
/* call_ui_open_esp_server()						*/
/*									*/
/* Calls ui_esp_server().						*/
/************************************************************************/

int call_ui_open_esp_server(int argc, char **argv)
{
	esp_server = ui_open_esp_client(argc, argv);
	return NOERROR;

} /* call_ui_open_esp_server() */

/************************************************************************/
/* esp_background()							*/
/*									*/
/* Tries to display status information.					*/
/************************************************************************/

int esp_background(void)
{
	int i;
	static int cycle = 0;

	process_server_socket(esp_server);

	if (++cycle >= 500)
	{
		esp_update(esp_server);
		cycle = 0;
	}

	wstandout(status_window);
	mvwaddstr(status_window,0,17,"Unit : ");
	wstandend(status_window);
	if (server_socket_open(esp_server))
		wprintw(status_window,"%6s",esp_unit_name[esp_server]);
	else
		wprintw(status_window,"NONE  ");

	wstandout(status_window);
	mvwaddstr(status_window,2,10,"Pos  mm");
	mvwaddstr(status_window,2,20,"Vel mm/S");
	mvwaddstr(status_window,2,30,"Acc mm/SS");
	mvwaddstr(status_window,2,40,"Min  mm");
	mvwaddstr(status_window,2,50,"Max  mm");
	mvwaddstr(status_window,2,60,"Pow");
	mvwaddstr(status_window,2,65,"Mov");
	mvwaddstr(status_window,2,70,"Beam");
	wstandend(status_window);

	for(i=0; i<ESP_MAX_STAGE; i++)
	{
		if (i < esp_num_axes[esp_server])
		{
			wstandout(status_window);
			mvwaddstr(status_window,3+i,0,"Beam : ");
			wstandend(status_window);
			wprintw(status_window,"%2d",i+1);

			mvwprintw(status_window,3+i,10,"%8.3f",
				esp_stage[esp_server][i].real_position);

			mvwprintw(status_window,3+i,20,"%8.3f",
				esp_stage[esp_server][i].velocity);

			mvwprintw(status_window,3+i,30,"%8.3f",
				esp_stage[esp_server][i].acceleration);

			mvwprintw(status_window,3+i,40,"%8.3f",
				esp_stage[esp_server][i].min);

			mvwprintw(status_window,3+i,50,"%8.3f",
				esp_stage[esp_server][i].max);

			if (esp_power_status_bit[esp_server] & (1<<i))
				mvwprintw(status_window,3+i,60," ON");
			else
				mvwprintw(status_window,3+i,60,"OFF");

			if (esp_motion_status_bit[esp_server] & (1<<i))
				mvwprintw(status_window,3+i,65," NO");
			else
				mvwprintw(status_window,3+i,65,"YES");

			mvwprintw(status_window,3+i,70,"%3d",
				esp_stage[esp_server][i].beam);
			
		}
		else
		{
			mvwaddstr(status_window,3+i,0,"         ");
			mvwaddstr(status_window,3+i,10,"        ");
			mvwaddstr(status_window,3+i,20,"        ");
			mvwaddstr(status_window,3+i,30,"        ");
			mvwaddstr(status_window,3+i,40,"        ");
			mvwaddstr(status_window,3+i,50,"        ");
			mvwaddstr(status_window,3+i,60,"   ");
			mvwaddstr(status_window,3+i,65,"   ");
			mvwaddstr(status_window,3+i,70,"   ");
		}
	}

        return NOERROR;

} /* esp_background() */
