/************************************************************************/
/* help.c								*/
/*                                                                      */
/*	This is the main file for the help routines. These routines	*/
/*	rely on the existence of the file "help.ini". Use of help.ini	*/
/*	is described in its own header comments. The help directory	*/
/*	MUST be below the home directory and, indeed, be called		*/
/*	.\help. The files in this directory are text only and can 	*/
/*	be modified/maintained by the user according to the 		*/
/*	instructions contained in the header comments of the already 	*/
/*	existing help text files.					*/
/*									*/
/*		ui_help()	- the main function			*/
/*		commands()	- Lists available commands 		*/
/*		init_help()	- initialises the help system		*/
/*		text_format()	- formats text into desired window	*/
/*		fill_text_format() - 	fills display structure with	*/
/*					formatted text.			*/
/*		scroll_text()	- allows the user to scroll text	*/
/*		init_display()	_ allocates memory for the display	*/
/*				  structure.				*/
/*		scripts()	- Lists available scripts 		*/
/*									*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/*
 * globals
 */

static bool help_is_available = FALSE;	/* flag set if init_help() works */
int number_text_lines = 0;	/* ...in file */
struct slist *help_list;

/************************************************************************/
/* ui_help()								*/
/* Process 4.5.0							*/
/*									*/
/* The function that does all the work for the help system.		*/
/* If no arguments are given, a list of the help topics are 		*/
/* displayed and the user may cursor to the selected help topic		*/
/* and choose it by hitting <ENTER>. If arguments are given,		*/
/* the system will go to directly displaying the help text (if it  	*/
/* exists) otherwise the appropriate error messages are displayed. 	*/
/* 									*/
/************************************************************************/

int 	ui_help(int argc,char **argv)
{
	/* internal variables */

	int	i;			/* looping variable */
	int	key;			/* ...hit by user */
	char	*choice;		/* choice from list of topics */
	char	*last_choice;		/* last choice from list of topics */
	char	command_line[81];	/* ...for ui_help() */

	/* check to see if help available or not */

	if (!help_is_available)
	{
		error(WARNING,WARN_3);
		return WARNING;
	}

	/*
	 * If there are no arguments to the function, display a list 
	 * of topics and allow the user to select one...
	 * If this is a socket just type out a list of the functions.
	 */

	/* First initialise last_choice so it points to a NULL */

	last_choice = (char *) &i;
	*last_choice = (char)NULL;

	if(argc == 1)
	{
		if (active_socket == -1)
		{
		    /* If this isn't a socket show the list on the screen */

		    while(TRUE)
		    {
			if((key = ui_list(HELP_HEADING,"",help_list,
			   last_choice,&choice,FALSE,FALSE,FALSE,FALSE)) 
				== KEY_ESC)
			{
				return NOERROR;
			}

			/* gets to here if a topic is selected from the list */

			else
			{
				last_choice = choice;
				text_format(main_window,choice);
			}
		    }
		}
		else
		{
		    /* This a socket, send the formatted list to it instead */

		    socket_print(active_socket,
			"\nThe following help is available:\n\n");
		    *command_line = 0;
		    for(i=0; i<help_list->number_items; i++)
		    {
			if (strlen(command_line) + 
				strlen(help_list->item[i]->description) > 80)
			{
				socket_print(active_socket,"%s\n",command_line);
				*command_line = 0;
			}
			else
			{
				strcat(command_line,
					help_list->item[i]->description);
				while(strlen(command_line)%20)
				{
					strcat(command_line," ");
				}
			}
		    }
		    if (strlen(command_line) != 0)
		    {
			socket_print(active_socket,"%s\n",command_line);
		    }
		    socket_print(active_socket,
			"\nType `help topic' to view help text.\n");

		    return NOERROR;
		}
	}

	/* otherwise, try to find the info wanted from the arguments */

	else
	{
		/* set the command_line to be first part */

		strcpy(command_line,argv[1]);

		/* make the command_line from the arguments */

		for(i=2; i<argc; i++)
		{
			strcat(command_line," ");
			strcat(command_line,argv[i]);
		}

		/* find the matching help file... */

		for(i=0; i<help_list->number_items; i++)
		{
			if(strcmpi(help_list->item[i]->description,command_line)
				== 0)
			{
			   /* ... and display it */

			   text_format(main_window,
				help_list->item[i]->filename);
			   return NOERROR;
			}

			else continue;
		}

		/* if it gets this far then it can't find the topic */

		error(WARNING,WARN_4,command_line);

		return WARNING;
	}

} /* ui_help() [4.5.0] */ 

/************************************************************************/
/* commands() 								*/
/* Lists the available commands in the display window and let's the	*/
/* user see them. If invoked from a socket simply speits them out.	*/
/************************************************************************/

int	commands(int argc, char **argv)
{
	int	i;

        /* Initialisation */

        number_text_lines = -1;
        clear_display();
        inc_number_text_lines();

	/* Fill the display structure with function names */

	for(i=1; functions[i].name != NULL; i++)
	{
		strcpy(display->string[number_text_lines], functions[i].name);

		if (!inc_number_text_lines()) break;
	}
        display->number_items = number_text_lines;


	/* Now either print it or display it */

	if (active_socket == -1)
        {
                /* Called by keyboard user, scroll the text on screen */

                active_window = main_window;
		heading(heading_window,
			"The following commands are available:","");
                scroll_text(sub_main_window);
        }
        else
        {
                /* Called by a socket, spit it out the socket */

		socket_print(active_socket,
			"The following commands are available:\n");
                for(i=0; i<display->number_items; i++)
                {
                        socket_print(active_socket,"%s\n",display->string[i]);
                }
        }

	return NOERROR;

} /* commands() */
	
/************************************************************************/
/* init_help()								*/
/* Process 3.5								*/
/*									*/
/* This function is called to initialise the help system.		*/
/* Searches for file help.ini in software directory and uses		*/
/* the information therein to fill in the "help" structure with		*/
/* a list of the help files available and the topics they relate	*/
/* to. See also comment header on help.ini and the individual		*/
/* help files in the helpfile directory.				*/
/* Help.ini is defined in help.h.					*/
/* 									*/
/************************************************************************/

void 	init_help(char *helpfilename)
{
	/* Internal Variables */

	FILE	*input;			/* file to work with : help.ini */
	FILE	*helpfile;		/* file to work with : .\help\*.hlp */
	char	line[81];		/* temp store of info from file */
	char	*filename;		/* temp store of info from file */
	char	*description;		/* temp store of info from file */
	int	i;			/* Looping variable */

	/*
	 * If the help initialisation file, helpfilename, can't be found,
	 * print a diagnostic and return without initialising any help
	 */

	if((input = fopen(helpfilename,"r")) == NULL)
	{
		error(WARNING,WARN_5,helpfilename);
		help_is_available = FALSE;
		return;
	}
	
	/* create the help list */

	if ((help_list = list_alloc()) == (struct slist *)NULL)
		error(FATAL,FATAL_1,"setting up help system");

	help_list->number_items = 0;

	/* Read help initialisation file into structure */

	while(getline_ui(line,81,input) != (char *)EOF)
	{	
		filename = split_string(line,&description);
	
		/* Turn any '\' characters into '/' in the file name */
 
		for(i=0; filename[i] != (char)NULL; i++)
		{
			if (filename[i] == '\\') filename[i] = '/';
		}

		/* print warning if can't find individual help files */

		if((helpfile = fopen(filename,"r")) == NULL)
		{
			error(WARNING,WARN_6,filename);
			continue;
		}
		else
		{
			fclose(helpfile);
		}

		if(alloc_list_item(help_list,help_list->number_items) 
								!= NOERROR)
		{
			error(WARNING,WARN_6);
			break;
		}

		strncpy(help_list->item[help_list->number_items]->filename,
				filename,FILENAME);
		strncpy(help_list->item[help_list->number_items]->description,
				description,DESCRIPTION);
		help_list->item[help_list->number_items]->tagged = FALSE;

		if (++help_list->number_items >= MAX_ITEMS)
		{
			error(WARNING,WARN_16);
			break;
		}
	}
	fclose(input);

	/* So now help is ready */

	help_is_available = (help_list->number_items != 0);
	return;

} /* init_help() [3.5] */

/************************************************************************/
/* text_format()							*/
/*									*/
/* Formats text from a file into a specified window.			*/
/* Uses fill_text_format function.					*/
/*									*/
/************************************************************************/

void 	text_format(WINDOW *win, char *filename)
{
	/* internal variables */

	int	i;			/* looping variable */
	FILE	*input;			/* file to work with */

	/* initialisation */

	i = 0;

	/* check to be able to open desired help file */

	if((input = fopen(filename,"r")) == NULL)
	{
		error(WARNING,WARN_7,filename);
		return;
	}

	fill_text_format(win,input);

	fclose(input);

	/* At this point, everything should be OK to allow viewing */
	
	if (active_socket == -1)
	{
		/* Called by keyboard user, scroll the text on screen */

		active_window = win;
		scroll_text(win);
	}
	else
	{
		/* Called by a socket, spit it out the socket */

		for(i=0; i<display->number_items; i++)
		{
			socket_print(active_socket,"%s\n",display->string[i]);
		}
	}

	return;

} /* text_format() */

/************************************************************************/
/* fill_text_format()							*/
/*									*/
/* Takes text from file and formats it into the structure		*/
/* "display" defined in charaui.h.					*/	
/* The window given will determine the length of line to format.	*/
/*									*/
/* The following words are special :					*/
/*									*/
/*	.tab		Forces a tab (Normal tabs will be ignored)	*/
/*									*/
/*	.paragraph	Forces a paragraph				*/
/*									*/
/*	.nl		Forces a new line				*/
/*									*/
/*	.center		Centres next line of text			*/
/*									*/
/************************************************************************/

void 	fill_text_format(WINDOW *win, FILE *filename)
{
	/* internal variables */

	int	i;			/* Looping */
	char	input_line[81];		/* store info from file */
	char	*line;			/* store info from file */
	char	*line2;			/* store info from file */
	char	*word;			/* word to use */
	char	*getline_result;	/* result from getline_ui() */

	/* Initialisation */

	number_text_lines = -1;
	clear_display();
	inc_number_text_lines();

	/* Read lines of file into structure */

	do 
	{	
		getline_result = getline_ui(input_line,81,filename);
		line = input_line;

		/* check to see if a paragraph space is needed */

		if(strcmpi(line,PARA_DELIM) == 0)
		{
			if (*(display->string[number_text_lines]) != (char)NULL)
				if(!inc_number_text_lines()) break;
			if(!inc_number_text_lines()) break;
			continue;
		}

		/* See if this line has to be centered */

		if(strcmpi(line,CENTRE_DELIM) == 0)
		{
			/* Get the line of test to be centered */

			if((getline_result = getline_ui(input_line,81,filename))
				== (char *) EOF) break;
			line = input_line;

			/* is text line too long to center? */

			if (strlen(line) > win->_maxx-2)
			{
			      error(WARNING,WARN_8);
			}
			else
			{
			      /* Ensure it is on a new line */

			      if (*(display->string[number_text_lines]) !=
								(char)NULL)
				      if(!inc_number_text_lines()) break;

			      /* Do it !! */

			      for(i=0; i<(win->_maxx-2 - strlen(line))/2; i++)
				      strcat(display->string[number_text_lines]
							," ");
			      strcat(display->string[number_text_lines],line);
			      if(!inc_number_text_lines()) break;
			      continue;
			}
		}

		while(strlen(line) != 0)
		{
			word = split_string(line,&line2);
			line = line2;
			
			/* check to see if new line is being forced */

			if(strcmpi(word,NEWLINE_DELIM) == 0)
			{
				if(!inc_number_text_lines()) break;
				continue;
			}

			/* Check for a tab */

			if(strcmpi(word,TAB_DELIM) == 0) 
			{
			    if(strlen(display->string[number_text_lines])==0)
			    {
				for(i = 0;i < TABSTOP; i++)
				    strcat(display->string[number_text_lines]
								," ");
				continue;
			    }
			    while(strlen(display->string[number_text_lines])
					%TABSTOP != 0)
			    {
			        if (strlen(display->string[number_text_lines])
					+ 1 <= win->_maxx-2)
			    	    strcat(display->string[number_text_lines]
			    		," ");
			    }
			    continue;
			}

			/* Put in the word */

			if(strlen(display->string[number_text_lines])
				+ strlen(word) + 1 > win->_maxx-2)
			{
				if(!inc_number_text_lines()) break;
			}

			if(getline_result == (char *)EOF) break;

			strcat(display->string[number_text_lines],word);
			strcat(display->string[number_text_lines]," ");
		}
	} 
	while(getline_result != (char *)EOF);

	inc_number_text_lines();

	display->number_items = number_text_lines;

} /* fill_text_format() */

/************************************************************************/
/* inc_number_text_lines()						*/
/*									*/
/* Increment the number of text lines. For counting how may		*/
/* formatted lines of text there are in the "display" structure.	*/
/* Maximum number of text lines defined in help.h.			*/
/* 									*/
/************************************************************************/

bool 	inc_number_text_lines(void)
{
	/* is text in file to long for structure to hold? */

	if(++number_text_lines >= NUM_TEXT_LINES)
	{
		error(WARNING,WARN_9);
		return FALSE;
	}

	if((display->string[number_text_lines] = (char *)malloc(LINE_LENGTH)) 
							== NULL)
	{
		display->string[number_text_lines] = "";
		error(WARNING,"Cannot allocate any more lines in display");
		return FALSE;
	}
	else
	{
		strcpy(display->string[number_text_lines],"");
	}

	return TRUE;

} /* inc_number_text_lines() */

/************************************************************************/
/* scroll_text()							*/
/* Process 4.5.5							*/
/*									*/
/* This function puts the display structure in a given window.		*/
/* All normal scrolling facilities are available.			*/
/*									*/
/************************************************************************/

void 	scroll_text(WINDOW *win)
{
	/* Internal variables */

	int	i;			/* for looping */
	int	keystroke;		/* key hit by user */
	int	line_counter;		/* keeps track of position on scrn */
	bool	finished;		/* flag for getting out of help */
	int	item;			/* keeps track of topic to display */
#ifdef __NCURSES_H
        MEVENT  mouse;          	/* A mouse event. */
#endif


	/* initialise screen */

	werase(win);
	scrollok(win,TRUE);

	/* Initialise variables */

	i = 1;			
	line_counter = 0;	
	item = 0;
	finished = FALSE;	/* No...we haven't finished yet */

	/* 
	 * Put up first page of text. 
	 */

	working();			/* inform the user */

	wmove(win,line_counter,1);

	/* put up first line of information */

	waddstr(win,display->string[item]);	

	/* put up the rest of the page */

	for(i=0; i<win->_maxy-1; i++)
	{
		down_key(win,&line_counter,&item,display->number_items,
			display->string[item+1],FALSE);
	}

	/* Reset everything to be at the top of the page */

	line_counter = 0;
	item = 0;
	wmove(win,0,0);
	touchwin(win);
	wrefresh(win);

	not_working();			/* inform the user */

	/* display list of keys that can be used in system window */

	help_line(TRUE);

	/*
	 * Allow scrolling 
	 */

	while(!finished)
	{
		keystroke = get_command();
#ifdef __NCURSES_H
                if (keystroke == KEY_MOUSE)
                {
                        getmouse(&mouse);

                        if (mouse.bstate == BUTTON1_CLICKED)
                        {
                                /* OK, what key did the mouse represent. */

                                if (mouse.y == SYSTEM_BEGIN_Y &&
                                        mouse.x == 1)
                                {
                                        keystroke = 'h';
                                }
                                else if (mouse.y == SYSTEM_BEGIN_Y &&
                                        mouse.x >= 28 && mouse.x <=30)
                                {
                                        keystroke = KEY_ESC;
                                }
			}
		}
#endif
		switch(keystroke)
		{
			case 'j':
			case KEY_DOWN:
			case KEY_LEFT:

				while(line_counter != win->_maxy-1)
				{
					line_counter++;
					item++;
				}

				wmove(win,line_counter,0);
				down_key(win,&line_counter,&item,
					display->number_items,
					display->string[item+1],FALSE);
				wrefresh(win);
				continue;

			case 'k':
			case KEY_UP:
			case KEY_RIGHT:

				while(line_counter != 0)
				{
					line_counter--;
					item--;
				}
				wmove(win,line_counter,0);
				up_key(win,&line_counter,&item,
					display->number_items,
					display->string[item-1],FALSE);
				wrefresh(win);
				continue;

			case 'f':
			case ' ':
			case KEY_NPAGE:		/* page down */
			case KEY_LL:		/* END key */
			case '$':		/* for UNIX */

				if(keystroke == KEY_LL || keystroke == '$')
				{
					line_counter = 0;
					item = display->number_items - 
						win->_maxy;
					if (item < 0) item = 0;
					werase(win);
					mvwaddstr(win,line_counter,1,
						display->string[item]);	
				}
				else while(line_counter != win->_maxy-1)
				{
					line_counter++;
					item++;
				}

				wmove(win,line_counter,0);

				for(i=0; i<win->_maxy-1;i++)
				{
					down_key(win,&line_counter,
						&item,display->number_items,
						display->string[item+1],FALSE);
				}
				wrefresh(win);
				continue; 

			case 'b':
			case KEY_PPAGE:		/* page up */
			case KEY_HOME:		/* HOME key */
			case '^':		/* for UNIX */

				if(keystroke == KEY_HOME || keystroke == '^')
				{
					line_counter = win->_maxy-1;
					item = win->_maxy-1;
					werase(win);
					mvwaddstr(win,line_counter,1,
						display->string[item]);	
				}
				else while(line_counter != 0)
				{
					line_counter--;
					item--;
				}

				wmove(win,line_counter,0);

				for(i=0; i<win->_maxy-1; i++)
				{
					up_key(win,&line_counter,&item,
						display->number_items,
						display->string[item-1],FALSE);
				}
				wrefresh(win);
				continue; 

			case KEY_ESC: /* ESCAPE */

				finished = TRUE;
				werase(system_window);
				wrefresh(system_window);
				continue;

			case 'h' : /* help on movement keys */

				key_help(FALSE,FALSE,FALSE,FALSE);
				touchwin(heading_window);
				wrefresh(heading_window);
				touchwin(win);
				wrefresh(win);
				touchwin(system_window);
				wrefresh(system_window);
				continue;

			default : continue;

		} /* end switch */

	} /* end while */

} /* scroll_text() [4.5.5] */

/************************************************************************/
/* init_display()							*/
/* Process 3.4								*/
/*									*/
/* Allocate memory for "display" structure defined in help.h.		*/
/*									*/
/************************************************************************/

void 	init_display(void)
{
	/* check for available memory */

	if((display = (struct sdisplay *)malloc(sizeof(struct sdisplay)))
			== (struct sdisplay *)NULL)
	{
		error(FATAL,FATAL_1,"setting up display system");
	}

	display->number_items = 0;

} /* init_display() [3.4] */

/************************************************************************/
/* scripts() 								*/
/* Lists the available scripts in the display window and let's the	*/
/* user see them. If invoked from a socket simply spits them out.	*/
/************************************************************************/

int	scripts(int argc, char **argv)
{
	int	i;
	struct	sscripts *current;

        /* Initialisation */

        number_text_lines = -1;
        clear_display();
        inc_number_text_lines();

	/* Fill the display structure with function names */

	for(current = first_script.next; 
            current != &last_script && current->next != NULL;
	    current = current->next)
	{
		strcpy(display->string[number_text_lines], current->name);
		if (!inc_number_text_lines()) break;
	}
        display->number_items = number_text_lines;

	if (display->number_items <= 0)
	{
		if (active_socket == -1)
		{
			message(system_window,"No scripts defined.");
		}
		else
		{	
			socket_print(active_socket,"No scripts defined.\n");
		}
		return NOERROR;
	}

	/* Now either print it or display it */

	if (active_socket == -1)
        {
                /* Called by keyboard user, scroll the text on screen */

                active_window = main_window;
		heading(heading_window,
			"The following scripts are available:","");
                scroll_text(sub_main_window);
        }
        else
        {
                /* Called by a socket, spit it out the socket */

		socket_print(active_socket,
			"The following scripts are available:\n");
                for(i=0; i<display->number_items; i++)
                {
                        socket_print(active_socket,"%s\n",display->string[i]);
                }
        }

	return NOERROR;

} /* scripts() */
