/************************************************************************/
/* list.c								*/
/*                                                                      */
/*	The functions in this file display a list			*/
/*	of items on the screen that					*/
/*	represents a description of the contents of files.		*/
/*	Text is highlighted to represent the position on the screen of	*/
/*	the cursor. The user may use various keys to move to a selected */
/*	item (see code) and hit <ENTER> to verify their choice.		*/
/*	Additional keys (see code) may be used to exit the ui_list	*/
/*	function. The return value is the value of the key ised to	*/
/*	exit. The variable for the name of the file that is required	*/
/*	must be passed to ui_list.					*/
/*									*/
/*	The structure is defined in list.h				*/
/*									*/
/*		ui_list()	- the main function			*/
/*		highlight()	- highlights a line of text		*/
/*		unhighlight()	- oh dear...				*/
/*		down_key()	- copes with scrolling down a line	*/
/*		up_key()	- copes with scrolling up a line	*/
/*		key_help()	- help on movement keys			*/
/*		help_line()	- limited help message			*/
/*		print_test()	- Prints display data			*/
/*		save_text()    - Sends display data to a file		*/
/*		find_list_item() - searches list for directory name	*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/************************************************************************/
/* ui_list()								*/
/* Process 4.5.2							*/
/*									*/
/* This function does all the work to display a list of the long	*/
/* descriptive titles of files and initialises a variable to hold	*/
/* the actual filename when the user selects their choice with the	*/
/* <ENTER> key. Other keys (see code) may also change the status of 	*/
/* the filenames in the list (tag, untag, view, delete).		*/
/*									*/
/* The cursor will start off on the entry that corresponds to the	*/
/* string lastchoice or the first entry if it does not exist.		*/
/*									*/
/************************************************************************/

int 	ui_list(
	char	*title,		/* ...for list of items */
	char	*sub_title,	/* ...for list of items */
	struct	slist *list,	/* pointer to list structure */
	char	*last_choice,	/* default place to start in list */
	char	**choice,	/* selected by user */
	bool	tag,		/* user is able to tag | untag items */
	bool	view,		/* user is able to view item */
	bool	can_delete,	/* user is able to delete item */
	bool	print)		/* user is able to print item */
{
	/* Internal variables */

	int	number_items;		/* num. of items in list */
	int	i;			/* for looping */
	int	keystroke;		/* key hit by user */
	int	line;			/* keeps track of position on scrn */
	int	current_item;		/* keeps track of topic to display */
	bool	finished;		/* flag for getting out of help */
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
#endif

	/* Set up which list to use */

	number_items = list->number_items;

	/* initialise screen and allow scrolling */

	werase(sub_main_window);
	scrollok(sub_main_window,TRUE);

	/* put up the title */

	heading(heading_window,title,sub_title);

	/* Initialise variables */

	i = 0;			
	line = 0;	
	current_item = 0;
	finished = FALSE;	/* No...we haven't finished yet */
	active_window = sub_main_window;

	/* 
	 * Put up first page of text. This information comes from
	 * the structure "list[][]". 
	 */

	working();			/* inform the user */

	/* put up first line of information */

	unhighlight(sub_main_window,line,0,
		list->item[current_item]->description,
		list->item[current_item]->tagged);

	/* put up the rest of the page */

	for(i=0; i<sub_main_window->_maxy-1; i++)
	{
		down_key(sub_main_window,&line,&current_item,number_items,
			list->item[current_item+1]->description,
			list->item[current_item+1]->tagged);
	}

	/* Reset everything to be at the top of the page */

	line = 0;
	current_item = 0;

	/* move to the top of the page and highlight the last choice */

	if (*last_choice == (char)NULL)
	{
	  highlight(sub_main_window,line,0,
		list->item[current_item]->description,
		list->item[current_item]->tagged);
	}
	else
	{
		/* Find the last choice */

		for(i=0; strcmpi(list->item[i]->filename,last_choice) != 0
				&& i < number_items; i++)
		{
		down_key(sub_main_window,&line,&current_item,number_items,
				list->item[current_item+1]->description,
				list->item[current_item+1]->tagged);
		}

		/* does help topic choice exist? */

		if (i >= number_items) error(WARNING,WARN_10,last_choice);

		highlight(sub_main_window,line,0,
		list->item[current_item]->description,
			list->item[current_item]->tagged);
	}

	touchwin(sub_main_window);
	wrefresh(sub_main_window);
	not_working();			/* inform the user */

	/* display list of keys that can be used in system window */

	help_line(FALSE);

	/*
	 * Allow scrolling and acceptance of topic.
	 *
	 * Unhighlight the text you are on...
	 * move to another position...
	 * highlight the new text.
	 * <ESC> gets you out of the help system.
	 * <ENTER> passes the text you are on back to the help system.
	 */

	while(!finished)
	{
		keystroke = get_command();
#ifdef __NCURSES_H
                if (keystroke == KEY_MOUSE)
                {
                        getmouse(&mouse);

                        if (mouse.bstate == BUTTON1_CLICKED)
                        {
                                /* OK, what key did the mouse represent. */

                                if (mouse.y == SYSTEM_BEGIN_Y &&
                                        mouse.x == 1)
                                {
                                        keystroke = 'h';
                                }
                                else if (mouse.y == SYSTEM_BEGIN_Y &&
                                        mouse.x >= 28 && mouse.x <=30)
                                {
                                        keystroke = KEY_ESC;
                                }
                                else if (mouse.y == SYSTEM_BEGIN_Y &&
                                        mouse.x >= 45 && mouse.x <=49)
                                {
                                        keystroke = '\r';
                                }
                        }
                }
#endif
		switch(keystroke)
		{
			case 'j':
			case KEY_DOWN:
			case KEY_LEFT:

				if (current_item >= number_items-1) continue;
				unhighlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				down_key(sub_main_window,&line,&current_item,
					number_items,
					list->item[current_item+1]->description,
					list->item[current_item+1]->tagged);
				highlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				wrefresh(sub_main_window);
				continue;

			case 'k':
			case KEY_UP:
			case KEY_RIGHT:

				if (current_item <= 0) continue;
				unhighlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
			up_key(sub_main_window,&line,&current_item,number_items,
					list->item[current_item-1]->description,
					list->item[current_item-1]->tagged);
				highlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				wrefresh(sub_main_window);
				continue;

			case 'f':
			case ' ':
			case KEY_NPAGE:

				unhighlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);

				for(i=0; i<sub_main_window->_maxy-1;i++)
				{
					if (current_item >= number_items-1) 
						break;
					down_key(sub_main_window,&line,
						&current_item,number_items,
					list->item[current_item+1]->description,
					list->item[current_item+1]->tagged);
				}

				highlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				wrefresh(sub_main_window);
				continue; 

			case KEY_LL:		/* END key */
			case '$':		/* for UNIX */

				unhighlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);

				for(i=0; i<number_items ;i++)
				{
					if (current_item >= number_items-1)
						break;
					down_key(sub_main_window,&line,
						&current_item,number_items,
					list->item[current_item+1]->description,
					list->item[current_item+1]->tagged);
				}

				highlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				wrefresh(sub_main_window);
				continue; 

			case 'b':
			case KEY_PPAGE:

				unhighlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				for(i=0; i<sub_main_window->_maxy-1; i++)
				{
					if (current_item <= 0) break;
					up_key(sub_main_window,&line,
						&current_item,number_items,
					list->item[current_item-1]->description,
					list->item[current_item-1]->tagged);
				}
				highlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				wrefresh(sub_main_window);
				continue; 

			case KEY_HOME:
			case '^':		/* for UNIX */

				unhighlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);

				for(i=0; i<number_items ; i++)
				{
					if (current_item <= 0) break;
					up_key(sub_main_window,&line,
						&current_item,number_items,
					list->item[current_item-1]->description,
					list->item[current_item-1]->tagged);
				}

				highlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				wrefresh(sub_main_window);
				continue; 

			case KEY_ESC: /* ESCAPE */

				finished = TRUE;
				werase(system_window);
				wrefresh(system_window);
				continue;

			case TAG : 

				if(!tag) continue;

				list->item[current_item]->tagged = TRUE;

				unhighlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				if (current_item < number_items - 1)
				{
				down_key(sub_main_window,&line,&current_item,
					number_items,
					list->item[current_item+1]->description,
					list->item[current_item+1]->tagged);
				}
				highlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);

				wrefresh(sub_main_window);
				continue;

			case UNTAG : 

				if(!tag) continue;

				list->item[current_item]->tagged = FALSE;

				unhighlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				if (current_item < number_items - 1)
				{
				down_key(sub_main_window,&line,&current_item,
					number_items,
					list->item[current_item+1]->description,
					list->item[current_item+1]->tagged);
				}
				highlight(sub_main_window,line,0,
					list->item[current_item]->description,
					list->item[current_item]->tagged);
				wrefresh(sub_main_window);

				continue;

			case VIEW : 

				if(!view) continue;

				/* set choice to be... */

				*choice = list->item[current_item]->filename;

				/* send back the keystroke */

				return keystroke;
				break;

			case DELETE_KEY : 

				if(!can_delete) continue;

				/* set choice to be... */

				*choice = list->item[current_item]->filename;

				/* send back the keystroke */

				return keystroke;
				break;

			case PRINT :

				if(!print) continue;

				/* set choice to be... */

				*choice = list->item[current_item]->filename;

				/* send back the keystroke */

				return keystroke;
				break;

			case '\r': 	/* ENTER */

				/* set choice to be... */

				*choice = list->item[current_item]->filename;

				/* send back the keystroke */

				return keystroke;
				break;

			case 'h' : /* more help wanted */

				key_help(tag,view,can_delete,print);
				touchwin(heading_window);
				wrefresh(heading_window);
				touchwin(sub_main_window);
				wrefresh(sub_main_window);
				touchwin(system_window);
				wrefresh(system_window);
				continue;

			default : continue;

		} /* end switch */

	} /* end while */

	/* return keystroke to calling routine to indicate finished */

	return keystroke;

} /* list() [4.5.2] */

/************************************************************************/
/* highlight()								*/
/*									*/
/* Highlights a given line of text.					*/
/*									*/
/************************************************************************/

void 	highlight(
	WINDOW 	*win,	/* window to operate in */
	int 	line,	/* position in window */
	int 	column,	/* position in window */
	char 	*text,	/* text to highlight */
	bool	tagged)	/* ...or not */
{
	wstandout(win);
	if(tagged) mvwaddch(win,line,column,CHAR_TAG);
	else mvwaddch(win,line,column,' ');
	mvwaddstr(win,line,column+1,text);
	wstandend(win);

} /* highlight() */

/************************************************************************/
/* unhighlight()							*/
/*									*/
/* Unhighlights a given line of text.					*/
/*									*/
/************************************************************************/

void 	unhighlight(
	WINDOW 	*win,	/* window to operate in */
	int 	line,	/* position in window */
	int 	column,	/* position in window */
	char 	*text,	/* text to unhighlight */
	bool	tagged)	/* ...or not */
{
	if(tagged) mvwaddch(win,line,column,CHAR_TAG);
	else mvwaddch(win,line,column,' ');
	mvwaddstr(win,line,column+1,text);

} /* unhighlight */

/************************************************************************/
/* down_key()								*/
/*									*/
/* Moves the cursor to the next line down.				*/
/* If current line is last line in the window, the window		*/
/* is scrolled up 1 line and the new text added at the last line.	*/
/* Increments line position in window and item counter.			*/
/* 									*/
/************************************************************************/

void 	down_key(
	WINDOW	*win,		/* window to operate in */
	int	*line,		/* position in window */
	int	*item,		/* item counter to increment */
	int	number_items,	/* total number of items */
	char	*next_text,	/* text for next line */
	bool	tagged)		/* ...or not */
{
	/* Increment the item counter if it's not already at the limit */

	if(*item >= number_items-1) return;

	(*item)++;

	/* Scroll the window if necessary and set the line counter */

	if(*line == win->_maxy-1)
	{
		scroll(win);
		*line = win->_maxy-1;
	}
	else (*line)++;

	unhighlight(win,*line,0,next_text,tagged);

	return; 

} /* down_key() */

/************************************************************************/
/* up_key()								*/
/*									*/
/* Moves the cursor to the next line up.				*/
/* If current line is first line in the window, the window		*/
/* is left unmodified else, the cursor is moved up a line.		*/
/* Decrements line position in window and item counter 			*/
/* (if appropriate)							*/
/* 									*/
/************************************************************************/

void 	up_key(
	WINDOW	*win,		/* window to operate in */
	int	*line,		/* position in window */
	int	*item,		/* item counter to increment */
	int	number_items,	/* total number of items */
	char	*next_text,	/* next line of text */
	bool 	tagged)		/* ...or not */
{
	/* Decrement item counter if not first item */

	if(*item <= 0) return;

	(*item)--;

	/* Insert a new line if necessary and decrement line pointer */

	if(*line == 0)
	{
		winsertln(win);
		*line = 0;
	}
	else (*line)--;

	unhighlight(win,*line,0,next_text,tagged);

	return; 

} /* up_key() */

/************************************************************************/
/* key_help()								*/
/*									*/
/* Prints up a list of the movement keys available (different under	*/
/* MS-DOS & UNIX) in the key_window. Waits for user to hit <SPACE BAR>	*/
/* before continuing.							*/
/* Prints a message in system_window as to what keys are available	*/
/* for scrolling and/or acceptance of choices in a list.		*/
/************************************************************************/

void 	key_help(
	bool	tag,	/* user is able to tag | untag items */
	bool	view,	/* user is able to view item */
	bool	can_delete,	/* user is able to delete item */
	bool	print)	/* user is able to print item */
{
	/* internal variables */

	int	line;		/* incrementing variable */
	WINDOW	*key_window;	/* dynamically allocated each time */
#ifdef __NCURSES_H
        MEVENT  mouse;          /* A mouse event. */
#endif


	/*  code is self explanatory */

	if ((key_window = newwin(MAIN_LINES, MAIN_COLS, MAIN_BEGIN_Y,
				  MAIN_BEGIN_X)) == (WINDOW *)ERR)
	{
		error(FATAL,FATAL_1,"key help window");
	}

	/* put up title */

	werase(key_window);
	wstandout(key_window);
	mvwaddstr(key_window,0,0,
	"\tThe following keys are available for moving around the text");
	wstandend(key_window);
	mvwaddstr(key_window,2,0,
	"move		key");
	mvwaddstr(key_window,3,0,
	"----		---");
	mvwaddstr(key_window,4,0,MOVE_LINE_D); 
	mvwaddstr(key_window,5,0,MOVE_LINE_U); 
	mvwaddstr(key_window,6,0,MOVE_PAGE_D); 
	mvwaddstr(key_window,7,0,MOVE_PAGE_U); 
	mvwaddstr(key_window,8,0,MOVE_BEGIN); 
	mvwaddstr(key_window,9,0,MOVE_END); 

	if(tag || view || can_delete || print)
	{
		mvwaddstr(key_window,2,40,
		"move		key");
		mvwaddstr(key_window,3,40,
		"----		---");
		line = 4;

		if(tag)
		{
			mvwaddstr(key_window,line++,40,TAG_TEXT);
			mvwaddstr(key_window,line++,40,UNTAG_TEXT);
		}

		if(view)
		{
			mvwaddstr(key_window,line++,40,VIEW_TEXT);
		}

		if(can_delete)
		{
			mvwaddstr(key_window,line++,40,DELETE_TEXT);
		}

		if(print)
		{
			mvwaddstr(key_window,line++,40,PRINT_TEXT);
		}

	}

	touchwin(key_window);
	wrefresh(key_window);

	message(system_ol_window,CONT_TEXT);

	while(TRUE)
	{
		line = get_command();
#ifdef __NCURSES_H
                if (line == KEY_MOUSE)
                {
                        getmouse(&mouse);

                        if (mouse.bstate == BUTTON1_CLICKED)
                        {
                                /* OK, what key did the mouse represent. */

                                if (mouse.y == SYSTEM_BEGIN_Y &&
                                        mouse.x >= 7 && mouse.x <=9)
                                {
                                        line = KEY_ESC;
                                }
			}
		}
#endif
		if (line == KEY_ESC) break;
	}

	/* free up memory */

	delwin(key_window); 

} /* key_help() */

/************************************************************************/
/* help_line()								*/ 
/*									*/
/* Displays limited help on keys to use in the system window.		*/
/* Has different meanings under UNIX and MS-DOS.			*/
/* If the scroll_flag is set to TRUE then the up/down keys		*/
/* will scroll the page up/down 1 line.					*/
/* If the scroll_flag is set to FALSE then the up/down keys		*/
/* will scroll the cursor up/down 1 line.				*/
/* If the scroll_flag is set to TRUE then the <ENTER> message		*/
/* is not displayed as the user is not able to make this choice.	*/
/* 									*/
/************************************************************************/

void	help_line(bool scroll_flag)
{
	werase(system_window);
	mvwaddstr(system_window,0,0,MOVE_KEY_HELP);

	if(!scroll_flag) waddstr(system_window,"<ENTER> select");

	wrefresh(system_window);

} /* help_line() */

/************************************************************************/
/* find_list_item()							*/
/*									*/
/* Searches a given list for a given directory name.			*/
/* Returns the position found or -1 if not in list.			*/
/************************************************************************/

int	find_list_item(struct slist *list,char *filename)
{
	/* Internal Variables */

	int	i;	/* For looping */

	for(i=0; i < list->number_items; i++)
		if (strcmpi(filename,list->item[i]->filename) == 0) return i;

	return -1;

} /* find_list_item() */

/************************************************************************/
/* alloc_list_item()							*/
/* 									*/
/* Allocates memory for an item in a list.				*/
/* Returns errorlevel. Will give warning if error occurs.		*/
/************************************************************************/

int	alloc_list_item(struct slist *list, int position)
{
	if((list->item[position] = (struct sitem *)malloc(sizeof(struct sitem)))
						== NULL)
		return error(WARNING,
			"Can not add any more items to this list");

	return NOERROR;

} /* alloc_list_item() */

/************************************************************************/
/* free_list_item()							*/
/*									*/
/* Deallocates memory for an entry in a list.				*/
/************************************************************************/

void	free_list_item(struct slist *list, int position)
{
	/* do it */

	free(list->item[position]);

} /* free_list_item() */
