/************************************************************************/
/* uiint.c								*/
/*                                                                      */
/*	This file contains the functions required to initialise		*/
/*	the user interface menu tree structure array menus[] and	*/
/* 	the auto list array autolist[]. There are seven functions	*/
/*	in this file. They are :					*/
/*									*/
/*	init_menu_structure() - Controler of initialisation.		*/
/*	passone() - First pass of initialisation.			*/
/*	checkone() - First check of menu structure.			*/
/*	passtwo() - Second pass of initialisation.			*/
/*	passthree() - Third pass of initialisation.			*/
/*	getline_ui() - Get an input line ignoring those that start with '#'*/
/*	split_string() - Separates first word out of a string		*/
/*									*/
/*	For more detail on these functions see the function headers.	*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/************************************************************************/
/* init_menu_structure()						*/
/* 									*/
/* This is the controling function of the menu structure 		*/
/* initialisation. See source and design for a description of		*/
/* exactly what it does. It returns a pointer to the main menu.		*/
/* 									*/
/************************************************************************/

struct 	smenu *init_menu_structure(char *menufile)
{
	/* Internal variables */

	FILE	*structure_file;	/* Pointer to structure text file */
	struct smenu *mainmenu;		/* Pointer to main menu (returned) */

	/* Initialise ptcome and ptgoto to zero */

	ptcome = 0;
	ptgoto = 0;
	move_down_menu_tree = FALSE;
	move_up_menu_tree = FALSE;

	/* Open menu structure file for reading */

	if (( structure_file = fopen(menufile,"r")) == NULL)
	{
		error(FATAL,FATAL_2,menufile);
	}

	/*
	 * First pass	 - Find menu names and count them
	 *		 - Check that nummenus <= MAXMENU
	 *		 - Allocate memory space for their structure
	 *		 - Place menu names into their structure
	 *		 - Fill rest of structure with NULLs
	 *		 - Set menus[nummenus] to NULL
	 *		 - Rewind the menu structure file
	 */

	 passone(structure_file);

	 /*
	  * First check  - Ensure there is exactly one main menu
	  *		 - Ensure all menu names are unique
	  *		 - Ensure all function names are unique
	  *		 - Ensure no menu names are the same as a function name
	  *		 - Return a pointer to the main menu
	  */

	  mainmenu = checkone();

	  /*
	   * Second pass - Read in menu data and place it in menu structures
	   *		 - Ensure all menu references exist
	   *		 - Ensure no menus are self referential 
	   *		 - Ensure no menus refer to the main menu
	   *		 - Rewind the menu structure file
	   */

	   passtwo(structure_file);

	   /* Third pass - Read in auto list data and fill in array autolist[]
	    *		 - Check that the number of functions is <= MAXAUTO
	    *		 - Ensure all function references exist
	    *		 - Place NULL at end of list
	    *		 - Ensure that the following functions are not in 
	    *		   the list ; auto, help, end, quit
	    *   	 - If a goto exists ensure the place it points to is
  	    *		   already in the autolist				
	    *		 - Do not allow the first function in the list to be
	    *		   a goto				
	    */

	    passthree(structure_file);

	    /* Close the menu structure file */

	    fclose(structure_file);

	    /* Return a pointer to the main menu */

	    return mainmenu;

} /* init_menu_structure() */

/************************************************************************/
/* passone()								*/
/* Process 3.7								*/
/*									*/
/* First pass of scanning of menu structure file. It performs the 	*/
/* following things :							*/
/*									*/
/*		 - Find menu names and count them			*/
/*		 - Check that nummenus <= MAXMENU			*/
/*		 - Allocate memory space for their structure		*/
/*		 - Place menu names into their structure		*/
/*		 - Fill rest of structure with NULLs			*/
/*		 - Set menus[nummenus] to NULL				*/
/*		 - Rewind the menu structure file			*/
/*									*/
/************************************************************************/

void 	passone(FILE *structure_file)
{
	/* Internal variables */

	char *menuname;	 /* Pointer to string holding menu name */
	char *ptr;	 /* Character pointer used for string manipulations */
	int  nummenu;	 /* Integer for counting menus */
	int  loop;	 /* Integer for controling loops */
	char string[81]; /* for holding input lines from get_line() */

	/* Set number of menus to zero */

	nummenu = 0;

	/* Loop through all lines in menu structure file */

	while( getline_ui(string, 81, structure_file) != (char *) EOF)
	{

		/* If this is not a menu name definition ignore it */

		if ( strcmpi("menu", split_string(string, &menuname)) != 0)
			continue;

		/* Allocate memory for menu structure */

		if ( (menus[nummenu] = menu_alloc() ) == (struct smenu *)NULL)
			error(FATAL,FATAL_1,"menu system");

		/* Set all of structure data to NULL */

		ptr = (char *) menus[nummenu];

		for ( loop = 0; loop < sizeof(struct smenu); loop++)
			*ptr++ = (char)NULL;

		/* Copy the menu name into the menu structure */

		strncpy( menus[nummenu]->name, menuname, MAXSTRING );

		/* Incriment number of menus and check it isn't too big */

		if ( ++nummenu >= MAXMENUS ) 
		{
			error(FATAL,FATAL_3,MAXMENUS-1);
		}
	}

	/* Place terminating NULL into menu structure array */

	menus[nummenu] = NULL;

	/* rewind the file */

	rewind(structure_file);

} /* passone() [3.7] */


/************************************************************************/
/* checkone()								*/
/* Process 3.8								*/
/*									*/
/* First (and only at the moment) check of the menu structure		*/
/* array. It performs the following functions :				*/
/*									*/
/* 		 - Ensure there is exactly one main menu		*/
/*		 - Ensure all menu names are unique			*/
/*		 - Ensure all function names are unique			*/
/*		 - Ensure no menu names are the same as a function name	*/
/*		 - Return a pointer to the main menu			*/
/*									*/
/************************************************************************/

struct 	smenu *checkone(void)
{
	/* Internal variables */

	int	count;     /* Number of occurences of a menu name */
	int	mainloop;  /* Looping integer for main loop through array*/
	int	subloop;   /* Looping integer for repeated loop through array */
	struct smenu *mainmenu;  /* Pointer to main menu (returned) */

	/* For each menu in the menu structure array ... */

	for (mainloop = 0; menus[mainloop] != (struct smenu *) NULL; mainloop++)
	{

		/* Check if its name is the same as a function name */

		if ( find_function( menus[mainloop]->name ) )
		{
			error(FATAL,FATAL_4,menus[mainloop]->name);
		}

		/* Count the number of menus in the list with this name */

		count = 0;
		for (subloop=0;menus[subloop] != (struct smenu *)NULL;subloop++)
		{
			if ( strcmpi( menus[mainloop]->name,
					menus[subloop]->name) == 0)
			{
				count++;
			}
		}

		/* Check that this count is one */

		if (count > 1)
		{
			error(FATAL,FATAL_5,menus[mainloop]->name);
		}

	}

	/* Check that function names are unique */

	for(mainloop = 1; functions[mainloop].name != NULL; mainloop++)
	{
		/* Count the number of functions in the list with this name */

		count = 0;
		for (subloop=1;functions[subloop].name != NULL;subloop++)
		{
			if ( strcmpi( functions[mainloop].name,
					functions[subloop].name) == 0)
			{
				count++;
			}
		}

		/* Check that this count is one */

		if (count > 1)
		{
			error(FATAL,FATAL_5,functions[mainloop].name);
		}

	}
	
	/* Find the main menu */

	if (( mainmenu = find_menu( "main" )) == (struct smenu *) NULL )
	{
		error( FATAL,FATAL_6);
	}

	return mainmenu;

} /* checkone() [3.8] */

/************************************************************************/
/* passtwo()								*/
/* Process 3.9								*/
/*									*/
/* Second pass of the menu structure file. It performs the		*/
/* following tasks :							*/
/*									*/
/*		- Read in menu data and place it in menu structures	*/
/*		- Ensure all menu references exist			*/
/*		- Ensure no menus are self referential 			*/
/*		- Ensure no menus refer to the main menu		*/
/*		- Rewind the menu structure file			*/
/*									*/
/************************************************************************/

void 	passtwo (FILE *structure_file)
{
	/* Internal variables */

	char	string[81];	/* Buffer for input string */
	int 	entrypt = 0;	/* Index for entry array in menu structure */
	struct smenu *menupt = NULL;/* Pointer to menu currently being filled */
	struct smenu *menuptemp = NULL;/* Temporary space for a menu pointer */
	int	functionptemp;  /* Temporary space for index to function array*/
	char	*rest;		/* Pointer to part of a string */
	char	*firstword;	/* First word in a string */
	bool	skipping_autolist; /* True if in autolist */

	/* Initialise */

	skipping_autolist = FALSE;

	/* Look at all lines in the structure file */

	while(getline_ui(string, 81, structure_file) != (char *) EOF)
	{
		/* Sepparate the first word and the rest of the string */

		firstword = split_string(string,&rest);

		/* If line is a menu definition set menu pointer to it */

		if (strcmpi("menu",firstword) == 0)
		{
			/* does menu exist */

			if ((menupt = find_menu(rest)) == NULL)
			{
				error(FATAL,FATAL_7,rest);
			}

			entrypt = -1;

			/* This also means we must have passed the auto list */

			skipping_autolist = FALSE;
			continue;
		}

		/* If we are skipping the autolist get the next line */

		else if (skipping_autolist)
		{
			continue;
		}

		/* If it is the autolist definition start skipping it */

		else if ( strcmpi("autolist", firstword) == 0 )
		{
			/* Skip to end of auto list */

			skipping_autolist = TRUE;
			continue;
		}

		/* Otherwise it is a menu entry */

		else
		{
			/* Incriment the entry pointer */

			if (++entrypt >= MAXENTRY)
			{
				error(FATAL,FATAL_8,menupt->name);
			}

			/* Ensure it doesn't refer to the main menu */

			if (strcmpi( "main", firstword) == 0)
			{
			     error(FATAL,FATAL_9,menupt->name);
			}

			/* Find the menu or function reffered to */

			menuptemp = menupt->entry[entrypt].pt_menu = 
							find_menu(firstword);

			functionptemp = menupt->entry[entrypt].pt_funct = 
						    find_function(firstword);

			/* Ensure it isn't self referential */
			/* ie. refer to itself */

			if (menupt == menuptemp)
			{
				error(FATAL,FATAL_10,menupt->name);
			}

			/* Ensure all references exist */

			if (menuptemp == (struct smenu *)NULL && 
					functionptemp == (int)NULL)
			{
				error(FATAL,FATAL_11,menupt->name,
								firstword);
			}

			/* If we get this far menu entry is OK */
			/* Copy the prompt message across */

			strncpy(menupt->entry[entrypt].prompt, rest,MAXSTRING);

		}
	}

	rewind(structure_file);

} /* passtwo() [3.9] */

/************************************************************************/
/* passthree()								*/
/* Process 3.10								*/
/*									*/
/* Third pass of the structure file syntax checking and data entry	*/
/* for getting the auto list. This function performs the following	*/
/* tasks :								*/
/* 		 - Read in auto list data and fill in array autolist[]	*/
/*		 - Check that the number of functions is <= MAXAUTO	*/
/*		 - Ensure all function references exist			*/
/*		 - Place NULL at end of list				*/
/*		 - Ensure that the following functions are not in 	*/
/*		   the list ; auto, help, end, quit			*/
/*		 - If a goto exists ensure the place it points to is	*/
/*		   already in the autolist				*/
/*		 - Do not allow the first function in the list to be	*/
/*		   a goto						*/
/*		 							*/
/************************************************************************/

void 	passthree(FILE *structure_file)
{
	/* Internal variables */

	int	autocount;	/* Number of functions in autolist */
	int	functpt;	/* A function index for general use */
	bool	foundlist;	/* True if the autolist has been found */
	char	*firstword;	/* First word of a string */
	char	*rest;		/* Pointer to rest of string */
	int	loop;		/* Integer used to loop through function array*/
	char	string[81];	/* Buffer for input string */

	/* Initialise */

	foundlist = FALSE;

	/* NOTE THAT AUTOLIST ARRAY STARTS AT POSITION 1 */

	autocount = 1;

	/* Loop through all the lines in the menu structure file */

	while( getline_ui(string, 81, structure_file) != (char *) EOF)
	{
		/* Ensure there are not too many names for autolist */

		if (autocount >= MAXAUTO-1)
		{
			if (autolist[autocount-1] > 0)
			{
				error(WARNING,WARN_12,
					functions[autolist[autocount-1]].name);
			}
			else
			{
				error(WARNING,WARN_13,
			functions[autolist[-autolist[autocount-1]]].name);
			}

			break;
		}

		/*
		 * If this is a menu definition we are either before
		 * or after the autolist
		 */

		if (strcmpi("menu",(firstword = split_string(string,&rest)))								== 0)
		{
			if (foundlist) break;

			continue;
		}

		/* If this is the start of the auto list set foundlist to TRUE*/

		if (strcmpi("autolist", firstword) == 0)
		{
			foundlist = TRUE;
			continue;
		}

		/* If we are not in the auto list go on */

		if (!foundlist)
		{
			continue;
		}

		/* See if this is a goto instruction */

		if (strcmpi("goto", firstword) == 0)
		{
			/* Ensure this isn't the first in the list */

			if (autocount == 1)
			{
				error(FATAL,FATAL_12);
			}

			/* Find function index for function asked to goto */

			functpt = find_function(rest);
			autolist[ autocount] = 0;

			/* Find out if and where it appears in the autolist */

			for (loop = 0; loop < autocount; loop++)
			{
				if (autolist[loop] == functpt)
				{
					autolist[autocount] = -loop;
					break;
				}
			}

			/* Error if it doesn't exist */

			if (autolist[autocount] == 0)
			{
				error(FATAL,FATAL_13,rest);
			}

			/* Incriment the autolist count */

			autocount++;
			continue;
		}

		/* If we get here it is a member of the autolist */
		/* First check function isn't help, auto, end, quit or exit */

		if (strcmpi("help",firstword) == 0)
		{
		      error(FATAL,FATAL_14,"help");
		}

		if (strcmpi("auto",firstword) == 0)
		{
		      error(FATAL,FATAL_14,"auto");
		}

		if (strcmpi("end",firstword) == 0)
		{
		      error(FATAL,FATAL_14,"end");
		}

		if (strcmpi("quit",firstword) == 0)
		{
		      error(FATAL,FATAL_14,"quit");
		}

		if (strcmpi("exit",firstword) == 0)
		{
		      error(FATAL,FATAL_14,"exit");
		}

		/* See if reference exists */

		if (( functpt = find_function(firstword)) == (int)NULL)
		{
			error(FATAL,FATAL_15,firstword);
		}

		/* Put it in the autolist array and argument into autoargs[] */

		autolist[autocount] = functpt;
		strcpy(autoargs[autocount++],rest);
	}

	/* Place terminating NULL on end of list */

	autolist[autocount] = (int)NULL;

} /* passthree() [3.10] */

/************************************************************************/
/* getline_ui()								*/
/* 									*/
/* Reads a line from a given stream as text. Leading white space is 	*/
/* ignored. A line whose first character is '#' is considered		*/
/* a comment and is ignored. The new line character at the end of the	*/
/* string (if present) is replaced by a NULL. Getline returns a pointer */
/* to the string or EOF if the end of the file has been reached.	*/
/* Getline will read up to n-1 characters and then terminate.		*/
/* If a '#' is in the string the rest of the string is considered	*/
/* a comment and is ignored.						*/
/* If the line contains the text "NULL", it will return a NULL string.	*/
/* 									*/
/************************************************************************/

char 	*getline_ui(char *string, int n, FILE *stream)
{
	/* Internal variables */

	int	c;		/* Character */
	int	loop;		/* Boolian loop control */
	char	*retval;	/* Return value of function */
	char	junk[81];	/* Junk string for comments */

	/* Set return value to string begining */

	retval = string;

	loop = TRUE;

	/* Find start of string */

	while(loop)
	{
		switch(c = getc(stream))
		{
			/* If character is white space ignore it */

			case ' '  :
			case '\t' : 
			case '\n' :
			case '\r' : continue;

			/* If character indicates a comment junk the comment */

			case '#'  : if (fgets(junk, n, stream) == NULL)
				            return (char *) EOF;

				    break;

			/* If reach EOF return EOF */

			case EOF  : return (char *) EOF;

			/* Otherwise we have found the strart of the string */

			default   : ungetc(c, stream);
				    loop = FALSE;
				    break;
		}
	}

	/* Read in the string */

	while ( --n > 0 && (c = getc(stream)) != EOF && c != '\n' && c != '\r'
			&& c != '#') *string++ = c;

	/* Look at how we exited the previous loop */

	switch( c )
	{
		/* If it was by EOF return value must be EOF */

		case EOF : retval = (char *) EOF;
			   break;

		/* If it was by a comment junk the comment */

		case '#' : if (fgets(junk, n, stream) == NULL)
			           retval = (char *) EOF;

			   break;

		/* Otherwise do nothing */

		default  : break;
	}

	/* Strip any tabs or spaces from the end of the string */

	while ( *--string == '\t' || *string == ' ')
		;

	/* Put terminating NULL on end of string */

	*++string = (char)NULL;

	/* see if the string is "NULL" text */

	if (strcmpi(retval,"NULL") == 0)
		*retval = (char)NULL;

	/* Return  pointer to the string */

	return retval;

} /* getline_ui() */

/************************************************************************/
/* split_string()							*/
/*									*/
/* Separates the first word of a string. Also sets a pointer to the 	*/
/* rest of the string. Ignores white space at start of string and	*/
/* between the first word and the rest. Returns pointer to first word.	*/
/*									*/
/************************************************************************/

char	*split_string(char *string, char **rest)
{
	/* Internal variables */

	char	*firstword;	/* Pointer to first word of string */

	/* Skip white space and find first word */

	while (*string == ' ' || *string == '\t')  string++;
	firstword = string;

	/* Find end of first word */

	while ( *string != ' ' && *string != '\t' && *string != (char)NULL) 
		string++;

	/* See if it is the only word */

	if (*string == (char)NULL)
	{
		*rest = string;
		return firstword;
	}

	/* Add terminating NULL  */

	*string++ = (char)NULL;

	/* Skip white space and find begining of rest of string */

	while (*string == ' ' || *string == '\t') string++;
	*rest = string;

	return firstword;

} /* split_string() */
