/************************************************************************/
/* v4l2_routines.c							*/
/*                                                                      */
/* FUnctions to talk to the video for linux version 2 interface.	*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar         		                */
/* Date   : Nov 2009 							*/
/************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <time.h>
#include <linux/videodev2.h>

/*
 * globals
 */

#define MAX_TV_BUFFERS 4

static struct v4l2_capability capability;
static struct v4l2_cropcap cropcap;
static struct v4l2_format format;
static struct v4l2_crop crop;
static struct v4l2_requestbuffers reqbuf;
static struct v4l2_buffer tvbuf;
static enum v4l2_buf_type buf_type;
static struct v4l2_control control;
static struct v4l2_input input;
static enum v4l2_priority priority;
static struct { void *start; size_t length; } tv_buffers[MAX_TV_BUFFERS];
static int v4l2_num_tv_buffers[4096];
static v4l2_std_id standard;
static char errmess[4096];

/************************************************************************/
/* xioctl()								*/
/* 									*/
/* Local version of ioctl().						*/
/************************************************************************/

static int xioctl(int fd, int request, void *arg)
{
        int r;

        do r = ioctl (fd, request, arg);
        while (-1 == r && EINTR == errno);

        return r;

} /* xioctl() */

/************************************************************************/
/* v4l2_initialize()							*/
/*									*/
/* Set initial values for the tv global structures.			*/
/* Returns file descriptor if it goes well or negative number on error.	*/
/* You can set the number of buffers setup by the driver here.		*/
/* -1: Failed to open device.						*/
/* -2: Failed to get capabilities.					*/
/* -3: Device not able to capture.					*/
/* -4: Error setting up crop region.					*/
/* -5: Error setting up format.						*/
/* -6: MOre than one byte per pixel.					*/
/* -7: Failed to setup buffers.						*/
/* -8: mmap() failure.							*/
/* -9: Failed to start capturing video.					*/
/************************************************************************/

int v4l2_initialize(char *device, int num_tv_buffers, int cols, int rows)
{
	int	i;
	int	fd;
	int	video_input;

	/* open the device */

	if ((fd = open(device, O_RDWR)) == -1) return -1;

	/* What can the device do? */

	memset(&capability, 0, sizeof(capability));
        if (xioctl(fd, VIDIOC_QUERYCAP, &capability) < 0) return -2;

	/* Check that it can do the mode we wish */

        if (!(capability.capabilities & V4L2_CAP_VIDEO_CAPTURE)) return -3;

	if (!(capability.capabilities & V4L2_CAP_STREAMING)) return -3;

	/* Not sure why these are required, but they are. See vidv4l2.c */

	memset(&capability, 0, sizeof(capability));
        if (xioctl(fd, VIDIOC_QUERYCAP, &capability) < 0) return -10;

	memset(&priority, 0, sizeof(priority));
        if (xioctl(fd, VIDIOC_G_PRIORITY, &priority) < 0) return -11;
        priority = V4L2_PRIORITY_RECORD;
        if (xioctl(fd, VIDIOC_S_PRIORITY, &priority) < 0) return -12;

	memset(&video_input, 0, sizeof(video_input));
        video_input = 0;
	if (xioctl(fd, VIDIOC_S_INPUT, &video_input) < 0) return -13;

        /* Set it to NTSC */

	memset(&input, 0, sizeof(memset));
        input.index = 0;
        if (xioctl(fd, VIDIOC_ENUMINPUT, &input) < 0)

        if (!(input.std & V4L2_STD_NTSC_M)) return -14;

        standard = V4L2_STD_NTSC_M;
        if (xioctl(fd, VIDIOC_S_STD, &standard) < 0) return -15;

	/* Setup for video input, standard etc */

        memset(&cropcap, 0, sizeof(cropcap));
        cropcap.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
        if (xioctl(fd, VIDIOC_CROPCAP, &cropcap) == 0)
        {
                crop.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
                crop.c = cropcap.defrect; /* Reset to full frame default */
                if (xioctl(fd, VIDIOC_S_CROP, &crop) < 0) return -4;
        }

	/* OK, what is the format? */

        memset(&format, 0, sizeof(format));
        format.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
        if (xioctl(fd, VIDIOC_G_FMT, &format) < 0) return -5;

	/* Set the format to what we want */

	memset(&format, 0, sizeof(format));
        format.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
        format.fmt.pix.width = cols;
        format.fmt.pix.height = rows;
        format.fmt.pix.pixelformat = V4L2_PIX_FMT_GREY;
        format.fmt.pix.field       = V4L2_FIELD_INTERLACED;

	if (xioctl(fd, VIDIOC_S_FMT, &format) < 0) return -5;

        /* Did that work? */

        if (xioctl(fd, VIDIOC_G_FMT, &format) < 0) return -5;

	/* Did we get the right number of bytes per pixel? */

	if (format.fmt.pix.bytesperline != format.fmt.pix.width) return -6;

	/* Now we set the IO method and allocate buffer space */

	memset(&reqbuf, 0, sizeof(reqbuf));
	reqbuf.count = num_tv_buffers;
	reqbuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	reqbuf.memory = V4L2_MEMORY_MMAP;
	if (xioctl(fd, VIDIOC_REQBUFS, &reqbuf) < 0) return -7;

	if (reqbuf.count != num_tv_buffers) return -7;

	for(i=0; i<num_tv_buffers; i++)
	{
		memset(&tvbuf, 0, sizeof(tvbuf));
		tvbuf.type   = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		tvbuf.memory = V4L2_MEMORY_MMAP;
		tvbuf.index  = i;
		if (xioctl(fd, VIDIOC_QUERYBUF, &tvbuf) < 0) return -7;
		tv_buffers[i].length = tvbuf.length;
		tv_buffers[i].start = mmap (NULL, tvbuf.length,
		      PROT_READ | PROT_WRITE, MAP_SHARED,
		      fd, tvbuf.m.offset);
		if (tv_buffers[i].start == MAP_FAILED) return -8;
	}
	v4l2_num_tv_buffers[fd] = num_tv_buffers;

        /* Tell the system to start capturing video */

	for (i = 0; i < num_tv_buffers; i++)
	{
		memset (&tvbuf, 0, sizeof(tvbuf));
		tvbuf.type        = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		tvbuf.memory      = V4L2_MEMORY_MMAP;
		tvbuf.index       = i;

		if (xioctl (fd, VIDIOC_QBUF, &tvbuf) < 0) return -9;
	}

	buf_type = V4L2_BUF_TYPE_VIDEO_CAPTURE;

	if (xioctl (fd, VIDIOC_STREAMON, &buf_type) < 0) return -9;

	return fd;

} /* v4l2_initialize() */

/************************************************************************/
/* v4l2_current_brightness()						*/
/*									*/
/* Return current brightness setting for the given device, or -1 	*/
/************************************************************************/

int v4l2_current_brightness(int fd)
{
	control.id = V4L2_CID_BRIGHTNESS;
        if (xioctl(fd, VIDIOC_G_CTRL, &control) < 0) return 0;

	return control.value;

} /* v4l2_current_brightness() */

/************************************************************************/
/* v4l2_current_contrast()						*/
/*									*/
/* Return current contrast setting for the given device, or -1 		*/
/************************************************************************/

int v4l2_current_contrast(int fd)
{
	control.id = V4L2_CID_CONTRAST;
        if (xioctl(fd, VIDIOC_G_CTRL, &control) < 0) return 0;

	return control.value;

} /* v4l2_current_contrast() */

/************************************************************************/
/* v4l2_stop()								*/
/*									*/
/* Closes port and tries to stop the tracking thread.			*/
/* Returns 0 if all goes well or					*/
/* -1: Failed to stop capturing frames.					*/
/* -2: unmap() failure.							*/
/************************************************************************/

int v4l2_stop(int fd)
{
	int	i;

	/* Stop the frame grabber */

	buf_type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	if (xioctl (fd, VIDIOC_STREAMOFF, &buf_type) < 0) return -1;

	/* Unmap the buffers */

	for (i = 0; i < v4l2_num_tv_buffers[fd]; ++i)
	{
		if (munmap(tv_buffers[i].start, tv_buffers[i].length) < 0)
			return -2;
	}

	/* Close the tv */

	close(fd);

	return 0;

} /* v4l2_stop() */

/************************************************************************/
/* v4l2_get_frame()							*/
/*									*/
/* Waits for next frame and returns pointer to current buffer.		*/
/* Returns pointer to buffer or NULL on error.				*/
/************************************************************************/

void *v4l2_get_frame(int fd)
{
	/* Wait for a frame to turn up */

	memset(&tvbuf,0,sizeof(tvbuf));
	tvbuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	tvbuf.memory = V4L2_MEMORY_MMAP;

	if (xioctl (fd, VIDIOC_DQBUF, &tvbuf) && errno != EIO) 
	{
		sprintf(errmess,
		"Failed to wait for a frame to show up (VIDIOC_DQBUF).");
		return NULL;
	}

	if (tvbuf.index >= v4l2_num_tv_buffers[fd] || tvbuf.index < 0) 
	{
		sprintf(errmess,
		"Failed to wait for a frame to show up (tvbuf.index).");
		return NULL;
	}

	/* Go ahead and ask for next frame */

	if (xioctl (fd, VIDIOC_QBUF, &tvbuf) < 0)
	{
		sprintf(errmess,"Failed to ask for next frame.");
		return NULL;
	}

	/* That should be it. */

	*errmess = 0;
	return tv_buffers[tvbuf.index].start;

} /* v4l2_get_frame() */		

/************************************************************************/
/* v4l2_set_brightness()						*/
/*									*/
/* Tries to set teh brightness. Returns 0 if it goes well or		*/

/************************************************************************/

int v4l2_set_brightness(int fd, long int brightness)
{
	control.value = brightness;
	control.id = V4L2_CID_BRIGHTNESS;
	if (xioctl(fd, VIDIOC_S_CTRL, &control) < 0) return -1;

	return 0;

} /* v4l2_set_brightness() */

/************************************************************************/
/* v4l2_set_contrast()							*/
/*									*/
/* Tries to set teh contrast. Returns 0 if it goes well or		*/
/************************************************************************/

int v4l2_set_contrast(int fd, long int contrast)
{
	control.value = contrast;
	control.id = V4L2_CID_CONTRAST;
	if (xioctl(fd, VIDIOC_S_CTRL, &control) < 0) return -1;

	return 0;

} /* v4l2_set_contrast() */

/************************************************************************/
/* v4l2_errmess()							*/
/*									*/
/* Returns any error message there may be.				*/
/************************************************************************/

char *v4l2_errmess(void)
{
	return errmess;

} /* v4l2_errmess() */
