/************************************************************************/
/* build_tables.c							*/
/*                                                                      */
/* Functions to build power spectra, frequency and coefficient tables.	*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <math.h>
#include <time.h>
#include "zernike.h"

/************************************************************************/
/* calc_freqs() 							*/
/*									*/
/* Calculate the vector of frequencies.					*/
/* The vector is allocated and returned by this function.		*/
/* Uses the time_step and number of time steps to work things out.	*/
/* Checks that the frequency range covers a suitable range based on the */
/* Zernike modes you have choosen, although you can ignore the warnings.*/
/* The vector created is float freq[1..header.n].			*/
/************************************************************************/

float	*calc_freqs(zernike_header header)
{
	int	i;
	float	critical_frequency1;
	float	critical_frequencyn;
	float	tau;
	float	*freqs;

	zernike_check_setup();

	/* Allocate space for frequencies and table */

        freqs = vector(1,header.n);

	tau = header.n * header.time_step;

        for(i=1;i<=header.n;i++)
        {
                freqs[i] = (float)i/tau;
        }

        /* Check that this range of frequencies covers what you want */

        critical_frequency1 =
                (n_m[1][N]+1.0) * header.v / (2.0 * M_PI * header.R);

        critical_frequencyn =
                (n_m[header.maxJ][N]+1.0) * header.v / (2.0 * M_PI * header.R);

#ifdef DEBUG
	printf("Start freq = %f\n",freqs[1]);
	printf("End freq   = %f\n",freqs[header.n]);
	printf("Critical 1 = %f\n",critical_frequency1);
	printf("Critical n = %f\n",critical_frequencyn);
#endif
        if (freqs[header.n] <= 2.0*critical_frequencyn ||
                freqs[1] >= critical_frequency1/2.0)
        {
                fprintf(stderr,
"Warning: You may not have enough frequency components to properly describe\n");
                fprintf(stderr,
"the wavefront. The range of critical frequencies is %f to %f Hz,\n",
				 critical_frequency1,critical_frequencyn);
                fprintf(stderr,"while the %d represented frequencies are ",
				header.n);
                fprintf(stderr,"from %f to %f Hz.\n",freqs[1],freqs[header.n]);
        }

	return freqs;

} /* calc_freqs() */

/************************************************************************/
/* build_table() 							*/
/*									*/
/* Calculate the power spectra table.					*/
/* Builds a matrix of Zernike mode power spectra values for these 	*/
/* frequencies.	The switch calcmode determines which type of calculation*/
/* is used. If it is LONG the Rodiier formulation is used. If it is 	*/
/* SHORT the ten Brummelaar approximation is used.			*/
/* Calls calc_freq() to build the frequency vector.			*/
/* The matrix created is float table[1..header.maxJ][1..header.n].	*/
/************************************************************************/

float	**build_ps_table(float **freqs, 
		    zernike_header header,
		    int	calctime)
{
	int	i,j;
	float	**powspec_table; 

	/* Allocate space for frequencies and table */

        powspec_table = matrix(1,header.maxJ,1,header.n);

	/* Build the frequency array */

	*freqs = calc_freqs(header);
	
        /* Build the powspec table ..... could take a while!!! */

        for(j=1;j<=header.maxJ;j++)
        {
                for(i=1;i<=header.n;i++)
                {
                        powspec_table[j][i] = power_spec_a(calctime,
							   j,
							   header.R,
							   header.v,
							   header.L0,
							   (*freqs)[i]
					  	           );
                }
        }

	return powspec_table;

} /* build_table() */

/************************************************************************/
/* calc_coefs() 							*/
/*									*/
/* Uses the matrix of power spectra values and vector of frequencies	*/
/* created by build_table() to create a random matrix of Zernike	*/
/* Coefficients that have the specified power spectra. The coefficients */
/* are normalized for an r0 of 1 and represent phase.			*/
/* The resulting coefficients are sent to the output file which should	*/
/* have already been created by write_header().				*/
/* The file is left open, so you'll have to remember to close it.	*/
/* NOTE THIS FUNCTION CORRUPTS THE FREQUENCY ARRAY!!!!			*/
/************************************************************************/

void	calc_coefs(FILE *output,
		   float **powspec_table, 
		   float *freqs, 
		   zernike_header ps_header,
		   zernike_header cf_header)
{
	float	**phases;	/* Matrix of random phase cosines */
	int	i,j,n;		/* Loop variable */
	float	t;		/* Time of current step */
	zernike_sample	sample;	/* Where each data sample is kept */
	float	norm;		/* Normaliztion factor */
	float	tpi;

	/* Check out number of frames */

	if (cf_header.n*cf_header.time_step > ps_header.n*ps_header.time_step)
	{
	    fprintf(stderr,
	    "calc_coefs: requested time period too long, data will repeat.\n");
	}

	/* Allocate memory */

	sample.a = vector(0,ps_header.maxJ);
	phases = matrix(1,ps_header.maxJ,1,ps_header.n);

	/* Construct the random phases */

	tpi = 2.0*M_PI;
	for(n=1; n<=ps_header.n; n++) /* For each frequency */
	{
		for(j=1; j<=ps_header.maxJ; j++) /* For each mode */
		{
				/* Generate a random phase */

				phases[j][n] = ran3(&random_seed) * tpi;

#ifdef DEBUG
				printf("%5.2f ",phases[j][n]);
#endif
		}
#ifdef DEBUG
		printf("\n");
#endif

	}

	/* Normalization */

	/* So what's with the sqrt anyway ????? */

	norm = sqrt(2.0/((float)ps_header.n*ps_header.time_step));

	/* Parts of header */

	sample.ldc =  0.0;
	sample.tip_tilt_x =  0.0;
	sample.tip_tilt_y = 0.0;
	sample.ople = 0.0;
	sample.a[0] = 1.0;

	/* Find square-root of power spec coefficients */

	for(n=1; n<=ps_header.n; n++) /* For each frequency */
	{
		for(j=1; j<=ps_header.maxJ; j++) /* For each mode */
		{
			powspec_table[j][n] = norm * sqrt(powspec_table[j][n]);
		}
		freqs[n] *= tpi;
	}

	/* Loop through each time step */

	for(t = 0.0, i=0; i<cf_header.n; i++, t += cf_header.time_step) 
	{
		/* Zero out the coefs */

		for(j=1;j<=ps_header.maxJ;j++) sample.a[j] = 0.0;

		/* Construct coefficients */

		for(n=1; n<=ps_header.n; n++) /* For each frequency */
		{

			for(j=1; j<=ps_header.maxJ; j++) /* For each mode */
			{
				sample.a[j] += (powspec_table[j][n] * 
					cos(freqs[n]*t-phases[j][n]));
			}
		}

		/* Send this to the output */

		write_sample(output, cf_header, sample);
	}

	/* Free memory */

	free_vector(sample.a,0,ps_header.maxJ);
	free_matrix(phases,1,ps_header.maxJ,1,ps_header.n);

} /* calc_coefs() */
