/************************************************************************/
/* covariance_zenike.c							*/
/*									*/
/* Calculates covariance of two Zernike coefs using fomulation by Noll. */
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <math.h>
#include "zernike.h"

/*
 * A global bool to see if we have been here before...
 * if we have, fewer calculations need to be done.
 */

static char entered_covar_before = 0;

/************************************************************************/
/* covariance_a()  	                                                */
/*                                                                      */
/* Given the two modes j1 and j2, the radius R(m), and r0(m)		*/ 
/* this function below calculates the covariance of the two 		*/
/* coefficients. Uses Noll formulation as set out by N. Roddier.	*/
/************************************************************************/

float covariance_a(int j1, 
		   int j2, 
		   float R, 
		   float r0) 
{
	/* Internals */
	
	float	result;
	int	sgn;
	float	Kzz;
	float	power;
	int	n1,n2;
	int	m1,m2;
	float	gam_factor;
	static float K_factor;

	zernike_check_setup();

	/* Make sure the request is resonable */

	if (j1 < 2 || j2 < 2 || j1 > zernike_max_j || j2 > zernike_max_j)
	{
		fprintf(stderr,
			"J's (%d,%d) out of range (2-%d) in covariance_a().\n",
			j1,j2,zernike_max_j);
		exit(-9);
	}

	/* Set up globals */

	n1 = n_m[j1][N];
	n2 = n_m[j2][N];
	m1 = n_m[j1][M];
	m2 = n_m[j2][M];

#ifdef DEBUG
	printf("covariance_a(%d,%d,%f,%f)\n",j1,j2,R,r0);
	printf("n1 = %d\n",n1);
	printf("n2 = %d\n",n2);
	printf("m1 = %d\n",m1);
	printf("m2 = %d\n",m2);
#endif

	/* If the m's are not the same the result is zero */

	if (m1 != m2) 
	{
#ifdef DEBUG
		printf("The m's are different result is zero.\n");
#endif
		return 0.0;
	}

	/* If m is non-zero and j1-j2 is odd the result is zero */

	if (m1 != 0 && (j1 - j2) % 2 != 0 ) 
	{
#ifdef DEBUG
		printf("j1-j2 is odd result is zero.\n");
#endif
		return 0.0;
	}

	/* Work out the sign here */

	sgn = (n1 + n2 - 2*m1)/2;

	if (sgn%2 != 0)
		sgn = -1;
	else
		sgn = 1;

#ifdef DEBUG
	printf("Sign chosen is %d\n",sgn);
#endif

	/* Constant factor, calculate it only the first time */

	if (!entered_covar_before)
	{
		K_factor = gammln(14.0/3.0) + 
		   	   5.0/6.0 * (log(24.0/5.0) + gammln(6.0/5.0)) +
			   2.0 * gammln(11.0/6.0);

		K_factor = exp((double)K_factor) / (2.0 * M_PI * M_PI);

		entered_covar_before = TRUE;

#ifdef DEBUG
		printf("K_factor = %f\n",K_factor);
#endif
	}

	/* Factor depending on n's */

	Kzz = K_factor * sgn * sqrt((double)((n1+1)*(n2+1)));

#ifdef DEBUG
	printf("Kzz = %f\n",Kzz);
#endif

	/* Power part .... */

	power = pow((double)(2.0*R/r0),5.0/3.0);

#ifdef DEBUG
	printf("Power = %f\n",power);
#endif

	/* Do the gamma factor part calculation */

	gam_factor = gammln(((float)(n1 + n2) - 5.0/3.0)/2.0) -
		     gammln(((float)(n1 - n2) + 17.0/3.0)/2.0) -
		     gammln(((float)(n2 - n1) + 17.0/3.0)/2.0) -
		     gammln(((float)(n1 + n2) + 23.0/3.0)/2.0);

#ifdef DEBUG
	printf("Gamma part = %lf\n",exp((double)gam_factor));
#endif

	/* Do the last bit..... */

	result = Kzz * power * exp((double)gam_factor);

	/* That's it */

#ifdef DEBUG
	printf("Result = %f\n",result);
#endif
	return result;

} /* covariance_a() */

/************************************************************************/
/* variance_phase_grad(R,r0,epsilon)					*/
/*									*/
/* Calculates the variance of the average phase gradient, or total	*/
/* power in the tilt power spectrum. Uses covariance_a().		*/
/* Radius R and r0 are in meters. Epsilon is the angle at which the 	*/
/* average pahse gradient is measured in radians.			*/
/************************************************************************/

float	variance_phase_grad(float R,
			    float r0,
			    float epsilon)
{
	int 	j;	/* Looping */
	float	result;
	float	multiplier;

	zernike_check_setup();

	result = 0.0;

	for(j=1;j<zernike_max_j;j++)
	{
                if (n_m[j][M] != 1) continue;
	
                multiplier = zernike(j,1.0,epsilon);
                multiplier *= multiplier;
                result += (multiplier*covariance_a(j,j,R,r0));
#ifdef DEBUG
            printf("j=%d multiplier=%f\n",j,multiplier);
#endif 
        }

#ifdef DEBUG
        printf("\nVARIANCE_PHASE_GRAD(%f,%f,%f)=%f\n",R,r,epsilon,result);
#endif
        return result;

} /* variance_phase_grad() */
