/************************************************************************/
/* display_fringe.c							*/
/*									*/
/* For displaying fringes given a bunch of info about it.		*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <stdlib.h>
#include <math.h>
#include "zernike.h"
#ifdef NOT_AN_APPLE
#include <values.h>
#endif

/*
 * Local globals for keeping track of the Window and image.
 */

static XImage   *theImage = (XImage *)NULL;
static char    *picture = (char *)NULL;

/************************************************************************/
/* display_fringe() 							*/
/* 									*/
/* Displays an fringe based on phase profile defined by coeffients.	*/
/* Assumes coefs are normalized for OPD in microns. Specified wavelength*/
/* should also be in microns. Routine also requires matrix of 		*/
/* aperture_pixel structures that defines  				*/
/* the aperture, created by sub_aperture() or 	aperture().		*/
/* The values of npix (size of side of box) and pixmult (pixel 		*/
/* multiplier) are used to determine the size of the image.		*/
/* Routine assumes that a window of the correct size has already been	*/
/* created. The value of disp_type can be LIN, LOG or EXP.		*/
/************************************************************************/

void display_fringe(Window theWindow,
	      aperture_pixel **pixels,
	      float lambda,
	      float	dlambda,
	      float *a1,
	      float *a2,
	      int maxJ,
	      int npix,
	      int pixmult,
	      int disp_type,
	      int   fill)
{
	float	**fringe;
	float	min, max;

        /* Get rid of old Image stuff */

        if (theImage != NULL) XDestroyImage(theImage);

	/* Calculate the fringe */

	if ((fringe=calculate_fringe_intensity(pixels,npix,maxJ,lambda,dlambda,
		a1,a2,fill))==NULL)
	{
		return;
	}

	if (fill == ZERO)
	{
		min = 0.0;
		max = (a1[0]*a1[0] + a2[0]*a2[0])/2 + a1[0]*a2[0];
	}
	else
	{
		min = max = 0.0;
	}

	/* Build up the `picture' */

	picture = make_scaled_picture(npix,npix,pixmult,fringe,disp_type,
			&min, &max);

	/* Make new picture */

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
                      picture,npix*pixmult,npix*pixmult,theBitmapPad, 0);

        if (!theImage)
        {
                fprintf(stderr,"Could not create image....\n");
                XDestroyWindow(theDisplay, theWindow);
		quitX();
                exit(-2);
        }

	XResizeWindow(theDisplay,theWindow,npix*pixmult,npix*pixmult);
        XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,
                npix*pixmult,npix*pixmult);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);

	free_matrix(fringe,1,npix,1,npix);

} /* display_fringe() */

/************************************************************************/
/* calculate_fringe_intensity()						*/
/*									*/
/* Creates a fringe intensity pattern from zernike coefficients.	*/
/* Assumes coefs are normalized for OPD in microns.			*/
/* The wavelength must also be specified in microns.			*/
/* The routine also requires matrix of aperture_pixel structures 	*/
/* that defines the aperture, created by sub_aperture() or aperture().  */
/* The fill parameter specifies what to place outside the aperture, 	*/
/* and can take the values MEAN, MIN, MAX or ZERO.			*/
/* Returned matrix is float fringe[1..npix][1..npix].			*/
/************************************************************************/

float **calculate_fringe_intensity(aperture_pixel **pixels,
		        int npix,
		        int maxJ,
			float	lambda,
			float	dlambda,
		        float *a1,
			float *a2,
			int   fill)
{
	int	k,l;
	float	value;
	float	**values;
	float	*a3;
	float	a12pa22,a1a2;
        float   min;
        float   max;
        float   mean;
	float	wn1,wn2,dwn;
	float	phase, x, sincx;
        int     num_values;

	/* Allocate the memory */

	values = matrix(1,npix,1,npix);
	a3 = vector(0,maxJ);

	/* Set up some constants */

	a12pa22 = (a1[0]*a1[0] + a2[0]*a2[0])/2;
	a1a2 = a1[0]*a2[0];
	wn1 = 1.0/(lambda - dlambda/2.0);
	wn2 = 1.0/(lambda + dlambda/2.0);
	dwn = wn1 - wn2;

	/* Find the difference in phases */

	for(k=1;k<=maxJ;k++) a3[k] = (a1[k] - a2[k]);

        /* Now go through, yes, the entire array */

        min = MAXFLOAT;
        max = -1.0 * MAXFLOAT;
        mean = 0.0;
        num_values = 0;

	for(k=1;k<=npix;k++)
        {
		for(l=1;l<=npix;l++)
                {
#ifdef DEBUG
			printf("Position (%d,%d)\n",k,l);
#endif
			/* Do nothing outside aperture */

			if (pixels[k][l].mask == OUTSIDE) 
			{
				values[k][l] = 0.0;
				continue;
			}

			/* sinc value */

			phase = pixel_phase(pixels[k][l],maxJ,lambda,a3);

			if ((x = M_PI * dwn * phase) == 0)
				sincx = 1.0;
			else
				sincx = sin(x)/x;

			/* Inside aperture do calculation */

			value = a12pa22 + a1a2 * sincx * cos(phase);

                        switch(fill)
                        {
                                case MEAN: num_values++;
                                           mean += value;
                                           break;

                                case MIN: if (value < min) min = value;
                                          break;

                                case MAX: if (value > max) max = value;
                                          break;

                                default: break;
                        }

                        values[k][l] = value;

#ifdef DEBUG
			printf("Value = %f.\n",values[k][l]);
#endif
		}
        }
        if (fill == MEAN) mean = mean/(float)num_values;

        /* Fill everything outside aperture with apropriate value */

        for(k=1;k<=npix;k++)
        {
                for(l=1;l<=npix;l++)
                {
                        if (pixels[k][l].mask == OUTSIDE)
                        {
                                switch(fill)
                                {
                                        case MEAN: values[k][l] = mean;
                                                   break;

                                        case MIN: values[k][l] = min;
                                                  break;

                                        case MAX: values[k][l] = max;
                                                  break;

                                        default:  values[k][l] = 0.0;
                                                  break;
                                }
                        }
                }
        }

	/* Clean up and go */

	free_vector(a3,0,maxJ);
	return values;

} /* calculate_fringe_intensity() */

/************************************************************************/
/* calculate_fringe_phase()						*/
/*									*/
/* Creates a fringe phase pattern from zernike coefficients.		*/
/* Assumes coefs are normalized for OPD in microns.			*/
/* The wavelength must also be specified in microns.			*/
/* The routine also requires matrix of aperture_pixel structures 	*/
/* that defines the aperture, created by sub_aperture() or aperture().  */
/* The fill parameter specifies what to place outside the aperture, 	*/
/* and can take the values MEAN, MIN, MAX or ZERO.			*/
/* Returned matrix is float phase[1..npix][1..npix].			*/
/************************************************************************/

float **calculate_fringe_phase(aperture_pixel **pixels,
		        int npix,
		        int maxJ,
			float	lambda,
		        float *a1,
			float *a2,
			int   fill)
{
	int	k,l;
	float	value;
	float	**values;
        float   min;
        float   max;
        float   mean;
	float	phase1, phase2;
        int     num_values;

	/* Allocate the memory */

	values = matrix(1,npix,1,npix);

        /* Now go through, yes, the entire array */

        min = MAXFLOAT;
        max = -1.0 * MAXFLOAT;
        mean = 0.0;
        num_values = 0;

	for(k=1;k<=npix;k++)
        {
		for(l=1;l<=npix;l++)
                {
			if (pixels[k][l].mask == OUTSIDE) 
			{
				values[k][l] = 0.0;
				continue;
			}

			/* Calculate the two phases */

			phase1 = pixel_phase(pixels[k][l],maxJ,lambda,a1);
			phase2 = pixel_phase(pixels[k][l],maxJ,lambda,a2);

			/* Calculate fringe phase, log book Bonn page  14 */

			value = atan2(a1[0]*sin(phase1) + a2[0]*sin(phase2),
				      a1[0]*cos(phase1) + a2[0]*cos(phase2));

                        switch(fill)
                        {
                                case MEAN: num_values++;
                                           mean += value;
                                           break;

                                case MIN: if (value < min) min = value;
                                          break;

                                case MAX: if (value > max) max = value;
                                          break;

                                default: break;
                        }

                        values[k][l] = value;
		}
        }
        if (fill == MEAN) mean = mean/(float)num_values;

        /* Fill everything outside aperture with apropriate value */

        for(k=1;k<=npix;k++)
        {
                for(l=1;l<=npix;l++)
                {
                        if (pixels[k][l].mask == OUTSIDE)
                        {
                                switch(fill)
                                {
                                        case MEAN: values[k][l] = mean;
                                                   break;

                                        case MIN: values[k][l] = min;
                                                  break;

                                        case MAX: values[k][l] = max;
                                                  break;

                                        default:  values[k][l] = 0.0;
                                                  break;
                                }
                        }
                }
        }

	/* That is all */

	return values;

} /* calculate_fringe_phase() */
