/************************************************************************/
/* display_image.c							*/
/*									*/
/* For displaying Images given a bunch of info about it.		*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <stdlib.h>
#include <math.h>
#include "zernike.h"
#ifdef NOT_AN_APPLE
#include <values.h>
#endif

/*
 * Local globals for keeping track of the Window and image.
 */

static XImage   *theImage = (XImage *)NULL;
static char    *picture = (char *)NULL;

/************************************************************************/
/* display_image() 							*/
/* 									*/
/* Displays an image based on phase profile defined by coeffients.	*/
/* Assumes coefs are normalized for OPD in microns. Specified wavelength*/
/* should also be in microns. Routine also requires matrix of 		*/
/* aperture_pixel structures that defines  				*/
/* the aperture, created by sub_aperture() or 	aperture().		*/
/* The values of npix (size of side of box) and pixmult (pixel 		*/
/* multiplier) are used to determine the size of the image.		*/
/* Routine assumes that a window of the correct size has already been	*/
/* created. The value of disp_type can be LIN, LOG or EXP.		*/
/* Parameter imbedp2 sets the size of the matrix used to perform FFT.	*/
/* It is the power of 2 to use, ie 4 uses a 16x16 matrix.		*/
/************************************************************************/

void display_image(Window theWindow,
	      aperture_pixel	**pixels,
	      float lambda,
	      float *a,
	      int maxJ,
	      int npix,
	      int pixmult,
	      int imbedp2,
	      int disp_type)
{
	float	**image;

        /* Get rid of old Image stuff */

        if (theImage != NULL) XDestroyImage(theImage);

	/* Calculate the image */

	if ((image = calculate_image(pixels,npix,imbedp2,maxJ,lambda,a))
		== (float **)NULL) 
	{
		return;
	}

	/* Build up the `picture' */

	picture = make_picture(npix,npix,pixmult,image,disp_type);

	/* Make new image */

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
                      picture,npix*pixmult,npix*pixmult, theBitmapPad, 0);

        if (!theImage)
        {
                fprintf(stderr,"Could not create image....\n");
                XDestroyWindow(theDisplay, theWindow);
		quitX();
                exit(-2);
        }

	XResizeWindow(theDisplay,theWindow,npix*pixmult,npix*pixmult);
        XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,
                npix*pixmult,npix*pixmult);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);

	free_matrix(image,1,npix,1,npix);

} /* display_image() */

/************************************************************************/
/* calculate_image()							*/
/*									*/
/* Creates an image from zernike coefficients.				*/
/* Airy disk will have 2^imbed2/npix pixels across it.			*/
/* Assumes coefs are normalized for OPD in microns.			*/
/* The wavelength must also be specified in microns.			*/
/* The routine also requires matrix of aperture_pixel structures 	*/
/* that defines the aperture, created by sub_aperture() or aperture().  */
/* The fill parameter specifies what to place outside the aperture, 	*/
/* and can take the values MEAN, MIN, MAX or ZERO.			*/
/* Returned matrix is float image[1..npix][1..npix].			*/
/* Parameter imbedp2 sets the size of the matrix used to perform FFT.	*/
/* It is the power of 2 to use, ie 4 uses a 16x16 matrix.		*/
/************************************************************************/

float **calculate_image(aperture_pixel **pixels,
		        int npix,
		        int imbedp2,
		        int maxJ,
			float lambda,
		        float *a)
{
	int	i,j;
	float	**phase;
	float	**amp;
	float	**image;

	/* Work out phases from the Zernike coefficients */

	phase = calculate_phase(npix,pixels,maxJ,lambda,a,ZERO);

	/* All the amplitudes are the same */

	amp = matrix(1, npix, 1, npix);

	for(i=1; i<=npix; i++)
	for(j=1; j<=npix; j++)
	{
		if (pixels[i][j].mask == OUTSIDE)
			amp[i][j] = 0.0;
		else
			amp[i][j] = a[0];
	}

	/* Work it out */

	image = calculate_image_amp_phase(pixels, npix, imbedp2, amp, phase);

	/* Clean up memory */

	free_matrix(phase,1,npix,1,npix);
	free_matrix(amp,1,npix,1,npix);

	/* That is all */
	
	return image;

} /* calculate_image() */

/************************************************************************/
/* calculate_image_amp_phase()						*/
/*									*/
/* Creates an image from arrays of amp[litudes and phases.		*/
/* Airy disk will have 2^imbed2/npix pixels across it.			*/
/* The routine also requires matrix of aperture_pixel structures 	*/
/* that defines the aperture, created by sub_aperture() or aperture().  */
/* The fill parameter specifies what to place outside the aperture, 	*/
/* and can take the values MEAN, MIN, MAX or ZERO.			*/
/* Returned matrix is float image[1..npix][1..npix].			*/
/* Parameter imbedp2 sets the size of the matrix used to perform FFT.	*/
/* It is the power of 2 to use, ie 4 uses a 16x16 matrix.		*/
/************************************************************************/

float **calculate_image_amp_phase(aperture_pixel **pixels, 
			int npix, 
        		int imbedp2, 
			float **amp, 
			float **phase)
{
	int	imbed;
	int	imbed2;
	float	*aperture;
	float	*ap;
	int	i,j;
	unsigned long int *nn;
	float	**image;
	int	ix,iy,offset;

	/* Get real values for npix and imbed */

	imbed = 1<<imbedp2;
	imbed2 = imbed >> 1;

	if (imbed > 1024)
	{
		fprintf(stderr,
	"Using VERY large arrays for image...  this will take forever.\n");
	}

	if (npix > imbed)
	{
		fprintf(stderr,"Number of pixels greater than array.\n");
		return NULL;
	}

	/* Allocate memory */

	aperture = vector(1,2*imbed*imbed);
	image = matrix(1,npix,1,npix);
	nn = lvector(1,2);

	/* Calculate offset */

	offset = (imbed - npix)>>1;

	/* Generate the aperture field */

	ap = &(aperture[1]);
        for(i=1;i<=imbed;i++)
        {
                for(j=1;j<=imbed;j++)
                {
			if (((ix = i-offset) < 1 || ix > npix ||
			    (iy = j-offset) < 1 || iy > npix) ||
			    (pixels[ix][iy].mask == OUTSIDE))
			{
			    *ap++ = 0.0;
			    *ap++ = 0.0;
			}
			else
			{
			    *ap++ = amp[ix][iy]*cos(phase[ix][iy]);
			    *ap++ = amp[ix][iy]*sin(phase[ix][iy]);
			}
		}
	}

	/* Set up for and do FFT */

	nn[1] = imbed;
	nn[2] = imbed;

 	fourn((float*)aperture, nn, 2, 1); 

	/* Now pull out the image .... */

	ap = &(aperture[1]);

        for(i=1; i<=imbed; i++)
        {
                for(j=1; j<=imbed ; j++)
                {
			/*
			 * Find the equivalent image position 
			 * including the shift to get image in center 
			 */

			if ((ix = (i+imbed2)%imbed) == 0) ix = imbed;
			if ((iy = (j+imbed2)%imbed) == 0) iy = imbed;

			/* Are we inside the image area ? */

			if ((ix -= offset) > 0 && ix <= npix &&
			    (iy -= offset) > 0 && iy <= npix)
			{
				image[ix][iy] = *ap * *ap + *(ap+1) * *(ap+1);
			}
			ap++;
			ap++;
                }
        }

	/* Clean up memory */

	free_vector(aperture,1,2*imbed*imbed);
	free_lvector(nn,1,2);

	/* That is all */
	
	return image;

} /* calculate_image_amp_phase() */
