/************************************************************************/
/* display_phase.c							*/
/*									*/
/* For displaying Phase given a bunch of info about it.			*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <stdlib.h>
#include <math.h>
#include "zernike.h"
#ifdef NOT_AN_APPLE
#include <values.h>
#endif

/*
 * Local globals for keeping track of the Window and image.
 */

static XImage	*theImage = (XImage *)NULL;
static char    *picture = (char *)NULL;

/************************************************************************/
/* display_sub_phase()							*/
/*									*/
/* Displays the phase profile in an Xwindow of gievn set of coordinates.*/
/* Assumes coefs are normalized for OPD in microns. Specified wavelength*/
/* should also be in microns. Routine also requires matrix of 		*/
/* aperture_pixel structures that defines  				*/
/* the aperture, created by sub_aperture() or 	aperture().		*/
/* The values of npix (size of side of box) and pixmult (pixel 		*/
/* multiplier) are used to determine the size of the image.		*/
/* Routine assumes that a window of the correct size has already been	*/
/* created. The value of disp_type can be LIN, LOG or EXP.		*/
/************************************************************************/

void 	display_phase(Window theWindow,
			  aperture_pixel **pixels,
			  float lambda,
		          float	*a,
		          int	maxJ,
		          int	npix,
		          int	pixmult,
		          int	disp_type)
{
	float	**values;

	/* Get rid of old Image stuff */

        if (theImage != NULL) XDestroyImage(theImage);

        /* Do calculations */

        values = calculate_phase(npix,pixels,maxJ,lambda,a,MIN);

        /* Now make the picture out of all this .......... */

        picture = make_picture(npix,npix,pixmult,values,disp_type);

	/* Make a new image */

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
                       picture,npix*pixmult,npix*pixmult,theBitmapPad,0);

        if (!theImage)
        {
                fprintf(stderr,"Could not create image....\n");
                free(values);
                free(picture);
                XDestroyImage(theImage);
                XDestroyWindow(theDisplay, theWindow);
                exit(-2);
        }

        /* Now display it */

	XResizeWindow(theDisplay,theWindow,npix*pixmult,npix*pixmult);
        XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,
                npix*pixmult,npix*pixmult);
        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);

	/* Free up memory */

	free_matrix(values,1,npix,1,npix);

} /* display_phase() */

/************************************************************************/
/* calculate_phase()							*/
/*									*/
/* Calculates phase across sub aperture given Zernike coefs.		*/
/* Assumes coefs are normalized for OPD in microns.			*/
/* The wavelength must also be specified in microns.			*/
/* The routine also requires matrix of aperture_pixel structures 	*/
/* that defines the aperture, created by sub_aperture() or aperture().  */
/* The fill parameter specifies what to place outside the aperture, 	*/
/* and can take the values MEAN, MIN, MAX or ZERO.			*/
/* Returned matrix is float phases[1..npix][1..npix].			*/
/************************************************************************/

float	**calculate_phase(int npix, 
			     aperture_pixel **pixels,
			     int maxJ, 
			     float lambda,
			     float *a, 
			     int fill)
{
        int 	k,l;	      /* Various integers */
        float   value;
	float	min;
	float	max;
	float	mean;
	int 	num_values;
	float	**values;

#ifdef DEBUG
	printf("Inside calculate_phase(%d,pixels,%d,%f,%f,a,%d)\n",npix,
		maxJ,lambda,fill);
#endif
	/* Allocate the memory */

	values = matrix(1,npix,1,npix);

        /* Now go through, yes, the entire array */

	min = MAXFLOAT;
	max = -1.0 * MAXFLOAT;
	mean = 0.0;
	num_values = 0;

	for(k=1;k<=npix;k++)
        {
		for(l=1;l<=npix;l++)
                {
			/* Do nothing outside the aperture */

			if (pixels[k][l].mask == OUTSIDE) 
			{
				values[k][l] = 0.0;
				continue;
			}
#ifdef DEBUG
	         	printf("Calling pixel_phase(pixel,%d,a)\n",a);
#endif
			value = pixel_phase(pixels[k][l],maxJ,lambda,a);

			switch(fill)
			{
				case MEAN: num_values++;
					   mean += value;
					   break;

				case MIN: if (value < min) min = value;
					  break;

				case MAX: if (value > max) max = value;
					  break;

				default: break;
			}

#ifdef DEBUG
			printf("Value = %f.\n",value);
#endif
			values[k][l] = value;
		}
        }
	if (fill == MEAN) mean = mean/(float)num_values;

#ifdef DEBUG
	printf("min = %f max = %f mean = %f num_values = %d\n",
		min,max,mean,num_values);
#endif

	/* Fill everything outside aperture with minimum value */

	for(k=1;k<=npix;k++)
        {
		for(l=1;l<=npix;l++)
                {
			if (pixels[k][l].mask == OUTSIDE) 
			{
				switch(fill)
				{
					case MEAN: values[k][l] = mean;
						   break;

					case MIN: values[k][l] = min;
						  break;

					case MAX: values[k][l] = max;
						  break;

					default:  values[k][l] = 0.0;
						  break;
				}
			}
		}
	}

	return values;

} /* calculate_phase() */
