/************************************************************************/
/* files.c 								*/
/*                                                                      */
/* File IO routines.							*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <stdio.h>
#include "zernike.h"

/************************************************************************/
/* write_header()							*/
/*									*/
/* Takes a zernike header structure and writes into a file. Returns	*/
/* file pointer for that file.						*/
/* The file is created, if it already exists an error is returned.	*/
/* If file name is NULL uses stdout and ascii.				*/
/* Returns NULL on error.						*/
/************************************************************************/

FILE	*write_header(char *filename,zernike_header header)
{
	FILE	*fp;

	/* Make sure we are not overwriting a file */

	if (filename != NULL && (fp = fopen(filename,"r")) != NULL)
	{
		fprintf(stderr,"File %s already exists.\n",filename);
		return NULL;
	}

	/* OK, open the file ... */

	if (filename != NULL && (fp = fopen(filename,"wb")) == NULL)
	{
		fprintf(stderr,"Could not open file %s for writing.\n",
				filename);
		return NULL;
	}

	/* Put the header into it */

	if (filename == NULL)
	{
		fp = stdout;
		printf("# Domain = ");
		switch(header.domain)
		{
			case TIME_DOMAIN: printf("TIME\n");
				   break;

			case FREQ_DOMAIN: printf("FREQUENCY\n");
				        break;

			case PHASE_GRAD: printf("PHASE_GRAD\n");
					 break;

			case PHASE_GRAD_PS: printf("PHASE_GRAD_PS\n");
					 break;

			default: fprintf(stderr,
				"Write_header: unknown domain in header.\n");
			 	exit(-5);
		}
		printf("# R(m)   = %f\n",header.R);
		printf("# v(m/s) = %f\n",header.v);
		printf("# max J  = %d\n",header.maxJ);
		printf("# N      = %d\n",header.n);
		printf("# dt (s) = %f\n",header.time_step);
		printf("# l0 (m) = %f\n",header.l0);
		printf("# L0 (m) = %f\n",header.L0);
		printf("# Res 1  = %f\n",header.reserved1);
		printf("# Res 2  = %f\n",header.reserved2);
		printf("# Res 3  = %f\n",header.reserved3);
		printf("# Res 4  = %f\n",header.reserved4);
		printf("# Res 5  = %f\n",header.reserved4);
	}
	else
	{
		if (fwrite(&header,sizeof(zernike_header),1,fp) != 1)
		{
			fprintf(stderr,"Unable to write header to file %s.\n",
					filename);
			fclose(fp);
			return NULL;
		}
	}

	/* Return the file pointer */

	return fp;

} /* write_header() */

/************************************************************************/
/* read_header()							*/
/*									*/
/* Takes a zernike header structure and reads from a file. Returns	*/
/* file pointer for that file.						*/
/* The file is opened by this function. If it does not exists an error	*/
/* is returned.								*/
/* If filename is NULL uses stdin and ascii.				*/
/* Returns NULL on error.						*/
/************************************************************************/

FILE	*read_header(char *filename,zernike_header *header)
{
	FILE	*fp;
	char	s[81];

	/* OK, open the file ... */

	if (filename != NULL && (fp = fopen(filename,"rb")) == NULL)
	{
		fprintf(stderr,"Could not open file %s for reading.\n",
				filename);
		return NULL;
	}

	/* Read the header from it */

        if (filename == NULL)
        {
                fp = stdin;
		scanf("# Domain = %s\n",s);
		if (strcmp(s,"TIME") == 0)
			header->domain = TIME_DOMAIN;
		else if (strcmp(s,"FREQUENCY") == 0)
			header->domain = FREQ_DOMAIN;
		else if (strcmp(s,"PHASE_GRAD") == 0)
			header->domain = PHASE_GRAD;
		else if (strcmp(s,"PHASE_GRAD_PS") == 0)
			header->domain = PHASE_GRAD_PS;
		else
		{
			fprintf(stderr,
				"Read_header: unknown domain in header.\n");
			exit(-5);
		}
		
                scanf("# R(m)   = %f\n",&(header->R));
                scanf("# v(m/s) = %f\n",&(header->v));
                scanf("# max J  = %d\n",&(header->maxJ));
                scanf("# N      = %d\n",&(header->n));
                scanf("# dt (s) = %f\n",&(header->time_step));
                scanf("# l0 (m) = %g\n",&(header->l0));
                scanf("# L0 (m) = %g\n",&(header->L0));
                scanf("# Res 1  = %f\n",&(header->reserved1));
                scanf("# Res 2  = %f\n",&(header->reserved2));
                scanf("# Res 3  = %f\n",&(header->reserved3));
                scanf("# Res 4  = %f\n",&(header->reserved4));
                scanf("# Res 5  = %f\n",&(header->reserved4));
        }
        else
	{
		if (fread(header,sizeof(zernike_header),1,fp) != 1)
		{
			fprintf(stderr,"Unable to read header from file %s.\n",
					filename);
			fclose(fp);
			return NULL;
		}
	}

	/* Return the file pointer */

	return fp;

} /* read_header() */

/************************************************************************/
/* copy_header()							*/
/*									*/
/* Coppies a Zernike header from one structure to another.		*/
/************************************************************************/

void copy_header(zernike_header *header_to, zernike_header header_from)
{
	unsigned char	*p1,*p2;
	int	i;

	p1 =  (unsigned char *)&header_from;
	p2 = (unsigned char *)header_to;

	for(i=0; i < sizeof(zernike_header) ;i++) *p2++ = *p1++;

} /* copy_header() */

/************************************************************************/
/* cmp_header()								*/
/*									*/
/* Compares two headers. 						*/
/* Returns TRUE if they are the same, FALSE otherwise. 			*/
/* If force_n is TRUE, n in both headers set to lowest n of the two.	*/
/************************************************************************/

int cmp_header(zernike_header *header_a, zernike_header *header_b,
		int force_n)
{

	if (header_a->domain != header_b->domain)
	{
		fprintf(stderr,"cmp_header: Headers have different domain.\n");
		return FALSE;
	}

	if (header_a->R != header_b->R)
	{
		fprintf(stderr,"cmp_header: Headers have different R.\n");
		return FALSE;
	}

	if (header_a->v != header_b->v)
	{
		fprintf(stderr,"cmp_header: Headers have different v.\n");
		return FALSE;
	}

	if (header_a->maxJ != header_b->maxJ)
	{
		fprintf(stderr,"cmp_header: Headers have different maxJ.\n");
		return FALSE;
	}

	if (!force_n)
	{
		if (header_a->n != header_b->n)
		{
			fprintf(stderr,
				"cmp_header: Headers have different n.\n");
			return FALSE;
		}
	}
	else
	{
		if (header_a->n < header_b->n)
			header_a->n = header_b->n;
		else
			header_b->n = header_a->n;
	}

	if (header_a->time_step != header_b->time_step)
	{
		fprintf(stderr,
		"cmp_header: Headers have different time_step.\n");
		return FALSE;
	}

	if (header_a->l0 != header_b->l0)
	{
		fprintf(stderr,"cmp_header: Headers have different l0.\n");
		return FALSE;
	}

	if (header_a->L0 != header_b->L0)
	{
		fprintf(stderr,"cmp_header: Headers have different L0.\n");
		return FALSE;
	}

	if (header_a->reserved1 != header_b->reserved1 ||
		header_a->reserved2 != header_b->reserved2 ||
		header_a->reserved3 != header_b->reserved3 ||
		header_a->reserved4 != header_b->reserved4 ||
		header_a->reserved5 != header_b->reserved5)
	{
		fprintf(stderr,
		"cmp_header: Headers have different reserved values.\n");
		return FALSE;
	}

	return TRUE;
}

/************************************************************************/
/* write_sample()							*/
/*									*/
/* Writes one lot of coefficients out of data file.			*/
/* It assumes that the file has been openned and the header is in place.*/
/* This is normally done by write_header().				*/
/************************************************************************/

void	write_sample(FILE *fp, zernike_header header, zernike_sample sample)
{
	int 	i;
	int	n;

	switch(header.domain)
	{
		case TIME_DOMAIN:
		case FREQ_DOMAIN: n = header.maxJ;
			break;

		case PHASE_GRAD:
		case PHASE_GRAD_PS: n = 1;
			break;

		default: fprintf(stderr,
			"Write_coefs: unknown domain in header.\n");
			exit(-5);
	}

	if (fp == stdout)
	{
		printf("%13.6e ",sample.ldc);
		printf("%13.6e ",sample.tip_tilt_x);
		printf("%13.6e ",sample.tip_tilt_y);
		printf("%13.6e ",sample.ople);
		for(i=0;i<=n; i++) printf("%13.6e ",sample.a[i]);
		putchar('\n');
	}
	else
	{
		if (fwrite(&sample,sizeof(float),4,fp) != 4)
		{
			fprintf(stderr,"Error writing sample data.\n");
		}
		if (fwrite(sample.a,sizeof(float),n+1,fp) != n+1)
		{
			fprintf(stderr,"Error writing sample data.\n");
		}
	}

} /* write_sample() */

/************************************************************************/
/* read_sample()							*/
/*									*/
/* Reads one lot of coefficients out of data file.			*/
/* Assumes file is open and header has been read in.			*/
/* This is normally done by read_header().				*/
/************************************************************************/

void	read_sample(FILE *fp, zernike_header header, zernike_sample *sample)
{
        int  	i;
        int     n;

        switch(header.domain)
        {
                case TIME_DOMAIN:
                case FREQ_DOMAIN: n = header.maxJ;
                        break;

                case PHASE_GRAD:
                case PHASE_GRAD_PS: n = 1;
                        break;

                default: fprintf(stderr,
                        "Read_coefs: unknown domain in header.\n");
                        exit(-5);
        }

        if (fp == stdin)
        {
		scanf("%e",&(sample->ldc));
		scanf("%e",&(sample->tip_tilt_x));
		scanf("%e",&(sample->tip_tilt_y));
		scanf("%e",&(sample->ople));
                for(i=0;i<=n; i++) 
		{
			scanf("%e",(sample->a)+i);
		}
        }
	else
	{
		if (fread(sample,sizeof(float),4,fp) != 4)
		{
			fprintf(stderr,"Error reading sample data.\n");
		}

		if (fread(sample->a,sizeof(float),n+1,fp) != n+1)
		{
			fprintf(stderr,"Error reading sample data.\n");
		}
	}

} /* read_sample() */

/************************************************************************/
/* write_powspec_table() 						*/
/*									*/
/* If we are in FREQUENCY domain print out table.  			*/
/* Uses write_header() and write_sample(). Creates normal file type	*/
/* in Frequency domain.							*/
/************************************************************************/

void	write_powspec_table(char *filename,
		      float *freqs,
		      float **powspec_table, 
		      zernike_header header)
{
	int	n,j;
	zernike_sample	sample;
	FILE	*output;

	/* Check that we are in FREQUENCY space */

	if (header.domain != FREQ_DOMAIN)
	{
		fprintf(stderr,"Can not write TIME domain to FREQUENCY file\n");
		return;
	}

        /* Write the header information out */

        if ((output = write_header(filename,header)) == NULL)
        {
		return;
        }

	/* Allocate memory */

	sample.ldc = sample.tip_tilt_x = sample.tip_tilt_y = sample.ople = 0.0;
	sample.a = vector(0,header.n);

	for(n=1; n<=header.n; n++) /* For each frequency */
	{
		sample.a[0] = freqs[n];
		for(j=1; j<=header.maxJ; j++) /* For each mode */
		{
			sample.a[j] = powspec_table[j][n];
		}
		write_sample(output,header,sample);
	}

	/* Close the file */

        if (output != stdout) fclose(output);

	/* Free memory */

	free_vector(sample.a,0,header.maxJ);

} /* write_powspec_table() */

/************************************************************************/
/* read_powspec_table() 						*/
/*									*/
/* If we are in FREQUENCY domain read in table.  			*/
/* Uses read_header() and read_sample(). Assumes normal file type	*/
/* in Frequency domain.							*/
/************************************************************************/

float	**read_powspec_table(char *filename,
			float **freqs,
		      zernike_header *header)
{
	int	n,j;
	zernike_sample	sample;
	FILE	*input;
	float	**powspec_table; 

        /* Read the header information in */

        if ((input = read_header(filename,header)) == NULL)
        {
                exit(-2);
        }

	/* Check that we are in FREQUENCY space */

	if (header->domain != FREQ_DOMAIN)
	{
		fprintf(stderr,"Can not read TIME domain to FREQUENCY table\n");
		fclose(input);
		return NULL;
	}

	/* Allocate memory */

        powspec_table = matrix(1,header->maxJ,1,header->n);
	*freqs = vector(1,header->n);
	sample.a = vector(0,header->n);
	
        for(n=1; n<=header->n; n++) /* For each frequency */
        {
		read_sample(input,*header,&sample);

		(*freqs)[n] = sample.a[0];

	        for(j=1; j<=header->maxJ; j++) /* For each mode */
	        {
			powspec_table[j][n] = sample.a[j]; 
	        }
        }

	/* Close the file */

        if (input != stdin) fclose(input);

	/* Free memory */

	free_vector(sample.a,0,header->maxJ);

	return	powspec_table;

} /* read_powspec_table() */
