/************************************************************************/
/* fit_zernike.c							*/
/*                                                                      */
/* Routines to fit zernike terms to a square array of floats.  		*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <math.h>
#include "zernike.h"

/* 
 * Some globals just for this file.
 */

static int global_maxJ;
static aperture_pixel **global_pixels;
static void	zern_fit_funct(float *ftn, float *var, float *param);

/************************************************************************/
/* fit_zernike() 							*/
/*									*/
/* Fits zernike polynomials to a square array of floats.		*/
/* Expects that you have already setup zernike calcs and an aperture    */
/* array.								*/
/* Returns a vector (internally allocated) a(0..maxJ) of zernike 	*/
/* terms found.								*/
/* If fit fails this is the BFI results found by simple numerical	*/
/* integration.								*/
/************************************************************************/

float	*fit_zernike(	int maxJ, 
			float **image, 
			float npix, 
			aperture_pixel **pixels,
			int display_type)
{
	int	i,j,k;
	float	*a;
	int	ndata;
	float	*data;
	float	**param;
	float	rmsres,maxres;

	/* Allocate some memory */

	global_maxJ = maxJ;
	global_pixels = pixels;
	a = vector(0,maxJ);
	a[0] = a[1] = 0.0;

	/* Use a direct numerical method to get the `first guess' */

	for(k=2;k<=maxJ;k++)
	{
		a[k] = 0;
		for(i=1;i<=npix;i++)
		for(j=1;j<=npix;j++)
		{
			a[k] += (zernike(k,pixels[i][j].rho,pixels[i][j].theta)
					* image[i][j]);
		}

		a[k] *= 2.0/(npix * npix);

	}

#ifdef DEBUG
	printf("Started fit_zernike()\n");
	printf("First guess at result: \n");
	for(i=1; i<=maxJ;i++) printf("a[%d] = %f\n",i,a[i]);
#endif

	/* First, let's work out how many data points there really are */

	ndata = 0;
	for(i=1; i<= npix; i++)
	for(j=1; j<= npix; j++)
	{
		if (pixels[i][j].mask == INSIDE) ndata++;
	}

#ifdef DEBUG
	printf("Number of data points = %d\n",ndata);
#endif

	/* Allocate the memory we need */

	data = vector(1,ndata);
	param = matrix(1,ndata,1,2);

	/* And fill things up in the right way... */

	k = 1;
	for(i=1; i<=npix; i++)
	for(j=1; j<=npix; j++)
	{
		if (pixels[i][j].mask == OUTSIDE) continue;

		data[k] = image[i][j];
		param[k][1] = i;
		param[k][2] = j;
		k++;
	}
		
	/* OK, try and do a fit.... */

	non_linear_fit(maxJ,a,ndata,data,2,param,TRUE,zern_fit_funct,
		&rmsres,&maxres, 1.0, 3, display_type);

	/* That is all */

	free_vector(data,1,ndata);
	free_matrix(param,1,ndata,1,2);

	return a;

} /* fit_zernike() */

/************************************************************************/
/* zern_fit_funct()							*/
/*									*/
/* Function used in fitting zernike data without a mask.		*/
/************************************************************************/

static void	zern_fit_funct(float *ftn, float *var, float *param)
{
	int	i;
#ifdef DEBUG
	
	printf("\nzern_fit_funct()\n");
	printf("param[1] = %f param[2] = %f\n",param[1],param[2]);
#endif

	ftn[0] = 0.0;
	for(i=1; i<=global_maxJ; i++)
	{
		ftn[i] = global_pixels[(int)param[1]][(int)param[2]].a[i];
		ftn[0] += (var[i]*ftn[i]);
#ifdef DEBUG
		printf("ftn[%d] = %f\n",i,ftn[i]);
#endif
	}
#ifdef DEBUG
	printf("ftn[0] = %f\n",ftn[0]);
#endif
}

/************************************************************************/
/* fit_zernike_masked()							*/
/*									*/
/* Fits zernike polynomials to a square array of floats.		*/
/* Expects that you have already setup zernike calcs and an aperture    */
/* array.								*/
/* Returns a vector (internally allocated) a(0..maxJ) of zernike 	*/
/* terms found.								*/
/* If fit fails this is the BFI results found by simple numerical	*/
/* integration.								*/
/* mask must be supplied to show which elements may be used.		*/
/************************************************************************/

float	*fit_zernike_masked(	int maxJ, 
			float **image, 
			int   **mask,
			float npix, 
			aperture_pixel **pixels,
			int display_type)
{
	int	i,j,k;
	float	*a;
	int	ndata;
	float	*data;
	float	**param;
	float	rmsres,maxres;

	/* Allocate some memory */

	global_maxJ = maxJ;
	global_pixels = pixels;
	a = vector(0,maxJ);
	a[0] = a[1] = 0.0;

	/* Use a direct numerical method to get the `first guess' */

	for(k=2;k<=maxJ;k++)
	{
		a[k] = 0;
		for(i=1;i<=npix;i++)
		for(j=1;j<=npix;j++)
		{
			if (mask[i][j])
			{
				a[k] += (zernike(k,
					 pixels[i][j].rho,
					 pixels[i][j].theta)
					* image[i][j]);
			}
		}

		a[k] *= 2.0/(npix * npix);

	}

#ifdef DEBUG
	printf("Started fit_zernike()\n");
	printf("First guess at result: \n");
	for(i=1; i<=maxJ;i++) printf("a[%d] = %f\n",i,a[i]);
#endif

	/* First, let's work out how many data points there really are */

	ndata = 0;
	for(i=1; i<= npix; i++)
	for(j=1; j<= npix; j++)
	{
		if (pixels[i][j].mask == INSIDE && mask[i][j]) ndata++;
	}

#ifdef DEBUG
	printf("Number of data points = %d\n",ndata);
#endif

	/* Allocate the memory we need */

	data = vector(1,ndata);
	param = matrix(1,ndata,1,2);

	/* And fill things up in the right way... */

	k = 1;
	for(i=1; i<=npix; i++)
	for(j=1; j<=npix; j++)
	{
		if (pixels[i][j].mask == OUTSIDE || !mask[i][j]) continue;

		data[k] = image[i][j];
		param[k][1] = i;
		param[k][2] = j;
		k++;
	}
		
	/* OK, try and do a fit.... */

	non_linear_fit(maxJ,a,ndata,data,2,param,TRUE,zern_fit_funct,
		&rmsres,&maxres, 1.0, 3, display_type);

	/* That is all */

	free_vector(data,1,ndata);
	free_matrix(param,1,ndata,1,2);

	return a;

} /* fit_zernike_masked() */
