/************************************************************************/
/* mem_alloc.c								*/
/*                                                                      */
/* Memory allocation routines for the aperture arrays.			*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.                               */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996	                                        */
/************************************************************************/

/* Include files */

#include <stdlib.h>
#include <math.h>
#include "zernike.h"

#define NR_END 1
#define FREE_ARG char*

/**************************************************************************/
/* appixmatrix()							  */
/*									  */
/* Allocate an aperture_pixel matrix with subscript m[nrl..nrh][ncl..nch] */
/**************************************************************************/

aperture_pixel **appixmatrix(int nrl, int nrh, int ncl, int nch)
{
  int i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  aperture_pixel **m;

  /* allocate pointers to rows */
  m=(struct APERTURE_PIXEL**) malloc((size_t)((nrow+NR_END)*
		sizeof(struct APERTURE_PIXEL*)));
  if (!m) nrerror("allocation failure 1 in appixmatrix()");
  m += NR_END;
  m -= nrl;

  /* allocate rows and set pointers to them */
  m[nrl]=(aperture_pixel *) malloc((size_t)((nrow*ncol+NR_END)*
		sizeof(struct APERTURE_PIXEL)));
  if (!m[nrl]) nrerror("allocation failure 2 in appixmatrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

  /* return pointer to array of pointers to rows */
  return m;
}

/************************************************************************/
/* free_appixmatrix()							*/
/*									*/
/* Free an aperture_pixel matrix allocated by fcmatrix()		*/
/************************************************************************/

void free_appixmatrix(aperture_pixel **m,int nrl,int nrh, int ncl, int nch)
{
  free((FREE_ARG) (m[nrl]+ncl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
}

/************************************************************************/
/* sub_obsc_aperture()							*/
/*									*/
/* Allocates memory for the whole parture structure.			*/
/* Calculates mask, rho and theta across sub aperture.			*/
/* All the values for all the Zernike coeffs are also calculated.	*/
/* This can take a long time, but it saves time later...		*/
/************************************************************************/

aperture_pixel	**sub_obsc_aperture(int npix, 
		     int maxJ,
		     float pos_rho, 
		     float pos_theta,
		     float size_rho,
		     float in_rho,
		     float out_rho
		     )
{
        float   rho,theta;
        int 	j,k,l;	      /* Various integers */
        float   frame_center; /* of aperture in pixel coords */
	float	units_per_pixel;
        float   x,y;          /* Coords of given pixel */
	float	x_offset;
	float	y_offset;
	float	aperture_size;
	float	r;
	aperture_pixel	**pixels;

	/* Allocate the memory */

	pixels = appixmatrix(1,npix,1,npix);

	/* Set up some calculations */

	frame_center = (float)npix / 2.0 + 0.5;
	units_per_pixel = size_rho * 2.0 / (float)npix;
	aperture_size = size_rho * size_rho;

	in_rho *= in_rho;
	if (in_rho < 0.0) 
		in_rho == 0.0;
	else if (in_rho > 1.0)
		in_rho = 1.0;

	out_rho *= out_rho;
	if (out_rho < 0.0) 
		out_rho == 0.0;
	else if (out_rho > 1.0)
		out_rho = 1.0;

	if (in_rho > out_rho)
	{
		r = out_rho;
		out_rho = in_rho;
		in_rho = r;
	}

#ifdef DEBUG
	printf("Frame center = %f\n",frame_center);
	printf("Units per pixel = %f\n",units_per_pixel);
	printf("Aperture size = %f\n",aperture_size);
#endif 

	if (pos_theta != 0.0)
	{
		x_offset = pos_rho * cos(pos_theta);
		y_offset = pos_rho * sin(pos_theta);
	}
	else
	{
		x_offset = pos_rho;
		y_offset = 0.0;
	}
#ifdef DEBUG
	printf("x_offset = %f\n",x_offset);
	printf("y_offset = %f\n",y_offset);
#endif 

        /* Now go through, yes, the entire array */

	for(k=1;k<=npix;k++)
        {
		for(l=1;l<=npix;l++)
                {
#ifdef DEBUG
			printf("Position (%d,%d)\n",k,l);
#endif
			/* Get pixel coordinates */

                        x = ((float)k - frame_center) * units_per_pixel;
                        y = ((float)l - frame_center) * units_per_pixel;
#ifdef DEBUG
			printf("Normalised (%f,%f)\n",x,y);
#endif

			/* Are we within the aperture ? */

                        if (x*x+y*y > aperture_size) /* NO */
                        {
				pixels[k][l].mask = OUTSIDE;
				pixels[k][l].x = 0.0;
				pixels[k][l].y = 0.0;
				pixels[k][l].rho = 0.0;
				pixels[k][l].theta = 0.0;
				pixels[k][l].maxJ = 0;
				pixels[k][l].a = NULL;
#ifdef DEBUG
				printf("Outside subaperture.\n");
#endif
				continue;
                        }

			/* Move coords */

			x = x + x_offset;
			y = y + y_offset;
#ifdef DEBUG
			printf("Calibrated (%f,%f)\n",x,y);
#endif
				
                        /* Are we outside the main aperture? */

                        if ((rho=x*x+y*y) > out_rho || rho < in_rho)
			{
				pixels[k][l].mask = OUTSIDE;
				pixels[k][l].x = 0.0;
				pixels[k][l].y = 0.0;
				pixels[k][l].rho = 0.0;
				pixels[k][l].theta = 0.0;
				pixels[k][l].maxJ = 0;
				pixels[k][l].a = NULL;
#ifdef DEBUG
				printf("Outside aperture.\n");
#endif
				continue;
			}

			rho = sqrt(rho);

			if (x != 0 || y != 0)
				theta = atan2(y,x);
			else
				theta = 0.0;

			if (theta<0) theta += (2.0*M_PI);

#ifdef DEBUG
			printf("Cylindrical (%f,%f).\n",rho,theta/M_PI*180);
#endif
			pixels[k][l].mask = INSIDE;
			pixels[k][l].x = x;
			pixels[k][l].y = y;
			pixels[k][l].rho = rho;
			pixels[k][l].theta = theta;

			/* Calculate all Zernike values for this pixel */

			pixels[k][l].maxJ = maxJ;
			pixels[k][l].a = vector(0,maxJ);
			pixels[k][l].a[0] = 1.0;
			for(j=1;j<=maxJ;j++)
			{
				pixels[k][l].a[j] = zernike(j,rho,theta);
			}
				
#ifdef DEBUG
			printf("X    = %f.\n",pixels[k][l].x);
			printf("Y    = %f.\n",pixels[k][l].y);
			printf("Rho  = %f.\n",pixels[k][l].rho);
			printf("Theta= %f.\n",pixels[k][l].theta);
#endif
		}
        }

	return pixels;

} /* sub_obsc_aperture() */

/************************************************************************/
/* free_sub_obsc_aperture()						*/
/*									*/
/* Cleans up memory from an aperture definition.			*/
/************************************************************************/

void free_sub_obsc_aperture(aperture_pixel **pixels, int npix) 
{
	int i,j;

	/* Loop through the pixel array and release memory */

	for(i=1;i<-npix;i++)
	for(j=1;j<-npix;j++)
	{
		if (pixels[i][j].mask == INSIDE)
		{
			free_vector(pixels[i][j].a,0,pixels[i][j].maxJ);
		}
	}

	/* Deallocate the memory for the whole thing */

	free_appixmatrix(pixels,1,npix,1,npix);

} /* free_sub_obsc_aperture() */
