/************************************************************************/
/* misc_calculations.c							*/
/*                                                                      */
/* Routines to perform miscilanious calculations.			*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.                               */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996	                                        */
/************************************************************************/

/* Include files */

#include <math.h>
#include "zernike.h"

/************************************************************************/
/* image_center_intensity()                                             */
/*                                                                      */
/* Calculates the image intensity at center of image.                   */
/* Works best for an odd number of pixels.                              */
/* Assumes Zernike coefficients are normalized for OPD in Microns and	*/
/* then uses wavelength (also in microns) to calculate phase.		*/
/* Uses phase information in zernike coeffs and aperture array.		*/
/************************************************************************/

float image_center_intensity(aperture_pixel **pixels,
                                int npix,
                                float *a,
                                int maxJ,
                                float lambda)
{
        int     i,j;
        float   real;
        float   imaginary;
        float   result;
        float   phase;

        real = imaginary = 0.0;

        for(i=1;i<=npix;i++)
        for(j=1;j<=npix;j++)
        {
                if (pixels[i][j].mask != INSIDE) continue;

                phase = pixel_phase(pixels[i][j],maxJ,lambda,a);
                real += cos(phase);
                imaginary += sin(phase);
        }

        result = real*real+imaginary*imaginary;

#ifdef DEBUG
        printf("Central Intensity found %f\n",result);
#endif

        return result;

} /* image_center_intensity() */

/************************************************************************/
/* total_aperture_values()						*/
/*									*/
/* Totals up the values inside the aperture, image or fringes.		*/
/* Requires matrix of pixel structures to define aperture.		*/
/************************************************************************/

float total_aperture_values(aperture_pixel **pixels, 
			    float **values,
			    int	  npix)
{
	int	i,j;
	float	result;

	result = 0.0;

	for(i=1;i<=npix;i++)
	for(j=1;j<=npix;j++)
	{
		if (pixels[i][j].mask == INSIDE) result += values[i][j];
	}

	return result;

} /* total_aperture_values() */

/************************************************************************/
/* tilt_var_to_r0() 							*/
/* 									*/
/* Uses Noll's formulation to convert tilt varriance to an r0 measure.	*/
/* Tilt var in radians squared.						*/
/* Requires beam radius in meters and wavelength in microns.		*/
/************************************************************************/

float tilt_var_to_r0(float var, float R, float lambda)
{
	float	D;
	float	DdivLambda;
	float	temp;
	float	r0;

	D = R * 2.0;
	DdivLambda = D / (lambda * 1.0e-6);

	temp = var / 0.183 * DdivLambda * DdivLambda;

	r0 = D * pow(temp,-0.6);

	return r0;
	
} /* tilt_var_to_r0() */
