/************************************************************************/
/* power_spec_zernike.c							*/
/*									*/
/* Routines for calculating the power spectra of Zernike coefficients.	*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <math.h>
#include "zernike.h"

/*
 * Local globals
 */

static float	wind_speed;
static int	zernike_order;
static float	frequency;
static float	radius;
static float	outer_scale_const;

/*
 * Local function decarations
 */

static float short_power_spec_a(int    j,
			 float R,
                         float v,
			 float L0,
                         float f);

static float long_integral_func(float fy);

static float long_power_spec_a(int    j,
			float R,
			float v,
			float L0,
			float f);

/************************************************************************/
/* power_spec_a()  	                                                */
/*                                                                      */
/* Given the parameters mode j, aperture radius R, wind speed v and 	*/
/* frequency f this functions 						*/
/* calculates the value of Waj(f), the the power of this    		*/
/* zernike coefficient due to atmospheric turbulence for r0 = 1m.	*/
/* The parameter calc_time chooses between LONG (Roddier method) and 	*/
/* SHORT (ten Brummelaar method) calculation times. 			*/
/* The outer-scale length L) must also be specified in meters, but	*/
/* often one can use MAXFLOAT (infinite outer scale...).		*/
/************************************************************************/

float	power_spec_a(int    calc_time,
		     int   j,
		     float R,
		     float v,
		     float L0,
		     float f)
{
	char	mess[81];

	/* Check we can at least try this */

	if (j < 1 || j > zernike_max_j) 
	{
	sprintf(mess,"j value %d out of range (1 to %d) in POWER_SPEC_A",j,
			zernike_max_j);
		nrerror(mess);
	}

	if (calc_time == LONG)
	{
		return long_power_spec_a(j,R,v,L0,f);
	}	
	else
	{
		return short_power_spec_a(j,R,v,L0,f);
	}

} /* power_spec_a(0 */

/*
 * short_power_spec_a()
 * 
 * A local function.
 *
 * Given the parameters j, R, v and f the functions below
 * calculate the value of Waj(f), the value of the power spectrum of
 * jth zernike coefficient.
 * Power is for r0 = 1 and at wavelength of LAMBDA_0.
 */

static float short_power_spec_a(int    j,
			 float R,
			 float v,
		         float L0,
			 float f)
{
	/* Internals */

	float 	result;
	float	temp1;
	float	temp2;
	float	f0;

	zernike_check_setup();

#ifdef DEBUG
	printf("\nSHORT_POWER_SPEC_A(%d,%f,%f,%f)=",j,R,v,f);
#endif

	/* Preliminary calculations */

	if (f == 0.0) return 0;

	if (v != 0.0) 
		f0 = f * 2 * M_PI * R / v;
	else
		return 0.0;

	if (n_m[j][N] == 0)
		temp1 = bessj1(f0)/f0;
	else
		temp1 = bessj(n_m[j][N]+1,f0)/f0;

	temp2 = f * L0 / (2.0 * M_PI * v);

	temp2 = 1.0 - exp(-1.0*temp2*temp2);

	if (v <= 0.0)
		result = 0;
	else
		result = 77.70471012 * pow(R,1.66666666666666666) * (R/v) *
			 ((float)n_m[j][N] +1.0)*pow(f0,-2.666666666666666)*
				temp1 * temp1 * temp2;

#ifdef DEBUG
	printf("%f\n",result);
#endif

	return result;

} /* short_power_spec_a() */

/*
 * long_integral_func()
 *
 * A local function only.
 *
 * Calculates function inside integral of Roddier papaer.
 */

static float long_integral_func(float fy)
{
	/* Internals */

	float fx;
	float f2;
	float temp1;
	float temp2;
	float temp3;
	float theta;
	float result;

	zernike_check_setup();

#ifdef DEBUG2
	printf("Inside integral_func().\n\n");
	printf("Frequency = %f\n",frequency);
	printf("Zernike order = %d\n",zernike_order);
	printf("Wind speed = %f\n",wind_speed);
#endif

	fx = radius * frequency / wind_speed;

	f2 = fx*fx + fy*fy;

	if (n_m[zernike_order][N] == 0)
		temp1 = bessj1(2.0*M_PI*sqrt(f2));
	else
		temp1 = bessj(n_m[zernike_order][N]+1,2.0*M_PI*sqrt(f2));

	if (n_m[zernike_order][M] == 0)
	{
		theta = 0;
		temp2 = 1;
	}
	else
	{
		theta = atan2(fy,fx);
		if (zernike_order%2 == 0) /* EVEN */
		{
			temp2 = cos(n_m[zernike_order][M]*theta);
			temp2 *= (2.0*temp2);
		}
		else /* ODD */
		{
			temp2 = sin(n_m[zernike_order][M]*theta);
			temp2 *= (2.0*temp2);
		}
	}
		

	temp3 = 1.0 - exp(-1.0*frequency*frequency*outer_scale_const);

#ifdef DEBUG2
	printf("temp1 = %f\n",temp1);
	printf("temp2 = %f\n",temp2);
	printf("temp3 = %f\n",temp3);
	printf("Theta = %f\n",theta/M_PI*180.0);
	printf("n = %d\n",n_m[zernike_order][N]);
	printf("m = %d\n",n_m[zernike_order][M]);
#endif
	result = pow(f2,-17.0/6.0)*temp1*temp1*temp2*temp3;

	return result;

} /* long_integral_func() */

/*
 * long_power_spec_a()
 * 
 * A local function only.
 *
 * Given the parameters j, R, v and f the functions below
 * calculate the value of Waj(f), the value of the power spectrum of
 * jth zernike coefficient.
 * This routine uses the `long' calculation time integral.
 * Power is for r0 = 1 and at wavelength of LAMBDA_0.
 */

static float long_power_spec_a(int    j,
			float R,
			float v,
		        float L0,
			float f)
{
        float result;
	float integral;
	float break_point;

	zernike_check_setup();

        zernike_order=j;
        wind_speed = v;
        frequency=f;
	radius = R;
	outer_scale_const = L0 / (2.0 * M_PI * v);
	outer_scale_const *= outer_scale_const;

	/* Choose a break point near first Bessel zero */

        break_point = 11.0 + n_m[zernike_order][N];
	integral = qromo(long_integral_func,0.0,break_point,midpnt) + 
		   qromo(long_integral_func,break_point,1.0e300,midinf);

	/* WARNING WILL ROBINSON !! A KLUDGE OF 2.0 * HAS BEEN ADDED */

        result = (2.33038722e-3 * (n_m[j][N]+1) * pow(R,5.0/3.0) * R / v) * 
			4.0 * integral;
#ifdef DEBUG
        printf("\nLONG_POWER_SPEC(%d,%f,%f)=%f\n",j,v,f,result);
#endif
        return result;

} /* long_power_spec() */
