/************************************************************************/
/* visibility.c								*/
/*									*/
/* Visibility calculation functions.					*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include "zernike.h"

/************************************************************************/
/* susi_visiblity_2()	 						*/
/*									*/
/* Calculate the visibility squared using method used in SUSI.		*/
/* Requires a matrix of aperture_pixel structures defining the aperture */
/* as created by sub_aperture() or aperture(), the number of pixel on	*/
/* each side of the phase matrix npix, the maximum number of Zernike 	*/
/* modes used maxJ, the wavelength in microns and the two sets of	*/
/* Zernike coefficients a1 and a2. 					*/
/* These coefficients must be normalize in microns of OPD.		*/
/************************************************************************/

float   susi_visibility_2(aperture_pixel **pixels,
                            int npix,
                            int maxJ,
                            float lambda,
			    float dlambda,
                            float *a1,
                            float *a2)
{
	float	sum_1,sum_2;
	float	**fringe;
	float	result;
	float	shift_180;

#ifdef DEBUG
	printf("susi_visibility_2(pixels,%d,%d,%f,%f,a,a)\n",npix,maxJ,
			lambda);
#endif
	/* Calculate fringe pattern 1 */

	fringe = calculate_fringe_intensity(pixels,npix,maxJ,lambda,dlambda,a1,a2,ZERO);
	sum_1 = total_aperture_values(pixels,fringe,npix);
	free_matrix(fringe,1,npix,1,npix);

	/* For PI phase change on one beam */

	shift_180 = lambda/2;

	a2[1] += shift_180;
#ifdef DEBUG
	printf("Change in a[1] parameter is %f\n",shift_180);
#endif

	/* Calculate fringe pattern 2 */

	fringe = calculate_fringe_intensity(pixels,npix,maxJ,lambda,dlambda,a1,a2,ZERO);
	sum_2 = total_aperture_values(pixels,fringe,npix);
	free_matrix(fringe,1,npix,1,npix);

	/* Undo PI phase change */

	a2[1] -= shift_180;

#ifdef DEBUG
	printf("Sum fringe A = %f\n",sum_1);
	printf("Sum fringe B = %f\n",sum_2);
#endif

	/* return the result */

	result = (sum_1-sum_2)/(sum_1+sum_2);
	result = 2.0*result*result;

#ifdef DEBUG
	printf("Result = %f\n",result);
#endif

	return result;
	
} /* susi_visibility_2() */

/************************************************************************/
/* tango_twiss_visibility_2() 						*/
/*									*/
/* Calculate the visibility squared using method outlined in 		*/
/* Tango/Twiss paper.							*/
/* Requires a matrix of aperture_pixel structures defining the aperture */
/* as created by sub_aperture() or aperture(), the number of pixel on	*/
/* each side of the phase matrix npix, the maximum number of Zernike 	*/
/* modes used maxJ, the wavelength in microns and the two sets of	*/
/* Zernike coefficients a1 and a2. 					*/
/* These coefficients must be normalize in microns of OPD.		*/
/************************************************************************/

float   tango_twiss_visibility_2(aperture_pixel **pixels,
                            int npix,
                            int maxJ,
                            float lambda,
			    float dlambda,
                            float *a1,
                            float *a2)
{
	float	**fringe;
	float	result;
	float	shift_90;
	float	n10,n11,n20,n21;
	float	n1,n2;
	float	q;

#ifdef DEBUG
	printf("tango_twiss_visibility_2(pixels,%d,%d,%f,%f,a,a)\n",npix,maxJ,
			lambda);
#endif
	/* Calculate Phase shift parameter */

	shift_90 = lambda/4;

	/* Calculate parameters */

	fringe = calculate_fringe_intensity(pixels,npix,maxJ,lambda,dlambda,a1,a2,ZERO);
	n10 = total_aperture_values(pixels,fringe,npix);
	free_matrix(fringe,1,npix,1,npix);
#ifdef DEBUG
	printf("n10 = %f\n",n10);
#endif

	a2[1] += shift_90;
	fringe = calculate_fringe_intensity(pixels,npix,maxJ,lambda,dlambda,a1,a2,ZERO);
	n11 = total_aperture_values(pixels,fringe,npix);
	free_matrix(fringe,1,npix,1,npix);
#ifdef DEBUG
	printf("n11 = %f\n",n11);
#endif

	a2[1] += shift_90;
	fringe = calculate_fringe_intensity(pixels,npix,maxJ,lambda,dlambda,a1,a2,ZERO);
	n20 = total_aperture_values(pixels,fringe,npix);
	free_matrix(fringe,1,npix,1,npix);
#ifdef DEBUG
	printf("n20 = %f\n",n20);
#endif

	a2[1] += shift_90;
	fringe = calculate_fringe_intensity(pixels,npix,maxJ,lambda,dlambda,a1,a2,ZERO);
	n21 = total_aperture_values(pixels,fringe,npix);
	free_matrix(fringe,1,npix,1,npix);
#ifdef DEBUG
	printf("n21 = %f\n",n21);
#endif

	/* Undo 3/2 PI phase change */

	a2[1] -= (3.0*shift_90);

	/* Do the rest of the calculations */

	n1 = n10 + n11;
	n2 = n20 + n21;
	q = (n10-n20)*(n10-n20) + (n11-n21)*(n11-n21);
	result = 4.0*q/((n1+n2)*(n1+n2));
#ifdef DEBUG
	printf("n1 = %f\nn2 = %f\nq = %f\n",n1,n2,q);
	printf("V^2 = %f\n",result);
#endif

	/* return the result */

	return result;
	
} /* tango_twiss_visibility_2() */
