/************************************************************************/
/* zernike.c 								*/
/*                                                                      */
/* Basic routines for Zernike polynomial calculations.			*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <math.h>
#include "zernike.h"

/* 
 * Some globals just for this file.
 */

static float	norm = (2.0*M_PI/LAMBDA_0);
static float    last_lambda = LAMBDA_0;

/* Uncomment this for tests */

//#define DEBUG

/************************************************************************/
/* zernike()								*/
/*									*/
/* Calculates value of jth zernike function at given position.		*/
/* Note all internals are doubles due to large numbers encountered.	*/
/************************************************************************/

float	zernike(int    j,
		float rho,
		float theta)
{
	double	R;
	char	err_mess[81];

	zernike_check_setup();

	/* Check we can at least try this */

	if (j < 1 || j > zernike_max_j) 
	{
	     sprintf(err_mess,"j value %d out of range (%d to %d) in ZERNIKE",
			j,1,zernike_max_j);

	     nrerror(err_mess);
	}

	if (rho<0.0 || rho > 1.0) 
	{
	  sprintf(err_mess,"rho value %f out of range (0 to 1) in ZERNIKE",rho);
	  nrerror(err_mess);
	}

	/* Calculate polynomial part */

#ifdef IMBEDD_R_CALCULATION
	R = 0;
	sign = 1;
	for(s=0; s <= ((n_m[j][N]-n_m[j][M])/2); s++)
	{
		if (n_m[j][N]-2*s == 0)
		{
			power = 1.0;
		}
		else
		{
			power = (rho == 0)?0.0:pow(rho,(double)(n_m[j][N]-2*s));
		}

#ifdef DEBUG
		printf("Power = %f\n",power);
		printf("factorial(n_m[j][N]-s)=%f\n",factorial(n_m[j][N]-s));
#endif
	 	R += (
			(sign * factorial(n_m[j][N]-s)) /
/*		---------------------------------------------------	*/
		(factorial(s)*factorial((n_m[j][N]+n_m[j][M])/2-s)*
		    factorial((n_m[j][N]-n_m[j][M])/2-s))*

			power
		      );
#ifdef DEBUG
		printf("Rint = %f\n",R);
#endif
		sign *= -1;
	}

#ifdef DEBUG
	printf("Rnm(%f) = %f (%f)\n",rho,R,zernike_R(j, rho));
#endif
#else
	R = zernike_R(j, rho);
#endif

	/* Do theta part */

	if (n_m[j][M] != 0)
	{
		if (j%2 == 0)
		{
			R *= ( sqrt(2.0) * cos(n_m[j][M] * theta));
		}
		else
		{
			R *= ( sqrt(2.0) * sin(n_m[j][M] * theta));
		}
	}

#ifdef DEBUG
	printf("R(%f,%f) = %f\n",rho,theta,R);
#endif

	/* Do last part */

	R *= sqrt(n_m[j][N] + 1.0);

#ifdef DEBUG
	printf("\nZERNIKE(%d,%f,%f) = %f\n",j,rho,theta,R);
#endif

	return (float)R;
	
} /* zernike() */

/************************************************************************/
/* zernike_R()								*/
/*									*/
/* Calculates the R polynomial part of the Zernike function.		*/
/************************************************************************/

float	zernike_R(int    j, float rho)
{
	double	R;
	int	s;
	int	p;
	int 	sign;
	double	power;
	char	err_mess[81];

	zernike_check_setup();

	/* Check we can at least try this */

	if (j < 1 || j > zernike_max_j) 
	{
	     sprintf(err_mess,"j value %d out of range (%d to %d) in ZERNIKE",
			j,1,zernike_max_j);

	     nrerror(err_mess);
	}

	/* Calculate polynomial part */

	R = 0;
	sign = 1;
	for(s=0; s <= ((n_m[j][N]-n_m[j][M])/2); s++)
	{
		if ((p = n_m[j][N]-2*s) == 0)
		{
			power = 1.0;
		}
		else
		{
			power = (rho == 0)?0.0:pow(rho,(double)p);
		}

#ifdef DEBUG
		printf("Power = %f\n",power);
		printf("factorial(n_m[j][N]-s)=%f\n",factorial(n_m[j][N]-s));
#endif
	 	R += (
			(sign * factorial(n_m[j][N]-s)) /
/*		---------------------------------------------------	*/
		(factorial(s)*factorial((n_m[j][N]+n_m[j][M])/2-s)*
		    factorial((n_m[j][N]-n_m[j][M])/2-s))*

			power
		      );
#ifdef DEBUG
		printf("Rint = %f\n",R);
#endif
		sign *= -1;
	}

	return (float)R;
	
} /* zernike_R() */

/************************************************************************/
/* zernike_dR_drho()							*/
/*									*/
/* Calculates the differential of the R polynomial part of the 		*/
/* Zernike function.							*/
/************************************************************************/

float	zernike_dR_drho(int    j, float rho)
{
	double	R;
	int	s;
	int	p;
	int 	sign;
	double	power;
	char	err_mess[81];

	zernike_check_setup();

	/* Check we can at least try this */

	if (j < 1 || j > zernike_max_j) 
	{
	     sprintf(err_mess,"j value %d out of range (%d to %d) in ZERNIKE",
			j,1,zernike_max_j);

	     nrerror(err_mess);
	}

	/* Calculate polynomial part */

	R = 0;
	sign = 1;
	for(s=0; s <= ((n_m[j][N]-n_m[j][M])/2); s++)
	{
		if ((p = n_m[j][N]-2*s-1) == 0)
		{
			power = 1.0;
		}
		else
		{
			power = (rho == 0)?0.0:pow(rho,(double)p);
		}

#ifdef DEBUG
		printf("Power = %f\n",power);
		printf("factorial(n_m[j][N]-s)=%f\n",factorial(n_m[j][N]-s));
#endif
	 	R += (
		(sign * (n_m[j][N]-2*s) * factorial(n_m[j][N]-s)) /
/*		---------------------------------------------------	*/
		(factorial(s)*factorial((n_m[j][N]+n_m[j][M])/2-s)*
		    factorial((n_m[j][N]-n_m[j][M])/2-s))*

			power
		      );
#ifdef DEBUG
		printf("Rint = %f\n",R);
#endif
		sign *= -1;
	}

	return (float)R;
	
} /* zernike_dR_drho() */

/************************************************************************/
/* dzernike_dX()							*/
/*									*/
/* Calculates dierivative of the jth zernike function at given position */
/* wrt an axis in a random direction with an angle eta to the x axis.	*/
/* Note all internals are doubles due to large numbers encountered.	*/
/************************************************************************/

float   dzernike_dX(int j, float rho, float theta, float eta)
{
	double	R;
	double	dR_drho;
	double  p1, p2;
	char	err_mess[81];
	double	result;
	double  sqrt2;

	zernike_check_setup();

	/* Check we can at least try this */

	if (j < 1 || j > zernike_max_j) 
	{
	     sprintf(err_mess,"j value %d out of range (%d to %d) in ZERNIKE",
			j,1,zernike_max_j);

	     nrerror(err_mess);
	}

	if (rho<0.0 || rho > 1.0) 
	{
	  sprintf(err_mess,"rho value %f out of range (0 to 1) in ZERNIKE",rho);
	  nrerror(err_mess);
	}

	/* Calculate polynomial parts */

	R = zernike_R(j, rho);
	dR_drho = zernike_dR_drho(j, rho);

	/* Do theta part */

	if (n_m[j][M] != 0)
	{
		sqrt2 = sqrt(2.0);
		if (j%2 == 0)
		{
			p1 = sqrt2 * cos(n_m[j][M] * theta);
			p2 = sqrt2 * sin(n_m[j][M] * theta);
		}
		else
		{
			p1 = sqrt2 * n_m[j][M] * sin(n_m[j][M] * theta);
			p2 = -1.0 * sqrt2 * n_m[j][M] * cos(n_m[j][M] * theta);
		}
	}
	else
	{
		p1 = 1.0;
		p2 = 0.0;
	}

	/* OK, combine these things */

	result = dR_drho * cos(theta - eta) * p1;
	if (rho != 0.0)
		result += (R / rho * sin(theta - eta) * p2);
	else
		result += (sin(theta - eta) * p2);

	
	/* Do last part */

	result *= sqrt(n_m[j][N] + 1.0);

	return (float)result;
	
} /* dzernike_dX() */

/************************************************************************/
/* zerike_phase()  	                                                */
/*									*/
/* Calculates the phase at a given position in the aperture.		*/
/* Requires array a defining zernike coefs. 				*/
/* Assumes coefficients are normalized for phase.			*/
/************************************************************************/

float zernike_phase(int    maxJ,
		    float rho,
		    float theta,
		    float *a)
{
        float  	result;
        int 	i;

        result = 0;
#ifdef DEBUG
	printf("Inside zernike_phase(%d,%f,%f,a)\n",maxJ,rho,theta);
#endif

        for(i=1; i<=maxJ; i++)
        {
		if (a[i] != 0) result += (a[i] * zernike(i,rho,theta));
        }

        return result;

} /* zernike_phase() */

/************************************************************************/
/* pixel_phase()							*/
/*									*/
/* Caluclates the phase information at a given pixel location.		*/
/* The pixel information is taken from the pixel structure.		*/
/* Assumes that the Zernike modes are normalized for OPD in microns.	*/
/* Uses wavelength to move things back to phase.			*/
/************************************************************************/

float	pixel_phase(aperture_pixel pixel, 
		    int  	   maxJ, 
		    float  	   lambda, 
		    float  	   *a)
{
	int	i;
	float	result;

	/* First check we are inside the aperture */

	if (pixel.mask != INSIDE) return 0.0;

	/* Now ensure the maxJ is reasonable */

	if (maxJ > pixel.maxJ)
	{
		fprintf(stderr,
			"pixel_phase(): WARNING: Mode number mismatch!\n");
		maxJ = pixel.maxJ;
	}

	/* OK do the calculation */

	if (lambda != last_lambda)
	{
		norm = 2.0*M_PI/lambda;
		last_lambda = lambda;
	}
	result = 0.0;
	for(i=1;i<=maxJ;i++) result += (a[i] * pixel.a[i]);
	result *= norm;

	/* That is all */

	return result;

} /* pixel_phase() */

/************************************************************************/
/* phase_gradient()  	                                                */
/*									*/
/* Calculates the average phase gradient based on Zernike coefficients.	*/
/* Requires array a defining zernike coefs. 				*/
/* Assumes coefficients are normalized for phase.			*/
/************************************************************************/

float phase_gradient(int  maxJ,
		    float theta,
		    float *a)
{
        float  result;
        int 	j;

	zernike_check_setup();

        result = 0;

        for(j=1; j<=maxJ; j++)
        {
		if (a[j] != 0.0 && n_m[j][M] == 1) 
		{
			result += (a[j] * zernike(j,1.0,theta));
		}
        }

        return result;

} /* phase_gradient() */

/************************************************************************/
/* factorial()                                                          */
/*                                                                      */
/* Calculates the factorial of a given number.                          */
/* Used instead of numerical recipes because large numbers are used.	*/
/************************************************************************/

double  factorial(int x)
{
        double  result;

        if (x<0) nrerror("value out of range in FACTORIAL");

        result = 1;

        for(; x>0; x--) result *= (double)x;

        return result;

} /* factorial() */
