/************************************************************************/
/* zernike.h								*/
/*                                                                      */
/* Header file for Zernike library.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#ifndef _ZERNIKE_H_ /* Let's make sure of only one pass through */
#define _ZERNIKE_H_

/*
 * Some of these declarations depend on  the nrc.h and nsimpleX.h files
 */

#ifndef _NRC_H_
#include <nrc.h>
#endif
#ifndef _SIMPLEX_H_
#include <nsimpleX.h>
#endif

#ifndef M_PI
#define M_PI 3.1415926537
#endif

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#define MAX_NOLL_CONST 22
#define N	0
#define M	1
#define LONG	0
#define SHORT	1

#define TIME_DOMAIN	0
#define FREQ_DOMAIN	1
#define PHASE_GRAD	2
#define PHASE_GRAD_PS	3

/* Note that these are also defined in nsimpleX.h and MUST BE THE SAME */

#ifndef MEAN
#define MEAN	3
#define MAX	2
#define	MIN	1
#define ZERO	0
#endif

#define INSIDE	1
#define OUTSIDE 0

#define LAMBDA_0	(0.5)	/* Standard wavelength in microns */
#define LAMBDA_0_2	(LAMBDA_0*LAMBDA_0)  /* Same thing squared */

#define LAMBDA_0_NORM	(LAMBDA_0/(2.0*M_PI))
#define LAMBDA_0_NORM_2	(LAMBDA_0_NORM*LAMBDA_0_NORM)

#define DEFAULT_N_SAMPLES 	(64)
#define DEFAULT_DT		(0.001)
#define DEFAULT_RESONANCE	(60.0)
#define DEFAULT_DAMP		(0.1)
#define DEFAULT_MAXIMUM		(300e-6)
#define DEFAULT_R		(0.5)

/*
 * Structure for holding header info for coefficient files
 */

typedef struct	ZERNIKE_HEADER {
		int	domain;  	/* TIME or FREQUENCY */
		float	R;	 	/* Radius in meters */
		float	v;	 	/* Wind speed meters per second */
		int	maxJ;	 	/* Maximum order of polynomial used */
		int	n;	 	/* Number of wavefronts in file */
		float	time_step;	/* in seconds */
		float	l0;	 	/* Inner scale length */
		float	L0;	 	/* Outer scale length */
		float	reserved1;	/* I'll think of something later! */
		float	reserved2;	/* I'll think of something later! */
		float	reserved3;	/* I'll think of something later! */
		float	reserved4;	/* I'll think of something later! */
		float	reserved5;	/* I'll think of something later! */

	} zernike_header;

typedef struct ZERNIKE_SAMPLE {
		float	ldc; 		/* LDC thickness in meters */
		float	tip_tilt_x;	/* tip/tilt mirror position x (a2) */
		float	tip_tilt_y;	/* tip/tilt mirror position y (a3) */
		float	ople;		/* OPLE position in m */
		float	*a;		/* The Zernike coefficients */

	} zernike_sample;

/* Structure for defining an aperture pixel */

typedef struct APERTURE_PIXEL {
		unsigned char mask;  	/* INSIDE or OUTSIDE */
		float	 x;		/* X position (normalized) */
		float	 y;		/* X position (normalized) */
		float	 rho; 		/* between 0 and 1 */
		float	 theta;		/* between 0 and 2*M_PI */
		int	 maxJ;		/* Number of modes */
		float	 *a;		/* The Zernike values */

	} aperture_pixel;

/* Wobbler structure */

typedef struct  QUADRANT_DETECTOR {

		int	maxJ;		/* Maximum Zernike mode */
		float	lambda;		/* Wavelength in microns */
		float	theta;		/* Angle of tilt axis in radians */
		float	maximum;	/* Maximum output */
		float	err_norm;	/* Normalization factor for error */

        } quadrant_detector;

typedef struct TIP_TILT_MIRROR {

		float	resonance;	  /* Natural resonance frequency */
		float	damp;		  /* Damping factor */
		float	f6db;		  /* -6 dB point */
		struct  DIGITAL_FILTER *response; /* The mirror response */
		float	maximum;	  /* Maximum throw */

	} tip_tilt_mirror;

typedef struct STRUCT_WOBBLER_SERVO {

		float	C1,C2;				/* Servo params */
		struct QUADRANT_DETECTOR *detector2;	/* The detector */
		struct TIP_TILT_MIRROR *mirror2;	/* The mirror */
		struct QUADRANT_DETECTOR *detector3;	/* The detector */
		struct TIP_TILT_MIRROR *mirror3;	/* The mirror */
		float  next_demand_position2;		/* Guess what */
		float  next_demand_position3;		/* Guess what */
		float  *detector_history2;		/* For running mean */
		float  *detector_history3;		/* For running mean */
		float	detector_output2;		/* Running mean */
		float	detector_output3;		/* Running mean */
		float	last_move_2;			/* of mirror */
		float	last_move_3;			/* of mirror */
		int    history_length;			/* History length */
		int    current_history_pos;		/* Guess again */

	} struct_wobbler_servo;

/* 
 * Macro functions.
 */

#define aperture(npix,maxJ) sub_obsc_aperture(npix,maxJ,0.0,0.0,1.0,0.0,1.0)

#define sub_aperture(npix,maxJ,pos_rho,pos_theta,size_rho) \
		sub_obsc_aperture(npix,maxJ,pos_rho,pos_theta,size_rho,0.0,1.0)

#define free_aperture(pixels, npix) free_sub_obsc_aperture(pixels, npix)

#define free_sub_aperture(pixels, npix) free_sub_obsc_aperture(pixels, npix)

#define normalize_r0(lambda,r0) (pow((lambda/LAMBDA_0),1.2)*r0)

#define zernike_check_setup() if (n_m == NULL)\
			      {\
				fprintf(stderr,\
					"CALL set_up_zernike_calcs()!\n");\
				exit(-1);\
			      }
				
/* 
 *  Globals
 */

extern int     **n_m; 
extern int     zernike_max_j;
extern long int    random_seed;

/* Function Declarations */

/* build_tables.c */

float *calc_freqs(zernike_header header);

float **build_ps_table(float **freqs, zernike_header header, int calctime);

void calc_coefs(FILE *output, float **powspec_table, 
	float *freqs, zernike_header ps_header, zernike_header cf_header);

/* covariance_zernike.c */

float covariance_a(int j1, int j2, float R, float r0);
float variance_phase_grad(float R, float r0, float epsilon);

/* display_fringe.c */

void display_fringe(Window theWindow, aperture_pixel **pixels, 
	float lambda, float dlambda, float *a1, float *a2, 
	int maxJ, int npix, int pixmult, int disp_type, int fill);

float **calculate_fringe_intensity(aperture_pixel **pixels,
		        int npix,
		        int maxJ,
			float	lambda,
			float	dlambda,
		        float *a1,
			float *a2,
			int   fill);

float **calculate_fringe_phase(aperture_pixel **pixels,
		        int npix,
		        int maxJ,
			float	lambda,
		        float *a1,
			float *a2,
			int   fill);

/* display_image.c */


void display_image(Window theWindow,
	      aperture_pixel	**pixels,
	      float lambda,
	      float *a,
	      int maxJ,
	      int npix,
	      int pixmult,
	      int imbedp2,
	      int disp_type);

float **calculate_image(aperture_pixel **pixels,
		        int npix,
		        int imbedp2,
		        int maxJ,
			float lambda,
		        float *a);

float **calculate_image_amp_phase(aperture_pixel **pixels, 
			int npix, 
        		int imbedp2, 
			float **amp, 
			float **phase);

/* display_phase.c */

void display_phase(Window theWindow, aperture_pixel **pixels, 
	float lambda, float *a, int maxJ, int npix, int pixmult, int disp_type);

float **calculate_phase(int npix, aperture_pixel **pixels, 
	int maxJ, float lambda, float *a, int fill);

/* files.c */

FILE *write_header(char *filename, zernike_header header);

FILE *read_header(char *filename, zernike_header *header);

void copy_header(zernike_header *header_to, zernike_header header_from);

int cmp_header(zernike_header *header_a, zernike_header *header_b, int force_n);

void write_sample(FILE *fp, zernike_header header, zernike_sample sample);

void read_sample(FILE *fp, zernike_header header, zernike_sample *sample);

void write_powspec_table(char *filename, float *freqs, 
	float **powspec_table, zernike_header header);

float **read_powspec_table(char *filename, 
	float **freqs, zernike_header *header);

/* fit_zernike.c */

float *fit_zernike(int maxJ, float **image, float npix, 
	aperture_pixel **pixels, int display_type);

float *fit_zernike_masked(int maxJ, float **image, int **mask, 
	float npix, aperture_pixel **pixels, int display_type);

/* mem_alloc.c */

aperture_pixel **appixmatrix(int nrl, int nrh, int ncl, int nch);

void free_appixmatrix(aperture_pixel **m, int nrl, int nrh, int ncl, int nch);

aperture_pixel  **sub_obsc_aperture(int npix, int maxJ, float pos_rho,
                     float pos_theta, float size_rho, float in_rho,
                     float out_rho);

void free_sub_obsc_aperture(aperture_pixel **pixels, int npix);

/* misc_calculations.c */

float image_center_intensity(aperture_pixel **pixels, int npix, 
	float *a, int maxJ, float lambda);

float total_aperture_values(aperture_pixel **pixels, float **values, int npix);

float tilt_var_to_r0(float var, float R, float lambda);

/* normalize.c */

void r0_normalization(float *a, int maxJ, float r0);

void phase_to_opd(float *a, int maxJ, float lambda);

void opd_to_phase(float *a, int maxJ, float lambda);

/* power_spec_phase_grad.c */

float power_spec_phase_grad(int calc_time, float R, float v, 
	float f, float L0, float epsilon);

/* power_spec_zernike.c */

float power_spec_a(int calc_time, int j, float R, float v, float L0, float f);

/* setup_zernike.c */

void set_up_zernike_calcs(int maxJ);

void clean_up_zernike_calcs(void);

/* visibility.c */

float susi_visibility_2(aperture_pixel **pixels, int npix, int maxJ, 
	float lambda, float dlambda, float *a1, float *a2);

float tango_twiss_visibility_2(aperture_pixel **pixels, int npix, 
	int maxJ, float lambda, float dlambda, float *a1, float *a2);

/* wobble.c */

quadrant_detector set_up_quad_detect(int maxJ, float lambda, float theta);

float quad_detect(quadrant_detector quad, float *a, float n);

tip_tilt_mirror set_up_tip_tilt_mirror(float dt, float resonance, 
	float damp, float maximum, float R);

void clean_up_tip_tilt_mirror(tip_tilt_mirror tip_tilt);

float move_mirror(tip_tilt_mirror tip_tilt, float new_pos);

struct_wobbler_servo set_up_wobbler_servo(float C1, float C2, int maxJ, 
	float lambda, float dt, float resonance, float damp, 
	float maximum, float R, int history);

void clean_up_wobbler_servo(struct_wobbler_servo wobbler);

void wobble(struct_wobbler_servo *wobbler, zernike_sample *sample, float n);

fcomplex wobbler_response(struct_wobbler_servo wobbler, float freq);

fcomplex computer_response(float freq, float dt, int nhist, float C1, float C2);

/* zernike.c */

float zernike(int j, float rho, float theta);

float   zernike_R(int    j, float rho);

float   zernike_dR_drho(int    j, float rho);

float   dzernike_dX(int    j, float rho, float theta, float eta);

float zernike_phase(int maxJ, float rho, float theta, float *a);

float pixel_phase(aperture_pixel pixel, int maxJ, float lambda, float *a);

float phase_gradient(int maxJ, float theta, float *a);

double factorial(int x);

#endif /* Ensures only one pass through header file */
